#!/usr/bin/perl

# TODO:
# STRUCTURE MENU PAGE
#    Create. DONE
#    Beautify & Better Explanations.
#    Link from Admin Menu Frame. DONE.
# CATEGORY SELECTING & EDITING
#    Selecting Functionality. DONE.
#    Editing Functionality. DONE
#    Use template for Form. DONE
#    Test user errors and responses. DONE.
#    Explain to the user what an alias is. Distinguish from synonym.
#    Make sure they're not re-entering a category that already exists (case insensitive): DONE
#    Possibly order the programs so that selected ones are first? HAVE ORDERED BY AGENCY NAME INSTEAD.
#    Look into Sharing code with scripts/guide/category_loader.pl
# SYNONYM SELECTING & EDITING:
#    - Create Synonyms.pm from code out of synonym_loader.pl DONE
#    - Refactor synonym_loader.pl to use MVHub::Synonyms.pm DONE
#    - Use Synonyms.pm in this script. DONE
#    - Create template form. DONE
#    - Recreate synonym_stem table with submitted word column, and load data into it. DONE.
#        - on dsiegal db. DONE.
#        - on production db. DONE.
#    - Create new function & aggregates on production database. DONE.
#    - Check http://archives.postgresql.org/pgsql-sql/ for answer to question.
#    - Possibly put in a check to catch (and drop) any duplicated stem within a stem set
#    - Any limitations on input?
#      Might want to drop at least commas, and possibly only keep alphas, hyphens, apostrophes.
# HEADINGS
#    Everything. TODO.
#
# MISC
#    Possibly, edit the Admin Home Page with the same links as in the top frame, but also explanations.
#    Check other TODOs.
#    Change Common to get only a specific version of FormBuilder: DONE
#    Add spell checking. Probably want to refactor how we incorporate it (here as well as in rest of application).
#    Test on IE
#    Throw out scripts/guide/add_category.pl.
#    Document IN PROGRESS
#    Usability Testing

use strict;
use warnings;

use CGI;
use CGI::Carp;
use DBI;

# TODO: trouble installing:
#use Text::Capitalize;

use MVHub::Utils;
use MVHub::HTMLTemplateUtils;

use MVHub::Common;
use MVHub::Utils::DB;
use MVHub::Headings;
use MVHub::HTML;
use MVHub::Page;
use MVHub::Synonyms;

my $EDIT_CATEGORIES_TEMPLATE
    = '../../../templates/guide/edit_categories.tmpl';
my $EDIT_SYNONYMS_TEMPLATE  = '../../../templates/guide/edit_synonyms.tmpl';
my $STRUCTURE_MENU_TEMPLATE = '../../../templates/guide/structure_menu.tmpl';

{
    my $cgi = CGI->new();
    MVHub::Util::clean_cgi_params($cgi);
    my $mode = $cgi->param('mode');
    $cgi->delete('mode');
    if ( !$mode || $mode eq 'structure_menu' ) {
        display_structure_menu($cgi);
    }
    elsif ( $mode eq 'category_chooser' ) {
        display_category_chooser($cgi);
    }
    elsif ( $mode eq 'edit_category' ) {
        process_category_form($cgi);
    }
    elsif ( $mode eq 'edit_synonyms' ) {
        process_synonym_form($cgi);
    }
    else {
        MVHub::Utils::assert( 0, "Unknown mode requested of $0: $mode" );
    }
}

sub process_synonym_form {
    my $cgi = shift;

    if ( $cgi->param('synonyms') ) {

        # Form was submitted; update the database
        my $synonyms_string = $cgi->param('synonyms');
        my @synonyms = split( '\n', $synonyms_string );
        Synonyms::submit_synonyms_to_db(@synonyms);
        my $redirect_url
            = $ENV{SCRIPT_NAME} . '?mode=structure_menu&success=1';
        print $cgi->redirect($redirect_url);
        exit(0);
    }
    else {

        # Display the form with pre-existing values from database
        my @synonyms = Synonyms::fetch_synonyms_from_db();
        my $synonyms_string = join( "\n", @synonyms );

     # FYI: Using Cgi::Formbuilder is overkill for a form with just one field,
     # so we've just layed out the form explicitly in the template.
        HTMLTemplateUtils::format_template(
            filename   => $EDIT_SYNONYMS_TEMPLATE,
            param_href => {
                synonyms => $synonyms_string,
                rows     => ( @synonyms + 2 )
            },
            print   => 1,
            doctype => 1
        );
    }
}

sub display_structure_menu {
    my $cgi = shift;
    HTMLTemplateUtils::format_template(
        filename   => $STRUCTURE_MENU_TEMPLATE,
        param_href => { $cgi->Vars() },
        print      => 1,
        doctype    => 1
    );
}

sub display_category_chooser {
    my $cgi           = shift;
    my $headings      = new Headings();
    my @heading_names = $headings->heading_names();

    my @output = ('<br>');
    if ( $cgi->param('success') ) {
        push( @output,
            CGI::p('<font color="red">Your changes have been saved.</font>')
        );
    }
    push( @output,
        '<p>',
        '<button onclick=',
        qq|location.href="$ENV{SCRIPT_NAME}?mode=edit_category"|,
        '> <strong>Create a New Category</strong></button>',
        '</p>',
        '<p>',
        '<strong>or Click a Category to Edit:</strong>',
        '</p>' );

    my $url = $ENV{SCRIPT_NAME} . '?mode=edit_category&category_id=';
    foreach (@heading_names) {
        push( @output, "<h2>$_</h2>" );
        my $categories_aref = $headings->categories($_);

        # Copied from Guide.pm. Consider refactoring.
        # Generate an html table of category links
        my @links;
        foreach (@$categories_aref) {
            my $link  = $url . $_->{category_id};
            my $label = CGI::escapeHTML( $_->{category_name} );

            # add space around forward slashes so a label like
            # "Translating/Interpreting" will wrap nicely
            $label =~ s|\s*/\s*| / |g;
            push( @links, CGI::a( { href => $link }, $label ) );
        }

        my $table = HTML::make_html_table(
            columns          => 4,
            content_aref     => \@links,
            table_attributes => 'width="100%" cellspacing="5"'
        );
        push( @output, $table );
    }
    print( $cgi->header('text/html'),
        $cgi->start_html(
            -style => { -src => '/guide/styles/admin.css' },
            -title => 'Directory Structure Administration'
        ),
        join( "\n", @output ),
        $cgi->end_html()
    );
}

sub process_category_form {
    my $cgi         = shift;
    my $category_id = $cgi->param('category_id');
    if ( defined $category_id ) {
        MVHub::Utils::assert( $category_id !~ /\D/,
            'Category id must be numeric only' );
    }

    my $preloaded_values_href;
    my $form = make_category_editing_form($cgi);

    if ( $form->submitted() ) {
        if ( $form->validate() && validate_more($form) ) {
            submit_category_form_data_to_db( scalar $form->field() );
            my $redirect_url
                = $ENV{SCRIPT_NAME} . '?mode=structure_menu&success=1';
            print $cgi->redirect($redirect_url);
            exit(0);
        }
        else {

            # TODO: add announcement?
        }
    }
    elsif ($category_id) {
        $preloaded_values_href
            = fetch_existing_category_and_dependent_data_from_db(
            $category_id);
    }
    print $form->render( header => 1, values => $preloaded_values_href );
}

# Returns a CGI::Formbuilder object representing our form. The form is blank (i.e.
# the fields are defined, but are not pre-populated with any data.)
# This CGI::Formbuilder object has no associated page header.
sub make_category_editing_form {
    my ($cgi) = @_;

    my %category_field_defs = (
        category_id   => { type     => 'hidden' },
        category_name => { required => 1 },
        aliases       => {
            type    => 'textarea',
            rows    => 3,
            comment => 'Optional. One per line.'
        },
        headings => {
            type     => 'checkbox',
            options  => get_all_headings(),
            columns  => 3,
            required => 1,
            comment  => 'Select one or more.'
        },
        programs => {
            type    => 'checkbox',
            options => get_all_programs(),
            columns => 1,
            comment => "Optional. Alternatively, you can assign categories"
                . " to programs via their program forms."
        }
    );

    my $form = MVHub::Common::prepare_form(
        form_attributes_href => {
            name   => 'category_editor_form',
            params => $cgi,

# TODO: listing the columns here is only while we're not using an html template,
# so that CGI:FB knows what order to display 'em in:
# fields => [qw/category_name aliases headings programs/]
# TODO: remove the above 3 lines.
            template => $EDIT_CATEGORIES_TEMPLATE
        },
        fields_href => \%category_field_defs
    );

    return $form;
}

# Returns a reference to an array of headings, e.g.:
# [ "Arts/Culture/Entertainment", "Financial Education/Credit", ... ]
sub get_all_headings {
    my $dbh = MVHub::Utils::DB::get_dbh();
    my $sql = MVHub::Utils::DB::get_sql_select_statement(
        'HEADING_X_HEADING_NAME');
    my $headings_aref = $dbh->selectcol_arrayref($sql);
    return $headings_aref;
}

# Returns a reference to an array of program hashrefs, e.g.:
# [
#   { 12345 => 'Youth Volleyball (YMCA)' },
#   { 156745 => 'Building Blocks (North Shore Arc)' },
#   ...
# ]
sub get_all_programs {
    my $dbh = MVHub::Utils::DB::get_dbh();

    my $sql = MVHub::Utils::DB::get_sql_select_statement(
        'PROGRAM_X_PROGRAM_AGENCY_ID');

    my $result_aref = $dbh->selectall_arrayref($sql);

    # see end of 'perldoc -f map' on why the '+' in the following:
    my @headings = map( +{ $_->[0] => $_->[1] }, @$result_aref );
    return \@headings;
}

# The return statement tells all.
sub fetch_existing_category_and_dependent_data_from_db {
    my $category_id = shift;
    my $dbh         = MVHub::Utils::DB::get_dbh();

    my $sql = MVHub::Utils::DB::get_sql_select_statement(
        'CATEGORY_X_CATEGORY_NAME');
    my @bind_values = ($category_id);

    my @result
        = @{ $dbh->selectall_arrayref( $sql, { Slice => {} }, @bind_values )
        };
    my $category_name = $result[0]->{category_name};

    $sql = MVHub::Utils::DB::get_sql_select_statement('ALIAS_X_ALIAS_NAME');
    my $aliases_aref
        = $dbh->selectcol_arrayref( $sql, { Slice => {} }, $category_id );
    my $aliases = join( "\n", @$aliases_aref );

    $sql = MVHub::Utils::DB::get_sql_select_statement(
        'HEADING_CATEGORY_X_HEADING_NAME');
    my $headings_aref
        = $dbh->selectcol_arrayref( $sql, { Slice => {} }, $category_id );
    MVHub::Utils::assert( @$headings_aref,
        "Expected at least one heading for this category (#$category_id)" );

    $sql = MVHub::Utils::DB::get_sql_select_statement(
        'PROGRAM_CATEGORY_X_PROGRAM_ID');
    my $programs_aref
        = $dbh->selectcol_arrayref( $sql, { Slice => {} }, $category_id );
    return {
        category_name => $category_name,
        aliases       => $aliases,
        headings      => $headings_aref,
        programs      => $programs_aref
    };
}

sub submit_category_form_data_to_db {
    my $values_href = shift;

# Pre-processing data destined for database here:
# TODO: get T:C installed or remove this line
#$values_href->{category_name} = Text::Capitalize::$values_href->{category_name};

    my $dbh = MVHub::Utils::DB::get_dbh();
    $dbh->begin_work();

# If we have a category id, we must be updating an existing category.
# In this case, in addition to updating the category record, we need to
# get rid of the dependent alias, heading_category and program_category records
# the user wants removed, update any changed ones, and add any new ones.
# That's a lot of work. The easiest way is to just delete the existing
# category record, whcih will cascade to delete the dependent records (because
# of the way the foreign keys are defined on the those tables).
# Then we insert everything, just as if we were creating a new category. The only
# difference is that for an update we re-use the existing category_id, and
# for a new category we'll make a new one.
    if ( $values_href->{category_id} ) {
        my $sql = MVHub::Utils::DB::get_sql_delete_statement(
            'CATEGORY_X_CATEGORY_ID');
        $dbh->do( $sql, undef, $values_href->{category_id} );
    }
    else {
        $values_href->{category_id}
            = MVHub::Utils::DB::get_next_in_sequence( $dbh,
            'category_id_sequence' );
    }
    insert_category_and_dependent_data( dbh => $dbh, %$values_href );
    $dbh->commit();
}

sub insert_category_and_dependent_data {
    my %args               = @_;
    my $dbh                = delete $args{dbh};
    my $category_id        = delete $args{category_id};
    my $category_name      = delete $args{category_name};
    my $aliases_string     = delete $args{aliases};
    my $heading_names_aref = delete $args{headings};
    my $program_ids_aref   = delete $args{programs};
    MVHub::Utils::assert( ( scalar keys %args ) == 0,
        "Unknown arguments: @{[%args]}" );

    MVHub::Utils::DB::insert_one_record(
        dbh         => $dbh,
        table       => 'category',
        values_href => {
            'category_name' => $category_name,
            'stems_name'    => Common::get_stems($category_name),
            'category_id'   => $category_id
        }
    );

    if ($aliases_string) {

        # TODO: are we guaranteed that the line separator is a \n ?
        my @aliases = split( '\n', $aliases_string );

        foreach my $alias (@aliases) {
            $alias = MVHub::Utils::trim($alias);
            next unless $alias;    # ignore blank lines
            Common::insert_one_record(
                dbh         => $dbh,
                table       => 'alias',
                values_href => {
                    'category_id' => $category_id,
                    'alias_name'  => $alias,
                    'stems_name'  => Common::get_stems($alias)
                }
            );
        }
    }

    foreach my $heading_name (@$heading_names_aref) {
        Common::insert_one_record(
            dbh         => $dbh,
            table       => 'heading_category',
            values_href => {
                'category_id'  => $category_id,
                'heading_name' => $heading_name
            }
        );
    }

    foreach my $program_id (@$program_ids_aref) {
        Common::insert_one_record(
            dbh         => $dbh,
            table       => 'program_category',
            values_href => {
                'category_id' => $category_id,
                'program_id'  => $program_id
            }
        );
    }
}

# This is the place for error checking that can't be done within
# CGI::FormBuilder->validate(), e.g.:
#   - complex validations that require access to other variables
#   - validations that need to produce a custom error message
# Returns 0 iff any validation fails, supplying an error message
# for each failed validation in the custom_error-category_name template
# variable on the form.
sub validate_more {
    my $form       = shift;
    my $return_val = 1;

    if (is_category_name_taken(
            category_name => $form->field('category_name'),
            category_id   => $form->field('category_id')
        )
        )
    {
        my $msg = "The category name you entered is "
            . "already being used by another category. Please enter a different one:";
        $form->tmpl_param( 'custom_error-category_name', $msg );

        $return_val = 0;
    }

    return $return_val;
}

# Returns true iff the given category name is already in use
# (without regard to case) in the database by another category. If a
# a category_id argument is supplied, then any record with that ID
# will be excluded from the search. Returns false otherwise.
sub is_category_name_taken {
    my %args          = @_;
    my $category_name = delete $args{'category_name'};
    my $category_id   = delete $args{'category_id'};
    MVHub::Utils::assert( defined $category_name && $category_name ne "",
        "Missing category_name argument" );
    MVHub::Utils::assert( ( scalar keys %args ) == 0,
        "Unknown arguments: @{[%args]}" );
    my $dbh = MVHub::Utils::DB::get_dbh( $ENV{MV_CONFIG_FILE} );

    # The SELECT EXISTS (SELECT 1 ...) syntax is generally faster than the
    # SELECT count(*) syntax, since the former only scans until it finds
    # the first match.
    my @bind_variables = ( lc($category_name) );
    my $sql            = MVHub::Utils::DB::get_sql_select_statement(
        'CATEGORY_X_EXISTS_ONE_CATEGORY_NAME');

    if ($category_id) {
        @bind_variables = ( lc($category_name), $category_id );
        $sql = MVHub::Utils::DB::get_sql_select_statement(
            'CATEGORY_X_EXISTS_ONE_CATEGORY_NAME_NOT_ID');
    }

    my @row
        = $dbh->selectall_arrayref( $sql, { Slice => {} }, @bind_variables );

    return ( $row[0] );
}
