#!/usr/bin/perl

use strict;
use warnings;

use MIME::Entity;

use Carp;

use MVHub::Common;
use MVHub::Utils;
use MVHub::Utils::ConfigSimple;

# Our specially doctored version of CGI::FormBuilder:
use CGI::FormBuilder 2004.0123456789;

my $cfg
    = MVHub::Utils::ConfigSimple::create_config_from( $ENV{MV_CONFIG_FILE} )
    or croak
    "couldn't create config object with file: '$ENV{MV_CONFIG_FILE}' ";
my $site_code = $cfg->param('SITE.website_code');

{    # main
    my $cgi = CGI->new();

    MVHub::Utils::clean_cgi_params($cgi);

    my $form = CGI::FormBuilder->new(
        template   => 'contact_form.tmpl',
        method     => 'POST',
        query      => $cgi,
        header     => 1,
        javascript => 0
    );

    $form->field(
        name     => 'name',
        size     => 45,
        label    => 'Your name',
        required => 1
    );
    $form->field(
        name     => 'liame',
        size     => 45,
        label    => 'Your e-mail',
        validate => 'EMAIL',
        required => 1
    );
    $form->field(
        name     => 'question',
        type     => 'textarea',
        rows     => 5,
        cols     => 40,
        label    => 'Your question',
        required => 1
    );

    if ( $form->submitted && $form->validate ) {

# The refering page should obviously be this script. Yet for some reason, spam bots
# don't get this right. Use this fact to stop 'em cold. No doubt at some point,
# they'll get around this and stronger tests will be needed.
        if (  !$ENV{HTTP_REFERER}
            or $ENV{HTTP_REFERER} !~ /$ENV{SCRIPT_NAME}$/ )
        {
            die("Detected possible spam bot using contact form. Exiting.");
        }

        my $fields = $form->field;

        # subject contains clues about testing/production
        # origin of msg
        my $subject
            = $cfg->param('SITE.website_name') . " Contact ($ENV{HTTP_HOST})";

        # send the confirmation e-mail
        my $mail = MIME::Entity->build(
            From    => $fields->{name} . ' <' . $fields->{liame} . '>',
            To      => $cfg->param('NOTIFICATION.admin_email'),
            Subject => $subject,
            Data    => $fields->{question},
        );
        MVHub::Common::sendmail($mail);

        # show a confirmation message
        print CGI::header();
        print << "RESULT";
Thank you!  Your message has been sent.<br>
We will respond within 2 business days.<br>
<a href="/">(go home) </a>
RESULT

    }
    else {
        my $show_nsp_left_menu = 0;
        if ( $site_code eq 'nsp' ) {
            $show_nsp_left_menu = 1;
        }
        $form->tmpl_param( 'show_nsp_left_menu' => $show_nsp_left_menu );

        $form->tmpl_param(
            'admin_name' => $cfg->param('NOTIFICATION.admin_name') );
        $form->tmpl_param(
            'admin_phone' => $cfg->param('NOTIFICATION.admin_phone') );
        $form->tmpl_param(
            'dev_name' => $cfg->param('NOTIFICATION.dev_name') );
        $form->tmpl_param(
            'dev_phone' => $cfg->param('NOTIFICATION.dev_phone') );

        # print the form for them to fill out
        print $form->render;
    }

}
