#!/usr/bin/perl 

# A very rudimentary and probably short-lived script for spitting out
# query results into HTML pages.

use warnings;
use strict;

use CGI;
use CGI::Carp;
use Date::Calc;
use DBI;
use DBIx::XHTML_Table;
use HTML::Template;

use MVHub::Utils::ConfigSimple;
use MVHub::Common;
use MVHub::Reports;

my $dbh = MVHub::Utils::DB::get_dbh( $ENV{MV_CONFIG_FILE} );
my $cfg
    = MVHub::Utils::ConfigSimple::create_config_from( $ENV{MV_CONFIG_FILE} );
my $max_notifications = $cfg->param('NOTIFICATION.max_notifications');

my @REPORTS = (

    # Use this as a template:
    #     {
    #         title => '',
    #         sql   => qq/
    #
    #         /
    #     },
    {   title   => 'Agency Addition Dates',
        comment => "This gives the date when each agency was added to MVHub.",
        sql     => MVHub::Utils::DB::get_sql_select_statement(
            'AGENCY_X_AGENCY_NAME_DATE_CREATED')
    },

    {   title => 'Program Addition Dates',
        comment =>
            "This gives the date when each program was added to MVHub.",
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'AGENCY_X_PROGRAM_NAME_AGENCY_NAME_DATE_CREATED')
    },

    {   title => 'All Searches',
        comment =>
            "'Category Score' is the score (on a scale from 0 to 1) of a "
            . "program category search's result.\n"
            . "'Agencies Displayed' is the number of agencies listed in an agency search's result.\n"
            . "From Feb. 12, 2005 on: 'Referer' (sic) is the address of the page from which the search "
            . "was conducted. A referer with an external site address probably means that site "
            . "has a link to a specific search. A blank referer probably means the user used a bookmark "
            . "(of a previously conducted search).",
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'AGENCY_SEARCH_LOG_X_ALL_SEARCHES')
    },

    {   title => 'Searchers per Day',
        comment =>
            'The number of people conducting searches per day. Multiple searches '
            . 'from the same IP address on the same day are attributed to the same '
            . 'person. Only dates with one or more searches are listed.',
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'SEARCH_LOG_X_SEARCHERS_PER_DAY')
    },

    {   title =>
            'Number &amp; percentage of Agencies &amp; Programs updated in the Last 6 Months',
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'AGENCY_PROGRAM_X_NUMBER_PERCENTAGE_UPDATED_LAST_SIX_MONTHS')
    },

    {   title =>
            'Age (in months) of Agency and Program Records since they were Last Updated',
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'MONTHS_X_AGENCY_PROGRAM_RECORD_LAST_UPDATE')
    },

    {   title =>
            'Date of the Oldest Record belonging to each Agency (ordered by date)',
        comment =>
            "(An agency's oldest record may be its agency record &\/or any of its program record(s))",
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'PROGRAM_AGENCY_ID_X_OLDEST_RECORD_DATE')
    },

    {   title => 'Number of Programs under each Category',
        sql   => MVHub::Utils::DB::get_sql_select_statement(
            'CATEGORY_PROGRAM_CATEGORY_X_NUMBER_OF_PROGRAMS_UNDER_CATEGORY')
    },

    {   title => 'Number of Categories used by each Program',
        sql   => MVHub::Utils::DB::get_sql_select_statement(
            'PROGRAM_X_NUMBER_OF_CATEGORIES_USED_BY_EACH_PROGRAM')
    },
    {   title => 'Program Names that Repeat the Agency Name',
        sql   => MVHub::Utils::DB::get_sql_select_statement(
            'PROGRAM_X_NAMES_THAT_REPEAT_AGENCY_NAME')
    },

    {   title => 'All Category Searches, in Order of Frequency Performed',
        comment =>
            "'High Score' indicates the \% of matching words in the highest scoring category match. 
For example, a search for 'youth league basketball' might get the 'Youth Basketball' category as its top 
match with a score of .67. A high score of 0 means there were no matching categories. "
            . "(Note: The search terms displayed are word stems (aka roots). A stem of 'hous' represents searches for 'house',
'housing' or 'houses', which we interpret here as being equivalent.)",
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'CATEGORY_SEARCH_LOG_X_ORDER_OF_FREQUENCY_PERFORMED')
    },

    {   title => 'Category Searches Peformed that Return No Matches',
        comment =>
            "(Note: The search terms displayed are word stems (aka roots). A stem of 'hous' represents searches for 'house',
'housing' or 'houses', which we interpret here as being equivalent.)",
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'CATEGORY_SEARCH_X_PERFORMED_THAT_RETURN_NO_MATCHES')
    },

#    {
#        title => 'NEW: All Agency Searches',
#        comment => "'Agencies Displayed' is the number of agencies listed in the search's result. ",
#        sql   => qq/
#            SELECT to_char(timestamp, 'Dy, Mon DD, YYYY') as search_date, search_phrase, hit_count as agencies_displayed
#            FROM agency_search_log
#            ORDER BY search_id DESC
#        /
#    },

    {   title => 'Programs with Newsletters',
        comment =>
            'All of the following programs have newsletters (though only some provided descriptions).',
        sql => MVHub::Utils::DB::get_sql_select_statement(
            'PROGRAM_X_HAS_NEWSLETTER')
    }
);    # @REPORTS

{
    my $cgi           = CGI->new();
    my $report_number = $cgi->param('report_number');
    my $dbh           = MVHub::Utils::DB::get_dbh( $ENV{MV_CONFIG_FILE} );

    my $output;

    if ( !defined $report_number ) {
        $output = $cgi->h1('Administrative Reports') . "\n";
        $output
            .= $cgi->p(
            'Our more technical reports of traffic, searches and data freshness, run each 	time you click the link'
            ) . "\n";
        $output .= MVHub::Reports::print_report_list( $cgi,
            '/cgi-bin/mvhub/reports.pl', @REPORTS );
        print $cgi->header();
    }
    else {
        $output = MVHub::Reports::print_report( $dbh, $cgi, $report_number,
            '/html/reports.shtml', @REPORTS );
        $output = MVHub::Page::prepare_page(
            body                          => $output,
            transfer_internal_style_sheet => 1,
            show_left_menu                => 1,
            show_search_bar               => 0,
            site_code                     => $cfg->param('SITE.website_code'),
        );
        print $cgi->header,
            $cgi->start_html( -title => 'Administrative Reports', );
    }

    print $output;
}

