#!/usr/bin/perl

our ($VERSION) = '$Revision: 1471 $' =~ /([.\d]+)/;
use strict;
use warnings;

use Carp;
use HTML::Template;

use MVHub::GeneratePDF;
use MVHub::Utils::DB;
use MVHub::Utils::ConfigSimple;

{    # main
    my $config_file = $ARGV[0] || $ENV{MV_CONFIG_FILE};
    my $dbh = MVHub::Utils::DB::get_dbh($config_file);
    my $cfg = MVHub::Utils::ConfigSimple::create_config_from($config_file);

    my $website       = $cfg->param('SITE.website_name');
    my $latex_file    = "${website}_agencies_only.tex";
    my $pdf_file      = "${website}_agencies_only.pdf";
    my $template_file = $cfg->param('ABSOLUTE_PATH.template_text_dir')
        . 'agency_only_pdf.tmpl';

    chdir $cfg->param('ABSOLUTE_PATH.tmp_dir');

    my $latex_source_file
        = generate_latex_source( $dbh, $cfg, $template_file, $latex_file );
    my $cmd = "/usr/bin/pdflatex $latex_source_file > /dev/null";
    ( system($cmd) == 0 ) or die "failed: '$cmd' ";

    my $reports_dir = $cfg->param('ABSOLUTE_PATH.reports_dir');
    ( system("cp $pdf_file $reports_dir") == 0 )
        or die "failed to copy $pdf_file to $reports_dir";

}

sub generate_latex_source {
    my $dbh             = shift or croak 'missing param: $dbh ';
    my $cfg             = shift or croak 'missing param: $cfg';
    my $template_file   = shift or croak 'missing prarm: $template_file';
    my $source_filename = shift or croak 'missing prarm: $source_filename';

    my $sql = MVHub::Utils::DB::get_sql_select_statement('AGENCY_X_ALL');

    my $agencies_aref = get_arrayref( $dbh, $sql );
    $agencies_aref
        = MVHub::GeneratePDF::clean_merge_sort( $agencies_aref, [] );

    my $tmpl = HTML::Template->new(
        filename          => $template_file,
        die_on_bad_params => 0,
    );

    $tmpl->param( website => $cfg->param('SITE.website_name') );
    $tmpl->param(
        website_description => $cfg->param('SITE.website_description') );
    $tmpl->param( website_location => $cfg->param('SITE.website_location') );
    $tmpl->param( agencies         => $agencies_aref );
    open LATEX, ">$source_filename"
        or die "open failed: '$source_filename' $!";
    print LATEX $tmpl->output();
    close LATEX;

    return $source_filename;
}

sub get_arrayref {
    my ( $dbh, $sql, @bind_params ) = @_;
    return $dbh->selectall_arrayref( $sql, { Slice => {} }, @bind_params );
}
__END__



=head1 NAME

generate_pdf_directory.pl - produce a  62 page (approximately) .pdf file listing agencies in mvhub

=head1 SYNOPSIS

    ./generate_agency_only_pdf_directory.pl

=head1 SEE ALSO

  generate_agency_program_pdf.pl

=head1 LAST MODIFIED

Last modified on $Date: 2008-12-11 16:47:52 -0500 (Thu, 11 Dec 2008) $ by $Author: omacneil $

=head1 BUGS

REPLACEME 

=head1 LICENSE

The GNU Affero General Public License, version 3.0 or later

=head1 COPYRIGHT

Community Software Lab, Inc ( http://thecsl.org )

=cut
