#! /usr/bin/perl

use warnings;
use strict;

use Test::More;

# TODO:
# add tests to test:
#   - ignoring filler word
#   - bad characters

use MVHub::GuideSearch;

my %expected_results = (
    "purple" => [],          # Not a category, no matches
    "pear"   => ["pear"],    # One direct unaliased match, one word

    "apple apple" => [ "apple", "apple pie" ],    # Repeated word

    "an is and or" => [],                         # Only filler, no matches

    "florida orange" => ["florida orange"]
    ,    # One direct unaliased match, two words
    "orange"   => ["florida orange"],    # One word partial unaliased match
    "american" => ["apple pie"],         # One-word alias match
    "granny smith" => [ "apple", "best brand" ],    # Two-word alias match

    "tater"      => ["potato"],                     # Synonym match
    "gooseberry" => ["kiwi tart"]
    ,    # Partial synonym match on first word of category
    "whortleberry" => ["stewed bilberry"]
    ,    # Partial synonym match on !first word of category
         # One word partially matches multiple categories
    "pie" =>
        [ "apple pie", "banana cream pie", "boston cream pie", "cherry pie" ],

# Multiple word phrase with some partial matches on both words and some partial matches on one wordr
    "cream pie" =>
        [ "banana cream pie", "boston cream pie", "apple pie", "cherry pie" ],

# Two words, on full match, one or more partial matches for each seperate word

    # Sorted first by relevance, secondarily alphabetically
    "apple pie" => [
        "apple pie",
        "apple",
        "banana cream pie",
        "boston cream pie",
        "cherry pie"
    ],

# One word of a multi-word search partially matches one category and fully matches another
    "banana split" => [ "banana", "banana cream pie" ],

    # creme is a synonym for cream
    "boston creme" => [ "boston cream pie", "banana cream pie" ],

    # Synonym (grandma] = Does not exist
    #     $expected->[1-granny) partial-matches alias (granny smith=>apple)
    "grandma" => [ "apple", "best brand" ],

    # Test against slash in name
    "delicious" => ["apple"],

    # There is a liver category, but no programs checked it off, so no match
    "liver" => [],

    # ...

);

foreach my $search_phrase ( keys %expected_results ) {
    is_deeply(
        MVHub::GuideSearch::test_category_search($search_phrase),
        $expected_results{$search_phrase},
        $search_phrase
    );
}

my $tmp_aref = [qw/Perl is a \\gre*.*at language/];
MVHub::Common::remove_filler($tmp_aref);
@$tmp_aref = sort(@$tmp_aref);
is_deeply(
    $tmp_aref,
    [ sort(qw/Perl great language/) ],
    "Testing MVHub::Common::remove_filler"
);

