#!/usr/bin/perl

# $Revison: $
# USAGE: add_category <category name> <heading 1> [, heading 2, heading 3 ...]
# Creates the given category and adds it to the given headings.

# If anything fails (e.g. category name already exists, heading doesn't exist, etc.)
# no changes are made to the database.

# Other than the above, this script has little error handling or other niceties, since
# it will hopefully be replaced by a swank Web GUI. If not, it should be refactored.

use strict;
use warnings;

use MVHub::Common;

my $category = shift;
my @headings = @ARGV;
print(qq/Will insert category "$category" into heading(s): @headings ...\n/);
my $dbh = MVHub::Utils::DB::get_dbh();
$dbh->begin_work();

my $category_id
    = MVHub::Utils::DB::get_next_in_sequence( $dbh, 'category_id_sequence' );
MVHub::Utils::DB::insert_one_record(
    dbh         => $dbh,
    table       => 'category',
    values_href => {
        'category_name' => $category,
        'stems_name'    => MVHub::Common::get_stems($category),
        'category_id'   => $category_id
    }
);

foreach my $heading (@headings) {
    MVHub::Utils::DB::insert_one_record(
        dbh         => $dbh,
        table       => 'heading_category',
        values_href => {
            'category_id'  => $category_id,
            'heading_name' => $heading
        }
    );
}

$dbh->commit();
print "Category added.\n";

# This is supposed to catch any fatal errors so we can do an explicit rollback. However
# MVHub::Common's CGI::Carp is catching it first. It doesn't seem to matter, tho'. When the
# connection is closed without a commit, the changes are rollbacked.
$SIG{__DIE__} = sub {
    my $message = $_[0] . "\n";
    $dbh->rollback();

    $message .= "No changes have been made to the database.\n";
    die $message;
};
