#!/usr/bin/perl -w

# Purpose: Fail a test if any error log file size
# 	   has increased, that is we have more
#          more errors in the log after tests
#          have run.

#          This file records the total size of the
#	   error logs before tests run.
#
#          zz-cleanup.t test script runs last
#          and checks if size has changed

use strict;
use warnings;
use Test::More;
use File::stat;

my @SITES    = qw/nsp mvh/;
my $BASE_DIR = "/var/www/mvhub/$ENV{USER}";

my $file_size_db = "$BASE_DIR/link-to-live-code/app-mvhub/t/file_size.tmp";

plan tests => 1;

my $error_log_size = 0;

foreach my $SITE (@SITES) {
    my $file = "$BASE_DIR/log/$SITE/error.log";

    my $err_msg = << "ERR";
missing: $file
         All tests cancelled. you need to change your log/ directory structure per 
         app-mvhub/doc/checklits/install_mvhub.txt
ERR

    BAIL_OUT($err_msg) if !-e $file;

    my $sb = stat($file);
    $error_log_size += $sb->size();
}

open( SIZE, '>', $file_size_db )
    or die "Can't open $file_size_db for writing\n";

print SIZE $error_log_size;
close SIZE
    or die
    "Can't close  $file_size_db possible failure to flush write buffer\n";

ok( 1, "Successfully wrote the log files size to file_size.tmp" );
