#! /usr/bin/perl -w

use strict;
use warnings;

use Test::WWW::Mechanize;
use Test::More;

use TestHelper;

{    #main
    my $host            = TestHelper::get_test_website_name('mvh');
    my @pages           = init_pages($host);
    my $number_of_tests = ( ( scalar @pages ) * 2 );

    plan tests => $number_of_tests;
    my $mech = Test::WWW::Mechanize->new();
    run_tests( $mech, @pages );
}    #main

sub init_pages {
    my $host = shift
        or die 'init_pages() missing param: $host';
    my @pages = (
        {   'url'              => "http://$host/html/admin/home.shtml",
            'auth'             => TestHelper::create_temp_auth(),
            'expected_content' => "Welcome to the Administrative Home Page",
            'todo_reason'      => '',
        },
        {   'url'              => "http://$host/cgi-bin/mvhub/logout.pl",
            'auth'             => TestHelper::create_temp_auth(),
            'expected_content' => "You have been logged out",
            'todo_reason'      => '',
        },
        {   'url'              => "http://$host/html/admin/home.shtml",
            'auth'             => '',
            'expected_content' => "Authorization Required",
            'todo_reason' =>
                'bug to be fixed later, possibly in major re-write',
        },

    );
    return @pages;
}
use Test::Harness;

sub run_tests {
    my $mech  = shift or die 'pages_get_ok() missing param: $mech';
    my @pages = @_    or die 'pages_get_ok() missing param: @pages';

    foreach my $page (@pages) {
        $mech->add_header( Authorization => $$page{auth} )
            if $$page{auth};
        $mech->get_ok( $$page{url} );

        if ( $$page{todo_reason} ) {
        TODO: {
                local $TODO = "$page->{expected_content} $$page{todo_reason}";
                $mech->content_contains( $$page{expected_content} );
            }
        }
        else {
            $mech->content_contains( $$page{expected_content} );
        }
    }

}

