#!/usr/bin/perl -w

use strict;
use warnings;

use Test::More;
use Test::WWW::Mechanize;

use MVHub::Utils::Setup;

use Params::Validate qw/:all/;

use TestHelper;

{    #main
    my $host           = TestHelper::get_test_website_name('nsp');
    my $agency_id      = '103520';
    my $quick_login_id = TestHelper::get_quick_login_from( $agency_id,
        $ENV{MV_CONFIG_FILE} );

    my @pages = init_pages( $host, $agency_id, $quick_login_id );
    my $number_of_tests = ( ( scalar @pages ) * 4 );

    plan tests => $number_of_tests;
    my $mech = Test::WWW::Mechanize->new();
    $mech->default_header( Authorization => TestHelper::create_temp_auth() );

    TestHelper::pages_get_ok( $mech, @pages );
    check_for_categorize_link( $mech, @pages );
}    # main

sub init_pages {
    ( my $host, my $agency_id, my $quick_login_id )
        = Params::Validate::validate_pos(
        @_,
        { type => SCALAR },
        { type => SCALAR },
        { type => SCALAR }
        );

    my @pages = (
        {   'url' =>
                "http://$host/cgi-bin/mvhub/agency.pl?rm=ql&aid=$agency_id&qlid=$quick_login_id",
            'expected_content' =>
                "Agency Account Home Page - North Shore Community Action Programs",
            'tests_to_skip' => '',
            'todo_reason'   => 'Need to fix diplay agency home page',
        },
    );
    return @pages;
}

sub check_for_categorize_link {
    my $mech  = shift or die 'page_links_ok() missing param: $mech';
    my @pages = @_    or die 'page_links_ok() missing param: @pages';

    foreach my $page (@pages) {
        $mech->get( $$page{url} );
        ok( $mech->find_link( text => 'categorize' ),
            "Found categorize link on the page"
        );
    }
}
