#!/usr/bin/perl -w

use strict;
use warnings;

use Carp;
use TestHelper;
use Test::More;
use Test::WWW::Mechanize;
use WWW::Mechanize;

{    #main
    my $host            = TestHelper::get_test_website_name('nsp');
    my @pages           = init_pages($host);
    my $number_of_tests = scalar @pages;

    plan tests => $number_of_tests;

    my $mech = Test::WWW::Mechanize->new();
    foreach my $page (@pages) {
        my $msg
            = "following link: '$$page{link_name_to_follow}' on: '$$page{url}'";
        my $regex         = qr/$$page{'link_name_to_follow'}/i;
        my $ignore_result = $mech->get( $$page{url} );
        $mech->follow_link_ok( { text_regex => $regex }, $msg );
    }

}    # main

sub init_pages {
    my $host = shift
        or croak 'init_pages() missing param: host';
    my @pages = (
        {   'url'                 => "http://$host/",
            'link_name_to_follow' => "Other",
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=Other",
            'link_name_to_follow' => "Clubhouse",
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program_results&category_id=803608",
            'link_name_to_follow' => "Pioneer House",
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=510351",
            'link_name_to_follow' => "www.cahns.org",
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=103872",
            'link_name_to_follow' => "www.cahns.org",
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_search_results&search_phrase=house",
            'link_name_to_follow' => "One Family ",
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803113",
            'link_name_to_follow' => "North Shore Community Action Program",
        },
    );
    return @pages;
}

