#!/usr/bin/perl -w

use strict;
use warnings;

use Test::More;
use Test::WWW::Mechanize;

use MVHub::Utils::Setup;

BEGIN {
    use FindBin qw($Bin);
    chdir $Bin;
    use lib qw( ../lib);
}    #BEGIN
use TestHelper;

{    #main
    my $host           = TestHelper::get_test_website_name('nsp');
    my $agency_id      = '103520';
    my $quick_login_id = TestHelper::get_quick_login_from( $agency_id,
        $ENV{MV_CONFIG_FILE} );

    my @pages = init_pages( $host, $agency_id, $quick_login_id );
    my $number_of_tests = ( ( scalar @pages ) * 3 );

    plan tests => $number_of_tests;
    my $mech = Test::WWW::Mechanize->new();
    $mech->default_header( Authorization => TestHelper::create_temp_auth() );

    TestHelper::pages_get_ok( $mech, @pages );
}    # main

sub init_pages {
    my $host = shift
        or die 'init_pages() missing param: $host';
    my $agency_id = shift or die 'init_pages() missing param: $agency_id';
    my $quick_login_id = shift
        or die 'init_pages() missing param: $quick_login_id';

    my @pages = (
        {   'url'              => "http://$host/html/admin/home.shtml",
            'expected_content' => "Welcome to the Administrative Home Page",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/agency.pl?rm=ql&aid=$agency_id&qlid=$quick_login_id",
            'expected_content' =>
                "Agency Account Home Page - North Shore Community Action Programs",
            'tests_to_skip' => '',
            'skip_reason'   => '',
        },

    );
    return @pages;
}

