#!/usr/bin/perl 

# Test to check that preparing_for_directory_submission.pdf is created
# after preparing_for_directory_submission.doc file in MVHub and NSP.

use strict;
use warnings;
use Test::More;
use File::stat;

{    #main
    plan tests => 2;
    my @websites = qw(nsp mvh);
    foreach my $i (@websites) {
        run_test($i);
    }
}    #main

sub run_test {
    my $host = shift or die 'Missing parameter : host';
    my $FILE_DIR
        = "/var/www/mvhub/$ENV{USER}/link-to-live-code/app-mvhub/DocumentRoot/static/$host/html";

    my $pdf_filename = "$FILE_DIR/preparing_for_directory_submission.pdf";
    my $doc_filename = "$FILE_DIR/preparing_for_directory_submission.doc";

    my $pdf_date = get_last_modified_date($pdf_filename);
    my $doc_date = get_last_modified_date($doc_filename);

    # diff_slop is to account for mtime being
    # an artifact of version control
    my $diff_slop = $doc_date - $pdf_date;

    ok( $diff_slop < 5, "pdf is enough older than doc cheatsheet" )
        or diag
        "regenerate $pdf_filename from $doc_filename  '$diff_slop' seconds too young";
}

sub get_last_modified_date {
    my $filename = shift or die 'Missing parameter:filename';
    my $lastmod_date_time = ( stat($filename) )->mtime;

    return $lastmod_date_time;
}
