#!/usr/bin/perl -w

use strict;
use warnings;

use Test::More;
use TestHelper::History;
use TestHelper;

{    #main
    my @files = TestHelper::get_files( '*.pl', '*.pm' );

    my $history_href;
    $history_href = TestHelper::History::load_test_history();

    my $files_to_test_aref = TestHelper::History::files_needing_retest(
        raw_files_aref => \@files,
        test_name      => 'check_syntax',
        history_href   => $history_href,
    );

    my $test_count = scalar @$files_to_test_aref;
    if ( !$test_count ) {
        plan skip_all => 'files not modified';
    }
    else {
        plan tests => $test_count;
        my @files_tested_ok = _test_check_syntax($files_to_test_aref);
        $history_href = TestHelper::History::set_time_tested(
            tested_files_aref => \@files_tested_ok,
            test_name         => 'check_syntax',
            history_href      => $history_href,
        );

        TestHelper::History::store_test_history($history_href);
    }
}    #main

sub _test_check_syntax {

    my $files_aref = shift;
    my @files_tested_ok;

    foreach my $file (@$files_aref) {

        my $cmd = "perl -wc $file 2>/dev/null";
        my $is = is( system($cmd), 0, "syntax of $file" );

        if ($is) {
            push @files_tested_ok, $file;
        }
    }

    return @files_tested_ok;
}
