#!/usr/bin/perl 

use strict;
use warnings;

use Test::More;
my %required_packages = (
    'analog'                          => [],
    'apache2-mpm-itk'                 => ['apache2-mpm-prefork'],
    'graphviz'                        => [],
    'jdresolve'                       => [],
    'libapache2-mod-macro'            => [],
    'libcgi-application-perl'         => [],
    'libcgi-application-plugins-perl' => [],
    'libcgi-session-perl'             => [],
    'libconfig-inifiles-perl'         => [],
    'libconfig-simple-perl'           => [],
    'libdate-calc-perl'               => [],
    'libdatetime-perl'                => [],
    'libdbd-mock-perl'                => [],
    'libdbd-pg-perl'                  => [],
    'libdbd-pg-perl'                  => [],
    'libdbd-sqlite'                   => [],
    'libdbd-sqlite3-perl'             => [],
    'libdbi-perl'                     => [],
    'libdevel-cover-perl'             => [],
    'libfile-readbackwards-perl'      => [],
    'libhtml-lint-perl'               => [],
    'libhtml-template-perl'           => [],
    'libio-all-perl'                  => [],
    'libio-prompt-perl'               => [],
    'liblingua-stem-perl'             => [],
    'libmime-tools-perl'              => [],
    'libparams-validate-perl'         => [],
    'libtemplate-perl'                => [],
    'libtest-base-perl'               => [],
    'libtest-class-perl'              => [],
    'libtest-differences-perl'        => [],
    'libtest-exception-perl'          => [],
    'libtest-html-content-perl'       => [],
    'libtest-mockmodule-perl'         => [],
    'libtest-mockobject-perl'         => [],
    'libtest-nowarnings-perl'         => [],
    'libtest-warn-perl'               => [],
    'libtext-template-perl'           => [],
    'liburi-perl'                     => [],
    'perltidy'                        => [],
    'postfix'                         => ['postfix'],
    'postgresql' => [ 'postgresql-8.1', 'postgresql-8.3' ],
    'tetex-base' => [ 'texlive', ],
    'tetex-bin'  => ['texlive'],
    'tetex-doc'   => [ 'tetex-doc-nonfree', 'texlive-base-bin-doc' ],
    'tetex-extra' => ['texlive-extra-utils'],
    'tidy'        => [],
    'wamerican'   => [],
    'visitors'    => [],
);

{    # main

    my @installed_packages = `dpkg -l`
        or die "failed to run dpkg -l";

    plan tests => ( scalar keys %required_packages );

    my $missing_packages = '';

    foreach my $package ( keys %required_packages ) {

        my $package_found = is_package_found( $package, @installed_packages );

        if ( !$package_found ) {
            my @alteratives = @{ $required_packages{$package} };
            foreach my $alternative (@alteratives) {
                $package_found
                    = is_package_found( $alternative, @installed_packages );
                last if $package_found;
            }
        }

        ok( $package_found, "$package (or equivalent) installed" );

        $missing_packages .= " $package"
            if !$package_found;

    }
    diag("\n\nto fix:\n\n \t sudo aptitude install $missing_packages \n\n")
        if ($missing_packages);
}    # main

sub is_package_found {
    my $package            = shift;
    my @installed_packages = @_;
    my $regex              = quotemeta($package);
    my $package_found      = grep ( /^ii\s+$regex\s+/, @installed_packages );
    die "\$package_found == $package_found unexpected value"
        if ( $package_found != 0 && $package_found != 1 );
    return $package_found;
}
__END__
kludge special case for postgresql

different required_packages for etch/hardy/intrepid 

alternative package names 

