#!/usr/bin/perl 

use strict;
use warnings;
use Carp;
use Test::More;
use Test::WWW::Mechanize;

use English '-no_match_vars';

use TestHelper;

{    # main

    plan tests => 6;
    my ( $agency_id, $program_id );

    diag "Adding agency";
    my $agency_data = get_agency_data();
    my $mech = submit_form( 'form_ref' => $agency_data );

    diag "Adding program";
    $agency_id = get_agency_id_from_agency_page($mech);
    my $program_data = get_program_data($agency_id);
    $mech = submit_form( 'mech' => $mech, 'form_ref' => $program_data );

    diag "Categorizing program";
    $program_id = get_program_id_from_program_page($mech);
    my $category_data = get_category_data( $agency_id, $program_id );
    do_categorize( $category_data, $mech );

}    # main

sub get_agency_data {
    my $host = TestHelper::get_test_website_name('nsp');

    my %agency_record = (
        url              => "http://$host/cgi-bin/mvhub/agency_form.pl",
        expected_content => "<strong>Program Form</strong>",
        number           => '1',
        button           => '_submit',
        fields           => {
            password                     => 'testing',
            password_verification        => 'testing',
            agency_name                  => "0000 $PROCESS_ID Testing Group",
            agency_alias                 => 'Test Guys',
            address1                     => '123 Testing Lane',
            address2                     => '',
            city                         => 'Lowell',
            state                        => 'MA',
            zip                          => '01852',
            main_phone                   => '978-555-1212',
            toll_free_phone              => '',
            hotline                      => '',
            fax                          => '',
            tty_tdd                      => '',
            website                      => 'http://testing123.net',
            email                        => '',
            email_verification           => '',
            mission                      => 'Test things',
            hours                        => '9-5 M-F',
            mail_address1                => '',
            mail_address2                => '',
            mail_city                    => '',
            mail_state                   => '',
            mail_zip                     => '',
            chief                        => 'Chester Tester',
            title                        => 'Lead Tester',
            parent_organization          => '',
            resource_library             => 'No',
            resource_library_description => '',
            meeting_space                => 'No',
            meeting_space_description    => '',
            staff_internet_access        => 'Limited',
            other_info                   => '',
            contact_first_name           => 'Chester',
            contact_last_name            => 'Tester',
            contact_phone                => '978-555-1212',
            contact_fax                  => '',
            contact_email                => "$ENV{USER}\@thecsl.org",
            contact_email_verification   => "$ENV{USER}\@thecsl.org",
            contact_email_available      => 'No',
        },
    );

    return \%agency_record;
}

sub submit_form {
    my %args = @_;
    my $mech = $args{'mech'};
    $mech = Test::WWW::Mechanize->new()
        if !$mech;

    my $form_ref = $args{'form_ref'} or croak 'missing param: $form_href';

    my %form = %$form_ref;
    croak 'Form not initialized'
        if not defined $form{url};
    $mech->default_header( Authorization => TestHelper::create_temp_auth() );

    my %params;

    $params{form_number} = $form{number};
    $params{fields}      = $form{fields};
    $params{button}      = $form{button};

    $mech->get( $form{url} );
    $mech->submit_form_ok( \%params, "$form{url} form number:$form{number}" );

    $mech->content_contains( $form{expected_content} )
        or diag "expected: dumping content: '$form{url}' \n\n"
        . $mech->content();
    return $mech;
}    # sub

sub get_agency_id_from_agency_page {
    my $mech    = shift or croak 'missing param: $mech';
    my $content = $mech->content();
    my @lines   = split /\n/, $content;
    my $raw     = ( grep /agency_id=\d.*program_id=\d/, @lines )[0];
    my ($agency_id) = ( $raw =~ /agency_id=(\d+).*program_id=\d+/ );

    return $agency_id;
}

sub get_program_data {

    my $agency_id     = shift or croak "Missing param: agency_id";
    my $host          = TestHelper::get_test_website_name('nsp');
    my $random_number = int( rand(1000) );
    my $url = "http://$host/cgi-bin/mvhub/program.pl?agency_id=$agency_id";

    my %program_record = (
        url              => $url,
        expected_content => "Categorize Your Program",
        number           => '1',
        button           => '_submit',
        fields           => {
            _submitted_program_form       => '1',
            _sessionid                    => '_sessionid',
            program_name                  => "Program: $random_number",
            program_alias                 => '',
            address1                      => '123 Testing Lane',
            address2                      => '',
            city                          => 'Lowell',
            state                         => 'MA',
            zip                           => '01852',
            main_phone                    => '978-555-1212',
            toll_free_phone               => '',
            hotline                       => '',
            tty_tdd                       => '',
            fax                           => '',
            website                       => 'http://testing123.net',
            email                         => '',
            email_verification            => '',
            mail_address1                 => '',
            mail_address2                 => '',
            mail_city                     => '',
            mail_state                    => '',
            mail_zip                      => '',
            short_program_description     => 'Short Program Description',
            long_program_description      => 'Long Program Description',
            eligibility                   => 'eligibility',
            area_type                     => 'Everywhere',
            cboxarea_TOWN                 => '',
            areas_served_description      => '',
            fees_charged                  => 'Free',
            fees_explained                => '',
            program_schedule              => 'program_schedule',
            hours                         => '24',
            languages                     => 'English',
            languages_description         => '',
            appointment                   => 'No',
            bring                         => '',
            online_registration           => 'Yes',
            parking                       => '',
            transportation                => '',
            handicapped_accessible        => 'No',
            child_care_provided           => 'No',
            child_care_description        => '',
            wait_list                     => 'No',
            wait_list_description         => '',
            other_information             => '',
            volunteer_opportunities       => 'Yes',
            volunteer_information         => '',
            resource_library              => 'No',
            resource_library_description  => '',
            meeting_space                 => 'No',
            meeting_space_description     => '',
            news_letter                   => 'Yes',
            news_letter_description       => '',
            public_net_access             => 'Limited',
            public_net_access_description => '',
            contact_first_name            => 'Chester',
            contact_last_name             => 'Tester',
            contact_phone                 => '978-555-1212',
            contact_fax                   => '',
            contact_email                 => "$ENV{USER}\@thecsl.org",
            contact_email_verification    => "$ENV{USER}\@thecsl.org",
            contact_email_available       => 'No',

            agency_id => "$agency_id",

        },
    );

    return \%program_record;
}

sub get_program_id_from_program_page {
    my $mech    = shift or croak 'missing param: $mech';
    my $content = $mech->content();
    my @lines   = split /\n/, $content;

    my $expression = qr/
	input\s+type\s*=\s*"hidden"	# hidden field	
	\s+name\s*=\s*"program_id"
	\s+value\s*=\s*"(\d+)"		# n digit program_id	
	/x;

    my $raw = ( grep /$expression/, @lines )[0]
        or die "Could not find program id in page\n";
    my ($program_id) = ( $raw =~ $expression );

    return $program_id;
}

#http://$host/cgi-bin/mvhub/program.pl?agency_id=$agency_id&program_id=$program_id&mode=show_new_categories
sub get_category_data {
    my $agency_id  = shift or croak "Missing param: agency_id";
    my $program_id = shift or croak "Missing param: program_id";
    my $host = TestHelper::get_test_website_name('nsp');
    my $url
        = "http://$host/cgi-bin/mvhub/program.pl?agency_id=$agency_id&program_id=$program_id&mode=show_new_categories";

    my %category_record = (
        url              => $url,
        expected_content => "Agency Account Home Page",
        number           => '1',
        button           => 'submit',
        fields           => {
            agency_id    => "$agency_id",
            program_id   => "$program_id",
            mode         => "set_program_categories",
            '.cgifields' => "category_id",
        },
        values_to_tick => [ 804697, 804389, 804037 ],
    );

    return \%category_record;
}

sub do_categorize {
    my $form_ref = shift or croak 'missing param: $form';
    my $mech     = shift;
    my %form     = %$form_ref;
    croak 'Form not initialized'
        if not defined $form{url};
    $mech->default_header( Authorization => TestHelper::create_temp_auth() );

    my %params;

    $params{form_number} = $form{number};
    $params{fields}      = $form{fields};
    $params{button}      = $form{button};

    $mech->get( $form{url} );
    for my $value ( @{ $form{'values_to_tick'} } ) {
        $mech->tick( 'category_id', $value );
    }

    $mech->submit_form_ok( \%params, "$form{url} form number:$form{number}" );
    $mech->content_contains( $form{expected_content} )
        or diag "expected: dumping content: '$form{url}' \n\n"
        . $mech->content();

}
