#!/usr/bin/perl -w

use strict;
use warnings;

use Test::More;
use Test::WWW::Mechanize;

use MVHub::Utils::Setup;

use Params::Validate qw/:all/;

use TestHelper;

{    #main
    my $host = TestHelper::get_test_website_name('mvh');
    my $mech = Test::WWW::Mechanize->new();


    my @mvh_pages_from_both = init_pages_for_both($host);
    my @pages_for_mvh       = init_pages_for_mvh_only($host);

    $host = TestHelper::get_test_website_name('nsp');
    my $agency_id = '103520';
    my $quick_login_id
        = get_quick_login_from( $agency_id, $ENV{MV_CONFIG_FILE} );

    my @nsp_pages_from_both = init_pages_for_both($host);
    my @pages_for_nsp
        = init_pages_for_nsp_only( $host, $agency_id, $quick_login_id );

    my @total_pages = add_mech_objects( @mvh_pages_from_both, @pages_for_mvh,
					@nsp_pages_from_both, @pages_for_nsp);

    my $NUMBER_OF_TESTS_PER_PAGE = 6;

    my $number_of_tests  = ( ( scalar @total_pages ) * $NUMBER_OF_TESTS_PER_PAGE );
    plan tests => $number_of_tests;

   #TestHelper::pages_get_ok( $mech, @total_pages );
    TestHelper::pages_get_ok(@total_pages);
    pages_links_ok(@total_pages );
    pages_lint_ok(@total_pages );
    pages_html_tidy_ok(@total_pages );
 
}    #main

sub init_pages_for_both {
    my $host = shift
        or die 'init_pages() missing param: $host';
    my @pages = (
        {   'url'              => "http://$host",
            'expected_content' => " Search for Social Services",
            'tests_to_skip'    => 'tidy lint link get',
            'skip_reason'      => 'dan 2008-02-06 testing ability to skip',
	    'mech'=>'',
        },
        {   'url'              => "http://$host",
            'expected_content' => "Search for Social Services",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/html/about.shtml",
            'expected_content' => "Our Purpose",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/html/help.shtml",
            'expected_content' => "Agency Account Help",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/html/register.shtml",
            'expected_content' => "Why register your agency",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/reports",
            'expected_content' => "PDF Reports",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/reports/",
            'expected_content' => "PDF Reports",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },

        {   'url'              => "http://$host/cgi-bin/mvhub/agency_form.pl",
            'expected_content' => "Password",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' => "http://$host/cgi-bin/mvhub/guide.pl?rm=browse_headings",
            'expected_content' => "Arts/Culture/Entertainment",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' => "http://$host/cgi-bin/mvhub/admin/admin_reports.pl",
            'expected_content' => "Administrative Reports",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/cgi-bin/mvhub/admin/admin.pl",
            'expected_content' => "Add a New Agency",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/cgi-bin/mvhub/logout.pl",
            'expected_content' => "You have been logged out",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/agency.pl?rm=ql&aid=00&qlid=00",
            'expected_content' =>
                "Invalid quick log-in ID, please log in manually",
            'tests_to_skip' => '',
            'skip_reason'   => '',
	    'mech'=>'',
        },
        {   'url' => "http://$host/cgi-bin/mvhub/agency.pl?rm=ql&id=00",
            'expected_content' =>
                "You used an old login URL that is no longer valid",
            'tests_to_skip' => '',
            'skip_reason'   => '',
	    'mech'=>'',
        },
    );
    return @pages;
}

sub init_pages_for_mvh_only {
    my $host = shift
        or die 'init_pages() missing param: $host';

    my @pages = (
        {   'url' =>
                "http://$host/cgi-bin/mvhub/agency.pl?rm=display_agency_home&agency_id=101903",
            'expected_content' => "You must log in to access this page",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host//cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=100088",
            'expected_content' => "United Teen Equality Center",
            'tests_to_skip'    => 'tidy lint',
            'skip_reason' =>
                'dan 2008-02-06 record uses smart quotes we dont strip ',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=101342",
            'expected_content' => "(The CSL)",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=503916",
            'expected_content' => "Community Software Lab",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url'              => "http://$host/html/reports.shtml",
            'expected_content' => "formatted for printing",
            'tests_to_skip'    => 'link',
            'skip_reason' => 'dan 2008-02-06 link to pdf not implemented',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program_results&category_id=804796",
            'expected_content' =>
                "programs related to: Visual Arts Classes, Youth/Teen",
            'tests_to_skip' => '',
            'skip_reason'   => '',
	    'mech'=>'',
        },
    );
}

sub init_pages_for_nsp_only {
    ( my $host, my $agency_id, my $quick_login_id )
        = Params::Validate::validate_pos(
        @_,
        { type => SCALAR },
        { type => SCALAR },
        { type => SCALAR }
        );

    my @pages = (
        {   'url' =>
                "http://$host/cgi-bin/mvhub/agency.pl?rm=display_agency_home&agency_id=103542",
            'expected_content' => "You must log in to access this page",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host//cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=103531",
            'expected_content' => "Operation Bootstrap",
            'tests_to_skip'    => 'tidy lint',
            'skip_reason' =>
                'dan 2008-02-06 record uses smart quotes we dont strip ',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=103542",
            'expected_content' => "Law Center of Massachusetts",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=509515",
            'expected_content' => "(NSCAP)",
            'tests_to_skip'    => '',
            'skip_reason'      => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/guide.pl?rm=show_program_results&category_id=801584",
            'expected_content' =>
                "programs related to: Adult Basic Education",
            'tests_to_skip' => '',
            'skip_reason'   => '',
	    'mech'=>'',
        },
        {   'url' => "http://$host/$agency_id/$quick_login_id",
            'expected_content' =>
                "Agency Account Home Page - North Shore Community Action Programs, Inc.",
            'tests_to_skip' => '',
            'skip_reason'   => '',
	    'mech'=>'',
        },
        {   'url' =>
                "http://$host/cgi-bin/mvhub/agency.pl?rm=ql&aid=$agency_id&qlid=$quick_login_id",
            'expected_content' =>
                "Agency Account Home Page - North Shore Community Action Programs, Inc.",
            'tests_to_skip' => '',
            'skip_reason'   => '',
	    'mech'=>'',
        },
    );
}


#This will populate the mech field in the hashes
sub add_mech_objects {
    my @pages=@_ or die 'add_mech_objects() missing param: @pages';

    foreach my $page (@pages) {
	my $mech=Test::WWW::Mechanize->new();
	$mech->default_header( Authorization => TestHelper::create_temp_auth() );
	$mech->get($$page{url});
	$$page{mech}=$mech;
    }
    @pages;
}

sub pages_links_ok {
    my @pages = @_    or die 'page_links_ok() missing param: @pages';
    my $mech = '';
PAGES:
    foreach my $page (@pages) {
    SKIP: {
            $mech=$$page{mech};
            if ( $$page{tests_to_skip} =~ 'link' ) {
                skip "$$page{url} $$page{skip_reason}", 1;
                next PAGES;
            }
            my ( $result, $msg ) = _links_ok($mech);
            ok( $result, "Links on: $$page{url}" );
            diag "bad links:\n $msg\n"
                if not $result;
        }
    }
}

{    # a closure
    my %checked_links;

    sub _links_ok {
        my $mech   = shift;
        my @links  = $mech->links();
        my $msg    = '';
        my $result = 1;
        foreach my $link (@links) {
            $checked_links{$link} += 1;
            next if exists $checked_links{$link} && $checked_links{$link} > 0;

            # skip external links
            next if $link =~ m| http:// |x;

            next if defined $link->text() && $link->text() =~ /remove/;
            my $response_code = $mech->head( $link->url )->code();

            # 200 is success,

            # 400 is "bad request" but "bad" doesn't mean
            # not working for example: / or /html/
            # missing host part of URL isn't broken

            # 401 is unauthorized
            # 501 is Not Implemented for Javascript

            if (not(   $response_code == 200
                    || $response_code == 400
                    || $response_code == 401
                    || $response_code == 501 )
                )
            {
                $result = 0;
                $msg .= sprintf( "\t%12s:%s: '%s'\n",
                    $link->text(), $response_code, $link->url() );
            }
        }
        return $result, $msg;
    }

}

sub pages_lint_ok {
    my @pages = @_    or die 'page_lint_ok() missing param: @pages';

    foreach my $page (@pages) {
    SKIP: {
            my $mech=$$page{mech};
            skip $$page{skip_reason}, 1
                if $$page{tests_to_skip} =~ 'lint';
            $mech->html_lint_ok("Lint: ");
        }
    }
}

sub pages_html_tidy_ok {
    my @pages = @_    or die 'pages_html_tidy_ok() missing param: @pages';

    foreach my $page (@pages) {
    SKIP: {
            my $mech=$$page{mech};
            skip $$page{skip_reason}, 1
                if $$page{tests_to_skip} =~ 'tidy';
            my $short_page = shorten_pagename( $$page{url} );
            html_tidy_ok( $mech->content(), $short_page );
        }
    }
}

sub shorten_pagename {
    my $name = shift;
    my $short = ( split '/', $name )[-1];
    $short =~ s/[\?&=]+/_/g;
    return $short;
}

# Test::HTML::Tidy is broken / refuses to install on
# ubuntu 8.10 so we wrap /usr/bin/tidy
sub html_tidy_ok {
    my ( $html, $filename ) = @_;
    my $filename_out    = "/tmp/${filename}_${$}.html";
    my $errors_filename = "/tmp/${filename}_${$}_errors.txt";

    my $cmd;

    $cmd = "|tidy -f $errors_filename -q -o /dev/null ";

    #$cmd = "|tidy -f $errors_filename -o /dev/null ";
    print_to_cmd( $html, $cmd );
    $cmd = "|nl -ba > $filename_out";
    print_to_cmd( $html, $cmd );

    # file with no errors will have size of 0
    my $ok = ( stat($errors_filename) )[7] == 0;
    ok( $ok, "HTML tidy: $filename" )
        or diag "check $filename_out and $errors_filename for details";
    if ($ok) {
        unlink $errors_filename, $filename_out, "${$}ignore.txt"
            or die
            "$! : failed to unlink $errors_filename and/or $filename_out";
    }

}

sub print_to_cmd {
    my ( $printme, $cmd ) = @_;
    local undef $/;

    open my $OUT, $cmd
        or die "$! : failed to open $cmd";
    print $OUT $printme
        or die "$! failed to write to $cmd";
}
