#!/usr/bin/perl 

use strict;
use warnings;

use Test::More;
use Test::WWW::Mechanize;

use TestHelper;

{    #main
    TestHelper::run_submit_forms_ok(
        get_forms_sub => \&get_forms,
        prefix        => 'nsp',
        )
}    # main

sub get_forms {
    my $host = shift
        or die 'missing param: $host';

    my @forms = (

        #test for bad password
        {   'url' => "http://$host/cgi-bin/mvhub/agency.pl",
            'expected_content' =>
                'Incorrect password for this agency. Please try again.',
            'number'        => '1',
            'button'        => 'submit_login',
            'skip_reason'   => '',
            'tests_to_skip' => '',
            'fields'        => {
                'agency_password' => 'badpassword',
                'rm'              => 'do_login',
                'agency_id'       => '103586'
            },
        },

        #test for no password
        {   'url'              => "http://$host/cgi-bin/mvhub/agency.pl",
            'expected_content' => 'Please enter a password',
            'number'           => '1',
            'button'           => 'submit_login',
            'skip_reason'      => '',
            'tests_to_skip'    => '',
            'fields'           => {
                'agency_password' => '',
                'rm'              => 'do_login',
                'agency_id'       => '103586'
            },
        },

        #test for good password and bad agency id
        {   'url' => "http://$host/cgi-bin/mvhub/agency.pl",
            'expected_content' =>
                'Incorrect password for this agency. Please try again.',
            'number'        => '1',
            'button'        => 'submit_login',
            'skip_reason'   => '',
            'tests_to_skip' => '',
            'fields'        => {
                'agency_password' => '@lunch4all',
                'rm'              => 'do_login',
                'agency_id'       => '1035'
            },
        },

        #test for bad password and bad agency id
        {   'url' => "http://$host/cgi-bin/mvhub/agency.pl",
            'expected_content' =>
                'Incorrect password for this agency. Please try again.',
            'number'        => '1',
            'button'        => 'submit_login',
            'skip_reason'   => '',
            'tests_to_skip' => '',
            'fields'        => {
                'agency_password' => 'badpassword',
                'rm'              => 'do_login',
                'agency_id'       => '1035'
            },
        },

        #test for good password and missing agency id
        {   'url' => "http://$host/cgi-bin/mvhub/agency.pl",
            'expected_content' =>
                'Incorrect password for this agency. Please try again.',
            'number'        => '1',
            'button'        => 'submit_login',
            'skip_reason'   => '',
            'tests_to_skip' => '',
            'fields'        => {
                'agency_password' => '@lunch4all',
                'rm'              => 'do_login',
            },
        },

    );    # @forms
    return @forms;
}
