#!/usr/bin/perl 

use strict;
use warnings;

use Params::Validate;

use Test::More;
use Test::WWW::Mechanize;

use TestHelper;

use MVHub::Utils::DB;

{    #main
    TestHelper::run_submit_forms_ok(
        get_forms_sub => \&get_forms,
        prefix        => 'nsp',
        )
}    # main

sub get_forms {
    my $host = shift
        or die 'missing param: $host';

    my @forms = (
        {   'url'              => "http://$host/cgi-bin/mvhub/agency.pl",
            'expected_content' => 'Agency Account Home Page',
            'number'           => '1',
            'button'           => 'submit_login',
            'skip_reason'      => '',
            'tests_to_skip'    => '',
            'fields'           => {
                'agency_password' => _get_password(103586),
                'rm'              => 'do_login',
                'agency_id'       => '103586'
            },
        },

    );    # @forms
    return @forms;
}

sub _get_password {
    Params::Validate::validate_pos( @_, 1 );
    my $agency_id = shift;
    my $dbh       = MVHub::Utils::DB::get_dbh( $ENV{MV_CONFIG_FILE} );

    my $sql = MVHub::Utils::DB::get_sql_select_statement('AGENCY_X_PASSWORD');
    my @bind_values = ($agency_id);
    my $aref
        = $dbh->selectall_arrayref( $sql, { Slice => {} }, @bind_values );
    my $password = $aref->[0]{password};

    return $password;

}
