#!/usr/bin/perl 

use strict;
use warnings;
use Test::More;
use Test::WWW::Mechanize;
use File::Spec;
use Carp;

use TestHelper;

{    #main
    my $dir = $ENV{HTML_TEMPLATE_ROOT};
    my @templ_inc_files = get_files_from( $dir, qw/*.tmpl *.inc/ );
    plan tests => scalar(@templ_inc_files) * 4;
    foreach my $path (@templ_inc_files) {
        my $lines_array_ref = get_lines_from_file($path);
        my ( $ignore_vol, $ignore_dir, $file ) = File::Spec->splitpath($path);
        my %tests = define_tests($file);
        foreach my $test_name ( sort keys %tests ) {

            my $result = line_found_in_lines(
                $tests{$test_name}{'pattern'},
                $tests{$test_name}{'line_num'},
                $lines_array_ref
            );
            my $testname = "$file has " . substr $test_name, 3;
            ok( $result, $testname )
                or diag("\nto fix:\n \t $tests{$test_name}{'fail_msg'}\n\n");
        }
    }
}    #main

sub get_lines_from_file {
    my $filepath = shift or croak "Missing param: filepath";
    my @lines;
    open FILE, "$filepath" or die "Cannot open file";
    @lines = <FILE>;
    close FILE;

    return \@lines;
}

sub line_found_in_lines {
    my $pattern = shift or croak "Missing param: pattern";
    my $line_num = shift;
    croak "Missing param: line_num" if ( !defined $line_num );
    my $lines_array_ref = shift or croak "Missing param: lines_array_ref";
    my @lines = @$lines_array_ref;

    return ( $lines[$line_num] =~ $pattern );
}

sub define_tests {
    my $file  = shift;
    my %tests = (
        '01 first line blank' => {
            'pattern'  => qr/^\s*\n/,
            'line_num' => 0,
            'fail_msg' => "Ensure first line is blank",
        },
        '02 start file comment' => {
            'pattern'  => qr/<!-- start $file -->\s*/,
            'line_num' => 1,
            'fail_msg' => "Comment must be '<!-- start $file -->'",
        },
        '03 stop file comment' => {
            'pattern'  => qr/<!-- stop $file -->\s*/,
            'line_num' => -2,
            'fail_msg' => "Comment must be '<!-- stop $file -->'",
        },
        '04 last line blank' => {
            'pattern'  => qr/^\s*\n/,
            'line_num' => -1,
            'fail_msg' => "Ensure there are TWO returns after last comment",
        }
    );
    return %tests;
}
