#!/usr/bin/perl -w

use strict;
use warnings;
use SQL::Library;

use Test::More;
use Test::NoWarnings;
use MVHub::Utils::DB qw/get_dbh/;

use TestHelper;

my $SQL_LIB_FILE
    = "/var/www/mvhub/$ENV{USER}/link-to-live-code/app-mvhub/conf/sql_select.lib";
my $NUM_MISC_TESTS = 2;

{    # main
    my $sql_lib = new SQL::Library { lib => $SQL_LIB_FILE };
    my @sql_lib_elements = $sql_lib->elements();
    plan tests => ( $NUM_MISC_TESTS + scalar @sql_lib_elements );

    statements_in_alphabetical_order_in($SQL_LIB_FILE);

    my $dbh            = MVHub::Utils::DB::get_dbh( $ENV{'MV_CONFIG_FILE'} );
    my $agency_id      = '103553';
    my $quick_login_id = TestHelper::get_quick_login_from( $agency_id,
        $ENV{'MV_CONFIG_FILE'} );

    statements_without_placeholders_do_ok_in( $sql_lib, $dbh );
    my $test_placeholders_ref
        = get_test_placeholders( $agency_id, $quick_login_id );
    statements_with_placeholders_do_ok_in( $sql_lib, $dbh,
        $test_placeholders_ref );
}    #main

sub get_test_placeholders {
    my $agency_id      = shift or die 'missing param: agency_id';
    my $quick_login_id = shift or die 'missing param: quick_login_id';

    my %data = (
        'AGENCY_HEADING_PROGRAM_X_AGENCY_ID'       => ['103553'],
        'AGENCY_HEADING_PROGRAM_X_AGENCY_ID_EQUAL' => ['103553'],
        'AGENCY_PROGRAM_X_ALL'                     => ['103553'],
        'AGENCY_PROGRAM_X_AGENCY_NAME'             => [ 103553, 103553 ],
        'AGENCY_PROGRAM_X_PROGRAM_WITH_MISSING_CONTACT_INFO' => [],
        'AGENCY_SEARCH_LOG_X_ALL_SEARCHES'                   => [],
        'AGENCY_SEARCH_LOG_X_IF_RECORDED_RECENT_AGENCY_SEARCH_BY_USER' =>
            [ '66.92.77.160.5111109', 'nscap' ],
        'AGENCY_X_AGENCY_ID'   => [103553],
        'AGENCY_X_AGENCY_NAME' => [103553],
        'AGENCY_X_AGENCY_NAME_PASSWORD' =>
            [ 'chester@testing123.net', 'chester@testing123.net' ],
        'AGENCY_X_AGENCIES_WITH_MISSING_CONTACT_INFO' => [],
        'AGENCY_X_AUTH_AGENCY' => [ '103553', '@lunch4all' ],
        'AGENCY_X_DATE_OF_OLDEST_RECORD'    => [ 103553,       103553 ],
        'AGENCY_X_IS_NAME_AVAILABLE'        => ['ymca'],
        'AGENCY_X_IS_NAME_AVAILABLE_NOT_ID' => [ 'ymca',       103553 ],
        'AGENCY_X_EXPIRED_RECORDS'          => [ '01-01-2008', 0 ],
        'AGENCY_X_SEARCH_WITH_TERMS'        => [
            '[[:<:]](ymca|counsel|immi|shelter)[[:>:]]',
            '[[:<:]](ymca|counsel|immi|shelter)[[:>:]]'
        ],
        'AGENCY_X_WEBSITE'                          => [103553],
        'AGENCY_X_PASSWORD'                         => [103586],
        'AGENCY_PROGRAM_X_AGENCY_NAME_AGENCY_ALIAS' => [510296],
        'AGENCY_X_LAST_UPDATED'                     => [103553],
        'AGENCY_X_QUICK_LOGIN_MATCH'  => [ $agency_id, $quick_login_id ],
        'AGENCY_X_QUICK_LOGIN_PASSWD' => [$agency_id],
        'ALIAS_X_ALIAS_NAME'          => [1],
        'ALL_NAMES_VIEW_CATS_WITH_VISIBLE_PROGRAMS_VIEW_X_ALL_FOR_SYNONYMS' =>
            ['[[:<:]](ymca|counsel|immi|shelter)[[:>:]]'],
        'CATEGORY_X_EXISTS_ONE_CATEGORY_NAME'        => ['cat'],
        'CATEGORY_X_EXISTS_ONE_CATEGORY_NAME_NOT_ID' => [ 'cat', 103553 ],
        'CATEGORY_PROGRAM_X_CATEGORY_NAME'           => [103553],
        'CATEGORY_PROGRAM_VIEW_HEADING_CATEGORY_X_ID_NAME_COUNT' =>
            ['Arts/Culture/Entertainment'],
        'CATEGORY_SEARCH_LOG_X_IF_RECORDED_RECENT_CATEGORY_SEARCH_BY_USER' =>
            [ '66.92.77.160.5111109', 'test' ],
        'CATEGORY_X_CATEGORY_NAME'               => [800363],
        'HEADING_CATEGORY_X_HEADING_NAME'        => [103553],
        'NEXTVAL_X_SEQUENCE'                     => ['program_id_sequence'],
        'PG_DATABASE_X_DATNAME'                  => ['dat'],
        'PG_ROLES_X_ROLNAME_CREATEROLE_CREATEDB' => ['rol'],
        'PG_ROLES_X_ROLNAME_ROLSUPER'            => ['rolname'],
        'PG_USER_X_USENAME'                      => ['use'],
        'PROGRAM_AGENCY_PROGRAM_CATEGORY_X_AGENCY_NAME_ID_PROGRAM_NAME' =>
            [800363],
        'PROGRAM_AGENCY_X_PROGRAM_AGENCY_NAME'  => ['103553'],
        'PROGRAM_CATEGORY_X_PROGRAM_ID'         => ['103553'],
        'PROGRAM_CATEGORY_X_CATEGORY_ID'        => [103553],
        'PROGRAM_ID_X_PROGRAM_NAME'             => [103553],
        'PROGRAM_LOWER_PROGRAM_NAME_ID_X_COUNT' => [ 'name', 103553, 103553 ],
        'PROGRAM_LOWER_PROGRAM_NAME_X_COUNT'        => [ 'name', 103553 ],
        'PROGRAM_X_AGENCY_ID'                       => [103553],
        'PROGRAM_X_AGENCY_ID_HIDDEN_REASON_IS_NULL' => [103553],
        'PROGRAM_X_AGENCY_PROGRAM_ID'               => [ 103553, 509548 ],
        'PROGRAM_X_COUNT_ALL'                       => [103553],
        'PROGRAM_X_DISTINCT_PROGRAM_CONTACT_FIRST_LAST_NAME_EMAIL' =>
            [ '103553', 'fname', 'lname', 'a@b.com' ],
        'PROGRAM_X_EXPIRED_RECORDS' => [ '01-01-2008', 0 ],
        'PROGRAM_X_PROGRAM_CONTACT_FIRST_LAST_NAME_EMAIL' =>
            [ '103553', 'fname', 'lname', 'a@b.com' ],
        'PROGRAM_X_PROGRAM_NAME' => [ '103553', 'fname', 'lname', 'a@b.com' ],
        'PROGRAM_X_WEBSITE'      => [509548],
        'PROGRAM_X_LAST_UPDATED' => [509548],
        'SETVAL_X_SYNONYM_SEQ'     => [ 'synonym_id_sequence', '900000' ],
        'SYNONYM_STEM_X_WORD_STEM' => ['alcohol'],
    );
    return \%data;
}
my $search_terms_aref = [ 'ymca', 'counsel', 'immi', 'shelter' ];

sub statements_in_alphabetical_order_in {
    my ($sql_lib_file) = @_;
    open my $IN_FH, $sql_lib_file
        or die "couldn't open $sql_lib_file because:$!";

    my @raw_lines = <$IN_FH>
        or die "failed to read from $sql_lib_file";
    my @statement_names = grep {/\[/} @raw_lines;
    my @sorted_statement_names = sort @statement_names;

    is_deeply \@statement_names, \@sorted_statement_names,
        "statements are sorted alphabetically in SQL::Library file ";
}

sub statements_without_placeholders_do_ok_in {
    my ( $sql_lib, $dbh ) = @_;

    $dbh->begin_work();

    foreach my $statement_name ( $sql_lib->elements() ) {
        my $sql = $sql_lib->retr($statement_name);
        next if $sql =~ /\?/xmgs;
        ok( $dbh->do($sql), "do: $statement_name" );
    }

    $dbh->rollback();
}

sub statements_with_placeholders_do_ok_in {
    my ( $sql_lib, $dbh, $placeholders_href ) = @_;

    $dbh->begin_work();

    foreach my $statement_name ( $sql_lib->elements() ) {
        my $sql = $sql_lib->retr($statement_name);
        next if $sql !~ /\?/xmgs;
        die "$statement_name is not in our test data"
            if not defined $$placeholders_href{$statement_name};
        my $placeholders_aref = $$placeholders_href{$statement_name};
        ok( $dbh->do( $sql, {}, @$placeholders_aref ),
            "do: $statement_name" );
    }

    $dbh->rollback();
}
