#!/usr/bin/perl 

use warnings;
use strict;

use Test::More tests => 1 + 1;    # +1 for NoWarnings
use Test::Exception;
use Test::NoWarnings;

use TestData qw/ notifications_href /;

use MVHub::Utils::ConfigSimple qw/ create_config_from /;
use MVHub::Notifications qw/ create_standard_notification_emails_with
    create_fourth_notification_emails_with
    create_final_notification_emails_with
    get_email_constants_from
    send_emails_contained_in
    shift_off_notifications_with_reminder_count /;

# Config File Variables
my $CFG
    = MVHub::Utils::ConfigSimple::create_config_from( $ENV{MV_CONFIG_FILE} );
my $dev_user = $CFG->param('NOTIFICATION.dev_email');

# Email Template Paths
my $TEMPLATE_PATH = $CFG->param('ABSOLUTE_PATH.template_text_dir');

my $FIRST_NOTIFICATION_TEMPLATE_FILE
    = "$TEMPLATE_PATH/reminder_email_01.tmpl";
my $SECOND_NOTIFICATION_TEMPLATE_FILE
    = "$TEMPLATE_PATH/reminder_email_02.tmpl";
my $THIRD_NOTIFICATION_TEMPLATE_FILE
    = "$TEMPLATE_PATH/reminder_email_03.tmpl";
my $FOURTH_NOTIFICATION_TEMPLATE_FILE
    = "$TEMPLATE_PATH/reminder_email_04.tmpl";
my $FINAL_NOTIFICATION_TEMPLATE_FILE
    = "$TEMPLATE_PATH/reminder_email_05.tmpl";

# Data Structures
my %all_notifications = %$TestData::notifications_href;

###
my $test_message = "Sending email templates to $dev_user";
###

my %final_notifications
    = shift_off_notifications_with_reminder_count( 4, \%all_notifications );
my %fourth_notifications
    = shift_off_notifications_with_reminder_count( 3, \%all_notifications );
my %third_notifications
    = shift_off_notifications_with_reminder_count( 2, \%all_notifications );
my %second_notifications
    = shift_off_notifications_with_reminder_count( 1, \%all_notifications );
my %first_notifications
    = shift_off_notifications_with_reminder_count( 0, \%all_notifications );

MVHub::Utils::assert( !scalar(%all_notifications), "Leftover notifications" );

my $email_constants_href = get_email_constants_from($CFG);

my @emails;
push @emails,
    create_standard_notification_emails_with(
    $FIRST_NOTIFICATION_TEMPLATE_FILE,
    $email_constants_href, %first_notifications );
push @emails,
    create_standard_notification_emails_with(
    $SECOND_NOTIFICATION_TEMPLATE_FILE,
    $email_constants_href, %second_notifications );
push @emails,
    create_standard_notification_emails_with(
    $THIRD_NOTIFICATION_TEMPLATE_FILE,
    $email_constants_href, %third_notifications );
push @emails,
    create_fourth_notification_emails_with(
    $FOURTH_NOTIFICATION_TEMPLATE_FILE,
    $email_constants_href, %fourth_notifications );
push @emails,
    create_final_notification_emails_with( $FINAL_NOTIFICATION_TEMPLATE_FILE,
    $email_constants_href, %final_notifications );

lives_ok { send_emails_contained_in(@emails) } $test_message;
