
package MVHub::Common;
require Exporter;

# #  Search Relate Subs
# get_stems
# remove_filler
#
# #  Page Display Subs
# display_agency_home_page_and_exit
# display_error_page_and_exit
#
# #  Email Subs
# notification_email
# sendmail
#
# #  Config Subs
# is_ip_address_excluded_from_logging
#
# #  Form Processing Subs
# normalize_phone
# prepare_form
#

our @ISA = qw(Exporter);

our @EXPORT = qw(
    delimit_field
    display_agency_home_page_and_exit
    get_agency_select_list
    prepare_form
    sendmail
);

our ($VERSION) = '$Revision: 1729 $' =~ /([.\d]+)/;

use strict;
use warnings;

use CGI;
use CGI::Carp;    # ('fatalsToBrowser');
use CGI::Session;
use DBI;
use SQL::Library;

use File::Basename;
use HTML::Template;
use Lingua::Stem::En;
use MIME::Entity;

use MVHub::Utils;
use MVHub::Utils::ConfigSimple;
use MVHub::Utils::DB;
use MVHub::Page;

# Our specially doctored version of CGI::FormBuilder:
use CGI::FormBuilder 2004.0123456789;
use MVHub::Page;

### CONSTANTS ###

# Our application configuration file. Contains database connection info,
# contact/notification info and other stuff.
my $CONFIG_FILE = $ENV{MV_CONFIG_FILE}
    || croak
    "\n\nEnvironment var MV_CONFIG_FILE undefined. Check apache config or source env_vars_setup.sh \n\n";

# to delimit multiple values stored in a single database column:
our $FIELD_DELIMITER = '!DELIM!';

our $ADMIN_ID      = -1;
our $NO_AGENCY_ID  = 42;
our $NO_PROGRAM_ID = 403;

# Our phone validation reg-ex is less restrictive than FormBuilder's:
our $PHONE_REG_EX = '/^\D*(\d\D*){10,}$/';
our @PHONE_FIELDS
    = qw(main_phone fax toll_free_phone hotline tty_tdd contact_phone contact_fax);

# Although we require program & agency contacts to enter a contact email address in their
# respective forms, we give the administrator the ability to type in these keywords
# in special cases:
#  i) unknown: the record was input by an admin, who doesn't know the email address
#  ii) noemail: the p/a contact is a luddite, who doesn't have an email address
# We need a regex that allows us to do general email validation but still
# accept these special values in the form:
our $ALLOW_CONTACT_EMAIL_REGEX
    = '/unknown|noemail|[\w\-]+(\.[\w\-]+)*@[\w\-]+\.[\w\-]+(\.[\w\-]+)*/';

# (BTW, in case you're wondering: it would be nice to do a case insensitive
#  match above, but CGI::Formbuilder chokes if you tack on the trailing 'i'. Sigh.
#  I've filed a bug report)

# We also need a regex that allows us to identify the special values
# elsewhere in the application. (If you change this be sure to update
# database/create.sql for any views that also use something similar).
our $IGNORE_CONTACT_EMAIL_REGEX = 'unknown|noemail';

# FILESYSTEM INFO for reports
our $AGENCY_USAGE_REPORTS_PATH = "../../reports/agency_usage_reports/";

# This is just to quiet the warnings during compilation about the above constants
# not being used more than once within this file:
1
    || $PHONE_REG_EX
    || $ALLOW_CONTACT_EMAIL_REGEX
    || $IGNORE_CONTACT_EMAIL_REGEX
    || $NO_PROGRAM_ID
    || @PHONE_FIELDS;

my $CONTACT_US_URL = "/cgi-bin/mvhub/contact_form.pl";
my $HOME_URL       = "/";

# Here we create the name for the session id that we'll send in our
# cookies. We want it to be specific to the particular installation of the
# application (i.e. session ids generated by an installation to
# /home/sites/mvhub won't conflict with one generated by an installation to
# /home/people/jdoe_the_developer).

my $numeric_user_id = $<;
MVHub::Utils::assert( $numeric_user_id,
    "\$numeric_user_id is '$numeric_user_id'" );

my $EP_ERROR_MESSAGE_TAG  = 'ERROR_MESSAGE_TAG';
my $EP_CONTACT_US_URL_TAG = 'CONTACT_US_URL_TAG';
my $EP_HOME_PAGE_URL_TAG  = 'HOME_PAGE_URL_TAG';

# List of words in the search phrase to ignore
my @FILLER = (
    '',     'a',    'i',    'am',    'an',   'as',
    'at',   'ax',   'be',   'by',    'do',   'go',
    'he',   'if',   'in',   'is',    'it',   'me',
    'my',   'no',   'of',   'on',    'or',   'ox',
    'so',   'to',   'up',   'us',    'we',   'de',
    'en',   'el',   'la',   'about', 'are',  'com',
    'for',  'from', 'how',  'that',  'the',  'this',
    'was',  'what', 'when', 'where', 'who',  'will',
    'with', 'und',  'the',  'www',   'like', 'off',
    'onto', 'program'
);

# Returns a string containing a delimited list of the given array ref.
sub delimit_field {
    my $values_aref = shift;
    return join( $FIELD_DELIMITER, @$values_aref );
}

# Displays the agency home page and exits
# This is semi-hackish right now, until we move over entirely to
# CGI::Application
sub display_agency_home_page_and_exit {
    my %args         = @_;
    my $agency_id    = $args{agency_id};       # required
    my $init_session = $args{init_session};    # optional
    my $message      = $args{message};         # optional
    croak("Missing agency_id argument")
        if ( !defined $agency_id or $agency_id eq "" );

    my $cgi = new CGI("");
    my $cookie;

    if ($init_session) {
        $cookie = init_session( caller_id => $agency_id );
    }

    # This returns the "http:" "//" host [ ":" port ]" section of the URL
    my $base_url = $cgi->url( -base => 1 );
    my $absolute_path = '/cgi-bin/mvhub/agency.pl';

    my $cgi_params_href = {
        rm        => 'display_agency_home',
        message   => $message,
        agency_id => $agency_id
    };

    use URI::URL;

    my $url = URI::URL->new_abs( $absolute_path, $base_url );
    $url->query_form($cgi_params_href);

    print $cgi->header(
        -cookie   => $cookie,
        -location => $url->as_string
    );

    exit(0);
}

# Displays an error page (takes an optional message as an arg) and exits
sub display_error_page_and_exit {
    my $err_msg = shift;

    my $template = HTML::Template->new( filename => 'error_page.tmpl' )
        or die "Unable to open error_page.tmpl";

    $template->param( $EP_ERROR_MESSAGE_TAG  => $err_msg );
    $template->param( $EP_CONTACT_US_URL_TAG => $CONTACT_US_URL );
    $template->param( $EP_HOME_PAGE_URL_TAG  => $HOME_URL );

    MVHub::Page::print_page_and_exit(
        body           => $template->output(),
        show_left_menu => 1
    );
}

sub get_agency_select_list {
    my $dbh      = shift;
    my $size     = shift;
    my %agencies = MVHub::Utils::DB::get_hash_of_agencies($dbh);

    return CGI::popup_menu(
        -name => 'agency_id',
        -values =>    #sort keys by val so list is in order of agency name
            [ sort { $agencies{$a} cmp $agencies{$b} } keys %agencies ],
        -labels => \%agencies,
        -size   => $size
            || 1

            # TODO : default to CSL
    );
}

sub get_stems {
    my $str = shift;

# We're splitting on everything except a-zA-Z0-9 and ' (for possesives)
# We're splitting on hyphens because it won't effect the search algorithm adversely if
# a single, hyphenated word is treated as two words
    my @words = split( /[^\w\']+/, $str );

    # Copy all numbers into an array (and put them back in later) because
    # Lingua::Stem::En::stem will strip them out
    my @numbers = grep( /^\d+$/, @words );

    my @acronyms = grep ( /^[A-Z]+$/, @words );
    @acronyms = map {lc} @acronyms;

    @words = grep ( !/^[A-Z]+$/, @words );

    remove_filler( \@words );

    my $stems_aref
        = Lingua::Stem::En::stem( { -words => \@words, -locale => 'en' } );

    return lc( join( ' ', ( @$stems_aref, @acronyms, @numbers ) ) );
}

# Returns a boolean indicating whether requests from the given IP address should be excluded
# from site usage logging.
sub is_ip_address_excluded_from_logging {
    my $excluded_ips_string = shift;
    my $ip                  = shift;
    MVHub::Utils::assert( $ip, "Missing ip address argument" );

    my @excluded_ips = split( '/[\s,]/', $excluded_ips_string );
    return grep( $ip =~ m/^$_/, @excluded_ips );
}

sub normalize_phone {
    my $valid_phone = shift;

    # If it's exactly 10 digits, normalize it to form xxx-xxx-xxxx
    if ( $valid_phone =~ m/^\D*(\d\D*){10}$/ ) {

        # We don't care about non-digits
        $valid_phone =~ s/\D//g;

        if ( $valid_phone =~ m/^(\d{3})(\d{3})(\d{4})$/ ) {
            $valid_phone = "$1-$2-$3";
        }

    }

    return $valid_phone;
}

# Routine for sending notifications of various events to the administrator or
# developer email addresses (specified in the config file).
sub notification_email {
    my %args    = @_;
    my $to      = delete $args{to};
    my $subject = delete $args{subject};
    my $message = delete $args{message};
    MVHub::Utils::assert( ( scalar keys %args ) == 0,
        "Unknown arguments: @{[%args]}" );

    my $footer = "This is an automated message. Please do not reply.";

    my $cfg = MVHub::Utils::ConfigSimple::create_config_from(
        $ENV{MV_CONFIG_FILE} );

    my $notice = $cfg->param('SITE.website_name') . " /($ENV{HTTP_HOST})";

    my $installation = $ENV{SERVER_NAME} || $ENV{USER} . ' site';

    # Send e-mail
    my $top = MIME::Entity->build(
        Type    => "text/plain",
        From    => 'MVHub software <>',
        To      => $to,
        Subject => "[MVHub-notice] $subject",
        Data    => $message . "\n\n" . $footer
    );

    sendmail($top);
}

# Return an CGI::FormBuilder object. This routine initializes it with all kinds of
# reasonable defaults, then further modifies it using the following optional named hash ref
# arguments:
#
# form_attributes_href: contains any CGI::FormBuilder object attributes that apply to the
#    form as a whole (i.e. the key-value pairs you would ordinarily pass into the
#    CGI::Formbuilder constructor aside from field definitions).
#
# fields_href: a hashref with the field name as keys, and whose values are each hashrefs
#    of attributes that apply to each field (ie. key-value pairs you'd ordinarily pass
#    into the CGI::FormBuilder's field() method).
#
# To pass in field values (i.e. to display a form with existing data), you probably want
# to include a
#    params => $cgi  (or whatever you call your CGI object)
# in form_attributes_href
sub prepare_form {
    my %args = @_;

    my $form_attributes_href = delete $args{form_attributes_href};
    my $fields_href          = delete $args{fields_href};

    MVHub::Utils::assert( ( scalar keys %args ) == 0,
        "Unknown arguments: @{[%args]}" );

    my $form = CGI::FormBuilder->new(

        #Here are the default form attributes
        method     => 'POST',
        keepextras => 0,
        messages   => {
            form_invalid_input =>
                'You must enter a valid value for the following field:',
            form_invalid_select =>
                'You must choose an option from the following list:',
            form_invalid_checkbox =>
                'You must choose an option from the following group:',
            form_invalid_radio =>
                'You must choose an option from the following group:',
            form_invalid_password =>
                'You must enter a valid password for the following field:',
            form_invalid_textarea => 'You must fill in the following field:',
        },

        header     => 0,
        javascript => 0,

        # Here we add any attributes passed in, overriding the defaults
        %$form_attributes_href

    );

    my %field_attribute_defaults = (
        text => {
            size      => 60,
            maxlength => 60
        },
        textarea => {
            cols => 45,
            rows => 3
        }
    );

    my @valid_field_attribute_names = qw(
        name      type      size      maxlength
        cols      rows      label     required
        value     validate  options   comment
        force     jsclick   labels    linebreaks
        multiple  nameopts  override  sortopts
        columns
    );

    foreach my $field_name ( keys %$fields_href ) {

        # Get the field definition as a hash. An example would be:
        #         (label     => 'Phone',
        #          size      => 20,
        #          maxlength => 20,
        #          comment   => '(incl. area code)',
        #          validate  => $MVHub::Common::PHONE_REG_EX
        #         )
        my %field_attributes_hash = %{ $fields_href->{$field_name} };

        # Expose any invalid attribute names in our field definition, by
        # doing a set difference operation between the set of used names
        # and the set of all valid names.
        use Set::Array;
        my @diff = Set::Array->new( keys %field_attributes_hash )
            ->difference( \@valid_field_attribute_names );

        MVHub::Utils::assert( !@diff,
            "In processing the definition of a form field ($field_name) "
                . "we encountered attribute(s) (@diff) that possibly are not allowed."
        );

        my $type = $field_attributes_hash{type};
        if ( !defined $type ) { $type = 'text' }

        if ( defined $field_attribute_defaults{$type} ) {
            %field_attributes_hash = (
                %{ $field_attribute_defaults{$type} },
                %field_attributes_hash
            );

# Textarea and select tags don't take a type attribute
# delete $field_attributes_hash{type} if ( grep( /$type/, qw(textarea select) ) );
        }

        $form->field(
            name => $field_name,
            %field_attributes_hash
        );
    }

    return $form;
}

# Removes the filler words  defined in @FILLER and non-word characters
sub remove_filler {
    my $phrase_words_aref = shift;

    foreach (@$phrase_words_aref) {

        # Strip all non-word characters - test out hyphens
        $_ =~ s/[^\w -]//g;
    }

    @$phrase_words_aref
        = Set::Array->new(@$phrase_words_aref)->difference( \@FILLER );

}

# Send the given Mime::Entity mail object
sub sendmail {
    my $top  = shift;
    my $from = $top->head()->get('From');
    MVHub::Utils::assert( $from, "Missing 'From' field in mail header" );

# By setting MailFrom, we hope to get bounced messages sent back to the address
# we want (rather than the default - the process owner)
    my $err_msg = "$0 email failed to: ";
    $err_msg .= "'" . $top->head()->get('To') . "'";
    chomp $err_msg;

    $top->smtpsend( MailFrom => $from )
        or warn $err_msg;

}

1;
__END__

=head1 NAME

MVHub::Common - misc utility functions

=head1 VERSION

This documentation refers to MVHub::Common version $Revision: 1729 $

=head1 SYNOPSIS

    use MVHub::Common;

=head1 DESCRIPTION

subs used by more than one script 

=head1 SEE ALSO

 MVHub::Utils

=head1 LAST MODIFIED

Last modified on $Date: 2009-02-25 22:10:10 -0500 (Wed, 25 Feb 2009) $ by $Author: omacneil $

=head1 BUGS

none known 

=head1 LICENSE AND COPYRIGHT

The GNU Affero General Public License, version 3.0 or later

Copyright (c) 2004-2008 Community Software Lab, Inc ( http://thecsl.org )

=cut

=head1 SUBROUTINES/METHODS 


=head2 sub name 

=head1 DIAGNOSTICS

Error messages are listed with documentation for sub routines.

=head1 CONFIGURATION AND ENVIRONMENT

MVHub::Common does not require configuration beyond that used by MVHub itself

=head1 DEPENDENCIES

CGI, CGI::Carp CGI::Session Config::IniFiles;
DBI File::Basename HTML::Template;
Lingua::Stem::En  MIME::Entity;

=head1 BUGS AND LIMITATIONS

There are no known bugs in this module.
Please report problems to help@thecsl.org
Patches are welcome.

=head1 AUTHOR

Community Software Lab help@thecsl.org
=head1 INCOMPATIBILITIES

No known incompatibilities
