
# Using this module will cause any "die" to:
# - send a very explicit email to the developer(s) listed in the application config file.
# - display a nice error page to the end user
# - exit
package MVHub::DieHandler;
use CGI::Carp;
use MVHub::Common;

use MVHub::Utils::ConfigSimple;

$SIG{__DIE__} = sub {

    # do special handling (unless die called from within eval)
    die @_ if $^S;
    my $message = $_[0] . "\n" . get_call_stack();

    #   $message .= "\n\nUser Agent Information\n";
    #   foreach ( sort ( keys %ENV ) ) { $message .= "\t\t$_: $ENV{$_}\n" }
    my $cfg = MVHub::Utils::ConfigSimple::create_config_from(
        $ENV{MV_CONFIG_FILE} );
    my $to = $cfg->param('NOTIFICATION.dev_email');
    chomp $to;
    MVHub::Common::notification_email(
        to      => $to,
        subject => 'Application Terminated',
        message => $message
    );
    warn("Error caught by the die handler:\n");
    warn($message);
    MVHub::Common::display_error_page_and_exit();
};

# this will generate a call stack (but without showing the stack frames
# within this die handler).
sub get_call_stack {
    local $@;
    eval { confess('') };
    my @stack = split m/\n/, $@;
    shift @stack for 1 .. 4;    # Cover our tracks.
    foreach (@stack) {

# If the stack trace includes a connect string, obscure the password ( we don't want
# passwords exposed in error logs and email)
        if (/^\s*DBI::connect\(.*?,.*?,.*?, '(.*?)'/) {
            s/$1/<password obscured>/;
        }
    }
    return wantarray ? @stack : join "\n", @stack;
}

1;
