our ($VERSION) = '$Revision: 1215 $' =~ /([.\d]+)/;

=head1 NAME

GeneratePDF.pm

=head1 SYNOPSIS

    use GeneratePDF 

=head1 DESCRIPTION

 This module contains the common subs of directory.pl and generate_pdf_directory.pl 

=head1 SEE ALSO


=head1 LAST MODIFIED

Last modified on $Date: 2008-09-21 18:02:10 -0400 (Sun, 21 Sep 2008) $ by $Author: omacneil $

=head1 BUGS


=head1 LICENSE

The GNU Affero General Public License, version 3.0 or later

=head1 COPYRIGHT

Community Software Lab, Inc ( http://thecsl.org )

=cut

package MVHub::GeneratePDF;

use strict;
use warnings;
use Exporter;

use Carp;

our @ISA       = qw(Exporter);
our @EXPORT    = ();
our @EXPORT_OK = qw(clean clean_merge_sort unclean);

# given a string,
# replaces special characters with spaces,
# (TODO: handle smart quotes better)
# and escapes special tex characters.
sub clean {

    local $_ = shift;
    s/[^[:print:]]/ /g;
    s/\\/====BACKSLASH====/g;    # good enough, not perfect
    s/([\#\$\%\&_{}])/\\$1/g;
    s/([~^])/\\${1}{}/g;
    s/====BACKSLASH====/\$\\backslash\$/g;
    return $_;
}

# unescapes special tex chars
# so we can test strings for
# equality in their original state
sub unclean {
    my $dirty_me = shift
        or croak 'missing param: $dirty_me';
    $dirty_me =~ s/\\(\&|\$|\%|\_|{|})/$1/;
    return $dirty_me;
}

# given a list of selectall_arefs,
# tex-clean the data they contain,
# synthesize the
#	'name',
#	'description', and
#	'id' fields
#	for each record,
# sort them by name (ignoring articles),
# and return a single aref.

# One ugly hack: if only a single arg is passed in,
# 'name' will be only the program name. This is to
# make this usable by list_programs.

sub clean_merge_sort {

    my $program_name_only = scalar(@_) == 1;

    my @entries = map { @{$_} } @_;
    for my $r (@entries) {
        for ( keys %$r ) {
            $r->{$_} = clean( $r->{$_} ) if $r->{$_};
        }

        $r->{'name'}
            = $program_name_only
            ? $r->{'program_name'}
            : join( ': ',
            map { $r->{"${_}_name"} || () } ( 'agency', 'program' ) );

        $r->{'description'} = join ' ', map {
            (   ( $_ eq 'agency' )
                ? $r->{'mission'}
                : $r->{'short_program_description'}
                )
                || ();
        } ( 'agency', 'program' );

        $r->{'id'} = join '_',
            map { $r->{"${_}_id"} && qq|${_}_$r->{"${_}_id"}| || () }
            ( 'agency', 'program' );

        $r->{'norm'} = lc $r->{'name'};
        $r->{'norm'} =~ s/^(a|an|all|the)\s+//i;
    }
    return [ sort { $a->{'norm'} cmp $b->{'norm'} } @entries ];
}

1;
