# Misc HTML formatting subroutines

package MVHub::HTML;
use strict;
use warnings;

use MVHub::Utils 'assert';

#### PUBLIC ####

# Make an HTML table from an array ref of contents
# args:
#    content_aref: each element will occupy a table cell. Required.
#    columns: specified number of columns. Required.
#    table_attributes: String (not hash) of table attributes. Optional.
#
sub make_html_table {
    my %args             = @_;
    my $content_aref     = delete $args{content_aref};
    my $columns          = delete $args{columns};
    my $table_attributes = delete $args{table_attributes};
    assert( ( scalar keys %args ) == 0, "Unknown arguments: @{[%args]}" );
    assert( $columns > 0,  "Must provide a positive columns argument" );
    assert( $content_aref, "Missing table content" );

    my $rows = ceil( @$content_aref / $columns );

    my $code = '<table summary="results"' . $table_attributes . '>';
    for ( my $row = 1; $row <= $rows; $row++ ) {
        $code .= "\n<tr>";
        for ( my $col = 1; $col <= $columns; $col++ ) {
            my $cell_content
                = $content_aref->[ ( ( $col - 1 ) * $rows ) + $row - 1 ];
            $code .= "\n\t<td>";
            $code .= $cell_content || '&nbsp;';
            $code .= "</td>";
        }
        $code .= "</tr>";
    }
    $code .= "\n</table>";
    return $code;
}

#### PRIVATE ####

# Why import the whole POSIX library just for one sub?
sub ceil {
    my $number = shift;
    return $number > int($number) ? int( $number + 1 ) : $number;
}

1;
