
package MVHub::HTMLTemplateUtils;
require Exporter;

our @ISA    = qw(Exporter);
our @EXPORT = qw(columnize format_template);
our ($VERSION) = '$Revision: 1315 $' =~ /([.\d]+)/;

use strict;
use warnings;
use Carp;
use HTML::Template;

use MVHub::Utils ('assert');
use MVHub::ArgChecker('get_args');

# Convenience subroutine for formatting the given template file with the
# given hashref of parameters. Unless the print argument is true, the
# result will be returned. Otherwise, the result will be printed with a
# content type text/html. Along with print, a specific doctype string
# can be supplied, or if just set to 1, a default doctype will be printed.
# Any other options that you would pass to HTML::Template's constructor
# you can provide via options_href
sub format_template {
    my ( $filename, $param_href, $print, $doctype, $options_href ) = get_args(
        [ 'filename', '?param_href', '?print', '?doctype', '?options_href' ],
        @_
    );

    my %new_args;
    if ($options_href) {
        %new_args = %$options_href;
    }
    $new_args{filename} = $filename;
    my $tmpl = HTML::Template->new(%new_args)
        or die "Creation of template object from $filename failed";
    if ( defined $param_href ) {
        $tmpl->param($param_href);
    }
    my $output = $tmpl->output();
    if ($print) {
        print "Content-Type: text\/html\n\n";
        if ($doctype) {
            if ( length($doctype) == 1 ) {
                print qq(
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"\n
"http://www.w3.org/TR/html4/loose.dtd">\n
);
            }
            else {
                print "$doctype\n";
            }
        }
        print $output;
    }
    else {
        return $output;
    }
}

# Check on any feedback from post to Monks; use some of the
#    comments you put there as documentation :
#    http://www.perlmonks.org/index.pl?node_id=274123
sub columnize {
    my $flat_loop_ref = shift;
    my $num_cols      = shift;
    my $order         = shift;
    if (   !defined $flat_loop_ref
        or !defined $num_cols
        or !defined $order )
    {
        croak("Error: Missing argument(s) to columnize()\n");
    }
    if ( $num_cols < 1 ) {
        croak
            "Error: Bad column-number argument ($num_cols) to columnize()\n";
    }
    if ( $order !~ /^(across|down)$/i ) {
        croak "Error: Bad order argument ($order) to columnize()\n";
    }

    use integer;    # use integer division for all division operations

    my $num_rows    = ( $#$flat_loop_ref / $num_cols ) + 1;
    my @result_rows = ();
    for ( my $row = 0; $row < $num_rows; $row++ ) {
        for ( my $col = 0; $col < $num_cols; $col++ ) {
            my $index;
            if ( $order eq "across" ) {
                $index = ( $row * $num_cols ) + $col;
            }
            else {
                $index = ( $col * $num_rows ) + $row;
            }

            # Skip any empty spots in the last row (if "across")
            # or in the last column (if "down"):
            next if ( $index > $#$flat_loop_ref );

            $result_rows[$row]->{COLUMN_LOOP}->[$col]
                = $flat_loop_ref->[$index];
        }
    }
    return \@result_rows;
}

1;
