
package MVHub::Headings;

use strict;
use warnings;

use MVHub::Utils;

use MVHub::Common;

our ($VERSION) = '$Revision: 1215 $' =~ /([.\d]+)/;

=head2 new()

Initializes a Heading object from the database.

=cut 

sub new {
    my $invocant = shift;
    my $class = ref($invocant) || $invocant;

    my $dbh = MVHub::Utils::DB::get_dbh();

    my $self = { HEADING_CATEGORIES_HREF => _fetch_heading_categories() };

    bless( $self, $class );
    return $self;
}

=head2 heading_names

Returns an array of heading names sorted alphabetically.

=cut

sub heading_names {
    my $self = shift;
    return sort( keys( %{ $self->{HEADING_CATEGORIES_HREF} } ) );
}

=head2 categories($heading_name)

# Returns a reference to a list of categories belonging to the given heading.
# Each category is a hashref of the form:
#    {category_id => 800011, category_name => 'Dance'}
# The list is sorted by category_name

=cut

sub categories {
    my $self         = shift;
    my $heading_name = shift;

    my $categories_aref = $self->{HEADING_CATEGORIES_HREF}->{$heading_name};

    MVHub::Utils::assert(
        defined $categories_aref && @$categories_aref,
        "$heading_name is not a known heading"
    );
    return $categories_aref;
}

# private
# Returns a hashref of heading names, each keyed to a list of their categories.
# Each category is a hashref with keys: category_id and category_name.
# e.g.:
# {
#     'Arts/Culture/Entertainment' => [
#                                         {category_id => 800011, category_name => 'Dance'},
#                                         {category_id => 800055, category_name => 'Music'},
#                                         {category_id => 800033, category_name => 'Visual Arts'},
#                                         ...
#                                      ],
#      ...
# }
sub _fetch_heading_categories {

    my $dbh = MVHub::Utils::DB::get_dbh();
    my $sql = MVHub::Utils::DB::get_sql_select_statement(
        "CATEGORY_HEADING_CATEGORY_X_FETCH_HEADING_CATEGORY");

    my $rows_aref = $dbh->selectall_arrayref( $sql, { Slice => {} } );

    my %heading_categories;
    foreach my $row_href (@$rows_aref) {
        my $heading_name = delete $row_href->{heading_name};
        push( @{ $heading_categories{$heading_name} }, $row_href );
    }
    return \%heading_categories;
}
