#!/usr/bin/perl -w

use strict;
use warnings;

use Test::More tests => 11 + 1;    # +1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;
use MVHub::ArgChecker;

my $test_message;
my $result = 1;

$test_message = "returns all params, even optional ones";
$result       = MVHub::ArgChecker::get_args(
    [ 'filename', '?param_href', '?print', '?doctype', '?options_href' ],
    (   'filename'     => 1,
        'param_href'   => 1,
        'print'        => 1,
        'doctype'      => 1,
        'options_href' => 1
    )
);
is( $result, 5, $test_message );

$test_message = "dies if parameter description is not defined";
dies_ok {
    MVHub::ArgChecker::get_args( undef, () );
};

$test_message = "dies if parameter description is not an array ref";
dies_ok {
    MVHub::ArgChecker::get_args( "not an array reference", () );
};

$test_message = "dies if missing required param and missing optional params";
dies_ok {
    MVHub::ArgChecker::get_args(
        [ 'filename', '?param_href', '?print', '?doctype', '?options_href' ],
        ()
    );
};

$test_message = 'die on missing required parameter';
dies_ok { MVHub::ArgChecker::get_args( [ 'name', 'email', 'address' ], () ) }
$test_message;

$test_message
    = q/don't die if no required or optional parameters and no parameters passed/;
$result = MVHub::ArgChecker::get_args( [], () );
is( $result, 0, $test_message );

$test_message = q/ die if extra parameters are passed/;
dies_ok {
    MVHub::ArgChecker::get_args( [],
        ( 'name' => 'yash', 'email' => 'yash@thecsl.org' ) );
}
$test_message;

$test_message = q/don't die if (3) required parameters are present/;
$result       = MVHub::ArgChecker::get_args(
    [ 'name', 'email', 'address' ],
    (   'name'    => 'yash',
        'email'   => 'yash@thecsl.org',
        'address' => '133 univ ave'
    )
);
is( $result, 3, $test_message );

$test_message = "die on unkown arguments or on extra arguments\n";
dies_ok {
    MVHub::ArgChecker::get_args( [ 'name', 'email', 'address' ],
        ( 'name' => 1, 'email' => 1, 'address' => 1, 'extra_param' => 1 ) );
}
$test_message;

$test_message = "doesn't die if missing optional params";
$result
    = MVHub::ArgChecker::get_args(
    [ 'filename', '?param_href', '?print', '?doctype', '?options_href' ],
    ( 'filename' => '/tmp/tmp.txt' ) );
is( $result, 5, $test_message );

$test_message
    = q/don't die if all parameters are optional and no params passed/;
$result = MVHub::ArgChecker::get_args( [ '?email', 'name?' ], () );
is( $result, 2, $test_message );
