#!/usr/bin/perl -w

use strict;

use Test::More tests => 12;
use CGI;
use MVHub::HTMLTemplateUtils;

if (@ARGV) {
    my ( $columns, $ordering ) = @ARGV;

    my $before = [
        { id => 1,  name => 'A' },
        { id => 2,  name => 'B' },
        { id => 3,  name => 'C' },
        { id => 4,  name => 'D' },
        { id => 5,  name => 'E' },
        { id => 6,  name => 'F' },
        { id => 7,  name => 'G' },
        { id => 8,  name => 'H' },
        { id => 9,  name => 'I' },
        { id => 10, name => 'J' },
        { id => 11, name => 'K' }
    ];

    my $after
        = MVHub::HTMLTemplateUtils::columnize( $before, $columns, $ordering );
    use Data::Dumper;
    print Dumper($after);
    exit(0);
}

# TEST PLAN
# For both "across" and "down":
#    On a simple data set of 5 items:
#       2 middle cases: MVHub::HTMLTemplateUtils::columns = 2, 3 (also 4 for down)
#       3 boundary cases: MVHub::HTMLTemplateUtils::columns = 1, 5
#       1 case where MVHub::HTMLTemplateUtils::column number exceeds data size: MVHub::HTMLTemplateUtils::columns = 10
#
# 1 case with a more complex data structure.

my $loop_ref = [
    { name => 'A' },
    { name => 'B' },
    { name => 'C' },
    { name => 'D' },
    { name => 'E' }
];

my $across_1 = [
    { 'COLUMN_LOOP' => [ { 'name' => 'A' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'B' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'C' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'D' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'E' } ] }
];

is_deeply( columnize( $loop_ref, 1, "across" ), $across_1 );

my $across_2 = [
    { 'COLUMN_LOOP' => [ { 'name' => 'A' }, { 'name' => 'B' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'C' }, { 'name' => 'D' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'E' } ] }
];

is_deeply( columnize( $loop_ref, 2, "across" ), $across_2 );

my $across_3 = [
    {   'COLUMN_LOOP' =>
            [ { 'name' => 'A' }, { 'name' => 'B' }, { 'name' => 'C' } ]
    },
    { 'COLUMN_LOOP' => [ { 'name' => 'D' }, { 'name' => 'E' } ] }
];
is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 3, "across" ),
    $across_3 );

my $across_5 = [
    {   'COLUMN_LOOP' => [
            { 'name' => 'A' },
            { 'name' => 'B' },
            { 'name' => 'C' },
            { 'name' => 'D' },
            { 'name' => 'E' }
        ]
    }
];

is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 5, "across" ),
    $across_5 );
is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 10, "across" ),
    $across_5 );    # same as across 5

is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 1, "down" ),
    $across_1 );    #same as across

my $down_2 = [
    { 'COLUMN_LOOP' => [ { 'name' => 'A' }, { 'name' => 'D' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'B' }, { 'name' => 'E' } ] },
    { 'COLUMN_LOOP' => [ { 'name' => 'C' } ] }
];

is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 2, "down" ),
    $down_2 );

my $down_3 = [
    {   'COLUMN_LOOP' =>
            [ { 'name' => 'A' }, { 'name' => 'C' }, { 'name' => 'E' } ]
    },
    { 'COLUMN_LOOP' => [ { 'name' => 'B' }, { 'name' => 'D' } ] }
];

is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 3, "down" ),
    $down_3 );

# For reasons stated in columnize docs, 4 cols and 3 cols will be same:
is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 4, "down" ),
    $down_3 );

is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 5, "down" ),
    $across_5 );    #same as across
is_deeply( MVHub::HTMLTemplateUtils::columnize( $loop_ref, 10, "down" ),
    $across_5 );    #same as across

my $struct = [
    { id => 1, name => 'April' },
    { id => 2, name => 'Dan' },
    { id => 3, name => 'David' },
    { id => 4, name => 'Kevin' }
];
my $struct_down_2 = [
    {   COLUMN_LOOP =>
            [ { id => 1, name => 'April' }, { id => 3, name => 'David' } ]
    },
    {   COLUMN_LOOP =>
            [ { id => 2, name => 'Dan' }, { id => 4, name => 'Kevin' } ]
    }
];
is_deeply( MVHub::HTMLTemplateUtils::columnize( $struct, 2, "down" ),
    $struct_down_2 );
