#!/usr/bin/perl

use strict;
use warnings;

use HTML::Template;

use Test::More tests => 9 + 1;    # +1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;
use Test::Warn;

use TestData qw/ @agency /;
use MVHub::Notifications qw/ _push_and_return_programs /;

my $test_message;
my @agency_dummy_data = @TestData::agency;

###
$test_message = 'dies on > 2 parameters';
###
dies_ok { _push_and_return_programs(qw/1 2 3 4/) } $test_message;

###
$test_message = 'dies on < 2 parameters';
###
dies_ok { _push_and_return_programs(1) } $test_message;

###
$test_message = 'dies when 1st parameter not an array reference';
###
dies_ok {
    _push_and_return_programs( 1, "pushme" );
}
$test_message;

###
$test_message = 'dies on undefined 2nd parameter';
###
dies_ok {
    _push_and_return_programs( $agency_dummy_data[0]{define_me}, undef );
}
$test_message;

###
$test_message = 'handles undefined array reference';
###
lives_ok {
    $agency_dummy_data[0]{define_me}
        = _push_and_return_programs( $agency_dummy_data[0]{define_me},
        "pushme" );
}
$test_message;

###
$test_message = 'creates array ref from undefined parameter';
###
is ref $agency_dummy_data[0]{define_me}, 'ARRAY', $test_message;

###
$test_message = 'successfully pushes';
###
is $agency_dummy_data[0]{define_me}[0], 'pushme', $test_message;

###
$test_message = 'handles defined array reference';
###
lives_ok {
    $agency_dummy_data[0]{define_me}
        = _push_and_return_programs( $agency_dummy_data[0]{define_me},
        "pushme_again" );
}
$test_message;

###
$test_message = 'successfully pushes again';
###
is $agency_dummy_data[0]{define_me}[1], 'pushme_again', $test_message;
