#!/usr/bin/perl 

use strict;
use warnings;

use Test::More tests => 5 + 1;    # +1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;

use TestData qw/ notifications_href /;

use MVHub::Utils::ConfigSimple qw/ create_config_from /;
use MVHub::Notifications qw/ create_final_notification_emails_with
    get_email_constants_from /;

my $test_message;
my @emails;
my %test_notifications = %$TestData::notifications_href;

####
$test_message = 'dies without hash reference parameter';
####
dies_ok { create_final_notification_emails_with( 1, 2, 3 ) } $test_message;

####
$test_message = 'dies on bad template file parameter';
####
my $cfg = MVHub::Utils::ConfigSimple::create_config_from(
    $ENV{MV_TEST_CONFIG_FILE} );
my $email_constants_href
    = MVHub::Notifications::get_email_constants_from($cfg);

my $template_file = "BAD_FILE.tmpl";

dies_ok {
    create_final_notification_emails_with( $template_file,
        $email_constants_href, %test_notifications );
}
$test_message;

####
$test_message = 'Skips notification if contact and public email are the same';
####
my $template_path = $cfg->param('ABSOLUTE_PATH.template_text_dir');
$template_file = "$template_path/reminder_email_01.tmpl";

$test_notifications{'1001joe_schmoe@example.com'}{'contact_email'}
    = 'joe_public@example.com';

push @emails,
    create_final_notification_emails_with( $template_file,
    $email_constants_href, %test_notifications );
is( @emails, 4, $test_message );

####
$test_message = 'Skips notification if public_email does not exist';
####
@emails = ();
$test_notifications{'1001joe_schmoe@example.com'}{'public_email'} = undef;

push @emails,
    create_final_notification_emails_with( $template_file,
    $email_constants_href, %test_notifications );
is( @emails, 4, $test_message );

####
$test_message = 'Otherwise, process notification';
####
@emails = ();
$test_notifications{'1001joe_schmoe@example.com'}{'public_email'}
    = 'notjoe@example.com';

push @emails,
    create_final_notification_emails_with( $template_file,
    $email_constants_href, %test_notifications );
is( @emails, 5, $test_message );

