#!/usr/bin/perl -w

use strict;
use warnings;

# EDIT
use Test::More tests => 3 + 1;    # +1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;

use MVHub::Notifications qw/
    get_email_constants_from
    /;

my $test_message;

###
$test_message = 'die with missing parameter';
###

dies_ok { get_email_constants_from() } $test_message;

###
$test_message = 'die with config object missing our NOTIFICATION keys';
###
my $cfg = new Config::Simple( syntax => 'ini' );
$cfg->param(
    -block  => 'META',
    -values => { 'this_config_file' => 'fake config file created in test' }
);
dies_ok { get_email_constants_from($cfg) } $test_message;

$cfg->param(
    -block  => 'NOTIFICATION',
    -values => {
        'admin_email' => 'admin_email',
        'admin_name'  => 'admin_name',
        'admin_phone' => 'admin_phone',
        'team_name'   => 'team_name',
    }
);

$cfg->param(
    -block  => 'SITE',
    -values => { 'website_name' => 'website_name', }
);

###
$test_message = 'Got keys out we expected';
###
my $email_constants_href = get_email_constants_from($cfg);
my @expected_keys
    = qw/admin_email admin_name admin_phone team_name from website_name/;
@expected_keys = sort @expected_keys;
my @got_keys = sort keys %$email_constants_href;
is_deeply( \@got_keys, \@expected_keys, $test_message );
