#!/usr/bin/perl

use strict;
use warnings;

use HTML::Template;

use Test::More tests => 3 + 1;    # +1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;
use Test::Warn;

use MVHub::Notifications qw/ print_emails_contained_in /;

my $CFG
    = MVHub::Utils::ConfigSimple::create_config_from( $ENV{MV_CONFIG_FILE} );

my $test_message;

###
$test_message = 'lives with empty array parameter';
###
lives_ok { print_emails_contained_in( () ) } $test_message;

###
$test_message = 'lives with undef parameter';
###
lives_ok { print_emails_contained_in() } $test_message;

###
$test_message = 'Prints emails successfully';
###
my @emails;
my $email = MIME::Entity->build(
    Type    => 'text/plain',
    From    => 'tester@thecsl.org',
    To      => $CFG->param('NOTIFICATION.dev_email'),
    Subject => 'send_emails_contained_in.t: Email 1',
    Data    => 'First Email',
);
push @emails, $email;
$email = MIME::Entity->build(
    Type    => 'text/plain',
    From    => 'tester@thecsl.org',
    To      => $CFG->param('NOTIFICATION.dev_email'),
    Subject => 'send_emails_contained_in.t: Email 2',
    Data    => 'Second Email',
);
push @emails, $email;

my $output;
open my $fh, '>', \$output;
{
    local *STDOUT = $fh;
    print_emails_contained_in(@emails);
    like( $output, qr/Subject: send_emails_contained_in.t: Email 1/,
        $test_message );
}
close $fh;
