#!/usr/bin/perl

use strict;
use warnings;
use Carp;

use Date::Calc;
use DBI;

use Test::More tests => 4 + 1;    # +1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;

use MVHub::Notifications qw/ increment_reminder_levels_for
    set_reminder_level_in_db /;
use MVHub::Utils::DB qw / get_dbh /;
use TestHelper qw/ add_test_data create_sqlite_db /;
use TestData qw/ notifications_href /;

my $cfg = MVHub::Utils::ConfigSimple::create_config_from(
    $ENV{MV_TEST_CONFIG_FILE} );
my $max_notifications = $cfg->param('NOTIFICATION.max_notifications');
my $expire_months     = $cfg->param('NOTIFICATION.expire_months');

my $test_message;

###
$test_message = 'die with missing parameters';
###
dies_ok { set_reminder_level_in_db(1) } $test_message;

my $dbh = MVHub::Utils::DB::get_dbh( $ENV{MV_TEST_CONFIG_FILE} );
TestHelper::create_sqlite_db($dbh) or croak "failed to setup test database\n";

my @date
    = Date::Calc::Add_Delta_YM( Date::Calc::Today(), 0, -($expire_months) );
my $expire_date = join '-', @date;

###
$test_message = 'No Agency / Program records need updating';
###
lives_ok {
    set_reminder_level_in_db( $dbh, $expire_date, undef,
        'AGENCY_X_REMINDERS_SENT' );
}
$test_message;

TestHelper::add_test_data($dbh);
increment_reminder_levels_for( $TestData::notifications_href, 5 );
###
$test_message = 'Agency Test Data equals expected result after update';
###
set_reminder_level_in_db( $dbh, $expire_date, $TestData::notifications_href,
    'AGENCY_X_REMINDERS_SENT' );
is_deeply(
    $dbh->selectcol_arrayref("SELECT reminders_sent FROM agency"),
    [ 1, 2, 3, 4, 5 ],
    $test_message
);

###
$test_message = 'Program Test Data equals expected result after update';
###
set_reminder_level_in_db( $dbh, $expire_date, $TestData::notifications_href,
    'PROGRAM_X_REMINDERS_SENT' );
is_deeply(
    $dbh->selectcol_arrayref("SELECT reminders_sent FROM program"),
    [ 1, 2, 3, 4 ],
    $test_message
);

unlink( $cfg->param('DATABASE.database_name') );
