#!/usr/bin/perl 

use warnings;
use strict;

use Config::Simple;
use Test::Exception;
use Test::More tests => 6 + 1;
use Test::NoWarnings;

use MVHub::Utils::ConfigSimple qw/ _calculate_extra_cfg_params_from /;

my $cfg = new Config::Simple( syntax => 'ini' );
my $test_name;

####
$test_name = "dies if not passed in Config::Simple object";
####
dies_ok { _calculate_extra_cfg_params_from(1) } $test_name;

####
$test_name = "dies if not passed in config_filename parameter";
####
dies_ok { _calculate_extra_cfg_params_from($cfg) } $test_name;

####
$test_name = "dies if not passed in username parameter";
####
dies_ok {
    _calculate_extra_cfg_params_from( $cfg, 'fake_config_file in .t file' );
}
$test_name;

####
$test_name = "dies if BASE.dir or RELATIVE_PATHS  not set";
####
dies_ok {
    _calculate_extra_cfg_params_from( $cfg, 'fake_config_file in .t file',
        'test_user' );
}
$test_name;

####
$test_name = "dies if RELATIVE_PATH not set";
####
$cfg->param( -block => 'BASE', -values => { 'dir' => '/var/tmp/' } );
dies_ok {
    _calculate_extra_cfg_params_from( $cfg, 'fake_config_file in .t file',
        'test_user' );
}
$test_name;

####
$test_name = "count of RELATIVE_PATHs eq count of ABSOLTUTE_PATHs out";
####
$cfg->param( -block => 'BASE', -values => { 'dir' => '/var/tmp/' } );
$cfg->param(
    -block  => 'RELATIVE_PATH',
    -values => {
        'test1' => 'path/one/',
        'test2' => 'second/path/',
        'test3' => 'last/one/to/do/'
    }
);

$cfg = MVHub::Utils::ConfigSimple::_calculate_extra_cfg_params_from( $cfg,
    $ENV{MV_CONFIG_FILE}, 'test_user' );
my $got = scalar keys %{ $cfg->param( -block => 'ABSOLUTE_PATH' ) };

is $got, 3, $test_name;

