#!/usr/bin/perl -w

use strict;
use warnings;

use Test::More tests => 5;

use MVHub::Utils::DB qw/get_data_source/;
use MVHub::Utils::ConfigSimple qw/create_config_from/;

use TestHelper;
eval {
    use Test::NoWarnings;
    use Test::Exception;
};
plan skip_all => ' need Test::NoWarnings Test::Exception '
    if $@;

my $test_message;
my $result = 1;
my $file_contents;
my $test_filename = 'test.tmp';

###
$test_message = 'works with SQLite ';
###
# BASE.dir RELATIVE_PATH not needed for this test,
# but must be added so create_config_from doesn't croak.

$file_contents = <<FILE;
[DATABASE]
  database_host=
  database_name=test.sqlite
  database_user=test
  # the database user's p---word
  database_magic_word=ignore
  database_type=SQLite
[BASE]
dir=/IGNORE/REALLY/IGNORE/
[RELATIVE_PATH]
dummy=/ignore/me/too

FILE

TestHelper::make_test_file( $test_filename, $file_contents );
like( get_data_source($test_filename), qr/SQLite/, $test_message );

###
$test_message = 'works with Pg (postgresql) ';
###
# BASE.dir not needed for this test, but must be added so create_config_from doesn't croak.
$file_contents = <<FILE;
[DATABASE]
  database_host=
  database_name=test
  database_user=test
  # the database user's p---word
  database_magic_word=ignore
  database_type=Pg
[BASE]
dir=/IGNORE/REALLY/IGNORE/
[RELATIVE_PATH]
dummy=/ignore/me/too
FILE
TestHelper::make_test_file( $test_filename, $file_contents );
like( get_data_source($test_filename), qr/Pg/, $test_message );

###
$test_message = 'dies on invalid $database_type ';
###
$file_contents = <<FILE;
[DATABASE]
FILE
TestHelper::make_test_file( $test_filename, $file_contents );

dies_ok { get_data_source($test_filename) } $test_message;

###
$test_message = 'dies on missing or unreadable config file';
###

unlink $test_filename or warn "failed to deleted $test_filename\n";
dies_ok { get_data_source($test_filename) } $test_message;
