#!/usr/bin/perl

use strict;
use warnings;

use Test::More tests => 4 + 1;    # + 1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;
use Test::MockObject;
use Config::Simple;

use MVHub::Utils::Setup;

my $test_message;

###
$test_message = 'dies when missing 1st parameter';
###
dies_ok { MVHub::Utils::Setup::_db_user_has_superuser_role() } $test_message;

###
$test_message = 'dies when missing 2nd parameter';
###
dies_ok { MVHub::Utils::Setup::_db_user_has_superuser_role(1) } $test_message;

###
$test_message = 'returns 1 if user is a superuser';
###
my $mock_dbh = Test::MockObject->new();
$mock_dbh->mock(
    'selectall_arrayref',
    sub {
        ( my $not_used, my $not_used_2, my @user_name ) = @_;
        if ( $user_name[1] eq 'true' ) {
            return [ { 'rolsuper' => '1' } ];
        }
        else { return [] }
    }
);

is( MVHub::Utils::Setup::_db_user_has_superuser_role( $mock_dbh, 'true' ),
    '1', $test_message );

###
$test_message = 'returns 0 if user is not a superuser';
###
is( MVHub::Utils::Setup::_db_user_has_superuser_role( $mock_dbh, 'false' ),
    '0', $test_message );
