#!/usr/bin/perl

use strict;
use warnings;

use English '-no_match_vars';

use File::Basename;
use File::Copy;

use Test::More tests => 3 + 1;    # + 1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;
use Test::Warn;
use Test::MockObject;
use Config::Simple;

use MVHub::Utils::Setup;

my $test_message;

###
$test_message
    = 'dies when sql file in directory does not conform to naming standard';
###
my $test_filename = "/tmp/get_sql_db_update_scripts_$PID.sql";

open my $test_fh, ">", $test_filename;
print $test_fh "This a test file. It can be deleted.\n";
close $test_fh;

dies_ok {
    MVHub::Utils::Setup::get_sql_db_update_scripts(
        path    => '/tmp/',
        version => 0
    );
}
$test_message;

###
$test_message = 'Adds scripts to array if properly formatted';
###
my $renamed = "/tmp/001_get_sql_db_update_scripts_$PID.sql";
File::Copy::move( $test_filename, $renamed ) or die;

my @returned_scripts = MVHub::Utils::Setup::get_sql_db_update_scripts(
    path    => '/tmp/',
    version => 0
);
is( scalar @returned_scripts, 1, $test_message );

###
$test_message = 'Does not add scripts if less than version number given';
###

@returned_scripts = MVHub::Utils::Setup::get_sql_db_update_scripts(
    path    => '/tmp/',
    version => 1
);
is( scalar @returned_scripts, 0, $test_message );

unlink $renamed;
