#!/usr/bin/perl

use strict;
use warnings;

use English '-no_match_vars';

use Test::More tests => 6 + 1;    # + 1 for NoWarnings
use Test::NoWarnings;
use Test::Exception;

use MVHub::Utils::Setup;

my $test_message;
###
$test_message = 'dies when not passed in array';
###
dies_ok { MVHub::Utils::Setup::make_dev_home_files() } $test_message;

###
$test_message = 'dies when passed invalid array';
###
my @test_data = ();
dies_ok { MVHub::Utils::Setup::make_dev_home_files(@test_data) }
$test_message;

###
$test_message = 'dies when passed array with invalid file path';
###
@test_data = ( { dest => '/this/is/a/bad/path' } );
dies_ok { MVHub::Utils::Setup::make_dev_home_files(@test_data) }
$test_message;

###
$test_message = 'lives when passed valid array';
###
my $group_id = ( split ' ', $GID )[0];
@test_data = (
    {   dest        => "/tmp/make_dev_home_files_t_$PID.txt",
        owner       => $UID,
        group       => $group_id,
        permissions => 'u=rwx,g=rwx,o=rx',
        contents    => <<"CONTENT",
This is a test file created by make_dev_home_files.t. You can delete it.
CONTENT
    },
);
lives_ok { MVHub::Utils::Setup::make_dev_home_files(@test_data) }
$test_message;

###
$test_message = 'test file created correctly';
###
open FILE, "<$test_data[0]->{dest}";
my $line = <FILE>;
close FILE;
is( $line, $test_data[0]->{contents}, $test_message );

###
$test_message = 'Backup created when file already exists';
###
my $original_file = $test_data[0]->{dest};
MVHub::Utils::Setup::make_dev_home_files(@test_data);
ok( ( -e "$test_data[0]->{dest}" ), $test_message );

unlink $original_file;
unlink $test_data[0]->{dest};
