#!/usr/bin/perl -w

use strict;
use Test::More tests => 7;
use CGI;
use MVHub::Utils;

my $q = CGI->new();
my $test_name;

####
$test_name = "parameterless cgi object ummodified";
####
is_deeply( $q, MVHub::Utils::clean_cgi_params($q), $test_name );

####
$test_name = "single clean parameter doesn't change";
####
$q->param( clean => "A clean parameter" );
MVHub::Utils::clean_cgi_params($q);

is( $q->param("clean"), "A clean parameter", $test_name );

####
$test_name = "multiple clean parameters don't change";
####
$q->param( "cleaner" => "clean" );
$q->param( "clean"   => "A clean parameter" );
MVHub::Utils::clean_cgi_params($q);

is( $q->param("clean") . $q->param("cleaner"),
    "A clean parameterclean", $test_name );

####
$test_name = "destroys entire param with HTML tags";
####
$q->param( all_html_code => "<script type=\"text/javascript\">" );
MVHub::Utils::clean_cgi_params($q);
is( $q->param("all_html_code"), "", $test_name );

####
$test_name = "removes HTML tags and leaves other text";
####
$q->param( part_html_code => "<p>This is a paragraph</p>" );
MVHub::Utils::clean_cgi_params($q);

is( $q->param("part_html_code"), "This is a paragraph", $test_name );

####
$test_name = "multitple clean parameters with same name don't change";
####
$q->param( multi_clean => "clean", "cleaner", "cleanest" );
MVHub::Utils::clean_cgi_params($q);
{
    my @param = $q->param("multi_clean");
    ok( (           ( $param[0] eq "clean" )
                and ( $param[1] eq "cleaner" )
                and ( $param[2] eq "cleanest" )
        ),
        $test_name

    );
}

####
$test_name
    = "multitple mixed parameters with same name are selectively cleaned";
####
$q->param(
    mixed_multi => "clean",
    "<input>not clean</p>", "sorta", "<br><hr>"
);
&MVHub::Utils::clean_cgi_params($q);
{
    my @param = $q->param("mixed_multi");
    ok( (           ( $param[0] eq "clean" )
                and ( $param[1] eq "not clean" )
                and ( $param[2] eq "sorta" )
                and ( $param[3] eq "" )
        ),
        $test_name
    );
}
