#!/usr/bin/perl -w

use strict;
use Test::More tests => 7;
use MVHub::Utils;

BEGIN {
    use FindBin qw($Bin);
    chdir $Bin;

    use_ok( 'MVHub::Utils', qw/ trim / ) or die "couldn't use Utils";
}

my ( $in, $out, $test_name );

######
$test_name = 'text with leading space';
######
$in  = '  test';
$out = 'test';
is( MVHub::Utils::trim($in), $out, $test_name );

######
$test_name = 'text with trailing space';
######
$in  = 'test   ';
$out = 'test';
is( MVHub::Utils::trim($in), $out, $test_name );

######
$test_name = 'text with leading and trailing spaces';
######
$in  = ' test   ';
$out = 'test';
is( MVHub::Utils::trim($in), $out, $test_name );

######
$test_name = 'call trim with no parameters';
######
$in  = ' test   ';
$out = 'test';
is( MVHub::Utils::trim(), undef, $test_name );

################ array tests ############

my ( @in, @expected_out, @out );

######
$test_name = 'call MVHub::Utils::trim in list context with a list';
######
@in           = ( " a", "b ", " c" );
@expected_out = qw/a b c/;
@out          = MVHub::Utils::trim(@in);
is_deeply( \@out, \@expected_out, $test_name );

######
$test_name
    = 'call MVHub::Utils::trim in list context with a list that has an undefined element';
######
@in = ( " a", "b ", " c" );
$in[2]           = undef;
@expected_out    = qw/a b/;
$expected_out[2] = undef;
@out             = MVHub::Utils::trim(@in);
is_deeply( \@out, \@expected_out, $test_name );

