package MVHub::AgencyAccount::Test;
use strict;
use warnings;

#TODO fix
# line below doesn't fail
# gracefully if module
# isn't installed
# can't wrap in eval
# because (apparently) to work modue
# module needs to be 'use'd
use Test::MockObject::Extends;

use Test::Exception;
use Test::Class;
use Test::More;

use MVHub::AgencyAccount;
use base qw(Test::Class MVHub::CGIAppBase );
use CGI;

# CGI::Application->run() returns output instead of sending to stdout
$ENV{CGI_APP_RETURN_ONLY} = 1;

# redefine MVHub::Common:: subs
# as to be more easily testable

# from Perl Testing: A Developer's Notebook
# chapter 5
{

    package MVHub::Common;

    use subs qw/sendmail /;

    package main;

    no warnings;
    *MVHub::Common::sendmail = sub {
        return 1;
    };
    use warnings;
}

{

    package MVHub::AuthAccount;

    use subs qw/is_correct_password/;

    package main;

    no warnings;

    *MVHub::AuthAccount::is_correct_password = sub {
        my ( $agency_id, $password ) = @_;
        return $password;
    };
    use warnings;
}

# setup methods are run before every test method.
sub make_fixture : Test(setup) {
    my $self = shift;
    $self->{test_cgi} = CGI->new();
}

sub show_login_page : Tests(3) {
    my $self = shift;

    my $test_cgi = $self->{test_cgi};
    $test_cgi->param( -name => 'rm', -value => 'show_login_page' );

    $test_cgi->param( agency_id => '99' );
    my $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );

    my $test_name;

    $test_name = 'show_login_page() before redirects';
    like( $app->run(), qr(<H3>Agency Log-In Page</H3>), $test_name );

    $test_name = 'show_login_page() with set error msg';
    $app->param( 'login_error', 'error msg is set to some text' );
    like( $app->run(), qr(error msg is set to some text), $test_name );

    $test_name = 'show_login_page() dies when template file is missing';
    my $mock = Test::MockObject::Extends->new($app);
    $mock->set_false('load_tmpl');
    dies_ok { $app->run() } $test_name;

}

sub request_password : Tests(3) {
    my $self = shift;

    my $test_cgi = $self->{test_cgi};
    $test_cgi->param( -name => 'rm', -value => 'request_password' );

    my $app;
    my $test_name;
##
    $test_name = 'request_password() called without email param';
##
    $test_cgi->param( agency_id => undef );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    like( $app->run(), qr{<h3>Password Reminder</h3}, $test_name, );

##
    $test_name = 'request_password() email param not in db';
##
    $test_cgi->param(
        -name  => 'email',
        -value => 'email not in database test'
    );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    like( $app->run(),
        qr(no agency or program contact listed in our database), $test_name,
    );

##
    $test_name = 'request_password() email param not in db';
##
    #TODO fix kludge
    $ENV{MV_CONFIG_FILE} = $ENV{MV_TEST_CONFIG_FILE};
    $test_cgi->param( -name => 'email', -value => 'dan@thecsl.org' );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    like( $app->run(), qr(We've sent you an email with your password),
        $test_name, );

}

sub do_login : Tests(4) {
    my $self = shift;
    my ( $test_cgi, $app, $test_name );

    $test_cgi = $self->{test_cgi};
    $test_cgi->param( -name => 'rm', -value => 'do_login' );

##
    $test_name = "do_login() called with \$agency_id == undef";
##
    $test_cgi->param( agency_id => undef );
    $test_cgi->param( -name => 'rm', -value => 'do_login' );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    like( $app->run(), qr{error_code.Please.select.an.agency.}, $test_name );
##
    $test_name = "do_login() called with  \$agency_id eq q{} ";
##
    $test_cgi->param( agency_id => q{} );
    $test_cgi->param( -name => 'rm', -value => 'do_login' );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    like( $app->run(), qr{error_code.Please\+select\+an\+agency.},
        $test_name );
##
    #    $test_name = "do_login() called with fake invalid \$agency_password";
##
    #   $test_cgi->param( agency_id       => '555' );
    #   $test_cgi->param( agency_password => '0' );
    #   $test_cgi->param( -name => 'rm', -value => 'do_login' );
    #   $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    #   like( $app->run(), qr{error_code.Incorrect.password.for.this.agency},
    #       $test_name );
##
    $test_name = "do_login(): fake valid \$agency_password, no cookie";
##
    $test_cgi->param( agency_id       => '555' );
    $test_cgi->param( agency_password => 1 );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi );
    like( $app->run(), qr{unable to set a cookie in your browser},
        $test_name );
##
    $test_name = "do_login(): \$agency_id set , good \$password, cookie set";
##
    $test_cgi->param( agency_id => 555 );
    my $test_cgi_with_mock_cookie
        = Test::MockObject::Extends->new( $test_cgi, );

    $test_cgi_with_mock_cookie->mock( 'cookie',
        sub { return 'COOKIE_SET'; } );

    $test_cgi_with_mock_cookie->param( agency_password => 1 );
    $app = MVHub::AgencyAccount->new( 'QUERY' => $test_cgi_with_mock_cookie );

    like( $app->run(), qr{rm.*display_agency_home}, $test_name );
}

# all good modules return 1;
1;

