package TestData;

use strict;
use warnings;

use MVHub::Utils::ConfigSimple;

BEGIN {
    use Exporter ();
    our (@EXPORT_OK);

    @EXPORT_OK = qw(@agency @program notifications_href);
}
our @EXPORT_OK;

my $test_data_cfg
    = MVHub::Utils::ConfigSimple::create_config_from( $ENV{MV_CONFIG_FILE} );
my $dev_user = $test_data_cfg->param('NOTIFICATION.dev_email');

our @agency = (
    {   agency_id          => '1001',
        agency_name        => 'Test Agency 1',
        record_name        => 'Test Agency 1 (Main Agency Record)',
        contact_first_name => 'Dan',
        contact_last_name  => 'MacNeil',
        contact_email      => 'dan@thecsl.org',
        email              => 'joe_public@example.com',
        last_updated       => '2009-02-15',
        reminders_sent     => '0',
    },
    {   agency_id          => '1002',
        agency_name        => 'Test Agency 2',
        record_name        => 'Test Agency 2 (Main Agency Record)',
        contact_first_name => 'Jane',
        contact_last_name  => 'Lane',
        contact_email      => 'jane_lane@example.com',
        email              => 'jane_public@example.com',
        last_updated       => '2008-12-02',
        reminders_sent     => '1',
    },
    {   agency_id          => '1003',
        agency_name        => 'Test Agency 3',
        record_name        => 'Test Agency 3 (Main Agency Record)',
        contact_first_name => 'Jack',
        contact_last_name  => 'Black',
        contact_email      => 'jack_black@example.com',
        email              => 'jack_public@example.com',
        last_updated       => '2008-06-05',
        reminders_sent     => '2',
    },
    {   agency_id          => '1004',
        agency_name        => 'Test Agency 4',
        record_name        => 'Test Agency 4 (Main Agency Record)',
        contact_first_name => 'Sue',
        contact_last_name  => 'Blue',
        contact_email      => 'sue_blue@example.com',
        email              => 'sue_public@example.com',
        last_updated       => '2007-08-20',
        reminders_sent     => '1',
    },
    {   agency_id          => '1005',
        agency_name        => 'Test Agency 5',
        record_name        => 'Test Agency 5 (Main Agency Record)',
        contact_first_name => 'Bob',
        contact_last_name  => 'Cobb',
        contact_email      => 'bobcobb@example.com',
        email              => 'bob_public@example.com',
        last_updated       => '2007-08-20',
        reminders_sent     => '4',
    },
);

our @program = (
    {   agency_id          => '1001',
        agency_name        => 'Test Agency 1',
        record_name        => 'Test Program 1',
        contact_first_name => 'Joe',
        contact_last_name  => 'Schmoe',
        contact_email      => 'joe_schmoe@example.com',
        email              => 'joe_public@example.com',
        last_updated       => '2009-02-15',
        reminders_sent     => '0',
    },
    {   agency_id          => '1002',
        agency_name        => 'Test Agency 2',
        record_name        => 'Test Program 2',
        contact_first_name => 'Jane',
        contact_last_name  => 'Lane',
        contact_email      => 'jane_lane@example.com',
        email              => 'jane_public@example.com',
        last_updated       => '2008-12-02',
        reminders_sent     => '0',
    },
    {   agency_id          => '1003',
        agency_name        => 'Test Agency 3',
        record_name        => 'Test Program 3',
        contact_first_name => 'Jack',
        contact_last_name  => 'Black',
        contact_email      => 'jack_black@example.com',
        email              => 'jack_publc@example.com',
        last_updated       => '2008-12-02',
        reminders_sent     => '2',
    },
    {   agency_id          => '1004',
        agency_name        => 'Test Agency 4',
        record_name        => 'Test Program 4',
        contact_first_name => 'Sue',
        contact_last_name  => 'Blue',
        contact_email      => 'sue_blue@example.com',
        email              => 'sue_public@example.com',
        last_updated       => '2008-12-02',
        reminders_sent     => '3',
    },
);

our $notifications_href = {

    '1001joe_schmoe@example.com' => {
        'contact_first_name' => 'Joe',
        'contact_last_name'  => 'Schmoe',
        'reminders_sent'     => 0,
        'expired_record_list' =>
            [ 'Test Program 1', 'Test Agency 1 (Main Agency Record)' ],
        'agency_contact_email' => 'joe_agency@example.com',
        'contact_email'        => $dev_user,
        'public_email'         => 'joe_public@example.com',
        'agency_id'            => '1001',
        'agency_name'          => 'Test Agency 1',
        'quick_login_passwd'   => '00000000',
    },
    '1002jane_lane@example.com' => {
        'contact_first_name' => 'Jane',
        'contact_last_name'  => 'Lane',
        'reminders_sent'     => 1,
        'expired_record_list' =>
            [ 'Test Program 2', 'Test Agency 2 (Main Agency Record)' ],
        'agency_contact_email' => 'jane_agency@example.com',
        'contact_email'        => $dev_user,
        'public_email'         => 'jane_public@example.com',
        'agency_id'            => '1002',
        'agency_name'          => 'Test Agency 2',
        'quick_login_passwd'   => '11111111',
    },
    '1003jack_black@example.com' => {
        'contact_first_name' => 'Jack',
        'contact_last_name'  => 'Black',
        'reminders_sent'     => 2,
        'expired_record_list' =>
            [ 'Test Program 3', 'Test Agency 3 (Main Agency Record)' ],
        'agency_contact_email' => 'jack_agency@example.com',
        'contact_email'        => $dev_user,
        'public_email'         => 'jack_public@example.com',
        'agency_id'            => '1003',
        'agency_name'          => 'Test Agency 3',
        'quick_login_passwd'   => '22222222',
    },
    '1004sue_blue@example.com' => {
        'contact_first_name' => 'Sue',
        'contact_last_name'  => 'Blue',
        'reminders_sent'     => 3,
        'expired_record_list' =>
            [ 'Test Program 4', 'Test Agency 4 (Main Agency Record)' ],
        'agency_contact_email' => 'sue_agency@example.com',
        'contact_email'        => $dev_user,
        'public_email'         => 'sue_public@example.com',
        'agency_id'            => '1004',
        'agency_name'          => 'Test Agency 4',
        'quick_login_passwd'   => '33333333',
    },
    '1005bob_cobb@example.com' => {
        'contact_first_name'   => 'Bob',
        'contact_last_name'    => 'Cobb',
        'reminders_sent'       => 4,
        'expired_record_list'  => ['Test Agency 5 (Main Agency Record)'],
        'agency_contact_email' => 'bob_agency@example.com',
        'contact_email'        => 'bob_cobb@example.com',
        'public_email'         => $dev_user,
        'agency_id'            => '1005',
        'agency_name'          => 'Test Agency 5',
        'quick_login_passwd'   => '44444444',
    },
};

# all good modules return true 1;
1
