package TestHelper;

require Exporter;

use Carp;
use DBI;
use File::Basename;
use MIME::Base64;
use Test::More;

use MVHub::Utils::DB;
use TestData;
use Params::Validate qw/:all/;

use strict;
use warnings;

our @ISA    = qw(Exporter);
our @EXPORT = qw(
    add_test_data
    create_sqlite_db
    create_temp_auth
    get_files
    get_files_from
    get_host_to_check
    get_quick_login_from
    get_test_website_name
    make_test_file
    no_difference
    pages_get_ok
    run_submit_forms_ok
    submit_forms_ok
);

our ($VERSION) = '$Revision: 1647 $' =~ /([.\d]+)/;

sub add_test_data {
    my $dbh = shift or croak "missing parameter: \$dbh\n";
    my $sql = MVHub::Utils::DB::get_sql_insert_statement(
        'AGENCY_X_REQUIRED_FIELDS');
    my @bind_variables;
    foreach my $test_data (@TestData::agency) {
        @bind_variables = (
            $test_data->{'agency_id'},     $test_data->{'agency_name'},
            $test_data->{'contact_email'}, $test_data->{'email'},
            $test_data->{'last_updated'},  $test_data->{'reminders_sent'}
        );
        $dbh->do( $sql, undef, @bind_variables );
    }
    $sql = MVHub::Utils::DB::get_sql_insert_statement(
        'PROGRAM_X_REQUIRED_FIELDS');
    foreach my $test_data (@TestData::program) {
        @bind_variables = (
            $test_data->{'agency_id'},   $test_data->{'program_id'},
            $test_data->{'record_name'}, $test_data->{'contact_email'},
            $test_data->{'email'},       $test_data->{'last_updated'},
            $test_data->{'reminders_sent'}
        );
        $dbh->do( $sql, undef, @bind_variables );
    }
}

sub create_sqlite_db {
    my $dbh = shift or croak "missing parameter: \$dbh\n";
    my $cfg = MVHub::Utils::ConfigSimple::create_config_from(
        $ENV{MV_TEST_CONFIG_FILE} );
    my $conf_path = $cfg->param('ABSOLUTE_PATH.conf_dir');
    my $sql_lib
        = SQL::Library->new( { lib => "$conf_path/create_sqlite.lib" } );

    foreach my $sql_label ( sort $sql_lib->elements() ) {
        my $sql = $sql_lib->retr($sql_label);
        $dbh->do($sql);
    }
    return 1;
}

# TODO review with ERIC
# eventually use wrap /usr/bin/htpasswd to
# add a temporary user
# will require putting password file location in all.conf
# and creating another sub remove_temp_auth
sub create_temp_auth {
    my $auth = 'Basic ' . encode_base64("test:test");
    return $auth;
}

# accept a list of wild card file patterns
# return a list (array) of all files in project directory matching
# examples:
# my @html_files=get_files('*.html', '*.shtml');
# my @html_template_files=get_files('*.tmpl');
sub get_files {

    # KLUDGE FIXME TODO get from conf file
    my $dir = "/var/www/mvhub/$ENV{USER}/link-to-live-code/";
    return get_files_from( $dir, @_ );
}

sub get_files_from {
    my $search_dir = shift;
    my $search     = " -name " . shift;
    $search .= join " ", map { " -or -name " . "'" . $_ . "'" } @_
        if scalar @_;

    my $cmd = "find  $search_dir/ $search";

    my @files = `$cmd `;
    chomp @files;

    return @files;
}

sub get_quick_login_from {
    ( my $agency_id, my $config_filename ) = Params::Validate::validate_pos(
        @_,
        { type => SCALAR },
        { type => SCALAR }
    );
    my $dbh = MVHub::Utils::DB::get_dbh($config_filename);
    my $sql = MVHub::Utils::DB::get_sql_select_statement(
        'AGENCY_X_QUICK_LOGIN_PASSWD');
    my $result = ( $dbh->selectrow_array( $sql, {}, $agency_id ) )[0];
    return $result;
}

# TODO return something like $USER.$SITE.testing123.net
# unless host eq 'groundhog';
# handle things like tests being run in staging site
# as staging user
# maybe change to: my @hosts=get_hosts_to_check
# to account for http://mvhub.com and http://northshoreport.org
sub get_hosts_to_check {
    return qw/ nsp.localhost mvh.localhost/;
}

sub get_test_website_name {
    my $prefix = shift;
    $prefix eq 'nsp' || $prefix eq 'mvh'
        or croak
        "need to supply prefix of 'mvh' or 'nsp' you supplied '$prefix'";

    croak '$ENV{USER} not defined '
        if not defined $ENV{USER};

    my $conf_path = ( File::Basename::fileparse( $ENV{MV_CONFIG_FILE} ) )[1];
    my $cfg       = MVHub::Utils::ConfigSimple::create_config_from(
        "$conf_path/$prefix.conf");

    return $cfg->param('SITE.website_name');
}

sub make_test_file {
    my ( $filename, $contents ) = @_;
    open my $OUT, '>', $filename
        or croak "couldn't open file '$filename' for writing because: $!\n";
    print $OUT $contents
        or croak "could't write to file '$filename'\n";
    close $OUT
        or croak "couldn't close file '$filename' because:$!\n";
}

sub no_difference {
    my ( $source, $dest ) = @_;
    croak "$source doesn't exist" if !-e $source;
    croak "$dest doesn't exist"   if !-e $dest;

    my $diff = `/usr/bin/diff -u $source $dest`;
    my $result = length($diff) == 0 ? 1 : 0;
    return $result;
}

sub pages_get_ok {
    my @pages = @_    or die 'pages_get_ok() missing param: @pages';

    foreach my $page (@pages) {
    SKIP: {
            skip "$page->{url} $$page{skip_reason}", 3
                if $$page{tests_to_skip} =~ 'get';

	    my $mech=$$page{mech};
            $mech->get_ok($$page{url}, "got ok: $$page{url}" );
            $mech->content_lacks(
                'an error in processing your request', "no error in: 
            " . $page->{url}
            );
            $mech->content_contains(
                $$page{expected_content}, " $$page{expected_content} 
            found in: $page->{url}"
            );
        }
    }
}

sub run_submit_forms_ok {
    my %args          = @_;
    my $prefix        = $args{prefix} or croak "missing param: \$prefix";
    my $get_forms_sub = $args{get_forms_sub}
        or croak "missing param: 'get_forms_sub'";

    my $mech = Test::WWW::Mechanize->new();

    my $host  = TestHelper::get_test_website_name('nsp');
    my @forms = &$get_forms_sub($host);

    plan tests => ( scalar @forms ) * 2;

    submit_forms_ok( $mech, @forms );

}

sub submit_forms_ok {
    my $mech  = shift or croak "missing \$mech parameter";
    my @forms = @_    or croak 'missing @forms parameter';

    my %params;
    for my $form (@forms) {
        croak 'Form not initialized'
            if not defined $$form{url};

        $mech->default_header( Authorization => $$form{auth} )
            if defined $$form{auth};

        $params{form_number} = $$form{number};
        $params{fields}      = $$form{fields};
        $params{button}      = $$form{button};

    SKIP: {
            skip $$form{skip_reason}, 2
                if $$form{tests_to_skip} =~ /submit/;
            $mech->get( $$form{url} );
            $mech->submit_form_ok( \%params,
                "$$form{url} form number:$$form{number}" );
            $mech->content_contains( $$form{expected_content} )
                or diag "dumping content: $$form{url} \n\n"
                . $mech->content();
        }    #skip
    }    #for
}    # sub

__END__

=head2 get_test_website_name

requires parameter $prefix, either 'nsp' or 'mvh'
indicating that the testing website url returned
is either a copy of the northshoreport.org or mvhub.com

returns the URL of a website that matches
the developer runing the test.
=cut
