--
-- PostgreSQL database dump
--

SET client_encoding = 'SQL_ASCII';
SET check_function_bodies = false;
SET client_min_messages = warning;

--
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--



SET search_path = public, pg_catalog;

--
-- Name: get_agency_name(integer); Type: FUNCTION; Schema: public; Owner: omacneil
--

CREATE FUNCTION get_agency_name(integer) RETURNS text
    AS $_$select agency_name from agency where agency_id = $1;$_$
    LANGUAGE sql STABLE STRICT;


--
-- Name: get_password(integer); Type: FUNCTION; Schema: public; Owner: omacneil
--

CREATE FUNCTION get_password(integer) RETURNS text
    AS $_$select password from agency where agency_id = $1;$_$
    LANGUAGE sql STABLE STRICT;


--
-- Name: our_concat(text, text); Type: FUNCTION; Schema: public; Owner: omacneil
--

CREATE FUNCTION our_concat(text, text) RETURNS text
    AS $_$select $1 || ', ' || $2;$_$
    LANGUAGE sql STABLE STRICT;


SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: agency; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE agency (
    agency_id integer DEFAULT nextval(('agency_id_sequence'::text)::regclass) NOT NULL,
    last_updated date NOT NULL,
    hidden boolean DEFAULT false NOT NULL,
    "password" text NOT NULL,
    agency_name text NOT NULL,
    agency_alias text,
    address1 text NOT NULL,
    address2 text,
    city text NOT NULL,
    state text NOT NULL,
    zip text NOT NULL,
    main_phone text NOT NULL,
    toll_free_phone text,
    hotline text,
    fax text,
    website text,
    email text,
    mission text,
    hours text NOT NULL,
    other_info text,
    mail_address1 text,
    mail_address2 text,
    mail_city text,
    mail_state text,
    mail_zip text,
    chief text NOT NULL,
    title text NOT NULL,
    parent_organization text,
    resource_library text NOT NULL,
    resource_library_description text,
    meeting_space text NOT NULL,
    meeting_space_description text,
    staff_internet_access text NOT NULL,
    contact_first_name text NOT NULL,
    contact_last_name text NOT NULL,
    contact_phone text NOT NULL,
    contact_fax text,
    contact_email text NOT NULL,
    contact_email_available text NOT NULL,
    tty_tdd text,
    reminders_sent integer DEFAULT 0 NOT NULL,
    admin_notes text,
    CONSTRAINT agency_address1 CHECK ((address1 <> ''::text)),
    CONSTRAINT agency_agency_name CHECK ((agency_name <> ''::text)),
    CONSTRAINT agency_chief CHECK ((chief <> ''::text)),
    CONSTRAINT agency_city CHECK ((city <> ''::text)),
    CONSTRAINT agency_contact_email CHECK ((contact_email <> ''::text)),
    CONSTRAINT agency_contact_email_available CHECK ((contact_email_available <> ''::text)),
    CONSTRAINT agency_contact_first_name CHECK ((contact_first_name <> ''::text)),
    CONSTRAINT agency_contact_last_name CHECK ((contact_last_name <> ''::text)),
    CONSTRAINT agency_contact_phone CHECK ((contact_phone <> ''::text)),
    CONSTRAINT agency_hours CHECK ((hours <> ''::text)),
    CONSTRAINT agency_main_phone CHECK ((main_phone <> ''::text)),
    CONSTRAINT agency_meeting_space CHECK ((meeting_space <> ''::text)),
    CONSTRAINT agency_password CHECK (("password" <> ''::text)),
    CONSTRAINT agency_resource_library CHECK ((resource_library <> ''::text)),
    CONSTRAINT agency_staff_internet_access CHECK ((staff_internet_access <> ''::text)),
    CONSTRAINT agency_state CHECK ((state <> ''::text)),
    CONSTRAINT agency_title CHECK ((title <> ''::text)),
    CONSTRAINT agency_zip CHECK ((zip <> ''::text))
);


--
-- Name: agency_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE agency_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: agency_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('agency_id_sequence', 103993, true);


--
-- Name: search_log_search_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE search_log_search_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: search_log_search_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('search_log_search_id_seq', 36826, true);


--
-- Name: search_log; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE search_log (
    search_id integer DEFAULT nextval('search_log_search_id_seq'::regclass) NOT NULL,
    session_id text,
    ip_address text,
    "timestamp" timestamp without time zone DEFAULT now() NOT NULL,
    search_phrase text,
    referer text
);


--
-- Name: agency_search_log; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE agency_search_log (
    hit_count integer NOT NULL
)
INHERITS (search_log);


--
-- Name: alias; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE alias (
    category_id integer NOT NULL,
    alias_name text NOT NULL,
    stems_name text NOT NULL,
    CONSTRAINT alias_alias_name CHECK ((alias_name <> ''::text)),
    CONSTRAINT alias_stems_name CHECK ((stems_name <> ''::text))
);


--
-- Name: category; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE category (
    category_id integer DEFAULT nextval(('category_id_sequence'::text)::regclass) NOT NULL,
    category_name text NOT NULL,
    stems_name text NOT NULL,
    CONSTRAINT category_category_name CHECK ((category_name <> ''::text)),
    CONSTRAINT category_stems_name CHECK ((stems_name <> ''::text))
);


--
-- Name: program; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE program (
    program_id integer DEFAULT nextval(('program_id_sequence'::text)::regclass) NOT NULL,
    agency_id integer NOT NULL,
    last_updated date NOT NULL,
    hidden_reason text,
    program_name text NOT NULL,
    program_alias text,
    address1 text NOT NULL,
    address2 text,
    city text NOT NULL,
    state text NOT NULL,
    zip text NOT NULL,
    main_phone text NOT NULL,
    toll_free_phone text,
    hotline text,
    fax text,
    website text,
    email text,
    short_program_description text NOT NULL,
    long_program_description text NOT NULL,
    eligibility text NOT NULL,
    fees_charged text NOT NULL,
    fees_explained text,
    program_schedule text NOT NULL,
    hours text NOT NULL,
    languages text NOT NULL,
    appointment text NOT NULL,
    bring text,
    bus_lines text,
    parking text,
    transportation text,
    handicapped_accessible text NOT NULL,
    child_care_provided text NOT NULL,
    child_care_description text,
    wait_list text NOT NULL,
    wait_list_description text,
    other_information text,
    mail_address1 text,
    mail_address2 text,
    mail_city text,
    mail_state text,
    mail_zip text,
    volunteer_opportunities text NOT NULL,
    volunteer_information text,
    resource_library text NOT NULL,
    resource_library_description text,
    meeting_space text NOT NULL,
    meeting_space_description text,
    news_letter text NOT NULL,
    news_letter_description text,
    public_net_access text NOT NULL,
    public_net_access_description text,
    contact_first_name text NOT NULL,
    contact_last_name text NOT NULL,
    contact_phone text NOT NULL,
    contact_fax text,
    contact_email text NOT NULL,
    contact_email_available text NOT NULL,
    area_type text NOT NULL,
    area_data text,
    tty_tdd text,
    areas_served_description text,
    languages_description text,
    online_registration text NOT NULL,
    reminders_sent integer DEFAULT 0 NOT NULL,
    admin_notes text,
    CONSTRAINT area_check CHECK ((((area_data IS NOT NULL) AND (area_data <> ''::text)) OR (area_type = 'EVERYWHERE'::text))),
    CONSTRAINT program_address1 CHECK ((address1 <> ''::text)),
    CONSTRAINT program_appointment CHECK ((appointment <> ''::text)),
    CONSTRAINT program_area_type CHECK ((area_type <> ''::text)),
    CONSTRAINT program_child_care_provided CHECK ((child_care_provided <> ''::text)),
    CONSTRAINT program_city CHECK ((city <> ''::text)),
    CONSTRAINT program_contact_email CHECK ((contact_email <> ''::text)),
    CONSTRAINT program_contact_email_available CHECK ((contact_email_available <> ''::text)),
    CONSTRAINT program_contact_first_name CHECK ((public_net_access_description <> ''::text)),
    CONSTRAINT program_contact_last_name CHECK ((contact_last_name <> ''::text)),
    CONSTRAINT program_contact_phone CHECK ((contact_phone <> ''::text)),
    CONSTRAINT program_eligibility CHECK ((eligibility <> ''::text)),
    CONSTRAINT program_fees_charged CHECK ((fees_charged <> ''::text)),
    CONSTRAINT program_handicapped_accessible CHECK ((handicapped_accessible <> ''::text)),
    CONSTRAINT program_hours CHECK ((hours <> ''::text)),
    CONSTRAINT program_languages CHECK ((languages <> ''::text)),
    CONSTRAINT program_long_program_description CHECK ((long_program_description <> ''::text)),
    CONSTRAINT program_main_phone CHECK ((main_phone <> ''::text)),
    CONSTRAINT program_meeting_space CHECK ((meeting_space <> ''::text)),
    CONSTRAINT program_news_letter CHECK ((news_letter <> ''::text)),
    CONSTRAINT program_online_registration CHECK ((online_registration <> ''::text)),
    CONSTRAINT program_program_name CHECK ((program_name <> 'program_name'::text)),
    CONSTRAINT program_program_schedule CHECK ((program_schedule <> ''::text)),
    CONSTRAINT program_public_net_access CHECK ((public_net_access <> ''::text)),
    CONSTRAINT program_resource_library CHECK ((resource_library <> ''::text)),
    CONSTRAINT program_short_program_description CHECK ((short_program_description <> ''::text)),
    CONSTRAINT program_state CHECK ((state <> ''::text)),
    CONSTRAINT program_volunteer_opportunities CHECK ((volunteer_opportunities <> ''::text)),
    CONSTRAINT program_wait_list CHECK ((wait_list <> ''::text)),
    CONSTRAINT program_zip CHECK ((zip <> ''::text))
);


--
-- Name: program_category; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE program_category (
    category_id integer NOT NULL,
    program_id integer NOT NULL
);


--
-- Name: cats_with_visible_programs_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW cats_with_visible_programs_view AS
    SELECT pc.category_id, c.category_name, count(pc.program_id) AS program_count FROM program_category pc, program p, category c WHERE (((pc.program_id = p.program_id) AND (pc.category_id = c.category_id)) AND (p.hidden_reason IS NULL)) GROUP BY pc.category_id, c.category_name;


--
-- Name: all_names_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW all_names_view AS
    SELECT all_names.category_id, all_names.name, all_names.stems_name, all_names.category_name FROM (SELECT category.category_id, lower(category.category_name) AS name, category.stems_name, category.category_name FROM category UNION SELECT a.category_id, lower(a.alias_name) AS name, a.stems_name, c.category_name FROM alias a, category c WHERE (a.category_id = c.category_id)) all_names WHERE (all_names.category_id IN (SELECT cats_with_visible_programs_view.category_id FROM cats_with_visible_programs_view));


--
-- Name: category_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE category_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: category_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('category_id_sequence', 805038, true);


--
-- Name: category_search_log; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE category_search_log (
    phrase_stems text NOT NULL,
    top_score numeric(4,2) NOT NULL,
    CONSTRAINT category_search_log_phrase_stems CHECK ((phrase_stems <> ''::text))
)
INHERITS (search_log);


--
-- Name: contacts; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW contacts AS
    SELECT agency.contact_first_name, agency.contact_last_name, agency.address1, agency.address2, agency.city, agency.state, agency.zip, agency.mail_address1, agency.mail_address2, agency.mail_city, agency.mail_state, agency.mail_zip, agency.contact_email, agency.contact_phone, agency.contact_fax, agency.agency_id FROM agency UNION SELECT program.contact_first_name, program.contact_last_name, program.address1, program.address2, program.city, program.state, program.zip, program.mail_address1, program.mail_address2, program.mail_city, program.mail_state, program.mail_zip, program.contact_email, program.contact_phone, program.contact_fax, program.agency_id FROM program;


--
-- Name: contact_email_list_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW contact_email_list_view AS
    SELECT DISTINCT ((((contacts.agency_id)::text || ' <'::text) || lower(contacts.contact_email)) || '>'::text) AS verbose_email FROM contacts WHERE ((contacts.contact_email !~* '(unknown|noemail)'::text) AND (contacts.contact_last_name !~* 'contact'::text)) ORDER BY ((((contacts.agency_id)::text || ' <'::text) || lower(contacts.contact_email)) || '>'::text);


--
-- Name: data_entry_agency_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE data_entry_agency_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: data_entry_agency_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('data_entry_agency_id_seq', 122, true);


--
-- Name: data_entry_agency; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE data_entry_agency (
    id integer DEFAULT nextval('data_entry_agency_id_seq'::regclass) NOT NULL,
    agency_id integer DEFAULT nextval(('agency_id_sequence'::text)::regclass),
    agency_name text NOT NULL,
    update_date timestamp without time zone DEFAULT now(),
    updated_by integer NOT NULL,
    update_status integer DEFAULT 99999 NOT NULL,
    agency_alias text,
    address1 text,
    address2 text,
    city text,
    state text,
    zip text,
    main_phone text,
    toll_free_phone text,
    hotline text,
    fax text,
    website text,
    email text,
    mission text,
    hours text,
    other_info text,
    mail_address1 text,
    mail_address2 text,
    mail_city text,
    mail_state text,
    mail_zip text,
    chief text,
    title text,
    parent_organization text,
    resource_library text,
    resource_library_description text,
    meeting_space text,
    meeting_space_description text,
    staff_internet_access text,
    contact_first_name text,
    contact_last_name text,
    contact_phone text,
    contact_fax text,
    contact_email text,
    contact_email_available text,
    tty_tdd text,
    CONSTRAINT data_entry_agency_address1 CHECK ((address1 <> ''::text)),
    CONSTRAINT data_entry_agency_agency_name CHECK ((agency_name <> ''::text)),
    CONSTRAINT data_entry_agency_chief CHECK ((chief <> ''::text)),
    CONSTRAINT data_entry_agency_city CHECK ((city <> ''::text)),
    CONSTRAINT data_entry_agency_contact_email CHECK ((contact_email <> ''::text)),
    CONSTRAINT data_entry_agency_contact_email_available CHECK ((contact_email_available <> ''::text)),
    CONSTRAINT data_entry_agency_contact_first_name CHECK ((contact_first_name <> ''::text)),
    CONSTRAINT data_entry_agency_contact_last_name CHECK ((contact_last_name <> ''::text)),
    CONSTRAINT data_entry_agency_contact_phone CHECK ((contact_phone <> ''::text)),
    CONSTRAINT data_entry_agency_hours CHECK ((hours <> ''::text)),
    CONSTRAINT data_entry_agency_main_phone CHECK ((main_phone <> ''::text)),
    CONSTRAINT data_entry_agency_meeting_space CHECK ((meeting_space <> ''::text)),
    CONSTRAINT data_entry_agency_resource_library CHECK ((resource_library <> ''::text)),
    CONSTRAINT data_entry_agency_staff_internet_access CHECK ((staff_internet_access <> ''::text)),
    CONSTRAINT data_entry_agency_state CHECK ((state <> ''::text)),
    CONSTRAINT data_entry_agency_title CHECK ((title <> ''::text)),
    CONSTRAINT data_entry_agency_zip CHECK ((zip <> ''::text))
);


--
-- Name: data_entry_program_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE data_entry_program_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: data_entry_program_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('data_entry_program_id_seq', 91, true);


--
-- Name: data_entry_program; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE data_entry_program (
    id integer DEFAULT nextval('data_entry_program_id_seq'::regclass) NOT NULL,
    program_id integer DEFAULT nextval(('program_id_sequence'::text)::regclass),
    agency_id integer NOT NULL,
    program_name text NOT NULL,
    updated_by integer NOT NULL,
    update_date timestamp without time zone DEFAULT now(),
    update_status integer DEFAULT 99999 NOT NULL,
    program_alias text,
    address1 text,
    address2 text,
    city text,
    state text,
    zip text,
    main_phone text,
    toll_free_phone text,
    hotline text,
    fax text,
    website text,
    email text,
    short_program_description text,
    long_program_description text,
    eligibility text,
    fees_charged text,
    fees_explained text,
    program_schedule text,
    hours text,
    languages text,
    appointment text,
    bring text,
    bus_lines text,
    parking text,
    transportation text,
    handicapped_accessible text,
    child_care_provided text,
    child_care_description text,
    wait_list text,
    wait_list_description text,
    other_information text,
    mail_address1 text,
    mail_address2 text,
    mail_city text,
    mail_state text,
    mail_zip text,
    volunteer_opportunities text,
    volunteer_information text,
    resource_library text,
    resource_library_description text,
    meeting_space text,
    meeting_space_description text,
    news_letter text,
    news_letter_description text,
    public_net_access text,
    public_net_access_description text,
    contact_first_name text,
    contact_last_name text,
    contact_phone text,
    contact_fax text,
    contact_email text,
    contact_email_available text,
    area_type text,
    area_data text,
    tty_tdd text,
    areas_served_description text,
    languages_description text,
    online_registration text,
    CONSTRAINT data_entry_program_address1 CHECK ((address1 <> ''::text)),
    CONSTRAINT data_entry_program_appointment CHECK ((appointment <> ''::text)),
    CONSTRAINT data_entry_program_area_type CHECK ((area_type <> ''::text)),
    CONSTRAINT data_entry_program_child_care_provided CHECK ((child_care_provided <> ''::text)),
    CONSTRAINT data_entry_program_city CHECK ((city <> ''::text)),
    CONSTRAINT data_entry_program_contact_email CHECK ((contact_email <> ''::text)),
    CONSTRAINT data_entry_program_contact_email_available CHECK ((contact_email_available <> ''::text)),
    CONSTRAINT data_entry_program_contact_first_name CHECK ((contact_first_name <> ''::text)),
    CONSTRAINT data_entry_program_contact_last_name CHECK ((contact_last_name <> ''::text)),
    CONSTRAINT data_entry_program_contact_phone CHECK ((contact_phone <> ''::text)),
    CONSTRAINT data_entry_program_eligibility CHECK ((eligibility <> ''::text)),
    CONSTRAINT data_entry_program_fees_charged CHECK ((fees_charged <> ''::text)),
    CONSTRAINT data_entry_program_handicapped_accessible CHECK ((handicapped_accessible <> ''::text)),
    CONSTRAINT data_entry_program_hours CHECK ((hours <> ''::text)),
    CONSTRAINT data_entry_program_languages CHECK ((languages <> ''::text)),
    CONSTRAINT data_entry_program_long_program_description CHECK ((long_program_description <> ''::text)),
    CONSTRAINT data_entry_program_main_phone CHECK ((main_phone <> ''::text)),
    CONSTRAINT data_entry_program_meeting_space CHECK ((meeting_space <> ''::text)),
    CONSTRAINT data_entry_program_news_letter CHECK ((news_letter <> ''::text)),
    CONSTRAINT data_entry_program_online_registration CHECK ((online_registration <> ''::text)),
    CONSTRAINT data_entry_program_program_name CHECK ((program_name <> 'program_name'::text)),
    CONSTRAINT data_entry_program_program_schedule CHECK ((program_schedule <> ''::text)),
    CONSTRAINT data_entry_program_public_net_access CHECK ((public_net_access <> ''::text)),
    CONSTRAINT data_entry_program_resource_library CHECK ((resource_library <> ''::text)),
    CONSTRAINT data_entry_program_short_program_description CHECK ((short_program_description <> ''::text)),
    CONSTRAINT data_entry_program_state CHECK ((state <> ''::text)),
    CONSTRAINT data_entry_program_volunteer_opportunities CHECK ((volunteer_opportunities <> ''::text)),
    CONSTRAINT data_entry_program_wait_list CHECK ((wait_list <> ''::text)),
    CONSTRAINT data_entry_program_zip CHECK ((zip <> ''::text))
);


--
-- Name: data_entry_status_status_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE data_entry_status_status_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: data_entry_status_status_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('data_entry_status_status_id_seq', 1, false);


--
-- Name: data_entry_status; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE data_entry_status (
    status_id integer DEFAULT nextval('data_entry_status_status_id_seq'::regclass) NOT NULL,
    status_name text NOT NULL
);


--
-- Name: date_agency_created; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE date_agency_created (
    agency_id integer NOT NULL,
    date_created date NOT NULL
);


--
-- Name: date_program_created; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE date_program_created (
    program_id integer NOT NULL,
    date_created date NOT NULL
);


--
-- Name: entity; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW entity AS
    SELECT agency.agency_id AS id, agency.contact_email AS email FROM agency UNION SELECT program.program_id AS id, program.contact_email AS email FROM program;


--
-- Name: heading; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE heading (
    heading_name text NOT NULL
);


--
-- Name: heading_category; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE heading_category (
    category_id integer NOT NULL,
    heading_name text NOT NULL
);


--
-- Name: log_category_search_search_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE log_category_search_search_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: log_category_search_search_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('log_category_search_search_id_seq', 1762, true);


--
-- Name: log_category_search; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE log_category_search (
    search_id integer DEFAULT nextval('log_category_search_search_id_seq'::regclass) NOT NULL,
    session_id text,
    ip_address text,
    "timestamp" timestamp without time zone DEFAULT now() NOT NULL,
    phrase_stems text NOT NULL,
    top_score numeric(4,2) NOT NULL,
    search_phrase text,
    CONSTRAINT log_category_search_phrase_stems CHECK ((phrase_stems <> ''::text))
);


--
-- Name: mailing_list; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW mailing_list AS
    SELECT ((c.contact_first_name || ' '::text) || c.contact_last_name) AS name, a.agency_name, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_address1 ELSE c.address1 END AS address1, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_address2 ELSE c.address2 END AS address2, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_city ELSE c.city END AS city, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_state ELSE c.state END AS state, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_zip ELSE c.zip END AS zip FROM contacts c, agency a WHERE (c.agency_id = a.agency_id) ORDER BY ((c.contact_first_name || ' '::text) || c.contact_last_name);


--
-- Name: opt_in_contact_email_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW opt_in_contact_email_view AS
    SELECT condensed_contacts_favoring_no_answers.contact_email FROM (SELECT DISTINCT ON (all_contacts.contact_email) all_contacts.contact_email, all_contacts.contact_email_available FROM (SELECT agency.contact_email, agency.contact_email_available FROM agency UNION SELECT program.contact_email, program.contact_email_available FROM program) all_contacts WHERE (all_contacts.contact_email !~* '(unknown|noemail)'::text) ORDER BY all_contacts.contact_email, all_contacts.contact_email_available) condensed_contacts_favoring_no_answers WHERE (condensed_contacts_favoring_no_answers.contact_email_available = 'Yes'::text);


--
-- Name: program_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE program_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: program_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('program_id_sequence', 509526, true);


--
-- Name: synonym_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE synonym_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: synonym_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('synonym_id_sequence', 900319, true);


--
-- Name: synonym_stem; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE synonym_stem (
    synonym_id integer NOT NULL,
    word_stem text NOT NULL,
    submitted_word text NOT NULL,
    CONSTRAINT synonym_stem_submitted_word CHECK ((submitted_word <> ''::text)),
    CONSTRAINT synonym_stem_word_stem CHECK ((word_stem <> ''::text))
);


--
-- Name: town; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE town (
    town_state text NOT NULL,
    county text NOT NULL,
    region text,
    display boolean DEFAULT false,
    CONSTRAINT town_county CHECK ((county <> ''::text)),
    CONSTRAINT town_region CHECK ((region <> ''::text)),
    CONSTRAINT town_town_state CHECK ((town_state <> ''::text))
);


--
-- Name: user_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE user_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: user_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('user_id_sequence', 300055, true);


--
-- Name: user_type_type_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE user_type_type_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: user_type_type_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('user_type_type_id_seq', 1, false);


--
-- Name: user_type; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE user_type (
    type_id integer DEFAULT nextval('user_type_type_id_seq'::regclass) NOT NULL,
    type_name text NOT NULL,
    CONSTRAINT user_type_type_name CHECK ((type_name <> ''::text))
);


--
-- Name: users; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE users (
    user_id integer DEFAULT nextval(('user_id_sequence'::text)::regclass) NOT NULL,
    first_name text NOT NULL,
    last_name text NOT NULL,
    username text NOT NULL,
    "password" text NOT NULL,
    user_type integer,
    created_on timestamp without time zone DEFAULT now() NOT NULL,
    CONSTRAINT users_first_name CHECK ((first_name <> ''::text)),
    CONSTRAINT users_last_name CHECK ((last_name <> ''::text)),
    CONSTRAINT users_password CHECK (("password" <> ''::text)),
    CONSTRAINT users_username CHECK ((username <> ''::text))
);


--
-- Name: web_requests_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE web_requests_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: web_requests_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('web_requests_id_seq', 749579, true);


--
-- Name: web_requests; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE web_requests (
    id integer DEFAULT nextval('web_requests_id_seq'::regclass) NOT NULL,
    host text NOT NULL,
    username text,
    date timestamp without time zone NOT NULL,
    rtype text NOT NULL,
    file text NOT NULL,
    proto text NOT NULL,
    code integer NOT NULL,
    bytes integer NOT NULL,
    refer text NOT NULL,
    agent text NOT NULL
);


--
-- Name: zip_distances; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE zip_distances (
    start_zip character varying(5) NOT NULL,
    end_zip character varying(5) NOT NULL,
    distance numeric(6,2) NOT NULL,
    CONSTRAINT zip_distances_end_zip CHECK (((end_zip)::text <> ''::text)),
    CONSTRAINT zip_distances_start_zip CHECK (((start_zip)::text <> ''::text))
);


--
-- Data for Name: agency; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY agency (agency_id, last_updated, hidden, "password", agency_name, agency_alias, address1, address2, city, state, zip, main_phone, toll_free_phone, hotline, fax, website, email, mission, hours, other_info, mail_address1, mail_address2, mail_city, mail_state, mail_zip, chief, title, parent_organization, resource_library, resource_library_description, meeting_space, meeting_space_description, staff_internet_access, contact_first_name, contact_last_name, contact_phone, contact_fax, contact_email, contact_email_available, tty_tdd, reminders_sent, admin_notes) FROM stdin;
100033	2008-07-15	f	@lunch4all	Community Teamwork, Inc.	CTI	167 Dutton Street	\N	Lowell	MA	01852	978-459-0551	800-698-0551	\N	978-453-9128	www.comteam.org	public_agency_email@testing123.net	The mission of Community Teamwork is to assist low-income people to become self-sufficient, to alleviate the effects of poverty in our communities, and to empower low-income people to participate in the decisions that affect their lives.	Monday-Friday 8:30AM-5:00PM	\N	\N	\N	\N	\N	\N	Karen Frederick	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100517	2008-07-15	f	@lunch4all	Elder Services of the Merrimack Valley, Inc.	ESMV	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-922-2275	978-687-1067	www.esmv.org	public_agency_email@testing123.net	The mission of Elder Services of the Merrimack Valley is to insure that choices of programs and services are available and accessible to meet the diverse needs and changing lifestyles of older adults.	Monday-Friday 8:00AM to 5:00PM	(800) 924-4222 TDD Number	\N	\N	\N	\N	\N	Rosanne J. DiStefano	Executive Director	\N	Yes	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101276	2008-07-15	f	@lunch4all	Massachusetts Health Officers Association	MHOA	P.O. Box 417	\N	West Boylston	MA	01583	508-835-3309	\N	\N	508-835-3309	www.mhoa.com	public_agency_email@testing123.net	The mission of MHOA is to assist and support staff of local health departments in meeting their statutory responsibilities to the public through programs of education, technical assistance, representation and resource development. MHOA also provides educational and informational programs to the general public on public health topics.	Per appt.	\N	\N	\N	\N	\N	\N	Mary-Isabel Luddy	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101430	2008-07-16	f	@lunch4all	Lowell Special Education Parent Advisory Council	Lowell Sped PAC	PO Box 1192	\N	Lowell	MA	01853	978-454-1967	\N	\N	\N	\N	public_agency_email@testing123.net	As a Parent Advisory Council(PAC), we promote understanding between parents, teachers and the Special Education Department.  We facilitate communication within the community regarding special education, provide information on available programs, offer information to parents and provide workshops and support.  The PAC also serves as a resource to other citizens of Greater Lowell who are concerned about special education and special needs issues.	Meetings are held on the first Thursday of the month October through June. The PAC meets at the Butler School (1140 Gorham St)from 7:00 P.M. to 9:00 P.M.	\N	\N	\N	\N	\N	\N	Linda Gignac	Member	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100341	2008-10-02	f	@lunch4all	CLASS Inc.	Citizens League for Adult Special Services	One Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	www.classinc.org	public_agency_email@testing123.net	CLASS strives to provide services which promote personal choice, growth and full participation in community life for individuals with disabilities.	Monday-Friday 8AM-4PM	\N	\N	\N	\N	\N	\N	Robert A. Harris	President	\N	No	\N	Yes	Several conference rooms may be available for professional development activities. Not open to general public.	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103146	2008-12-23	f	@lunch4all	Non-Profit Alliance of Greater Lowelll	NPA	c/o UMass Lowell Center for Family, Work, and Community	600 Suffolk Street, First Floor South	Lowell	MA	01854	978-934-4682	\N	\N	978-934-3026	http://www.npagl.org/	public_agency_email@testing123.net	The Mission of The Non Profit Alliance is to build collaboration that strengthens the influence and impact of community-based organizations through sharing resources and advancing excellence in administration.	8:30am-4:30pm	\N	\N	\N	\N	\N	\N	David Turcotte	Chair	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103047	2008-09-22	f	@lunch4all	Groundwork Lawrence	GWL	60 Island St 2nd Floor	\N	Lawrence	MA	01840	978-974-0770	\N	\N	978-974-0882	www.groundworklawrence.org	public_agency_email@testing123.net	Groundwork Lawrence, Inc. (GWL) is a locally-based 501(c)3 non-profit organization working to create sustainable environmental change through community-based partnerships. Groundwork is committed to "changing places and changing lives" through on-the-ground projects, education, and volunteer programs that help to transform local communities.	Weekdays 9:00 - 5:00	\N	\N	\N	\N	\N	\N	Kate O'Brien	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100242	2008-12-01	f	@lunch4all	Greater Lowell Technical High School	\N	250 Pawtucket Blvd.	\N	Tyngsborough	MA	01879	978-454-5411	\N	978-441-0990	978-441-5344	www.gltech.org	public_agency_email@testing123.net	The mission of the GLTHS is to provide its students with a dynamic academic and technical education preparing them to be lifelong learners who will be competent, respectful and responsible citizens capable of adapting to an ever-changing and increasingly technological society.	Contact for more information	\N	\N	\N	\N	\N	\N	James M. Cassin	Superintendent - Director	\N	Yes	Contact GLTHS for information about resource libraries.	Yes	Theater in the round style lecture hall. Function room adjacent to school's Artisan Restaurant. Contact school for information	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100759	2008-12-15	f	@lunch4all	Merrimack Valley Catholic Charities	\N	70 Lawrence Street	\N	Lowell	MA	01852	978-452-1421	\N	978-452-1421	978-454-9968	www.ccab.org	public_agency_email@testing123.net	The mission of Catholic Charities is to help those most in need, to advocate for social justice, to convene the church and others of good will to do the same and to provide hope for all. Catholic Charities has multiple sites in Lowell, including Lawrence Street, Pawtucket Blvd. and Merrimack Street.	Contact Agency	\N	\N	\N	\N	\N	\N	Dorothy Duval	Program Manager	Catholic Charitable Bureau Archdiocese of Boston	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100506	2009-01-15	f	@lunch4all	Family Continuity Programs, Inc.	FCP, Inc	11 Lawrence Street	\N	Lawrence	MA	01840	978-687-1617	\N	978-687-1617	978-687-1597	www.familycontinuity.org	public_agency_email@testing123.net	The Family Continuity Program provides mental health, behavioral and social services by building relationships that accommodate clients' needs and preferences in order to promote growth and change in people's lives and their communities. We establish treatment partnerships characterized by: A full range of individualized services and interventions, a culturally and professionally diverse staff, and the ability to access all available resources and funding.	Monday-Friday 9AM-7PM	\N	\N	\N	\N	\N	\N	Robin Risso	Regional Director	\N	No	\N	Yes	A conference room that seats about 20 is available for meetings. Contact agency for more information.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100451	2008-06-19	f	@lunch4all	Child Care Circuit	\N	190 Hampshire Street	\N	Lawrence	MA	01840	978-686-4288	877-823-2273	\N	978-975-3120	www.childcarecircuit.org	public_agency_email@testing123.net	Child Care Circuit provides child care resource and referral information, parent and provider trainings, state voucher management, wait list information and management for other state funded subsidies, corporate work life benefits, child care center development and a resource room for parents and providers. NAEYC consulting is also provided.	Monday 9AM-7PM; Tuesday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Sue Halloran	Director	\N	Yes	Contact Child Care Circuit for more information on the resource library	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100561	2008-06-20	f	@lunch4all	Anne Sullivan Center	\N	126 Phoenix Ave.	\N	Lowell	MA	01852	978-453-8331	\N	\N	978-453-9254	www.thomchild.org	public_agency_email@testing123.net	The Anne Sullivan Center Early Intervention Program is committed to family support for the purpose of strengthening and enhancing family life and the growth and development of children. The program views each family as the primary agent in facilitating their child's development and has organized program services via an approach including parents as members of the team and primary advocates for their child / family.	Monday-Friday 8AM-4:30PM	\N	\N	\N	\N	\N	\N	Margaret Mahoney / Alden Wood	Clinical Director /Program Director	Thom Child and Family Services	Yes	Call 978-453-8331 for more information about the Resource Library	Yes	Space is available during the evenings. Call 978-453-8331 for more information	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100682	2008-07-14	f	@lunch4all	South Bay Mental Health Center	\N	77 East Merrimack Street Unit 1	\N	Lowell	MA	01852	978-453-6800	800-244-4691	\N	978-453-6767	www.southbaymentalhealth.com	public_agency_email@testing123.net	The mission of the South Bay Mental Health Center is to provide behavioral health care that is grounded within the community. Individual, family and group counseling are available in the office or outreach. South Bay has offices in Lowell, Lawrence, Brockton, Weymouth, Fall River, Plymouth and Cape Cod, Attleboro, Worcester.	Monday-Friday 9AM-5PM, Appointments are scheduled with clinicians and availability varies.	\N	\N	\N	\N	\N	\N	Peter Scanlon Ph.D	Executive Director	\N	Yes	Contact South Bay for more information about the resource library	Yes	Space is available for professional use/ counseling sessions	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
103256	2008-12-23	f	@lunch4all	Mental Health Resources Plus	MHRP	One Canal St.	\N	Lawrence	MA	01840	978-685-6609	\N	\N	978-685-1941	\N	public_agency_email@testing123.net	ADD	Monday-Friday, 9AM-5PM	\N	\N	\N	\N	\N	\N	Kathleen O'Keefe	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103443	2008-08-26	f	@lunch4all	The Northeast Independent Living Program, Inc.	NILP	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	800-845-6457	\N	978-689-4488	www.nilp.org	public_agency_email@testing123.net	The Northeast Independent Living Program, Inc. is a consumer controlled, cross disability, Independent Living Center providing advocacy and services to people with all disabilities who wish to live as independently as possible within the Northeast area of Massachusetts.	Monday to Friday\r\n9:00AM to 5:00PM	All staff have access to the Internet.	\N	\N	\N	\N	\N	June Cowen	Executive Director	N/A	Yes	Pamphlets and materials on many aspects of disability and relevant programs and services.	Yes	Limited conference room area 25+ people. Priority to agency programs and participants.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	800-845-6457	0	
102959	2008-10-01	f	@lunch4all	Greater Lawrence Rowing	\N	1 Eaton Street	\N	Lawrence	MA	01842	603-553-6349	\N	\N	\N	http://www.greaterlawrencerowing.com	public_agency_email@testing123.net	Greater Lawrence Rowing seeks to establish a challenging yet supportive environment that encourages rowers of all levels to succeed. \r\n\r\nThe vision of Greater Lawrence Rowing is to build an active, community-based rowing program that offers competitive and recreational programs that are accessible to all residents in the Merrimack Valley.  Program fees are modest and scholarships are available.  Outdoor programs are offered in the Spring, Summer and Fall in addition to an indoor training program offered in the Winter.	Practice runs 5 days a week, however schedule changes may occur at the staffs discretion.	\N	\N	\N	\N	\N	\N	Paul Bea	Rowing Director/Head Coach	Greater Lawrence Community Boating Program	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103971	2009-06-05	f	@lunch4all	D.R.A.W.	Doing Real Art WorK	319 wILDER sT	\N	Lowell	MA	01852	978-452-4522	\N	\N	\N	\N	public_agency_email@testing123.net	D.R.A.W. or doing real art work provides a weekly free arts course to residents at The G.R.I.P. Project as well as teens 16-21 in the community	4-8pm Thursday	\N	\N	\N	\N	\N	\N	Corey Felt	Coordinator	The Grip Project	Not Applicable	\N	Yes	319 wilder St	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102684	2008-10-07	f	@lunch4all	Tri-Town Council on Youth and Family Services	Tri-Town Council	5 Main Street	\N	Topsfield	MA	01983	978-887-6512	\N	978-771-4619	978-887-6512	http://www.tritowncouncil.org	public_agency_email@testing123.net	The Tri-Town Council strives to reduce at-risk behavior and improve the emotional health and safety of Tri-Town youth by delivering specialized prevention and intervention services that meet the needs of our community.	M-F, 9 a.m.-5 p.m.	\N	P.O. Box 293	\N	Topsfield	MA	01983	Nancy Coughlin	Manager	Tri-Town Counil	No	\N	Yes	12 people at conference table	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101804	2008-11-17	f	@lunch4all	Lowell Housing Authority (Transitional Housing Program)	LHATHP	21 Salem Street	PO Box 60	Lowell	MA	01854	978-364-5368	\N	\N	978-656-4665	www.lhma.org	public_agency_email@testing123.net	To provide housing, support, guidance, and resources to homeless families in order to facilitate their transition to self-sufficiency.	Monday-Friday 8:30-4:30	\N	21 Salem Street	P.O. Box  60	Lowell	MA	01854	Michelle L. Richter	Family Self-Sufficiency/Supportive Services Director	Lowell Housing Authority	Not Applicable	\N	Yes	Mercier Center\r\n21 Salem Street\r\nLowell MA, 01854	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102640	2008-12-15	f	@lunch4all	Generazio & Nguyen	\N	77 Franklin Street	Suite 300	Boston	MA	02110	617-423-0700	\N	\N	\N	www.gnimmigration.com	public_agency_email@testing123.net	Generazio & Nguyen is a Massachusetts-based law firm with offices in Andover and Boston devoted exclusively to the practice of immigration and nationality law. Our firm's unique experience enables us to navigate our clients effectively and efficiently through the complex immigration process. \r\n\r\nProviding quality cost effective immigration solutions represents the cornerstone of our practice.	Monday thru Saturday 8 to 6 pm	\N	\N	\N	\N	\N	\N	Marc R. Generazio	Attorney	\N	Yes	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100352	2008-12-23	f	@lunch4all	Open Pantry of Greater Lowell	\N	200 Central Street	\N	Lowell	MA	01852	978-453-6693	\N	\N	\N	\N	public_agency_email@testing123.net	The mission of the Open Pantry of Greater Lowell is to provide 5 days of nutritious food for the hungry persons of Greater Lowell.	Monday-Thursday 9AM-12PM\r\nWednesday 5:30PM-7:30PM	\N	\N	\N	\N	\N	\N	Liz Butz	Executive Director	\N	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100913	2008-12-30	f	@lunch4all	Department of Early Education and Care	EEC	600 Washington Street, Suite 6100	350 Main St. Malden, MA 02148	Boston	MA	02111	617-988-6600	\N	\N	617-988-2453	www.mass.gov/eec	public_agency_email@testing123.net	The Department of Early Education and Cares  web site is the place for parents and child care providers to get answers to their important questions about child care and early education.The agency aim is to make sure all children receive the highest quality child care from providers in Massachusetts.	Contact agency for more information	\N	\N	\N	\N	\N	\N	Ann Reale	Commissioner	Commonwealth of Massachusetts	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100957	2008-07-01	f	@lunch4all	Challenge Unlimited at Ironstone Farm	\N	450 Lowell Street (Rt. 133)	\N	Andover	MA	01810	978-475-4056	877-632-8386	\N	978-475-4046	www.challengeunlimited.org	public_agency_email@testing123.net	The mission of Challenge Unlimited is to provide effective therapy for people with physical, cognitive and emotional disabilities and speech / language disorders using horses and the environment of a working farm.	Office hours: Monday-Saturday 8AM-5PM	\N	\N	\N	\N	\N	\N	Deedee O'Brien	Executive Director	\N	No	\N	Yes	A tent is available in the spring, summer and fall only. Call for more information.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
103223	2008-06-05	f	@lunch4all	American Training, Inc.	\N	102 Glenn Street	\N	Lawrence	MA	01843	978-685-2151	\N	\N	978-683-5124	americantraininginc.com	public_agency_email@testing123.net	Throughout our history we have held steadfast to our belief in the unlimited potential of the human spirit. And the dramatic success that can be achieved by people from all walks of life when they experience the "Power" of success!	Open Monday through Friday, from 8:00 am - 4:30 pm.  Closed holidays	\N	\N	\N	\N	\N	\N	Tom Connors	President/CEO	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	1-800-439-2370	2	
100803	2009-01-15	f	@lunch4all	Cambodian Mutual Assistance Association	CMAA	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_agency_email@testing123.net	The mission of CMAA is to improve the quality of life for the Cambodian-American community of Greater Lowell and other ethnic minorities through educational, cultural, social, community and economic development efforts.	Contact Agency	\N	\N	\N	\N	\N	\N	Vong Ros	Executive Director	\N	No	\N	Yes	Some space, 30 seats, is available on a limited basis, preferably business related to CMAA	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101188	2009-01-16	f	@lunch4all	ADA Technology Corporation	Civil Rights for the disabled	35 Seventh Street	\N	Lowell	MA	01850	978-937-5170	\N	\N	978-937-5170	http://ADAtech.org	public_agency_email@testing123.net	ADA Technology is a non-profit devoted to eliminating barriers to accessing computers and the Internet for people with disabilities, through adapting technology to their needs. It is intended to supplement and enhance current school curriculum. \r\n\r\nADA Technology focuses on serving students with severe motor, visual and other impairments. These students are the most vulnerable and under-served. They often can not physically access publicly available computers. Even if these students manage to access them, they are unable to work due to the nonexistence of interfaces and applications adjustable to their disabilities.  Access to technology is the only way for severe disabled wheelchair students to have the same education and employment opportunities as other people\r\nADA Technology pursues information technology accessibility for severely disabled people through three primary means: technology, community outreach, and research and development. One of it's major goals is to\r\nenhance and supplement an ADA Technology Corp. project called L.E.L.P. i.e., Lowell Elementary Literacy Program while pro-actively working with the Citywide \r\nParent Council; http://www.lowellcpc.org/\r\n\r\nTechnology:\r\nADA Technology is working toward\r\n developing fully accessible interfaces adjustable to particular needs of the disabled with severe motor and visual impairments\r\n designing, implementing and maintaining the ADA Tech Fully Accessible Network\r\n building a library of software packages accessible and adjustable for disabled users for connecting, emailing, learning, training, video conferencing, organizing personal data, playing, etc.\r\n\r\nCommunity outreach:\r\n\r\nADA Technology will\r\n provide disabled students with interfaces to access and use computers\r\n connect disabled users into the ADA Tech Fully Accessible Network to make them able to communicate and work on the Web\r\n provide disabled users with fully accessible online courses, video conferencing, interactive assessments and other applications for communicating and learning\r\n train disabled users how to use computers and the Network\r\n encourage those who work/ help disabled people become members of the Network to communicate with students and share their knowledge there.\r\n\r\n\r\nResearch and Development: \r\n\r\nADA Technology will use new W3C standards and principles of Universal design to be able to develop interfaces with accessibility and usability levels sufficient for users with serious health problems. \r\n\r\nADA Technology will build and maintain its own server(s) where databases, functionality applications and software packages will be kept and updated for further distribution through the Network on user's demands. \r\n\r\nThe effectiveness of ADA Technology's interfaces and distributed applications as well as stability and functionality of the ADA Tech fully Accessible Network will be tested in real-life situations by Network users and will be improved on the basis of feedbacks and evaluations. ADA Technology user's models will be built and updated following improvements in interfaces, applications and equipment, which serve real disabled people. \r\n \r\nADA Technology brings together the community and new technologies to design and implement effective lasting solutions that will work at the local level and serve as a model nationwide. We encourage replication of activities.	24/7	\N	\N	\N	\N	\N	\N	Cary L. Renault, M.Sc.	Chief Executive Officer	\N	Not Applicable	\N	Yes	City Hall of Lowell, Major Reception room. 2nd floor. (elevator accessible)	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-937-5170	0	
102552	2008-06-20	f	@lunch4all	Boy Scouts of America, Yankee Clipper Council	\N	36 AMESBURY RD	\N	Haverhill	MA	01830	978-372-0591	\N	\N	978-373-9134	http://www.yccbsa.org	public_agency_email@testing123.net	The mission of the Boy Scouts of America is to prepare young people to make ethical and moral choices over their lifetimes by instilling in them the values of the Scout Oath and Law.	Mon-Fri 8am-5pm	\N	\N	\N	\N	\N	\N	Chuck Marvin	Finance Director	Boy Scouts of America	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100847	2008-07-15	f	@lunch4all	Chelmsford Board of Health	\N	50 Billerica Road	\N	Chelmsford	MA	01824	978-250-5241	\N	\N	978-250-5244	www.townofchelmsford.us	public_agency_email@testing123.net	The Chelmsford Board of Health serves the community of Chelmsford by protecting the public through the efficient implementation of the various regulations mandated by the Commonwealth of Massachusetts Division of Public Health and emphasizing prevention and education for the health and well being of the residents.	Monday-Friday 8:30AM-4:30PM	\N	\N	\N	\N	\N	\N	Richard Day	Director	Town of Chelmsford	Yes	Contact Board to access health information resources	Yes	Space available for meetings pertinent to BOH concerns, ie health, medical, West Nile Fever, bio-terrorism, etc.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100968	2008-08-02	f	@lunch4all	St. Theresa's Bereavement Program	\N	1 Grace Ave	\N	Billerica	MA	01821	978-663-8816	\N	\N	978-663-0577	http://sttheresaparish.com/	public_agency_email@testing123.net	St. Theresa's  bereavement support groups meets on the second and fourth Thursday of each month from September through June.We do not meet during the summer.	Meetings held in the Parish Hall,One Grace Way,located behind the church.\r\n7to 9 pm\r\n\r\nCall facilitator for information.	\N	\N	\N	\N	\N	\N	Joan	Kilday	\N	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103454	2008-09-03	f	@lunch4all	Home Preservation Center	\N	450 Merrimack St.	\N	Lowell	MA	01852	978-970-0600	\N	\N	978-970-0609	\N	public_agency_email@testing123.net	To provide foreclosure prevention servies in efforts to combat predatory lending and preserve homwownership for families facing foreclosure.	Monday-Friday, 9-5.  Some evening appointments available.	\N	\N	\N	\N	\N	\N	Rasy An	Foreclosure Prevention Coordinator	Coallition for a Better Acre, Community Teamwork, Inc.,	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102035	2008-10-21	f	@lunch4all	T.H.E. FARM	Tewksbury Hospital Equestrian FARM	Livingston Street	365 East Street, PO Box 754	Tewksbury	MA	01876	978-851-5540	\N	\N	\N	www.t-h-e-farm.org	public_agency_email@testing123.net	Equine therapeutic riding and non-riding programs for adults and children with disabilities. Riding lessons also available for the able-bodied.	Closed Mondays.	\N	\N	\N	\N	\N	\N	MJ Marcucci	President	\N	Not Applicable	\N	Yes	Small meeting room; seats 15-20	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103157	2008-12-23	f	@lunch4all	New England Prison Ministries	\N	PO Box 1785	\N	Lowell	MA	01853	978-458-1721	\N	\N	\N	www.nepm.org	public_agency_email@testing123.net	Support and encouragement for prisoners, discharged prisoners and their families.	Weekdays	\N	\N	\N	\N	\N	\N	Mark Hemenway	President	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100110	2008-12-16	f	@lunch4all	Girls Incorporated of Greater Lowell	\N	220 Worthen Street	\N	Lowell	MA	01852	978-458-6529	\N	978-458-6529	978-458-4837	www.girlsinclowell.org	public_agency_email@testing123.net	Girls Incorporated of Greater Lowell inspires all girls to be strong, smart and bold.  Through creative programs in a safe and nuturing environment, we strive to teach the independence and personal responsibility girls need to succeed.	Contact program for more information	\N	\N	\N	\N	\N	\N	Carol Duncan	Executive Director	Girls Incorporated	No	\N	Yes	Girls Inc. will rent out meeting space to qualified non-profit agencies and groups.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101067	2008-07-11	f	@lunch4all	Acre Family Day Care	\N	14 Kirk St	\N	Lowell	MA	01852	978-937-5899	\N	978-937-5899	978-937-5148	www.acrefamily.org	public_agency_email@testing123.net	Acre Family Day Care offers community-based family child care for children, and provides training and support for home-based family child care providers	Monday-Friday 6AM-6PM, Saturday-Sunday as needed	\N	\N	\N	\N	\N	\N	Kathy Reticker	Executive Director	\N	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
102563	2007-07-26	f	@lunch4all	Boys & Girls Clubs of Lower Merrimack Valley	\N	18 Maple St.	\N	Salisbury	MA	01952	978-462-7003	\N	\N	978-462-7003	http://www.boysandgirlscluboflmv.com/	public_agency_email@testing123.net	To inspire and enable all young people, especially those that need us the most, to realize their full potential as productive, responsible and caring citizens.	2:30pm-7:00pm, Monday-Friday. Saturday programs available 9:00am -2:00pm	\N	\N	\N	\N	\N	\N	Jim Keenan	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	4	
102860	2008-01-05	f	@lunch4all	Child Development Network, LLC	CDNKids	76 Bedford Street	Suite 12	Lexington	MA	02420	781-861-6655	\N	\N	781-861-6654	www.CDNKids.com	public_agency_email@testing123.net	The Child Development Network (CDN) is an inter-disciplinary team of pediatric health care specialists who provide evaluations and ongoing care for children, teens, and their parents.\r\n\r\nWe offer a variety of clinical services, including Pediatric Neuropsychology, Behavioral Medicine, Cognitive-Behavioral Therapy, Speech and Language Pathology, and Education Services.\r\n\r\nOur goal is to optimize each child's development and "to help kids achieve their personal best."	9:00 AM-7:00 PM	\N	\N	\N	\N	\N	\N	Dr. Scott Sokol	Clinical Neuropsychologist	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	4	
101848	2008-07-15	f	@lunch4all	Kreative Kids Child Care	Kreative Kids Family Child Care	10 Wamesit Road	\N	Tewksbury	MA	01876	978-851-0719	\N	\N	\N	http://www.geocities.com/kreative_kids_childcare/Childcare.h	public_agency_email@testing123.net	http://www.geocities.com/kreative_kids_childcare/Childcare.html\r\n\r\nLicensed Family Child Care since 1991. Infant through age 4.  President of the Family Child Care Association.	We are open Monday - Friday.  Hours are flexible.  Full or part time.	\N	\N	\N	\N	\N	\N	Linda Rowe	Owner	FCCA	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100627	2008-08-14	f	@lunch4all	Northeast Center for Healthy Communities	Mass Prevention Center	1 Canal Street	Entrance C	Lawrence	MA	01840	978-688-2323	\N	\N	978-975-7779	www.nc4hc.org	public_agency_email@testing123.net	The Northeast Center for Healthy Communities provides capacity building and resources to prevention programing in the Northeast of Massachusetts.\r\n\r\nIncluding a Health Library.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Jim Ryan	Director	\N	Yes	Lending library. Contact program for more information. Access holding at www.nc4hc.org	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101661	2008-09-15	f	@lunch4all	Greater Lowell Elder Mental Health Collaborative	\N	18 East Meadow Lane	\N	Lowell	MA	01854	978-458-1512	\N	\N	\N	www.eldermentalhealth.org	public_agency_email@testing123.net	To advocate for and to facilitate the development of mental health services for elders	7 days/ 24 hours	\N	\N	\N	\N	\N	\N	frank e. baskin,LICSW	coordinator	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103003	2008-10-01	f	@lunch4all	Lighthouse Art School for children	\N	251 Foster Street	\N	Lowell	MA	01851	978-937-1602	\N	\N	\N	lighthouseartschool.com	public_agency_email@testing123.net	The Lighthouse Art School in Lowell is a place for all ages to learn how to express themselves with multiple mediums of visual art, gain a greater appreciation for the beauty of art and have a lot of fun doing it.	Morning / Afternoon\r\nDay Summercamps	\N	\N	\N	\N	\N	\N	Engelie Blomerus	Mrs.	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101749	2008-06-20	f	@lunch4all	Salem State College	Center for International Education	8 Harrison Road	\N	Salem	MA	01970	978-542-6351	\N	\N	978-542-7104	http://www.salemstate.edu/cie	public_agency_email@testing123.net	Our English as a Second Language program prepares students and professionals. Class sizes are small and students come from all over the world. Courses include: conversation, reading and writing, academic skills, and TOEFL preparation. The Center for International Education offers academic and immigration advice (including I-20's).	Monday-Friday 7:45am-5:00pm	\N	Salem State College  Center for International Education	352 Lafayette Street	Salem	MA	01970	Sarah Dietrich, EdD	Assistant Director/ESL Coordinator	Salem State College	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
101342	2008-06-19	f	@lunch4all	Community Software Lab	The CSL	PO Box 1418	\N	Lowell	MA	01853	978-934-4350	\N	\N	\N	thecsl.org	public_agency_email@testing123.net	The Community Software Lab writes free software. Our principle project is http://mvhub.com	Monday-Friday 10-6\r\nSaturday (variable)\r\nemail to help@thecsl.org is generally answered within a couple hours.	\N	PO Box 1418	\N	Lowell	MA	01853	Dan Macneil	Fearless Leader	n/a	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
101221	2008-07-10	f	@lunch4all	Lowell Parks & Conservation Trust, Inc.	LP&CT	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	978-454-7637	www.lowelllandtrust.org	public_agency_email@testing123.net	The LP&CT mission is to improve the quality of life for the people of Lowell through the creation, conservation, and preservation of parks, open spaces, and special places.	Monday-Friday \r\n9:00 AM - 5:00 PM	\N	\N	\N	\N	\N	\N	Jane L. Calvin	Executive Director	\N	Not Applicable	\N	Yes	Spalding House, 383 Pawtucket Street, Lowell\r\n50 person meeting/presentation room - Pentucket Hall	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
102728	2008-07-15	f	@lunch4all	M/A-COM Federal Credit Union	MFCU	83 Merrimack Street	\N	Lowell	MA	01852	978-328-5100	866-622-6628	\N	978-458-8933	www.macomfcu.org	public_agency_email@testing123.net	Primary Mission: To serve as the primary financial resource for our members.\r\n\r\nWe strive to be your primary financial resource by offering you the most convenient and cost-effective method to manage your finances.  M/A-COM FCU is owned and operated by its members, where all members benefit from the continued drive for growth and improved earnings.  We take pride in offering all of our members the highest quality financial products and services at the most competitive rates, while nurturing the friendly and cooperative spirit of the credit union.  We are committed to the financial well-being of our members and will continue to work for the secure future of our community.	83 Merrimack Street\r\nLowell, MA 01852\r\nMonday: 9:00 a.m.  4:00 p.m. \r\nTuesday: 9:00 a.m.  4:00 p.m.\r\nWednesday: 9:00 a.m.  4:00 p.m.\r\nThursday: 9:00 a.m.  7:00 p.m.\r\nFriday: 9:00 a.m.  5:00 p.m.\r\nSaturday: 9:00 a.m.  12:00 p.m	\N	\N	\N	\N	\N	\N	Danielle M. Bergeron	Marketing Director	\N	Not Applicable	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102695	2008-07-15	f	@lunch4all	Lowell Public Schools	Parent Information Center	17 Kirk St.	\N	Lowell	MA	01852	978-937-8986	\N	\N	978-937-3468	www.lowell.k12.ma.us	public_agency_email@testing123.net	The mission of the Parent Information Center (PIC) is to assist parents, guardians and students in accessing the educational services of the Lowell Public Schools and other community resources as well as to provide the services necessary to effectively and efficiently implement Lowells Voluntary Revised Desegregation and Educational Improvement Plan.  The Parent Information Center is devoted to making a parent's introduction to the Lowell Public Schools a positive experience.	Monday - Friday\r\n8:00 AM - 4:00 PM\r\n\r\nThursday\r\n5:00 PM - 7:00 PM	\N	\N	\N	\N	\N	\N	Dr. Phala Chea	Parent Information Center Coordinator	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101386	2008-07-15	f	@lunch4all	Angel Homecare	\N	348 North Main Street	\N	Andover	MA	01810	978-475-2244	800-349-1142	978-475-2244	978-475-2448	www.angelhomecare.com	public_agency_email@testing123.net	Angel Homecare is a full-service home care agency located in Andover Massachusetts.  Our mission is to provide our clients with affordable private care and support in a home-based environment.  By providing your loved one with quality in-home care, your loved one will be in an environment that is familiar, reducing stress and allowing him/her to maintain the level of independence they are accustomed to.	Office hours are Monday through Friday 9:00 AM to 4:30 PM.  On-call service available 24/7 365 days per year.	Services include: Nursing, 24 hour live-in care, personal care, Medication reminders, caregiver relief, companionship, meal preparation, errands, transportation, light housekeeping,and homemaking services	\N	\N	\N	\N	\N	Terrie Marr	Clinical Manager	Privately owned	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100649	2008-07-18	f	@lunch4all	Arbour Counseling Services	\N	10 Bridge Street	Third F.	Lowell	MA	01852	978-453-5736	\N	978-453-5736	978-970-5595	www.ArbourHealth.com	public_agency_email@testing123.net	The mission of Arbour Counseling Services is to provide high quality out-patient mental health services.	Monday-Friday 9AM-9PM; Saturday 9AM-3PM	\N	\N	\N	\N	\N	\N	Sarah A. Quinn Ph.D.	Director	Universal Health Services	Yes	Resources on mental health with a focus on Southeast Asians available. Appointments needed, contact agency.	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101034	2008-07-18	f	@lunch4all	American Red Cross of Merrimack Valley	\N	177 Ward Hill Avenue	\N	Ward Hill	MA	01835	978-372-6871	978-372-6871	978-372-6871	978-374-2233	www.arcmv.org	public_agency_email@testing123.net	The American Red Cross, a humanitarian organization led by volunteers and guided by its Congressional Charter and the fundamental principles of the International Red Cross Movement, will provide relief to victims of disaster and help people prevent, prepare for and respond to emergencies.	Monday-Friday 9AM-4:30PM	\N	\N	\N	\N	\N	\N	Deb Duxbury	Dir of Emergency and Volunteer Services	American Red Cross	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103421	2008-08-08	f	@lunch4all	National Society of Creative Dyslexics	\N	Blakes Hill Road	\N	Westford	MA	01886	978-621-6619	\N	\N	\N	www.creativedyslexics.org	public_agency_email@testing123.net	The mission of the National Society of Creative Dyslexics is to honor the creative and intellectual aspects of being an individual with dyslexia.  Bright, Talented, Intelligent.  And learning differently.\r\n\r\nOffering reading tutoring services in Orton Gillingham, and supporting families to successful educational outcomes.	Monday through Friday, 9am to 6pm	\N	8 Blakes Hill Road	\N	Westford	MA	01886	Bonnie Haley	Founder, Advocate, & Tutor	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102574	2008-08-18	f	@lunch4all	Merrimack River Watershed Council, Inc.	MRWC	600 Suffolk Street	Fifth Floor	Lowell	MA	01854	978-275-0120	\N	\N	978-275-0125	www.merrimack.org	public_agency_email@testing123.net	Our mission is to protect the Merrimack River and promote the wise use of its watershed.  We fulfill our mission through environmental monitoring, education and outreach, advocacy, recreation, and community organizing	M-Th 10am to 6pm	\N	\N	\N	\N	\N	\N	Christine Tabak	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101496	2008-10-01	f	@lunch4all	Universal Child Incorporated	\N	194 River Road	\N	Lowell	MA	01852	508-572-8828	\N	\N	\N	http://www.universal-child.org	public_agency_email@testing123.net	Our mission is to provide an online educational forum for the prevention of addiction.	Online - 24/7	\N	194 River Rd.	\N	Lowell	MA	01852	F. LaBranche	Director	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100539	2008-12-15	f	@lunch4all	Massachusetts Alliance of Portuguese Speakers	MAPS	11 Mill Street	\N	Lowell	MA	01852	978-970-1250	\N	\N	978-970-0843	www.maps-inc.org	public_agency_email@testing123.net	The mission of the Massachusetts Alliance of Portuguese Speakers is to serve people from all Portuguese-speaking communities and their descendants and to help them become self-reliant and productive members of American society, while maintaining strong ethnic identity and pride.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Osvalda Rodrigues	Director MAPS Lowell Office	MAPS has additional offices in Massachusetts	No	\N	Yes	Yes, there is a conference room. Please call for more details.	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102981	2008-12-30	f	@lunch4all	Greater Lawrence Technical High School	\N	57 River Road	\N	Andover	MA	01810	978-686-0194 ext. 3041	\N	\N	\N	\N	public_agency_email@testing123.net	Greater Lawrence Technical School is committed to providing high quality education to our students.	Monday - Friday 7AM - 3PM	\N	\N	\N	\N	\N	\N	Jenna Simard	At-Risk Counselor	\N	Not Applicable	\N	Not Applicable	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101815	2009-01-15	f	@lunch4all	The Northeast Independent Living Program	NILP	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	\N	\N	978-689-4488	www.nilp.org	public_agency_email@testing123.net	Provides services to all people with disabilities. \r\nInformation & Referral\r\nPeer Counseling\r\nskills Training\r\nAdvocacy	M-F 9:00 am -5:00 pm	\N	\N	\N	\N	\N	\N	June Cowen	Executive Director	\N	Yes	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-687-4288	0	
101870	2008-07-18	f	@lunch4all	Vinfen	Vinfen	950 Cambridge Street	\N	Cambridge	MA	02141	617-441-1800	877-284-6336	\N	617-441-1858	http://www.vinfen.org	public_agency_email@testing123.net	Established in 1977, Vinfen is a private, nonprofit human services organization that provides a comprehensive array of services to children, adolescents and adults with psychiatric, developmental, and behavioral disabilities. Vinfen supports thousands of individuals each year in more than 350 residential, respite, day, and crisis intervention programs and services in eastern Massachusetts and northern Connecticut. Vinfens 2,000 dedicated employees are experienced, highly trained professionals who provide a full range of educational, rehabilitative, and clinical services.	Main office: M-F / 9 a.m. to 5 p.m.	\N	\N	\N	\N	\N	\N	Gary W. Lamson	President & CEO	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	617-225-2000	0	
102783	2008-07-15	f	@lunch4all	Lawrence Families & Youth Initiative	F.Y.I.	96 East Haverhill Street	\N	Lawrence	MA	01841	978-557-8400	\N	800-792-5200	978-557-8410	\N	public_agency_email@testing123.net	The mission of F.Y.I. is to combine and integrate community, agency and family resources; to promote policy changes that will support creative, nurturing services for strengthening families and communities.  Building on principles of family-centered casework and Family Support Principles, F.Y.I. provides a unique opportunity to affect changes in the way the community responds to the needs of individual families and the needs of the community as a whole.  The Families & Youth Initiative partners work together to promote practice development and improvements in systemic functioning that support child safety well-being and permanency.	Monday through Friday\r\n9:00am till 5:00pm	\N	\N	\N	\N	\N	\N	Delilah Rivera	Program Coordinator	GLCAC, INC.	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100726	2008-07-15	f	@lunch4all	Pollard Memorial Library	\N	401 Merrimack Street	\N	Lowell	MA	01852	978-970-4120	\N	\N	978-970-4117	http://www.pollardml.org/	public_agency_email@testing123.net	The Pollard Memorial Library, established in 1844, provides educational, informational, recreational, and cultural materials and services to the urban population of the city of Lowell.	Monday - Thursday 9 AM - 9 PM, Friday & Saturday 9 AM - 5 PM (Summer closed Saturday)	(978) 970-4129 TTY Number	\N	\N	\N	\N	\N	Victoria Woodley	Acting Director	City of Lowell	Yes	\N	Yes	Contact library for more information	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101463	2008-07-15	f	@lunch4all	Project Bread FoodSource Hotline	FoodSource Hotline	145 Border Street	\N	East Boston	MA	02128	1-800-645-8333	1-800-645-8333	\N	\N	www.gettingfoodstamps.org	public_agency_email@testing123.net	Project Bread's mission is to alleviate hunger in Massachusetts.  The FoodSource Hotline provides information and screenings for the food stamp program and information about emergency food programs and low cost food programs across the state.	M-F 8-5	\N	\N	\N	\N	\N	\N	Diane Dickerson	Dir. of Emergency Food Resources	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	1-800-377-1292	0	
101738	2008-07-15	f	@lunch4all	Lowell House Inc. S.O.A. P. Program	Enhansed Structured Outpatient Addiction Program	555 Merrimack Street	\N	Lowell	MA	01854	978-454-2997	\N	\N	978-937-2559	\N	public_agency_email@testing123.net	The S.O.A.P. program is dedicated to treating those with drug/alcohol addiction in a full-time day program. We treat all clients who are currently on Mass Health (MBHP) as well as those with no insurance. This is a 8 week intensive program operating Monday through Friday from 9:00am to 4:00pm. Evening program is 5:30 pm to 9:00 pm\r\nOur goal is to treat all issues in the clients life that are stopping them from a sober and productive lifestyle.	Monday through Friday 9:00 am to 4:00 pm\r\nMonday through Thursday 5:30 pm to 9:00 pm	night program now available: Monday through Thursday 5:30 A.M. to 9:00 A.M.	\N	\N	\N	\N	\N	Harry G. Providakes M.Div. CL1	Program Director	\N	No	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101210	2008-07-15	f	@lunch4all	The Jericho Road Project	\N	20 Lexington Road	\N	Concord	MA	01742	978 369 9602 X457	\N	\N	978-369-8701	www.jerichoroadproject.org	public_agency_email@testing123.net	The Jericho Road Project bridges communities by matching the professional talents of volunteers with the needs of community-based nonprofit organizations and businesses to promote community development, strengthen social services, and enrich the lives of volunteers.\r\n\r\nJericho Road's approach to a community is pro-active (we seek out business and try to anticipate and address emerging problems or opportunities); holistic (we define our service offerings by the many different needs of our clients) and long-term (we create a permanent connection between two communities and the people who live in them.\r\n\r\n Since our launch in early 2003, our volunteers and our corporate partners have over 60 non profit organizations and small businesses through two core programs: Springboard and B.E.A.N.E.\r\n\r\nSpringboard - Jericho Road's flagship program provides customized services to nonprofits to help them achieve their goals. The type of services provided is virtually unlimited, ranging from board building, organizational assessment, and strategic planning to marketing, professional coaching and website development.\r\n\r\nJericho Road's other core program - Business Experts Assisting New Entrepreneurs, or BEANE - helps entrepreneurs launch and grow small businesses in Lowell by providing business training and one-on-one coaching to prospective and new entrepreneurs in need of help. This service is provided largely in partnership with the Lowell Small Business Assistance Center.	M-F/9-5	\N	\N	\N	\N	\N	\N	Dan Holin	Executive Director	\N	No	\N	Yes	Lovely, small.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100594	2008-07-15	f	@lunch4all	House of Hope Inc.	\N	812 Merrimack Street	\N	Lowell	MA	01854	978-458-2870	\N	978-458-2870	978-458-6679	www.hopelowell.org	public_agency_email@testing123.net	The mission of House of Hope is to provide advocacy and care for 18 homeless families while affirming the dignity of life to all who are served and guiding them to self-sufficiency. Families are referred to the shelter via the State Department of Transitional Assistance, hence, rarely are we able to accommodate walk-ins.  House of Hope Housing Inc. has recently created an affiliated organization called House of Hope Housing, Inc. whose objective is to create affordable housing for families in shelter.\r\nHouse of Hope Housing has completed construction on New Hope Apartments.  New Hope is ten (10) units of service supported housing for homeless families.  New Hope is located at 203 Salem Street, behind the shelter.	24 hours/day, 7 days/week	\N	\N	\N	\N	\N	\N	Deborah Chausse	Executive Director	\N	No	\N	Yes	A conference room may be available. Contact agency for details.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102893	2008-07-15	f	@lunch4all	UnionStudio Yoga	\N	305 North Main Street	Tactician Building	Andover	MA	01810	978-866-7944	\N	\N	\N	www.unionsudioyoga.com	public_agency_email@testing123.net	At UnionStudio Yoga we are aspiring to bring together a variety of class offerings, yoga styles, and individual and group workshops to support the differing needs and lifestyles of our students.\r\n\r\nWe are committed to offering free yoga classes to the community so that all may experience the benefits of yoga.	Classes offered 7 days a week.\r\nFree Community Yoga offered Friday 11:15-12:30pm.	\N	\N	\N	\N	\N	\N	Connie Glore RYT	co-owner	\N	Yes	Books available on loan from the studio.	Yes	Located in the Tactician Building in Shawsheen Village, centrally located with plenty of off-street parking, UnionStudio is a beautiful place to learn and practice yoga.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102596	2008-07-15	f	@lunch4all	The Children and Family Law Program	CAFL Lowell	40-44 Church Street	\N	Lowell	MA	01852	978-446-3917	\N	\N	978-446-3916	\N	public_agency_email@testing123.net	The Children and Family Law Program of the Massachusetts Committee for Public Counsel Services provides legal representation to indigent parents and children in state intervention/child welfare matters, including care and protection proceedings, children in needs of services cases (CHINS), actions to dispense with parental consent to adoption, and any other proceeding regarding child custody where the Department of Social Services is a party or where the court is considering granting custody to the Department. See G.L. c.119, §29; G.L. c.210, §3; Balboni v. Balboni, 39 Mass. App. Ct. 210 (1995). Representation is provided by a panel of private court appointed attorneys and by staff attorneys in our Boston, Brockton, Lowell, Salem, Springfield and Worcester offices.	Monday through Friday 9am-5pm	\N	40-44 Church Street	\N	Lowell	MA	01852	Anita Sullivan	Attorney in Charge	CAFL	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102827	2008-07-15	f	@lunch4all	Prime Staffing Services,LLC	Support and Relief Staff	40 Witherbee Ave	\N	Revere	MA	02151	781-336-9123	\N	781-336-9123	781-715-6171	\N	public_agency_email@testing123.net	Prime staffing services,LLC. Takes a great pride in the ability to offer highly qualified staff who stand ready to meet the needs of individuals with developmental disabilities,with mental illness,behavioral health disabilities,emotinaldifficulties,by assisting and supervising with daily living activities.\r\nOur highly trained staff are skilled at finding solutions to help each person to live a full and enjoyable life.	7 days/ 24hours	\N	\N	\N	\N	\N	\N	Driss Abainou	President	Prime Staffing Services,LLC	Not Applicable	\N	Not Applicable	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101485	2008-07-15	f	@lunch4all	Lawrence Tobacco Free Partnership	\N	One Canal St, Entrance C	\N	Lawrence	MA	01840	978-688-2323 x 8618	\N	\N	978-975-7779	\N	public_agency_email@testing123.net	The Lawrence Tobacco Free Partnership strives to reduce youth access to tobacco products; prevent first time use of tobacco products; help smokers to quit; and protect nonsmokers from the effects of environmental tobacco smoke by implementing smoking intervention projects.We accomplish this in collaboration and partnership with individuals,organizations and communities in Lawrence. The Partnership is a program of the Mass Dept of Public Health.	Monday-Friday 9am-5PM	\N	Lawrence Tobacco Free Partnership	One Canal St., Entrance C	Lawrence	MA	01840	Diane Knight	Coordinator	Greater Larence Family Health Center/ Ma Dept of Public Heal	Yes	The Lawrence Tobacco Free Partnership office is located at the Northeast Center for Healthy Communities, which includes a free, health resource library. On line at www.nc4hc.org	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102222	2008-07-15	f	@lunch4all	Forward Motion Coaching	\N	PO Box 89	\N	West Boylston	MA	01583	508-835-2482	\N	\N	\N	www.ForwardMotion.info	public_agency_email@testing123.net	Career development coaching for adults with Asperger's Syndrome and Nonverbal Learning Disorder; consultation and training to organizations. Individual coaching can include career exploration, skills practice, interview skills, strategies on the job. I also provide advocacy services for individuals who are facng a disciplinary action at work or need help with an accommodation request.	In-person sessions are by appointment only in Wellesley and Worcester MA (Mon -- Fri)	\N	\N	\N	\N	\N	\N	Barbara Bissonnette	Principal	\N	No	\N	No	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102530	2008-07-15	f	@lunch4all	YMCA Haverhill	\N	81 Winter Street	\N	Haverhill	MA	01830	978-374-0506	\N	\N	978-373-0710	http://www.northshoreymca.org/pages/11491_haverhill_ymca.cfm	public_agency_email@testing123.net	The YMCA of the North Shore is committed to the values of caring, honesty, respect and responsibility.  Our YMCA provides all children, adults and families, regardless of income, with opportunities to develop a healthy spirit, mind and body. Through organized activities called programs we are able to work toward this.  Our goals are to nurture the healthy development of children and teens, to strengthen families, and to make our community a healthier, safer and better place to live.	Mon-Fri: 8:30AM-9:PM, Sat: 8:AM-8:PM, Sun: 9:AM-5:PM.	\N	\N	\N	\N	\N	\N	Tracy Fuller	Executive Director	YMCA of the North Shore	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100121	2008-07-15	f	@lunch4all	Greater Lowell Family YMCA	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_agency_email@testing123.net	The mission of the Greater Lowell Family YMCA is to put Christian principles into practice through programs that build healthy spirit, mind and body for all.	Monday - Friday 5:00 AM - 10:00 PM\r\nSaturday 7:00 AM - 7:00 PM\r\nSunday 7:00 Am - 4:00 PM\r\nMemorial Day Weekend - Labor Day Weekend\r\nSunday hours 7:00 Am - Noon\r\n\r\nSummer Hours - The only change is Sundays 7:00 AM to NOON	\N	\N	\N	\N	\N	\N	Ray Adams	Executive Director	YMCA of the USA	No	\N	Yes	Space is available for racquetball soup kitchen; pool parties, etc.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102090	2008-07-15	f	@lunch4all	North Star Guidance Center, Inc.	\N	21 Glen Ave	\N	Chelmsford	MA	01824	978-256-0667	\N	978-507-8009	978-256-5567	http://www.northstarguidanceinc.com	public_agency_email@testing123.net	At NSGC, Inc. we believe in a family-centered, community focused practice. We maintain an unwavering commitment to building strong, healthy relationships between children, parents, caretakers, supportive community resources, and service providers.  \r\n\r\nOur organization provides a variety of comprehensive counseling services in the Greater Lowell Area.  Individual, group, couples, and family therapy are all approached from a strength based, solution focused, action oriented, expressive, and systemic perspective. \r\n\r\nAt NSGC, Inc. we employ seasoned professionals that are experienced and confident in providing home based, school based, and outpatient services to children, adolescents, adults, couples, and families.	Monday-Friday 9:00-8:00	\N	\N	\N	\N	\N	\N	Laura McLaughlin, M.Ed., LMFT, LMHC	Executive Director	\N	Yes	Website links to community resources.	Yes	Classroom and group space available for 12 people	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100748	2008-07-15	f	@lunch4all	Merrimack Valley Housing Partnership	MVHP	10 Kirk Street	\N	Lowell	MA	01852	978-459-8490	\N	\N	\N	www.mvhp.org	public_agency_email@testing123.net	The mission of the Merrimack Valley Housing Partnership is to promote home ownership opportunities for low and moderate income earners.	Monday-Friday 9AM-5PM. Classes are held in the evening at Middlesex Community College from 6:30 p.m. until 9:00 p.m.	\N	P.O. Box 1042	\N	Lowell	MA	01853	Jim Wilde	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101628	2008-07-15	f	@lunch4all	Center for Women & Work at UMass Lowell	CWW	850 Broadway Street, Suite  #1,  Room B11	\N	Lowell	MA	01854	978-934-4380	\N	\N	978-934-4053	http://www.uml.edu/centers/women-work/	public_agency_email@testing123.net	The Center for Women and Work (CWW) at the University of Massachusetts Lowell is an interdisciplinary center dedicated to exploring the gendered conditions of work, challenging inequities, and thereby enhancing well-being and economic opportunities for all women.	Monday-Thursday 9am-2pm	\N	\N	\N	\N	\N	\N	Meg A. Bond	Director	University of Massachusetts Lowell	Yes	small library related to women and work	Yes	We have a small conference room that can comfortably accommodate small groups.  Please call to schedule.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100638	2008-07-16	f	@lunch4all	Brush Art Gallery and Studios	A Brush With History	256 Market Street	\N	Lowell	MA	01852	978-459-7819	\N	\N	978-453-9886	www.thebrush.org	public_agency_email@testing123.net	The mission of the Brush Art Gallery and Studios is to nurture the working artist, to keep the visual arts alive and healthy in the heart of the Lowell Historic District, to demonstrate through exhibitions and cultural programming the diverse and rich heritage that exists in Lowell; and to make Lowell a destination for viewing some of the finest art being produced in New England and beyond.	April 1 - December 31: Tuesday-Saturday 11AM-4PM; Sunday 12PM-4PM; January 1-March 31: Wednesday-Saturday 11PM-4PM; Sunday 12-4PM	\N	\N	\N	\N	\N	\N	E. Linda Poras	Executive Director	\N	No	\N	Yes	One medium size classroom for education / art classes or private lessons are available on request. The gallery is available to be rented out for meetings and receptions.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100924	2008-07-16	f	@lunch4all	Habit OPCO	Habit Management	650 SUFFOLK STREET	\N	Lowell	MA	01854	978-452-5155	\N	\N	978-970-0713	www.habitopco.com	public_agency_email@testing123.net	Habit Management is a Substance Abuse Treatment organization that provides outpatient opiate treatment services to communities in need in a manner which respects an individual's worth and recognizes their desire to improve the quality of their lives.	Monday-Friday 6AM-1PM (counseling) Staff available until 4PM; Dosing Monday-Friday 6AM-11:15AM; Saturday-Sunday 6:15-10:15AM	\N	ATTN: Jerry Myers	650 Suffolk St.	Lowell	MA	01854	Jerry Myers	Program Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-970-0724	0	
101353	2008-07-16	f	@lunch4all	Better Business Bureau, Inc. Serving Eastern MA, ME & VT	BBB	235 West Central Street Suite 1	\N	Natick	MA	01760	508-652-4800	\N	\N	508-652-4833	www.boston.bbb.org	public_agency_email@testing123.net	Our mission is to promote and foster the highest ethical relationship between businesses and the public through voluntary self-regulation, consumer and business education,and service excellence.	Normal business hours.  Also online 24 hours a day, 7 days a week on www.boston.bbb.org.	\N	\N	\N	\N	\N	\N	Paula Fleming	Vice President of Communications and Marketing	\N	Yes	The BBB has a wealth of information and topics for consumers and businesses.  Visit www.boston.bbb.org for purchase and protection tips, practical pointers, advisories, warnings, alerts, news releases, educational campaigns, etc	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100429	2008-07-16	f	@lunch4all	Saints Medical Center	SMC	One Hospital Drive	\N	Lowell	MA	01852	978-458-1411	\N	\N	\N	www.saintsmedicalcenter.com	public_agency_email@testing123.net	The mission of Saints Medical Center:\r\nTo continue the healing ministry of Jesus to those in need according to the times.	24 Hours a day, 7 days a week	\N	\N	\N	\N	\N	\N	Michael Guley	President & CEO	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101089	2008-07-16	f	@lunch4all	Habitat for Humanity of Greater Lowell	Habitat for Humanity	66 Tadmuck Road	Suite 5	Westford	MA	01886	978-692-0927	\N	\N	\N	www.lowellhabitat.org	public_agency_email@testing123.net	Habitat for Humanity of Greater Lowell (HFHGl) strengthens families and communities in need through affordable homeownership. Through volunteer labor and in-kind and monetary donations, HFHGL builds and rehabilitates homes for families earning between 25 and 50 percent of the area median income.	9:00 a.m. - 5:00 p.m. If the HFHGL voice mail picks up, please leave you name and number and someone will return your call.	\N	\N	\N	\N	\N	\N	Bill Mersch	Board of Directors President	Habitat for Humanity International	Not Applicable	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102189	2008-07-16	f	@lunch4all	Team Coordinating Agency, Inc.	TCA	66-76 Winter Street	\N	Haverhill	MA	01830	978-373-1181 x38	\N	\N	978-374-7605	\N	public_agency_email@testing123.net	Team Coordinating Agency, Inc. is a private, nonprofit social service agency that advocates for the physical and mental well being of individuals, families, and the community.  We provide a wide range of services that include prevention and treatment of addiction, children's services, and mental health treatment in residential, outpatient and forensic settings.  TCA, Inc. remains committed to its founding mission of providing quality services by responding to the changing needs of the community.  We accomplish this by providing advocacy, education, and treatment to promote personal development and growth that empowers individuals to lead personally satisfying, meaningful and productive lives.	Agency hours are Monday through Thursday 9AM to 8PM, Fridays 9AM to 5PM and Saturdays 10AM to 2PM.	\N	\N	\N	\N	\N	\N	Susan Grady, LICSW	Ambulatory Services Director	\N	Not Applicable	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100660	2008-07-16	f	@lunch4all	MassRIDES	\N	P.O. Box 121242	\N	Boston	MA	02112	888-426-6688	888-426-6688	\N	617-892-6090	www.commute.com	public_agency_email@testing123.net	The mission of MassRIDES is for Commuters to reduce roadway congestion, conserve energy and improve air quality in Massachusetts by encouraging alternatives to driving alone such as biking, walking, taking transit, carpooling, vanpooling, etc.	Monday-Friday 8:00AM - 5:00PM	\N	\N	\N	\N	\N	\N	Kay Carson	Project Manager	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100066	2008-07-16	f	@lunch4all	Center For Family Development of Lowell	\N	45 Merrimack Street, Suite 200	\N	Lowell	MA	01852	978-459-2306	\N	\N	978-453-9394	\N	public_agency_email@testing123.net	The mission of the Center for Family Development is to provide comprehensive, high quality mental health services to the greater Lowell Community.	Monday-Thursday 8AM-8PM; Friday 8AM-5PM	\N	\N	\N	\N	\N	\N	Richard Hassinger	Clinical Director	Comprehensive Outpatient Services	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102002	2008-07-16	f	@lunch4all	Aspirations	\N	The University of Massachusetts Lowell, Dept. of Psychology	1 Mahoney Hall, 870 Broadway Street	Lowell	MA	01854	978-934-2930	\N	\N	978-934-3074	http://faculty.uml.edu/ahillier/aspirations.htm	public_agency_email@testing123.net	Aspirations is a social and vocational skills support group for high functioning adolescents and young adults on the autism spectrum aged from 18-30 years.  Aspirations is designed to provide the opportunity to discuss social and vocational skills in a small group setting. Group members are guided in ways to improve their social and vocational skills. Group leaders familiar with autism spectrum disorders facilitate an atmosphere of acceptance and friendship. Developing self-awareness, social insight, and independence are central themes of Aspirations. Aspirations also provides an opportunity for group members to meet others on the autism spectrum and gain support and understanding from one another.  Participants are encouraged to develop friendships with other group members. \r\nAspirations runs for nine weeks and meets once a week on Tuesdays from 6.15-7.30pm at the University of Massachusetts, Lowell south campus. We run the program twice a year, once in Fall and once in Spring. \r\nWe also have a facilitated parent support group which runs at the same time and place as the Aspirations program. This gives parents the opportunity to meet, talk and share useful information.	Once a week for 9-weeks, Tuesdays 6.15-7.30pm.	\N	\N	\N	\N	\N	\N	Ashleigh Hillier, Ph.D.	Assistant Professor of Psychology	The University of Massachusetts Lowell	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102849	2008-07-16	f	@lunch4all	Ellen H. Korin, M.Ed.	\N	10 COACH RD	\N	Lexington	MA	02420	781-861-6431	\N	339-223-0928	781-652-0436	www.ellenhkorin.com	public_agency_email@testing123.net	Providing consultation and coaching services to persons with autism spectrum disorders, notably Asperger Syndrome and Non Verbal Learning Disabilities.  Accepts clients from the age of 12 through adulthood. \r\n  In addition, offers customized professional development programs for educators, clinicians, parents and teens and adults on the spectrum.	Appointments arranged per client needs - Office hours on Wednesdays in Arlington; in home office on other days.	\N	\N	\N	\N	\N	\N	Ellen H. Korin	Consultant and Coach	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103399	2008-07-17	f	@lunch4all	International Medical Equipment Collaborative	IMEC	1600 Osgood Street	\N	North Andover	MA	01845	978-557-5510	\N	\N	978-557-5525	www.imecamerica.org	public_agency_email@testing123.net	International Medical Equipment Collaborative is a non-profit organization\r\nin North Andover whose mission is to serve doctors and nurses in\r\ndeveloping countries by providing them with quality medical equipment to\r\nadvance the standard of health for their nation's poor. IMEC fulfills its\r\nmission by working with other humanitarian organizations to revitalize\r\nexisting, impoverished hospitals with donated surplus medical equipment\r\nand supplies that are acquired, repaired and packaged by donated\r\nvolunteers.	Monday through Friday 9am-4pm\r\nSaturdays (on scheduled basis)	\N	\N	\N	\N	\N	\N	Thomas Keefe	President and Founder	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101122	2008-07-22	f	@lunch4all	The Caleb Group	\N	491 Humphrey Street	\N	Swampscott	MA	01907	781-595-4665	\N	\N	781-592-0770	www.thecalebfoundation.org	public_agency_email@testing123.net	The Caleb Group, Inc.\r\n\r\nBuilding Community, One Spirit at a Time\r\n\r\nThe Caleb Group is an interfaith organization that coordinates services to enhance and empower the lives and spirits of low and moderate-income elderly, disabled and families in affordable housing communities. This is accomplished by, both responding to felt resident needs, and intentionally building a strong sense of community and mutual respect.\r\n\r\nSome programs \r\n-Community centers\r\n-Computer learning centers\r\n-Self-enrichment workshops\r\n-Transportation to health care\r\n-Tutoring and mentoring for at-risk youth and teens.\r\n-Summer camp opportunities for youth\r\n-Job training and legal service referrals\r\n-G.E.D. certification courses\r\n-support groups. \r\n\r\n**********************************************************\r\n\r\n\r\nMission Statement\r\n\r\n\tThe mission of The Caleb Group, Inc. is the changing of lives, freeing of the human spirit, and enhancing and promoting self-reliance by coordinating resident services for low and moderate income elderly and families as well as the homeless and handicapped. This is accomplished by responding to felt resident needs, and by intentionally building a strong sense of community and mutual respect.\r\n\r\nTo accomplish the mission The Caleb Group will: \r\n\r\nProvide qualified staff to serve as Resident Service Coordinators on-site at each property affiliated with the Caleb Foundation or under contract with independent owners.\r\n\t\r\nEmpower tenants by involving them in various aspects of decision making in managing the property, often by organizing Resident Advisory Committees.\r\n\r\nSurvey tenant needs to determine what supportive services are needed for the residents and arrange low-cost solutions such as computer learning centers, adult education opportunities, developmental programs for community youth, and the coordination of meals, medical and social support.\r\n\r\nEncourage a healthy community environment by inviting local service agencies onto each \tproperty to run specific developmental programs for youth, family and or elderly.\r\n\r\nNetwork with like organizations and service agencies in the local area to leverage available resources and optimize community services.	Monday through Friday 9-5 PM	Eight computers are linked to the internet.	77 Chestnut Street	\N	Lowell	MA	01852	Veronica Holmes	Service Coordinator	\N	Not Applicable	\N	Yes	There is a community room with warming kitchen as well as a computer learning center with 8 computers and training software for Lowell residents wishing to learn computer skills.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100132	2008-07-23	f	@lunch4all	Pathfinder / Safe Haven	\N	94 Rock Street	\N	Lowell	MA	01854	978-459-3387	\N	\N	978-970-1096	http://www.bridgewell.org	public_agency_email@testing123.net	Pathfinder/Safe Haven's mission is to work toward the end of homelessness in Lowell. This is executed with an attack on homelessness. Pathfinder/Safe Haven specifically serves the needs of the mentally ill homeless population with safe-havens and generally serves the needs of all the homeless with cooperative outreach efforts. By understanding the nature of mental illness and the other causes and perpetuators of homelessness, Pathfinder/Safe Haven is able to help the homeless better manage their lives. After fulfilling immediate safety and survival needs, Pathfinder/Safe Haven broadens the pyramid to teach the homeless it serves the skills necessary to regain and maintain the highest degree of autonomy possible. Pathfinder/Safe Haven harnesses the education and life experience of its staff and participants to provide a multi level service plan capable of satisfying the diverse needs of the homeless. Placing emphasis on housing and hunger issues, medical and mental health care, substance abuse counseling, and job training and education, Pathfinder/Safe Haven provides the homeless a safe, sober environment and the support necessary to make a successful transition from the street.	Monday-Sunday 8AM-8PM	\N	\N	\N	\N	\N	\N	Kristin Ross-Sitcawich	Program Director	Bridgewell, Inc	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101958	2008-07-24	f	@lunch4all	Lowell Treatment Center	\N	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5160	\N	800-222-2237	\N	www.arbourhealth.com	public_agency_email@testing123.net	Lowell Treatment Center offers inpatient treatment for children and adolescents, partial hospitalization programs for adolescents and adults, and a Family Stabilization Team.	24 hours a day, 7 days a week.	\N	\N	\N	\N	\N	\N	Ralph Lawson	Director	Arbour Health System	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102453	2008-07-24	f	@lunch4all	Lawrence Boys and Girls Club	\N	136 Water Street	\N	Lawrence	MA	01841	978-683-2747	\N	\N	978-725-5989	http://www.lawrencebgc.com	public_agency_email@testing123.net	The mission of the  Boys & Girls Club of Lawrence is to maintain a Club for all youth without distinction of race, sex, color or creed; to provide behavior guidance, and to promote the health, social, educational, vocational and character development of all its members.	Mon - Fri: Summer Months: 9:45AM - 3:45PM\r\nMon - Fri: School Months: 3:00PM - 7:00PM for ages 7 thru 9, and 3:00 PM - 10:30PM for ages 10+.	\N	\N	\N	\N	\N	\N	Markus Fischer	Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100297	2008-07-24	f	@lunch4all	Lowell Transitional Living Center, Inc.	\N	189 Middlesex Street	\N	Lowell	MA	01852	978-458-9888	\N	978-458-9888	978-452-2186	\N	public_agency_email@testing123.net	Lowell Transitional Living Center is a community supported non-profit organization that provides safe, temporary shelter for homeless men and women of Greater Lowell. In a multi-service environment, volunteers and staff provide opportunities for individuals to end their homelessness.	24 hours/day, 7 days a week, 365 days per year	\N	\N	\N	\N	\N	\N	Peter Duda	Executive Director	\N	No	\N	Yes	AA meetings daily 2-3 pm	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100099	2008-07-30	f	@lunch4all	Boys & Girls Club of Greater Lowell, Inc.	Lowell Boys Club	657 Middlesex Street	\N	Lowell	MA	01851	978-458-4526	\N	\N	978-453-9740	www.lbgc.org	public_agency_email@testing123.net	The mission of the Boys & Girls Club of Greater Lowell is to inspire and enable all youth, especially those from disadvantaged circumstances, to realize their full potential as caring, responsible citizens.	School Year:  \r\nMon.-Fri. 2:00-6:00 7-18 year olds\r\nMon & Wed 2:00-8:00 teens (13-18 years\r\nHolidays/School vacation weeks:  10:00 - 5:00	Seasonal programs are offered in five core areas of the Club:  Sports, Fitness & Recreation, Education & Careers, Health & Life Skills, Leadership & Development and The Arts \r\n\r\nThe Club also offers off-site programming in collaboration with Lowell Housing Authority at the North Common Village and Flanagan Centers.  The Club offers a satellite program in Centralville at the Monsignor Keenan Playground (aka Eagle Park) and during the winter will identify an indoor location to serve the youth of Centralville in their neighborhood.	\N	\N	\N	\N	\N	Michelle Hatem-Meehan	President	Boys & Girls Clubs of America	Yes	Newly developed Learning Center complete with books for youth and teens, computers for research, homework help and reading and writing projects.	Yes	Teen Room, computer lab, matted room, multipurpose room, Fitness facility, gym,pool, Learning Center, Gameroom, Arts & Crafts room, Snack Room. Call for more information if you would like to learn more about room rentals for meetings and events.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101001	2008-07-30	f	@lunch4all	Consumer Credit Counseling Service of Southern New England	CCCS	8 Winter Street	Suite 700	Boston	MA	02108	866-889-9347	866-889-9347	\N	617-960-8299	www.moneymanagement.org	public_agency_email@testing123.net	We improve lives through financial education.	Monday-Thursday 9AM-5PM - counseling available on phone, internet and in person.	\N	\N	\N	\N	\N	\N	Ivan Hand	President	Money Management International	No	Resources on-line at http://www.moneymanagement.org	Yes	Facilities for conducting literacy education are available at several sites in MA, CT and RI -- contact for more information.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100946	2008-07-30	f	@lunch4all	Clarendon Family Day Care, Inc.	\N	C/O Holden Center 220 Pawtucket Street, 2nd floor	\N	Lowell	MA	01854	978-454-3026	800-875-1234	\N	978-452-2073	www.clarendonfamilydaycare.org	public_agency_email@testing123.net	Clarendon's mission is to provide quality, culturally sensitive and affordable child care services to local families; promote small business and professional development opportunities for men and women in the Massachusetts family child care industry; provide professional child care services and additional support services to assist the efforts of parents working to improve their family's quality of life and to ensure a safe, nurturing environment to enhance the growth and development of each child to whom we provide services.	Hours vary - contact program	\N	200 The Great Road, Suite 258	\N	Bedford	MA	01730	Y Ruiz	Regional Director	\N	Yes	Books, Toys, Puzzles, Computers with Childrens learning games and internet browsing (Great to do research on).	Yes	Many different area's within the Hospital, ranging from small to ex-large.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101936	2008-08-01	f	@lunch4all	Greater Lawrence Family Health Center	GLFHC	34 Haverhill Street	\N	Lawrence	MA	01841	978-686-0090	\N	\N	\N	www.glfhc.org	public_agency_email@testing123.net	To improve and maintain the health of individuals and families in the Merrimack Valley by providing a network of high quality comprehensive services and by training health care professionals to respond to the needs of a culturally diverse population.	Headquarters  Clinica Principal\r\n34 Haverhill Street (978) 686-0090\r\nMonday, Wednesday, & Thursday: 8:00 AM  8:30 PM\r\nTuesday: 8:00 AM  9:30 PM\r\nFriday: 9:00 AM  9:00 PM\r\nSaturday: 8:45 AM  4:30 PM\r\nSunday: 12:30 PM  4:30 PM\r\n\r\nNorth Site  Clinica del Norte\r\n150 Park Street (978) 685-1770\r\nMonday, Wednesday, & Thursday: 8:00 AM  4:30 PM\r\nTuesday: 8:00 AM  8:30 PM\r\nFriday: 9:00 AM  4:30 PM\r\n\r\nSouth Site  Clinica del Sur\r\n73D Winthrop Avenue (Plaza 114) (978) 686-3017\r\nMonday & Wednesday: 8:00 AM  8:30 PM\r\nTuesday & Thursday: 8:00 AM  4:30 PM\r\nFriday: 9:00 AM  4:30 PM\r\n\r\nWest Site  Clinica del Oeste\r\n700 Essex Street (978) 689-2400\r\nMonday, Tuesday, & Thursday: 8:00 AM  4:30 PM\r\nWednesday: 8:00 AM  8:30 PM\r\nFriday: 9:00 AM  4:30 PM	\N	\N	\N	\N	\N	\N	Robert Ingala	CEO	\N	Yes	Northeast Center for Healthy Communities: A resource information center serving 48 north of Boston communities	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101903	2009-02-08	f	@lunch4all	A Woman's Concern Pregnancy Health Services	AWC	103 Broadway	\N	Revere	MA	02151	781-284-8747	\N	866-435-7292	781-284-1167	www.awomansconcern.org	public_agency_email@testing123.net	To provide free pregnancy testing and ultrasound exams (when indicated)as well parenting programs, mentoring programs for both mothers and dads, referrals, infant & toddler clothing, maternity clothing	Monday - Thursday 9:30-5:30\nFriday 9:30-3:30\r\nSaturday 2x per month	\N	\N	\N	\N	\N	\N	Teresa Larkin	Center Director	\N	Not Applicable	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100374	2008-08-01	f	@lunch4all	Rape Crisis Services of Greater Lowell	\N	144 Merrimack Street, Suite 304	\N	Lowell	MA	01852	978-452-7721	800-542-5212	800-542-5212	978-458-2822	www.rcsgl.org	public_agency_email@testing123.net	Rape Crisis Center of Greater Lowell is dedicated to the healing and empowerment of survivors of rape and sexual assault through counseling, advocacy, and education. RCSGL is committed to creating an environment where sexual violence and other forms of oppression are eradicated. RCSGL is committed to create a community where individuals and society can break the silence that surrounds assault.	Monday-Friday 9AM-5PM; After hours availability through 24 hour hotline	(978) 452-8723 TDD Number	\N	\N	\N	\N	\N	Sun He Pak Gage	Executive Director	\N	Yes	Available by pre-arrangement only	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-452-8723	0	
101595	2008-08-01	f	@lunch4all	All Staff, LLC	All Staff	1565 Main Street	Building 2, 3rd floor	Tewksbury	MA	01876	978-863-0361	\N	978-863-0361	978-863-1974	http://www.allstaff.us	public_agency_email@testing123.net	All Staff mission is to provide relief staff to the human service field.	Office is open 9a-5p Mon-Fri \r\n24/7 phone coverage	\N	\N	\N	\N	\N	\N	Frederick Porter	CEO	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101584	2008-08-01	f	@lunch4all	Orchestra of Indian Hill	Indian Hill Music	36 King Street	\N	Littleton	MA	01460	978-486-9524	\N	\N	978-486-9844	www.indianhillmusic.org	public_agency_email@testing123.net	Greater Boston's premier professional orchestra, led by Artistic Director & Conductor Bruce Hangen, presents a six-concert classical and pops series at Littleton High School Performing Arts Center (56 King Street). Additionally, Indian Hill Music offers jazz, chamber music, musical theater and family events at its own concert venue at 36 King Street.	Office: 9 am - 5 pm\r\nTicket prices vary. Free concerts also offered.	\N	PO Box 1484	\N	Littleton	MA	01460	Susan Randazzo	Executive Director	\N	Not Applicable	\N	Yes	150 seat hall is available for meetings and functions	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	800-439-2370	0	
100440	2008-08-01	f	@lunch4all	Visiting Nurse Association of Greater Lowell	VNA of Greater Lowell	336 Central Street	\N	Lowell	MA	01852	978-459-9343	800-349-8585	\N	978-441-0007	www.vnalowell.org	public_agency_email@testing123.net	The VNA of Greater Lowell, a trusted community leader since 1909, is committed to providing the highest quality home health care. We believe that every person is entitled to receive individualized, comprehensive and culturally appropriate health care. We use every available resource and collaborate with others in providing our services. We recognize the dedication of our employees and commit our resources to their growth and development. Our values: Respect, Integrity, Compassion, Enhancement of Practice.	24 hours a day, 7 days a week	\N	336 Central Street	\N	Lowell	MA	01852	Nancy L. Pettinelli	Executive Director	\N	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101166	2008-08-01	f	@lunch4all	The Single Life Inc.	TSL	P O Box 8222	.	Lowell	MA	01853	978-441-2765	\N	\N	\N	thesinglelife.org	public_agency_email@testing123.net	The Single Life is a non-profit organization of independent but related chapters whose purpose is to provide single people with a social outlet and the opportunity to meet, communicate and establish friendships with each other. We are not a support group; however, members are supportive of and toward each other. Discussions meeting nights allow members to express ideas if they wish and listen to others, which promotes deeper understanding and friendship. In addition, organized social activities provide fun and entertainment for everyone.\r\n\r\nThe Single Life is for mature singles age 50 plus who are Divorced, Widowed, Separated, (not living with spouse or partner)and Single.	Office Hours: By appointment only	Individual chapters are located throughout Northeastern Massachusetts and Southern New Hampshire and serve the following areas:\r\nActon, Billerica, Beverly, Cape Ann, Concord, Chelmsford, Danvers, Everett, Greater Haverhill, Ipswich, Lexington, Lowell, Lynn, Peabody, Malden, Medford, Merrimack Valley, Melrose, Methuen,North Shore, Reading, Rockport, Tewksbury, Wilmington, Woburn and Winchester. \r\n \r\nWe also serve Southern New Hampshire including Nashua, Pelham, Salem, Plaistow and Windham areas.	\N	\N	\N	\N	\N	John Lawlor, Chairman	Co-Founder	\N	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102211	2008-08-04	f	@lunch4all	Merrimack Valley Chapter Of Parents Of Murdered Children	P.O.M.C.	Saints Medical Center	Stackpole St	Lowell	MA	01852	978-452-5858	\N	\N	978-555-5121	http://www.pomcma.com/	public_agency_email@testing123.net	To provide support for parents of murder children  and also to provide support for family and friends that also died by violence	Every 2nd Tuesday each month 6:30 - 8:30 pm	\N	P.O. 8737	\N	Lowell	MA	01853	Arnie Muscovitz	Chapter Leader	Parents Of Murdered Children,Inc	Not Applicable	\N	Yes	Board room  which is located on the first floor of the Residence Building on Stackpole St.	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101243	2008-08-04	f	@lunch4all	Merrimack Valley Food Bank, Inc.	\N	735 Broadway Street	\N	Lowell	MA	01854	978-454-7272	\N	\N	978-454-1717	www.mvfb.org	public_agency_email@testing123.net	To provide adequate nutrition, freedom from hunger and to share vital information regarding food safety and security to our community.  This is accomplished by our many feeding programs and through a network of over 100 emergency feeding agencies throughout Greater Lowell, the Merrimack Vally, North Shore and Southern NH.  Our agencies include shelters, meals programs, food pantries and group homes.  Clients served by our agencies include:  low income elderly, physically & developmentally disabled, children, working families, survivors of abuse, veterans and recovering addicts.\r\n\r\nPrograms include:\r\nFood Distribution Program\r\nMEFAP (State food commodities)\r\nUSDA (Federal food commodities)\r\nGuy Francesshelli Food Rescue Program\r\nMobile Pantry (formerly M/A-COM Food Share)\r\nHealthy Summer -Summer Lunch Program\r\nCo-Op Program\r\nNutrition and Food Safety Program	Mon - Fri 9am - 4pm	\N	\N	\N	\N	\N	\N	Amy L. Pessia	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102398	2008-08-04	f	@lunch4all	Career Resources Corporation	\N	165 Winter Street	\N	Haverhill	MA	01830	978-374-9122	\N	\N	978-374-1532	http://www.crc-mass.org/	public_agency_email@testing123.net	The Career Resources Corporation's Mission is to to empower people to be as self-sufficient, self-determining, and successful as possible. Individuals with developmental disabilities are supported in their own jobs and homes and integrated into community life, recognizing that they're unique, valued people and that communities are enriched by their participation.	Mon-Fri: 8:AM-5:PM	\N	\N	\N	\N	\N	\N	Brad Howell	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100308	2008-08-07	f	@lunch4all	Mental Health Association of Greater Lowell, Inc.	MHA	99 Church Street	\N	Lowell	MA	01852	978-458-6282	\N	978-458-6282	978-441-9826	www.mhalowell.org	public_agency_email@testing123.net	The Mental Health Association of Greater Lowell, Inc. was organized in 1953 as a private, non-profit corporation. The Association's services over the years have changed and developed to keep pace with the mental health needs and concerns of the Greater Lowell area. Since its inception, the Association has actively sponsored and participated in the delivery of a wide range of services and activities including: rehabilitative and preventive mental health programs for children and adults, resource information and referral projects, residential services to mentally ill and developmentally disabled persons, consultation and education services and various other special projects.	Monday-Thursday 8AM-7PM\r\nFriday 8AM-5PM	\N	\N	\N	\N	\N	\N	Albert H. Scott	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103410	2008-08-08	f	@lunch4all	Transformations	\N	144 Merrimack Street, Suite 435	\N	Lowell	MA	01852	781-333-4696	\N	\N	\N	www.transformationshealing.org	public_agency_email@testing123.net	Transformations offers empowering, compassionate, creative individual, couples and group psychotherapy.\r\nWe value the relationship between client and practicioner as the foundation for healing.\r\nWithin a feminist, anti-oppression framework, our intention is to help people use their voice, discover their own strengths and needs, and find peace and happiness.\r\nOur healing center located in hip downtown Lowell offers a space that fosters a safe, creative and nurturing environment for our clients. We believe in meeting people where they are and using a holistic approach. \r\n\r\nSliding scale available for all services.  \r\nMassachusetts Victim Compensation accepted for all services	various	\N	\N	\N	\N	\N	\N	shared: Trish Bakaitis-Glover, Andrea Canty, Lisa Twente	Therapist	\N	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100550	2008-08-11	f	@lunch4all	Justice Resource Institute	\N	380 Massachusetts Avenue	\N	Acton	MA	01720	978-263-3006	\N	\N	978-263-3088	www.JRI.org	public_agency_email@testing123.net	Justice Resource Institute is dedicated to addressing the most confounding challenges of both the human services and educational systems and the persons and families these systems were created to serve. Justice Resource Institute pursues the social justice inherent in opening doors to opportunity and independence. Although our range of services is as varied as those we serve, our approach is uniformly characterized by compassionate support, innovation, and community leadership.\r\n\r\nJRI works in partnership with individuals, families, communities and government to address their shared challenges in a comprehensive, coordinated, systematic, and effective manner. JRI seeks new knowledge and improved evidence-based practice, in research and in the field, in order to inform our continuous search for excellence in service.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Gregory Canfield	Executive Director	Justice Resource Institute	No	\N	Yes	Please call 978-263-3006,for more information.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100825	2008-08-12	f	@lunch4all	Moms and Tots of Billerica	\N	216 River Street	\N	Billerica	MA	01821	978-667-9771	\N	\N	\N	www.momsandtots.org	public_agency_email@testing123.net	The mission of Moms and Tots is to bring parents and children together and, in doing so, help make the challenging job of parenting more fun and rewarding. We offer quality activities for infants, toddlers, preschoolers and their families. Including playgroups, day trips, parties, mom's night out and more.	We offer a monthly newsletter with the dates and times of events and meetings.	\N	\N	\N	\N	\N	\N	Sarah SanClemente	President	\N	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100209	2008-08-18	f	@lunch4all	Greater Lowell Chamber of Commerce	\N	131 Merrimack Street	\N	Lowell	MA	01852	978-459-8154	\N	\N	978-452-4145	www.glcc.biz	public_agency_email@testing123.net	The mission of the Chamber is to provide a significant unified voice in shaping the future growth of the business community which it services and to provide a vehicle through which individuals can work to improve the areas business climate and quality of life.	Monday - Friday 8:30 AM - 5:00 PM	\N	\N	\N	\N	\N	\N	Jeanne Osborn	President & CEO	\N	No	\N	Yes	Board Room	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102442	2008-08-15	f	@lunch4all	Adelante Inc.	Adelante Youth Center	40 Lawrence Street	\N	Lawrence	MA	01840	978-687-9432	\N	\N	978-794-3407	http://www.adelanteyouthcenter.org	public_agency_email@testing123.net	Adelante! (Spanish for "moving forward")  provides Lawrence's urban youth with opportunities to enhance their education, leading them to greater economic success in adulthood.	Monday thru Friday, 9am - 5:30pm\r\n7th and 8th Grade Prep Classes Tuesdays 4-5pm, Wednesdays and Thursdays 4-6pm	\N	\N	\N	\N	\N	\N	David Hildt	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100176	2008-08-15	f	@lunch4all	Merrimack Valley Legal Services, Inc.	Merrimack Valley - North Shore Legal Services, Inc.	35 John Street, Suite 302	\N	Lowell	MA	01852	978-458-1465	800-336-2262	978-458-1465	978-458-3481	www.mvlegal.org	public_agency_email@testing123.net	The mission of Merrimack Valley Legal Services is to provide high quality legal representation and information to low income families and individuals with civil matters.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Kenneth F. MacIver,Jr.	Executive Director	\N	Yes	Contact agency for more information about the resource library	Yes	Meeting space is available if it concerns an MVLS client.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-452-4740	0	
102871	2008-08-15	f	@lunch4all	Junior Achievement of Eastern MA	\N	33 Broad St	7th Floor	Boston	MA	02109	617-368-3566	\N	\N	617-368-6898	www.jaem.org	public_agency_email@testing123.net	Our mission is to inspire and prepare young people to succeed in a global economy.  \r\n\r\nJA Worldwide (Junior Achievement) is the worlds largest organization dedicated to educating students about work readiness, entrepreneurship and financial literacy through experiential, hands-on programs. \r\nJunior Achievement programs help prepare young people for the real world by showing them how to generate wealth and effectively manage it, how to create jobs which make their communities more robust, and how to apply entrepreneurial thinking to the workplace. Students put these lessons into action, and help strengthen their communities.	M-F \r\n9 AM - 5 PM	\N	\N	\N	\N	\N	\N	David Eustis	President	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101573	2008-08-15	f	@lunch4all	Coalition for a Better Acre	CBA	517 Moody Street	3rd floor	Lowell	MA	01854	978-452-7523	\N	\N	978-452-4923	http://www.coalitionforabetteracre.org	public_agency_email@testing123.net	The Coalition for a Better Acre is a membership based community development corporation dedicated to resident empowerment and sustainable community revitalization for current and future residents of Lowell. We promote healthy, vibrant neighborhoods by developing resident leaders, affordable housing and economic opportunities, and by responding to community needs through collective action.	M-F 9:00-5:00 (but we often have evening and weekend activities)	\N	\N	\N	\N	\N	\N	Emily Weitzman-Rosenbaum	Executive Director	\N	Not Applicable	\N	Yes	Meeting room that can accommodate up to 15 people	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100902	2008-08-20	f	@lunch4all	Latin American Health Institute	\N	9 Central St	Suite 400. 4th Floor	Lowell	MA	01852	978-459-3366	\N	\N	978-446-0817	www.lhi.org	public_agency_email@testing123.net	The Latin American Health Institute is a community-based professional organization that promotes the health of the community, its institutions, families and individuals through effective interventions that are culturally competent and technologically appropriate.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Oscar Patino	Director	\N	No	\N	Yes	Space is available for meetings, group workshops and training sessions. Call for more information.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103432	2008-08-18	f	@lunch4all	Second World	New England Culture Fest	172 Middle Street	\N	Lowell	MA	01852	978-455-6025	\N	\N	\N	www.second-world.com	public_agency_email@testing123.net	Second World is a dynamic non profit arts and development organization combining media, music and art for a better world. Based on a decade of work, Second World aims to address the crucial need for imaginative and effective educational methods regarding cultural competency, diversity & fair trade. Fair Trade Films, Lectures and Art Exhibitions are some of the ways to interact with us at Second World. Second World helps to bridge the gap between the First and Third World through media documentation, musical events and fair trade art empowerment.	By Appointment	All volunteer staff have full internet access on wireless network.	\N	\N	\N	\N	\N	Valerie Parker	Director	\N	Yes	10 years worth of multimedia footage in archive from world travels and outreach efforts on behalf of fair trade.  In development for documentaries and fair trade promotional materials.	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102057	2008-08-19	f	@lunch4all	Art Ship Enterprise	\N	365 West Meadow Rod	\N	Lowell	MA	01854	978-821-0953	\N	\N	\N	www.cultureiscool.org/whatiscool/cool-projects/coolmobile	public_agency_email@testing123.net	The Art Ship Enterprise, a bright red bus loaded with supplies and imagination, is bring art to a neighborhood near you! In its mission to "connect and inspire communities through the arts," this vibrant classroom on wheels offers art activities and inspiration to the city of Lowell. \r\n\r\nRanging from poetry workshops at homeless shelters to painting classes at childcare centers, the Art Ship Enterprise seeks to bring art into all peoples lives. \r\n\r\nThe Art Ship Enterprise, in partnership with the Revolving Museum, is engaging citizens of Lowell in projects to promote positive expression, empowerment and peace. Through its exciting programs, the Art Ship Enterprise provides people with the tools to envision and create their ideal world and identity.	Monday through Thursday, 9:00 a.m. - 12:00 p.m.	\N	\N	\N	\N	\N	\N	Marta Magnus	Supervisor	Community Teamwork Inc.	Not Applicable	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101947	2008-08-19	f	@lunch4all	Neighborhood Health Plan	\N	253 Summert St	\N	Boston	MA	02210	617-772-5500	1-800-462-5449	\N	617-772-5513	www.nhp.org	public_agency_email@testing123.net	Neighborhood Health Plan's mission is to provide health care that is both member-focused and quality driven. To accomplish this, NHP works in partnership with community health centers and other community responsive providers to maintain a comprehensive, high quality network.\r\nNHP is NCQA Accredited.	Monday-Friday\r\nSwithboard 8:30am to 5:30pm\r\nMember Services 8:30am to 6:00pm	\N	253 summer St	\N	Boston	MA	02210	Deborah Enos	President CEO	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100143	2008-08-20	f	@lunch4all	Career Center of Lowell	\N	18 John Street	\N	Lowell	MA	01852	978-458-2503	\N	\N	978-805-4913	www.cclowell.org	public_agency_email@testing123.net	The Career Center of Lowell, under the guidance of the Greater Lowell Workforce Investment Board, is a One-Stop Career Center. Through this model, local employment and training services have been consolidated to build a strong workforce development system. The Charter of the Career Center is to work closely with the business community to identify and meet their employment needs, and provide job seekers with a wide range of employment and training services. The Career Center is a membership organization, which means that you must complete a membership form to access services which are Free of Charge. Our staff is committed to providing you with high quality, individualized service which is tailored to your needs.	Monday, Tuesday, & Wednesday 8:30 AM - 5:00 PM, Thursday 8:30 AM - 7:00 PM, Friday 9:30 AM - 5:00 PM	(978) 970-3303 TDD Number	\N	\N	\N	\N	\N	Michael McQuaid	Executive Director	City of Lowell	Yes	Call for more information	Yes	Meeting space accommodating up to 25 people. Call for availability.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100935	2008-08-20	f	@lunch4all	Social Security Administration	\N	151 Warren Street, Suite 300	\N	Lowell	MA	01852	978-452-5509	800-772-1213	\N	978-452-8945	www.socialsecurity.gov	public_agency_email@testing123.net	The Social Security Administration is the government agency responsible for administering the Social Security, Social Security Disability Insurance and Supplemental Security Income programs.	Monday-Friday\r\n9:00AM-4:00PM	\N	\N	\N	\N	\N	\N	Thomas Abbott	Manager, Lowell office	United States government	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-458-5702	0	
102585	2008-08-20	f	@lunch4all	Little Sprouts Child Enrichment Centers, Inc	\N	900 Chelmsford Street	\N	Lowell	MA	01851	978-291-0342	\N	\N	978-291-0344	www.littlesprouts.com	public_agency_email@testing123.net	To create a nurturing and safe atmosphere that encourages growth, development & fulfillment encompassing each child, family and professional in the Little Sprouts community while supporting our innovative philosophy through effective communication thus enriching the lives of all we touch.	Monday - Friday 6 am to 6 pm	\N	\N	\N	\N	\N	\N	James Carlson	Director of Programs	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100495	2008-08-20	f	@lunch4all	CAB Health & Recovery Services, Inc.	\N	111 Middleton Rd	\N	Danvers	MA	01923	978-777-2121	800-323-2224	\N	978-750-3620	www.cabhealth.org	public_agency_email@testing123.net	CAB Health & Recovery Services, Inc. is committed to building healthy communities by providing a continuum of quality substance abuse and related health treatment and prevention services that strengthen the capacity of all families, businesses and communities to lead satisfying lives.	Please contact each program directly.	\N	\N	\N	\N	\N	\N	Kevin P. Norton	President	Northeast Health System	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101056	2008-08-20	f	@lunch4all	International Institute of Lowell	I I Lowell	144 Merrimack St., Suite 202	\N	Lowell	MA	01852	978-459-9031	\N	\N	978-459-0154	www.iilowell.org	public_agency_email@testing123.net	Since 1918, the International Institute has assisted immigrants and refugees from over 70 different countries to build their lives as new residents of the Lowell area. We are dedicated to promoting the acceptance of all, regardless of race, religion, country of origin or ethnic background, as equal participants in life in the United States.	Monday through Friday, 9 to 5	Volunteer Opportunity\r\n\r\nDescription\r\nThe International Institute of Lowell is open to dedicated people who would like to volunteer in the following areas:\r\n\r\n1.Office: Tasks will include answering calls to the Institute and setting appointments for our clients, other office duties such as copying, filing, etc., and being the smiling face to greet our clients when they walk in for the first time.\r\n2.Intern or volunteer with some interest or experience in immigration law, to assist us with forms, review client files, etc.\r\n3.Employment/ESL/citizenship: Tasks may include literacy tutoring, searching the Internet for employment opportunities for our clients, researching jobs, and assisting in citizenship classes.\r\n4.Case management with recent asylees who have either recently arrived in the US or recently been granted asylum by the US government.\r\n\r\nThe tasks above may be combined to meet the interests and skill sets of our volunteers.  \r\n\r\nRequired Skills\r\nThere is no professional expertise required.  However, we require our volunteers to have the ability to work well with the immigrant population.  Our volunteers must show patience and have the ability to work well with others, be self-motivated, and stay dedicated to their schedule throughout their commitment with the Institute.  Commitment is a minimum of 8 weeks, with a minimum of 4 hours/wk.	\N	\N	\N	\N	\N	Rebecca Feldman	Director	International Institute of Boston (I I B)	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100396	2008-08-27	f	@lunch4all	The Family Connection of Greater Lowell, Inc.	TFC	c/o Pawtucket Congregational Church	15 Mammoth Road	Lowell	MA	01854	781-738-2392	\N	\N	\N	www.TFCplayspace.weebly.com	public_agency_email@testing123.net	The Family Connection is a nonprofit, non-denominational, volunteer-run organization designed for infants through preschoolers. Our mission is to provide social and recreational opportunities to Merrimack Valley families with young children.\r\n\r\nThe TFC features a large indoor area where young children can play in a safe environment, and where parents can connect with one another. Every playdate includes time for free play, snacks, arts and crafts, and a music circle.	We are open Monday and Thursday mornings from 10 a.m. to 12 noon.	\N	\N	\N	\N	\N	\N	Sarah Seguin and  Anne Leslie	Co-directors	\N	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102134	2008-08-27	f	@lunch4all	Alternative House, Inc.	Alternative House, Inc.	P.O. Box 2100	\N	Lowell	MA	01851	978-937-5777	1-888-291-6228	1-888-291-6228	978-937-5595	www.alternative-house.org	public_agency_email@testing123.net	The mission of Alternative House is to facilitate the creation of a society in which violence against women will no longer exist.  As a means to this end, we offer shelter, support, options, counseling, and advocacy for all battered women and their children who seek our help.\r\n\r\nWe are committed to the empowerment of women towards self-sufficiency.  We do not discriminate against any race, class, culture, age group, or sexual orientation.\r\n\r\nIn addition, we provide community education to reform societal attitudes that permit violence against women.\r\n\r\nWe abide by the Principles of Unity put forth by the Massachusetts Coalition of Battered Women's Service Groups.	Business Office 9:00 a.m.-5:00 p.m. M-F\r\nEmergency Hotline is 24 hours, seven days a week.	\N	\N	\N	\N	\N	\N	Kathleen Kelley	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100330	2008-09-04	f	@lunch4all	ONE Lowell: Building a New American Community	\N	9 Central Street	Suite 203	Lowell	MA	01852	978-654-6957	\N	\N	978-454-6992	www.onelowell.net	public_agency_email@testing123.net	ONE Lowell strives to be the voice that improves the life, opportunities and involvement of culturally diverse people in the  Lowell community.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Dr. Victoria Fahlberg	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100022	2008-09-02	f	@lunch4all	YWCA of Lowell	\N	97 Central St	Unit #302	Lowell	MA	01852	978-454-5405	\N	\N	978-454-5043	www.ywcaoflowell.org	public_agency_email@testing123.net	The mission of the YWCA of Lowell is: (1) to respect, honor and celebrate diversity; (2) to initiate and foster local partnerships to maximize community resources; and (3) to advocate and provide services that promote physical and emotional wellness for women and families.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Katherine A. Kobos	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102486	2008-09-01	f	@lunch4all	P.O.V. Stage Company	\N	15 Kenoza Avenue	\N	Haverhill	MA	01832	978-518-1432	\N	\N	\N	\N	public_agency_email@testing123.net	P.O.V. Stage Companys mission is to\r\nproduce professional quality, thought \r\nprovoking theatre for a diverse audience;\r\nadvocate for theatre and the health \r\nof the arts community; and\r\nprovide theatre arts education\r\nfor teenagers and adults.	Various. Check website - povstageco.com.	\N	P.O.V. Stage Company	100 Macy St, Suite F Box 222	Amesbury	NH	01913	Lisa Zaleski	Director	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102882	2008-09-01	f	@lunch4all	Robert F. Kennedy Children's Action Corps	\N	10 Favor Street	\N	Lowell	MA	01852	978-458-5777	\N	\N	978-458-7328	www.rfkchildren.org	public_agency_email@testing123.net	The mission of The Mentoring Program, an agency-based program, is to offer system involved youth the chance to develop a trusting relationship with an adult mentor who can become their friend, role model and advocate.\r\n\r\nThrough specific, mentor-guided activities we hope to assist youth in building many skills such as confidence, self esteem, and leadership. Youth will also develop or refine skill, career and personal goals. The mutual experiences they impart, with the support of program staff will ultimately instill pro-social behavior; reduce delinquency, recidivism and other destructive and self destructive behaviors.\r\n\r\nThis new initiative hopes to expand the scope of that care and galvanize our belief that responsibility to our society does not end after treatment, but is our lifelong mandate to support the poor and disadvantaged. I would like to offer you the opportunity to find out more about our program and discover that we have the same commitment to our communities that you do.\r\n\r\nInspire action. Become a Mentor.	Mon thru Fri - 9am until 6pm	\N	10 Favor Street	\N	Lowell	MA	01852	Mark Speller	Program Coordinator	Robert F. Kennedy Children's Action Corps	No	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102167	2008-09-02	f	@lunch4all	Home Health VNA	Home Health VNA	360 Merrimack Street	\N	Lawrence	MA	01843	978-552-4000	800-933-5593	\N	978-552-4444	www.homehealthvna.org	public_agency_email@testing123.net	Home Health Foundation is committed to creating and delivering innovative and \r\nresponsive community health programs which provide effective \r\nand compassionate care.	24 hours a day, 7 days a week, 365 days a year	\N	\N	\N	\N	\N	\N	Joan Stygles Hull, RN, MBA	President / CEO	Home Health Foundation	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100011	2008-09-02	f	@lunch4all	Big Brothers Big Sisters of Greater Lowell, Inc.	\N	45 Merrimack Street, Suite 227	\N	Lowell	MA	01852	978-453-9339	\N	\N	978-452-0838	www.bbbslowell.org	public_agency_email@testing123.net	The mission of Big Brother Big Sister is to provide quality volunteer and professional services for children and youth and to assist them in achieving their highest potential as they grow to become responsible men and women.	Monday - Thursday 8:00AM - 6:00PM\r\nFriday   9:00AM-5:00PM	\N	\N	\N	\N	\N	\N	Dave Blanchard	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100770	2008-09-02	f	@lunch4all	Families First Parenting Programs	\N	99 Bishop Richard Allen Drive	\N	Cambridge	MA	02139	617-868-7687	\N	\N	617-354-2902	www.families-first.org	public_agency_email@testing123.net	Families First Parenting Programs builds positive relationships between parents and their children by educating both families and the professionals who serve them through inspiring, hands-on workshops. Our skilled workshop leaders stand beside parents and caregivers to lead them through the crucible of child rearing by transforming their experiences, doubts and fears into the knowledge and confidence they need to build positive discipline, communication, and self-esteem in families. Our training for professionals enhances their ability to better support and engage families. We help communities thrive by strengthening families first.	Office hours: Monday-Friday 9AM-5:30PM. Workshop hours vary	\N	\N	\N	\N	\N	\N	Crista Martinez Padua	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101540	2008-09-04	f	@lunch4all	Saints Medical Center   Growth & Nutrition Clinic	4th Floor	One Hospital Drive	\N	Lowell	MA	01852	978-934-8458	\N	\N	978-934-8358	saintsmedicalcenter.com	public_agency_email@testing123.net	We provide family-centered evaluation and treatment for children with growth delay.\r\nOur goal is to offer family focused help for infants and children who are not gaining and growing properly.  Some children are unable or unwilling to eat enough to gain and grow properly and don't gain weight over time as expected compared to children their own age.  From birth to age 2 is the most important period for brain growth.  Under nutrition during this time makes it difficult for a child to met his developmental milestones.  Under nutrition especially in very young children is associated with many behavior and developmental delays.  We work with families of children who are not gaining and growing, who may have medical issues such as reflux or other GI issues, gagging when solid foods are introduced, or don't transition to table foods, have frequent ear infections, get sick a lot, who have very small appetites, who are difficult to feed, may be anemic, were born premature or small and are having trouble gaining weight.  We work with families who have "picky eaters" and feel a lot of stress trying to feed their infant or toddler. Our goal is to help parents and children enjoy the feeding relationship, have children gain and grow properly, prevent or treat under nutrition, help families obtain the resources, supplements, and help they need.	Mondays 8:30am-5:30pm  A referral from the child's Primary Care Physician is required.	\N	\N	\N	\N	\N	\N	Margaret Reppenhagen,MA,RD,LDN	Director, Growth & Nutrition	Saints Medical Center	No	\N	Yes	Hospital conference rooms are available	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100253	2008-09-04	f	@lunch4all	Lowell Association for the Blind	LAB	169 Merrimack Street	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_agency_email@testing123.net	The mission of the Lowell Association for the Blind is to: support, educate and nurture the blind and visually impaired of the Greater Merrimack Valley by assisting them in the enrichment of their lives and by helping them to gain independence and to stimulate the interest and education of the general public to the problems and issues facing the blind and visually impaired as productive members of society seeking fulfillment of their human potential.	Monday - Friday  8:00am - 4:00pm	\N	\N	\N	\N	\N	\N	Elizabeth M. Cannon	Executive Director	\N	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102838	2008-09-09	f	@lunch4all	Massachusetts Alliance on Teen Pregnancy	\N	105 Chauncy Street, 8th floor	\N	Boston	MA	02111	617-482-9122	\N	800-645-3750	617-482-9129	http://www.massteenpregnancy.org	public_agency_email@testing123.net	The Massachusetts Alliance on Teen Pregnancy provides statewide leadership to prevent adolescent pregnancy and to promote quality services for pregnant and parenting teens and their children through policy analysis, education, research and advocacy.	9-5 Monday-Friday	\N	\N	\N	\N	\N	\N	Patricia Quinn	Executive Director	\N	Yes	Resource library in our office consisting of research materials, journals, and reports pertaining to teen pregnancy prevention and public policy related to pregnant and parenting teens.	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102926	2008-09-09	f	@lunch4all	St. Patrick Parish - Irish Cultural Committee	Irish Cultural Week	282 Suffolk Street	\N	Lowell	MA	01854	978-459-0561	\N	\N	\N	\N	public_agency_email@testing123.net	The Irish Cultural Committee was established in 1983 to raise the consciousness-level of people of the Greater Lowell area to the contribution made by the Irish Immigrants to our City, and especially to the community of St. Patrick Parish. The rich heritage of art, music and literature, and the spirituality of these first immigrants to the Acre of Lowell is the very foundation on which this first Roman Catholic Church in the city was built.	Meetings are the 2nd Tuesday of the Month, 7:00pm in the Church Hall. No meetings in the summer.	\N	\N	\N	\N	\N	\N	2009 - Bridget Reilly	Chairperson	\N	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100715	2008-09-16	f	@lunch4all	Lowell Telecommunications Corporation	LTC	246 Market Street	\N	Lowell	MA	01852	978-458-5400	\N	\N	978-937-0361	http://www.ltc.org	public_agency_email@testing123.net	Lowell Telecommunications Corporation is a community media and technology center serving the diverse urban population of Lowell, MA. The organization is committed to first-come, first-served telecommunication services to anyone in the community interested in expressing themselves. We also provide training and access to the media communication skills of the 21st century.\r\n\r\nThe organization's vision has always been about telecommunication technology - from video cameras to computers, cable channels to the Internet - and its uses in community information and content production.	Monday - Thursday \r\n10am to 9:30pm\r\n\r\nFriday\r\n10am to 6pm\r\n\r\nSaturday\r\n10am to 4pm\r\n\r\nClosed Saturdays preceding a Monday Holiday.	\N	246 Market St	\N	Lowell	MA	01853	Ellie Pye	Executive Director	\N	No	\N	Yes	10-12 person conference room\r\n50-75 person studio space	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102970	2008-09-24	f	@lunch4all	Girl Scouts of Eastern Massachusetts	\N	1740 Turnpike Street	\N	North Andover	MA	01845	978-689-8015	800-842-5566	\N	978-688-1846	www.girlscoutseasternmass.org	public_agency_email@testing123.net	Girl Scouting Builds Girls of Courage, Confidence, and Character, who make the world a better place.	Mon-Fri 8:30-5pm	\N	\N	\N	\N	\N	\N	Ruth N. Bramson	CEO	\N	Yes	Girl Scout leadership and personal development resources	Yes	conference room which accomodates 20-25 people; off site meeting space at our camp facilities in Andover, Rowley, Beverly, Pelham, and throughut Eastern Massachsetts	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102948	2008-09-17	f	@lunch4all	The Psychological Center	TPC	11 Union Street	\N	Lawrence	MA	01843	978-685-1337	\N	978-685-1337	978-681-1281	tpc1.org	public_agency_email@testing123.net	Our Mission is to help individuals, children, and families learn positive coping responses to the stressors of life that they experience in the home, school, workplace, and society.\r\n\r\n \r\n\r\nThe delivery of behavioral health services  that includes the disciplines of psychology, social work, psychiatry, counseling, and nursing is an important and integral component of the overall health and well-being of all people. Behavioral health care plays an essential and broadening role in our society, economy, education and government.\r\n\r\n \r\n\r\nWe are committed to creating and sustaining healthy and secure communities, achieved through a system that holds the needs of consumers paramount, regardless of their ability to pay. Vital to this commitment is a continuum of care that promotes services of the highest quality and value.\r\n\r\n \r\n\r\nHistorically, our values and practices provide a strong foundation for the delivery of accessible community-based programs for consumers from all walks of life.\r\n\r\n \r\n\r\nOur services will be based on the needs and strengths of the consumer who will be treated with dignity and respect.\r\n\r\n \r\n\r\nWe will provide responsive, accessible, and culturally competent services that includes the involvement of consumers, their families, and their social support network.\r\n\r\n \r\n\r\nServices will reflect a multi-modal approach that reflects the need of our consumers throughout their loves.\r\n\r\n \r\n\r\nWe are committed to effective, evidence-based interventions and measured outcomes. We believe that in today's healthcare environment, competence-based management and clinical practice are essential. We are committed to an environment that fosters learning and sharing within our agency and with the greater community.\r\n\r\n \r\n\r\nWe remain champions of effective, innovative, science-based interventions that will foster the health and well-being of our consumers and our communities.	Monday-Thursday 8:30-9\r\nFriday 8:30-7\r\nSaturday 9-1	The Psychological Center (TPC) Programs include DMH and DPH Residential Housing.  TPC offers Outpatient Counseling, a Recovery Center, Psychopharmacology, Structured Outpatient Addictions Program (SOAP),Daybreak Shelter, Driver Alcohol Education (DAE), and hosts community substance abuse support groups.	\N	\N	\N	\N	\N	Joe Cotton	Chief Executive Officer	\N	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy,Motivational Interviewing, Continigency Management,Substance Abuse, Harm Reduction, Mental Illness, Dual Diagnosis, Spirituality in Recovery, Healthy Relationships,	Yes	Very Large Community Meeting Space,Several Large Group Rooms	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103058	2008-09-24	f	@lunch4all	Merrimack Valley YMCA	Andover/North Andover YMCA, Lawrence YMCA, Methuen YMCA	101 Amesbury Street	4th Floor	Lawrence	MA	01840	978-725-6681	\N	\N	978-681-1126	www.mvymca.org	public_agency_email@testing123.net	The Merrimack Valley YMCA, a community-based organization government by its members, is dedicated to putting Christian principles into practice through programs, membership opportunities, and community services that strengthen spirit, mind, and body for all.	8:00 a.m. to 5:00 p.m. - Monday through Friday. (Branches open at 5:30 a.m. to 10:00 p.m.; weekends 6:00 a.m. to 9:00 p.m.	\N	\N	\N	\N	\N	\N	Stephen C. Ives	President/CEO	\N	Not Applicable	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102508	2008-09-29	f	@lunch4all	Lazarus House Ministries	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/	public_agency_email@testing123.net	Lazarus House, Inc. is a spiritually based organization that welcomes all in the name of God. It provides a continuum of care encompassing, but not limited to food, shelter, clothing, advocacy, job training, medical and dental care, a listening ear, a welcoming heart, the restoration of dignity and self respect to the poor and needy.	Mon-Fri: 8:30AM-4:30PM	\N	Lazarus House Ministries	P.O. Box 408	Lawrence	MA	01842	H. Bridget Shaheen	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103091	2008-10-01	f	@lunch4all	MSPCC Healthy Families	\N	439 South Union	\N	Lawrence	MA	01840	978-682-9222	1-800-541-3004	\N	978-681-9534	www.mspcc.org	public_agency_email@testing123.net	Healthy Families Massachusetts is a statewide home visiting program administered by the Massachusetts Children's Trust Fund.  Healthy Families offers first-time parents  ages 20 and under the information and individual support they need to raise healthy, happy and safe kids.  MSPCC is the lead agency providing Healthy Families in five areas: Springfield, Holyoke, Worcester, Lawrence, and Lowell.  \r\n\r\nFor families in need of support, we offer comprehensive program of services delivered where they have been shown to have the greatest impact - in the home. Services are presented as an integrated package designed specifically to meet the needs of each family.	Monday - Friday, 8:45- 5:00	\N	\N	\N	\N	\N	\N	Kristin Gohl Green	Program Coordinator	MSPCC	Not Applicable	\N	Yes	Conference room 20-30 person capacity.  Must be scheduled in advance.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100088	2008-10-01	f	@lunch4all	United Teen Equality Center	UTEC	34 Hurd St	\N	Lowell	MA	01852	978-441-9949	\N	978-265-7173	978-654-6727	www.utec-lowell.org	public_agency_email@testing123.net	UTECs mission is to be a youth-led safe-haven for youth development and grassroots organizing. \r\n\r\nOverall, our core values focus on peace, positivity, and empowerment. Through intensive street outreach, UTEC serves over 1,500 youth annually and over 150 teens every day (ages 13-23), particularly those most often overlooked and labeled as at-risk. UTEC offers programming (all at no cost) in the following 4 main centers:  Streetwork (outreach and gang peacemaking), Youth Development (cultural and performing arts), Education and Work Skills Training (new alternative diploma school program), and Youth Organizing (grassroots community organizing). \r\n\r\nUTEC was recently recognized as Citizens Bank and New England Cable Networks (NECN) Community Champion in Action and honored by the Robert Wood Johnson Foundation as a model for violence prevention. \r\n\r\nSee www.boston.com/news/necn/Shows/champs/ \r\nand www.lifp.org/html/project/focus_UTEC.html.	Monday-Thursday, 2pm until 8pm	\N	\N	\N	\N	\N	\N	Gregg Croteau	Executive Director	\N	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a large room with conference table.	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103080	2008-10-01	f	@lunch4all	Community Gardens Greenhouse	Beautification sub-committee of Keep Lowell Beautiful	220 Aiken Street	\N	Lowell	MA	01853	978-761-5902	\N	\N	\N	www.communitygardensgreenhouse.org	public_agency_email@testing123.net	The Community Gardens Greenhouse is a not-for profit initiative, the beautification sub-committee of Keep Lowell Beautiful, dedicated to creating social change through the art of gardening, building communities from the ground up. We focus on educational gardening workshops that encourage creativity, collaboration, and innovation while fostering relationships that transcend ethnicity, character, class and age.	Greenhouse is currently open 9AM-1PM on Saturdays.	\N	Community Gardens Greenhouse	220 Aiken Street	Lowell	MA	01853	Deb Harding	Greenhouse Operations Manager	Keep Lowell Beautiful	Not Applicable	\N	Yes	Greenhouse grounds in season	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100990	2008-10-07	f	@lunch4all	Billerica (Town of)	\N	Town Hall	365 Boston Road	Billerica	MA	01821	978-671-0942	\N	\N	978-671-0947	www.town.billerica.ma.us	public_agency_email@testing123.net	The town of Billerica provides a variety of services to residents. For more information, click on one of the programs below.	8:30am to 4:00pm	\N	\N	\N	\N	\N	\N	Rocco Longo	Town Manager	\N	No	\N	Yes	Space is used for municipal functions	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100891	2008-10-07	f	@lunch4all	Florence Crittenton League Adoption Agency, Inc.	\N	119 Hall Street	\N	Lowell	MA	01854	978-452-9671	\N	\N	978-970-0070	www.fcleague.org	public_agency_email@testing123.net	The Florence Crittenton League Adoption Agency is a non-profit, non-sectarian adoption agency. We have just been reaccredited for our Russian program.  We also work with China and make referrals to Guatemala and Ethiopia through out of state agencies. Occasionally we place a local child as well. We have information meetings or can meet with you to explain our program, for free.  Check our web site.	Monday-Friday 9AM-4PM	\N	\N	\N	\N	\N	\N	Ilze Keegan	Executive Director	\N	Yes	\N	Yes	Meeting space may be available, contact agency for more details	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101144	2008-10-07	f	@lunch4all	Youth Enhancement Services	YES	1360 Main St.	\N	Tewksbury	MA	01879	978-863-0099	\N	\N	978-851-5192	www.yes4kids.com	public_agency_email@testing123.net	YES' mission is to:\r\n\r\nEmpower children with emotional, behavioral and developmental disabilities to find, use and express their unique strengths and talents. \r\n\r\nProvide each child with the skills and opportunities they need to become successful and productive members of society, and encourage them to have some fun along the way.\r\n\r\nOffer support and guidance to families who face the financial and emotional challenges of raising a disabled child.	Weekday and weekend programs; summer day camp	\N	\N	\N	\N	\N	\N	Kevin Kelley	Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101716	2008-10-07	f	@lunch4all	St.Patrick School and Educational Center	\N	311 Adams Street	\N	Lowell	MA	01854	978-458-4232	\N	\N	978-458-4233	www.saintpatricklowell.org	public_agency_email@testing123.net	MISSION AND PURPOSE\r\nInspired by the mission of the Sisters of Notre Dame de Namur, St.Patrick School and Educational Center makes known the Goodness of God by providing for the educational needs of children, youth and adults within a caring, culturally diverse environment based upon Gospel values.\r\n\r\nSt.Patrick School serves children in grades K-1 through Grade 8. The Educational Center provides Adult classes in English, Citizenship preparation and Basic Computer.	School Hours: 8:20 AM - 2:50 PM, Monday - Friday. Before and After School care is also provided: 7:00AM -8:20 and 2:50 - 6:00 PM\r\n\r\nAdult English Classes are offered Tuesday and Thursday mornings (8:30 - 10:00 AM), afternoons (3:30 - 5:00 PM) and evenings \r\n(6:00-7:30 PM). Adult Computer classes are offered Tuesday evenings (6:00-7:30 PM and Wednesday afternoons ( 3:30 - 5:00 )Citizenship Classes begin October 1. Please call for information.	\N	\N	\N	\N	\N	\N	Sr.Joanne Sullivan,SND	Principal	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101474	2008-10-07	f	@lunch4all	Newbury Council on Aging	\N	63 Hanover Street	\N	Newbury	MA	01951	978-462-8114	\N	\N	978-499-0675	\N	public_agency_email@testing123.net	The Newbury Council on Aging will identify, develop, implement, and advocate for programs and services designed to enhance the quality of life and independence of elders in the community and will assist all people with questions relating to aging issues.	Monday thru Friday 8:00a.m. - 3:00 p.m.	\N	\N	\N	\N	\N	\N	Marty Joe	Director	\N	Yes	Extensive user friendly cross referenced file of materials relavent to elders; staff available to assist and make copies.	Yes	Newbury Elementary School -500 square feet	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102475	2008-10-07	f	@lunch4all	Bread and Roses	\N	58 Newbury Street	\N	Lawrence	MA	01840	978-681-8768	\N	\N	\N	www.breadandroseslawrence.org	public_agency_email@testing123.net	Bread & Roses is a soup kitchen in Lawrence, MA with a strong and rich tradition of being a center of hope and caring to the hundreds of people in the Merrimack Valley who are struggling for survival.  Founded in 1980, Bread & Roses' primary purpose is to ease the hunger of people in our community  by providing a well-balanced, nutritious evening meal.	Monday thru Thrusday, 5 pm - 7:30 pm, Saturday, 5 pm - 7:30 pm.	\N	\N	\N	\N	\N	\N	Bob Lanzoni	Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101694	2008-08-01	f	@lunch4all	J.J. Referral Services	\N	506 Katahdin Drive	\N	Lexington	MA	02421	781-572-0219	\N	\N	\N	www.jjreferralservices.com	public_agency_email@testing123.net	I am starting a referral service for the homeless, low-income, disabled, and anyone else who needs help navigating through the human services system. Agencies are welcome to respond. I am a statewide vendor. I specialize in the homeless. New pamphlets are available and you can contact me at the above number.	contact by phone by social worker from 9-5; access to e-mails anytime. Clients must sign up on web site giving full details about the help needed.	\N	\N	\N	\N	\N	\N	Joanne Stephen	owner/referral consultant	\N	Not Applicable	\N	Yes	starting in 2008 I will be available to take appointments at Baldwin Park. Call for details	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102992	2008-02-29	f	@lunch4all	Health & Education Services	HES	30 General Street	\N	Lawrence	MA	01841	978-683-3128	\N	978-620-1250	978-682-7296	www.hes-inc.org	public_agency_email@testing123.net	Largest behavioral health organization in the state of MA.  Full spectrum of comprehensive services from inpatient to outpatient and everything in-between.  We have over 100 programs and 32 sites serving individuals of all ages and treatment disorders.  \r\n     The Lawrence Clinic offers Outpatient Services, Adult Day Treatment Program, Emergency Services, Therapeutic After School Program, Family Stabilization Team, Community Rehabilitation Services and HIV program.	Week days for most programs, 24 hour 7 days a week for Emergency Services.	\N	\N	\N	\N	\N	\N	Maggie Padro,LMHC	Site Director	North East Health Systems	No	\N	Yes	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	4	
102024	2008-10-16	f	@lunch4all	Central Food Ministry, Inc.	\N	370 West Sixth Street	\N	Lowell	MA	01850	978-454-7445	\N	\N	978-452-4043	CFMinistry.org	public_agency_email@testing123.net	Provide food and clothing to those in need; treating them with dignity and respect through our Christ-centered volunteers.	Tuesday, Wednesday, Thursday: 9:30 a.m. to 1:00 p.m\r\nThursday evening: 7:00 p.m. to 8:00 p.m.	\N	\N	\N	\N	\N	\N	Daniel Carey	Board President	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103124	2008-10-17	f	@lunch4all	The Pettengill House, Inc.	Community Social Service Agency	PO Box 5551	13 Lafayette Road	Salisbury	MA	01952	978-463-8801	\N	\N	978-463-8743	www.pettengillhouse.org	public_agency_email@testing123.net	Formed in 1994, the mission statement of The Pettengill House, Inc. is to support, empower and coordinate community and state services, provide basic needs, emergency assistance, prevention education, case management and supports to those in need.	Monday-Thursday: 9:00am - 5:00pm\r\nFriday: By appointment only	\N	\N	\N	\N	\N	\N	Deborah M. Smith	Executive Director	\N	No	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102915	2008-10-22	f	@lunch4all	Rebuilding Together Greater Haverhill, Inc.	\N	PO Box 5161	\N	Haverhill	MA	01835	978-469-0800	\N	\N	978-556-9376	\N	public_agency_email@testing123.net	Rebuilding Together Greater Haverhill (RTGH) is a non-profit organization working in partnership with the community, volunteers and skilled labor, to stabilize and revitalize homes in the city and surrounding communities at no cost to the recipient. The RTGH initiative brings homeowners in need, together with volunteers from all walks of life, to preserve home ownership and reduce the risk of homelessness by improving the quality of the lives of the recipients.	Weekdays by appointment	\N	\N	\N	\N	\N	\N	Lynn Murphy	Program Manager	Rebuilding Together, Inc.	No	\N	Yes	Pentucket Bank Community Room\r\n35 Merrimack Street\r\nHaverhill, MA 01830	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102420	2008-10-29	f	@lunch4all	Youth Empowerment Services Inc.	YES Inc.	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	public_agency_email@testing123.net	The mission of Youth Empowerment Services Inc., YES, is to create opportunities for youth of all backgrounds and adults, who work with youth, to learn peaceable, healthy relationship skills to be able to communicate effectively; resolve conflicts nonviolently; solve problems creatively and cooperatively; and collaborate in leadership.	Mon-Fri: 8:AM-5:PM	\N	19 Acadia	Kimball Rd.	Amesbury	MA	01913	Barbara Hildt	Executive Director	\N	Not Applicable	\N	Yes	Amesbury Friends Peace Center\r\nspace for trainings and meetings	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103069	2008-10-29	f	@lunch4all	Windrush Farm Therapeutic Equitation, Inc.	WFTE, Inc. / Windrush Farm	30 Brookview Road	\N	Boxford	MA	01921	978-682-7855	\N	\N	\N	www.windrushfarm.org	public_agency_email@testing123.net	Our mission is to use the horse to expand the personal, emotional, and physical boundaries of all who ride and work with us.	8-6	\N	\N	\N	\N	\N	\N	Amanda Carey Hogan	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100737	2008-10-31	f	@lunch4all	New England Quilt Museum	\N	18 Shattuck Street	\N	Lowell	MA	01852	978-452-4207	\N	\N	978-452-5405	www.nequiltmuseum.org	public_agency_email@testing123.net	The New England Quilt Museum is the only museum in New England dedicated solely to the preservation, interpretation, and education of American quiltmaking past and present. Through its exhibitions, workshops, programs, and library, the museum is a resource for quilters, collectors, and all those who love quilts. We are located in beautiful, historic, downtown Lowell, Massachusetts.	Monday Closed, Tuesday-Saturday 10AM-4PM, Sunday May through December: 12PM-4PM	\N	\N	\N	\N	\N	\N	Connie Colom	Executive Director	\N	Yes	Resource library available on quilt history and quilt making, contact museum for more information	Yes	Multi-purpose available for meetings, catered luncheons, workshops, and evening events. Contact museum for details.	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102937	2008-03-24	f	@lunch4all	Christ Jubilee International Ministries	Formerly International Christian Fellowship Ministries	101 Smith St	P. O. BOX 1704	Lowell	MA	01851	978-452-9977	\N	978-490-4369	978-452-9976	http://christjubilee.com/	public_agency_email@testing123.net	/	Fridays 1-5pm	\N	101 Smith St	P. O. BOX 1704	Lowell	MA	01851	Jeremiah Menyongai	Senior Pastor	Christ Jubilee International Ministries	Not Applicable	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	4	
103135	2008-11-03	f	@lunch4all	Chelmsford Stamp Club	\N	P.O. Box  2342	\N	Littleton	MA	01460	978-256-2256	\N	\N	\N	\N	public_agency_email@testing123.net	The Chelmsford Stamp Club exists to promote and encourage the study of philately and to promote a spirit of good fellowship among its members. Meetings are held on the 1st & 3rd Monday of every month except July, August & December at the Central Baptist Church at 9 Academy St. Chelmsford, MA	1st & 3rd Monday of every month (except July, August & December). \r\n7:30pm - 9:00PM	\N	27 Brentwood Rd.	\N	Chelmsford	MA	01824	Linda Gilmore	President	\N	Yes	Complete set of Scott's Catalogue	Yes	Large, well lighted room, with bathroom facilities.	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101518	2008-10-21	f	@lunch4all	Renaissance Club	\N	176 Walker Street, 2nd Floor	\N	Lowell	MA	01854	978-454-7944	\N	\N	978-937-7867	\N	public_agency_email@testing123.net	Renaissance Club aims to provide a supportive community for individual recovery from mental illness. Our community is a safe place where members can form lasting relationships with others and build on those relationships to become more vocationally and socially active in the clubhouse and in the world. Renaissance Club guarantees all members: a place to belong; a place to return; the opportunity to develop meaningful relationships, and the opportunity for meaningful work.	Monday to Thursday  7:30 a.m.- 5:00 p.m.\r\nFriday              7:30 a.m.- 6:00 p.m.\r\nSaturday & Sunday  12:00 p.m.- 4:00 p.m.\r\n\r\nYouth Group\r\nThursday              3:00 p.m.- 7:00 p.m.	\N	\N	\N	\N	\N	\N	Elaine Walker	Director	Eliot Community Human Services,Inc	Yes	Self-help and addiction books.	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100473	2008-09-03	f	@lunch4all	Bridgewell Counseling Services	\N	35 Market Street	Suite 200	Lowell	MA	01852	978-459-0389	\N	\N	\N	\N	public_agency_email@testing123.net	The mission of Bridgewell is to serve and support adults with serious mental illness and mental retardation so that their lives can be as self-directed and satisfying as possible. To this end we offer complete Outpatient Mental Health and Day Treatment Services, Residential and Psychiatric Day Rehabilitation Services through the Massachusetts Department of Mental Health, Residential services through the Department of Mental Retardation and complete Homeless Services and referral for mentally ill adults through Pathfinder Safe Haven Shelter and Outreach.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Gary Gastman	Director	Bridegwell	No	\N	Yes	10 seat conference room	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103465	2008-11-12	f	@lunch4all	Merrimack Valley Health Enrollment Network	MVHEN	9 Central Estreet, Suit 400	\N	Lowell	MA	01852	978-394-2026	866-362-6006	\N	\N	http://mvhealthenrollment.net/	public_agency_email@testing123.net	We envision communities of individuals and families who value preventive health care and have equal opportunity to quality health coverage. \r\n\r\nThe Network is made up of community organizations in the Merrimack Valley, working together to raise awareness about health insurance requirements and options among their constituencies, and increase enrollment in state-sponsored health insurance programs. The MVHEN provides clear, consistent health insurance information and enrollment assistance to uninsured citizens throughout the region, particularly minorities and low-income individuals and families.	Monday to Friday \r\n9:00 A.M-5:00 P.M	\N	\N	\N	\N	\N	\N	Carlos M. Marin Ph.D.	Network Coordinator/Research Associate	Latin American Health Institute	No	\N	Yes	Conference room for 20 people	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101881	2008-07-29	f	@lunch4all	Beacon Hospice, Inc.	\N	350 Main Street	\N	Haverhill	MA	01830	800-981-4805	\N	\N	\N	www.beaconhospice.com	public_agency_email@testing123.net	Beacon Hospice is a Medicare and Medicaid certified program providing superior end-of-life care services.  Founded in 1993 to better serve the needs of terminally ill patients, Beacon Hospice believes in "quality in living."  Driven by patients' choices and the manner that they wish to live out their lives, we strive to establish the highest standard in hospice care for you and your family.	Monday-Friday 8am-5pm\r\n24 hour on-call services	\N	\N	\N	\N	\N	\N	Julie Herget	Volunteer Coordinator	\N	Yes	\N	Yes	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100077	2008-12-01	f	@lunch4all	Center for Family, Work and Community at UMASS Lowell	\N	600 Suffolk Street	First Floor	Lowell	MA	01854	978-934-4677	\N	\N	978-934-3026	www.uml.edu/centers/cfwc	public_agency_email@testing123.net	The Center for Family, Work and Community is dedicated to providing exceptional organizational development, consultation and training. Staff is available to consult about creating partnerships with university faculty, staff and students. The Center can help enhance community and organizational capacity by bringing University skills, strategic planning and expertise to groups outside the University.	Monday-Friday 9:00AM-5:00PM	\N	\N	\N	\N	\N	\N	Dr. Linda Silka	Director	UMASS Lowell	No	\N	Yes	Limited space is available, call 978-934-4677 to make arrangements	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101529	2008-12-01	f	@lunch4all	Community Health Education Center	CHEC	144 Merrimack St	2nd Floor	Lowell	MA	01826	978-452-0003	978-452-0003	\N	978-452-6602	\N	public_agency_email@testing123.net	To establish and anchor outreach education as a valued practice within public health.	Monday-Friday 830am - 430pm	\N	\N	\N	\N	\N	\N	Daniel Moriarty	Regional Manager	Lowell Community Health Center and Mass DPH	Yes	Library with Health Topic information. Books, videos. CDC information sheets are also available.We are now adding dvd's to our library, please come in to see what we have available.	Yes	Large meeting space. Can fit 30 people comfortably.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103113	2008-04-04	f	@lunch4all	Health Quarters	\N	101 Amesbury	\N	Lawrence	MA	01840	978-681-5258	800-892-0234	\N	\N	www.healthq.org	public_agency_email@testing123.net	Our mission is to protect reproductive health, build healthy communities where every pregnancy is intended, and improve the health of all who seek our services.  We provide quality, sensitive, and affordable clinical care, education and advocacy.  We believe everyone should have access to confidential reproductive health care and the opportunity to make informed choices.  In all aspects of our work we encourage relationships that are respectful and that value sexuality as an integral part of life.	Monday 9:00-5:00 \r\nTuesday 9:00-5:00 \r\nWednesday 9:00-5:00 \r\nThursday 9:00-5:00 \r\nFriday Closed	\N	19 Broadway	\N	Beverly	MA	01970	Health Quarters	Admin	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	4	
102739	2008-12-01	f	@lunch4all	SAHELI	SAHELI BOSTON	P O BOX  52063	\N	Boston	MA	02205	1-866-472-4354	1-866-472-4354	1-866-472-4354	\N	www.saheliboston.org	public_agency_email@testing123.net	Saheli, Friendship for South Asian Women is a group dedicated to helping South Asian women in Boston and surrounding areas. . It provides friendship, support, guidance and resources in the areas of career and economic empowerment, physical and mental health, legal and immigration issues, support for families, and social and cultural volunteer opportunities.	24/7	\N	\N	\N	\N	\N	\N	Gouri Banerjee	Founder	IAGB	Not Applicable	\N	Yes	Burlington Public Library	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101232	2008-12-01	f	@lunch4all	Horizons for Homeless Children	formerly The Horizons Initiative	60 Island Street	\N	Lawrence	MA	01840	978-557-2182	\N	\N	978-557-2184	www.horizonsforhomelesschildren.org	public_agency_email@testing123.net	The mission of Horizons for Homeless Children is to improve the lives of homeless children and their families. We provide homeless children in Massachusetts with the nurturing, stimulation and opportunities for early education and play that all children need to learn and grow in healthy ways. \r\n\r\nTo improve the lives of the children we serve over the long term, we connect their parents with the tools they need to achieve social and economic self-sufficiency. \r\n\r\nWe provide leadership in advocating for homeless children and their families through leveraging and sharing our expertise with others and advocating with policy makers and the public.	Volunteer shifts mostly on weekday evenings with some daytime or weekend options.	\N	\N	\N	\N	\N	\N	Amanda Browne	Playspace Programs Northeast Region Director	\N	Not Applicable	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102145	2008-12-01	f	@lunch4all	Lowell Tobacco Control	\N	341 Pine St.	\N	Lowell	MA	01851	978-970-4010 Ext. 1073	\N	\N	978-970-4011	\N	public_agency_email@testing123.net	To prevent youth smoking in the city of Lowell and to enforce various smoke free workplace laws as well as educate the public on smoking issues.	Monday through Friday\r\n8:00 to 4:00	\N	\N	\N	\N	\N	\N	Chris Morin	Tobacco Control Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101452	2008-12-02	f	@lunch4all	Helpingpatients.org	\N	Not Applicable	\N	Not Applicable	MA	01007	800-762-4636	800-762-4636	\N	\N	Helpingpatients.org	public_agency_email@testing123.net	Helping Patients brings together Americas pharmaceutical companies, doctors, patient advocacy organizations and civic groups to help low-income, uninsured patients get free or nearly free brand-name medicines. Its mission is to increase awareness of and enrollment in existing patient assistance programs for those who may be eligible. Through this site, Helping Patients offers a single point of access to more than 275 public and private patient assistance programs, including more than 150 programs offered by pharmaceutical companies.	On Web 24/7	\N	\N	\N	\N	\N	\N	N/A	N/A	PhRMA	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100055	2008-12-02	f	@lunch4all	Lowell General Hospital	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_agency_email@testing123.net	Lowell General Hospital is committed to providing excellent, patient-focused care, a commitment that is summed up in its mission statement, "Patients First in Everything We Do."	24 Hours a day, 7 days a week.	\N	\N	\N	\N	\N	\N	Normand E. Deschene	President and Chief Executive Officer	\N	Yes	Please call 978-937-6000 for more information	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
101441	2008-12-02	f	@lunch4all	Department of Mental Health-Adult Case Management	DMH-Adult Case Management	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5000	\N	978-322-5120	978-322-5077	\N	public_agency_email@testing123.net	The Department of Mental Health's mission is to improve the quality of life for adults with serious and persistent mental illness and severe emotional disturbance.  This is accomplished by assuring access to an integrated network of effective and efficient services that promotes consumer rights, responsibilities, rehabilitation, and recovery.	Mon - Fri 9 am - 5 pm	\N	\N	\N	\N	\N	\N	John Gilmore	Director of Residential Services	\N	Not Applicable	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101925	2008-12-08	f	@lunch4all	Home Health Foundation	Home Health VNA	360 Merrimack Street, Building 9	\N	Lawrence	MA	01843	978-552-4525	800-933-5593	\N	\N	www.homehealthfoundation.org	public_agency_email@testing123.net	Home Health Foundation is committed to creating and delivering innovative and responsive community health programs which provide effective and compassionate care.\r\n\r\nVolunteers are an important part of our team and helping us administratively, serving as companions, contacting newly admitted patients by phone and assisting with fundraising events.	M-F between 8:30 and 5:00 and occasional weekend events.  Volunteer commitment varies and includes annual events, one time only projects and ongoing weekly assignments.  Generally volunteers contribute between 4-10 hours a month.	\N	\N	\N	\N	\N	\N	Kim Eubanks	Volunteer Coordinator	Home Health Fundation	Yes	The organization has a number of health care resources including print and electronic format.  We provide free Continueing Educaton Credit for our professional nursing staff and student volunteers.	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101507	2008-12-15	f	@lunch4all	Greater Lowell Workforce Investment Board	Greater Lowell WIB	107 Merrimack Street	\N	Lowell	MA	01852	978-937-9816	\N	\N	978-459-2111	www.glwib.org	public_agency_email@testing123.net	The Greater Lowell Workforce Investment Board is a private non profit organization consisting of private business representatives, educational leaders, labor groups, community based organizations, local and state agencies and economic workforce development groups.  The Board works in partnership with the CEO (Lowell City Manager) in providing policy guidance for workforce development issues in Gr. Lowell.  The Board oversees the charter of the Career Center of Lowell, provides technical assistance and information on grant writing initiatives, tax credits, provides labor market information/data reports, expands awareness and support of workforce development issues in Greater Lowell and partners with economic development agencies to assist in the creation, retention and expansion of business opportunities in the Greater Lowell Workforce Investment Area.(City of Lowell and the towns of Billerica, Chelmsford, Dracut, Dunstable, Tewksbury, Tyngsboro and Westford)	Monday-Friday, 9:00 a.m.-5:00 p.m.	\N	\N	\N	\N	\N	\N	Barbara O'Neil	Director	\N	No	\N	Yes	The Greater Lowell WIB can assist groups with meeting space needs by utilizing the Career Center of Lowell.  Please contact Barbara for information	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102761	2008-12-15	f	@lunch4all	Mass Rehab Commission	MRC-Homecare Assistance Program	Massachusetts Rehabilitation Commission	27 Wormwood Street, Suite 600	Boston	MA	02210	617-204-3853	1800-223-2559	\N	617-727-2809	www.mass.gov/mrc	public_agency_email@testing123.net	To promote equality, empowerment and productive independence of individuals with disabilities. The purpose of the MRC is to provide comprehensive services to people with disabilities to maximize their quality of life and economic social self-sufficiency in the community.\r\nThis is accomplished through multiple programs in the MRC: the vocational rehabilitation services program, the community services program, and the disability determination services program	Monday-Friday - 8:30-5pm	Program provides paid homemaking services only - ( laundry, light cleaning, food shopping, meal preparation and medication pickup) to persons with physical, psychiatric and cognitive disabilities 18-59 years throughout the State, to prevent further injury, decompensation or hospitalization. Eligibility is based on medical documentation of a person's need for homemaking services, as well as determination of age, living situation and income. Services are free to eligible consumers. Case managers provide information and referral to other independent living resources in the consumer's community. There is currently a 4-5 month waiting list upon approval. Approval can take up to 3 months due to need to gather documentation from various sources.	\N	\N	\N	\N	\N	Betty Maher	Director, Homecare Assistance Program	Massachusetts Rehabilitation Commission	Yes	Information on our program is available on the MRC website and can be mailed to interested parties.	Yes	Conference rooms on the 5th and 6th floor available for booking	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102706	2008-12-15	f	@lunch4all	Shriver Job Corps Center	Job Corps	270 Jackson Road	\N	Devens	MA	01434	978-784-2600	\N	\N	978-784-2710	http://bostonregion.jobcorps.gov	public_agency_email@testing123.net	Job Corps is a federally funded residential training program for youth ages 16 - 24. Shriver Job Corps is one of over 120 centers nationwide. (There are 2 other centers in Massachusetts.) Job Corps provides career skills training, as well as the opportunity to earn a GED or HSD and a driver's license. Students who have already obtained a GED or HSD are eligible for the program and are encouraged to apply. Students live at the Job Corps Center and attend classes daily to obtain a trade certificate in C.N.A., Auto, Culinary, Computer Technologies, Business Clerical, Carpentry, Painting, Cement and many more. Students receive a weekly stipend as well as room and board, wellness services, recreational activities and uniform allowance. Upon completion, students can earn up to $1200 in transitional assistance.	24 hours a day, 365 days per year. Classroom training takes place weekdays from 8:00 am - 3:30 pm.	Most staff have access via the Job Corps Portal.	\N	\N	\N	\N	\N	Tscherina Telesford	Center Director	Job Corps, The Department of Labor, Adams & Assoc., Inc.	Yes	Not open to the public, staff and students have access to career assessment and career placement resources.	Yes	Conference room, gymnasium, classrooms and cafeteria	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102717	2008-12-15	f	@lunch4all	Department of Children and Families -- Haverhill Office	DCF	3 Ferry Street	\N	Bradford	MA	01835	978-469-8800	\N	1-800-792-5200	978-469-8990	www.mass.gov/dss	public_agency_email@testing123.net	The Department of Social Service (DSS) is the Massachusetts state agency charged with the responsibility of protecting children from child abuse and neglect. DSS is committed to protecting children and strengthening families. When children are abused or neglected by the people responsible for caring for them, DSS will intervene to ensure the safety of the children. \r\nDSS responds to reports of abuse or neglect 24 hours a day. DSS becomes involved if there are any concerns that caretakers, parents, step-parents, guardians or other persons responsible for caring for children may be abusing or neglecting these children.\r\nWhenever possible and appropriate, DSS attempts to keep families intact. DSS reviews all the reports of child abuse and neglect received by the agency. If it is determined that abuse or neglect has occurred, or if a child appears to be at risk of being hurt, or is being neglected, DSS takes action to protect that child.	Monday  Friday 8:45  5:00 \r\nAfter Hours: Child-At-Risk Hotline: 1-800-792-5200	\N	\N	\N	\N	\N	\N	Nancy J. Fagan	Area Director	\N	Yes	Northeast Regional Resource Center\r\nhttp://www.marrcne.org \r\n\r\nAdoption Journeys of Massachusetts\r\n1-800-972-2734\r\nhttp://www.adoptionjourneysma.org  \r\n\r\nMassachusetts Adoption Resource Exchange, Inc. (MARE)1(800)54-ADOPT\r\nhttp://www.mareinc.org \r\n\r\nMassachusetts Society for the Prevention of the Cruelty of Children (MSPCC), Kids Net Program\r\n(617) 587-1500\r\nhttp://www.mspcc.org \r\n\r\nThe Center for Family Connections\r\n(617) 547-0909\r\nhttp://www.kinnect.org \r\n \r\nThe Center for Adoption Research\r\nhttp://www.centerforadoptionresearch.org  \r\n\r\nOpen Door Society\r\nhttp://www.odsacone.org \r\n\r\nAdoptUSKids\r\nwww.adoptuskids.org \r\n\r\nBaby Safe Haven\r\nhttp://www.babysafehaven.com	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
102662	2008-12-17	f	@lunch4all	Dept. of Mental Health -- Essex North	\N	15 Union St., Suite #2	\N	Lawrence	MA	01840	978-738-4500	\N	\N	978-738-4559	www.state.ma.us/dmh	public_agency_email@testing123.net	The Department of Mental Health, as the State Mental Health Authority, assures and provides access to services and supports to meet the mental health needs of individuals of all ages, enabling them to live, work and participate in their communities. The Department establishes standards to ensure effective and culturally competent care to promote recovery. The Department sets policy, promotes self-determination, protects human rights and supports mental health training and research. This critical mission is accomplished by working in partnership with other state agencies, individuals, families, providers and communities. \r\n\r\nThe Massachusetts Department of Mental Health (DMH) sets the standards for the operation of mental health facilities and community residential programs and provides clinical, rehabilitative and supportive services for adults with serious mental illness, and children and adolescents with serious mental illness or serious emotional disturbance.	9 am - 5 pm	\N	\N	\N	\N	\N	\N	Dorothy Smith	Site Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-738-4558	0	
103476	2008-12-20	f	@lunch4all	119 Gallery	\N	119 Chelmsford Street	\N	Lowell	MA	01851	978-452-8138	\N	\N	\N	www.119gallery.org	public_agency_email@testing123.net	119 Gallery promotes contemporary and new media art, innovative ideas and cutting-edge techniques with a rich and diverse program of exhibitions, performances and community-based arts services; and welcomes people of all ages, backgrounds and means to explore and experience new, innovative art.	Tuesday - Saturday 12noon - 5pm\r\nCheck our calendar for evening and weekend programming.	\N	\N	\N	\N	\N	\N	Walter Wright	Director	\N	Not Applicable	\N	Yes	Gallery is available for meetings.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100462	2009-01-06	f	@lunch4all	Massachusetts Rehabilitation Commission	Mass Rehab (MRC)	325 Chelmsford St.	\N	Lowell	MA	01851	978-458-4544	\N	\N	978-937-9879	www.state.ma.us/mrc/vr/towns.htm	public_agency_email@testing123.net	The mission of the Massachusetts Rehabilitation Commission VR division is to provide vocational rehabilitation services to eligible persons with disabilities geared toward increasing individual self-sufficiency through competitive integrated employment.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Carolyn L. Langevin	Lowell Area Director	Massachusetts Rehabilitation Commission - State Agency	No	\N	Yes	Limited space for up to 15 people limited to 9-5PM hours available by appointment only. Call 978-458-4544 x303	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103289	2008-12-23	t	@lunch4all	Methuen Arlington Neighborhood, Inc.	MAN, Inc.	141 Tenney Street	\N	Methuen	MA	01844	978-691-5645	\N	\N	978-691-5645	\N	public_agency_email@testing123.net	The Methuen Arlington Neighborhood is a densely populated, low to moderate income area located close to the City of Lawrence. Sharing the same problems as Lawrence - poverty, lack of employment opportunities, and social ills - this area is able to receive some financial assistance due to the fact that Methuen is a small cities mini-entitlement area. \r\n\r\nTo begin formally working on improving the quality of life in the area, neighborhood residents formed a Community Developement Corporation (CDC), the Methuen Arlington Neighborhood, Inc. (M.A.N., Inc.). In March of 1995, this organization became incorporated as M.A.N. Inc. The neighborhood is also the site of the Department of Justice's Weed and Seed Program.\r\n\r\nM.A.N. Inc's prime target area for the purpose of representation and service is within the Town of Methuen, primarily within the borders of the Spicket River, Horne Street, Lawrence Street, and the Lawrence City Line of Center Street.	ADD	\N	141 Tenney Street	PO Box 715	Methuen	MA	01844	Pamela Marchand	Program Director	Greater Lawrence Community Action Council, Inc (GLCAC, Inc.)	Yes	Library for kids in the after school homework center.	Yes	Occasionally. After an elevator to the second floor is installed, we will be better able to accommodate local groups.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103487	2008-12-29	f	@lunch4all	Youth Villages, Inc.	Youth Villages, Inc.	400 West Cummings Park	\N	Woburn	MA	01801	781-937-7913	\N	781-937-7924	781-937-7901	www.youthvillages.org	public_agency_email@testing123.net	Our Guiding Principles\r\n\r\nAll Youth Villages employees are committed to the mission and values that guide our organization. Adherence to these simple, specific principles ensures that we offer the very best, most effective help to every child in our care and provides the foundation for our organization's dynamic growth. \r\n\r\nOur Mission\r\nYouth Villages helps children and families live successfully. \r\n\r\nOur Values \r\nKids' needs come first...Always \r\nWe make every decision in the best interest of each child. We adapt our programs to accommodate the special needs of children and families. Often we make personal sacrifices in order to help children and families achieve their potential. \r\n\r\nChildren are raised best by their families. \r\nWhen at all possible, children belong with their families. We help families provide the support and structure that all children need. \r\n\r\nWe provide a safe place. \r\nWe provide care and treatment for children in an open, safe environment. We ensure that young people are physically and emotionally safe. \r\n\r\nWe strive to achieve positive, lasting results. \r\n\r\nWe help children and families develop skills to live successfully by focusing on areas that have a long-term impact on the family. \r\n\r\nWe are committed to our staff. \r\nWe recognize the many challenges our staff face each day. We value teamwork and help staff achieve their potential through an atmosphere of open communication, learning and fun. \r\n\r\nWe are each responsible for providing the highest level of service to our customers. \r\nWe deliver our best by listening and responding to our customers...every time, every day. \r\n\r\nWe constantly improve our performance to achieve excellence. \r\n\r\nWe measure our efforts by accessing our strengths and needs to identify areas for improvement. We believe that anything can be made better. \r\n\r\nWe create new programs to meet the needs of children, families and the community. \r\nWe develop innovative programs that serve children and families facing the most challenging circumstances. Our entrepreneurial spirit leads us to test the limits of existing services and create new opportunities. \r\n\r\nWe do what we say we do. \r\nOur mission and values are more than just something to talk about. They guide all of our decisions. We believe that our integrity can only be measured by how we live by these values each day.	Please call to schedule an appointment.	\N	400 West Cummings Park	\N	Woburn	MA	01801	Matt Stone	Massachusetts State Manager	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103322	2008-12-30	f	@lunch4all	Family Service, Inc.	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6600	\N	\N	978-327-6601	www.familyserviceinc.com	public_agency_email@testing123.net	Family Services' mission is to strengthen individuals and families by building skills for successful home, school, work, and community life, has led the agency to implement a range of prevention and treatment programs that are evidence-based and result in self-sufficiency and improved economic and health outcomes for its clients. Over 20 programs are currently offered in the areas of child welfare, parent education, youth development, employee assistance, mental health treatment and suicide prevention.	M-F 8:00am-5:00pm	\N	\N	\N	\N	\N	\N	Kay Berthold Frishman	Executive Director	\N	Yes	\N	Yes	Conference room seats 75 persons.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100484	2009-01-05	f	@lunch4all	Middlesex Community College	\N	33 Kearney Square	\N	Lowell	MA	01852	978-656-3105	800-818-3434	\N	978-656-3150	www.middlesex.mass.edu	public_agency_email@testing123.net	Middlesex Community College is a progressive and dynamic learning community, committed to providing educational programs and services that support personal growth and economic opportunity for its diverse student population.  Dedicated to student success, the College provides excellence in teaching, personal attention, and extensive opportunities for exploration and growth.  Closely linked to the fabric of the community, Middlesex's partnerships with school, business and service organizations provide leadership in economic and community development and foster a culture of civic engagement and responsive workforce development.  The College's state-of-the-art programs iin the liberal arts, basic skills, and more than fifty career and technical fields respond to student and community needs, providing a strong foundation for college transfer, employment, professional development and lifelong learning.	Monday-Thursday 7AM-10PM; Friday-Saturday 7AM-5PM	\N	\N	\N	\N	\N	\N	Dr. Carole A. Cowan	President	Board of Higher Education	Yes	\N	Yes	Meeting space is available when space not occupied. Contact Mandy Vozzo, 978-656-3106 for information	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103498	2009-01-14	f	@lunch4all	Balas, Alphen & Santos, PC	\N	200 Littleton Road	\N	Westford	MA	01886	978-692-3107	\N	\N	978-692-5454	www.lawbas.com	public_agency_email@testing123.net	We are a law firm in existence since 1996.  Attorney Maria L. Santos focuses her practice on employment based and family based immigration law and has done so since 1996.	By appointment anytime, but we are open 8:30 to 5:00.	\N	\N	\N	\N	\N	\N	Maria L. Santos	Managing Partner	\N	Not Applicable	\N	Yes	We have a conference room that will accomodate 12-15 people for a meeting	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103311	2008-06-16	t	@lunch4all	Lawrence Methuen Community Coalition	LMCC	112 A East Haverhill Street	\N	Lawrence	MA	01841	978-975-8793	\N	\N	978-975-3294	http://www.lmcconline.com/aboutus/index.htm	public_agency_email@testing123.net	The mission of the Lawrence/Methuen Community Coalition is to provide an accessible continuum of services that fosters and preserves the health, safety and well being of individuals and families, thus supporting communities in which they through following the principles of Family Support, the prevention of substance abuse, the prevention of child maltreatment and increasing awareness of the prevention of domestic and family violence.	M-F ADD	\N	\N	\N	\N	\N	\N	Harold Magoon	Director	Greater Lawrence Community Action Council Inc.	No	\N	Yes	Conference room seats 20 persons	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
100154	2008-06-17	f	@lunch4all	City of Lowell	City Hall	375 Merrimack Street	\N	Lowell	MA	01852	978-970-4000	\N	\N	\N	http://www.lowellma.gov	public_agency_email@testing123.net	Lowell offers a variety of services to residents. Please see programs for additional information.	City Hall open 8AM-5PM	\N	\N	\N	\N	\N	\N	Bernie Lynch	City Manager	\N	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
100044	2008-06-19	f	@lunch4all	Lowell Community Health Center	LCHC	585-597 Merrimack Street	\N	Lowell	MA	01854	978-937-9700	\N	\N	978-970-0359	www.lchealth.org	public_agency_email@testing123.net	Lowell Community Health Center provides caring, quality, and culturally appropriate health services to the people of greater Lowell, regardless of their financial status. We are committed to enhancing the health of our community and to empower each individual to maximize overall well being. Drawing from this mission our motto is: Linking Community to Health Care.	Monday-Friday 9AM-5PM; Tuesday & Thursday Evening Clinic; please call for office hours.	978-452-4774 TTY Number	585-597 Merrimack Street	\N	Lowell	MA	01854	Dorcas Grigg-Saito	Chief Executive Officer	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
101837	2008-06-19	f	@lunch4all	Massachusetts Children's Trust Fund	\N	55 Court Street	4th Floor	Boston	MA	02108	617-722-8957	888-775-4543	617-722-8957	617-727-8997	www.onetoughjob.org	public_agency_email@testing123.net	Being a parent is one tough job. Full of many rewards, beautiful moments, great sharing, laughs and love. But the reality of raising children today is that parents become overwhelmed, exhausted and stressed.\r\nWe get it. We're the Massachusetts Children's Trust Fund and we know how hectic the life of a parent is today. In communities throughout the state and in the pages of this website, we provide parents with the expert information, tips and support they need and deserve to be the best parent they can be.	Monday through Friday/8:45 a.m. to 5 p.m.	\N	\N	\N	\N	\N	\N	Suzin Bartley	Executive Director	Board of Directors of the MAChildren's Trust Fund	Yes	Over 300 curricula, technical assistance resources, videos, and other material available free to community-based groups to better meet the needs and interests of families.	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
100407	2008-06-19	f	@lunch4all	University of Massachusetts Lowell	UMass Lowell	One University Avenue	\N	Lowell	MA	01854	978-934-4000	\N	\N	\N	www.uml.edu	public_agency_email@testing123.net	The mission of the University of Massachusetts Lowell is to enhance the intellectual, personal and cultural development of its students through excellent, affordable educational programs. The University seeks to meet the needs of the Commonwealth today and into the future and supports the development of sustainable technologies and communities through its teaching, research, scholarship and engagement.\r\n Lowell offers its 12,000 undergraduate and graduate students more than 80 degree programs in the colleges of Arts and Sciences, Engineering, and Management; School of Health and Environment; and the Graduate School of Education.	Varies by program	\N	\N	\N	\N	\N	\N	Marty Meehan	Chancellor	University of Massachusetts System	Yes	A wide variety of subjects available at three locations. For information visit http://www.uml.edu/libraries	Yes	Meeting space is available for large and small groups. Contact University Special Events for more information at 978-934-3888 or events@uml.edu.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
102750	2008-06-19	f	@lunch4all	Massachusetts Rehabilitation Commission/Turning 22 Program	MRC	27 Wormwood Street	\N	Boston	MA	02210	617-204-3600	\N	\N	617-727-1354	www.mass.gov/mrc	public_agency_email@testing123.net	The MRC Turning 22 Program was established in 1986 to design and implement transition services for young people who have a severe physical disability with a mobility impairment, and who would need residential services after high school graduation or turning 22.  The program utilizes the independent living philosophy in which consumers are empowered to make their own decisions about their lives, based on informed choice.	Monday through Friday\r\n9-5	\N	\N	\N	\N	\N	\N	James Durant	Turning 22 Program Coordinator	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100165	2008-06-19	f	@lunch4all	Mass Society for the Prevention of Cruelty to Children	MSPCC, Massachusetts Society for the Prevention of Cruelty	439 South Union Street	\N	Lawrence	MA	01843	978-682-9222	\N	800-422-3035	978-681-9508	www.mspcc.org	public_agency_email@testing123.net	The Massachusetts Society for the Prevention of Cruelty to Children is a private, non-profit society dedicated to leadership in protecting and promoting the rights and well being of children and families. There are 2 locations serving the Merrimack Valley: - 439 South Union Street Lawrence, MA 01843v - 126 Phoenix Ave 3rd Floor, Lowell, MA 01852	Monday-Friday 9:00AM-5:00PM	\N	\N	\N	\N	\N	\N	Marylou Sudders	Executive Director	\N	No	\N	Yes	One conference room in Lawrence may be available. Sign up through receptionist	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
102618	2008-06-19	f	@lunch4all	Summers, Summers & Associates	Criminal Defense and Immigration Lawyers	321 Central Street	\N	Lowell	MA	01851	978-452-6655	978-452-6655	\N	\N	www.summersatlaw.com	public_agency_email@testing123.net	Summers, Summers & Associates, P.C. provides legal services and expertise in all areas of criminal law, immigration and nationality law.	Monday through Friday 8:00 AM to 6:00 PM.	visit our web site at www.summersatlaw.com	179 Great Road	\N	Acton	MA	01720	Philip Summers	Partner	\N	Not Applicable	\N	Yes	321 Central Street\r\nLowell, MA  01851	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
102079	2008-06-19	f	@lunch4all	YouthStorm of New England	\N	101 Main St	\N	Salem	NH	03089	978-416-2002	\N	\N	\N	www.youthstorm.org	public_agency_email@testing123.net	YouthStorm is an independent, faith based but non-denominational 501(c)(3) nonprofit organization. Our vision is to unite, equip, and mobilize the youth in New England in order to raise up the next generation to pursue holiness, to fulfill their purposes in the local church and the Kingdom and to ignite the flames of revival in their communities and their world.	M-F 10am-5pm	\N	YouthStorm	44 Range Road	Windham	NH	03087	Rebekah Schmitz	Office Manager	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
101606	2008-06-19	f	@lunch4all	Neighbors In Need	\N	95 East Haverhill Street	\N	Lawrence	MA	01841	978-685-8321	\N	\N	978-689-2006	NeighborsinNeed.net	public_agency_email@testing123.net	Neighbors In Need has fed the poor and hungry of Greater Lawrence for nearly 25 years.  We give food to those in need without judgment, respecting the dignity of each individual.  All food, formula, diapers, clothing and household items are provided free of charge to those who request them.\r\n\r\nNeighbors In Need was founded in 1983 to provide food to needy families in the Greater Lawrence Area under the sponsorship of Rudolf Canon Devik, Rector of Grace Church in Lawrence.  Two food pantries were opened the first year with more added as the organization grew.  Since its founding in 1983, Neighbors In Need has been a faith-based community organization relying heavily on area churches for support. \r\n\r\nToday, Neighbors In Need serves families in eight food pantries throughout Lawrence and Methuen. The Infant in Need Program provides baby formula and diapers for at-risk children. The Adopt-A-Family Program provides holiday meals and gifts.  Neighbors In Need distributes clothing and household items donated by individuals, schools and local corporations.	Office hours: M-F, 9am-12pm	Office space is rented at 95 East Haverhill Street in Lawrence.  The 450 square foot basement is used to store, sort and package diapers and formula for the Infants In Need program, and clothing for the Clothes Bin program.\r\n\r\nNeighbors In Need maintains a 1,200 square foot warehouse in Andover where we store, sort and repackage the food donations.  The warehouse has an 8 ft. x 8 ft. walk-in freezer and refrigerator.  Food is separated into major food groups and stored for future repackaging and delivery to the food pantries.  Eight institutions donate space for the weekly food pantries.\r\n\r\n\r\nCommunity Partners:\r\n\r\nSince its founding in 1983, Neighbors In Need has been a faith-based community organization relying heavily on area churches for support.  Neighbors In Need works closely with area churches, the Greater Lawrence Council of Churches, Pike School, Phillips Academy, Merrimack College, public schools in Lawrence, Andover, Methuen, North Andover, corporations, and civic organizations who have food drives to support our mission.  Many are financial contributors as well.    Donated food, clothing, diapers and baby goods are picked up by the Neighbors In Need van from these generous organizations.  These supporters donate 40% of food distributed annually by Neighbors In Need.	P.O. Box 447	95 East Haverhill Street	Lawrence	MA	01841	Linda Zimmerman	Executive DIrector	\N	Not Applicable	\N	Not Applicable	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
102013	2008-06-20	f	@lunch4all	Sangeeta Dey, Psy.D; Pediatric Neuropsychologist	\N	594 Marrett Road, Suite 22	\N	Lexington	MA	02421	781-652-9594	\N	\N	781-652-9557	\N	public_agency_email@testing123.net	Provide neurodevelopmental and neuropsychological evaluation for children ages 2 and above with developmental delays, attentional problems, and learning disorders due to various conditions, such as PDD, Genetic & Metabolic Disorders.	M-F\r\n8AM-6PM	\N	\N	\N	\N	\N	\N	Sangeeta Dey, Psy.D.	Pediatric Neuropsychologist	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
101969	2008-06-20	f	@lunch4all	Rehabilitation Options, PC	Robert Brownson, MS	564 Dutton Street	2nd Floor	Lowell	MA	01854	978-479-2923	\N	\N	978-746-9502	www.rehabilitation-options.com	public_agency_email@testing123.net	Mental Health and Psychotherapy Practice\r\n\r\nClinical Competency:\r\nInter-Ethnic, Adult and Adolescent Orientation\r\nHighly Individualized\r\nBroad Clinical Interest, including: Depression, Traumatic Experience and Loss, Periods of Life Change, Addictive Traits and Conduct Problems                               \r\n\r\nForensic Services:\r\nCourts and Probation/Parole programs: Anger Management, Domestic Violence Intervention, Offense Specific Treatment\r\nSupportive/Supervisory Counseling to  reduce Recidivism and Re-Incarceration\r\nPsychosocial Evaluations\r\nRisk Assessments for Anger Management\r\nSubstance Abuse Evaluations\r\nOrientation to the Legal Process\r\nTrial and Testimony Preparation\r\nMitigating Circumstances Review and Alternative Sentencing Recommendations: G.L. c. 123, s. 15 (e, f)\r\nParole Hearing Preparation\r\nAttorney/Client Consultations\r\n\r\nRehabilitation Services:\r\nJob Skills Inventory and Advising\r\nWorkplace Advocacy\r\nLife Balance Counseling\r\n\r\nSpecialty Consultations:\r\nMens Work Programs\r\nSteps to Reduce Work Place Violence\r\nConflict Resolution and Mediation\r\nGuardian ad Litum  Middlesex County\r\n\r\nMASS Health Provider, and other health insurances, as well	by appointment	\N	\N	\N	\N	\N	\N	Robert Brownson, MS	Lead Clinician	\N	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
102673	2008-06-20	f	@lunch4all	Greater Lawrence Educational Collaborative	GLEC	480 Broadway	\N	Methuen	MA	01844	978-685-3000	\N	\N	978-689-7466	www.glec.org	public_agency_email@testing123.net	The mission of the Greater Lawrence Educational Collaborative is to serve member school districts in meeting their unique needs in a changing global community through cooperative visionary leadership and coordination of resources and services.  Collaborative services maximize efficiency, quality and equity of educational opportunity.	Monday - Friday\r\n8:00 AM - 4:00 PM	\N	480 Broadway	\N	Methuen	MA	01844	Robert McArdle	Executive Director	Greater Lawrence Educational Collaborative	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
101023	2008-06-20	f	@lunch4all	Parents Helping Parents	\N	108 Water Street	\N	Watertown	MA	02472	617-926-5008	800-882-1250	\N	617-926-5011	www.parentshelpingparents.org	public_agency_email@testing123.net	PHPEmpowering parents to Nurture children and prevent child abuse.	Office hours: Monday-Friday 9AM-5PM\r\nSupport groups meet in the evenings, usually Mondays through Thursdays.	website	\N	\N	\N	\N	\N	Randy Block	Executive Director	Parents Helping Parents	No	we have a group in Lowell Mon 5:30-7:00\r\nSalem Tuesday 10:30-12:30, and Beverly Monday 6:30-8:00	Yes	small conference room	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100616	2008-06-20	f	@lunch4all	St. Ann's Home, Inc.	\N	100A Haverhill Street	\N	Methuen	MA	01844	978-682-5276	\N	\N	978-688-4932	www.st.annshome.org	public_agency_email@testing123.net	The mission of St. Ann's Home, Inc. is to support, nurture, rehabilitate, educate and advocate for children who are emotionally and/or behaviorally disturbed. We seek to reverse the impact of physical, emotional, and/or developmental traumas experienced in the early years of these children's lives through our Residential, Day and Outpatient Services Programs. In addition to our treatment and education programs, St. Ann's Home, Inc. is a committed advocate for children and families. In so doing St. Ann's Home seeks to alleviate the conditions that prevent our children from reaching their full potential.	Monday-Friday 8:30AM-5PM with evening and weekend emergency coverage	\N	\N	\N	\N	\N	\N	Denis Grandbois	President and CEO	\N	No	\N	Yes	Conference rooms are occasionally made available for public or professional education use. Call for more info.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
100220	2008-06-20	f	@lunch4all	Greater Lowell Community Foundation	GLCF	100 Merrimack St, Suite 202	\N	Lowell	MA	01852	978-970-1600	\N	\N	978-970-2444	www.glcfoundation.org	public_agency_email@testing123.net	The Greater Lowell Community Foundation is an 11 year-old, $18 million charitable foundation dedicated to improving the quality of life in the Nashoba Valley, Minuteman towns and Greater Lowell region of Massachusetts.  We achieve this mission by assisting individuals, families, nonprofit organizations and corporations to create permanent endowments and scholarship funds.  The Community Foundation invites you to work with us to create practical investments and imaginative solutions in our communities. \r\n\r\nThrough its portfolio of 259 community endowments and scholarship funds, and allied grant services, the Community Foundation has awarded more than $5 million in scholarships to high school students, grants to worthy charitable organizations identified by both donors and community grantmaking committees.	Monday-Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	David Kronberg	Executive Director	\N	No	\N	Yes	A small conference room (maximum 12)	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100187	2008-06-23	f	@lunch4all	North Shore Arc	\N	64 Holten Street	\N	Danvers	MA	01923	978-762-4878	\N	978-762-4878	978-777-6149	www.nsarc.org	public_agency_email@testing123.net	The North Shore Arc is dedicated to providing lifelong individualized support to people with or at risk of developing disabilities, and their families, thus ensuring opportunities to choose and realize their goals of where and how they learn, live, work, socialize and play. The North Shore Arc will provide leadership through the advocacy and education necessary to secure resources and ensure that the rights of individuals are protected. In order to achieve this mission, the North Shore Arc vows to build community partnerships that promote shared responsibility to become more self-sufficient.	Monday-Friday 9AM-5PM	(978) 624-2301 or 2337 -Bilingual Spanish speaker available \r\n(978) 762-4873 TDD Number	\N	\N	\N	\N	\N	Jerry McCarthy	Executive Director	\N	Yes	Contact North Shore Arc for information about the resource library	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-762-4873	1	
100528	2008-06-24	f	@lunch4all	American Textile History Museum	\N	491 Dutton Street	\N	Lowell	MA	01854	978-441-0400	\N	\N	978-441-1412	www.athm.org	public_agency_email@testing123.net	The American Textile History Museum tells America's story through the art, science and history of its textiles, with a newly redesigned core exhibit - "Textile Revolution - An Exploration through Space and Time," special exhibits (several each year), both onsite and traveling educational programs for schools and groups, textile arts lessons (weaving, spinning, felting, quilting), special public programs, and an extensive library and online catelogue.	The Museum is currently closed to the public while a major redesign of our core exhibit is underway, with a fall 2008 grand reopening anticipated. To schedule offsite and select onsite programming available during the closure, call 978-441-0400 X250. The library is unaffected by the closure, and may be contacted through the main phone number 978-441-0400.	\N	\N	\N	\N	\N	\N	James Coleman	Executive Director	\N	Yes	Resource materials on: Textile History, cloth, fabric, Lowell history, Lawrence history, historic images	Yes	Our newly redesigned facility, reopening in fall 2008, has various meeting space configurations. Call 978-441-0400 X241 for more information.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
101298	2008-07-01	f	@lunch4all	Massachusetts MENTOR, Lawrence Children's Program	MENTOR	Everett Mills	15 Union Street, 5th Floor	Lawrence	MA	01840	978-794-7966	866-375-5114	\N	978-794-9890	www.TheMentorNetwork.com	public_agency_email@testing123.net	Services for Children with Emotional and Behavioral Challenges \r\n\r\nNationally known for our expertise in working with children and their families, Massachusetts MENTOR approaches each child as a unique individual. Our services, customized to fit each young person's needs, are designed to improve child and family functioning, assist with permanency planning where appropriate and maximize the effectiveness of limited state resources. \r\n\r\nThis individualized, flexible approach allows us to serve a diverse range of children of different ages, backgrounds and needs, including emotional and behavioral issues, juvenile offenders and those with complex medical needs. \r\n\r\nMassachusetts MENTORs Lawrence Childrens Program in Lawrence, MA offers Specialized Foster Care services in the service areas North and Northwest of Boston. \r\n\r\nThis flexible program provides youths from troubled families, institutions and other settings  including teen parents and their babies  with a warm, stable environment coupled with the interventions needed to address severe emotional and behavioral disorders often stemming from abuse and neglect. Placement in the home of a specially trained caregiver, called a Mentor, is carefully matched according to each childs needs. Mentors provide children in their care with therapeutic intervention, nurturing support and the consistent guidance in addition to routine family- and community-based activities. A Clinical Coordinator oversees implementation of an Individualized Treatment Plan with emphasis on discharge planning to assure the youths appropriate progress and transition to home, permanent placement or independent living.	Office Hours are Mondays-Fridays, 9am-5pm.\r\nOn-call 24/7.	\N	\N	\N	\N	\N	\N	Luigi Grimaldi	Program Manager	The MENTOR Network	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
103355	2008-06-30	f	@lunch4all	Essex Art Center	\N	56 Island Street	\N	Lawrence	MA	01840	978-685-2343	\N	\N	\N	www.essexartcenter.com	public_agency_email@testing123.net	The Essex Art Center (EAC) is a nonprofit art organization with a mission to inspire and nurture the diverse artistic potential of the Greater Lawrence Community through classroom exploration and gallery exhibitions, making the creation and enjoyment of art accessible to all.\r\n\r\nIn November 1993 three women opened a gallery and small classroom on Essex Street and were determined to show fine art and teach art classes to any citizen of Lawrence and the Merrimack Valley who was interested. Since then Linda Maddox has moved on leaving Leslie Costello and Helen Tory to continue the dream of building a valuable art resource in downtown Lawrence. In 1995 they moved to a larger premises, with 7,200 square feet of galleries and classrooms in the Mill district, where the dream strengthens and grows... and as of July 2001 we occupy 13,200 sq. ft of creative space.	Office Hours: M-F, 10am-6pm	\N	\N	\N	\N	\N	\N	Leslie Costello	Executive Director	\N	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
103388	2008-07-10	f	@lunch4all	ValleyWorks Career Center -- Haverhill	VWCC-Haverhill	Landmark Building	80 Merrimack St.	Haverhill	MA	01830	978-722-7000	\N	\N	617-727-5739	www.valleyworks.cc	public_agency_email@testing123.net	The ValleyWorks Career Center (VWCC) is a workforce development collaboration of the Department of Training and Development and the Division of Career Services. A federal- and state-funded center, the VWCC is designed to provide comprehensive services to job seekers and businesses of the Merrimack Valley.\r\n\r\nThe VWCC is an Equal Opportunity Program. Auxiliary aids and services are available upon request to individuals with disabilities.\r\n\r\n*This listing is for the ValleyWorks Carrer Center in Haverhill, MA.	Mon-Thu:  8AM - 4:30PM\r\nFri:  9:30AM - 4:30PM	Chartered by the Merrimack Valley Workforce Investment Board (MVWIB), the VWCC is funded through the Workforce Investment Act; the Commonwealth Corporation; the Division of Career Services; the Department of Workforce Development; the Department of Transitional Assistance; the Department of Labor; the Department of Education and the Commonwealth of Massachusetts.	\N	\N	\N	\N	\N	Arthur Chilingirian	Executive Director	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-469-7859	1	
103377	2008-07-10	f	@lunch4all	ValleyWorks Career Center -- Lawrence	VWCC-Lawrence	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_agency_email@testing123.net	The ValleyWorks Career Center (VWCC) is a workforce development collaboration of the Department of Training and Development and the Division of Career Services. A federal- and state-funded center, the VWCC is designed to provide comprehensive services to job seekers and businesses of the Merrimack Valley. \r\n\r\nThe VWCC is an Equal Opportunity Program. Auxiliary aids and services are available upon request to individuals with disabilities.\r\n\r\n* This listing is for the ValleyWorks Carrer Center and Main Office in Lawrence, MA.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 7:00PM\r\nEvening hours include Workshops and Career Center Seminars only.	Chartered by the Merrimack Valley Workforce Investment Board (MVWIB), the VWCC is funded through the Workforce Investment Act; the Commonwealth Corporation; the Division of Career Services; the Department of Workforce Development; the Department of Transitional Assistance; the Department of Labor; the Department of Education and the Commonwealth of Massachusetts.	\N	\N	\N	\N	\N	Arthur Chilingirian	Executive Director	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-722-7088	1	
103344	2008-06-24	f	@lunch4all	Lawrence CommunityWorks	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-685-3115	\N	\N	\N	www.lcworks.org	public_agency_email@testing123.net	Lawrence CommunityWorks (LCW) is a nonprofit community development corporation and a Network of residents and other stakeholders working to transform the physical, economic, and civic landscape of Lawrence, MA. Our mission is to (1) foster individual and neighborhood empowerment and leadership ` through organizing residents to improve their lives economically, politically, and socially, (2) produce and preserve safe, decent, and affordable housing for low- and moderate-income families, (3) create programs and facilities that build the educational and economic assets of neighborhood young people, adults, and families, and (4) build a sustained institutional infrastructure for community revitalization through strategic local, regional, and national partnerships.	Monday --Friday, 9am-5pm\r\n\r\nNote that some programs and classes run on Saturdays.	\N	\N	\N	\N	\N	\N	William Traynor	Executive Director	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
102178	2009-01-15	f	@lunch4all	Greater Lawrence Community Action Council, Inc	GLCAC, Inc.	305 Essex St	\N	Lawrence	MA	01840	978-681-4900	\N	\N	978-681-4949	www.glcac.org	public_agency_email@testing123.net	The Greater Lawrence Community Action Council Inc. (GLCAC, Inc.) is a private non-profit agency, designated as the official federal and state anti-poverty agency for the Greater Lawrence area.  At GLCAC, Inc. we understand that individuals cannot attain a quality of life until basic needs are met.   The GLCAC, Inc sponsors many programs to assist individuals and families attain self-sufficiency by providing a comprehensive system of education, training, support and resources.	Monday - Friday 8:30 am - 4:30 p.m.	\N	GLCAC, Inc.	305 Essex St	Lawrence	MA	01840	Philip F. Laverriere, Sr.	Executive Director/CEO	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101705	2008-06-20	f	@lunch4all	Living Waters Ministries of Hope	\N	250 - 254 Middlesex Street	\N	Lowell	MA	01852	978-446-9800	\N	\N	\N	\N	public_agency_email@testing123.net	Living Waters Ministry of Hope is a non-denominational faith based ministry. Our mission is to offer support to all people in the community, by helping to rebuild lives, fostering healthy relationships, Teaching life skills and providing supportive and affordable housing.\r\n\r\nWe share the heart of God in building relationships.\r\n\r\nLiving Waters storefront offers an alternative place for the people of the community to come and spend time in an atmosphere of love and acceptance. We provide on a daily/weekly basis various meetings; groups and functions that help strengthen individuals in both their  spiritual and emotional growth.\r\n\r\nWe believe that we can make a meaningful contribution in eliminating homelessness among people with addiction, mental illness & disabilities.\r\n\r\nWith God's help we attempt to restore self-worth in all individuals.\r\n\r\nWe provide, weekly bible studies, Prayer and fellowship, Clothing, Referrals, functions, Trips, food packs, Community outreach,	Sunday Evenings 7pm-9:30pm\r\nTuesday Mornings 9am -12pm\r\nTuesday Evenings 7pm-9:30pm\r\nWeds.9am-4pm\r\nThursday Evenings 7pm-9:30pm\r\nFriday Mornings 9am- 12pm\r\nFriday Evenings 7pm-9:30pm\r\nSaturday Mornings 9:30-11:30	Watch for our up coming web page\r\nwww.livingwaterslowell.org	P.O. Box  2157	\N	Lowell	MA	01851	Diane Waddell	Director	\N	Not Applicable	\N	Yes	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
102519	2008-06-04	f	@lunch4all	Educational Surrogate Parent Program	\N	P. O. Box 1184	167 Lyman Street, Hadley Building, 3rd Floor	Westboro	MA	01581	508-792-7679	\N	\N	508-616-0318	espprogram.org	public_agency_email@testing123.net	The Educational Surrogate Parent Program (ESPP) recruits, trains and appoints volunteers to act as decision makers in special education matters for children who are in the custody of a state agency, whose parent or guardian are unknown or unavailable. Volunteers with the ESP Program provide a strong voice of advocacy for these students to ensure that they receive the services they deserve and are entitled to. The Educational Surrogate Parent (ESP) acts on behalf of the child and has all the rights and authority of a parent. These rights inlcude:\r\n* Meet and observe the child at school;\r\n* Review school records and receive progress reports;\r\n* Attend school meetings concerning the child'\r\n* Approve an Individual Education Program (IEP); and\r\nMonitor the child's services and educational placement.\r\n\r\nVolunteers with the Educational Surrogate Parent Program must:\r\n* Be at least eighteen years of age;\r\n* Have or be willing to obtain knowledge of the special education process through our  free training workshop;\r\n*Submit and application, provide two (2) references, and complete a CORI(Criminal Offender Record Inquiry). \r\n\r\nESP Program Volunteers spend 10 -20 hours per\r\nschool year supporting a student with special educational needs. Volunteers receive ongoing support from the ESP Program staff, have access to professional assistance and optional trainings and workshops. Volunteers    can attend an Annual Professional Development and Recognition Conference	Monday - Friday 8am - 4pm	\N	\N	\N	\N	\N	\N	Michelle Poulin	Program Director	EDCO Collaborative/EducationCollaborative Of Greater Boston,	Yes	The Educational Surrogate Parent Program (ESPP) maintains and updates periodically topical information regarding special education laws and regulations, advocacy skills, effective communication strategies and other relevant resources and documents. The ESP Program website links to various local, state and national resources.	Yes	The ESP Program is located in Westboro, MA. The program provides services throughout the entire state. The Training Programs are held in community centers, libraries and  community based organizations statewide. At the offices in Westboro, we have a large conference room suitable for meetings.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	2	
102607	2008-06-05	f	@lunch4all	Andover Youth Services	\N	36 Bartlett St.	Andover Town Offices, 3rd Floor	Andover	MA	01810	978-623-8241	\N	\N	\N	andoveryouthservices.com	public_agency_email@testing123.net	Out of our 300 square feet of space, the AYS runs programs year-round for the 6,500 young people of Andover. \r\n\r\nA dedicated staff of 4 full time employees and several seasonal workers provides recreational, educational, social and support programs for over 6,000 participants annually. \r\n\r\nWe've been going strong for thirteen years and its because we know that it is all about the kids. In an effort to better serve the needs of the community, the AYS and Andover Youth Foundation have been trying to fundraise for a Youth Center to give a home to this flourishing program. \r\n\r\nPlease surf around our website and find out what we have to offer and let us know if you have any ideas for programs.	School Hours \r\nTuesday through Friday: 3pm to 7:00pm \r\nSaturday/Sunday: 12pm to 7pm \r\nClosed Mondays \r\nSummer Hours \r\nTuesday through Sunday: 12pm to 7:30pm \r\nClosed Mondays	\N	\N	\N	\N	\N	\N	Bill Fahey	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	2	
102387	2008-06-05	f	@lunch4all	Jericho Road Lawrence, Inc.	\N	190 Academy Road,	\N	North Andover	MA	01845	978-604-8803	\N	\N	\N	http://www.jerichoroadlawrence.org/	public_agency_email@testing123.net	Jericho Road Lawrence's mission is to address economic inequities by matching the needs of non profit organizations in Lawrence, Massachusetts to the talents and resources of the surrounding suburban communities through the use of volunteers who offer no cost, customized professional services to those non profit organizations focused on helping individuals and families become economically self-sustaining.	Monday thru Friday, 8:30 am - 5 pm.	\N	\N	\N	\N	\N	\N	Joan Kulash	Executive Director	\N	No	\N	Yes	North Parish Unitarian Universalist Church, 190 Academy Road, North Andover.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	2	
102200	2008-01-28	f	@lunch4all	Citizen Schools / Community Teamwork, Inc.	\N	365 West Meadow Rd.	\N	Lowell	MA	01854	978-821-0953	\N	\N	\N	http://www.citizenschools.org	public_agency_email@testing123.net	Citizen Schools operates a national network of apprenticeship programs for middle school students, connecting adult volunteers to young people in hands-on learning projects. At Citizen Schools, students develop the academic and leadership skills they need to do well in school, get into college, and become leaders in their careers and in their communities.	Program: Monday - Thursday afterschool until 5:45\r\nApprenticeships: Tuesday or Thursday from about 4:00 - 5:30	\N	\N	\N	\N	\N	\N	Rebecca Jervey	Teaching Fellow	Citizen Schools	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	4	
101562	2007-07-01	f	@lunch4all	Greyhound Pets of America-New Hampshire Chapter	GPA-CNHC	P.O. Box 778	\N	Concord	NH	03302	888-507-9597	888-507-9597	\N	\N	www.gpa-cnhc.org	public_agency_email@testing123.net	Greyhound Pets of America (GPA) is a national nonprofit, volunteer-supported organization dedicated to the placement of ex-racing greyhounds in ideal home situations. \r\n\r\nGPAs Central New Hampshire Chapter shall be responsible for finding homes for greyhounds that fail to qualify or which no longer qualify for the racetrack, to acquaint the public as to the desirability of greyhounds for pets and to inform the public of the availability of these dogs for adoption. GPA of Central New Hampshire will be working to place greyhounds from the track in Belmont and other tracks in Central New Hampshire. \r\n\r\nGPA Central New Hampshire volunteers work closely with each prospective greyhound owner, matching them, with a dog whose personality is most ideally suited to its new home situation to ensure a lasting, rewarding relationship.	7 days per week, varies by appointments	\N	\N	\N	\N	\N	\N	Carolyn and Ken Wright	Keeper of the Hounds and Director	none	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	4	
102101	2007-09-28	f	@lunch4all	Aved Therapy	\N	21 Central Street, 2nd floor	\N	Andover	MA	01810	978-470-3343	\N	617-968-2546	978-470-3323	\N	public_agency_email@testing123.net	Diruhi Mattian, LICSW is a psychotherapist well respected for her work with adolescents, adults, couples, and families. She earned her MSW from Simmons College. She specializes in psychological trauma, treating mood disorders, depression, anxiety spectrum disorders(panic disorders, phobias, OCD)She works successfully with ADHD and runs Parenting Workshops.  She uses EMDR, IFS, DBT, and CBT as treatment approaches.	Tuesday, Thursday from 7PM to 9 PM\r\nWednesday from 2PM to 9PM, Friday all day, and Saturday from 9AM to 5PM	\N	\N	\N	\N	\N	\N	Diruhi Mattian	Owner of the practice	\N	Yes	Related literature and handouts are available for clients use	Yes	The office is on the second floor, there is a spacious waiting room and bathrooms available	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	4	
100781	2008-04-28	f	@lunch4all	LifeLinks, Inc.	\N	145 Lexington Avenue	\N	Lowell	MA	01854	978-459-6179	\N	\N	978-453-7077	www.lifelinks.ws	public_agency_email@testing123.net	LifeLinks is chartered to advocate for and provide services and supports for people with developmental disabilities that are community based, are cost effective, enable people to become contributing members of their community, and are of a quality we would want for our loved ones. LifeLinks services and supports include Family and Individual Supports; Residential Living (including group home settings); Shared Living; Community Center; Recreation and Education (including independent living seminars and therapeutic and medical services.	Monday-Friday 8AM-4PM	\N	145 Lexington Ave	\N	Lowell	MA	01854	Yvonne La-Garde	Chief Executive Director	\N	No	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	1	
101914	2008-05-14	f	@lunch4all	Depression and Bipolar Support Alliance	DBSA Merrimack Valley	The First Church, 1195 Varnum Ave., Lowell, MA	\N	Lowell	MA	01852	978-458-0541	\N	978-322-5120	\N	www.dbsalliance.org	public_agency_email@testing123.net	To improve the lives of people living with mood disorders.\r\nWe offer support groups, web site with extended resources,networking,service opportunities,tools and acceptance. Free and confidential.	Thursday's 6pm at The First Church, 1195 Varnum Ave., Lowell, MA	Contact information available from facilitator and other participants at meetings.	246 Market Street	Apt. 327	Lowell	MA	01852	Alicia Page	Facilitator	Depression and Bipolar Support Alliance (DBSA)	Yes	books, CD's, video and literature available at groups\r\nthrough the main web site www.dbsa.org	Yes	Comfortable conference room at The First Church holds up to 30 people.	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	1	
100880	2009-01-15	f	@lunch4all	Billerica Board of Health	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0931	\N	978-671-0900	978-671-0919	www.town.billerica.ma.us	public_agency_email@testing123.net	The mission of the Billerica Board of Health is to provide preventative services to Billerica residents and to community well-being.	Monday-Friday 8:30AM-4PM\r\nSign-offs 8:30-10am and 3-4pm	\N	\N	\N	\N	\N	\N	Richard Berube	Director	Town of Billerica	No	\N	Yes	Space may be available, however town departments receive priority	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101408	2009-01-15	f	@lunch4all	Network Health	\N	101 Station Landing	Fourth Floor	Medford	MA	02155	888-257-1985	888-257-1985	\N	\N	www.network-health.org	public_agency_email@testing123.net	Network Health is a Massachusetts health plan that provides high-quality health care coverage to more than 150,000 members through its Network Health Together (MassHealth) and Network Health Forward (Commonwealth Care) plans.	Monday - Friday, 8:30 a.m. - 5 p.m.	\N	\N	\N	\N	\N	\N	Christina Severin	Executive Director	Cambridge Health Alliance	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	888-391-5535	0	
101826	2009-01-15	f	@lunch4all	Billerica Council on Aging	Billerica Senior Center	25 Concord Road	\N	Billerica	MA	01821	978-671-0916	\N	\N	978-671-1347	www.town.billerica.ma.us	public_agency_email@testing123.net	The Senior  Center is a community focal point where adults 60+ come together for activities, programs and services that reflect their experience and skills; respond to their diverse needs and interests; enhance their dignity; support their independence; and encourage their involvement in and with the Center and the community.\r\n\r\nThe Center also serves as a resource for the entire community for information on aging, support for family caregivers and for development of innovative approaches to addressing aging issues.	Monday through Friday 8:30-4:00	\N	\N	\N	\N	\N	\N	Donna Popkin	Director	Town of Billerica	Yes	The Out Reach department maintains resources on a variety of elder care options.  From services available in the privacy and dignity of ones home to institutional care and other housing alternatives.  We offer a subscription to our monthly newlsetter the Outlook.  Contact the senior center for more information.	Yes	Meeting space is limited to Billerica community non-profit organizations	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103102	2009-01-15	f	@lunch4all	JJ Referral Services	\N	506 Katahdin Drive	\N	Lexington	MA	02421	781-572-0219	\N	\N	\N	www.jjreferralservices.com	public_agency_email@testing123.net	Help to those in need of services through the human services system. Homeless, low-income, disabled, and those who have become homeless because of floods and fires. I help clients obtain the services they need and deserve. I am a statewide vendor and whenever possible go through the state to get paid. will receive referrals from social service agencies only. Hoping to expand services at different food pantries but its a work in progress.\r\n\r\nI am planning on becoming a Certified Peer Support Counselor this Spring. Please go to web site for more details.	Weekly-9 to 5	\N	\N	\N	\N	\N	\N	Joanne Stephen	owner	\N	Not Applicable	\N	Yes	12 Alfred Street Baldwin Park. Need to make appointments beforehand by calling the above number	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100572	2009-01-15	f	@lunch4all	Community Teamwork, SuitAbility Program	SuitAbility	536 Pawtucket Street	\N	Lowell	MA	01854	978-934-8898	\N	\N	978-934-8899	http://www.suitabilityma.org	public_agency_email@testing123.net	The mission of SuitAbility, Inc. is to provide free interview/work clothing and work related support services to low-income women in Middlesex and Essex Counties.We provide free workshops and interview/work clothing for women.	Monday-Thursday 9AM-5PM	\N	CTI SuitAbility Program	536 Pawtucket St.	Lowell	MA	01854	Kathleen Hayes	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100704	2009-01-15	f	@lunch4all	Retarded Adult Rehabilitative Association Inc.	RARA	295 High Street	\N	Lowell	MA	01852	978-453-5598	\N	\N	978-446-0270	raraagency.org	public_agency_email@testing123.net	The mission of RARA is to promote public awareness of issues specific to the developmentally challenged and their families. We serve our members through educational, recreational, and sports-related programs.	Office hours\r\nTuesday, Wednesday, Thursday 9AM-3PM\r\n\r\nEvening Program based on calendar events	\N	\N	\N	\N	\N	\N	Donna Michaels	Office Administrator	\N	No	\N	Yes	Contact RARA for meeting space availability and arrangements	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101727	2009-01-15	f	@lunch4all	The Peter Pan Center	\N	280 Ayer Road	P.O. Box 312	Harvard	MA	01451	978-772-1255	\N	\N	978-772-4711	www.peterpancenter.homestead.com	public_agency_email@testing123.net	The mission of the Peter Pan Center is to provide a safe and welcoming "parent-to-parent" atmosphere for families seeking support, skills and resources to more effectively parent as well as to enrich the lives of spirited and sensitive children. From toddlers to teens, including typically developing children to those with more significant learning, social and behavioral issues, our goal is to provide answers to parent questions; support for both the home and school environments; skill-building programs for both parents and children as well as to connect families with appropriate services and resources.	By appointment only	\N	\N	\N	\N	\N	\N	Donna Shea	Director	\N	Yes	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
100671	2009-01-15	f	@lunch4all	Department of Children and Families (Formerly DSS)	DCF   (formerly DSS)	33 East Merrimack Street	\N	Lowell	MA	01852	978-275-6800	800-792-5200	800-792-5200	978-452-5896	www.state.ma.us/dss/	public_agency_email@testing123.net	The Department of Children and Families(formerly the Depatment of Social Services) is dedicated to the safety, well being, and permanency for children who have been abused or neglected in a family setting. We work first and foremost to keep families together. To achieve this goal, DCF provides family-based services to assist parents in better caring for their children. For those children who are unable to remain at home because of severe abuse or neglect or who are at risk for abuse and neglect, DCF provides temporary out of home care through kinship placement, foster care, group care, or residential programs. These short-term measures are meant to provide interim guidance and support to families until the children can safely be returned home. In case where this is not possible, DCF will seek to provide a child with an alternate permanent situation, such as adoption, guardianship, or, as with older teenagers, independent living and lifelong connections.  The Lead Agency for the Lowell DCF FamilyNetworks services is MSPCC,	Monday-Friday 8:45AM-5PM; After hours/ weekend hotline available.	\N	\N	\N	\N	\N	\N	Lori Ortiz	Acting Area  Director	Commonwealth of Massachusetts	Yes	Resource library available through our central office, contact DSS for details.	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-275-6800	0	
101078	2009-01-15	f	@lunch4all	Casey Family Services	\N	18 Palmer Street	\N	Lowell	MA	01852	978-937-1877	800-883-8836	978-937-1877	978-937-2262	www.caseyfamilyservices.org	public_agency_email@testing123.net	Casey Family Services is committed to improving the lives of at-risk children and to strengthening families and communities by providing high-quality, cost-effective services that advance both positive practices and sound public policy.	Monday, Tuesday,Wednesday Thursday, Friday 9AM-5PM	\N	\N	\N	\N	\N	\N	Raymond Torres	Executive Director	Annie E. Casey Foundation	Yes	Contact Casey for more information on resource library materials	Yes	Meeting space is available by arrangement. Contact Casey Dennis at Casey Family Services for reservations.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102772	2009-01-15	f	@lunch4all	Cedarland Aquatic Center	Cedardale Swim Lessons	931 Boston Road	\N	Haverhill	MA	01835	978-521-7700	\N	\N	\N	www.cedarland.net	public_agency_email@testing123.net	To provide a safe and enjoyable water experience for children ages 8 months through 16 years old.  Our lessons are designed to develop your child in stages from water orientation through pre-competitive swim team.  We use creative and innovative drills with equipment designed to build strength and endurance as well as making lessons fun, motivating and productive.  Our program has been developed with a base from the American REd Cross and the National Swim School Association guidelines.	Lessons are available Saturdays, Mondays - Thursdays, Mornings and afternoons.  PreTeam programs are offered weekday after school.	\N	\N	\N	\N	\N	\N	Stacy Roberts	Aquatic Director	Cedardale Health & Fitness	Not Applicable	\N	Yes	Conference space available at the Aquatic Center as well as at our main club Cedardale Health & Fitness	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
102112	2009-01-15	f	@lunch4all	Light of Cambodian Children, Inc.	\N	9 Central Street, Suite 203	\N	Lowell	MA	01852	978-459-0200	\N	\N	\N	www.reaksmey.org	public_agency_email@testing123.net	Light of Cambodian Children, Inc., (LCC) is a 501(c)(3) organization established in 1998. Through interactive advocacy, educational, and social programs, Light of Cambodian Children, Inc. encourages youth to seek higher education, become self-sufficient, and serve as role models in our communities.	Each program run on different time. Please see individual program for specific time and date.	\N	\N	\N	\N	\N	\N	Sayon Soeun	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101177	2009-01-15	f	@lunch4all	Cultural Organization of Lowell	COOL	375 Merrimack Street	3rd Floor	Lowell	MA	01852	978-446-7162	\N	\N	978-446-7089	www.cultureiscool.org	public_agency_email@testing123.net	The Cultural Organization of Lowell (COOL) is a collaborative arts organization that supports visual and performing artists, arts groups and cultural activities to stimulate a vibrant downtown Lowell!	COOL's office hours are Monday through Friday from 9 a.m. to 5 p.m., but appointments must be arranged in advance to guarantee that someone will be present.	COOL provides the community with a central on-line calendar of cultural events.  For the most up-to-date listing of these events, visit: www.cultureiscool.org	\N	\N	\N	\N	\N	LZ Nunn	Executive Director	\N	No	\N	Yes	The COOL office can comfortably accommodate a meeting for 8-10 people.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
100319	2009-01-16	f	@lunch4all	National Alliance on Mental IllnessGreaterLowell	NAMI	P.O. Box 692	\N	Tyngsborough	MA	01879	978-677-0618	\N	\N	\N	www.nami.org/sites/greaterlowell	public_agency_email@testing123.net	NAMI Greater Lowell is a support group of concerned individuals responding to the tragic neglect of our mentally ill citizens. They provide sharing and caring, compassionate support and understanding for all our members and anyone else seeking this support.	3rd Wednesday of every month 7PM at Solomon Mental Health Center 391 Varnum Ave, Lowell, MA	\N	\N	\N	\N	\N	\N	Marilyn Jonas	President	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101331	2009-01-16	f	@lunch4all	Lawrence HERC	\N	78-82 Amesbury St.	\N	Lawrence	MA	01841	978-738-7634	\N	\N	\N	http://www.lawrence-herc.org/	public_agency_email@testing123.net	The core mission of Lawrence HERC involves recruiting and preparing motivated students for high school and college achievement.\r\nLawrence HERC focuses on high school Juniors and Seniors with weekly classes that develop them as youth and prepare them for college, college visits, SAT prep, computer lab access, early college awareness and a summer program with Let's Get Ready (www.letsgetready.org)	Monday thru Friday	\N	Lawrence HERC	PO Box 1758	Lawrence	MA	01842	Meghan Aceto	HERC Program Coordinator	\N	Yes	College Access materials\r\nSAT Prep material\r\nScholarship Information	Yes	The weekly classes are held in the Northern Essex Community College Extension Campus.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
101375	2008-11-04	f	@lunch4all	Asian Center of Merrimack Valley, Inc.	Asian Center	1 Ballard Way	Suite 101	Lawrence	MA	01843	978-683-7316	\N	\N	978-683-9075	asiancentermv.org	public_agency_email@testing123.net	Multi-service organization serving Asian immigrant and refugee families, supporting their effors to build a stable future and to become participating members of their community.	Monday through Friday, 9 - 5\r\nEvening classes Tuesday - Thursday, 6 - 9\r\nSome weekend events	\N	\N	\N	\N	\N	\N	Kaori Kelts	Executive Director	Asian Center of Merrimack Valley, Inc.	No	\N	Yes	3 large classrooms	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-683-7316	0	
100869	2009-06-05	f	@lunch4all	Asian Task Force Against Domestic Violence Lowell Project	\N	319 Wilder St.	\N	Lowell	MA	01852	978-452-4522	800-456-7890	617-338-2355	617-338-2354	www.atask.org	public_agency_email@testing123.net	The Asian Task Force's mission is to eliminate family violence and to strengthen Asian families and communities	Contact agency for hours	\N	319 wilder st	\N	lowell	MA	01852	Qingjian Shi	Interim Director of Education and Outreach	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	123-456-7890	0	
103982	2009-06-05	f	@lunch4all	Save The Whales	\N	319 Wilder Street	\N	Lowell	MA	01852	978-452-4522	666-666-6666	888-888-8888	555-555-5555	www.savethewhales.org	public_agency_email@testing123.net	Our mission here at Save The Whales is to proctect the whales in our ocean from exstinction. To provide them with a safe nd healthy habitat in order to live happy lives.	M-F 8a-7p	\N	319 Wilder Street	\N	Lowell	MA	01852	Jennifer Cloutier	Program Director	World Wildlife Foundation	No	\N	Yes	Meeting space is able to comfotably sit 25 people comfortably.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	444-444-4444	0	
\.


--
-- Data for Name: agency_search_log; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY agency_search_log (search_id, session_id, ip_address, "timestamp", search_phrase, referer, hit_count) FROM stdin;
36654	96.237.29.220.17312109	96.237.29.220	2009-03-31 17:03:29.093943	revolving museum	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36664	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:53:34.581903	community teamwork inc	http://mvh.omacneil.testing123.net	2
36671	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:43:24.175685	communit team work	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36672	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:43:32.157881	community team work	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36673	96.237.27.110.13134109	96.237.27.110	2009-05-13 13:58:31.403543	shelters	http://mvh.omacneil.testing123.net/	0
36675	96.237.27.110.13134109	96.237.27.110	2009-05-13 13:59:56.070322	community software lab	http://mvh.omacneil.testing123.net/	1
36678	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:05:04.517602	c.t.i	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	1
36687	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:16:28.258154	community teamwork 	http://mvh.omacneil.testing123.net/	3
36688	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:17:11.786513	neighbors in need	http://mvh.omacneil.testing123.net/	1
36689	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:17:52.548764	ann sullivan center	http://mvh.omacneil.testing123.net/	1
36735	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:12:30.374655	dancing agencies	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=804807;search_phrase=lowell%20dancing%20classes%2F%20tap	0
36739	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:14:03.769525	ymca	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	3
36743	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:19:18.523336	neighbors in need	http://mvh.omacneil.testing123.net/	1
36744	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:20:15.977215	ann sullivan canter	http://mvh.omacneil.testing123.net/	0
36745	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:20:27.875318	anne sullivan center	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	1
36746	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:40:18.812269	cti	http://mvh.omacneil.testing123.net/	1
36749	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:18:01.705973	questions	http://mvh.omacneil.testing123.net/	0
36750	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:22:41.570257	dance lessons	http://mvh.omacneil.testing123.net/	0
36752	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:31:08.446054	neighbors in nee	http://mvh.omacneil.testing123.net/	1
36760	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:00:24.132697	Neighbors In Need	http://mvh.omacneil.testing123.net/	1
36761	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:00:58.167229	Youth Build	http://mvh.omacneil.testing123.net/	0
36763	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:01:37.934524	YouthBuild	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36764	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:02:09.62431	Annie Sullivan Center	http://mvh.omacneil.testing123.net/	0
36765	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:02:31.303466	Anne Sullivan Center	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	1
36769	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:52:37.032288	UTCH	http://mvh.omacneil.testing123.net/	0
36772	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:53:49.301537	CTI	http://mvh.omacneil.testing123.net/	1
36773	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:55:30.622334	youth bulid	http://mvh.omacneil.testing123.net/	0
36774	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:55:46.336253	youth build	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36775	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:58:19.924482	anne sullivan center	http://mvh.omacneil.testing123.net/	1
36803	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:18:14.373806	neighbors in need	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801254;search_phrase=neighbors%20in%20need	1
36804	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:18:41.172381	youth build	http://mvh.omacneil.testing123.net/	0
36806	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:20:00.816983	anne sullivan center	http://mvh.omacneil.testing123.net/	1
36808	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:00:56.978023	help with paying utilities	http://mvh.omacneil.testing123.net/	0
36809	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:01:05.357261	paying utilities	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36810	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:01:11.438185	paying bills	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36811	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:01:37.588362	 bills	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36812	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:01:42.718865	bills	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36815	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:08:15.979039	The GRIP Project	http://mvh.omacneil.testing123.net/	0
36816	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:08:21.025323	GRIP Project	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36817	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:08:26.370085	GRIP	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36818	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:08:40.498718	G.R.I.P	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	0
36823	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:13:15.26041	Neighbors In Need	http://mvh.omacneil.testing123.net/	1
36824	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:13:51.704791	Youth Build	http://mvh.omacneil.testing123.net/	0
36826	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:16:36.128973	Anne Sullivan Center	http://mvh.omacneil.testing123.net/	1
\.


--
-- Data for Name: alias; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY alias (category_id, alias_name, stems_name) FROM stdin;
800000	History	histori
800121	Code Violations	code violat
800132	Trash	trash
800132	Sanitation Inspection	sanit inspect
800198	Volunteering	volunt
800286	Neighborhood Organizations	organ neighborhood
800297	Community Access	access commun
800297	Public Access	access public
800308	Computer, Open Access	open access comput
800396	Counseling Bereavement, Counseling Grief	counsel bereav grief
800407	Support Group, Death	death group support
800473	Couples Counseling	counsel coupl
800539	Hotlines, Battered Woman	hotlin batter woman
800539	Crisis Lines, Domestic Violence	crisi domest line violenc
800539	Crisis Lines Battered Woman	crisi batter woman line
800561	Hotlines, Battered	hotlin batter
800561	Crisis Lines, Child Abuse	crisi child abus line
800561	Crisis Lines, Battered Children	children crisi batter line
800594	Advocacy, Battered Women	advocaci batter women
800605	Counseling, Battered Children	counsel children batter
800616	Battered Women	batter women
800616	Domestic Violence	domest violenc
800627	Counseling, Battered Women	counsel batter women
800638	Hotlines, Sexual Assault	hotlin sexual assault
800638	Crisis Lines Rape	crisi rape line
800638	Crisis Lines Sexual Assault	crisi sexual assault line
800649	Abuse, Elderly	elderli abus
800660	Domestic Violence	domest violenc
800671	Advocacy, Battered Children	children advocaci batter
800693	Battered Children Prevention	prevent children batter
800715	Information and Referral, Battered Children	children inform batter and referr
800726	Information and Referral, Sexual Assault	inform sexual and assault referr
800759	Information and Referral, Battered Woman	inform batter and referr woman
800770	Counseling Sexual Assault	counsel sexual assault
800792	Victims' Advocacy Rape	advocaci rape victim
800792	Victims' Advocacy Sexual Assault	advocaci sexual assault victim
800803	Sexual Assault	sexual assault
800517	Support Group Rape	group rape support
800869	Advocacy Mental Illness	ill advocaci mental
800880	Halfway House, Disabled	disabl halfwai hous
800935	ADD	add
800979	Blind	blind
801023	Screen Reader/Magnification Software	softwar reader screen magnif
801034	SSI	ssi
801034	SSDI	ssdi
801056	Deaf	deaf
801067	Special Education	special educ
801078	Developmental Disability	development disabl
801078	Mental Disability	mental disabl
801111	Adaptive Aids/Equipment	equip aid adapt
801188	Education Grade School	grade educ school
801210	Skipping School	school skip
801210	Playing Hooky	hooki plai
801243	English As a Second Language	languag english a second
801243	English for Speakers of Other Foreign Languages	english languag speaker foreign other
801254	Advocacy, Special Education	special educ advocaci
801265	Tutoring Junior High School	school junior high tutor
801342	Homework Assistance Junior High School	homework school junior assist high
801386	Education, Junior High School	educ school junior high
800913	Special Education	special educ
801485	Tutoring Grade School	grade school tutor
801529	Homework Assistance Grade School	grade homework school assist
801540	Information and Referral, Special Education	special inform educ and referr
801551	Hotlines, Special Education	special educ hotlin
801584	ABE	abe
801672	FMLA	fmla
801694	Employment Discrimination Assistance	assist discrimin employ
801749	Unemployment Disputes	disput unemploy
801749	Unemployment Rights	right unemploy
801749	Legal Assistance, Unemployment	legal unemploy assist
801771	Job Search Support Groups	search group job support
801826	Job Prep and Readiness Training	train readi and prep job
801837	Business Training	train busi
801892	Advocacy, Debt	debt advocaci
801903	Tax Assistance	assist tax
801903	VITA	vita
801914	Information and Referral, Paying for College	inform colleg pai and referr
801925	Debt Management	debt manag
801925	Cleaning Up Credit	up credit clean
801947	Hotlines, Debt	debt hotlin
801958	Financial Advisor	advisor financi
801969	Budgeting	budget
801980	Information and Referral, Debt	inform debt and referr
801991	Paying for College	colleg pai
802002	Homeless, Families	homeless famili
802035	Food Program	food program
802035	Woman Infants and Children	infant children and woman
802046	Homeless, Women	homeless women
802079	Advocacy, TANF	advocaci tanf
802079	Advocacy, Welfare	advocaci welfar
802090	Homeless, Men	men homeless
802123	Information and Referral Food Stamps	inform food and referr stamp
802134	Information and Referral Homeless	inform homeless and referr
802156	Furniture	furnitur
802178	TANF	tanf
802178	Welfare	welfar
802189	Hotlines Homeless	homeless hotlin
802222	Phone Assistance	assist phone
802233	Information and Referral, TANF	inform and referr tanf
802233	Information and Referral AFDC	inform and referr afdc
802255	Advocacy Shelter	advocaci shelter
802288	Disability Insurance	insur disabl
802299	Eating Healthy	eat healthi
802343	Weight Control	weight control
802343	Weight loss	weight loss
802343	Over Weight	weight over
802343	Diet	diet
802343	Lose Weight	weight lose
802376	Hotlines, Free Medical Care	free hotlin care medic
802453	Information and Referral, Free Medical Care	free inform care and medic referr
802508	Nursing	nurs
802519	Shots	shot
802574	Health Care, Pediatric	care health pediatr
802640	Sexually Transmitted Diseases	diseas transmit sexual
802662	Nurses Assistant	assist nurs
802717	Birth Control	birth control
802750	Giving Blood	blood give
802805	Dentist	dentist
802849	Find Doctor	doctor find
802915	Hotlines, Homeless	homeless hotlin
802981	Rental Assistance	rental assist
803069	Information and Referral, Homeless	inform homeless and referr
803080	Homeless Shelter	homeless shelter
803102	Get the Lead Out	lead out get
803113	Advocacy, Homeless	homeless advocaci
803124	Hotlines, Refugees	refuge hotlin
803179	Advocacy, Refugees	refuge advocaci
803201	Refugee Rights	refuge right
803245	Information and Referral Refugees	refuge inform and referr
803256	Legal Assistance, Refugees	refuge legal assist
803300	Hotlines, Law	hotlin law
803300	Hotlines, Lawyer	hotlin lawyer
803311	Consumer Protection, Consumer Advocacy	advocaci protect consum
803311	Consumer Complaints	complaint consum
803344	Lawyer Criminal	crimin lawyer
803366	Legal Assistance, Lawyer	legal lawyer assist
803388	Lawyer, Civil	civil lawyer
803399	Hotlines, Consumer Protection	hotlin protect consum
803399	Hotlines, Consumer Advocacy	advocaci hotlin consum
803399	Hotlines Consumer Complaints	hotlin complaint consum
803487	Bipolar	bipolar
803487	Depressed	depress
801034	SSI, SSDI	ssi ssdi
804026	Sports/Recreation, Junior High School	recreat school junior sport high
804081	Sports/Recreation, Grade School	recreat grade school sport
804103	Little League	littl leagu
804169	Pop Warner	warner pop
804191	Residential Program, Drug Addiction	drug addict program residenti
804202	Support Group, Drug Addiction	group drug addict support
804235	Counseling, Drug Addiction	counsel drug addict
804279	Recovery, Drug Addiction	drug addict recoveri
804279	Drug Rehabilitation	drug rehabilit
804301	Prevention Program, Drug Addiction	prevent drug addict program
804312	Hotlines, Drug Addiction	hotlin drug addict
804356	Halfway House, Drug Addiction	halfwai drug addict hous
804378	Advocacy, Drug Addiction	advocaci drug addict
804389	Information and Referral, Drug Addiction	inform drug and addict referr
804444	Advocacy, Teen	advocaci teen
804444	Advocacy Infant/Toddler	toddler infant advocaci
800693	Domestic Violence Prevention	prevent domest violenc
804477	Prevention Programs Junior High School	prevent school junior high program
804521	Companion Programs Youth	youth program companion
804554	Hotlines, Teen	hotlin teen
804554	Hotlines Infant/Toddler	toddler infant hotlin
804587	Homework Assistance Junior High School	homework school junior assist high
804609	Companion Programs Teen	teen program companion
804620	After School Programs, Grade School	grade school after program
804631	Information and Referral, Teen	inform teen and referr
804631	Information and Referral Infant/Toddler	toddler inform infant and referr
804708	Prevention Programs Grade School	prevent grade school program
804741	Day Care, 0 - 05	care dai
804719	Day Care, 05 - 12	care dai
804532	Youth Camp	camp youth
804686	Teen Camp	camp teen
804686	Youth Camp	camp youth
804598	Teen Camp	camp teen
804598	Youth Camp	camp youth
804466	After School Programs Teen	teen school after program
804466	After School Programs Youth	youth school after program
804587	After School Programs Teen	teen school after program
804587	After School Programs Youth	youth school after program
804620	After School Programs Youth	youth school after program
804796	Painting Classes, Youth/Teen	youth class paint teen
804796	Drawing Classes, Youth/Teen	youth class teen draw
804796	Video Classes, Youth/Teen	youth class teen video
804796	Graphic Design Classes, Youth/Teen	youth class teen graphic design
804807	Acting Classes, Youth/Teen	youth class teen act
804807	Theater Classes, Youth/Teen	youth class teen theatre
804026	Youth Sports	youth sport
804026	Teen Sports	teen sport
804081	Youth Sports	youth sport
804092	Youth Sports	youth sport
804092	Teen Sports	teen sport
804851	Painting Drawing Video Graphic Design Classes, Adult	paint graphic draw design video adult class
804807	Dance Classes, Youth/Teen	youth class teen danc
804862	Acting Theater Dance Classes, Adult	class danc act adult theater
804884	Discount Health Care	care health discout
804884	Sliding Scale Health Care	scale care health slide
802519	Vaccinations	vaccin
804917	Affordable Housing Elderly	elderli hous afford
802222	Bills	bill
804939	Meals	meal
804939	Food	food
804950	Going back to school	back school go
804961	Financial Education	educ financi
804961	Banking	bank
804961	Credit	credit
804532	Summer	summer
804598	Summer	summer
804686	Summer	summer
805005	Bulimia	bulimia
805005	Anorexia	anorexia
802244	Soup Kitchen	kitchen soup
805027	Karate	karat
805027	Tae Kwon Do	do kwon tae
805027	Taekwon Do	do taekwon
805027	Taekwondo	taekwondo
801221	Childcare Training	childcar train
801408	Home Based Childcare Certification	childcar certif home base
804774	Information and Referral, Childcare	childcar referr inform and
804774	Information and Referral, Day Care	referr inform dai and care
804653	Childcare Vouchers	childcar voucher
804763	Home Based Childcare 0 - 05	childcar   home base 0 05
804763	Home Based Day Care 0 - 05	  home base dai care 0 05
804752	Home Based Childcare 05 - 12	childcar  home base  05 12
804752	Home Based Day Care 05 - 12	 home base dai  care 05 12
803707	DSS	dss
800550	Battered Children	children batter
800550	DSS	dss
805038	Discount Drugs	drug discount
802211	SNAP	snap
\.


--
-- Data for Name: category; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY category (category_id, category_name, stems_name) FROM stdin;
800000	Cultural Heritage	cultur heritag
800022	Music	music
800033	Arts/Entertainment	entertain art
800044	Theater/Performance Art	art perform theater
800055	Museum	museum
800066	Arts, Youth	youth art
800077	Culture	cultur
800088	Information and Referral, City Services	servic inform citi and referr
800099	Hotlines, City Services	servic hotlin citi
800110	Waste Management	wast manag
800121	Code Enforcement	enforc code
800132	Sanitation	sanit
800143	Voter Registration	registr voter
800154	Public Transportation	public transport
800165	Recycling	recycl
800176	City Services	servic citi
800187	Elections / Campaign	campaign elect
800198	Community Service	servic commun
800209	Crime Watch	watch crime
800220	Library	librari
800231	Newspapers	newspap
800242	Media	media
800253	Community Gardens	garden commun
800264	Bookmobile	bookmobil
800275	Book Club	club book
800286	Neighborhood Groups	group neighborhood
800297	Television/Radio	radio televis
800308	Computer, Public Access	access public comput
800319	Counseling, Suicide	counsel suicid
800330	Counseling, Teen	counsel teen
800341	Suicide Hotline	hotlin suicid
800352	Counseling, Crisis	counsel crisi
800363	Hotlines, Counseling	counsel hotlin
800374	Support Group, Mental Health	group health mental support
800385	Support Group, Medical	group medic support
800396	Counseling, Death	counsel death
800407	Support Group, Bereavement	group bereav support
800418	Counseling, Mental Health	counsel health mental
800429	Information and Referral, Counseling	counsel inform and referr
800440	Counseling, Child	counsel child
800451	Counseling, Sexual Assault	counsel sexual assault
800462	Counseling, Health	counsel health
800473	Marital Counseling	counsel marit
800484	Pastoral Care	care pastor
800495	Support Group, Other	group other support
800517	Support Group, Sexual Assault	group sexual assault support
800528	Counseling, Family	counsel famili
800539	Hotlines, Domestic Violence	hotlin domest violenc
800561	Hotlines, Child Abuse	hotlin child abus
800572	Incest	incest
800583	Safe House	safe hous
800594	Advocacy, Domestic Violence	advocaci domest violenc
800605	Counseling, Child Abuse	counsel child abus
800616	Abuse, Domestic	domest abus
800627	Counseling, Domestic Violence	counsel domest violenc
800638	Hotlines, Rape	hotlin rape
800649	Abuse, Elder	elder abus
800660	Crisis Intervention, Abuse	intervent crisi abus
800671	Advocacy, Child Abuse	advocaci child abus
800682	Adult Protective Services	protect servic adult
800693	Child Abuse Prevention	prevent child abus
800704	Abuse, Sexual	sexual abus
800715	Information and Referral, Child Abuse	inform and child referr abus
800726	Information and Referral, Rape	inform rape and referr
800737	Family Violence	famili violenc
800748	Domestic Violence Prevention	prevent domest violenc
800759	Information and Referral, Domestic Violence	inform and referr domest violenc
800770	Counseling, Rape	counsel rape
800781	Battered Women Shelter	shelter batter women
800792	Advocacy, Rape	advocaci rape
800803	Rape	rape
800814	Caregivers, Disabled	disabl caregiv
800825	Autism	autism
800836	Vocational Assessment, Disabled	disabl vocat assess
800847	Information and Referral, Disabled	disabl inform and referr
800858	Hotlines Disabled	disabl hotlin
800869	Advocacy, Disabled	disabl advocaci
800880	Transitional Living Program, Disabled	disabl transition program live
800891	Housing, Disabled	disabl hous
800902	Transportation, Disabled	disabl transport
800913	Special Needs	special ne
800924	Residential Programs, Disabled	disabl program residenti
800935	Attention Deficit Disorder	deficit disord attent
800946	Services, Disabled	disabl servic
800957	Adult Day Care, Disabled	disabl care dai adult
800968	Dyslexia	dyslexia
800979	Visually Impaired	impair visual
800990	Independent Living Facility, Disabled	disabl independ facil live
801001	Life Skills Training, Disabled	disabl train life skill
801012	Independent Living, Disabled	disabl independ live
801023	Reading Aids	read aid
801034	Social Security	secur social
801045	Employment, Disabled	disabl employ
801056	Hearing Impaired	impair hear
801067	Learning Disabilities	learn disabl
801078	Mental Retardation	retard mental
801089	Independent Living Program, Disabled	disabl independ program live
801100	Vocational Rehabilitation, Disabled	disabl vocat rehabilit
801111	Assistive Devices	devic assist
801122	Braille	braill
801133	Education/Training, Disabled	disabl train educ
801144	Meals on Wheels	meal wheel
801155	Mental Illness	ill mental
801166	Therapeutic Riding	ride therapeut
801177	Computer Training, Adult	train comput adult
801188	Education, Elementary	educ elementari
801199	Life Skills Training, Adult	train life adult skill
801210	Truancy	truanci
801232	Head Start	head start
801243	ESL/ESOL	esl esol
801254	Advocacy, Special Needs	special advocaci ne
801265	Tutoring, Middle School	school middl tutor
801276	Certificate Programs	certif program
801287	First Aid Classes	aid class first
801298	Literacy	literaci
801309	Professional Training	train profession
801320	School, Alternative/Charter	charter school altern
801331	High School Diploma	diploma school high
801342	Homework Assistance, Middle School	homework school assist middl
801353	Information and Referral, Education	inform educ and referr
801364	Adult Education	educ adult
801375	Homework Assistance, Adult	homework assist adult
801386	Education, Middle School	educ school middl
801397	CPR Classes	class cpr
801419	Medical Training	train medic
801430	Continuing Education	educ continu
801441	GED	ged
801452	Computer Training, Youth	youth train comput
801463	Tutoring, High School	school high tutor
801474	School, Public	school public
801485	Tutoring, Elementary School	school elementari tutor
801496	Life Skills Training, Youth	youth train life skill
801507	School Private/Parochial	school parochi privat
801518	Homework Assistance, College	colleg homework assist
801529	Homework Assistance, Elementary School	homework school assist elementari
801540	Information and Referral, Special Needs	special inform ne and referr
801551	Hotlines, Special Needs	special hotlin ne
801562	Advocacy, Education	educ advocaci
801573	Life Skills Training, Teens	train life teen skill
801584	Adult Basic Education	educ basic adult
801595	Homework Assistance, High School	homework school assist high
801606	Dropout Program	dropout program
801617	Computer Training, Teen	train teen comput
801628	Education, High School	educ school high
801639	Vocational Assessment	vocat assess
801650	Hotlines, Unemployment	hotlin unemploy
801661	Hotlines, Employment Discrimination	hotlin discrimin employ
801672	Family Medical Leave Act	leav famili act medic
801683	Unemployment	unemploy
801694	Advocacy, Employment Discrimination	advocaci discrimin employ
801705	Job Search	search job
801716	Employee Rights	right employe
801727	Job Bank	bank job
801738	Information and Referral, Employment	inform and referr employ
801749	Advocacy, Unemployment	advocaci unemploy
801760	Employment, Youth/Teens	youth teen employ
801771	Job Club	club job
801782	Employment, Counseling	counsel employ
801793	Resume Assistance	resum assist
801804	Vocational Rehabilitation	vocat rehabilit
801815	Employment, Senior	senior employ
801826	Job Training	train job
801837	Business Assistance	assist busi
801848	Employment, Veterans	veteran employ
801859	Employment, Services	servic employ
801870	Business Loans	loan busi
801881	Information and Referral, Retirement Planning	retir inform and referr plan
801892	Advocacy, Credit	advocaci credit
801903	Tax Preparation Assistance	assist prepar tax
801914	Information and Referral, Financial Aid	aid inform and financi referr
801925	Credit Counseling	counsel credit
801936	Retirement	retir
801947	Hotlines, Credit	hotlin credit
801958	Financial Counseling	counsel financi
801969	Budget Counseling	counsel budget
801980	Information and Referral, Credit	inform credit and referr
801991	Financial Aid College	aid colleg financi
802002	Shelter, Families	famili shelter
802013	Hotlines, Utilities	hotlin util
802024	Clothing	cloth
802035	WIC	wic
802046	Shelter, Women	shelter women
802057	Advocacy, Utilities	advocaci util
802068	Shelter, Veterans	veteran shelter
802079	Advocacy, TAFDC	advocaci tafdc
802090	Shelter, Men	men shelter
802101	Shelter, Runaways	runawai shelter
802112	FEMA	fema
802123	Information and Referral, Food	inform food and referr
802134	Information and Referral, Shelter	inform shelter and referr
802145	Shelter, Homeless	homeless shelter
802156	Household Items	item household
802167	Financial Assistance	assist financi
802178	TAFDC	tafdc
802189	Hotlines, Shelter	hotlin shelter
802200	Furniture	furnitur
802222	Utility Assistance	assist util
802233	Information and Referral, Welfare	inform and welfar referr
802255	Advocacy, Homeless	homeless advocaci
802266	Shelter, Elderly	elderli shelter
802277	Homeless Services	servic homeless
802288	SSDI	ssdi
802299	Nutrition	nutrition
802310	Tobacco Education/Cessation, Adult	educ cessat adult tobacco
802321	Information and Referral, AIDS/HIV	inform and referr aids hiv
802332	Hotlines, AIDS/HIV	hotlin aids hiv
802343	Weight Management	weight manag
802354	Pregnancy Prevention	prevent pregnanc
802365	Yoga	yoga
802376	Hotlines, Health Insurance	hotlin health insur
802387	Information and Referral, Cancer	inform and referr cancer
802398	Childbirth Classes	class childbirth
802409	Fitness	fit
802420	Hotlines, Cancer	hotlin cancer
802431	Health Education	educ health
802442	Information and Referral, Health	inform health and referr
802453	Information and Referral, Health Insurance	inform health insur and referr
802464	AIDS/HIV, Prevention & Education	prevent educ aids hiv
802475	Tobacco Education/Cessation, Youth/Teen	youth educ teen cessat tobacco
802486	Rehabilitation, Medical	medic rehabilit
802497	Pain Management	manag pain
802508	Home Visiting	home visit
802519	Immunizations	immun
802530	Advocacy, AIDS/HIV	advocaci aids hiv
802541	First Aid	aid first
802552	Pregnancy Testing	test pregnanc
802563	Health Insurance	health insur
802574	Health Care, Children	children care health
802585	Cancer	cancer
802596	Prevention Programs, Health	prevent health program
802607	Free Health Care	free care health
802618	Hospice Care	care hospic
802629	Home Health Care	home care health
802640	STD	std
802651	Health Care, Adults	adult care health
802662	Homemaker Services	servic homemak
802673	Advocacy, Cancer	advocaci cancer
802684	AIDS/HIV, Testing & Screening	screen test aids hiv
802695	Blood Pressure	blood pressur
802706	Men's Health	health men
802717	Contraceptive Services	contracept servic
802728	Health Care, Teens	care health teen
802739	Abortion	abort
802750	Donating Blood	blood donat
802761	Family Planning	famili plan
802772	AIDS/HIV, Treatment & Support	treatment support aids hiv
802783	Advocacy, Health	advocaci health
802794	Medicare	medicar
802805	Dental Care	care dental
802816	Alternative Health	health altern
802827	Women's Health	women health
802838	Advocacy, Health Insurance	advocaci health insur
802849	Physician Referral	physician referr
802860	Early Intervention	intervent earli
802871	MassHealth/Medicaid	masshealth medicaid
802882	Health Care, Infants/Toddlers	infant care health toddler
802893	Code Violations	code violat
802904	Landlord Trainings	train landlord
802915	Hotlines, Housing	hotlin hous
802926	Home Buying	home bui
802937	Lead Paint	lead paint
802948	AIDS/HIV, Housing	hous aids hiv
802959	Loans, Housing	loan hous
802970	Housing, Affordable	hous afford
802981	Section 8 Housing	 hous section 8
802992	Public Housing	hous public
803003	Tenant Rights	right tenant
803014	Housing, Transitional	hous transition
803025	Housing, Counseling	counsel hous
803036	Housing, Financial Assistance	assist hous financi
803047	Fair Housing	hous fair
803058	Landlord Rights	right landlord
803069	Information and Referral, Housing	inform hous and referr
803080	Housing, Shelter	hous shelter
803091	Mediation, Housing	hous mediat
803102	Home Improvement/Modification	home improv modif
803113	Advocacy, Housing	advocaci hous
803124	Hotlines, Immigrants	hotlin immigr
803135	Immigrant Employment Services	servic immigr employ
803146	Refugees	refuge
803157	Translating/Interpreting	interpret translat
803168	Asylum	asylum
803179	Advocacy, Immigrants	advocaci immigr
803190	Citizenship Preparation	citizenship prepar
803201	Immigrant Rights	right immigr
803212	Immigrant Education Assistance	educ assist immigr
803223	Discrimination	discrimin
803234	Immigration/Naturalization	natur immigr
803245	Information and Referral Immigrants	inform immigr and referr
803256	Legal Assistance, Immigrants	legal immigr assist
803267	Crime Prevention	prevent crime
803278	Court	court
803289	Divorce	divorc
803300	Hotlines, Legal	legal hotlin
803311	Consumer Rights	right consum
803322	Information and Referral	inform and referr
803333	Legal Assistance, Child Custody	legal assist child custodi
803344	Legal Representation Criminal	crimin legal represent
803355	Offenders	offend
803366	Legal Services	servic legal
803377	Legal Advocacy	advocaci legal
803388	Legal Representation, Civil	civil legal represent
803399	Hotlines, Consumer Rights	hotlin right consum
803410	Benefits Law	law benefit
803421	Dual Diagnosis	dual diagnosi
803432	Advocacy, Mental Retardation	retard advocaci mental
803443	Information and Referral, Mental Health	inform health mental and referr
803454	Hotlines, Mental Health	hotlin health mental
803465	Panic Disorder	panic disord
803476	Psychiatric Services	servic psychiatr
803487	Depression/Manic Depression	manic depress
803498	Mental Health, Services	servic health mental
803509	Residential Programs, Psychiatric	psychiatr program residenti
803520	Information and Referral, Mental Retardation	retard inform mental and referr
803531	Anxiety	anxieti
803542	Advocacy, Mental Health	advocaci health mental
803553	Mental Health, Information and Education	inform educ health and mental
803564	Stress Management	stress manag
803575	Coalition/Partnership	partnership coalition
803586	Environmental Education	educ environment
803597	Planning/Funding Organizations	fund organ plan
803608	Clubhouse	clubhous
803619	Environment	environ
803630	Veterans	veteran
803641	Gay/Lesbian/Bisexual	bisexu lesbian gai
803652	Transportation, Other	other transport
803663	Membership Organizations	organ membership
803674	Parenting Education	educ parent
803685	Parenting Groups	parent group
803696	Teen Parents	teen parent
803718	Supervised Visitation	visit superv
803729	Single Parents	singl parent
803740	Hotlines, Parenting	hotlin parent
803751	Baby Items	item babi
803762	Advocacy, Parenting	advocaci parent
803773	Information and Referral, Parenting	inform and parent referr
803784	Parenting Support	parent support
803795	Mediation/Conflict Resolution, Family	conflict resolut famili mediat
803806	Adoption	adopt
803817	Hotlines, Crime	crime hotlin
803828	Police	polic
803839	Gang Prevention/Intervention	intervent prevent gang
803850	Law Enforcement	enforc law
803861	Juvenile Delinquency Prevention	prevent juvenil delinqu
803872	Public Safety	safeti public
803883	Information and Referral, Public Safety	inform safeti public and referr
803894	Hotlines, Public Safety	safeti hotlin public
803905	Crime Victim	crime victim
803916	Senior Center	senior center
803927	Adult Day Care, Senior	senior care dai adult
803938	Transportation, Senior	senior transport
803949	Hotlines, Senior	senior hotlin
803960	Elder Services	elder servic
803971	Information and Referral, Senior	senior inform and referr
803982	Advocacy, Senior	senior advocaci
803993	Caregivers	caregiv
804004	Elder Abuse	elder abus
804015	Soccer	soccer
804026	Sports/Recreation, Middle School	recreat school sport middl
804037	Parks	park
804048	Sports/Recreation, Adult	recreat sport adult
804059	Swimming Lessons	lesson swim
804070	Water Safety	safeti water
804081	Sports/Recreation, Elementary School	recreat school sport elementari
804092	Sports/Recreation, High School	recreat school sport high
804103	Baseball	basebal
804114	Basketball	basketbal
804136	Sports/Recreation, Preschool	preschool recreat sport
804147	Gymnastics	gymnast
804158	Swimming	swim
804169	Football	footbal
804180	Prevention Program, Alcohol Addiction	prevent alcohol addict program
804191	Residential Program, Drug Abuse	drug abus program residenti
804202	Support Group, Drug Abuse	group drug abus support
804213	Advocacy, Alcohol Addiction	alcohol advocaci addict
804224	Gambling Addiction	addict gambl
804235	Counseling, Drug Abuse	counsel drug abus
804246	Counseling, Alcohol Addiction	counsel alcohol addict
804257	Halfway House, Alcohol Addiction	halfwai alcohol addict hous
804268	Support Group, Alcohol Addiction	alcohol group addict support
804279	Recovery, Drug Abuse	drug abus recoveri
804290	Methadone Treatment	methadon treatment
804301	Prevention Program, Drug Abuse	prevent drug abus program
804312	Hotlines, Drug Abuse	hotlin drug abus
804323	Information and Referral, Alcohol Addiction	inform alcohol and addict referr
804334	Recovery, Alcohol Addiction	alcohol addict recoveri
804345	Residential Program, Alcohol Addiction	alcohol addict program residenti
804356	Halfway House, Drug Abuse	halfwai drug abus hous
804367	Detoxification	detoxif
804378	Advocacy, Drug Abuse	advocaci drug abus
804389	Information and Referral, Drug Abuse	inform drug and referr abus
804400	Hotlines, Alcohol Addiction	alcohol hotlin addict
804411	Peer Leadership	leadership peer
804422	Babysitter Training	train babysitt
804433	Infants/Toddlers	infant toddler
804444	Advocacy, Youth	youth advocaci
804466	After School Programs, High School	school after high program
804477	Drug Prevention Programs Middle School	prevent school drug program middl
804488	Drug Prevention Programs High School	prevent school drug high program
804499	Youth Programs, 05 - 10	 youth  program 05 10
804510	Mediation/Conflict Resolution, Youth	youth conflict resolut mediat
804521	Mentoring, Youth	youth mentor
804532	Camp, 05 - 10	  camp 05 10
804543	Runaways	runawai
804554	Hotlines, Youth	youth hotlin
804565	Teen Programs	teen program
804576	Youth Programs, 14 - 19	youth  program  14 19
804587	After School Programs, Middle School	school after program middl
804598	Camp, 14 - 19	 camp  14 19
804609	Mentoring, Teens	teen mentor
804620	After School Programs, Elementary School	school after elementari program
804631	Information and Referral, Youth	youth inform and referr
804642	Children's Playgroups	children playgroup
804664	Youth Programs, 11 - 15	youth  program  11 15
804675	Preschool Programs	preschool program
804686	Camp, 11 - 15	 camp  11 15
804697	Youth Programs, 17 - 22	youth   program 17 22
804708	Drug Prevention Programs Elementary School	prevent school drug elementari program
804719	Child Care, 05 - 12	 care child  05 12
804730	Kinship Care	care kinship
804741	Child Care, 0 - 05	  care child 0 05
804785	Music Lessons, Youth/Teen	youth lesson teen music
804796	Visual Arts Classes, Youth/Teen	youth class teen art visual
804807	Performance Art Classes, Youth/Teen	youth class teen art perform
804818	Senior Recreation/Social	recreat senior social
804829	Hockey	hockei
804840	Music Lessons, Adult	lesson music adult
804851	Visual Art Classes, Adult	adult class art visual
804862	Performance Art Classes, Adult	class art perform adult
804873	Prenatal	prenat
804884	Affordable Health Care	care health afford
804895	Uninsured Services	servic uninsur
804906	Support Group, Cancer	group cancer support
804917	Housing, Elderly	elderli hous
804928	Food	food
804939	Soup Kitchen	soup kitchen
804950	College Preparation, Adult	colleg prepar adult
804961	Financial Literacy	literaci financi
804972	Counseling, Group	counsel group
804983	Social/Singles Groups	singl social group
804994	Computer/Technology Providers	technologi provid comput
805005	Eating Disorder	eat disord
805016	Diabetes	diabet
802244	Food Pantry	pantri food
805027	Martial Arts	art martial
801221	Child Care Training	child train care
801408	Home Based Child Care Certification	certif home base child care
804774	Information and Referral, Child Care	referr inform child and care
804653	Child Care Vouchers	voucher child care
804763	Home Based Child Care 0 - 05	  home base child care 0 05
804752	Home Based Child Care 05 - 12	 home base child  care 05 12
803707	Foster Care	foster care
800550	Abuse, Child	abus child
805038	Prescription Drugs	drug prescript
800011	Dance	danc
802211	Food Stamps	stamp food
\.


--
-- Data for Name: category_search_log; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY category_search_log (search_id, session_id, ip_address, "timestamp", search_phrase, referer, phrase_stems, top_score) FROM stdin;
36648	66.92.77.160.22211109	66.92.77.160	2009-02-21 22:47:56.844043	test	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	test	1.00
36649	129.63.96.191.1432109	129.63.96.191	2009-03-03 14:08:00.640147	hj	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	hj	0.00
36650	209.235.0.248.16312109	209.235.0.248	2009-03-31 16:17:55.23427	legal	http://mvh.omacneil.testing123.net/	legal	1.00
36651	209.235.0.248.16312109	209.235.0.248	2009-03-31 16:18:38.101805	law	http://mvh.omacneil.testing123.net/	law	1.00
36652	96.237.29.220.17312109	96.237.29.220	2009-03-31 17:02:15.050425	youth program	http://mvh.omacneil.testing123.net/	youth	1.00
36653	96.237.29.220.17312109	96.237.29.220	2009-03-31 17:03:19.900033	revolving museum	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=804466;search_phrase=youth%20program	museum revolv	0.50
36655	75.150.72.65.10134109	75.150.72.65	2009-05-13 10:25:28.563008	community software lab	http://mvh.omacneil.testing123.net/	commun lab softwar	0.33
36656	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:38:57.844143	homeless agencies	http://mvh.omacneil.testing123.net	agenc homeless	0.50
36657	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:40:06.844511	homeless shelters	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803113;search_phrase=homeless%20agencies	homeless shelter	1.00
36658	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:42:32.419976	community software lab	http://mvh.omacneil.testing123.net	commun lab softwar	0.33
36659	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:45:11.917957	community software lab agency	http://mvh.omacneil.testing123.net/	agenc commun lab softwar	0.25
36660	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:47:01.076452	dancing schools in lowell	http://mvh.omacneil.testing123.net	danc lowel school	0.33
36661	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:48:50.556147	help in lowell area	http://mvh.omacneil.testing123.net	area help lowel	0.33
36662	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:50:01.96591	president of community teamwork inc.	http://mvh.omacneil.testing123.net	commun inc presid teamwork	0.25
36663	75.150.72.65.11134109	75.150.72.65	2009-05-13 11:51:48.472254	director of community teamwork inc.	http://mvh.omacneil.testing123.net	commun director inc teamwork	0.25
36665	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:35:23.863837	shelters	http://mvh.omacneil.testing123.net	shelter	1.00
36666	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:37:58.927737	community sofware lab	http://mvh.omacneil.testing123.net/	commun lab sofwar	0.33
36667	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:38:16.267654	community software lab	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	commun lab softwar	0.33
36668	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:41:16.905134	dancing classes	http://mvh.omacneil.testing123.net/	class danc	1.00
36669	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:42:58.779427	cti 	http://mvh.omacneil.testing123.net/	cti	0.00
36670	75.150.72.65.12134109	75.150.72.65	2009-05-13 12:43:10.433408	communit team work	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	communit team work	0.00
36674	96.237.27.110.13134109	96.237.27.110	2009-05-13 13:58:44.420437	shelters	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	shelter	1.00
36676	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:03:43.65019	dance	http://mvh.omacneil.testing123.net/	danc	1.00
36677	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:04:59.284704	c.t.i	http://mvh.omacneil.testing123.net/	c t	0.00
36679	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:34:35.795373	community software lab	http://mvh.omacneil.testing123.net/	commun lab softwar	0.33
36680	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:39:23.104295	dancing schools in lowell	http://mvh.omacneil.testing123.net/	danc lowel school	0.33
36681	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:41:28.006932	cti	http://mvh.omacneil.testing123.net/	cti	0.00
36682	96.237.27.110.14134109	96.237.27.110	2009-05-13 14:44:21.856848	karate lesson	http://mvh.omacneil.testing123.net/	karat lesson	0.50
36683	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:07:46.160532	community services	http://mvh.omacneil.testing123.net/	commun servic	1.00
36684	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:10:20.213185	electric company	http://mvh.omacneil.testing123.net/	compani electr	0.50
36685	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:13:28.482865	dance class	http://mvh.omacneil.testing123.net/	class danc	1.00
36686	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:15:06.268875	employment	http://mvh.omacneil.testing123.net/	employ	1.00
36690	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:44:27.373244	lowell homeless shelter	http://mvh.omacneil.testing123.net/	homeless lowel shelter	0.67
36691	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:47:04.122625	Lowell Ma utilities services	http://mvh.omacneil.testing123.net/	lowel ma servic util	0.25
36692	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:49:11.376855	Information about MVHub	http://mvh.omacneil.testing123.net/	inform mvhub	0.50
36693	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:51:49.984481	google	http://mvh.omacneil.testing123.net/	googl	0.00
36694	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:52:41.809641	Information about MVHub website	http://mvh.omacneil.testing123.net/	inform mvhub websit	0.33
36695	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:54:51.307236	Lowell MA dance school	http://mvh.omacneil.testing123.net/	danc lowel ma school	0.25
36696	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:57:05.263995	wall street journal	http://mvh.omacneil.testing123.net/	journal street wall	0.00
36697	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:57:32.154773	stock market	http://mvh.omacneil.testing123.net/	market stock	0.00
36698	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:57:46.597832	brokers	http://mvh.omacneil.testing123.net/	broker	0.00
36699	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:57:56.803857	money	http://mvh.omacneil.testing123.net/	monei	0.00
36700	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:58:09.743895	careers	http://mvh.omacneil.testing123.net/	career	1.00
36701	74.104.19.137.11274109	74.104.19.137	2009-05-27 11:59:23.137174	citi inc	http://mvh.omacneil.testing123.net/	citi inc	0.50
36702	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:01:19.430024	neihbors in need mailing address	http://mvh.omacneil.testing123.net/	address mail ne neihbor	0.25
36703	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:03:00.438256	fees for youth build and the cost	http://mvh.omacneil.testing123.net/	and build cost fee youth	0.40
36704	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:05:50.954809	when is the anne sullivan center open	http://mvh.omacneil.testing123.net/	ann center open sullivan	0.25
36705	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:32:31.819065	homeless shelters	http://mvh.omacneil.testing123.net/	homeless shelter	1.00
36706	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:34:58.43848	fuel assistance	http://mvh.omacneil.testing123.net/	assist fuel	1.00
36707	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:36:28.000953	complaints	http://mvh.omacneil.testing123.net/	complaint	1.00
36708	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:38:03.086292	about mvhub	http://mvh.omacneil.testing123.net/	mvhub	0.00
36709	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:39:04.173858	home website	http://mvh.omacneil.testing123.net/	home websit	0.50
36710	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:39:28.026296	browse website	http://mvh.omacneil.testing123.net/	brows websit	0.00
36711	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:39:53.724713	mvhub agency	http://mvh.omacneil.testing123.net/	agenc mvhub	0.00
36712	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:40:37.402227	dancing lessons	http://mvh.omacneil.testing123.net/	danc lesson	1.00
36713	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:42:50.485138	information about mvhub	http://mvh.omacneil.testing123.net/	inform mvhub	0.50
36714	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:44:04.799492	cti management	http://mvh.omacneil.testing123.net/	cti manag	0.50
36715	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:44:40.149669	director for c.t.i	http://mvh.omacneil.testing123.net/	c director t	0.00
36716	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:44:56.227736	director for CTI	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	cti director	0.00
36717	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:45:28.594382	director for community teamwork incorporated	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	commun director incorpor teamwork	0.25
36718	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:46:46.088551	address for Neighbors in Need	http://mvh.omacneil.testing123.net/	address ne neighbor	0.33
36719	74.104.19.137.12274109	74.104.19.137	2009-05-27 12:48:03.132253	mailing address for Neighbors in need	http://mvh.omacneil.testing123.net	address mail ne neighbor	0.25
36720	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:10:00.735939	shelter	http://mvh.omacneil.testing123.net/	shelter	1.00
36721	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:12:10.771199	fuel assistance	http://mvh.omacneil.testing123.net/	assist fuel	1.00
36722	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:14:17.857511	ball room dance class	http://mvh.omacneil.testing123.net/	ball class danc room	0.50
36723	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:16:06.290778	dance teacher	http://mvh.omacneil.testing123.net/	danc teacher	0.50
36724	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:16:53.86601	cti	http://mvh.omacneil.testing123.net/	cti	0.00
36725	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:17:05.489277	cti management	http://mvh.omacneil.testing123.net/	cti manag	0.50
36726	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:18:32.224975	neighbors in need	http://mvh.omacneil.testing123.net/	ne neighbor	0.50
36727	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:19:36.968783	youthbuild	http://mvh.omacneil.testing123.net/	youthbuild	0.00
36728	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:19:43.062141	youth build	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	build youth	0.50
36729	74.104.19.137.13274109	74.104.19.137	2009-05-27 13:20:46.009237	ann sullivan center	http://mvh.omacneil.testing123.net/	ann center sullivan	0.33
36730	76.118.140.125.17274109	76.118.140.125	2009-05-27 17:23:39.026633	test	http://mvh.omacneil.testing123.net/	test	1.00
36731	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:04:57.512457	homeless shelter	http://mvh.omacneil.testing123.net/	homeless shelter	1.00
36732	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:05:08.531782	homeless shelter in lowell	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	homeless lowel shelter	0.67
36733	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:07:22.727768	how to pay a gas bill	http://mvh.omacneil.testing123.net/	bill ga pai	0.33
36734	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:11:49.427207	lowell dancing classes/ tap	http://mvh.omacneil.testing123.net/	class danc lowel tap	0.50
36736	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:12:55.013267	dancing classes/ schools	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	class danc school	0.67
36737	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:13:30.351197	dancing classes	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=800011;search_phrase=dancing%20classes%2F%20schools	class danc	1.00
36738	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:14:01.093505	ymca	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=804807;search_phrase=dancing%20classes	ymca	0.00
36740	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:14:51.512357	schools for dancing	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=504884	danc school	0.50
36741	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:15:16.245434	tap dancing classes	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	class danc tap	0.67
36742	74.94.147.165.1055109	74.94.147.165	2009-06-05 10:18:02.556026	whats the name of the person in charge on cti	http://mvh.omacneil.testing123.net/	charg cti name person what	0.00
36747	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:10:50.469022	homeless	http://mvh.omacneil.testing123.net	homeless	1.00
36748	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:14:17.129406	help to pay expenses	http://mvh.omacneil.testing123.net/	expens help pai	0.33
36751	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:24:26.276337	job	http://mvh.omacneil.testing123.net/	job	1.00
36753	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:34:07.797376	anne sullivan	http://mvh.omacneil.testing123.net/	ann sullivan	0.00
36754	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:51:30.891937	homeless services	http://mvh.omacneil.testing123.net	homeless servic	1.00
36755	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:53:36.732512	fuel assistance	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802255;search_phrase=homeless%20services	assist fuel	1.00
36756	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:54:11.525024	car assistance	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	assist car	0.50
36757	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:54:32.934213	heating oil assistance	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	assist heat oil	0.67
36758	74.94.147.165.1155109	74.94.147.165	2009-06-05 11:56:14.058567	dance classes	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802222;search_phrase=heating%20oil%20assistance	class danc	1.00
36759	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:00:03.560394	Neighbors In Need	http://mvh.omacneil.testing123.net/	in ne neighbor	0.33
36762	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:01:13.759426	Youth Build	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	build youth	0.50
36766	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:46:11.678598	homeless shelters	http://mvh.omacneil.testing123.net	homeless shelter	1.00
36767	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:47:44.271381	heating company	http://mvh.omacneil.testing123.net/	compani heat	0.50
36768	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:51:00.409694	dancing classes	http://mvh.omacneil.testing123.net/	class danc	1.00
36770	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:52:44.358176	UTCH	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	utch	0.00
36771	74.94.147.165.1255109	74.94.147.165	2009-06-05 12:52:56.442131	united teen equality center	http://mvh.omacneil.testing123.net/	center equal teen unit	0.25
36776	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:26:34.288887	shelters	http://mvh.omacneil.testing123.net	shelter	1.00
36777	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:29:50.275092	company that will help you pay on behind gas bills	http://mvh.omacneil.testing123.net/	behind bill compani ga help pai you	0.29
36778	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:30:43.34165	help to pay on behing gass bills	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802222;search_phrase=company%20that%20will%20help%20you%20pay%20on%20behind%20gas%20bills	behe bill gass help pai	0.20
36779	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:31:15.754872	gas bills	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	bill ga	0.50
36780	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:31:42.565334	help with overdue gas bills	http://mvh.omacneil.testing123.net/	bill ga help overdu	0.50
36781	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:36:11.059918	dancing classes in lawrence	http://mvh.omacneil.testing123.net/	class danc lawrenc	0.67
36782	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:36:24.005387	dancing classes 	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	class danc	1.00
36783	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:37:39.247939	dance studio in lawrence to take dance classes	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=800011;search_phrase=dancing%20classes%20in%20lawrence	class danc lawrenc studio take	0.40
36784	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:39:27.499256	the contact person for commmunity team work	http://mvh.omacneil.testing123.net/	commmun contact person team work	0.00
36785	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:39:33.291724	 commmunity team work	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	commmun team work	0.00
36786	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:40:23.686	contact community team work	http://mvh.omacneil.testing123.net/	commun contact team work	0.25
36787	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:41:39.852072	mailing address for neighbors in needs	http://mvh.omacneil.testing123.net/	address mail ne neighbor	0.25
36788	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:42:13.33292	neighbors in needs	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801254;search_phrase=mailing%20address%20for%20neighbors%20in%20needs	ne neighbor	0.50
36789	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:43:06.971836	ann sullivan	http://mvh.omacneil.testing123.net/	ann sullivan	0.00
36790	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:43:19.421666	ann sullivan openina	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	ann openina sullivan	0.00
36791	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:43:29.774588	ann sullivan opening	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	ann open sullivan	0.33
36792	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:43:50.576896	Ann Sulllivan	http://mvh.omacneil.testing123.net/	ann sulllivan	0.00
36793	74.94.147.165.1355109	74.94.147.165	2009-06-05 13:57:19.274504	homeless shelter	http://mvh.omacneil.testing123.net/	homeless shelter	1.00
36794	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:01:05.82637	help payment of bills	http://mvh.omacneil.testing123.net/	bill help payment	0.33
36795	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:08:34.272102	dancing	http://mvh.omacneil.testing123.net/	danc	1.00
36796	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:08:45.062296	dance class	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	class danc	1.00
36797	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:12:14.158396	art	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=804807;search_phrase=dancing	art	1.00
36798	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:14:07.513876	food	http://mvh.omacneil.testing123.net/	food	1.00
36799	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:14:57.722606	cti	http://mvh.omacneil.testing123.net/	cti	0.00
36800	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:15:08.659609	community team work	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	commun team work	0.33
36801	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:16:51.149743	furniture assistance	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=800198;search_phrase=community%20team%20work	assist furnitur	0.50
36802	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:17:43.778458	neighbors in need	http://mvh.omacneil.testing123.net/	ne neighbor	0.50
36805	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:18:44.053577	youth build	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	build youth	0.50
36807	74.94.147.165.1455109	74.94.147.165	2009-06-05 14:55:51.674801	adult homeless shelter 	http://mvh.omacneil.testing123.net/	adult homeless shelter	0.67
36813	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:02:21.689341	bills	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	bill	1.00
36814	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:05:47.678396	dance	http://mvh.omacneil.testing123.net/	danc	1.00
36819	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:08:45.647013	G.R.I.P	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	g i p r	0.00
36820	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:08:49.099849	GRIP	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	grip	0.00
36821	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:09:13.8326	UTEC	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	utec	0.00
36822	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:09:38.76203	United Teen Equality Center	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	center equal teen unit	0.25
36825	74.94.147.165.1555109	74.94.147.165	2009-06-05 15:13:55.695376	Youth Build	http://mvh.omacneil.testing123.net/cgi-bin/mvhub/guide.pl	build youth	0.50
\.


--
-- Data for Name: data_entry_agency; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY data_entry_agency (id, agency_id, agency_name, update_date, updated_by, update_status, agency_alias, address1, address2, city, state, zip, main_phone, toll_free_phone, hotline, fax, website, email, mission, hours, other_info, mail_address1, mail_address2, mail_city, mail_state, mail_zip, chief, title, parent_organization, resource_library, resource_library_description, meeting_space, meeting_space_description, staff_internet_access, contact_first_name, contact_last_name, contact_phone, contact_fax, contact_email, contact_email_available, tty_tdd) FROM stdin;
37	102387	Jericho Road Lawrence, Inc.	2007-07-10 15:51:31.315021	300022	2	\N	190 Academy Road,	\N	North Andover	MA	01845	978-604-8803	\N	\N	\N	http://www.jerichoroadlawrence.org/	jkulash@aol.com	Jericho Road Lawrence's mission is to address economic inequities by matching the needs of non profit organizations in Lawrence, Massachusetts to the talents and resources of the surrounding suburban communities through the use of volunteers who offer no cost, customized professional services to those non profit organizations focused on helping individuals and families become economically self-sustaining.	Monday thru Friday, 8:30 am - 5 pm.	\N	\N	\N	\N	\N	\N	Joan Kulash	Executive Director	\N	No	\N	Yes	North Parish Unitarian Universalist Church, 190 Academy Road, North Andover.	Yes	Joan	Kulash	978-604-8803	\N	jkulash@aol.com	Yes	\N
57	102442	Adelante Youth Center	2007-07-16 13:02:20.14866	300022	2	\N	Adelante Youth Center	40 Lawrence Street	Lawrence	MA	01841	978-794-3407	\N	\N	978-794-3407	http://www.adelanteyouthcenter.org	info@adelanteyouthcenter.org	Adelante! (Spanish for "moving forward")  provides Lawrence's urban youth with opportunities to enhance their education, leading them to greater economic success in adulthood.	Monday thru Friday, 9am - 5:30pm	\N	\N	\N	\N	\N	\N	David Hildt	Executive Director	\N	Yes	\N	Not Applicable	\N	Yes	David	Hildt	978-687-9432	978-794-3407	dhildt@adelanteyouthcenter.org	Yes	\N
62	102420	Youth Empowerment Services Inc.	2007-07-17 10:03:58.149575	300022	2	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	The mission of Youth Empowerment Services Inc., YES, is to create opportunities for youth of all backgrounds and adults, who work with youth, to learn peaceable, healthy relationship skills to be able to communicate effectively; resolve conflicts nonviolently; solve problems creatively and cooperatively; and collaborate in leadership.	Mon-Fri: 8:AM-5:PM	\N	\N	\N	\N	\N	\N	Barbara Hildt	Director	\N	Not Applicable	\N	No	\N	Yes	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N
71	102398	Career Resources Corporation	2007-07-17 11:16:42.58528	300022	2	\N	165 Winter Street	\N	Haverhill	MA	01830	978-374-9122	\N	\N	978-374-1532	http://www.crc-mass.org/	contact@crc-mass.org	The Career Resources Corporation's Mission is to to empower people to be as self-sufficient, self-determining, and successful as possible. Individuals with developmental disabilities are supported in their own jobs and homes and integrated into community life, recognizing that they're unique, valued people and that communities are enriched by their participation.	Mon-Fri: 8:AM-5:PM	\N	\N	\N	\N	\N	\N	Brad Howell	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Brad	Howell	978-374-9122	978-374-1532	bhowell@crc-mass.org	Yes	\N
74	102398	Career Resources Corporation	2007-07-17 16:24:16.550679	300000	3	\N	165 Winter Street	\N	Haverhill	MA	01830	978-374-9122	\N	\N	978-374-1532	http://www.crc-mass.org/	contact@crc-mass.org	The Career Resources Corporation's Mission is to to empower people to be as self-sufficient, self-determining, and successful as possible. Individuals with developmental disabilities are supported in their own jobs and homes and integrated into community life, recognizing that they're unique, valued people and that communities are enriched by their participation.	Mon-Fri: 8:AM-5:PM	\N	\N	\N	\N	\N	\N	Brad Howell	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Brad	Howell	978-374-9122	978-374-1532	bhowell@crc-mass.org	Yes	\N
69	102409	CLASS, Inc.	2007-07-17 11:14:21.953165	300022	2	\N	1 Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	http://www.classinc.org/	agraciano@classinc.org	Through collaboration with individuals, families and the community, CLASS empowers people with disabilities by providing services that promote personal choice, growth and full participation in their communities.	Mon & Tues: 8:AM-4:PM, Wed: 8:AM-4:30PM, Th: 8:AM-4:PM, Fri: 8:AM-3:30PM	\N	\N	\N	\N	\N	\N	Robert Harris	President/CEO	\N	No	\N	Not Applicable	\N	Yes	Bob	Harris	978-975-8587 ext:1202	978-975-0498	bharris@classsinc.org	Yes	\N
77	102508	Lazarus House Ministries    	2007-07-18 09:26:50.449739	300022	2	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	Lazarus House, Inc. is a spiritually based organization that welcomes all in the name of God. It provides a continuum of care encompassing, but not limited to food, shelter, clothing, advocacy, job training, medical and dental care, a listening ear, a welcoming heart, the restoration of dignity and self respect to the poor and needy.	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N	\N	\N	\N	H. Bridget Shaheen	Executive Director	\N	No	\N	No	\N	Yes	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	\N
79	102475	Bread and Roses	2007-07-18 11:45:36.846742	300022	2	\N	58 Newbury Street	\N	Lawrence	MA	01840	978-681-8768	\N	\N	\N	www.breadandroseslawrence.org	breadandroses@verizon.net	Bread & Roses' primary purpose is to ease the hunger of people in our community  by providing a well-balanced, nutritious evening meal.	Monday thru Thrusday, 5 pm - 7:30 pm, Saturday, 5 pm - 7:30 pm.	\N	\N	\N	\N	\N	\N	Bob Lanzoni	Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Bob	Lanzoni	978-681-8768	\N	breadandroses@verizon.net	Yes	\N
80	102431	Greater Lawrence Educational Collaborative	2007-07-18 12:19:29.806576	300022	1	\N	480 Broadway	\N	Methuen	MA	01844	978-685-3000	\N	\N	978-689-7466	http://www.glec.org/index.php	rmcardle@glec.org	The Greater Lawrence Educational Collaborative (GLEC) is a public educational agency serving the Massachusetts school districts of Andover, Boxford, Haverhill, Lawrence, Lowell, Methuen, Middleton, North Andover, Topsfield and the Greater Lawrence Technical School. Through the cooperation of our member districts, we provide educational programs and services to students throughout northeastern Massachusetts.	Mon-Fri: 8:AM-4:PM	\N	\N	\N	\N	\N	\N	Robert McArdle	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Robert	McArdle	978-685-3000 ext.120	978-689-7466	rmcardle@glec.org	Yes	\N
82	102530	YMCA-Haverhill	2007-07-19 15:54:13.521385	300022	2	\N	81 Winter Street	\N	Haverhill	MA	01830	978-374-0506	\N	\N	978-373-0710	http://www.northshoreymca.org/pages/11491_haverhill_ymca.cfm	fullert@northshoreymca.org	The YMCA of the North Shore is committed to the values of caring, honesty, respect and responsibility.  Our YMCA provides all children, adults and families, regardless of income, with opportunities to develop a healthy spirit, mind and body. Through organized activities called programs we are able to work toward this.  Our goals are to nurture the healthy development of children and teens, to strengthen families, and to make our community a healthier, safer and better place to live.	Mon-Fri: 8:30AM-9:PM, Sat: 8:AM-8:PM, Sun: 9:AM-5:PM.	\N	\N	\N	\N	\N	\N	Tracy Fuller	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Tracy	Fuller	978-374-0506	978-373-0710	fullert@northshoreymca.org	Yes	\N
85	102552	Boy Scouts of America	2007-07-26 12:14:40.063502	300022	2	\N	36 AMESBURY RD	\N	HAVERHILL	MA	01830	978-372-0591	\N	\N	978-373-9134	http://www.yccbsa.org	info@yccbsa.org	The mission of the Boy Scouts of America is to prepare young people to make ethical and moral choices over their lifetimes by instilling in them the values of the Scout Oath and Law.	Mon-Fri 9am-5pm	\N	\N	\N	\N	\N	\N	Chuck Marvin	Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chuck	Marvin	978-372-0591	978-373-9134	cmarvin@bsamail.org	Yes	\N
87	102563	Boys & Girls Clubs of Lower Merrimack Valley	2007-07-26 12:59:01.649743	300022	2	\N	18 Maple St.	\N	Salisbury	MA	01952	978-462-7003	\N	\N	978-462-7003	http://www.boysandgirlscluboflmv.com/	jimkeenan@boysandgirlscluboflmv.com	To inspire and enable all young people, especially those that need us the most, to realize their full potential as productive, responsible and caring citizens.	2:30pm-7:00pm, Monday-Friday. Saturday programs available 9:00am -2:00pm	\N	\N	\N	\N	\N	\N	Jim Keenan	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Jim	Keenan	978-462-7003	978-462-7003	jimkeenan@boysandgirlscluboflmv.com	Yes	\N
88	102552	Boy Scouts of America, Yankee Clipper Council	2007-08-02 15:33:04.720328	300000	3	\N	36 AMESBURY RD	\N	HAVERHILL	MA	01830	978-372-0591	\N	\N	978-373-9134	http://www.yccbsa.org	info@yccbsa.org	The mission of the Boy Scouts of America is to prepare young people to make ethical and moral choices over their lifetimes by instilling in them the values of the Scout Oath and Law.	Mon-Fri 8am-5pm	\N	\N	\N	\N	\N	\N	Chuck Marvin	Finance Director	Boy Scouts of America	Not Applicable	\N	Not Applicable	\N	Yes	Chuck	Marvin	978-372-0591	978-373-9134	cmarvin@bsamail.org	Yes	\N
90	102453	Lawrence Boys and Girls Club	2007-08-07 10:19:12.007767	300022	2	\N	136 Water Street	\N	Lawrence	MA	01841	978-683-2747	\N	\N	978-725-5989	http://www.lawrencebgc.com	mfischer@lawrencebgc.com	The mission of the  Boys & Girls Club of Lawrence is to maintain a Club for all youth without distinction of race, sex, color or creed; to provide behavior guidance, and to promote the health, social, educational, vocational and character development of all its members.	Mon - Fri: Summer Months: 9:45AM - 3:45PM\r\nMon - Fri: School Months: 3:00PM - 7:00PM for ages 7 thru 9, and 3:00 PM - 10:30PM for ages 10+.	\N	\N	\N	\N	\N	\N	Markus Fischer	Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Markus	Fischer	978-689-0608 (Ext.111)	978-725-5989	mfischer@lawrencebgc.com	Yes	\N
92	102607	Andover youth services	2007-08-21 12:55:02.08645	300022	2	\N	36 Bartlett, Towns office  3rd. floor	\N	Andover	MA	01810	978-623-8221	\N	\N	\N	\N	\N	Out of our 300 square feet of space, the AYS runs programs year-round for the 6,500 young people of Andover. \r\n\r\nA dedicated staff of 4 full time employees and several seasonal workers provides recreational, educational, social and support programs for over 6,000 participants annually. \r\n\r\nWe've been going strong for thirteen years and its because we know that it is all about the kids. In an effort to better serve the needs of the community, the AYS and Andover Youth Foundation have been trying to fundraise for a Youth Center to give a home to this flourishing program. \r\n\r\nPlease surf around our website and find out what we have to offer and let us know if you have any ideas for programs.	School Hours \r\nTuesday through Friday: 3pm to 7:00pm \r\nSaturday/Sunday: 12pm to 7pm \r\nClosed Mondays \r\nSummer Hours \r\nTuesday through Sunday: 12pm to 7:30pm \r\nClosed Mondays	\N	\N	\N	\N	\N	\N	Bill Fahey	Executive Director	\N	No	\N	No	\N	No	Bill	Fahey	978-623-8221	\N	wfahey@andoverma.gov	Yes	\N
94	102541	Eric's Test Agency	2008-06-02 18:58:16.170633	300044	2	\N	Happiness Lane	\N	Lowell	MA	01853	978-934-4350	\N	\N	\N	\N	\N	mission	hours	\N	\N	\N	\N	\N	\N	God	Mitlon's Ideation	\N	No	\N	No	\N	Yes	eric	bryant	978-934-4350	\N	ebryant@thecsl.org	No	\N
95	103168	A	2008-06-02 19:15:23.487695	300044	2	\N	B	\N	C	MA	02138	458-456-2234	\N	\N	\N	\N	\N	o	o	\N	\N	\N	\N	\N	\N	i	ho	\N	No	\N	No	\N	No	e	b	978-934-4350	\N	ebryant@thecsl.org	No	\N
96	102607	Andover youth services	2008-06-04 21:00:30.637364	300000	3	\N	36 Bartlett, Towns office  3rd. floor	\N	Andover	MA	01810	978-623-8221	\N	\N	\N	\N	\N	Out of our 300 square feet of space, the AYS runs programs year-round for the 6,500 young people of Andover. \r\n\r\nA dedicated staff of 4 full time employees and several seasonal workers provides recreational, educational, social and support programs for over 6,000 participants annually. \r\n\r\nWe've been going strong for thirteen years and its because we know that it is all about the kids. In an effort to better serve the needs of the community, the AYS and Andover Youth Foundation have been trying to fundraise for a Youth Center to give a home to this flourishing program. \r\n\r\nPlease surf around our website and find out what we have to offer and let us know if you have any ideas for programs.	School Hours \r\nTuesday through Friday: 3pm to 7:00pm \r\nSaturday/Sunday: 12pm to 7pm \r\nClosed Mondays \r\nSummer Hours \r\nTuesday through Sunday: 12pm to 7:30pm \r\nClosed Mondays	\N	\N	\N	\N	\N	\N	Bill Fahey	Executive Director	\N	No	\N	No	\N	No	Bill	Fahey	978-623-8221	\N	wfahey@andoverma.gov	Yes	\N
97	103234	eric test II	2008-06-05 15:07:45.249403	300044	2	\N	a	\N	l	MA	02138	694-238-4832	\N	\N	\N	\N	\N	blah	bine	\N	\N	\N	\N	\N	\N	me	a	\N	No	\N	No	\N	Yes	eric	bryant	238-234-1234	\N	ebryant@thecsl.org	No	\N
98	103234	eric test II	2008-06-05 15:08:43.921473	300000	3	\N	a	\N	l	MA	02138	694-238-4832	\N	\N	\N	\N	\N	blah	bine	\N	\N	\N	\N	\N	\N	me	a	\N	No	\N	No	\N	Yes	eric	bryant	238-234-1234	\N	ebryant@thecsl.org	No	\N
99	103245	eric test III	2008-06-05 15:16:50.027797	300044	2	\N	a	\N	a	MA	02138	987-987-4257	\N	\N	\N	\N	\N	987-987-4257	987-987-4257	\N	\N	\N	\N	\N	\N	me	a	\N	No	\N	No	\N	No	eric	bryant	987-987-4257	\N	ebryant@thecsl.org	No	\N
100	103245	eric test III	2008-06-05 15:18:34.436994	300000	3	\N	a	\N	a	MA	02138	987-987-4257	\N	\N	\N	\N	\N	987-987-4257	987-987-4257	\N	\N	\N	\N	\N	\N	me	a	\N	No	\N	No	\N	No	eric	bryant	987-987-4257	\N	ebryant@thecsl.org	No	\N
102	103267	awer	2008-06-05 17:47:13.575517	300044	2	\N	c	\N	aser	MA	09877	978-986-8676	\N	\N	\N	\N	\N	asdf	adsf	\N	\N	\N	\N	\N	\N	aser	adsf	\N	Yes	asdf	No	\N	Yes	eric	bryant	989-897-8678	\N	ebryant@thecsl.org	No	\N
105	103256	Mental Health Resources Plus	2008-06-06 19:12:09.621861	300044	2	MHRP	One Canal St.	\N	Lawrence	MA	01840	978-685-6609	\N	\N	978-685-1941	\N	mhrp@verizon.net	ADD	Monday-Friday, 9AM-5PM	\N	\N	\N	\N	\N	\N	Kathleen O'Keefe	Executive Director	\N	No	\N	No	\N	Yes	Kathleen	O'Keefe	978-685-6609	978-685-1941	kathleenokeefe@verizon.net	No	\N
107	103278	Chester's Testing Club	2008-06-09 20:17:17.403681	300044	2	Testing 4 U	123 Testing lane	\N	Lowell	MA	01852	978-555-1212	\N	\N	\N	\N	\N	asedf	asdf	\N	\N	\N	\N	\N	\N	Chester Teste	Chief Tester	\N	No	\N	No	\N	Not Applicable	er	ab	978-934-4350	\N	ebryant@thecsl.org	No	\N
108	103278	Chester's Testing Club	2008-06-09 20:19:33.241156	300000	3	Testing 4 U	123 Testing lane	\N	Lowell	MA	01852	978-555-1212	\N	\N	\N	\N	\N	asedf	asdf	\N	\N	\N	\N	\N	\N	Chester Teste	Chief Tester	\N	No	\N	No	\N	Not Applicable	er	ab	978-934-4350	\N	ebryant@thecsl.org	No	\N
110	102464	Big Brothers and Big Sisters of Greater Lawrence, Inc.	2008-06-10 14:57:39.430009	300044	1	Big Brothers Big Sisters	439 South Union Street, Suite 1118	\N	Lawrence	MA	01843	978-687-1370	\N	\N	978-688-6175	http://www.bigbrothersbigsisters.net	info19@bigbrothersbigsisters.net	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Bill	Santore	978-687-1370 x15	978-688-6175	bill@bigbrothersbigsisters.net	\N	\N
111	103289	Methuen Arlington Neighborhood, Inc.	2008-06-12 12:53:48.014215	300044	2	MAN, Inc.	141 Tenney Street	\N	Methuen	MA	01844	978-691-5645	\N	\N	978-691-5645	\N	methuenarlington@yahoo.com	The Methuen Arlington Neighborhood is a densely populated, low to moderate income area located close to the City of Lawrence. Sharing the same problems as Lawrence - poverty, lack of employment opportunities, and social ills - this area is able to receive some financial assistance due to the fact that Methuen is a small cities mini-entitlement area. \r\n\r\nTo begin formally working on improving the quality of life in the area, neighborhood residents formed a Community Developement Corporation (CDC), the Methuen Arlington Neighborhood, Inc. (M.A.N., Inc.). In March of 1995, this organization became incorporated as M.A.N. Inc. The neighborhood is also the site of the Department of Justice's Weed and Seed Program.\r\n\r\nM.A.N. Inc's prime target area for the purpose of representation and service is within the Town of Methuen, primarily within the borders of the Spicket River, Horne Street, Lawrence Street, and the Lawrence City Line of Center Street.	ADD	\N	141 Tenney Street	PO Box 715	Methuen	MA	01844	Pamela Marchand	Program Director	Greater Lawrence Community Action Council, Inc (GLCAC, Inc.)	Yes	Library for kids in the after school homework center.	Yes	Occasionally. After an elevator to the second floor is installed, we will be better able to accommodate local groups.	Yes	Pamela	Marchand	978-691-5645	\N	methuenarlington@yahoo.com	Yes	\N
112	103300	YouthBuild Lawrence	2008-06-13 13:58:21.882578	300044	1	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N
115	103311	Lawrence Methuen Community Coalition	2008-06-16 21:36:21.479346	300044	2	LMCC	112 A East Haverhill Street	\N	Lawrence	MA	01841	978-975-8793	\N	\N	978-975-3294	http://www.lmcconline.com/aboutus/index.htm	mail@lmcconline.com	The mission of the Lawrence/Methuen Community Coalition is to provide an accessible continuum of services that fosters and preserves the health, safety and well being of individuals and families, thus supporting communities in which they through following the principles of Family Support, the prevention of substance abuse, the prevention of child maltreatment and increasing awareness of the prevention of domestic and family violence.	M-F ADD	\N	\N	\N	\N	\N	\N	Harold Magoon	Director	Greater Lawrence Community Action Council Inc.	No	\N	Yes	Conference room seats 20 persons	Yes	Harold	Magoon	978-975-8793	978-975-3294	hmagoon@lmcconline.com	No	\N
118	103322	Family Service, Inc.	2008-06-16 21:38:25.479086	300044	2	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6600	\N	\N	978-327-6601	www.familyserviceinc.com	info@familyserviceinc.com	Family Services' mission is to strengthen individuals and families by building skills for successful home, school, work, and community life, has led the agency to implement a range of prevention and treatment programs that are evidence-based and result in self-sufficiency and improved economic and health outcomes for its clients. Over 20 programs are currently offered in the areas of child welfare, parent education, youth development, employee assistance, mental health treatment and suicide prevention.	M-F 8:00am-5:00pm	\N	\N	\N	\N	\N	\N	Kay Berthold Frishman	Executive Director	\N	Yes	\N	Yes	Conference room seats 75 persons.	Yes	Lori	Howe	978-327-6600	978-327-6601	lhowe@familyserviceinc.com	No	\N
119	103322	Family Service, Inc.	2008-06-17 09:51:03.400325	300000	3	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6600	\N	\N	978-327-6601	www.familyserviceinc.com	info@familyserviceinc.com	Family Services' mission is to strengthen individuals and families by building skills for successful home, school, work, and community life, has led the agency to implement a range of prevention and treatment programs that are evidence-based and result in self-sufficiency and improved economic and health outcomes for its clients. Over 20 programs are currently offered in the areas of child welfare, parent education, youth development, employee assistance, mental health treatment and suicide prevention.	M-F 8:00am-5:00pm	\N	\N	\N	\N	\N	\N	Kay Berthold Frishman	Executive Director	\N	Yes	\N	Yes	Conference room seats 75 persons.	Yes	Lori	Howe	978-327-6600	978-327-6601	lhowe@familyserviceinc.com	No	\N
120	103333	Arlington Community Trabajando	2008-06-17 13:43:50.581905	300044	1	ACT	\N	\N	\N	\N	\N	\N	\N	\N	\N	www.actinc.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N
121	103366	eric test 4	2008-06-30 17:17:02.322168	300044	2	\N	asdf	\N	asdf	NH	99999	222-222-2222	\N	\N	\N	\N	\N	adsf	adsf	\N	\N	\N	\N	\N	\N	adsf	adsf	\N	No	\N	No	\N	Yes	adsf	asdf	888-999-7867	\N	ebryant@thecsl.org	No	\N
122	103366	eric test 4	2008-06-30 17:18:55.042355	300000	3	\N	asdf	\N	asdf	NH	99999	222-222-2222	\N	\N	\N	\N	\N	adsf	adsf	\N	\N	\N	\N	\N	\N	adsf	adsf	\N	No	\N	No	\N	Yes	adsf	asdf	888-999-7867	\N	ebryant@thecsl.org	No	\N
\.


--
-- Data for Name: data_entry_program; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY data_entry_program (id, program_id, agency_id, program_name, updated_by, update_date, update_status, program_alias, address1, address2, city, state, zip, main_phone, toll_free_phone, hotline, fax, website, email, short_program_description, long_program_description, eligibility, fees_charged, fees_explained, program_schedule, hours, languages, appointment, bring, bus_lines, parking, transportation, handicapped_accessible, child_care_provided, child_care_description, wait_list, wait_list_description, other_information, mail_address1, mail_address2, mail_city, mail_state, mail_zip, volunteer_opportunities, volunteer_information, resource_library, resource_library_description, meeting_space, meeting_space_description, news_letter, news_letter_description, public_net_access, public_net_access_description, contact_first_name, contact_last_name, contact_phone, contact_fax, contact_email, contact_email_available, area_type, area_data, tty_tdd, areas_served_description, languages_description, online_registration) FROM stdin;
10	506226	102387	Pro-bono professional assistance	300022	2007-07-10 17:49:36.207117	2	\N	190 Academy Road,	\N	North Andover	MA	01845	978-604-8803	\N	\N	\N	http://www.jerichoroadlawrence.org/	jkulash@aol.com	Pro-bono professional expertise	Jericho Road Lawrence provides Lawrence non-profits with professional assistance on specific projects.  Some examples include strategic planning, board development, technical assistance, business development, etc.	Must be a non-profit that provides a charitable service to the City of Lawrence.  Should be a 501c3, or qualify for being a 501c3	Free	Our services are strictly pro-bono	Open year round	5 days/week	English	Yes	Call 978-604-8803 for an appointment	\N	\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Call 978-604-8803 for volunteer opportunities	No	\N	Yes	North Parish Unitarian Universalist Church, 190 Academy Road, North Andover.	Yes	Quarterly	Yes	www.jerichoroadlawrence.org	Joan	Kulash	978-604-8803	\N	jkulash@aol.com	Yes	TOWN	Lawrence, MA	\N	The non-profit can reside in a town/city outside of Lawrence, but must dedicate a suabstantial portion of its charitable activities to the City of Lawrence	\N	No
11	506215	102387	DEFUNCT: Springboard	300022	2007-07-11 09:42:25.106556	1	\N	190 Academy Road,	\N	North Andover	MA	01852	978-604-8803	\N	\N	\N	http://www.jerichoroadlawrence.org/index.html	jkulash@aol.com	\N	\N	\N	\N	\N	\N	Monday thru Friday, 8:30 am - 5 pm.	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	Joan	Kulash	978-604-8803	\N	jkulash@aol.com	\N	\N	\N	\N	\N	\N	\N
26	506237	102409	Employment and Production Services	300022	2007-07-11 10:33:45.892352	1	\N	1 Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	http://www.classinc.org/	agraciano@classinc.org	Employment Services provides job training and employment opp	Employment Services provides job training and employment opportunities in a variety of settings. Individuals work in our production facilities, at job sites, and at competitive employment placements in area businesses. Through production facilities, individuals are provided with a variety of paid work and job training opportunities.	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	\N	\N	\N	\N	Joy	Sabolevski	978-975-8587	978-975-0498	jsabolevski@classsinc.org	Yes	\N	\N	\N	\N	\N	\N
27	506248	102409	Habilitative Services	300022	2007-07-11 10:36:10.028504	1	\N	1 Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	http://www.classinc.org/	agraciano@classinc.org	\N	Habilitative Services works with individuals challenged by extensive clinical, medical and therapeutic needs. The services we provide are designed to enhance the quality of their lives.	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	\N	\N	\N	\N	Joy	Sabolevski	978-975-8587	978-975-0498	jsabolevski@classsinc.org	Yes	\N	\N	\N	\N	\N	\N
28	506259	102409	Clinical & Case Management Support	300022	2007-07-11 10:37:28.500828	1	\N	1 Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	http://www.classinc.org/	agraciano@classinc.org	Provides individuals with a variety of clinical support.	CLASS provides individuals with a variety of clinical support. Services include daily on-site health services provided by our nursing staff, planning and treatment for behavioral needs, occupational therapy provided by certified personnel and communication services to learn effective usage of language and other communication skills.	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	\N	\N	\N	\N	Joy	Sabolevski	978-975-8587	978-975-0498	jsabolevski@classsinc.org	Yes	\N	\N	\N	\N	\N	\N
29	506270	102409	Transportation Services	300022	2007-07-11 10:38:38.747071	1	\N	1 Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	http://www.classinc.org/	agraciano@classinc.org	\N	Transportation Services provides daily passenger service to our individuals who reside throughout the Merrimack Valley, North Shore and Metro North areas.	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	\N	\N	\N	\N	Joy	Sabolevski	978-975-8587	978-975-0498	jsabolevski@classsinc.org	Yes	\N	\N	\N	\N	\N	\N
32	506303	102420	Help Increase the Peace Programs	300022	2007-07-11 15:16:44.63126	1	HIPP	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
33	506314	102420	Developing Peaceable Schools and Communities with Youth	300022	2007-07-11 15:17:14.258902	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
34	506325	102420	HIPP for Camp Staff to Create a Peaceable Camp Community	300022	2007-07-11 15:17:54.265726	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
35	506336	102420	Empowering Youth to Reach Goals	300022	2007-07-11 15:18:36.121661	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	Affirming and Developing Assests	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
37	506358	102420	Youth and Adult Collaboration Skills Workshops	300022	2007-07-11 15:19:46.925903	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
39	506380	102420	Interpersonal Skills for Success	300022	2007-07-11 15:20:42.659585	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
40	506380	102420	Interpersonal Skills for Success	300022	2007-07-11 15:21:39.891045	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
45	506446	102431	Lifeways	300022	2007-07-16 08:41:28.018014	1	\N	476 Broadway	\N	Methuen	MA	01844	978-685-3000 ext. 503	\N	\N	978-687-9348	http://www.glec.org/index.php	mcardle@glec.org	\N	\N	\N	\N	\N	\N	School hours: Mon-Fri: 9:AM-3:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Margaret	Moran	978-685-3000 ext.501	978-687-9348	\N	\N	\N	\N	\N	\N	\N	\N
49	506479	102431	Home Training Services	300022	2007-07-16 09:50:10.71774	1	\N	476 Broadway Street	\N	Methuen	MA	01844	978-685-3000 ext.518	\N	\N	978-687-9648	http://www.glec.org/index.php	\N	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:AM-4:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Marilyn	Kelly	978-685-3000 ext.518	978-687-9648	\N	\N	\N	\N	\N	\N	\N	\N
51	506468	102431	SCALE & VISIONS	300022	2007-07-16 09:56:50.461932	1	\N	476 Broadway	\N	Methuen	MA	01844	978-685-3000 ext.302	\N	\N	978-687-9324	http://www.glec.org/index.php	\N	An experiential school-to-work transition program/	The VISIONS/SCALE Program is an experiential school-to-work transition program serving students with special needs and young adults. The VISIONS program focuses on vocational training and experience so as to maximize choices for individuals, while SCALE is a school-to-work transition program for students assessed as needing further education in their transition to adult life.	\N	\N	\N	\N	School hours:Mon-Fri:7:45AM-2:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Mary Ellen	Dixon	978-685-3000 ext.201	978-687-9324	\N	\N	\N	\N	\N	\N	\N	\N
52	506457	102431	Essex Academy	300022	2007-07-16 10:20:14.750889	1	\N	476 Broadway Street	\N	Methuen	MA	01844	978-685-3000 ext.302	\N	\N	978-687-9324	http://www.glec.org/index.php	mcardle@glec.org	A therapeutic high school for students, grades 6-12.	Essex Academy is a therapeutic high school for student, grades 6-12, who need highly individualized instruction as well as frequent therapeutic support.	\N	\N	\N	\N	School hours: Mon-Fri:7:45AM-2:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Greg	Curtis	978-685-3000 ext.301	978-687-9324	\N	\N	\N	\N	\N	\N	\N	\N
53	506490	102431	Multicultural "Enrichment" Program	300022	2007-07-16 11:03:54.208553	1	\N	480 Broadway	\N	Methuen	MA	01844	978-685-3000 ext.123	\N	\N	\N	http://www.glec.org/index.php	\N	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:AM-4:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	James	McConaughy	978-685-3000 ext.123	\N	\N	\N	\N	\N	\N	\N	\N	\N
54	506501	102431	Transportation Services	300022	2007-07-16 11:11:13.516193	1	\N	480 Broadway Street	\N	Methuen	MA	01844	978-685-3000 ext.133	\N	\N	978-681-8849	http://www.glec.org/index.php	\N	\N	\N	\N	\N	\N	\N	Mon-Fri: 7:AM-4:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Richard	Prue	978-685-3000 ext.133	978-681-8849	\N	\N	\N	\N	\N	\N	\N	\N
55	506512	102431	Professional Development	300022	2007-07-16 11:13:27.73389	1	\N	476 Broadway	\N	Methuen	MA	01844	978-685-3000 ext.110	\N	\N	978-689-7466	http://www.glec.org/index.php	mcardle@glec.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:AM-4:PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Tammy	Dobrosielski	978-685-3000 ext.116	978-689-7466	\N	\N	\N	\N	\N	\N	\N	\N
56	506292	102442	School Success	300022	2007-07-17 10:51:14.957573	1	\N	Adelante Youth Center	40 Lawrence Street	Lawrence	MA	01841	978-794-3407	\N	\N	978-794-3407	http://www.adelanteyouthcenter.org	info@adelanteyouthcenter.org	7th-8th grade High School prep programs	\N	\N	\N	\N	\N	Monday thru Friday, 9am - 5:30pm	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	\N	\N	\N	\N	David	Hildt	978-687-9432	978-794-3407	dhildt@adelanteyouthcenter.org	\N	\N	\N	\N	\N	\N	\N
57	506424	102431	Collaborative Alternative School	300022	2007-07-17 10:54:18.74361	1	CAS	476 Broadway	\N	Methuen	MA	01844	978-685-3000 x402	\N	\N	978-794-9462	http://www.glec.org/index.php	\N	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-3:00PM	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Steve	Burnham	978-685-3000 ext.402	978-794-9368	\N	\N	\N	\N	\N	\N	\N	\N
58	506281	102409	Arc of Greater Lawrence	300022	2007-07-17 12:03:28.055823	1	\N	1 Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	http://www.classinc.org/	agraciano@classinc.org	Supports parents and guardians with disabled children.	The Arc of Greater Lawrence works with families to provide information, guidance, and tools to develop a vision for the future. The Arc supports parents and guardians in setting personal goals that address the financial and legal needs of their sons and daughters with disabilities.	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	\N	\N	\N	\N	Joy	Sabolevski	978-975-8587	978-975-0498	jsabolevski@classsinc.org	Yes	\N	\N	\N	\N	\N	\N
59	506347	102420	Empowering Youth with Effective Leadership Skills	300022	2007-07-17 13:51:19.716841	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
60	506369	102420	Service Leadership Opportunities and Skills for Youth	300022	2007-07-17 15:38:44.359054	1	\N	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	barbara@empowertheyouth.org	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	\N	\N	\N	\N	Barbara	Hildt	978-388-3647	\N	barbara@empowertheyouth.org	Yes	\N	\N	\N	\N	\N	\N
61	506556	102508	Job Training	300022	2007-07-18 10:24:06.775767	1	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	EVERYWHERE		\N	\N	\N	\N
62	506567	102508	Transitional Housing	300022	2007-07-18 10:24:27.163275	1	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	EVERYWHERE		\N	\N	\N	\N
64	506578	102508	Sheltering	300022	2007-07-18 11:58:07.113145	1	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	EVERYWHERE		\N	\N	\N	\N
65	506589	102508	Volunteer Program	300022	2007-07-19 13:15:33.394356	1	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	\N		\N	\N	\N	\N
66	506600	102508	Food Services	300022	2007-07-19 14:32:29.679039	1	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	EVERYWHERE		\N	\N	\N	\N
67	506611	102508	Soup Kitchen	300022	2007-07-19 14:37:47.096215	1	\N	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/default.asp	mary@lazarushouse.org	\N	\N	\N	\N	\N	\N	Mon-Fri: 8:30AM-4:30PM	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	H. Bridget	Shaheen	978-689-8575 Ext 200	978-682-7004	bridget@lazarushouse.org	Yes	EVERYWHERE		\N	\N	\N	\N
69	506633	102464	Mentoring Program	300022	2007-07-23 11:45:00.589723	1	\N	439 South Union street, suite 1118	\N	Lawrence	MA	01843	978-687-1370	\N	\N	978-688-6175	http://www.bigbrothersbigsisters.net	info19@bigbrothersbigsisters.net	\N	\N	Ages 6-14, Boys and Girls	\N	\N	1 year Program, 2-4 times a month (meet)	\N	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Bill	Santore	978-687-1370(Ext.15)	978-688-6175	bill@bigbrothersbigsisters.net	\N	EVERYWHERE		\N	\N	\N	\N
71	506842	102607	Vacation-Programs	300022	2007-08-21 15:34:08.343262	1	\N	36 Bartlett, Towns office  3rd. floor	\N	Andover	MA	01810	978-623-8241	\N	\N	\N	andoveryouthservices.com	ays@andoverma.gov	\N	\N	\N	\N	\N	Winter, February, April  Vacations	School Hours \r\nTuesday through Friday: 3pm to 7:00pm \r\nSaturday/Sunday: 12pm to 7pm \r\nClosed Mondays \r\nSummer Hours \r\nTuesday through Sunday: 12pm to 7:30pm \r\nClosed Mondays	English	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	Suzie	Allen	978-623-8241	\N	sallen@andoverma.gov	Yes	EVERYWHERE		\N	\N	\N	\N
72	506853	102607	Expeditions	300022	2007-08-21 15:36:28.480702	1	\N	36 Bartlett, Towns office  3rd. floor	\N	Andover	MA	01810	978-623-8241	\N	\N	978-623-8221	andoveryouthservices.com	ays@andoverma.gov	\N	\N	\N	\N	\N	\N	School Hours \r\nTuesday through Friday: 3pm to 7:00pm \r\nSaturday/Sunday: 12pm to 7pm \r\nClosed Mondays \r\nSummer Hours \r\nTuesday through Sunday: 12pm to 7:30pm \r\nClosed Mondays	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	\N	\N	\N	\N	Glenn	Wilson	978-623-8241	978-623-8221	wfahey@andoverma.gov	Yes	EVERYWHERE		\N	\N	\N	\N
73	506864	102607	The Zone	300022	2007-08-21 15:40:36.92338	1	\N	West Middle School 70  Shawsheen Road	\N	Andover	MA	01810	978-623-8700	\N	\N	978-623-8720	andoveryouthservices.gov	ays@andoverma.gov	\N	\N	\N	\N	\N	\N	School Hours \r\nTuesday through Friday: 3pm to 7:00pm \r\nSaturday/Sunday: 12pm to 7pm \r\nClosed Mondays \r\nSummer Hours \r\nTuesday through Sunday: 12pm to 7:30pm \r\nClosed Mondays	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	36 Bartlett Street, 3rd floor Towns Office	\N	Andover	MA	01810	\N	\N	No	\N	No	\N	\N	\N	\N	\N	Suzie	Allen	978-623-8241	9778-623-8221	sallen@andoverma.gov	Yes	EVERYWHERE		\N	\N	\N	\N
74	506226	102387	Pro-bono professional assistance	300000	2008-02-11 09:09:31.754468	3	\N	190 Academy Road,	\N	North Andover	MA	01845	978-604-8803	\N	\N	\N	http://www.jerichoroadlawrence.org/	jkulash@aol.com	Pro-bono professional expertise	Jericho Road Lawrence provides Lawrence non-profits with free professional assistance on specific projects.  Some examples include strategic planning, board development, technical assistance, business development, etc.	Must be a non-profit that provides a charitable service to the City of Lawrence.  Should be a 501c3, or qualify for being a 501c3	Free	Our services are strictly pro-bono	Open year round	5 days/week	English	Yes	Call 978-604-8803 for an appointment	\N	\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Call 978-604-8803 for volunteer opportunities	No	\N	Yes	North Parish Unitarian Universalist Church, 190 Academy Road, North Andover.	Yes	Quarterly	Yes	www.jerichoroadlawrence.org	Joan	Kulash	978-604-8803	\N	jkulash@aol.com	Yes	TOWN	Lawrence, MA	\N	The non-profit can reside in a town/city outside of Lawrence, but must dedicate a suabstantial portion of its charitable activities to the City of Lawrence	\N	No
75	507854	102541	Eric's Test Program	300044	2008-06-02 18:56:32.334416	1	ETP	Happiness Lane	\N	Lowell	MA	01853	978-934-4350	\N	\N	\N	\N	\N	Hoover Repair	Multi-Cultural Vaccum Cleaner Services	Young persons	Free	\N	\N	hours	\N	Yes	\N	32 Andover!DELIM!03 South Lowell	\N	\N	Not Applicable	Not Applicable	\N	Yes	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	eric	bryant	978-934-4350	\N	ebryant@thecsl.org	\N	STATE	MA	\N	\N	\N	Not Applicable
76	507854	102541	Eric's Test Program	300044	2008-06-02 18:56:46.002796	1	ETP	Happiness Lane	\N	Lowell	MA	01853	978-934-4350	\N	\N	\N	\N	\N	Hoover Repair	Multi-Cultural Vaccum Cleaner Services	Young persons	Free	\N	\N	hours	\N	Yes	\N	32 Andover!DELIM!03 South Lowell	\N	\N	Not Applicable	Not Applicable	\N	Yes	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	eric	bryant	978-934-4350	\N	ebryant@thecsl.org	\N	STATE	MA	\N	\N	\N	Not Applicable
77	506435	102431	Education Promoting Inclusive Classrooms	300044	2008-06-04 15:42:48.225616	1	EPIC	480 Broadway Street	\N	Methuen	MA	01844	978-685-3000 ext. 119	\N	\N	978-557-9368	http://www.glec.org/index.php	mcardle@glec.org	\N	\N	\N	\N	\N	\N	School hours: Vary by location	\N	\N	\N		\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	Bernie	Pierce	978-685-3000 ext.122	978-557-9368	\N	\N	EVERYWHERE		\N	\N	\N	\N
78	507876	103234	w	300044	2008-06-05 15:10:38.299207	2	\N	a	\N	l	MA	02138	694-238-4832	\N	\N	\N	\N	\N	a	c	d	Free	\N	d	bine	English	Yes	\N		\N	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Not Applicable	\N	Limited	\N	eric	bryant	238-234-1234	\N	ebryant@thecsl.org	No	STATE	MA	\N	\N	\N	No
79	507887	103245	mi	300044	2008-06-05 15:20:58.53398	2	\N	a	\N	a	MA	02138	987-987-4257	\N	\N	\N	\N	\N	a	cas	asdfe	Free	\N	asdf	adsf	English!DELIM!Khmer	No	\N	19 Summer St. / Westgate	\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Not Applicable	\N	Limited	\N	eric	bryant	987-987-4257	\N	ebryant@thecsl.org	No	EVERYWHERE		\N	\N	\N	No
85	507964	103278	a	300044	2008-06-09 20:20:15.142985	2	\N	123 Testing lane	\N	Lowell	MA	01852	978-555-1212	\N	\N	\N	\N	\N	se	sgsg	asdf	Free	\N	asdf	sdfr	Portuguese	Not Applicable	adsf	21 Andover Shuttle	\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Yes	\N	Yes	\N	eric	bryant	978-934-4350	\N	ebryant@thecsl.org	No	EVERYWHERE		\N	\N	\N	Yes
88	507920	103278	Testing web apps	300044	2008-06-09 20:23:04.74987	2	smoke out bug program	123 Testing lane	\N	Lowell	MA	01852	978-555-1212	\N	\N	\N	\N	\N	asdf	adsf	adsf	Sliding Scale	\N	adsf	asdf	Portuguese	No	\N	18 Riverside	\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Not Applicable	\N	Yes	\N	eric	bryant	978-934-4350	\N	ebryant@thecsl.org	No	EVERYWHERE		\N	\N	\N	No
90	508217	103366	asdfwe	300044	2008-06-30 17:19:41.755161	2	\N	asdf	\N	asdf	NH	99999	222-222-2222	\N	\N	\N	\N	\N	asdf	adsfc	sadfd	Sliding Scale	\N	adsf	adsf	Khmer	Yes	\N		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Yes	\N	adsf	asdf	888-999-7867	\N	ebryant@thecsl.org	No	EVERYWHERE		\N	\N	\N	No
91	508228	103366	asdfete	300044	2008-06-30 18:31:04.332972	1	\N	asdf	\N	asdf	NH	99999	222-222-2222	\N	\N	\N	\N	\N	asdf	adsf	adsf	Free	\N	adsf	adsf	Portuguese	No	\N		\N	\N	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	\N	Yes	\N	adsf	asdf	888-999-7867	\N	ebryant@thecsl.org	No	\N		\N	\N	\N	No
\.


--
-- Data for Name: data_entry_status; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY data_entry_status (status_id, status_name) FROM stdin;
1	Saved
2	Sent for agency approval
3	Approved
\.


--
-- Data for Name: date_agency_created; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY date_agency_created (agency_id, date_created) FROM stdin;
102167	2007-05-30
102178	2007-06-06
102200	2007-06-14
102189	2007-06-14
102211	2007-06-18
102222	2007-06-19
102387	2007-07-10
102486	2007-07-12
102420	2007-07-17
102552	2007-08-02
102398	2007-07-17
102662	2007-09-24
102519	2007-07-18
102640	2007-09-19
102673	2007-10-02
102585	2007-08-09
102695	2007-10-03
102574	2007-07-31
102618	2007-09-12
102596	2007-08-15
102684	2007-10-02
102728	2007-10-23
102739	2007-10-30
102750	2007-10-30
102761	2007-10-30
102772	2007-11-01
102783	2007-11-05
102827	2007-12-03
102838	2007-12-18
102849	2008-01-04
102860	2008-01-05
102871	2008-01-08
102882	2008-01-10
102893	2008-01-11
102915	2008-01-22
102926	2008-01-30
102937	2008-02-21
102948	2008-02-26
102959	2008-02-27
102970	2008-02-29
102981	2008-02-29
102992	2008-02-29
103003	2008-03-02
103047	2008-03-06
103058	2008-03-07
103069	2008-03-12
103080	2008-03-13
103091	2008-03-14
103102	2008-03-19
103113	2008-04-04
103124	2008-04-08
103135	2008-04-23
103146	2008-05-30
103157	2008-06-02
103223	2008-06-05
103256	2008-06-06
103289	2008-06-12
103311	2008-06-16
103322	2008-06-16
103344	2008-06-24
103355	2008-06-30
103377	2008-07-10
103388	2008-07-10
103399	2008-07-17
103410	2008-08-08
103421	2008-08-08
103432	2008-08-18
103443	2008-08-26
103454	2008-09-03
103465	2008-11-12
103476	2008-12-20
103487	2008-12-29
103498	2009-01-14
103971	2009-06-05
103982	2009-06-05
\.


--
-- Data for Name: date_program_created; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY date_program_created (program_id, date_created) FROM stdin;
506754	2007-08-07
506743	2007-08-02
506732	2007-08-02
506534	2007-07-17
506545	2007-07-17
506523	2007-07-17
506996	2007-09-24
507073	2007-10-09
506094	2007-06-19
506908	2007-09-19
506688	2007-07-25
506699	2007-07-25
506160	2007-07-03
506710	2007-07-25
506226	2007-07-10
507062	2007-10-04
507040	2007-10-04
507051	2007-10-04
506776	2007-08-09
506787	2007-08-09
506798	2007-08-09
506028	2007-06-12
506941	2007-09-21
506963	2007-09-21
506127	2007-06-23
506875	2007-08-24
506886	2007-08-24
506677	2007-07-24
506974	2007-09-21
506952	2007-09-21
507029	2007-10-03
506721	2007-08-01
506204	2007-07-09
506072	2007-06-18
506820	2007-08-20
506831	2007-08-20
506897	2007-09-12
506039	2007-06-14
506655	2007-07-23
506050	2007-06-18
506061	2007-06-18
506105	2007-06-19
506006	2007-06-01
506644	2007-07-23
507139	2007-10-23
507150	2007-10-30
507161	2007-10-30
507183	2007-11-05
507194	2007-11-05
507249	2007-11-26
507271	2007-12-03
507282	2007-12-13
507293	2007-12-13
507304	2007-12-14
507315	2007-12-14
507348	2008-01-04
507359	2008-01-05
507370	2008-01-07
507381	2008-01-10
507392	2008-01-11
507436	2008-01-14
507458	2008-01-22
507469	2008-01-30
507480	2008-02-01
507491	2008-02-12
507513	2008-02-21
507524	2008-02-26
507535	2008-02-27
507546	2008-02-27
507557	2008-03-02
507623	2008-03-06
507634	2008-03-06
507645	2008-03-06
507656	2008-03-07
507667	2008-03-07
507678	2008-03-12
507689	2008-03-14
507700	2008-03-14
507722	2008-03-28
507733	2008-04-08
507744	2008-04-23
507755	2008-04-28
507766	2008-05-08
507777	2008-05-09
507788	2008-05-16
507799	2008-05-22
507810	2008-05-23
507832	2008-05-30
507843	2008-06-02
507865	2008-06-05
507909	2008-06-08
507931	2008-06-09
507942	2008-06-09
507953	2008-06-09
507975	2008-06-10
507986	2008-06-10
507997	2008-06-11
508008	2008-06-12
508019	2008-06-12
508030	2008-06-13
508041	2008-06-13
508063	2008-06-17
508074	2008-06-17
508085	2008-06-18
508096	2008-06-19
508107	2008-06-19
508118	2008-06-20
508129	2008-06-20
508140	2008-06-20
508151	2008-06-20
508162	2008-06-20
508173	2008-06-20
508184	2008-06-24
508195	2008-06-24
508206	2008-06-25
508239	2008-07-02
508250	2008-07-02
508261	2008-07-02
508272	2008-07-02
508283	2008-07-02
508294	2008-07-02
508305	2008-07-03
508316	2008-07-03
508327	2008-07-03
508338	2008-07-03
508349	2008-07-03
508360	2008-07-03
508371	2008-07-03
508382	2008-07-03
508393	2008-07-03
508404	2008-07-03
508415	2008-07-03
508426	2008-07-03
508437	2008-07-03
508448	2008-07-03
508481	2008-07-03
508492	2008-07-03
508503	2008-07-03
508514	2008-07-03
508525	2008-07-03
508536	2008-07-03
508547	2008-07-07
508558	2008-07-07
508569	2008-07-07
508580	2008-07-07
508591	2008-07-07
508602	2008-07-08
508613	2008-07-08
508624	2008-07-08
508635	2008-07-09
508646	2008-07-09
508657	2008-07-09
508668	2008-07-09
508679	2008-07-09
508690	2008-07-09
508701	2008-07-09
508712	2008-07-10
508723	2008-07-10
508734	2008-07-10
508745	2008-07-10
508756	2008-07-10
508767	2008-07-10
508778	2008-07-10
508789	2008-07-10
508800	2008-07-10
508811	2008-07-10
508822	2008-07-10
508833	2008-07-10
508844	2008-07-10
508855	2008-07-10
508866	2008-07-10
508877	2008-07-10
508888	2008-07-10
508899	2008-07-11
508910	2008-07-11
508921	2008-07-11
508932	2008-07-11
508943	2008-07-11
508954	2008-07-11
508965	2008-07-12
508976	2008-07-12
508987	2008-07-12
508998	2008-07-13
509009	2008-07-13
509020	2008-07-14
509031	2008-07-14
509042	2008-07-15
509053	2008-07-17
509064	2008-07-17
509075	2008-07-17
509086	2008-07-17
509097	2008-07-17
509108	2008-07-18
509119	2008-07-18
509130	2008-08-04
509141	2008-08-04
509163	2008-08-08
509174	2008-08-08
509185	2008-08-15
509196	2008-08-18
509207	2008-08-26
509218	2008-08-26
509229	2008-08-26
509240	2008-08-27
509251	2008-08-27
509262	2008-08-27
509273	2008-09-03
509284	2008-09-05
509295	2008-10-01
509306	2008-10-27
509317	2008-10-31
509328	2008-10-31
509339	2008-11-12
509350	2008-12-02
509361	2008-12-02
509372	2008-12-02
509383	2008-12-20
509394	2008-12-29
509405	2009-01-05
509416	2009-01-05
509427	2009-01-05
509438	2009-01-14
509515	2009-06-05
509526	2009-06-05
\.


--
-- Data for Name: heading; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY heading (heading_name) FROM stdin;
Arts/Culture/Entertainment
City/Town Services
Community/Civic
Counseling
Crisis/Abuse/Rape
Disabled/Handicapped
Education/Training
Employment
Financial Education/Credit
Food/Shelter & Financial Assistance
Health Education/Fitness/Well-being
Health/Health Insurance
Housing
Immigrant Services
Legal/Consumer Protection
Mental Health
Other
Parenting/Family
Public Safety
Senior/Elderly
Sports/Recreation
Substance Abuse/Addiction
Youth
\.


--
-- Data for Name: heading_category; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY heading_category (category_id, heading_name) FROM stdin;
800000	Arts/Culture/Entertainment
800022	Arts/Culture/Entertainment
800033	Arts/Culture/Entertainment
800044	Arts/Culture/Entertainment
800055	Arts/Culture/Entertainment
800066	Arts/Culture/Entertainment
800077	Arts/Culture/Entertainment
800088	City/Town Services
800099	City/Town Services
800110	City/Town Services
800121	City/Town Services
800132	City/Town Services
800143	City/Town Services
800154	City/Town Services
800165	City/Town Services
800176	City/Town Services
800187	City/Town Services
800198	Community/Civic
800209	Community/Civic
800220	Community/Civic
800231	Community/Civic
800242	Community/Civic
800253	Community/Civic
800264	Community/Civic
800275	Community/Civic
800143	Community/Civic
800286	Community/Civic
800297	Community/Civic
800308	Community/Civic
800319	Counseling
800330	Counseling
800341	Counseling
800352	Counseling
800363	Counseling
800374	Counseling
800385	Counseling
800396	Counseling
800407	Counseling
800418	Counseling
800429	Counseling
800440	Counseling
800451	Counseling
800462	Counseling
800473	Counseling
800484	Counseling
800495	Counseling
800517	Counseling
800528	Counseling
800539	Crisis/Abuse/Rape
800561	Crisis/Abuse/Rape
800572	Crisis/Abuse/Rape
800583	Crisis/Abuse/Rape
800594	Crisis/Abuse/Rape
800605	Crisis/Abuse/Rape
800616	Crisis/Abuse/Rape
800627	Crisis/Abuse/Rape
800638	Crisis/Abuse/Rape
800649	Crisis/Abuse/Rape
800660	Crisis/Abuse/Rape
800671	Crisis/Abuse/Rape
800682	Crisis/Abuse/Rape
800693	Crisis/Abuse/Rape
800704	Crisis/Abuse/Rape
800715	Crisis/Abuse/Rape
800726	Crisis/Abuse/Rape
800737	Crisis/Abuse/Rape
800748	Crisis/Abuse/Rape
800759	Crisis/Abuse/Rape
800770	Crisis/Abuse/Rape
800781	Crisis/Abuse/Rape
800792	Crisis/Abuse/Rape
800803	Crisis/Abuse/Rape
800517	Crisis/Abuse/Rape
800814	Disabled/Handicapped
800825	Disabled/Handicapped
800836	Disabled/Handicapped
800847	Disabled/Handicapped
800858	Disabled/Handicapped
800869	Disabled/Handicapped
800880	Disabled/Handicapped
800891	Disabled/Handicapped
800902	Disabled/Handicapped
800913	Disabled/Handicapped
800924	Disabled/Handicapped
800935	Disabled/Handicapped
800946	Disabled/Handicapped
800957	Disabled/Handicapped
800968	Disabled/Handicapped
800979	Disabled/Handicapped
800990	Disabled/Handicapped
801001	Disabled/Handicapped
801012	Disabled/Handicapped
801023	Disabled/Handicapped
801034	Disabled/Handicapped
801045	Disabled/Handicapped
801056	Disabled/Handicapped
800682	Disabled/Handicapped
801067	Disabled/Handicapped
801078	Disabled/Handicapped
801089	Disabled/Handicapped
801100	Disabled/Handicapped
801111	Disabled/Handicapped
801122	Disabled/Handicapped
801133	Disabled/Handicapped
801144	Disabled/Handicapped
801155	Disabled/Handicapped
801166	Disabled/Handicapped
801177	Education/Training
801188	Education/Training
801199	Education/Training
801210	Education/Training
801232	Education/Training
801243	Education/Training
801254	Education/Training
801265	Education/Training
801276	Education/Training
801287	Education/Training
800220	Education/Training
801298	Education/Training
801309	Education/Training
801320	Education/Training
801331	Education/Training
801342	Education/Training
801353	Education/Training
801364	Education/Training
801375	Education/Training
801386	Education/Training
800913	Education/Training
800935	Education/Training
801397	Education/Training
800968	Education/Training
801419	Education/Training
801430	Education/Training
801441	Education/Training
801452	Education/Training
801463	Education/Training
801474	Education/Training
801485	Education/Training
801496	Education/Training
801507	Education/Training
801518	Education/Training
801067	Education/Training
801529	Education/Training
801540	Education/Training
801551	Education/Training
801562	Education/Training
801573	Education/Training
801584	Education/Training
801595	Education/Training
801606	Education/Training
801617	Education/Training
801628	Education/Training
801177	Employment
801639	Employment
801650	Employment
801661	Employment
801672	Employment
801683	Employment
801694	Employment
801705	Employment
801716	Employment
801727	Employment
801045	Employment
801738	Employment
801749	Employment
801760	Employment
801771	Employment
801309	Employment
801782	Employment
801793	Employment
801804	Employment
801815	Employment
801826	Employment
801837	Employment
801848	Employment
801859	Employment
801870	Employment
801881	Financial Education/Credit
801892	Financial Education/Credit
801903	Financial Education/Credit
801914	Financial Education/Credit
801925	Financial Education/Credit
801936	Financial Education/Credit
801947	Financial Education/Credit
801958	Financial Education/Credit
801969	Financial Education/Credit
801980	Financial Education/Credit
801991	Financial Education/Credit
802002	Food/Shelter & Financial Assistance
802013	Food/Shelter & Financial Assistance
802024	Food/Shelter & Financial Assistance
802035	Food/Shelter & Financial Assistance
802046	Food/Shelter & Financial Assistance
801683	Food/Shelter & Financial Assistance
802057	Food/Shelter & Financial Assistance
802068	Food/Shelter & Financial Assistance
802079	Food/Shelter & Financial Assistance
802090	Food/Shelter & Financial Assistance
802101	Food/Shelter & Financial Assistance
802112	Food/Shelter & Financial Assistance
802123	Food/Shelter & Financial Assistance
802134	Food/Shelter & Financial Assistance
801034	Food/Shelter & Financial Assistance
802145	Food/Shelter & Financial Assistance
802156	Food/Shelter & Financial Assistance
802167	Food/Shelter & Financial Assistance
802178	Food/Shelter & Financial Assistance
802189	Food/Shelter & Financial Assistance
802200	Food/Shelter & Financial Assistance
802222	Food/Shelter & Financial Assistance
802233	Food/Shelter & Financial Assistance
802255	Food/Shelter & Financial Assistance
802266	Food/Shelter & Financial Assistance
802277	Food/Shelter & Financial Assistance
802288	Food/Shelter & Financial Assistance
802299	Health Education/Fitness/Well-being
802310	Health Education/Fitness/Well-being
802321	Health Education/Fitness/Well-being
802332	Health Education/Fitness/Well-being
802343	Health Education/Fitness/Well-being
801397	Health Education/Fitness/Well-being
802354	Health Education/Fitness/Well-being
802365	Health Education/Fitness/Well-being
802376	Health Education/Fitness/Well-being
802387	Health Education/Fitness/Well-being
801287	Health Education/Fitness/Well-being
802398	Health Education/Fitness/Well-being
802409	Health Education/Fitness/Well-being
802420	Health Education/Fitness/Well-being
802431	Health Education/Fitness/Well-being
802442	Health Education/Fitness/Well-being
802453	Health Education/Fitness/Well-being
802464	Health Education/Fitness/Well-being
802475	Health Education/Fitness/Well-being
802486	Health/Health Insurance
802497	Health/Health Insurance
802519	Health/Health Insurance
802332	Health/Health Insurance
802530	Health/Health Insurance
802541	Health/Health Insurance
802552	Health/Health Insurance
802563	Health/Health Insurance
802574	Health/Health Insurance
802585	Health/Health Insurance
802596	Health/Health Insurance
800385	Health/Health Insurance
802607	Health/Health Insurance
802618	Health/Health Insurance
802354	Health/Health Insurance
802629	Health/Health Insurance
802640	Health/Health Insurance
802376	Health/Health Insurance
802651	Health/Health Insurance
802662	Health/Health Insurance
802673	Health/Health Insurance
802684	Health/Health Insurance
802695	Health/Health Insurance
802706	Health/Health Insurance
802717	Health/Health Insurance
802728	Health/Health Insurance
802739	Health/Health Insurance
802420	Health/Health Insurance
802750	Health/Health Insurance
802453	Health/Health Insurance
802761	Health/Health Insurance
801166	Health/Health Insurance
802772	Health/Health Insurance
802783	Health/Health Insurance
802794	Health/Health Insurance
802805	Health/Health Insurance
802816	Health/Health Insurance
802321	Health/Health Insurance
802827	Health/Health Insurance
802838	Health/Health Insurance
802387	Health/Health Insurance
802849	Health/Health Insurance
802860	Health/Health Insurance
802871	Health/Health Insurance
802442	Health/Health Insurance
802882	Health/Health Insurance
802893	Housing
802904	Housing
800891	Housing
802915	Housing
802926	Housing
802937	Housing
802948	Housing
802959	Housing
802970	Housing
802981	Housing
802992	Housing
803003	Housing
803014	Housing
803025	Housing
802222	Housing
803036	Housing
803047	Housing
803058	Housing
803069	Housing
802277	Housing
803080	Housing
803091	Housing
803102	Housing
803113	Housing
803124	Immigrant Services
803135	Immigrant Services
803146	Immigrant Services
803157	Immigrant Services
803168	Immigrant Services
803179	Immigrant Services
803190	Immigrant Services
803201	Immigrant Services
801243	Immigrant Services
803212	Immigrant Services
803223	Immigrant Services
803234	Immigrant Services
803245	Immigrant Services
803256	Immigrant Services
803267	Legal/Consumer Protection
803278	Legal/Consumer Protection
803289	Legal/Consumer Protection
803300	Legal/Consumer Protection
801749	Legal/Consumer Protection
803311	Legal/Consumer Protection
803322	Legal/Consumer Protection
803333	Legal/Consumer Protection
803344	Legal/Consumer Protection
803355	Legal/Consumer Protection
803366	Legal/Consumer Protection
803377	Legal/Consumer Protection
803388	Legal/Consumer Protection
803399	Legal/Consumer Protection
803410	Legal/Consumer Protection
803421	Mental Health
800341	Mental Health
800374	Mental Health
803432	Mental Health
803443	Mental Health
803454	Mental Health
803465	Mental Health
803476	Mental Health
803487	Mental Health
803498	Mental Health
801078	Mental Health
803509	Mental Health
803520	Mental Health
801155	Mental Health
803531	Mental Health
803542	Mental Health
803553	Mental Health
803564	Mental Health
803575	Other
803586	Other
803597	Other
803608	Other
803619	Other
803630	Other
803641	Other
803652	Other
803663	Other
803674	Parenting/Family
803685	Parenting/Family
803696	Parenting/Family
801232	Parenting/Family
803718	Parenting/Family
803729	Parenting/Family
803740	Parenting/Family
800693	Parenting/Family
803751	Parenting/Family
803289	Parenting/Family
803762	Parenting/Family
803773	Parenting/Family
803784	Parenting/Family
802860	Parenting/Family
803795	Parenting/Family
800473	Parenting/Family
803806	Parenting/Family
800209	Public Safety
803817	Public Safety
803828	Public Safety
802937	Public Safety
803839	Public Safety
800121	Public Safety
803850	Public Safety
803861	Public Safety
803872	Public Safety
803883	Public Safety
803894	Public Safety
803905	Public Safety
800682	Senior/Elderly
803916	Senior/Elderly
803927	Senior/Elderly
803938	Senior/Elderly
803949	Senior/Elderly
803960	Senior/Elderly
801936	Senior/Elderly
801144	Senior/Elderly
803971	Senior/Elderly
801034	Senior/Elderly
803982	Senior/Elderly
803993	Senior/Elderly
804004	Senior/Elderly
804015	Sports/Recreation
804026	Sports/Recreation
804037	Sports/Recreation
804048	Sports/Recreation
804059	Sports/Recreation
804070	Sports/Recreation
804081	Sports/Recreation
804092	Sports/Recreation
804103	Sports/Recreation
802365	Sports/Recreation
804114	Sports/Recreation
804136	Sports/Recreation
802409	Sports/Recreation
804147	Sports/Recreation
804158	Sports/Recreation
804169	Sports/Recreation
804180	Substance Abuse/Addiction
804191	Substance Abuse/Addiction
804202	Substance Abuse/Addiction
804213	Substance Abuse/Addiction
804224	Substance Abuse/Addiction
804235	Substance Abuse/Addiction
804246	Substance Abuse/Addiction
804257	Substance Abuse/Addiction
804268	Substance Abuse/Addiction
804279	Substance Abuse/Addiction
804290	Substance Abuse/Addiction
804301	Substance Abuse/Addiction
804312	Substance Abuse/Addiction
804323	Substance Abuse/Addiction
804334	Substance Abuse/Addiction
804345	Substance Abuse/Addiction
804356	Substance Abuse/Addiction
804367	Substance Abuse/Addiction
804378	Substance Abuse/Addiction
804389	Substance Abuse/Addiction
804400	Substance Abuse/Addiction
804411	Youth
803696	Youth
801210	Youth
804422	Youth
801232	Youth
803861	Youth
804433	Youth
802354	Youth
801265	Youth
804444	Youth
804466	Youth
800693	Youth
804477	Youth
804488	Youth
803839	Youth
804499	Youth
804510	Youth
804521	Youth
804532	Youth
804543	Youth
801342	Youth
804554	Youth
804565	Youth
802475	Youth
804576	Youth
804587	Youth
804598	Youth
804609	Youth
804620	Youth
804631	Youth
804642	Youth
801463	Youth
804664	Youth
801485	Youth
804675	Youth
804686	Youth
804697	Youth
804708	Youth
801529	Youth
804719	Youth
802860	Youth
804730	Youth
804741	Youth
801595	Youth
801606	Youth
801760	Youth
804785	Youth
804796	Youth
804807	Youth
804785	Arts/Culture/Entertainment
804796	Arts/Culture/Entertainment
804807	Arts/Culture/Entertainment
804818	Sports/Recreation
804829	Sports/Recreation
804840	Arts/Culture/Entertainment
804851	Arts/Culture/Entertainment
804862	Arts/Culture/Entertainment
804884	Health/Health Insurance
804895	Health/Health Insurance
804906	Health/Health Insurance
804818	Senior/Elderly
804917	Senior/Elderly
804917	Housing
804950	Education/Training
804961	Financial Education/Credit
804972	Counseling
802508	Parenting/Family
800770	Counseling
804983	Other
804994	Community/Civic
805005	Health/Health Insurance
805016	Health Education/Fitness/Well-being
802244	Food/Shelter & Financial Assistance
805027	Sports/Recreation
801221	Education/Training
801221	Employment
801408	Education/Training
801408	Employment
804774	Parenting/Family
804774	Youth
804653	Parenting/Family
804653	Youth
804763	Parenting/Family
804763	Youth
804752	Parenting/Family
804752	Youth
803707	Parenting/Family
800550	Crisis/Abuse/Rape
805038	Health/Health Insurance
800011	Arts/Culture/Entertainment
802211	Food/Shelter & Financial Assistance
\.


--
-- Data for Name: log_category_search; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY log_category_search (search_id, session_id, ip_address, "timestamp", phrase_stems, top_score, search_phrase) FROM stdin;
\.


--
-- Data for Name: program; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY program (program_id, agency_id, last_updated, hidden_reason, program_name, program_alias, address1, address2, city, state, zip, main_phone, toll_free_phone, hotline, fax, website, email, short_program_description, long_program_description, eligibility, fees_charged, fees_explained, program_schedule, hours, languages, appointment, bring, bus_lines, parking, transportation, handicapped_accessible, child_care_provided, child_care_description, wait_list, wait_list_description, other_information, mail_address1, mail_address2, mail_city, mail_state, mail_zip, volunteer_opportunities, volunteer_information, resource_library, resource_library_description, meeting_space, meeting_space_description, news_letter, news_letter_description, public_net_access, public_net_access_description, contact_first_name, contact_last_name, contact_phone, contact_fax, contact_email, contact_email_available, area_type, area_data, tty_tdd, areas_served_description, languages_description, online_registration, reminders_sent, admin_notes) FROM stdin;
504532	101628	2008-07-15	\N	Gathering at the Well	Spring Forum on Women and Work	850 Broadway Street, Suite 1, Room B11	\N	Lowell	MA	01854	978-934-4380	\N	\N	978-934-4053	http://www.uml.edu/centers/women-work/	public_program_email@testing123.net	Annual Forum	The forum is designed to promote dialogue on issues of women and work striving for answers and solutions.  Every forum involves faculty, staff, students, and community members.	open to all	Free	\N	April 2, 2009	Typically held from 8am-12:30pm	English	Yes	\N		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
505082	101848	2008-07-10	\N	Kreative Kids Child Care	Kreative Kids Family Child Care	10 Wamesit Road	\N	Tewksbury	MA	01876	978-851-0719	\N	\N	\N	http://www.geocities.com/kreative_kids_childcare/Childcare.h	public_program_email@testing123.net	Licensed Family Daycare since 1991	Licensed Family Child Care.  Convenient to Rt. 93 and 495.  Located in Tewksbury off Rt 38.  Snacks and meals included.  Preschool program.	All are welcome.	Sliding Scale	\N	Open year round.	Monday - Friday   Hours are flexible	English	Yes	\N		Ample parking	\N	Not Applicable	Yes	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
505500	101969	2008-06-20	\N	Rehabilitation Options, PC	Robert Brownson, MS	564 Dutton Street	2nd Floor	Lowell	MA	01854	978-479-2923	\N	\N	978-746-9502	www.rehabilitation-options.com	public_program_email@testing123.net	Forensic Mental Health and Psychotherapy Practice	Clinical Competency:\r\nInter-Ethnic, Adult and Adolescent Orientation\r\nHighly Individualized\r\nBroad Clinical Interest, including: Depression, Traumatic Experience and Loss, Periods of Life Change, Addictive Traits and Conduct Problems                               \r\n\r\nForensic Services:\r\nCourts and Probation/Parole programs: Anger Management, Domestic Violence Intervention, Offense Specific Treatment\r\nSupportive/Supervisory Counseling to  reduce Recidivism and Re-Incarceration\r\nPsychosocial Evaluations\r\nRisk Assessments for Anger Management\r\nSubstance Abuse Evaluations\r\nOrientation to the Legal Process\r\nTrial and Testimony Preparation\r\nMitigating Circumstances Review and Alternative Sentencing Recommendations: G.L. c. 123, s. 15 (e, f)\r\nParole Hearing Preparation\r\nAttorney/Client Consultations\r\n\r\nRehabilitation Services:\r\nJob Skills Inventory and Advising\r\nWorkplace Advocacy\r\nLife Balance Counseling\r\n\r\nSpecialty Consultations:\r\nMens Work Programs\r\nSteps to Reduce Work Place Violence\r\nConflict Resolution and Mediation\r\nGuardian ad Litum  Middlesex County\r\n\r\nMASS Health Provider through MBHP	adolescent to adult services provided\r\nBillable to MassHealth and other health insurances\r\nAlso, Fee-for-Service rates to individual clients and businesses	Yes	Set fees, insurances billed or by Fee Adjustment Schedule	by appointment	by appointment	English	Yes	\N	06 Broadway / UMass Lowell!DELIM!07 Pawtucketville / UMass Lowell North!DELIM!09 Downtown Circulator!DELIM!16 Chelmsford via Chelmsford Street!DELIM!17 North Chelmsford via Middlesex Street	on site, off street parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	Yes	1	\N
504268	100880	2009-01-15	\N	Monthly well-child clinics, Ages 18 and under	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0932	\N	\N	978-671-0919	www.town.billerica.ma.us	public_program_email@testing123.net	well-child visits	Free monthly well-child visits by appointment only.  Town pediatrician does physicals (also sports physicals), exams, and immunizations.	uninsured, or unable to make an appointment with own MD because of time frame.  New to town.	Free	\N	Monthly (6-7 months year).\r\nAdvertised.\r\nCall for an appointment.	Usually held on a Friday afternoon (1:30-3pm).	English	Yes	immunization records/school forms		available	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA	\N	\N	\N	No	0	\N
507370	102684	2008-10-07	\N	Prevention	\N	5 Main Street	\N	Topsfield	MA	01983	978-887-6512	\N	978-771-4619	978-887-6512	http://www.tritowncouncil.org	public_program_email@testing123.net	Educational speaker series	Quarterly public presentations or forums on topics of interest primarily to students and parents, K-12.	All	Free	\N	See website: www.tritowncouncil.org	As announced.	English	Not Applicable	\N		\N	\N	Yes	No	\N	Not Applicable	\N	\N	P.O. Box 293	\N	Topsfield	MA	01983	Yes	\N	No	\N	Yes	12 people at conference table	Yes	The Reality Check	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Boxford, MA	\N	Also Topsfield, Middleton	\N	Not Applicable	0	\N
502519	100759	2008-12-15	\N	Initial Response Program	\N	70 Lawrence Street	\N	Lowell	MA	01852	978-452-1421	\N	\N	978-454-9968	\N	public_program_email@testing123.net	HIV support education and services	The Merrimack Valley Catholic Charities Initial Response program is designed to meet immediate needs by helping people with emergency food, fuel, rental/utility assistance, housing search information and referral, counseling, and holiday assistance.	Must be HIV positive or have AIDS and bring letter from MD or HIV case manager or HIV provider	Free	\N	Services available year round.	Monday-Friday 9AM-5PM	English	Yes	Must have medical verification of HIV / AIDS positive status.	02 Belvidere	On-site parking available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
509042	102090	2008-07-15	\N	Counseling	\N	21 Glen Ave	\N	Chelmsford	MA	01824	978-256-0667	\N	978-507-8009	978-256-5567	http://www.northstarguidanceinc.com	public_program_email@testing123.net	Counseling	Counseling	Aetna, United, Harvard, Neighborhood Health Plan, BC/BS, Teamsters, Cigna	Yes	\N	Open year round	Mon-Fri 9am-8pm	English	Yes	\N	16 Chelmsford via Chelmsford Street	\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Website links to community resources.	Yes	Classroom and group space available for 12 people	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	0	\N
508041	101936	2008-08-01	\N	Girls Exercise and Empowerment Groups at Alternative Clinic	\N	73D Winthrop Ave.	\N	Lawrence	MA	01844	978-689-6761	\N	\N	978-685-4280	www.glfhc.org	public_program_email@testing123.net	Girls Empowerment and Exercise Groups Ages 8-17	This an exercise and activity program for girls who need to exercise due to obesity.	Must be female and have obesity, ages 8-17.	Free	\N	Open year round.	Tuesday 4:00-5:15pm\r\nWednesday 4:00-5:15pm\r\nWednesday 5:15-6:30 pm	English!DELIM!Spanish	Yes	Insurance information, referral from doctor if not a patient of Greater Lawrence Family Health Center.		Parking available.	Transportation available to participants who are not able to be dropped off by parents.	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Large conference room.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA	\N	\N	\N	No	0	\N
509174	103421	2008-08-08	\N	Reading for Dyslexia	Private Tutoring	Blakes Hill Road	\N	Westford	MA	01886	978-621-6619	\N	\N	\N	www.creativedyslexics.org	public_program_email@testing123.net	Orton Gillingham Approach for Reading	Multisensory and individualized instruction to teach those with dyslexia.  A seven step lesson plan includes: phonograms (letter to sound association) and phonemic drills for blending and fluency; Review words; New Feature Work; Sound to letter association drill; Oral Spelling; Oral Sentence Dictation; and Oral Reading for fluency and comprehension.	Children and Adults, Northwest Massachusetts and Southern NH	Yes	Hourly tutoring fee, plus travel if I come to your home, school, or daycare environment	You will need to schedule two hourly sessions per week with me.	Open	English	Yes	Private and School Evaluations, diagnosis of Specific Learning Disability and Dyslexia		\N	\N	Not Applicable	Not Applicable	\N	Yes	\N	\N	8 Blakes Hill Road	\N	Westford	MA	01886	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	Tips for parenting children with dyslexia, coming in fall of 2008	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Southern NH	\N	No	0	\N
502684	100495	2008-08-20	\N	Transitional Support Services	\N	100-110 Green Street	\N	Lynn	MA	01902	781-598-1270	\N	\N	978-750-3620	www.cabhealth.org	public_program_email@testing123.net	Residential facility for men and women	The Transitional Support Services program runs a residential facility for men and women who are recently detoxed from drugs and alcohol and who are not yet ready or who are waiting for placement in a half-way house.	Please contact program directly or visit www.cabhealth.org for more information	Yes	Fees vary. Medicaid and most private insurance plans accepted.	Services available year round. Program enrollment ongoing	Please contact CAB directly	English!DELIM!Spanish	Yes	Appointments required. Referrals recommended - please visit main site, www.cabhealth.org for more information		\N	Program provides some transportation to other services.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Contact program for more information about volunteer opportunities	No	\N	Yes	Meeting space may be available. Please contact CAB directly.	Yes	"Insights," the agency newsletter is published twice yearly. Available by U.S. mail or on the agency website as a PDF file, www.cabhealth.org	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
508987	102508	2008-09-15	\N	St. Luke's Dental Clinic	\N	412 Hampshire St.	\N	Lawrence	MA	01841	978-269-5284	\N	\N	\N	http://www.lazarushouse.org/	public_program_email@testing123.net	Dental care in modern facility.	A team of volunteer dental professionals provides care that all patients deserve, with special emphasis on children.\r\n\r\nPatients are treaded with respect, dignity and confidentiality, all in the comfort and convenience of a modern two-chair, full-service facility.	Low-income and homeless individuals and families.	Free	\N	Open year round.	Call to schedule an appointment.\r\n\r\nOffice Hours:\r\nMon-Fri: 8:30AM-4:30PM	English!DELIM!Spanish	Yes	ADD INFO		\N	Van transportation available.	Yes	No	\N	Yes	\N	Dental Services\r\n\r\n+ Four Oral Surgeons\r\n+ Two Restorative Dentists\r\n+ Prioritize Preventative Care for Children	St. Luke's Dental Clinic	P.O. Box 408	Lawrence	MA	01842	Yes	Join a team of dedicated professionals. Contact program for more information about volunteering as DMD / DDS.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
506655	100088	2008-10-01	\N	Computer Repair Training Program	\N	34 Hurd St	\N	Lowell	MA	01852	978-441-9949	\N	978-265-7173	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	Learn how to repair and network computers	UTECs PC Repair Training offers the A+ Certification curriculum. A+ Certification is a testing program sponsored by the Computer Technology Industry Association (CompTIA) that certifies the competency of service technicians in the computer industry. Anyone who wants an internationally-recognized credential as a competent entry-level computer service professional can take the A+ examination. The program is backed by major computer hardware and software vendors, distributors, resellers and publications. Earning A+ certification means that the individual possesses the knowledge, skills, and customer relations skills essential for a successful entry-level computer service technician, as defined by experts from companies across the industry. The test covers a broad range of hardware and software technologies, but is not related to any vendor-specific products. To become certified, you must pass two parts -- the Core Hardware and the Microsoft Windows module portion. When both the Core and the Windows portions are passed, the candidate receives the A+ designation.\r\nThe A+ Certification can benefit you if: \r\n\tyou want to present recognized proof of professional achievement to employers and clients \r\n\tyou seek enhanced job opportunities \r\n\twant the opportunity for advancement \r\n\tare end-user help desk personnel needing to develop information skills\r\nA+ Certification was founded with the view that to be successful, the expectation level needs to be above average. Companies are requiring skill sets that are far beyond what was expected even five years ago. This curriculum addresses these issues. The course is broken into two subsections, Supporting Microcomputers and Supporting MS-DOS/Microsoft Windows and will prepare students for the two-part A+ exam. In addition, this course is also useful to any individual who needs to broaden his/her knowledge of microcomputers. \r\nThere are two tests that cover the information presented in this class. Below are descriptions of both sections: \r\nSection One - Core Hardware(Test #220-301)\r\nThis part of the course covers the following topics: \r\n\tComputer basics \r\n\tTools and safety \r\n\tInside the PC \r\n\tBus architecture \r\n\tMemory overview \r\n\tMicroprocessors \r\n\tDisk storage \r\n\tInput/output devices \r\n\tPrinters \r\n\tMiscellaneous hardware \r\n\tLocal Area Networks \r\n\tTroubleshooting \r\n\tCustomer service\r\nSupporting Microcomputers focuses on servicing and supporting personal computers. Upon course completion, students will be able to provide technical support and maintain and service personal computers. They will also be able to: \r\n\tRecognize essential components of a PC system \r\n\tIdentify and understand PC peripherals including monitors, keyboards, pointing devices, scanners, modems, and cables \r\n\tUnderstand how to safely use common and specialized tools \r\n\tRecognize the impact of electrostatic discharge (ESD), electromagnetic interference (EMI), and radio frequency interference (RFI) on computer components \r\n\tIdentify power requirements and troubleshoot power problems \r\n\tIdentify and understand bus architectures \r\n\tIdentify and understand different types of memory and understand memory allocation \r\n\tIdentify and understand different types of microprocessors \r\n\tIdentify and understand various types of disk storage \r\n\tIdentify and understand input/output ports, their uses, and cable restrictions \r\n\tIdentify and understand different types of printers and how to install them \r\n\tUnderstand the functions and operations of docking stations for portable computers \r\n\tIdentify multimedia components such as sound cards and speakers \r\n\tUnderstand the basic components of local area networks (LANs) \r\n\tUnderstand the differences between peer-to-peer and client/server LANs \r\n\tUnderstand and identify LAN topologies \r\n\tUnderstand Ethernet, Token Ring, and ARCnet protocols \r\n\tIdentify and understand the uses of twisted-pair, coaxial, and fiber-optic cable \r\n\tIdentify and correct common hardware problems and use software and hardware diagnostic tools Respond properly to customer service questions \r\n\tSet up PCs in the proper environment \r\n\tVerify errors in the POST \r\n\tTroubleshoot PC components and peripheral devices \r\n\tSet system interrupts \r\n\r\nSection Two - Supporting MS-DOS/Microsoft Windows Course Content (Test #220-302)\r\nThis part of the course covers the following topics: \r\n\tOperating system overview \r\n\tWindows 9x\r\n\tWindows NT\r\n\tQuantifying memory \r\n\tMicrosoft Windows 2000 \r\n\tUpgrading to Windows 2000 or XP\r\n\tInstalling Windows 2000\r\n\tRunning Windows 2000 and XP\r\n\tNetworking Windows 2000 and XP\r\n\tTroubleshooting\r\nSupporting Microsoft Windows focuses on servicing and supporting personal computers running Microsoft Windows 9x through Microsoft Windows XP. Upon course completion, students will be able to provide technical support and maintain and service personal computers and the related operating systems. They will also be able to: \r\n\tIdentify the major desktop components and interfaces, and their functions.\r\n\tIdentify the names, locations, purposes, and contents of major system files.\r\n\tDemonstrate the ability to use command-line functions and utilities to manage the operating system.\r\n\tManage files and directories.\r\n\tIdentify the major operating system utilities, their purpose, location, and available switches.\r\n\tUnderstand and Implement procedures for installing, upgrading, and troubleshooting Windows 2000 and XP\r\n\tIdentify the basic system boot sequence.\r\n\tIdentify procedures for installing/adding a device and configuring drivers.\r\n\tIdentify procedures necessary to optimize the operating system.\r\n\tRecognize and Interpret meaning of common error codes.\r\n\tRecognize common operational and usability problems and determine how to resolve them.\r\n\tUnderstand various elements of the Microsoft Windows 2000 environment, including WIN.COM, .INI files, .DLL files, drivers, and font files \r\n\tUnderstand and use Program Manager \r\n\tIdentify and configure the a network on a Microsoft Windows System\r\n\tIdentify the basic internet protocols and terminologies.\r\n\tCreate a startup disk \r\n\tSet up e-mail \r\n\tConfigure Web browsers \r\n\tCreate an emergency repair disk (ERD) \r\n\tTroubleshoot Windows-specific printing problems \r\n\tExamine other common problems and their causes \r\n\tUse Windows utilities and DOS commands and switches to diagnose and troubleshoot problems \r\n\tUnderstand viruses and virus types	Anyone between ages of 13-23	Free	\N	Six week courses.  Please contact Sovanna to sign up for next course.	Tue, Wed, and Thu - 6pm - 8pm	English!DELIM!Spanish!DELIM!Khmer	Yes	\N		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	A++ certified individuals interested in coordinating classes.	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a large room with conference table.	Yes	Sign up at www.utec-lowell.org	Limited	30 minutes sessions to teens.	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County	\N	Lowell and surrounding	\N	No	0	\N
503575	101144	2008-10-07	\N	Afterschool Alternatives	\N	1360 Main St.	\N	Tewksbury	MA	01879	978-863-0099	\N	\N	978-851-5192	www.yes4kids.com	public_program_email@testing123.net	After school/school break activities for disabled children	The program is designed to improve the emotional, mental and physical well-being of school age children with disabilities, by offering a variety of after school and school break activities.	5-15 years	Sliding Scale	\N	School Year	After school. Call or check website for calendar of activities.	English	Yes	Intake appointment required. Call for appointment.		\N	\N	Yes	No	\N	Yes	Variable -- contact for information.	\N	\N	\N	\N	\N	\N	Yes	Share your talent and volunteer to teach disabled children your skill or hobby.	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	No explicit geographic limitation, however YES primarily draws from Greater Lowell	\N	Not Applicable	0	\N
501034	100121	2008-07-15	\N	Yoga	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Yoga Classes	Yoga classes.	Open to all	Yes	14 classes: Free - member $112 - non-member; 21 classes: Free - member $170 - non-member	Program runs year round. Classes are Sun. 9am, Wed. 7:30pm,  & Thur. 9am	Monday-Friday 5:30AM-10PM; Saturday 7AM-7PM; Sunday 7AM-4PM Memorial thru Labor Day Weekend Sunday hours 7:00 Am - noon	English	No	Contact program for more information		Parking is available	\N	Yes	Yes	Babysitting available mornings,  9:00 Am - 1:15 PM	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501485	100308	2008-08-07	\N	Community & Family Counseling Services	Formerly, Community Counseling Center	99 Church Street	\N	Lowell	MA	01852	978-458-6282	\N	978-458-6282	978-441-9826	\N	public_program_email@testing123.net	Provide a range of outpatient counseling services	Community & Family Counseling Services provides a range of counseling and consultation services for children, adults, couples, groups and families. These services include: Individual and Family Counseling; Group Counseling; School Based Counseling; Crisis Intervention; Victim Services Program; Employee Assistance Program; Consultation and Evaluation; Psychological Testing and Medication Services. Interdisciplinary teams that include adult and child psychiatrists, clinical nurse specialists, psychologists, clinical social workers and mental health counselors staff our clinics. A broad range of issues are addressed, including: Anxiety and Panic Disorders, Depression, Divorce and Separation Issues, Family Adjustment Programs , School Related Problems, Stress Management and Substance Abuse.	Open to Massachusetts and New Hampshire residents ages 3 and older. Most insurance plans accepted.	Sliding Scale	Limited sliding fee based on federal regulations: # in household & gross income. Must provide copy of tax forms to determine fee.	Please call 978-458-6282 to schedule an appointment.	Monday - Thursday 8AM-9PM\r\nFriday 8AM-5PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Social security information and insurance information.	09 Downtown Circulator	Parking available in Central Plaza parking lot next to the Family Dollar Store	\N	Yes	No	\N	No	Depending on the specific insurance or limitations on when someone can come in for an appointment there may be a short wait.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	Occasional newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Primarily serve Greater Lowell but clients from outside this area are not refused services.	\N	No	0	\N
506798	102585	2008-08-20	\N	Pathways	\N	900 Chelmsford Street	\N	Lowell	MA	01851	978-291-0342	\N	\N	978-291-0344	http:.//www.littlesprouts.com	public_program_email@testing123.net	School to Career program for DSS involved youth.	Program assists DSS youth in identifying strengths and vocational interests through linking academic learning to the workplace.  Services include case management and outreach, truancy prevenation, lifeskills, workforce development, job placement and support, family home visiting.	DSS involved youth.	Free	\N	7 days a week year round.	times vary	English!DELIM!Spanish	No	Referal from DSS Social Worker		\N	\N	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
506754	100011	2008-09-02	\N	Destination College	\N	45 Merrimack Street, Suite 227	\N	Lowell	MA	01852	978-453-9339	\N	\N	978-452-0838	www.bbbslowell.org	public_program_email@testing123.net	Mentoring High School Students toward College.	Volunteers are matched with High School students in order to help with college preparation, application, financial aide and a successful transition to college. Generally mentors see their mentee weekly, college visits, group trainings, goal setting.	18 years old, screening process includes CORI check, interview, references.	Free	\N	Flexible	Weekly and bi-weekly	English!DELIM!Khmer	Yes	\N		\N	\N	Yes	No	\N	Yes	Call 978-453-9339 to inquire	\N	\N	\N	\N	\N	\N	Yes	Mentoring	No	\N	No	\N	Yes	\N	Limited	Mentors use internet for info.	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	Lowell	\N	Yes	0	\N
501166	100154	2008-09-29	\N	Parks and Recreation	\N	25 Shattuck Street	\N	Lowell	MA	01852	978-970-4174	\N	\N	978-446-7002	www.lowellma.gov	public_program_email@testing123.net	Recreation programs for Lowell residents	The mission of the Parks and Recreation Department is to provide recreational activities to the city of Lowell and to provide all Lowell residents an opportunity to experience all types of leisure time experiences. Programs operate at various parks and schools around Lowell.	Open to all	Free	\N	Program runs year round	Monday-Friday 9AM-5PM	English	No	\N		Parking available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should call 978-970-4174. Volunteer opportunities vary.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
507513	102937	2008-03-24	\N	Christ Jubilee Food Pantry	\N	101 Smith St	\N	Lowell	MA	01851	978-452-9977	\N	\N	\N	http://www.icfmministries.com/	public_program_email@testing123.net	Food distribution to families in need	/	Massachusetts resident	Free	\N	Open year round	Fridays 1-5pm	English	No	Proof of residency, SS#, or Mass Health card	05 Westford Street/Drum Hill	Available parking	\N	Yes	No	\N	No	\N	\N	101 Smith St	\N	Lowell	MA	01851	Yes	\N	Not Applicable	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA	\N	\N	\N	No	4	\N
503267	100484	2008-12-01	\N	Out of School Youth Development Center	\N	33 Kearney Square	\N	Lowell	MA	01852	978-656-3104	\N	\N	978-656-3150	www.middlesex.cc.ma.us	public_program_email@testing123.net	Out of School Youth educational and case management services	The Out of School program provides out of school youth (16-21) with educational and case management services.	Out of school youth age 16-21 residing in Lowell and surrounding towns	Free	\N	year round	Monday-Friday 9AM-5:00PM	English!DELIM!Portuguese!DELIM!Spanish	Yes	Contact program for information	09 Downtown Circulator	\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Tutors are always needed to work with youth that are preparing fo rGED	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	Program is offered in English but in addition to English staff is fluent is Portuguese and Spanish.	No	0	\N
504917	101023	2008-06-20	\N	Parental Stress Line	\N	108 Water Street	\N	Watertown	MA	02472	617-926-5008	800-882-1250	800-632-8188	617-926-5011	www.parentshelpingparents.org	public_program_email@testing123.net	stress line	a 24 hour 7 days a week hot line for parents and caregivers under stress. Trained volunteer telephone counselors who are sympathetic, supportive and non-judgmental	any person in a care giving role with children	Free	\N	open year round	24hours 7 days a week	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	any one who would like to volunteer are welcomed 10 hours of training required in Watertown	Yes	Listing of Parental stress line, resources legal, and some counselors state wide	Yes	A small conference room in our Watertown office sits 10 comfortably.	Yes	Lifelines twice a year	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	The whole state	A translation service can be accessed	Not Applicable	1	\N
509009	102508	2008-07-13	\N	Good Shepherd Center Soup Kitchen	\N	Good Shepherd Center	410 Hampshire St.	Lawrence	MA	01841	978-682-2227 x100	\N	\N	\N	http://www.lazarushouse.org/	public_program_email@testing123.net	Soup kitchen.	The Good Shepherd Center Soup Kitchen offers a continental breakfast and a hot noon-time meal to over 200 guests.	Low-income and homeless individuals.	Free	\N	May  October\r\nOpen Monday  Friday\r\n\r\nNovember  April\r\nOpen 7 days a week	May  October:\r\nContinental Breakfast served\r\n8:00 am  9:30 am; Monday  Friday\r\nLunch served\r\n11:30 am  1:00 pm; Monday  Friday\r\n\r\nNovember  April:\r\nContinental Breakfast served\r\n8:00 am  9:30 am; Daily\r\nLunch served\r\n11:30 a.m.  1:00 pm; Daily	English	No	ADD INFO		\N	\N	Yes	No	\N	No	\N	\N	Lazarus House Ministries	P.O. Box 408	Lawrence	MA	01842	Yes	Meals are warmly served by volunteers within our community who generously donate their time.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
501584	100407	2008-06-19	\N	Learning in Retirement Association	LIRA	71 Wilder St., Suite 6	\N	Lowell	MA	01851	978-934-3135	\N	\N	\N	http://www.uml.edu/community/lira/	public_program_email@testing123.net	Membership program for the retired and semi-retired	The Learning in Retirement Association is a membership program for the retired and semi-retired of the Merrimack Valley.	Contact program for more information	Yes	Contact program for more information	Program runs year round	Contact the program for details	English	Yes	contact program for more information		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	A wide variety of subjects available at three locations. For information visit http://www.uml.edu/libraries	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
508151	100044	2008-06-20	\N	Lowell Community Health Center  Family Practice	\N	17 Warren Street	2nd floor	Lowell	MA	01854	978-937-9700	\N	\N	978-446-0236	www.lchealth.org	public_program_email@testing123.net	primary healthcare for all ages	Family practice provides primary care services to individuals of all ages with physician, nurse practioner and physician assistant providers	services open to all	Sliding Scale	\N	open year round	Mon 8:30 to 5:00  Tue 8:30 to 8:00  \r\nWed-Fri  8:30 to 5:00	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Insurance card, if applicable	09 Downtown Circulator	Several spaces adjacent to building and\r\nLower locks garage next to Doubletree hotel	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Wilmington, MA	\N	\N	interpretation services available in additional languages	No	1	\N
503652	101221	2008-07-10	\N	Urban Forestry	\N	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	\N	www.lowelllandtrust.org	public_program_email@testing123.net	Urban Forest Program	The Trusts program of planting more than 4,000+ new public shade trees is a nationally recognized model for urban reforestation, through which the City of Lowell was designated a Tree City USA.	Progam provides trees for public landscaping projects located in Lowell only.	Free	\N	Plantings are done mainly in spring and fall.\r\n\r\nArbor Day celebration Friday, May 11th at Rogers Fort Hill Park at 10am.  Please call for more information.	Not applicable	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	Not Applicable	1	\N
503850	100429	2008-05-27	\N	Neuro Diagnostics	EEG and EMG	One Hospital Drive	\N	Lowell	MA	01852	978-934-8231	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Neuro Diagnostics	Diagnostic testing to assist the physician in the diagnosis of neuro/muscular diseases.	Contact department for eligibility.	Yes	\N	EEG: Monday-Friday \r\nEMG: Physician Scheduled	Office Hours: Monday-Friday 8:00am to 3:00pm, excluding holidays	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Physician order		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	3	\N
507051	102112	2009-01-15	\N	Future Stars Sports and Leadership Development	\N	9 Central Street, Suite 203	\N	Lowell	MA	01852	978-459-0200	\N	\N	\N	www.reaksmey.org	public_program_email@testing123.net	Team building	The Future Stars camp offers at-risk youth experiences in sports, the arts, leadership and positive interactions with community members.  Their experiences enable them to become excellent mentors while gaining valuable work experience.	all youth ages 7 - 14	Free	\N	It's a six weeks program, usually on the last week of June to the beginning of August of each year.	Monday - Friday from 10:00 Am - 3:00 PM	English!DELIM!Spanish!DELIM!Khmer	No	Primary care such name of doctor and registration form.		\N	Transportation is provided	Yes	No	\N	Yes	Overwhelming applications	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Yes	0	\N
509515	103971	2009-06-05	\N	Graffit	\N	319 wILDER sT	\N	Lowell	MA	01852	978-452-4522	\N	\N	\N	\N	public_program_email@testing123.net	D.R.A.W. Presents : Graffiti	Teaching teens 16-21 about the history and artform off graffiti	16-21	Free	\N	for the month of october	thursday 4-8p	English!DELIM!Spanish	No	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	contact cfelt@jri.org	Not Applicable	\N	Yes	319 wilder St	No	\N	Limited	\N	Corey	Felt	978-697-2142	\N	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
505104	100429	2008-12-16	\N	YOGA	Saints Memorial Medical Center	One Hospital Drive	\N	Lowell	MA	01852	978-934-8591	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Strength Training, Stretching, Toning	Kripalu Yoga	All Levels Are Welcome	Free	$10.00 per class	On-going Year Round	Thursdays 6:00PM - 10:00PM	English	No	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	Not Applicable	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
501859	100517	2008-07-15	\N	Elder Care Advisors	\N	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-922-2275	978-687-1067	www.esmv.org	public_program_email@testing123.net	Advise families on options for care and care planning	Elder Care Advisers are available to meet with families at home or in the workplace to discuss options for care, care planning and ongoing short term advice. Provided at no cost and without regard to income. Call Elder Services for referrals.	Program is open to everyone	Free	\N	Year round	Monday through Friday 8:00AM to 5:00PM	English	Yes	Call Elder Services for referrals.		\N	\N	Yes	No	\N	No	\N	(800) 924-4222 TDD#	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	Also serving Boxford, Georgetown, Groveland, Newbury, West Newbury, Rowley, North Andover	\N	No	0	\N
506094	102222	2008-07-15	\N	Workshops and Professional Training	\N	PO Box 89	\N	West Boylston	MA	01583	508-835-2482	\N	\N	\N	www.ForwardMotion.info	public_program_email@testing123.net	When, How and Whether to Disclose at Work	Workshops and professional training cover topics of interest to professionals working with people who have Asperger's Syndrome and Nonverbal Learning Disorder. Programs cover disclosure at work, finding the right career, job readiness and more.	Groups can sponsor workshops and professional trainings. Contact me for details.	Yes	Depends on number of attendees and program.	Your group can sponsor me to present this workshop.	Workshop/trainings are between 90 minutes to 7 hours long.	English	Yes	Pre-registration is required; visit www.ForwardMotion.info or call 508-835-2482		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
504279	100429	2008-12-16	\N	Baby-Sitter Training Course	\N	One Hospital Drive	\N	Lowell	MA	01852	978-937-8591	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Baby-Sitter Training Course	Offered to boys and girls 10 - 14 years old\r\nSafety,Choking and First Aid.	All are Eligable	Yes	$30.00 per participant	School Vacations and Summer Time	9:00-11:00AM and 8:00AM - 12:00Noon	English	Yes	\N		\N	\N	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
504521	101628	2008-07-15	\N	CWW Associates Program	\N	850 Broadway Street, Suite 1, Room B11	\N	Lowell	MA	01854	978-934-4380	\N	\N	\N	http://www.uml.edu/centers/women-work/	public_program_email@testing123.net	Associates Program	The goal of the program is to establish CWW as a hub for innovative scholarly projects on the gendered conditions of work.	Contact Meg Bond for more information.	Free	\N	Meetings held throughout the academic year.	n/a	English	Not Applicable	\N		\N	\N	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	\N	Yes	We have a small conference room that can comfortably accommodate small groups.  Please call ahead to schedule.	Yes	One in the fall and one in the spring	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
505522	101958	2008-07-24	\N	Adult Partial Hospitalization	\N	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5160	\N	800-222-2237	\N	www.arbourhealth.com	public_program_email@testing123.net	Adult Partial Hospitalization	Lowell Treatment Centers Adult Partial Hospitalization Program (PHP) is a short-term, intensive treatment program designed to meet the needs of individuals suffering from any psychiatric disorder who are in crisis and require stabilization. This level of care offers transition after inpatient hospitalization or can often prevent patients from needing hospitalization.\r\n\r\nGroups begin daily at 10:00 AM and continue\r\nthrough 3:30 PM, Monday through Friday.	The program offers treatment to adults who are in crisis and require stabilization.	Yes	\N	Open Year round	Groups begin daily at 10:00 am and continue\r\nthrough 3:30 pm, Monday through Friday.	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
507436	100099	2008-07-30	\N	Fitness & Recreation Programs	\N	657 Middlesex Street	\N	Lowell	MA	01851	978-458-4526	\N	\N	978-453-9740	www.lbgc.org	public_program_email@testing123.net	Fitness & Recreation Programs	The Boys & Girls Club of Greater Lowell will be offering the following "open gym" programs during the winter months.  \r\n\r\nCompetitive games including dodgeball, relay races, obstacle courses, four square, basketball, soccer, floor hockey and kick ball.  \r\n\r\nWinter specialized programs include:\r\nJr. NBA/WNBA - learn the fundamentals of basketball with in-house instruction in passing, dribbling, shooting and game rules.  All participants are members of a Club team which takes part in competition twice a week with Wednesday & Friday games.  Jr NBA/WNBA is open to all Club members ages 9-12, sign ups are still being accepted.\r\n\r\nTeen Basketball - The Boys & Girls Club is open each Wednesday and Friday night for pick-up basketball for teens ages 13-18.  \r\n\r\nFitness Center- The Club's fitness center is open each day after school with acces to free weights, aerobic equipment and boxing equipment\r\n\r\nSwimming - Swimming lessons are offered at the Club through Pool School Plus.  For more information, log on to www.poolschool.org\r\n\r\nGym Rentals - The Boys & Girls Club gymnasium and swimming pool are available for private rental to host birthday parties, community events or team outings.  Contact Michelle Phillips for more information	All registered Club members are welcome to join in open gym activities	Yes	All fitness and recreation programs (with the exception of Pool School swim lessons) are offered as part of the Club's membership.	Open Year round	Monday through Friday 2:00 - 6:00 pm\r\nMonday & Wednesday, 6:00 -8:00 pm (13-18)	English!DELIM!Spanish!DELIM!Khmer	No	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Newly developed Learning Center complete with books for youth and teens, computers for research, homework help and reading and writing projects.	Yes	Teen Room, computer lab, matted room, multipurpose room, Fitness facility, gym,pool, Learning Center, Gameroom, Arts & Crafts room, Snack Room. Call for more information if you would like to learn more about room rentals for meetings and events.	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
504488	100044	2008-08-01	\N	Health Benefits	\N	585-597 Merrimack Street, 17 Warren St., 135 Jackson St	\N	Lowell	MA	01854	978-746-7856	\N	\N	978-970-0359	www.lchealth.org	public_program_email@testing123.net	Enrollment & Information On Health Insurance Plans	Health Benefit Counselors can help patients apply for various types of free care and insurance plans. Health Benefit staff provide information on sliding scale fees, and health insurance coverage. Call to setup an appointment today.	All new patients without insurance coverage must meet with a Health Benefits Counselor.	Free	Free to patients to meet and determine eligibility for coverage.	Operates year round.	Monday - Friday 9:00AM - 5:00PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	Bring 2 current paychecks and a photo ID, as well as proof of address.		Parking available at all LCHC sites in patient lots.	\N	Yes	No	\N	No	\N	\N	586 Merrimack Street	\N	Lowell	MA	01854	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County	\N	\N	\N	No	0	\N
506072	102211	2008-08-04	\N	Merrimack Valley Parents Of Murdered Children Support Group	P.O.M.C.	Saints Medical Center	Stackpole St	Lowell	MA	01852	978-452-5858	\N	\N	978-555-5121	http://www.pomcma.com/	public_program_email@testing123.net	We provide Support	POMC makes the difference through on-going emotional support, education, prevention, advocacy, and awareness	Membership is open to those who have been cruelly bereaved by the murder of a loved one.	Free	\N	Open Year round	2nd Tuesday of each month 6:30 - 8:30pm	English	Not Applicable	\N		On Stackpole St or the parking garage	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	p.o. 8737	\N	Lowell	MA	01853	Yes	Professionals who are in frequent contact with grieving families are also welcome to join.	Not Applicable	\N	Yes	Board Room which is located on the first floor of the Residence Building on Stackpole St.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
506160	101056	2008-08-15	\N	Case Management Services for refugees and asylees	\N	144 Merrimack St., Suite 202	\N	Lowell	MA	01852	978-459-9031	\N	\N	978-459-0154	www.iilowell.org	public_program_email@testing123.net	Case management services for refugees and political asylees	Case manager assists refugees and asylees with accessing community services, applying for MassHealth when appropriate, applying for refugee cash assistance when applicable, offering cultural orientations for newly arrived refuges and political asylees.	Refugees and political asylees	Free	\N	Open year round	Monday-Friday 9:00-5:00	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	\N		Area parking garages	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501331	100209	2008-08-18	\N	Public Relations/Promotional/Community Relations	\N	131 Merrimack Street	\N	Lowell	MA	01852	978-459-8154	\N	\N	978-452-4145	www.glcc.biz	public_program_email@testing123.net	Promotional and Public & Community Relations	The mission of the Chamber is to provide a significant unified voice in shaping the future growth of the business community which it services and to provide a vehicle through which individuals can work to improve the areas business climate and quality of life. The chamber provides Promotional and Public Relations as well as Community Relations for its members.	Business in Greater Lowell	Yes	Annual Membership Fee	Open year-round	Monday - Friday 8:30 AM - 5:00 PM	English!DELIM!French	No	Go to www.glcc.biz to register or call for more information		Metered street parking & area garages	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Board Room	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
503476	100330	2008-09-04	\N	Lowell Vote and Advocacy	\N	9 Central Street	Suite 203	Lowell	MA	01852	978-654-6957	\N	\N	978-454-6992	www.onelowell.net	public_program_email@testing123.net	Lowell Vote	Lowell Vote and Advocacy is a program designed to increase the political power of the immigrant and refugee communities in Lowell, which  constitute nearly half of Lowells residents.  ONE Lowell has successfully increased voter registration, initiated electoral reform to ease immigrant access to the polls, and organized successfully around immigrant issues since 2002. It has accomplished this through grassroots organizing of immigrant and refugee residents, and through nonprofits and churches that serve immigrants and refugees.	Open	Free	\N	Open year round	Mon-Friday 9am-5pm	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Please contact us for more information regarding volunteer positions	No	\N	No	\N	Yes	We do not have a newsletter but we do have voting information available in 4 languages.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Not Applicable	0	\N
500121	100022	2008-09-15	\N	Young Parent Support Program	YPSP	97 Central Street	Unit #302	Lowell	MA	01852	978-454-5405, ext. 111	\N	\N	978-454-5043	www.ywcaoflowell.org	public_program_email@testing123.net	Support for pregnant /parenting youth age 23 or under	The Young Parents Support Program provides services/assistance with:  parenting education and support, weekly/monthly home visits, connection to free/low-cost community resources, obtaining child care, enrollment in educational/training programs, choosing a doctor/pediatrician, and information on topics that are important to the client.	Pregnant/parenting youth, age 13-23 (young men are also welcome)	Free	\N	Program runs year round.	Monday - Friday, 9:00 AM - 5:00 PM	English	Yes	Referrals can be taken over the phone by a provider or client call call directly.		Parking available in Leo Roy Parking Garage on Market Street, or meter parking.	We provide parenting education at the client's home/YWCA office, and transportation to appointments, if staff are available.	No	Yes	Children are always welcome at visits.	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	We are in the process of developing a resource library.	Yes	97 Central Street, Unit #302, or Acre Youth Center, 41 Rock Street	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
507557	103003	2008-10-01	\N	Art Classes and Summer Camps	\N	251 Foster Street	\N	Lowell	MA	01851	978-937-1602	\N	\N	\N	lighthouseartschool.com	public_program_email@testing123.net	Art School	The Lighthouse Art School in Lowell is a place for all ages to learn how to express themselves with multiple mediums of visual art, gain a greater appreciation for the beauty of art and have a lot of fun doing it.	Age groups: 5yrs to 18yrs	Yes	$20.00 per class of one and half hours (per week)\r\n$250 per week for Summer Camps	Open year round	During School year - afternoons\r\nDuring Summer - Summercamps	English	Yes	\N		\N	\N	No	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
507733	103124	2008-10-17	\N	Food Pantry	\N	PO Box 5551	13 Lafayette Road	Salisbury	MA	01952	978-463-8801	\N	\N	978-463-8743	www.pettengillhouse.org	public_program_email@testing123.net	Pettengill House Food Pantry	The food pantry is open Tuesdays, all day long. The pantry serves non parishable and limited parishable items to individuals and families in need.	Reside in Salisbury, Amesbury, Newburyport, Newbury, West Newbury, Merrimac, Rowley, Byfield, Groveland	Free	\N	Open Year Round	Tuesday 9:00am - 2:00pm & 4:00pm - 7:00pm	English	Yes	proof of residency, picture ID	51 Haverhill - Amesbury - Newburyport	Parking out back of building	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Not Applicable	\N	Not Applicable	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!Rowley, MA!DELIM!Salisbury, MA!DELIM!Merrimac, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
509306	100088	2008-10-27	\N	UTEC GED Program	\N	34 Hurd St	\N	Lowell	MA	01852	978-856-3913	\N	978-265-7173	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	This GED program is open to young people ages 16-23.	The UTEC GED program will prepare young people ages 16-23 to take and pass their GED test.  Students must register with Emilie, the Open School Coordinator by calling 978.856.3913 before they can begin classes.	Ages 16-23 and resident of Lowell.	Free	\N	The program runs on the Lowell High School Calendar year.	Tuesday, Wednesday, and Thursday mornings from 9:00 to 12:00	English	Yes	\N	09 Downtown Circulator	Limited street parking available.	\N	No	No	\N	Yes	Students are moved into classes off of the waiting list as soon as space is available.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Limited	Internet access is available for student use.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	Not Applicable	0	\N
508162	102145	2008-12-01	\N	TryToStop.org	1-800-TRY-TO-STOP	341 pine st	\N	Lowell	MA	01851	978-970-4010	1-800-879-8678	\N	978-970-4011	\N	public_program_email@testing123.net	Expert advice to quit smoking	Your new life without cigarettes is just a click away! Read success stories from former smokers. Use our Quit Wizard&#8480; to create your own action plan. Check out the TryToSTOP community where, every day, people just like you are helping each other quit. Best of all, it's free!	Everyone welcome	Free	\N	Open year round	Monday thru Friday	English!DELIM!Spanish	No	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Yes	0	\N
500693	100055	2008-12-02	\N	REIKI Level I Training	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Simple natural process for healing by the laying on of hands	Reiki is a gift and an art, a simple natural process for healing by the laying on of hands. Reiki is one of the more widely known forms of energy healing. Energy healing involves direct application of Chi for the purpose of strengthening a clients energy system. Modern physics is discovering what the ancients knew, that our bodies are not solid entities but rather interacting fields of permeable energy, Chi, given to continuous movement and fluctuation both within its perceived boundary as well as outside it. Through the hands of a Reiki practitioner, recipients are affected by the flow of Universal Energy directed into the energetic body of a recipient for health, healing and vitality. \r\n\r\nWear comfortable loose clothing, feel free to bring lunch or buy in cafeteria. CEU'S will be provided for nurses. \r\n\r\nTo register call 1-877-LGH-WELL. \r\n\r\nDuring a Reiki level I training practitioners will:\r\n- Learn to channel basic Reiki energy for healing others\r\n- Develop awareness of energy flow and perceive areas of reduced vitality\r\n- Administer Reiki energy to one self\r\n\r\nTeaching will include:\r\n- The History of Reiki\r\n- Patterns of hand placement\r\n- Four Reiki attunements\r\n- Supervision and practicum\r\n\r\nInstructor information:\r\nCarole Legro, MA,RNC,LMT is a body oriented psychotherapist and Reiki Master with over 20 years experience in Reiki and 30+ years practicing the healing arts.	Open to anyone interested	Yes	For more information or to register call 1-877-LGH-WELL (1-8770544-9355)	Program runs all year by appointment only	For more information or to register call 1-877-LGH-WELL (1-8770544-9355)	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking available in in front of the main entrance	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number.  Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500627	100055	2008-12-02	\N	Nutrition Counseling	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6227	\N	978-937-6000	\N	www.lowellgeneral.org	public_program_email@testing123.net	1-on-1 nutrition counseling with registered dietitian	With a referral from your primary care physician, you will learn healthy eating habits for weight control and disease management during a one-on-one consultation with a registered dietitian. Please call Central Scheduling at 1-978-937-6023 to make an appointment.	Patients must have a physician referral	Yes	Call 1-978-937-6023 for more registration information	Ongoing by appointment	By appointment- call 1-978-937-6023 to register	English	Yes	A physician referral is required. Call for more information and to make an appointment 1-978-937-6023	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506666	100055	2008-12-02	\N	Take Back Your Life- Support Grp. for Chronic Disease Mgmt.	Chronic Disease Support Group	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support Group for chronic disease management	This group is sponsored by Elder Services of the Merrimack Valley and Lowell General Hospital. The Support Group is held the 2nd and 4th Tuesday of the month from 1-2pm in Hanchett Auditorium. \r\n\r\nChronic disease self management is for anyone living with an ongoing medical condition such as arthritis, asthma, cancer, chronic back pain, chronic fatigue syndrome, diabetes, fibromyalgia, heart disease, high blood pressure, lupus, Parkinson's Disease or stroke.\r\n\r\nFor more information, please call 978-937-6038.	Open to all	Free	\N	Open year round	The support group meets the 2nd and 4th Tuesday of the month from 1:00-2:00pm in the Hanchett Auditorium at Lowell General Hospital	English	No	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the front of the main entrance to the hospital	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505797	100759	2008-12-15	\N	Young Parent Program	YPP	45 Merrimack St  2nd Floor	\N	Lowell	MA	01852	978-459-2387	\N	978-452-1421	978-454-9968	www.ccab.org	public_program_email@testing123.net	GED Prep, job prep, internships and job search &placement	Program services pregnant and parenting teens and out of school youth who have "dropped out".  We are funded through DTA and the Career Center of Lowell:  WIA Program. Program operational 5 days per week.  Open at 8AM.	Referral from DTA or qualifying referral screened and approved by WIA (must meet low income guidelines).	Free	\N	Open year round.	Mon-Fri, open at 8AM	English	Yes	\N		Public Parking Garage, on street meter.	Some bus passes.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Tutors	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
502541	100759	2008-12-15	\N	Lowell Food Pantry	\N	174 Central Street	\N	Lowell	MA	01852	978-454-9946	\N	978-452-1421	978-454-1469	\N	public_program_email@testing123.net	Food Pantry provides emergency bagged groceries	The Food Pantry provides emergency bagged groceries to low income clients once a month. We also provide clothes, shoes and kitchen items if available. We distribute furniture to clients in need.	Residents of Lowell, Dracut,Chelmsford,Billerica,Westford,Dunstable,Pepperell, Tyngsboro. Call for financial eligibility.	Free	\N	Services available year round except holidays and weekends.	Service hours: Monday-Thursday 1PM-3:30PM; Program hours: Monday-Thursday 9AM-4PM	English!DELIM!Spanish!DELIM!Khmer	No	Contact program for eligibility requirements. Bring picture id, proof of address for all adults, and social security cards.		Parking is available	\N	Yes	No	\N	No	\N	\N	Delis Rivera	70 Lawrence Street	Lowell	MA	01852	Yes	Contact Delis Rivera for information on volunteer opportunities.	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
507931	102178	2008-12-15	\N	Lawrence Adolescent Development	A Pathways to Family Success Program	70-71 North Parish Road	\N	Lawrence	MA	01843	978-681-4996	\N	\N	\N	www.destinationservices.org	public_program_email@testing123.net	After school adolescent literacy program	Immigrant high school students receive academic support services and ESOL. Their parents receive ESOL instruction.\r\n\r\nThis is a Pathways to Family Success Program. The mission of Pathways to Family Success is to establish a coalition of human service providers who will provide integrated literacy and support services to families with the intent of improving their lives.	Lawrence families	Free	\N	Program follows the school calendar year.	Adolescents: Monday-Thursday 3:15-4:45 PM	English	Yes	Contact Lawrence International High School for enrollment information.		Yes	\N	Yes	No	\N	Yes	\N	\N	Pathways to Family Success	305 Essex Street	Lawrence	MA	01840	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
509383	103476	2008-12-20	\N	119 Gallery	\N	119 Chelmsford Street	\N	Lowell	MA	01851	978-452-8138	\N	\N	\N	www.119gallery.org	public_program_email@testing123.net	Where art meets innovation.	119 Gallery promotes contemporary and new media art, innovative ideas and cutting-edge techniques with a rich and diverse program of exhibitions, performances and community-based arts services; and welcomes people of all ages, backgrounds and means to explore and experience new, innovative art.	Everyone.	Free	\N	www.119gallery.org	See our online calendar of events.	English	No	\N	16 Chelmsford via Chelmsford Street	On street parking.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	Gallery is available for meetings.	Yes	Available to members, sent out online.	Yes	WI-FI	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
508074	103322	2008-12-30	\N	AMIGOS Mentoring Program	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6638	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Adult mentoring for students in grades 4-8	The AMIGOS Mentoring Program matches adult mentors in one-on-one relationships with students from grades 48 at the Gerard A. Guilmette School and Emily G. Wetherbee Schools in Lawrence. Mentors and mentees meet weekly at the students school and focus on improving student's academic achievement.	Students in grades 48 at the Guilmette and Wetherbee Schools.	Free	\N	Open during school year.	Variable; contact for more info about volunteering or student enrollment.	English!DELIM!Spanish	Yes	CORI check (ADD) required for volunteers.		Varies based upon school location.	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are needed to meet for one hour each week at the students school during the lunch hour.	Not Applicable	\N	Yes	Conference room seats 75 persons.	Yes	Fam.ily Services provides agency newsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
508107	101606	2008-06-19	\N	The Clothes Bin	\N	95 East Haverhill Street	\N	Lawrence	MA	01841	978-685-8321	\N	\N	978-689-2006	NeighborsinNeed.net	public_program_email@testing123.net	Provides clothing and goods to those in need.	The Clothes Bin Program receives donations of goods and distributes them. These items include clothing, furniture and infant needs. The Clothes Bin program will also distribute these items through other homeless shelters, nursing homes, social service providers and day care centers.	All clothing and household items are provided free of charge to those who request them.	Free	\N	Open year round.	M-F, 9am-12pm	English!DELIM!Spanish	No	\N		At office site.	\N	Yes	No	\N	No	\N	\N	P.O. Box 447	95 East Haverhill Street	Lawrence	MA	01841	Yes	Volunteers are needed for warehouse operations and special food programs. Call (978) 685-8321	Yes	Many pamphlets about local services or containing other information.	No	\N	Yes	Periodic agency newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	1	\N
508558	103355	2008-07-07	\N	Field Trip Space	\N	56 Island Street	\N	Lawrence	MA	01840	978-685-2343	\N	\N	\N	www.essexartcenter.com	public_program_email@testing123.net	Fieldtrip space for groups of adults and children	EAC offers art-making fieldtrips in our space, including a tour of our galleries, to schools and other groups of adults and children. \r\n\r\nFieldtrip projects can include sculpture, painting, mask making, and mixed media, among others. \r\n\r\nFieldtrips are scheduled for two hours, on average, and cost $40 per hour, plus the cost of art materials.	All	Yes	$40 per hour group rate, plus cost of materials.	Open year round.	Office Hours, 10am-6pm	English	Yes	Please contact Cathy to schedule a fieldtrip: 978-685-2343		\N	\N	Yes	No	\N	Yes	Schedule in advance to avoid conflict with classes or other events.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508734	103355	2008-07-10	\N	Vacation Art Ventures	\N	56 Island Street	\N	Lawrence	MA	01840	978-685-2343	\N	\N	\N	www.essexartcenter.com	public_program_email@testing123.net	3-6 hour, 1-4 day camps for children ages 5-14	A great opportunity to emerge into art in a one to four day adventure. Kid's and Adults!	all	Yes	$20 through $90	Summer and vacation weeks	Monday through Friday. Times vary.	English	Yes	\N		\N	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	1	\N
508349	103322	2008-07-03	\N	Project Venture	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6635	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Youth adventure-learning and community service.	By combining adventure-learning and community service, Project Venture aims to develop the positive potential of young people in the City of Lawrence. Project Venture operates during after school hours at the Guilmette Middle School.	ADD INFO	Free	ADD INFO	ADD INFO	ADD INFO	English	Yes	ADD INFO		\N	\N	No	Not Applicable	\N	Yes	ADD INFO	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	Yes	Family Services, Inc. publishes a periodic newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508371	103322	2008-07-03	\N	Young Childrens Mental Health	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6619	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Mental health support for parents and teachers.	Family Service provides mental health counseling, consultation, and support to parents and teachers of children ages 2.9 to six. Generally, these participants interact with children who exhibit signs of significant mental and emotional stress.	ADD INFO	Yes	ADD INFO	ADD INFO	ADD INFO	English	Yes	ADD INFO		\N	\N	Yes	No	ADD INFO	No	ADD INFO	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	Family Services, Inc. publishes a periodic newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	ADD INFO	No	1	\N
506743	102552	2008-06-20	\N	Boy Scouting	\N	36 AMESBURY RD	\N	Haverhill	MA	01830	978-372-0591	\N	\N	978-373-9134	http://www.yccbsa.org	public_program_email@testing123.net	Boys ages 11-14	Outdoor program designed to teach character, fitness, citizenship, leadership.  Many outdoor activities in a fun safe learning environment.	all	Yes	depends on program offered	year round	varies	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
501573	100407	2008-06-19	\N	Division of Continuing Studies, Corporate & Distance Educ.	\N	One University Avenue	\N	Lowell	MA	01854	978-934-2480	\N	\N	\N	continuinged.uml.edu	public_program_email@testing123.net	Continuing Education	The UMass Lowell Division of Continuing Studies, Corporate and Distance Education is one of the largest professional education programs in New England. Courses are offered at campus, online and on-site at companies and lead to a wide range of undergraduate and graduate degrees. Online graduate degrees are available in Educational Administration, 12 associate and baccalaureate degrees, and more than 30 certificate programs in specialized technical or professional areas. On-site corporate programs are available for customization.	Contact program for more information	Yes	Fees vary.  Contact Financial Aid office for info on assistance, 978-934-4220	Fall, spring and two summer sessions; Winter intersession. On-Campus, online, on-site at companies. Visit website for current course listings and enrollment information	Office hours Monday-Friday 8:30AM-5:30PM.  Ours vary by programs	English	Yes	Contact program or visit website for more information		Parking available on campus	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	A wide variety of subjects available at three locations. For information visit www.uml.edu/libraries	No	\N	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	1	\N
503916	101342	2008-06-19	\N	Internet & Technology Services	Web and email hosting and development	Basement Alumni Hall,	UMass Lowell North Campus, 1 University Ave	Lowell	MA	01854	978-934-4350	\N	\N	\N	thecsl.org	public_program_email@testing123.net	No/Low Cost Internet Services for Non-Profits	The Community Software Lab writes free software  for non-profits \r\nhttp://MVHub.com is an example of our work.	Any non-profit may apply for services.	Sliding Scale	For Fee Services (fee varies with time required):\r\n   - application/database development and/or hosting	Year round.	Mon-Fri (10-6)\r\nSat (variable)	English	Yes	\N		We highly recommend you check our website for directions and parking information before traveling here.	\N	No	Not Applicable	\N	Not Applicable	\N	\N	PO Box 1418	\N	Lowell	MA	01853	Yes	We are always grateful to get full or part-time volunteers with experience in ANY of: Linux, web/software development, networking, graphic design, fundraising, grant-writing, OR marketing.\r\n\r\nInterested in a longer (1 or 2 year) contribution of your time at nominal pay? Consider joining us as an Americorps*VISTA volunteer (www.americorps.org/vista/) in the tradition of all of our staff.\r\n\r\nSee our website for details.	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	1	\N
507293	100803	2008-07-01	\N	Connecting Consumer with Care	\N	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	Connecting Consumer with Care	Connecting Consumer with Care funded by the Blue Cross Blue shield of Massachusetts assists programs participants to complying with the MA Health Care Reform Law that mandates every Massachusetts's resident to have health care coverage by the end of 2007.	Uninsured resident of Massachusetts whose earned income is about or less than 300% of Federal Poverty Level.	Free	\N	Open year round	Monday-Friday from 8:30 AM-5:00 PM	English!DELIM!Khmer	No	proof of residency, proof of income		Available across the street from CMAA	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Some space, 30 seats, is available on a limited basis, preferably business related to CMAA	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Not Applicable	1	\N
502288	100429	2008-05-27	\N	Merrimack Valley Dialysis Center	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8555	\N	\N	978-441-5133	www.saintsmedicalcenter.com	public_program_email@testing123.net	Merrimack Valley Dialysis Center	Dialysis treatment cleanses the blood of wastes and excess fluids that build up when the kidneys are no longer working. MVDC provides patients diagnosed with chronic kidney failure the option of two types of dialysis: hemodialysis and peritoneal dialysis. MVDC has sites in Lowell and Methuen.	Contact program	Sliding Scale	\N	Open year round	Tuesday, Thursday, Saturday 8AM-3:30PM, Monday, Wednesday, Friday 8AM-9PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Parking available in garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	1	12-18-08 left message
507315	101408	2009-01-15	\N	Network Health is a Commonwealth Care managed care plan	\N	101 Station Landing	Fourth Floor	Medford	MA	02155	888-257-1985	888-257-1985	\N	\N	www.network-health.org	public_program_email@testing123.net	A program for adults who need affordable health care	Network Health Forward, our Commonwealth Care plan offers: \r\n- Monthly premiums and co-payments based on what you make\r\n- No deductibles\r\n- Coverage for doctors' visits, prescriptions, and hospital stays\r\n- Vision services\r\n- X-rays and lab tests\r\n- Dental coverage to those who qualify\r\n- Free health support by phone any time you need it with our 24/7 NurseLine\r\n- Doctors and a customer service team who speak many languages\r\n- Personal health coaches to help you with your health choices\r\n- Discounts on Weight Watchers and gym memberships to help you stay healthy	To be eligible, you must be:\r\n- 19 or older\r\n- uninsured\r\n- A Massachusetts resident\r\n- A U.S. citizen, U.S. national, eligible resident, or resident with special status\r\n- Make a yearly income under $31,000	Free	\N	Open year round	Monday - Friday\r\n8:30 a.m. - 5:00 p.m.	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	\N		Parking garage on-site	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Yes	0	\N
508525	102178	2009-01-15	\N	Lawrence Lead Abatement Program	City of Lawrence Community Development Department	147 Haverhill Street	\N	Lawrence	MA	01840	978-620-3510	\N	\N	978-683-4894	www.glcac.org	public_program_email@testing123.net	Asssistance for deleading homes with children under age 6.	Provides lead safe housing to the residents of Lawrence through community outreach, education training, technical assistance and financial assistance for lead abatement renovations. The City has contracted with GLCAC Inc. to provide support for implementation of this program. \r\n\r\nLow and moderate-income homeowners and developers may be eligible for Lead Paint Hazard Control Grants of up to $8,000 per unit for lead abatement to residential properties. In addition, the program offers zero percent interest loans to owner-occupants to subsidize costs in excess of the grant award.	Owner-occupants and developers of single or multi-family residential properties must income qualify and have a child under the age of 6 reside at the property or regularly visit the property. Income guidelines change annually. For income guideline information, please contact the program.	Yes	Grants have no fees associated with them, however, the loans do have some fees.  Loans are at 0.0% interest and are deferred until property sold or deed changed.	Open year round.	M-F, 8:30 am - 4:30 pm	English!DELIM!Spanish	Yes	For income guideline information, please contact the Lawrence Lead Abatement Program.		\N	\N	Yes	Not Applicable	\N	Yes	Wait list is on a first come, first serve basis.  However, priority is given to families with children under age 6 with elevated blood lead levels or at risk.	\N	GLCAC, Inc.	305 Essex St	Lawrence	MA	01840	No	Some internship opportunities.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	Properties must be located in the City of Lawrence.	\N	No	0	\N
504620	101485	2008-07-15	\N	Lawrence Tobacco Free Partnership	\N	One Canal St, Entrance C	\N	Lawrence	MA	01840	978-688-2323 x 8618	\N	\N	978-975-7779	www.tobaccofreepartnership.org	public_program_email@testing123.net	Community smoking intervention projects	A grant funded smoking intervention project of the Mass Tobacco Control Program that works with communities, organizations and residents to reduce youth acess to tobacco, promote tobacco cessation, protect the public from secondhand smoke and eliminate tobacco related disparities.	none	Free	\N	none	none	English!DELIM!Spanish	Not Applicable	\N		Parking	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	Lawrence Tobacco Free Partnership	One Canal St,, Entrance C	Lawrence	MA	01840	Not Applicable	\N	Yes	The Tobacco Free Partnership office is located at the Northeast Center for Healthy Communities, which includes a free, health resource library. On line at www.nc4hc.org	No	\N	No	\N	Yes	www.tobaccofreepartnership.org\r\nwww.trytostop.org	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	Not Applicable	0	\N
505269	101903	2008-07-30	\N	A Woman's Concern Pregnancy Health Services	AWC	103 Broadway	\N	Revere	MA	02151	781-284-8747	\N	866-435-7292	781-284-1167	www.awomansconcern.org	public_program_email@testing123.net	Free Pregnancy Testing & Ultrasound	We offer free pregnancy testing and ultrasound exams when indicated. We also provide Earn While You Learn parenting programs, mentoring programs for both moms and dads, infant & maternity clothing, baby supplies. Options counseling for women in an unplanned pregnancy. All our services are offered at no charge.	Are services are free to anyone in need.	Free	\N	Open Year Round	Mon - Thurs 9:30-5:30\r\nFri 9:30-3:30	English!DELIM!Spanish	No	\N		Free parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	\N	Yes	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	0	\N
504400	101595	2008-08-01	\N	All Staff, LLC	\N	1565 Main St	Building 2, floor 3	Tewksbury	MA	01876	978-863-0361	\N	978-863-0361	978-863-1974	http://www.allstaff.us	public_program_email@testing123.net	Support Staff	We provide relief staff to human service agencies.	Must be CPR and first aid trained	Sliding Scale	\N	Open year round	24/7 phone coverage	English	No	Proof of CPR and First Aid certificate		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Yes	0	\N
502222	100715	2008-08-02	\N	Educational Director	LTC Channel 8 & 95	246 Market Street	\N	Lowell	MA	01852	978-458-5400	\N	\N	978-937-0361	http://www.lctc.org	public_program_email@testing123.net	Bridging the Digital Divide in Lowell	The Lowell Community Technology Consortium is a collaboration of organizations and individuals using technology to serve the citizens of Lowell, Massachusetts. Over twenty non-profit organizations participate in the Consortium through the development of technology labs and training programs to educate and enrich the lives of Lowell residents.	varied by organization.	Sliding Scale	varies according to organization.	varies	varies	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Each member organization has a different requirment.		varies	\N	Yes	No	\N	No	\N	\N	PO Box 803	\N	Lowell	MA	01853	Yes	variety of needs go to:\r\nhttp://www.lctc.org/go/vol/index.shtml	No	\N	No	\N	No	\N	Yes	varied.	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
509273	103454	2008-09-03	\N	Home Preservation Center	\N	450 Merrimack St.	\N	Lowell	MA	01852	978-970-0600	\N	\N	978-970-0609	\N	public_program_email@testing123.net	Foreclosure prevention services	Foreclosure prevention counseling provided to homeowners who are facing or at risk of facing foreclosure.	greater Lowell area	Free	\N	year round	M-F, 9-5 with evening hours available upon request.	English!DELIM!Spanish!DELIM!Khmer	Yes	\N	41 Lawrence-Lowell	metered, on street	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	0	\N
509295	103091	2008-10-01	\N	Connecting Families	\N	439 South Union	\N	Lawrence	MA	01840	978-682-9222	1-800-541-3004	\N	978-681-9534	www.mspcc.org	public_program_email@testing123.net	Home visiting and parenting education	Home visiting for any parent in the gReater Lawrence area that needs case management, connection to community resources and parenting education	Any parent with child under 16 years old	Free	\N	open year round	M-F 8:45-5:00pm	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	Conference room 20-30 person capacity.  Must be scheduled in advance.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	Not Applicable	0	\N
501210	100154	2008-08-01	\N	Elections / Census Commission	\N	375 Merrimack Street	Room 5	Lowell	MA	01852	978-970-4046	\N	\N	978-970-4089	www.lowellma.gov	public_program_email@testing123.net	Voter registration, election over-site & local census	The Elections / Census Commission conducts the local census, is a voter registration site, and oversees elections.	Not applicable	Yes	3.00 for residential certificate	Open year round	Monday-Friday 8AM-5PM	English!DELIM!Khmer	No	photo id		Metered street parking is available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
500220	100033	2008-07-15	\N	Homeless Family Shelters	Milly's Place and Merrimack House	767 Merrimack Street & 205 Worthen Street	\N	Lowell	MA	01852	978-452-5410	\N	\N	978-459-4558	www.comteam.org	public_program_email@testing123.net	Homeless shelter and services for families	CTI's Homeless Shelters assist families in locating affordable housing while providing a temporary, safe, and supportive environment for homeless families. Through the use of advocacy and education, we assist families in becoming increasingly self-sufficient. Our program offers educational, financial, legal, parenting and housing resources.	Referral ONLY through MA Dept of Transitional Assistance.  Income eligible determined by DTA emergency assistance. Must be 20 years old with a family and free from drugs & alcohol	Free	\N	Program available year round	Monday-Friday 8:30AM-5PM. 24 hour staff coverage	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	DTA referral, contact MA DTA.		Parking available	Transportation provided by program	Yes	No	\N	Not Applicable	\N	\N	CTI, Attn: Atilio Winfrey, Kiki Gavin	767 Merrimack Street	Lowell	MA	01852	Yes	Interested volunteers should call 978-452-5410	No	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
509141	101243	2008-08-04	\N	Community Market	\N	735 Broadway Street	\N	Lowell	MA	01854	978-454-7272	\N	\N	978-454-1717	www.mvfb.org	public_program_email@testing123.net	Produce Market	Free produce is available to residents of the NORTH COMMON VILLAGE in Lowell.  Whenever possible, locally grown fruits and vegetables are set up in a market style at the corner of Hancock AV and Father John Sarantos Way, and residents of the development select fresh items that are typically more expensive than their canned counterparts.  Children's atcitities and health screenings are also available.	Residents of the North Common Village.	Free	A $1 donation is suggested for a reusable bag.	July 12-November 15, 2008	Saturdays from 9:30-11:30am	English!DELIM!Spanish!DELIM!Khmer	No	Proof of address must be presented.		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	E-mail or call for an application and description	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
507491	102574	2008-08-18	\N	Merrimack River Stream Team	\N	600 Suffolk Street	Fifth Floor	Lowell	MA	01854	978-275-0120	\N	\N	978-275-0125	www.merrimack.org	public_program_email@testing123.net	Merrimack River Stream Team	Merrimack River Stream Team Volunteer\r\n\r\nJoin the new Merrimack River Stream Team to help follow up on the Shoreline Survey of the entire 50 mile length of the Merrimack River in Massachusetts conducted May 2008.  The Shoreline Survey was a one-time visual and qualitative survey of the river that identified some problems on the river that need to be rectified and areas that need to be preserved.  Projects will include river clean-ups, creating an educational brochure for Merrimack Valley businesses and homeowners, and other projects important to Stream Team members that will protect the Merrimack River watershed and its inhabitants.	Everyone	Free	\N	Projects can be undertaken year round	M - Th 10:00 AM to 6:00 PM	English	Yes	\N		Visitor parking lot and free on-street parking	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	Mid-Stream News - view online at www.merrimack.org	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
505665	102057	2008-08-19	\N	Art on Wheels	\N	365 West Meadow Rod	\N	Lowell	MA	01854	978-821-0953	\N	\N	\N	www.cultureiscool.org/whatiscool/cool-projects/coolmobile	public_program_email@testing123.net	Art and literacy projects for children 2-6	Art on Wheels is a series of art and literacy projects for children ages 2-6 at childcare centers in Lowell. Currently serving Children's Corner, Art on Wheels connects reading to arts and crafts.	Children in Lowell	Free	\N	Open year round	Monday through Thursday, 9:30 a.m. - 10:30 p.m.	English	Not Applicable	\N		\N	Art on Wheels, a program of the Art Ship Enterprise, brings art projects directly to childcare centers.	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	We are looking for creative, energetic and caring individuals to volunteer one  morning a week facilitating art activities with children ages 2-6.	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	Not Applicable	0	\N
502981	100935	2008-08-20	\N	Social Security, SSI & SSDI	\N	151 Warren Street, Suite 300	\N	Lowell	MA	01852	978-452-5509	800-772-1213	\N	978-452-8945	www.socialsecurity.gov	public_program_email@testing123.net	Administer Social Security, SSDI & SSI	The Social Security Administration is the government agency responsible for administering the Social Security, Social Security Disability Insurance (SSDI) and Supplemental Security Income(SSI)programs.	Contact program	Free	\N	Services available year round	Monday-Friday 9AM-4PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Contact program	02 Belvidere	Limited parking available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		978-458-5702	Federal program - covers all areas through many local offices	Available by calling 800 number: English, Spanish	Yes	0	\N
500033	100011	2008-09-02	\N	A.D.A.M Project	\N	45 Merrimack Street, Suite 227	\N	Lowell	MA	01852	978-654-6731	\N	978-996-3052	978-452-0838	www.bbbslowell.org	public_program_email@testing123.net	After School Program for Boys ages 12 - 18	The A.D.A.M. Project is an after school program for boys ages 12-18 using psycho-educational groups to help teens with gang issues, school, violence prevention, and pregnancy prevention. A combination of classroom discussion, leadership training, community service and recreational activities are used.	Must be a male resident of Lowell, age 12 - 18	Free	\N	Program runs year round, with a summer schedule as well.	After school and out-of-school hours. Contact program for specifics.	English	Yes	Please contact Jim Conlon at Big Brother Big Sister		Parking is available in the John Street garage	Limited transportation provided by program.	Yes	No	\N	Yes	There is a waiting list with an average of 5-10 people	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Jim Colon	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
501056	100121	2008-09-02	\N	Youth Drop-In	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Youth Drop-In Program for ages 6 - 13	Youth Drop-In is open to youths aged 6-13. Youth can play gym games, group activities, crafts, and go swimming (45 minute swim time). Swimming is always supervised by trained lifeguards.	Ages 6-13	Yes	Members: FREE\r\nNonmembers: $4/night	Program runs September-June.	Monday, Wednesday, Friday nights 6:30-8:30PM	English!DELIM!Spanish	No	\N		Parking is available.	\N	Yes	No	\N	No	\N	On the phone only English is available	\N	\N	\N	\N	\N	Yes	Volunteers assist with supervising children. Contact Fred Braun.	No	\N	Yes	Space is available for racquetball/ volleyball leagues; soup kitchen; pool parties, etc.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
507249	100330	2008-09-04	\N	Cambodian Citizenship	\N	9 Central Street	Suite 203	Lowell	MA	01852	978-654-6957	\N	\N	978-454-6992	www.onelowell.net	public_program_email@testing123.net	to assist the Cambodian Refugee Communities in Lowell	ONE Lowell, Building a New American Community is to assist the Cambodian Refugee Communities in Lowell to understand their eligibility to become the US Citizen and the benefits from being the Citizen. Our efforts will bring the Cambodian Refugee Communities to use the power of citizenship in order for their voice to be heard and therefore focusing on organizing around citizenship, providing training for the civic questions, demonstrating the use of a voting booth and more. Organizing is a participatory approach that can build the relationships, teaches skills, educates and informs.	Cambodian Refugee Communities in Lowell Wanting to understand their eligibility to become the US Citizen and the benefits from being the Citizen.	Free	\N	Open year round	Mon - Fri  9 am to 5 pm	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
507744	103135	2008-11-03	\N	Chelmsford Stamp Club	\N	P.O. Box  2342	\N	Littleton	MA	01460	978-256-2256	\N	\N	\N	\N	public_program_email@testing123.net	We meet to learn and talk about philately.	We discuss different aspect of collecting stamps or covers and postal history for all countries. We often have special speakers or show films/slide shows on some aspect of philately.	anyone	Yes	$15 per year membership for adults\r\n$1 per year membership for youths (under 18 years old)	1st & 3rd Monday every month except July, August and December	7:30PM - 9:00PM	English	No	\N		Parking lot behind the church	not provided	No	No	\N	Not Applicable	\N	\N	27 Brentwood Rd.	\N	Chelmsford	MA	01824	Not Applicable	\N	Yes	Complete set of Scott's Catalogue	Yes	Large, well lighted room, with bathroom facilities.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500451	100055	2008-12-02	\N	Children's Bereavement Group	\N	Lowell General Hospital	295 Varnum Ave.	Lowell	MA	01854	978-937-6129	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support group for children who have had a loved one die	The Children's Bereavement Support Group is a bereavement support group for children ages 11-13 who have suffered the loss of a parent or loved one. Meets the second Monday of each month from 3:30pm to 4:30pm. Please call Cammie Caron at 978-937-6129 for more information.	Open to children ages 11-13 who have experienced the death of a loved one	Free	\N	Meets Year Round	Meets the second Monday of each month from 3:30 p.m. to 4:30 p.m.	English	No	Call Cammie Caron at 978-937-6129 for more information.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509361	100055	2008-12-02	\N	Woman's Alcoholics Anonymous/AWOL Meeting	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Alcoholics Anonymous Meeting for Women	Woman's Alcoholics Anonymous/AWOL Meeting held every Sunday evening from 6:30-8:00pm in Hanchett Auditorium at Lowell General Hospital. The meeting will be open for the first three weeks and then will be a closed meeting. For more information please contact Debbie Linnehan at 978-454-1991.	Open to any women who are interested.	Free	\N	Open year round	Meetings held every Sunday evening from 6:30-8:00pm in Hanchett Auditorium at Lowell General Hospital.	English	Yes	For more information please contact Debbie Linnehan at 978-454-1991.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the hospital	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505478	100055	2008-12-02	\N	New Mothers Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	1-877-544-9355	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	drop-in support group for new mothers	LGH New Mothers Support Group is a drop-in support group for new mothers. This group provides a place to come with your questions and concerns or just to meet other new mothers. This group meets in the Lowell Room every Tuesday from 10:00-12:00pm and every Wednesday from 4:30-6:30pm. The program is free and registration is not necessary. LGH Registered Nurses/Lactation Consultants lead this group. Please call Doreen Gallant or Laurie Sanz at 978-937-6334.	Open to all new mothers	Free	\N	offered year round	This group meets every Tuesday from 10:00-12:00pm and every Wednesday from 4:30-6:30pm.	English	No	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the front of the hospital as well as in the front of the School of Nursing.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
509416	100484	2009-01-05	\N	Middlesex Community College  MCAS Pathways Program	MCAS Program	33 Kearney Square	\N	Lowell	MA	01852	978-656-3284	800-818-3434	\N	978-656-3150	www.middlesex.mass.edu	public_program_email@testing123.net	Free MCAS Prep. course	Middlesex Community College offers a Department of Elementary and Secondary education funded MCAS Program available to any student who has taken and failed the MCAS test in Math and/or ELA.  This course is offered free of charge in the Fall, Spring and Summer.	MCAS Program available to any student who has taken and failed the MCAS test in Math and/or ELA.	Free	\N	3 cycles	The next program is scheduled for January 26-March 5, 3-5pm.\r\nELA meets Mondays and Wednesdays and Math meets Tuesdays and Thursdays.	English	Yes	Completed Registration form prior to the start of the program.		Lower Locks Garage	N/A	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	\N	Yes	Middlesex Community College, Lowell Campus, City Building	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
503751	100616	2008-06-20	\N	Residential and Education Program	\N	100A Haverhill Street	\N	Methuen	MA	01844	978-682-5276	\N	\N	978-688-4932	www.st.annshome.org	public_program_email@testing123.net	Chapter 766 approved residential school	Residential School for emotionally and behaviorally disturbed children ages 5-17	Determined by local school district or other referring agency.	Yes	Tuition typically paid by public schools or agencies.	365 days per year, 24 hours per day	365 days per year, 24 hours per day	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Yes	No	\N	Yes	Sometimes there is a short wait list	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Conference rooms are occasionally made available for public or professional education use. Call for more info.	Yes	Twice a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Generally Northeastern Mass and Southern NH but there are no gepgraphic limitations	\N	No	1	\N
508569	103355	2008-07-07	\N	Gallery Exhibitions	\N	56 Island Street	\N	Lawrence	MA	01840	978-685-2343	\N	\N	\N	www.essexartcenter.com	public_program_email@testing123.net	Free gallery events and exhibits.	EAC's spacious galleries shows the work of local artist in a great variety of mediums. Gallery shows change every 4-6 weeks and are free to the public. \r\n\r\nOn average the opening receptions attract an audience of 200 persons.	Free, open to all.	Free	\N	Open year round	Open M-F, 10am-6pm	English	No	\N		On street or in lot nearby.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508547	103355	2008-07-10	\N	Daytime, Afternoon and Evening Art Classes	\N	56 Island Street	\N	Lawrence	MA	01840	978-685-2343	\N	\N	\N	www.essexartcenter.com	public_program_email@testing123.net	Daytime, after school and evening art classes	Provides daytime, after school and evening programs for children and adults. Art classes are taught in a variety of mediums, including: ceramics, photography, drawing and painting to silk-screening, bookmaking, sculpture and frescoes.	All. Children and adult workshops separate.	Yes	Fees about $100 for adult programs.	Open year round.	Office hours, 10am-6pm	English	Yes	Contact to enroll in classes		Parking on road or nearby in lot.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Artists and those enthusiastic about art. Also looking for help with fundraising.	No	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	Yes	1	\N
508327	103322	2008-07-03	\N	Adolescent Family Life Program	AFL	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6629	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Community collaboration helping pregnant/parenting teens.	The AFL Program is a community collaboration, led by Family Service, that aims to help pregnant and parenting teens gain the skills they need to create a healthy future for themselves and their children. \r\n\r\nThe program provides individual case management, group support, parent education, family involvement and assistance with accessing community resources.	ADD INFO	Free	ADD INFO	ADD INFO	ADD INFO	English!DELIM!Spanish	Yes	ADD INFO		\N	\N	Yes	No	ADD INFO	Yes	ADD INFO	\N	\N	\N	\N	\N	\N	No	ADD INFO	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
504477	100044	2008-06-19	\N	Community Outreach & Health Promotion	\N	17 Warren St.	\N	Lowell	MA	01852	978-746-7838	\N	\N	978-453-8043	www.lchealth.org	public_program_email@testing123.net	Outreach on health center services and health topics	The Community Health Promotion Department is comprised of several multi-lingual Community Outreach Workers who facilitate workshops and community outreach sessions on health topics and services available at the health center. Community Health Workers are bi-lingual and speak Portuguese, Spanish, Swahili and Khmer.	Free to all interested individuals and organizations.	Free	No fees charged.	Running year round. Call for information on community involvement and events or to have a Community Health Worker drop off literature about the Health Center and programs offered.	Monday - Friday 8:00AM-5:00PM. Also special community events.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County	\N	\N	Swahili	No	1	\N
506897	102618	2008-06-19	\N	Immigration / Naturalization Assessment	\N	321 Central Street	\N	Lowell	MA	01851	978-452-6655	978-452-6655	\N	\N	www.summersatlaw.com	public_program_email@testing123.net	Legal representation for criminal and immigration law.	Summers, Summers & Associates, P.C. provides legal services and expertise in all areas of criminal law, immigration and nationality law.	None	Yes	$150 for an initial consultation.	Opened year round.	Monday through Friday	English	Yes	\N		Yes, off-street parking.	\N	Yes	No	\N	Not Applicable	\N	\N	321 Central Street	\N	Acton	MA	01720	Not Applicable	\N	Not Applicable	\N	Yes	321 Central Street\r\nLowell, MA  01851	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
507304	100803	2008-07-01	\N	Elderly Service	\N	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	Elderly Service	With a major funding from the MA Department of Public Health(DPH)and small partial fundings from CDBG and ORI, the program provides Lowell's Southeast Asian elders with such assistance and services as health education on chronic diseases common among the elderly population; transportation (interpretation on a case by case basis)to a medical appointment; assistance in filling out N-400 form (citizenship form.	Lowell's Southeast Asian elders with chronic diseases	Free	\N	Open year round	Monday-Friday from 8:30AM-5:00PM	English!DELIM!Khmer	Yes	Southeast Asian elders with chronic a disease(s)		Available across the street from CMAA building	Case by case basis	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Some space, 30 seats, is available on a limited basis, preferably business related to CMAA	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	1	\N
506226	102387	2008-06-05	\N	Pro-bono Professional Assistance	\N	190 Academy Road	\N	North Andover	MA	01845	978-604-8803	\N	\N	\N	http://www.jerichoroadlawrence.org/	public_program_email@testing123.net	Pro-bono professional expertise	Jericho Road Lawrence provides Lawrence non-profits with free professional assistance on specific projects.  Some examples include strategic planning, board development, technical assistance, business development, etc.	Must be a non-profit that provides a charitable service to the City of Lawrence.  Should be a 501c3, or qualify for being a 501c3	Free	Our services are strictly pro-bono	Open year round	5 days/week	English	Yes	Call 978-604-8803 for an appointment		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Call 978-604-8803 for volunteer opportunities	No	\N	Yes	North Parish Unitarian Universalist Church, 190 Academy Road, North Andover.	Yes	Quarterly	Yes	www.jerichoroadlawrence.org	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	The non-profit can reside in a town/city outside of Lawrence, but must dedicate a suabstantial portion of its charitable activities to the City of Lawrence	\N	No	2	\N
502387	100429	2008-05-27	\N	Pulmonary Rehabilitation	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8238	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Pulmonary Rehabilitation	Outpatient program for people who have emphysema, bronchitis, asthma or other lung disorders.	Contact program for information	Free	\N	Open year round	Monday-Friday	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Parking available in lot on Pawtucket Street.	\N	Yes	No	\N	No	\N	\N	220 Pawtucket St.	\N	Lowell	MA	01854	Yes	Interested volunteers should contact Cheryl 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	1	12-18-08 left message and paged
505060	100407	2009-01-15	\N	STARTS (Student and Teacher Arts)	Durgin Hall , Room 116	35 wilder St., Suite 1	\N	Lowell	MA	01854	978-934-4452	\N	\N	978-934-2017	www.uml.edu/centerforarts	public_program_email@testing123.net	Performing arts presented to students & teachers	A series of professional, performing arts experiences presented to school children who are bused to UML's Durgin Hall.  Serves over 35,000 middle & elementary school students per year.  Includes theatre, music, dance, storytelling, poetry, physical theatre & musical theatre.	Public, private and homeschool groups with students over the age of 4.	Yes	Student tickets are between $6 - $9 based on performance and number of students in group.	Performances from October - May.	Various days; shows at 9:30 & 11:30 am	English	Not Applicable	\N		Instructions mailed with reservations confirmation.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers, usually retired teachers, serve as ushers in seating school groups.	Not Applicable	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
506699	101936	2009-01-15	\N	Community Support Services	CSS	11 Lawrence St.	\N	Lawrence	MA	01841	978-686-0090	\N	\N	\N	www.glfhc.org	public_program_email@testing123.net	Case managmenet and services referrals	Child and Youth C&Y Offers case management and referrals to adults, children over the age of six, and the elderly.\r\n\r\nIf you are...\r\n pregnant and scared\r\n in need of health care\r\n in need of health insurance\r\n living in an unsafe home\r\n in need of transportation to medical appointments\r\n at-risk and need HIV counseling and testing\r\n HIV positive and need services\r\n struggling with an addiction\r\n homeless, alone and looking for shelter\r\n confused about how to manage electric or gas bills\r\n in need of help with DTA or SSI applications\r\n looking for referrals to food pantries, or thrift shops\r\n...let Community Support Services help you.	Anyone	Free	\N	Walk in service	Various	English!DELIM!Spanish	No	\N	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	On street	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Other languages spoken	Not Applicable	0	\N
503960	101375	2008-08-20	\N	Citizenship Classes	\N	1 Ballard Way	Suite 101	Lawrence	MA	01843	978-683-7316	\N	\N	978-683-9075	asiancentermv.org	public_program_email@testing123.net	Citizenship classes	10 week classes are offered (in English) to prepare participants for citizenship interviews.	no eligibility restrictions.	Free	\N	Ten-week classes are offered between September and June as needed.  Citizenship classes are not offered during the summer.	P.M. classes held on Monday 6 - 8:30	English	No	Registration Monday through Thursday 9 - 2.  Otherwise it is recommended to call for an appointment		Parking available	Not available	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	3 large classrooms	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
506006	102079	2008-06-19	\N	Summer Storm	\N	101 Main St	\N	Salem	NH	03089	978-416-2002	\N	\N	\N	www.youthstorm.org	public_program_email@testing123.net	Summer Camp	Summer Storm is a missions trip to SoulFest 2008. Those who participate in Summer Storm will be ministering through prayer, prophecy and listening in the YouthStorm prayer tent. They will also partner with SoulFest in specific ministry opportunities throughout the Fest, as well as serve in practical areas around the Fest.	All ages, races, and sexes over 12.	Yes	See website for details.	This years trip will be held July 30-Aug 3rd.	One-time per year.	English	Not Applicable	\N		\N	\N	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Camp Counselors	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	1	\N
507392	102893	2008-07-15	\N	Free Community Yoga	\N	305 North Main Street	Tactician Building	Andover	MA	01810	978-866-7944	\N	\N	\N	www.unionsudioyoga.com	public_program_email@testing123.net	Free Community Yoga	Free yoga class Friday 11:15-12:30pm for both the beginner and experienced student.	all	Free	Community Class is free.\r\nOther classes are 14.00.\r\nDiscount class cards available.	Year round	Studio holds yoga classes daily.	English	No	\N	21 Andover Shuttle!DELIM!32 Andover!DELIM!73 Lawrence	yes	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Books available on loan from the studio.	Yes	Located in the Tactician Building in Shawsheen Village, centrally located with plenty of off-street parking, UnionStudio is a beautiful place to learn and practice yoga.	Yes	email updates	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	Yes	0	\N
507183	102783	2008-07-15	\N	Lawrence Families & Youth Initiative	F.Y.I.	96 East Haverhill Street	\N	Lawrence	MA	01841	978-557-8400	\N	800-792-5200	978-557-8410	\N	public_program_email@testing123.net	Community Social Service Program	Combine and integrate community, agency and family resources; to promote policy changes that will support creative, nurturing services for strengthening families and communities.  Building on principles of family centered casework and strenght-based services, Patch provides a unique opportunity to affect changes in the way the community responds to the needs of individual families and the needs of the community as a whole. The Patch partners work together to promote practice development and improvements in systemic functioning that support child safety well-being and permanency.	If you live in the area we serve you are eligible for services.	Free	\N	Open year round	Monday through Friday\r\n9:00am till 5:00pm	English!DELIM!Spanish	No	You may call and set up an appointment but walk-ins are available.	34 Prospect Hill	Plenty of parking behind the church where we are located.	\N	Yes	Not Applicable	\N	Not Applicable	\N	Youth Community Coalition - Youth group for all teens - Meets every second and fourth Tuesday of the month at 5:30pm\r\n\r\nParent Council - Meets every 3rd Monday of the month at 5:30pm	\N	\N	\N	\N	\N	Yes	Volunteer opportunities exist at F.Y.I., please contact Delilah Rivera, Program Coordinator at 978-557-8400, Ext. 5	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	If you live on one of the following streets you may be serviced at F.Y.I.\r\n\r\nAlbion St., Allyn Terrace, Arlitt Place, Arrow Terrace, Avon St., BelmontSt., Bennington St., Berkely St., Berkely Ct., Birch St., Boehm St., Brook St., Bruce St., Bunker Hill, Buswell St., Campo Seco, Canton St., Chestnut St., Custer St., East Haverhill St. (to High St.), Elm St., Eutaw St., Fern St., Fitz St., Fulton St., Garden St., Giuffrida Pl., Gratchen St., Hershfield Ct., High St., Highlawn St., Howe St., Jackson St., Keilmau Ct., Kendall St., Kendrick St., Kenshley Ct., Knox St., Lawrence St. (to Haverhill St.), Lexington St., Lincoln St., Logan St., Maple St. (to Lawrence St.), McAuliffe Ave, Mechanic St., Milton St., Monmouth St., Montgomery St., Newbury St., Oak St., Orchard St., Park St. (to Lawrence St.), Prospect St., Saratoga St., Saunders St., Stearns St., Summer St., Summit St., Sunray St., Sunset St., Swan St., Thorndike St., Trenton St., Trinity St., Union St. (to Haverhill St.), Webster Ct. and Wilmot Street.	90% of our staff are bilingual: English and Spansih	Not Applicable	0	\N
507194	102783	2008-07-15	\N	Youth Community Coalition	YCC	96 East Haverhill Street	\N	Lawrence	MA	01841	978-557-8400	\N	800-792-5200	978-557-8410	\N	public_program_email@testing123.net	Youth Group for all teens	Vision Statement\r\nTo create community ownership, hope, Self-determination, and pride where children and families are safe and have meaningful futures through collaboration and mutual responsibility. We will achieve this through promoting empowerment and involvement of community members who seek support and guidance from each other, the TAM, and PATCH agencies.	You must be between 13 and 19 years of age.	Free	\N	YCC meets every 2nd and 4th Tuesday of the month.	2nd and 4th Tuesday of the month\r\n5:30pm	English!DELIM!Spanish	Not Applicable	\N	34 Prospect Hill	Plenty of parking behind the church where we are located.	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	Some of the youth speak Spanish and all of the youth speak English.	Not Applicable	0	\N
503828	101276	2008-07-15	\N	Massachusetts Health Officers Association	MHOA	P.O. Box 417	\N	West Boylston	MA	01583	508-835-3309	\N	\N	508-835-3309	www.mhoa.com	public_program_email@testing123.net	Educational Events	Quarterly Education Programs, in March, June, Sept.\r\n\r\nAnnual Health Conference, see Website	Municipal and State Public Health Officials as well as interested Massachusetts Residents	Yes	Vary from program to program, see website.	Year round w/ periodic trainings.	Mon.-Fri. 9:00-2:00	English	Yes	\N		Event locations vary.	\N	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Not Applicable	0	\N
504202	101463	2008-07-15	\N	FoodSource Hotline	\N	145 Border Street	\N	East Boston	MA	02128	1-800-645-8333	1-800-645-8333	\N	\N	www.gettingfoodstamps.org	public_program_email@testing123.net	Information and referral for food stamps and emergency food.	The FoodSource Hotline provides information and screenings for the food stamp program. Counselors can also provide information about emergency food programs and low cost food programs across the state.	none	Free	\N	Year round	M-F, 8-5	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	Helping Hands newsletter for agencies that assist clients with food stamp applications	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	all languages	Not Applicable	0	\N
504213	101463	2008-07-15	\N	www.gettingfoodstamps.org	\N	145 Border Street	\N	East Boston	MA	02128	1-800-645-8333	\N	\N	\N	www.gettingfoodstamps.org	public_program_email@testing123.net	Food Stamps eligibility calculator and application info.	This website offers easy-to-read, friendly information about the Food Stamp Program in Massachusetts, including a confidential eligibility calculator, a printable application and other forms.	useful for anyone who is a Massachusetts resident	Free	\N	website available: 24 hours a day, 7 days a week	website available: 24 hours a day, 7 days a week	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	Project Bread staff offer Food Stamp regulations and resource training sessions to interested non-profits and other organizations across Massachusetts.	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Not Applicable	0	\N
503630	101210	2008-07-15	\N	SpringBoard	\N	20 Lexington Road	\N	Concord	MA	01742	978 369 9602 X457	\N	\N	\N	www.jerichoroadproject.org	public_program_email@testing123.net	No-cost capacity building services for Lowell non profits	Jericho Road offers a wide range of free executive and technical services--from strategic planning and board development to data base and web site design--to help Lowell organizations save money AND improve their performance. We work with 47 nonprofit clients and have collaborated on 170 projects from 2003-2005.	Lowell based non profits with a preference to projects and organizations that improve Lowell's economic sustainability	Free	\N	Year around	9-5	English	Not Applicable	\N		We are located in Concord, MA, and have available parking. However, we usually come to a nonprofit client's location in Lowell for any meetings.	\N	Not Applicable	Not Applicable	\N	No	There may be a waitlist for certain types of professional assistance. In general, if you need services within a week, we will not be able to find you a match that fast.	We offer a WIDE range of services to nonprofits to help them build capacity.	\N	\N	\N	\N	\N	Yes	Always looking for a good volunteer intern to do administrative or program-related work.	No	\N	Yes	Lovely, small.	Yes	Dedicated to informing our Lowell clients about the work we are doing and can do in Lowell.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	Not Applicable	0	\N
501837	100517	2008-07-15	\N	Area Agency on Aging	\N	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-892-0890	978-687-1067	www.esmv.org	public_program_email@testing123.net	Builds capacity of communities to provide elder services	Elder Services is the designated Area Agency on Aging for this region, to build the capacity of the communities to provide services and programs for elders. Administers the funding of Older Americans Act funds to local agencies. Develops an area plan with an action plan, based on assessment of the communities. Targets services and programs to those in the greatest social and financial need. Funded programs include, meals on wheels, congregate meals, health promotion programs, legal assistance, emergency shelter, minority outreach, support for caregivers.	Not applicable	Free	Elders participating in services funded with Older Americans Act funds are given the opportunity to make a contribution	Services available year round.	Monday through Friday 8:00AM to 5:00PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	Not Applicable		specific to individual programs funded in the community.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Money managers, nursing home ombudsmen, health benefits counselors,drivers, shoppers, companions, medical advocates	No	\N	No	\N	Yes	Our newletter is free and published 4 times a year.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	we serve 23 cities and towns in the Merrimack Valley.  Along with those checked above - we also serve - Newbury, West Newbury, Georgetown, Groveland, and Rowley.	Our portugese and khmer language capacity is limited.	No	0	\N
501848	100517	2008-07-15	\N	Elder Abuse and At-Risk Elders Program	Crisis Intervention Unit	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-922-2275	978-687-1067	www.esmv.org	public_program_email@testing123.net	Investigates  physical & emotional, neglect of elderly	Massachusetts law requires all licensed professionals who have a reasonable cause to believe an elder person, age 60 plus, may be suffering from or has died as a result of abuse or neglect, are required to report the abuse. Elder Services is the local protective service agency investigating all reports of physical, emotional, sexual neglect, and financial exploitation. Elder Services will follow up on all reports and provide services as appropriate to relieve the situation.	Age 60+; Referrals may be made to Elder Services Age Info Department	Free	No fees charged	Services are available all year	Monday-Friday 8:00AM to 5:00PM.  Coverage of open protective cases provided 7 days a week 24 hours a day.  Hotline is also available 5:00 pm to 8:00 am 7 days a weel.	English!DELIM!Spanish	Not Applicable	Call for information on the program. Any information needed will be requested over the phone		Parking available at corporate office in Lawrence	Located on bus route. Transportation may be provided if authorized.	Yes	No	\N	No	\N	(800) 924-4222 TDD Number	\N	\N	\N	\N	\N	Yes	Volunteers needed as companions. Contact program for more information.	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	We also serve Georgetown, Groveland, Newbury, North Andover, Rowley, and West Newbury	Interpretors can be arranged for other languages.	No	0	\N
501870	100517	2008-07-15	\N	Elder Care Fund	\N	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-922-2275	978-687-1067	www.esmv.org	public_program_email@testing123.net	Assistance for elders when no other financial resources	The Elder Care Fund, established in 1990, provides assistance for elders when there are no other financial resources. First priority is emergency assistance and second, in-home services. Solely supported through fund raising activities and other donations. Elders are offered the opportunity to make donations.	Contact information and referral department at Elder Services	Free	\N	Contact program	Monday through Friday 8:00AM to 5:00PM	English	Yes	Contact program		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Georgetown, Groveland, Newbury, West Newbury, Boxford, Rowley	\N	No	0	\N
501881	100517	2008-07-15	\N	Family Caregiver Support Program	\N	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-922-2275	978-687-1067	www.esmv.org	public_program_email@testing123.net	Support for families providing care for elder relative	The Elder Services Family Caregiver Support Program offers a variety of services and programs, many at low or no cost, to assist families who are providing care for an elder relative. Core services include: Elder Care Advice - professional assistance to identify every option to plan for care; scholarships - funds for occasional or emergency relief from care giving; support groups; education and training; and information and referral.	Program open to anyone providing care for elderly family members	Free	\N	Program available year round	Monday-Friday 8:00AM to 5:00PM; until 8PM at 1-800-AGE-INFO; Saturday 9AM-12PM at 1-800-AGE-INFO	English!DELIM!Spanish!DELIM!Khmer	Yes	Referrals accepted from agencies, organizations and self referrals. Contact program for more information		Free parking available in lot next to building	\N	Yes	No	\N	No	\N	On the phone: Spanish; At program site: Spanish, Khmer	\N	\N	\N	\N	\N	Yes	Many opportunities are available to help older people with tasks such as money management. Contact program for more info.	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Boxford\r\nGeorgetwon\r\nNewbury\r\nWest Newbury\r\nNorth Andover\r\nRowley	\N	No	0	\N
501892	100517	2008-07-15	\N	Home Care Program	\N	360 Merrimack Street, Building 5	\N	Lawrence	MA	01843	978-683-7747	800-892-0890	800-922-2275	978-687-1067	www.esmv.org	public_program_email@testing123.net	In-home supportive services for elderly	Elder Services manages the Massachusetts Home Care Program for this region. A range of in-home supportive services to help older people remain safe and independent at home as long as possible are offered. Eligibility is based on income and need. Clients cost share based on income. Services such as homemaker, personal care, meals on wheels or meal preparation, transportation, and adult day care are a few of the service options. Case managers, trained social workers, provide assessment, care planning, and authorize services as appropriate, in coordination with registered nurses.	Ages 60+; Financial requirements apply if home care services are needed. Call for more information.	Sliding Scale	Clients cost share based on income. Call for more information.	Program services are available all year round.	Monday-Friday 8:00AM to 5:00PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Call for more information		\N	\N	Yes	No	\N	Yes	At times diuring the fiscal year, because of budget issues - we are forced to start a waiting list for folks who need services.  Call for more information	(800) 924-4222 TDD Number	\N	\N	\N	\N	\N	Yes	Volunteers used as shoppers, drivers, nursing home Ombudsman, health benefits counselor, money managers and more. Please call.	No	\N	No	\N	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Boxford\r\nGeorgetown\r\nNewbury\r\nWest Newbury\r\nNorth Andover\r\nRowley	\N	No	0	\N
502849	100847	2008-07-15	\N	Health Screenings & Immunizations	\N	50 Billerica Road	\N	Chelmsford	MA	01824	978-250-5241	\N	\N	978-250-5244	www.townofchelmsford.us	public_program_email@testing123.net	Blood Pressure, immunizations, sreening and health education	The Chelmsford Board of Health provides:\r\nBlood pressure checks the first Thursday of each month 9AM-12PM on a walk-in basis; Appointments required for immunizations for children and adults; Mantoux (TB) testing; Lead Paint screening (age 9 months - 6 years); cholesterol; Home visits for assessment - referral by MD or home health agency.	Services available to Chelmsford residents only	Yes	Fees vary:\r\nTB testing: $10 \r\nCholesterol sreening: $15 (dates announced in Lowell Sun)\r\nImmunizations $15.00 each for adults and children\r\nCopies of immunizations/screening records done previously at CBOH: $5.00	Board operates year round	Monday-Friday 8:30AM-4:30PM	English	Yes	Appointments necessary. Call program for more information.	15 Chelmsford via Rte. 129!DELIM!16 Chelmsford via Chelmsford Street	Parking available on site	\N	Yes	No	\N	Yes	If Cholesterol screening is full we will put names on a waiting list and call if there are any cancellations	\N	\N	\N	\N	\N	\N	Yes	Volunteers are used for flu clinics and other events, contact Public Health Nurse for more information.	Yes	Contact Board to access health information resources.	Yes	Space available for meetings pertinent to BOH concerns, ie health, medical, West Nile Fever, bio-terrorism, etc.	Yes	Section in Town of Chelmsford community newsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Chelmsford, MA	\N	\N	\N	No	0	\N
507271	102827	2008-07-15	\N	Prime Staffing Services,LLC	\N	40 Witherbee Ave	\N	Revere	MA	02151	781-336-9123	\N	781-336-9123	781-715-6171	\N	public_program_email@testing123.net	Relief Staff Agency	As a Relief and Support Staff Agency that provides and promotes services and support for people with:\r\n- Developmental disabilities\r\n- Mental illness\r\n- Emotional difficulties\r\n- Behavioral health disabilities\r\n- Bipolar disorder\r\n- Schizophrenia\r\n- Autism\r\n- Paranoia\r\n- Alzheimer's\r\n- Substance and alcohol abuse\r\n- Domistic violance\r\n- and the Elderly\r\nWe take great pride in the ability to offer highly qualified staff exactly when and where you need them\r\nPrime Staffing maintains the highest standards when it comes to staff selection and training. The goal of our services is to provide assistance and support that will enable the people we serve to live rewarding and productive lives in the community.	We are an equal opportunity employer.Customer satisfaction and competitive pricing have placed us at the forfront of temporary help, staff placement.	Sliding Scale	\N	Open year round	24 hours/7 days week , phone coverage	English	No	\N		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	Merrimack Valley, Greater Lowll,Greater Lawrence,Northshore, everywhere in Massachusetts.	\N	No	0	\N
501012	100121	2008-07-15	\N	YMCA Camp Massapoag	\N	234 Hall Street	\N	Dunstable	MA	01827	978-649-7521	\N	\N	978-454-8982	www.greaterlowellymca.org	public_program_email@testing123.net	Summer day camp ages 5 - 15	Camp Massapoag is a traditional YMCA day camp on 42 wooded acres on the shores of Massapoag Pond providing: swimming lessons, boating, nature crafts, teambuilding activities, sports and games, and a variety of weekly special events. Open to children ages 5 to 15.  Busing from area towns and extended care available.	Ages 5-15.	Yes	Day Camp $160-$200 Weekly\r\nPioneer Teen Camp $260-$300  2 Week session\r\nCIT Camp $360-$400 3 Week session\r\nFinancial Assistance for those who qualify	10 weeks  June - August. Call for info beginning Feb.	June-August: Monday-Friday 9AM-4PM\r\nExtended care available  7AM-6PM	English	Yes	Call for information		Parking available on site	Private bus transportation is provided, with stops throughout Greater Lowell	No	Not Applicable	\N	No	\N	\N	PO Box 153	\N	Dunstable	MA	01827	Yes	Qualified applicants attend staff training and complete background and reference checks before working at camp.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
503927	101353	2008-07-16	\N	Check Out a Company	\N	235 West Central Street Suite 1	\N	Natick	MA	01760	508-652-4800	\N	\N	\N	www.boston.bbb.org	public_program_email@testing123.net	Check Out a Company Before Doing Business	The Bureau offers a variety of programs, including checking out companies before doing businesses.  Visit our website and search 90,000 business Reliability Reports in Eastern MA, ME & VT.  All of our services are free to the public.  Access all other services through our website	Consumers and businesses.	Free	\N	24 hours a day, 7 days a week online.  Other services available during normal business hours.	24 hours a day, 7 days a week online.  Other services available during normal business hours.	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	By being an Arbitrator (Judge for the Day), individuals are able to give back to the community and assist consumers and business in this quick free process.  Our volunteer arbitrators hear and decide on cases ranging from Automobiles to Swimming Pools and everything in between.\r\n\r\nArbitration is a valuable service to consumers and businesses.  It serves the Bureaus mission and goals by creating an environment to easily resolve consumer-business disputes.\r\n\r\nOther volunteer opportunities are available here: http://www.boston.bbb.org/about/volunteer.asp	Yes	The BBB has a wealth of information and topics for consumers and businesses.  Visit www.boston.bbb.org for purchase and protection tips, practical pointers, advisories, warnings, alerts, news releases, educational campaigns, etc.  We also have an educational video series for individuals, libraries, etc.	Not Applicable	\N	Yes	The BBB provides a newsletter to its members through the For Members Only section of our website.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Translate the website by clicking the appropriate flag.	Not Applicable	0	\N
506039	102189	2008-07-16	\N	Structured Outpatient Addiction Program	SOAP	66-76 Winter Street	\N	Haverhill	MA	01830	978-373-1181 ext 45	\N	\N	978-374-7605	\N	public_program_email@testing123.net	SOAP is a full time outpatient addiction treatment program.	The Structured Outpatient Addictions Program provides treatment to homeless and other individuals with addiction or cooccuring mental health disorders while allowing them to maintain participation in the community, continue to work or attend school, and be a part of family life.  Program services include; Intake and assessment, treatment planning, group and individual therapy, family therapy as needed, case management, random drug screens, aftercare planning, aftercare groups, assistance with shelter placements as needed, and referrals for psychiatric and other needed services.	The day program treats adult males and females and teens age 16 and up. An evening SOAP for Adolescents is also available M-F from 4-6:30 pm for youth ages 12 to 16.   Individuals appropriate for this program are diagnosed with a substance use disorder or cooccuring mental health disorders.  They are able to participate in a group setting but have been unable to maintain recovery with traditional weekly treatment.  For clients covered under the Massachusetts Behavioral Health Partnership (MBHP/Mass Health), SOAP services are covered for eight weeks.  Services are also covered for those who have Neighborhood/Mass Health and Network.  The program also can accept uninsured Massachusetts residents under our Department of Public Health contract.  A copay may be required for those individuals.An Adolescent Program ages 12-18 will be opening soon.	Sliding Scale	\N	The program Schedule is Monday through Friday from 9AM to 4PM.  Aftercare groups are held weekly on Tuesdays at 1:30.  Adolescent SOAP runs M-F from 4:00 - 6:30 pm.  Extended recovery groups are also available in the evening.	Monday through Friday 9AM to 4PM (day/adult)\r\nAdolescent SOAP M-F 4:00-6:30pm	English	No	\N		On and off street parking are available.	Bus lines are nearby.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	Serving the Haverhill Area	\N	No	0	\N
502112	100660	2008-07-16	\N	Commuter Education, Information & Ridematching	\N	P.O. Box 121242	\N	Boston	MA	02112	1-888-426-6688	888-426-6688	\N	\N	www.commute.com	public_program_email@testing123.net	Information on 50 public & private transportation providers	Bilingual customer service line to receive information on travel options by carpooling, vanpooling, transit, biking or walking....live service M-F 8:00 - 5:30 pm -- with automated after hours service	Open to everyone	Free	\N	Services available all year	Monday-Friday 8:00AM - 5:00PM	English!DELIM!Spanish	Not Applicable	Contact program with origin, destination and commute times		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	submit your email address on our website to receive our newsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Yes	0	\N
502123	100660	2008-07-16	\N	MassRIDES -- Employer Outreach/Worksite TDM Programs	\N	P.O. Box 121242	\N	Boston	MA	02112	1-888-426-6688	888-426-6688	888-426-6688	617-892-6090	www.commute.com	public_program_email@testing123.net	employer outreach	Locally based outreach coordinators, onsite transportation events, customized promotional materials, regional training seminars, individual ridematching for employees, assistance with vanpool formation and promotion of transit services to worksite, bilingual customer service	Open to everyone	Free	\N	Services available year round	Monday-Friday 8:00AM - 5:30PM	English!DELIM!Spanish	No	\N		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	send us your email to receive enewsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Yes	0	\N
503025	100968	2008-07-16	\N	A Ripple in Time	a pregnancy loss support group	c/o Donna McDonnell	14 Blossom Drive	Billerica	MA	01821	19783761559	\N	19783761559	\N	http://rindyshope.org	public_program_email@testing123.net	Pregnancy Loss Support Group	A Ripple in Time (and the HOPE Group)offers support for families/friends experiencing pregnancy loss from miscarriage, stillbirth and/or neo-natal deaths. This group meets on the first Monday of each month at 7:00 p.m.  St. Theresa's Church Parish Hall, 370R Boston Road, Billerica, MA.	All parents, grandparents, families and friends affected by pregnancy loss are welcome.	Free	\N	Program runs year round	A Ripple in Time meets on the first Monday of each month at 7:00 p.m. in Room Six of St. Theresa's Parish Hall, 370R Boston Road, Billerica, MA.	English	No	Self-referral.	13 Billerica via Edson	Parking available in lot next to building	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Includes books on pregnancy loss. Many news letters on same from local and national organizations	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
507348	102849	2008-07-16	\N	Ellen H. Korin, M.Ed.	\N	10 COACH RD	\N	Lexington	MA	02420	781-861-6431	\N	339-223-0928	781-652-0436	www.ellenhkorin.com	public_program_email@testing123.net	Consultation, Coaching & Professional Development	Expert, confidential consultation, coaching and professional development services.  Specializing in autism spectrum disorders, particularly Asperger Syndrome and Non Verbal Learning Disabilities. Experienced special educator and author of Asperger Syndrome An Owner's Manual: Volume One for ages 10 to 15 and Volume Two for older adolescents and adults.	Ages 12 through adulthood	Yes	Private payment - will adjust fee under certain conditions	Year round	Will arrange	English	Yes	\N		yes	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
509053	103399	2008-07-17	\N	Prepare medical equipment to go to poor countries overseas	\N	1600 Osgood Street	\N	North Andover	MA	01845	978-557-5510	\N	\N	978-557-5525	www.imecamerica.org	public_program_email@testing123.net	Health Care Global Relief Volunteers Needed	Advance Standards of health for poor countries by preparing medical equipment to be shipped overseas	N/A	Free	\N	Open Year Round	Monday-Friday  9-4\r\nSaturdays on scheuled basis	English	Not Applicable	\N		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	We offer multiple opportunites for volunteering at our facility in North Andover MA. Opportunities include  but not limite to :\r\npreparation of medical equipment\r\npacking and preparing hospital suites\r\nsorting medical supplies\r\nassisting with preparation of linens	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	International Relief effort	\N	Not Applicable	0	\N
503201	101034	2008-07-18	\N	Health and Safety Services	\N	177 Ward Hill Ave.	\N	Ward Hill	MA	01835	978-372-6871	\N	\N	978-374-2233	www.arcmv.org	public_program_email@testing123.net	Health & Safety Trainings	The American Red Cross of Merrimack Valley provides health and safety courses such as First Aid, CPR, babysitting, workplace CPR and First Aid training, pet first aid, information regarding all water safety courses and sites, life guarding, boating, CPR for the professional rescuer, automated external defibrillator, preventing disease transmission.	There are no eligibility requirements for these programs	Yes	Varies depending on the course. Contact program for fees.	Classes offered year round. Class length varies depending on program. Program capacity is generally 10 per class	Monday-Friday 9:00AM-4:30PM	English	Yes	Contact program with the name of the course you are interested in taking		Parking is available in front of the office	Located on the Ward Hill bus route.	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	Contact Deb Duxbury for information on volunteering 978-372-6871	No	\N	No	\N	Yes	\N	Yes	Our Website is www.arcmv.org	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA	\N	We also cover these additional Massachusetts towns, North Andover, Georgetown, Groveland, Pepperell, Billerica	All classes are given in English. Staff are available to speak Spanish on the phone.	Yes	0	\N
503256	100484	2008-07-22	\N	One Stop Career Center Services	The Career Place	100 Sylvan Road, Suite G100	\N	Woburn	MA	01801	781-932-5500	888-273-9675	\N	781-932-5566	www.careerplacejobs.com	public_program_email@testing123.net	Provides employment and career development services	Provides employment and career development services to individuals, and workforce development assistance to employers. A resource library is available with materials on career development, job training, employer job listings and financial aid information.\r\nA chair lift and computers with voice assisted software are available, as well as large print forms and screen enlargement software. Several workstations use adjustable tables to allow wheelchair access.  Services are free, with a few, advanced workshops carrying minimal fees.	Services open to all -  no fee to register, use library and attend most workshops.  Advanced workshops may have a minimal fee.	Free	All core services are free. Some workshops, individual services or some services for employers may have a fee attached.	Services available year round	Monday, Wednesday, Friday 8:30AM-5:00PM; Tuesday & Thursday 8:30AM-7:30PM; Saturday 9AM-1PM	English!DELIM!Spanish	No	\N		Parking available in lot next to Center.	Local Woburn transportation stop (Rt. 38) is roughly 1/4 mile from Center	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer opportunities are available, especially in the resource library. Contact Kathy Andre 781-932-5523	Yes	\N	Yes	Meeting space is available, but limited. Contact Jan Flight at 781-932-5501	Yes	Monthly calendar with schedule of all workshops	Yes	For job search purposes only.	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	No residency requirement to access general services. Residency in the Metro North region may apply for some training programs.	Haitian Creole, American Sign Language	Yes	0	\N
501111	100154	2008-07-22	\N	Basic Education / Wilson Reading Classes	\N	50 Father Morrissette Blvd.	\N	Lowell	MA	01852	978-937-8989	\N	\N	978-458-9007	\N	public_program_email@testing123.net	Basic literacy classed for adults	Classes are offered to adults who are illiterate, meaning that they read at a below fourth grade academic level. Classes are offered at night in a small group of no more than eight students. Classes are offered from 6PM-9PM Monday & Wednesday or Tuesday & Thursday.	Must be 16 or older and live in Lowell or one of the seven surrounding communities (must have picture id).	Free	\N	Classes run Sept.-May. It's best to get into small classes in Sept. before they fill, although registration is ongoing.	Monday-Thursday 9AM-9PM; Friday 9AM-3PM	English	Yes	Registration is daily at 9AM or 6PM. Calling ahead is not necessary. Bring picture id with proof of residency. Placement tests		Parking available in nearby city lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Program recommends volunteers contact Pollard Memorial Library's Literacy Volunteers program to volunteer	No	\N	No	\N	No	\N	Yes	Lowelladulteducation.com	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
508338	103322	2008-07-03	\N	Mujeres en Accion Protegiendose Ahora	MAPA	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6646	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Integrated substance abuse and HIV prevention program.	MAPA engages women at risk for HIV/AIDS in a group program that utilizes an integrated substance abuse and HIV prevention curriculum. The goal of the program is to reduce womens risk for HIV infection resulting from their own or their partners substance abuse.	ADD INFO	Free	ADD INFO	ADD INFO	ADD INFO	English!DELIM!Spanish	Yes	ADD INFO		\N	\N	Yes	Yes	ADD INFO	No	ADD INFO	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	Family Services, Inc. publishes periodic newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508360	103322	2008-07-03	\N	Strengthening Multi-Ethnic Families & Communities	Family Strengthening	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6600	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Parent education curriculum for minority, at-risk families.	Family Strengthening is a nationally recognized parent education curriculum developed for minority, at-risk families that enables them to improve their parenting skills by examining the influences of personal history and culture.	ADD INFO	Yes	ADD INFO	ADD INFO	ADD INFO	English	Yes	ADD INFO		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	ADD INFO	Not Applicable	1	\N
508063	103322	2008-07-22	\N	Counseling Center	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6618	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Professional mental health and substance abuse counseling	The Counseling Center provides professional mental health and substance abuse counseling to the Greater Lawrence community. Family Service is deeply committed to supporting individuals, couples, families, and children through our mental health counseling, psychiatric consultation, and various support groups.\r\n\r\nThe Counseling Center is licensed by the Massachusetts Department of Public Health. The staff can help with issues such as:\r\n\r\n    * Depression and anxiety\r\n    * Alcohol and other substances\r\n    * Relationship problems\r\n    * Stress\r\n    * Separation and divorce\r\n    * Loss\r\n    * Difficulties at work/school	Residents of Greater Lawrence	Sliding Scale	Most of the clinical services are reimbursable through health insurance. \r\n\r\nFamily Service is a provider for the following insurances as well as others: Blue Cross/Blue Shield, Children's Medical Security Plan, Harvard Pilgrim Health Care, HMO Blue, MassHealth, Tufts Health Plans.	Open Year Round	M-F, 8AM-5PM	English!DELIM!Spanish	Yes	Insurance information, IDs.		Parking	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Conference room seats 75 persons.	Yes	Family Services has a newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	Spanish services depending on staff availability.	No	0	\N
508294	103322	2008-07-22	\N	Employee Assistance Program	EAP	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6666	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Employer-based mental health services.	Family Service EAP offers a range of mental health services to local employers. Professional counselors provide confidential support to employees and their family members.\r\n\r\nThese services include:\r\n + Problem assessment and referral\r\n + Counseling\r\n + 24-hour emergency phone line\r\n + Drug-free workplace services\r\n + Crisis management services.	Employers in Greater Lawrence area.	Yes	Please call 978-327-6666 or email: EAP@familyserviceinc.com. Employers who which to join EAP services may ask for Kay Berthold Frishman. \r\n\r\nIn addition, EAP, in collaboration with the Merrimack Valley Chamber of Commerce, offers the Chamber Consortium EAP to small businesses with 250 or fewer employees.	Open year round.	M-F 9am-5pm	English	Yes	Employers must contact EAP.		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
508140	103322	2008-07-22	\N	Northern Essex County Court Appointed Special Advocates	CASA	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6615	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Volunteer program assisting abused and negleted children.	The Northern Essex County CASA (Court Appointed Special Advocate) Program recruits, trains and supervises volunteers who serve as advocates for abused and neglected children whose cases are heard in the Lawrence Juvenile and Newburyport District Courts.\r\n\r\nA high level of training and support is given to community volunteers who come from all walks of life to work with the child welfare system to make sure that children who have been abused or neglected can move quickly through the foster care system and into safe, loving and permanent homes. No special experience is required. Once accepted, volunteers are trained in courtroom procedures, social services, the juvenile justice system and the special needs of child victims.	Adult volunteers only.	Free	Volunteer service opportunity.	Open year round.	Contact program during:\r\nM-F, 8am-5pm	English	Yes	CORI check, photo ID required.		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	CASA Volunteers spend only 15 to 20 hours a month helping a child in need, but make a difference that lasts a lifetime.	Not Applicable	\N	Yes	Conference room seats 75 persons.	Yes	Periodic newsletter of Family Services covers CASA program.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
508129	103322	2008-07-22	\N	Putting Children 1st	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6600	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Court mandated parent education class	Putting Children 1st is a court mandated parent education class for divorcing parents in Massachusetts (Approved by the Massachusetts Probate and Family Court). \r\n\r\nAll classes are held at Merrimack College's McQuade Library in the Arundel Room on the 3rd floor. Please do not bring children to class. The court approved Parent Education guidelines require that spouses attend separate sessions. \r\n\r\nThe focus of class time is to understand the effect of divorce on children. The assumptions are that parents want to do a good job raising their kids and that divorce is an unsettling experience for kids and for each of their parents.\r\n\r\nExperienced, state-approved counselors lead this course. The leader must have more than five years licensed counseling experience and group-training experience.	According to court regulation, all parents of children under eighteen years old, filing for divorce must attend the classes.	Yes	The cost of a two session class is $80.00 per person. \r\n\r\nThe $80.00 fee is nonrefundable unless two full business days notice is given prior to date/time of class. To cancel call (978) 327-6650 and leave a message.	Open year round.	Contact program for dates of classes.	English	Yes	Complete all the necessary information at:\r\nhttp://www.familyserviceinc.com/Program_files/ParentEd.htm		Parking at Merrimack College's campus.	\N	Yes	No	Please do not bring children to class.	No	\N	Feel free to call the receptionist to confirm your place in class. To receive a confirmation call, you must enter a Confirmation Phone on the registration form. If you do not leave a confirmation phone you will not receive a confirmation call.	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	\N	Yes	Conference room seats 75 persons.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Yes	0	\N
503124	101001	2008-07-30	\N	Debt Management Plan	\N	8 Winter St.	Suite 700	Boston	MA	02108	866-889-9347	866-889-9347	\N	617-960-8299	www.moneymanagement.org	public_program_email@testing123.net	Assist in setting up payment plan to creditors	More than 17,000 families and individuals are currently enrolled in a Debt Management plan through Consumer Credit Counseling Service of Southern New England. This repayment program, developed with the cooperation of creditors, helps consumers regain control of their financial lives.	Certain requirements are necessary for particular debt management plans, contact program for more information	Yes	Fees vary. Payment options: money order or automatic payment	Services available year round. Program length varies, depending on client - enrollment dates and deadlines are client choice	Monday-Thursday 8AM-8PM; Friday 8AM-6PM; Saturday 8AM-2PM; Sundays: appointments available by phone, internet, in person may call to schedule	English!DELIM!French!DELIM!Spanish	Yes	Contact program with information on: current budget, current credit card bills and income statements		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Occasional volunteer opportunities may be available. Contact HR representative Crystal Hodgkins, 617-960-8133 for information	No	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Southern New England: MA, CT, RI	\N	Yes	0	\N
503135	101001	2008-07-30	\N	Financial Education Programs	\N	8 Winter St.	Suite 700	Boston	MA	02108	866-889-9347	866-889-9347	\N	617-960-8299	www.moneymanagement.org	public_program_email@testing123.net	Financial Education Programs	More than 20,000 area residents attended one of the more than 800 CCCS educational programs on money management, budgeting, credit and housing in 1999. Committed to the prevention of financial difficulties through education, CCCS works with social service organizations, schools and the business community to prepare individuals for lifelong financial well-being. CCCS co-sponsors The Center for Personal Financial Education and Information with the University of Rhode Island on their Kingston, RI campus. The center offers a variety of educational programs such as workplace financial literacy programs, including the popular Women's Financial Information Program.	Programs open to everyone	Yes	Call for more information	Services available year round	Monday-Thursday 8AM-8PM; Friday 8AM-6PM; Saturday 8AM-2PM; Sunday: counseling can be done in person, by internet and by telephone.	English!DELIM!Spanish	Yes	Call for information on the program you would like to attend. You will be informed if you need to bring any materials		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Southern New England - MA, RI and CT	On the phone: English and Spanish; At program site: English, occasionally Spanish	No	0	\N
503146	101001	2008-07-30	\N	Housing Counseling	\N	8 Winter St. 7th Floor	\N	Boston	MA	02108	866-889-9347	866-889-9347	\N	617-960-8299	www.moneymanagement.org	public_program_email@testing123.net	Mortgage default, pre-purchase & reverse mortgage counseling	CCCS is a HUD-approved housing counseling agency offering mortgage default, pre-purchased and reverse mortgage counseling.	Program open to all. HUD may refer clients, but it is not required.	Yes	Fees vary. Contact program for more information. Payment options: cash or money order	Services available year round, enrollment is ongoing	Monday-Thursday 8AM-8PM; Friday 8AM-6PM; Saturday 8AM-2PM; appointments done by phone, internet, in person.  Sunday: appointment scheduling only	English!DELIM!Spanish	Yes	Contact program. Have information about current budget, mortgage information, credit debt and income available		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Southern New England: MA, RI, CT	\N	Yes	0	\N
503157	101001	2008-07-30	\N	Personal Financial Counseling	\N	8 Winter St. 7th Floor	\N	Boston	MA	02108	617-960-8136	800-208-2227	\N	617-960-8299	www.creditcounseling.org	public_program_email@testing123.net	Financial Counseling for individuals	Certified professionals conducted more than 25,000 first time counseling sessions for Southern New England residents in 1999. In-depth counseling delivers lasting improvements to people's financial well-being unlike 20 minute quick fixes, which do not address the root cause.	Program open to all	Free	Fees vary. Payment options: cash or money order	Services available year round, enrollment is ongoing. Deadlines dependent on enrollment date	Monday-Thursday 8AM-8PM; Friday 8AM-6PM; Saturday 8AM-2PM; Sunday: appointment scheduling only.  appointments can be done in person, by internet, by telephone	English!DELIM!Spanish	Yes	Bring all credit card statements and bills, current budget, and income statements to appointment		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Southern New England - MA, CT, RI	\N	Yes	0	\N
503102	100990	2008-08-01	\N	Billerica Council on Aging	Senior Center	25 Concord Road	\N	Billerica	MA	01821	978-671-0916	\N	978-671-0916	978-671-1347	www.town.billerica.ma.us, wwwbillericacoa.org	public_program_email@testing123.net	Programs and services for senior citizens of Billerica	The Billerica Council on Aging provides outreach to homebound seniors, volunteer services, support groups, senior property tax work-off program, respite-companion care, daily lunch program and meals on wheels.\r\nActivities include aerobics, country line dancing, yoga, tap dancing, tai chi, bingo, social time,cribbage, chess, crafts, quilting, knitting, folk art, acrylic art, blood pressure screening, podiatry, flu shots, AARP taxes preparation, tax relief program, help with health insurance through SHINE program, social events, etc.  The Senior Center operates a volunteer run gift shop 10-2 Monday through Friday.	Must be age 60 or older to participate in social,health and recreational programs. Must be 60 or older to participate inthe senior lunch program and referrals required for Meals on Wheels	Yes	Some program activities such as craft and exercise classes have fees usually $3.00 per class.  Call for more information.   Lunch program requests a $2.00 donation	Services available year round	Monday-Friday 8:30AM-4PM & Some Evenings	English	No	Unless calling for more information on a particular area of interest.		Parking available on site.	Road Runner available - need 2 days notice, $1 each way	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Susan Dollar. Volunteers always needed for medical transportation, shopping, kitchen support, etc.	No	Paperback book exchange	Yes	Call for more information	Yes	Monthly publication, THE OUTLOOK, available by subscription and add drop offs spots  town of Billerica.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Westford, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
506688	101936	2008-08-01	\N	The Family Pharmacy	\N	34 Haverhill Street	\N	Lawrence	MA	01841	978-686-0090	\N	\N	\N	www.glfhc.org	public_program_email@testing123.net	The Family Pharmacy is a service of GLFHC	A pharmacy designed for both patients of GLFHC and members of the general public.  Located on the first floor of Greater Lawrence Family Health Center	Everyone	Yes	Typical pharmacy fees vary depending on insurance	Open Year Round	M-F 9A-5P\r\nSat. varied	English!DELIM!Spanish	Not Applicable	\N		Yes - multiple locations around GLFHC Clinic	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	0	\N
504378	101584	2008-08-01	\N	Indian Hill Music School	\N	36 King Street	\N	Littleton	MA	01460	978-486-9524	\N	\N	978-486-9844	www.indianhillmusic.org	public_program_email@testing123.net	Music lessons and classes for all ages.	Indian Hill Music School offers private music lessons, group classes, ensembles and workshops for all ages, abilities and musical styles. Professional faculty of 75 musician/educators and a personalized placement process. Financial aid available for all programs.	Programs open to all.	Yes	Fees vary by program. Financial aid available for all programs.	Fall, Spring and Summer Terms	Lessons and classes held 9 am - 10 pm on most days.	English	Yes	Registration form required. (All forms can be downloaded from our website.)		Free parking behind building.	\N	Yes	No	\N	Not Applicable	\N	\N	PO Box 1484	\N	Littleton	MA	01460	Yes	We offer volunteer opportunities for seniors, moms and student interns. Projects include office help, mailings, and ushering at concerts.	Not Applicable	\N	Yes	150 seat recital hall is available for meetings and functions	Yes	Published twice a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509130	102475	2008-08-04	\N	Weekly food pantry	\N	58 Newbury Street	\N	Lawrence	MA	01840	978-681-8768	\N	\N	\N	www.breadandroseslawrence.org	public_program_email@testing123.net	Soup kitchen	We serve an evening meal monday thru thursday\r\nand saturday	All are welcome	Free	\N	open year round	10am to 7pm	English	No	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
506545	102398	2008-08-04	\N	Community Day Supports	\N	165 Winter Street	\N	Haverhill	MA	01830	978-374-9122	\N	\N	978-374-1532	http://www.crc-mass.org/	public_program_email@testing123.net	Flexible day supports including work and non-work activities	In an effort to provide the most flexible and individualized supports as possible, CRC provides a day service that allows people to participate in an array of day services ranging from community recreational and social activities to volunteer and paid work opportunities.  The program provides wheelchair accessible transportation to and from community activities	Eligible for Department of Mental Retardation or Region 10 New Hampshire funding.	Yes	Funding is typically through the Department of Mental Retardation or Region 10 New Hampshire	Open all year	Monday through Friday 9am to 3pm	English	Not Applicable	\N	01 Haverhill - Methuen - Lawrence	\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Career Resources also serves Southern New Hampshire	\N	Not Applicable	0	\N
501826	100341	2008-08-04	\N	Assistive Technology Solutions	\N	One Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	www.classinc.org/assistive_technology.htm	public_program_email@testing123.net	Provides technology training	The Assistive Technology Solutions program at CLASS provides individual assessment, installation and training of assistive technology equipment that best suits the person's needs. Follow-up and adjustment are provided. As Assistive Technology Center is located on site in the Lawrence location. Work is also done with individuals in their homes.	Must be 17 or older and live in the North Shore / Northeast Region of Massachusetts.	Sliding Scale	None if eligible for Massachusetts Rehabilitation Commission funding. Fees are negotiated on sliding scale for other consumers	Services are available year round. There is a waiting list with an average length of 3 months.	Monday-Friday 8AM-4PM	English	Yes	\N		\N	Parking is available on site. Limited transportation may be available through program.	Yes	No	\N	Yes	\N	Call with information on disability and functional limitations as well as your goal for use of assistive technology	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Several conference rooms may be available for professional development activities. Not open to general public.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	0	\N
506534	102398	2008-08-04	\N	Employment Services	\N	165 Winter Street	\N	Haverhill	MA	01830	978-374-9122	\N	\N	978-374-1532	http://www.crc-mass.org/	public_program_email@testing123.net	Range of community and center based employment supports	Employment Services\r\nVocational Assessment:\r\nPeople are provided with short term vocational assessments to help identify the persons interests and aptitudes.\r\nJob Placement Services:\r\nPeople are provided assistance in becoming employed in the community.  These services include resume development, job search activities and job matching support.  Short and long term job coaching services help ensure a successful employment outcome.	Funding through DMR, MRC, school systems or other funding sources required.	Yes	Funding through State agencies, schools or other sources required.	Open all year.	Typically Monday through Friday 8am - 3pm	English	Not Applicable	\N	01 Haverhill - Methuen - Lawrence	\N	Depending on funding and location, transportation may be available.	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Career Resources also serves Southern New Hampshire	\N	Not Applicable	0	\N
506523	102398	2008-08-04	\N	Residential Services	\N	165 Winter Street	\N	Haverhill	MA	01830	978-374-9122	\N	\N	978-374-1532	http://www.crc-mass.org/	public_program_email@testing123.net	Wide range of community living support services	Community Living Services\r\n\r\nPeople with developmental disabilities are supported to live in their community in a variety of settings including apartments, condominiums, their own homes, or with family care providers The overall goal is to help people to live safe, healthy and comfortable lives while being while being fully engaged members of their community.\r\n\r\nCommunity Support Services provides individualized support for people living on their own, with family or other caregivers.  Support ranges from 3 hours to 20 hours per week. This service can also provide fiscal services such as helping someone manage their checking account, bill paying and spending.\r\n\r\n\r\nEnhanced Family Care  assists people in finding a shared living setting within someones home.\r\n\r\n\r\nResidential Support Services provides 24 hour support to people living in their own home or in Career Resources residences.	Eligible for Department of Mental Retardation residential services.	Free	Funding from DMR required.	Depends on the type of supports required.	Depends on the type of supports required.	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	0	\N
502772	100825	2008-08-12	\N	Playgroups	\N	Not Listed	\N	Billerica	MA	01821	978-667-9771	\N	\N	\N	www.momsandtots.org	public_program_email@testing123.net	Play Group for Children	The purpose of playgroup is to give children of similar ages the opportunity to play together, communicate and socialize as well as to give parents, both mothers and fathers, the opportunity to socialize and exchange ideas.	Primarily families with infants and preschoolers (ages 0-6), up to elementary age children.\r\n\r\nSummer activies may be open to grammar school age children.	Yes	$25 / year membership dues, rate is prorated if you join later in the year	Program runs year round. Enrollment is open and ongoing, membership can be renewed each year.	Hours vary, contact program.	English	Yes	Call program with basic info (number and ages of children), see newsletter for events.		Event locations vary.	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	All board members are volunteers, there are no paid positions in Moms and Tots.	No	\N	No	\N	Yes	All events are listed in monthly newsletter.	No	Access is for members only.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Burlington	\N	No	0	\N
507480	102442	2008-08-15	\N	School Success	\N	Adelante Inc.	40 Lawrence Street	Lawrence	MA	01840	978-687-9432	\N	\N	978-794-3407	http://www.adelanteyouthcenter.org	public_program_email@testing123.net	Youth Development and Education	Adelante Inc.\r\n40 Lawrence Street  Lawrence, MA  01840  978-687-9432\r\nSCHOOL SUCCESS PROGRAM 2008-2009\r\n\r\nNow located in the Lawrence YMCA building, Adelante Youth Center, founded in 1992 by Fr. Joachim Lally, brings sixteen years of professional experience, observation, and thoughtful evaluation to the issues and outcomes of youth development in Lawrence, Massachusetts.\r\nAdelante represents both the corporate name and the mission of the Adelante Youth Center. It is the Spanish word for moving forward and symbolizes our goal to provide a safe and productive environment for city youth from the surrounding neighborhoods and engage them in positive after-school programs to move forward personally, academically and toward positive life choices. \r\n\r\n\r\nThe Prep Program: Classes for Seventh and Eighth Grade Students\r\nImplemented 2003-2004, this program focuses on entrance exam preparation and academic enrichment for middle school students seeking to attend area private high schools. To date 92 of 103 participants (89%) have succeeded in the private admissions test.\r\n\r\nThe Prep programs combined are a two-year commitment with the potential to lead to a four-year high school scholarship.  Students who complete this program may become eligible to receive a four-year scholarship of up to $2,000 as well as a matching scholarship from one of Adelantes partner high schools (Central Catholic, Presentation of Mary Academy, and Notre Dame High School).\r\n\r\n\tFor our 7th graders we have developed an intensive academic and skill-building course for those who are motivated for secondary school success and a future that includes college. Literacy exercises focus on reading, speaking and writing development. Test preparation activities will introduce students to the SSAT (Secondary School Admissions Test). Tuesday science and math enrichment classes are conducted by Merrimack College students who serve in the colleges Lawrence Math/Science partnership. The class will meet on Mondays, Tuesdays and Thursdays from 4 to 6pm. during the school year.\r\n\r\n\tOur 8th grade program, now in its sixth year, is a class that focuses on entrance exam preparation for area private high schools.  Course curriculum includes activities such as essay writing, public speaking, interviewing and introductory economics and business programs offered through Junior Achievement. Intensive test preparation is offered for the High School Placement Test (HSPT) and Secondary School Admissions Test (SSAT)\r\nAdelante Scholars\r\nCurrently there are 42 Adelante Scholars in grades 9-12. In the past eleven years, Adelante has granted 4-year scholarships ranging from $800 to $2,000 to 160 students attending area private high schools. Through a partnership arrangement, scholarships are matched by the schools. \r\n\r\nIn addition to the scholarships Adelante provides, each student receives a 100% matching scholarship from the school they attend. In accepting the scholarship award, each student agrees to perform up to 160 hours of community service during the school year. \r\n\r\nNinety-seven Adelante Scholars thus far have graduated from 28 different colleges and universities from New England and beyond.\r\n\r\n\r\nFor more information, visit Adelante Inc. or call 978-687-9432.	Lawrence, MA area youth, grades 7 - 12; limited income families	Yes	Tuition for the Eighth Grade Prep Program only. $10/week	School Success program operates during the school year. Six-week summer academic enrichment and recreation program for youth entering grades six, seven and eight. Intensive SAT Prep courses available in Spring and Summer.	Tuesdays 4-5pm; Wednesdays and Thursdays 4-6pm	English!DELIM!Spanish	Yes	Proof of Residency, proof of income, most recent report card.		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
503036	100968	2008-08-19	\N	General Bereavement Support Group	\N	1 Grace Ave	Church Hall, Room 6	Billerica	MA	01821	978-663-3393	\N	\N	978-663-0577	http://sttheresaparish.com/	public_program_email@testing123.net	Support group for people who lost a loved one	The General Bereavement Support Group offers support for individuals who have lost family members, close friends and children. The group meets biweekly, on the second and fourth Thursday from 7 to 9 PM.	All are welcome to this faith-based program.	Free	\N	Program runs year round.	Call for day and time. 2nd and 4th Thursday of each month, 7-9.	English	Yes	Self-referral, but must call before attending first group.		Parking available in lot next to building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers welcome! Call for more information.	No	\N	Yes	Room 6 in Church Hall.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
503949	101375	2008-08-20	\N	ESOL classes	\N	1 Ballard Way	Suite 101	Lawrence	MA	01843	978-683-7316	\N	\N	978-683-9075	asiancentermv.org	public_program_email@testing123.net	English as a Second or Other Language Classes	Two levels of English classes are offered in the morning, three levels of English classes are offered in the evening.	All are welcome.  No eligibility requirements.	Yes	Evening classes are supported by the Department of Education so there are no fees.\r\nMorning classes cost $20 per trimester.  We do not offer classes in the summer.  Fee includes books.	We are open year round, although we only offer ESOL classes September - June.	A.M. classes are held Tuesday and Thursday 9 - 11 am\r\nP.M. classes are held Mon-Wed for beginning level, Tuesday, Wednesday Thursday for higher levels 6 - 9 pm	English!DELIM!Khmer!DELIM!Vietnamese	No	Without an appointment Monday through Thursday from 9 - 2.  Otherwise appointment recommended		Parking available	Van transportation for A.M. students is available for a fee.	Yes	Yes	Preschool for children ages three to five is available for students in the morning classes.	Yes	We sometimes have a waitlist for ESOL classes.	\N	\N	\N	\N	\N	\N	Yes	We welcome volunteers.  Primarily for tutoring.	No	\N	Yes	3 large classrooms	Yes	\N	No	Our students have access to the internet.	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Most of our participants come from Lawrence and the surrounding cities and towns.	We have two outreach workers.  One speaks English and Khmer.  The other speaks English, Vietnamese, and Chinese (both Cantonese and Mandarin)	No	0	\N
504818	101375	2008-08-20	\N	Family Literacy Program	\N	1 Ballard Way	Suite 101	Lawrence	MA	01843	978-683-7316	\N	\N	978-683-9075	asiancentermv.org	public_program_email@testing123.net	Family Literacy Program for Parents and Pre-Schoolers	Program consists of Parenting Classes, ESL classes, Inter-Active Literacy and Pre-school.	Parents with children between the ages of 2.9 - 5 years.	Yes	$190 per semester	September - June, three 10 week semesters.	Mondays - Parenting Class 9 - 11 am\r\nTues & Thurs - ESL Classes 9 am - 12 pm\r\nWednesdays - Preschool 9 - 2 pm\r\nFriday - Interactive Literacy  9 - 11\r\n(Preschool is provided every day while parents/guardians are attending classes)	English!DELIM!Khmer!DELIM!Vietnamese	No	Social Security card or Green card		Parking is available.	\N	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	3 large classrooms	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Chinese	No	0	\N
503993	101375	2008-08-20	\N	Preschool	\N	1 Ballard Way	Suite 101	Lawrence	MA	01843	978-683-7316	\N	\N	978-683-9075	asiancentermv.org	public_program_email@testing123.net	Preschool program for 2.9 - 5 year olds	Preschool program is provided for adults studying ESOL in our morning program.	open to all studying Engish at the Center	Yes	There is a charge of $950 for each preschooler. There are two sessions from September to June. \r\nOr, enroll a child from 9 - 2 p.m. for $1650.	From 9 - 12 a.m or 9 - 2 p.m. weekdays. September through June. (Please see our Family Literacy program for more options/days)	From 9 - 12 a.m or 9 - 2 p.m. weekdays.	English!DELIM!Khmer!DELIM!Vietnamese	No	Green card.  Social Security card.		Free parking available	Van pick-up within Lawrence is available for a fee.	Yes	Yes	Preschool classes for ESOL morning students.	No	No waitlist at the moment	\N	\N	\N	\N	\N	\N	Yes	Tutoring ESOL, computer or assisting in the preschool class.	No	\N	Yes	3 large classrooms	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Primarily Greater Lawrence but open to all.	Chinese	No	0	\N
505049	100495	2008-08-20	\N	Adolescent Residential Recovery Program	\N	111 Middleton Rd	\N	Danvers	MA	01923	978-739-7600	\N	\N	978-736-3620	www.cabhealth.org	public_program_email@testing123.net	Substance abuse treatment for adolescents age 13-17	During this very structured program, we will serve 15 boys at a time, with each boy staying with us for 60 days (maximum 90 days). \r\nEach boy will receive intensive, customized treatment, including individual, group and family therapy, educational tutoring, recreational opportunities and aftercare placement.. For one year following discharge, our program staff will continue to follow-up with each young client and his family. Although the youth will temporarily live away from home, the family has a very active role in treatment.	Male- age 13-17	Yes	Most major insurances including medicare/medicaid accepted.	Accepting new admissions on a year round basis.	Please call program for specific admissions hours.	English!DELIM!Spanish	Yes	\N		Parking in front of the building.	\N	No	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502629	100495	2008-08-20	\N	Boston Treatment Center	\N	784 Mass. Ave, Rear	\N	Boston	MA	02118	617-247-1001	800-763-5363	\N	\N	www.cabhealth.org	public_program_email@testing123.net	Detox facility for clients addicted to drugs and alcohol	The Boston Treatment Center is a medically supervised detoxification facility for clients who are addicted to drugs and alcohol. We take clients from across Massachusetts and are accessible by public transportation.	+. Contact program or visit main website, www.cabhealth.org for referral and admissions information	Yes	Fees vary. Medicaid and most insurance plans accepted	Services available year round; Program length varies. Program capacity is 45 beds - men and women on separate floors	Please contact CAB directly	English!DELIM!French!DELIM!Spanish	Yes	Contact program or visit main website, www.cabhealth.org for information		Parking garage nearby	Facility located close to MBTA line and Boston bus routes	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Meeting space may be available. Please contact CAB directly.	Yes	"Insights," the agency newsletter is send out twice yearly. Sign up for a mailed copy or you may read a PDF file on line at www.cabhealth.org	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502640	100495	2008-08-20	\N	Danvers Treatment Center	\N	111 Middleton Road	\N	Danvers	MA	01923	978-777-2121	800-323-2224	\N	\N	www.cabhealth.org	public_program_email@testing123.net	Provides substance abuse treatment programs	The Danvers Treatment Center provides four substance abuse treatment programs in one location: medical detoxification, a dry homeless shelter for women, buprenorphine outpatient treatment and a methadone treatment program. Also includes a family information group on Tuesday night. Learn about drug and alcohol treatment and how it works for and with your loved on.	Referrals recommended. Please contact program or visit main website, www.cabhealth.org for referral / admissions information	Yes	Fees vary. Medicaid and most insurance plans accepted	Contact program for scheduling information	Please contact CAB directly	English!DELIM!Spanish	Yes	Varies by program. Contact program or visit main website, www.cabhealth.org for more information		Parking available on site. Public bus service from the MBTA station.	Transportation is available to medical and other facilities and to other CAB facilities	Yes	No	\N	No	Varies. Clients are encouraged to call. If a wait list, prospective client will be instructed on how to remain current on that list.	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact program for more information	No	\N	Yes	Meeting space may be available. Please contact CAB directly.	Yes	"Insights" is available via U.S. mail or on the agency's website as a PDF file, www.cabhealth.org.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502651	100495	2008-08-20	\N	H.A.R.T. House	\N	P.O. Box 477	\N	Tewksbury	MA	01876	978-851-0969	\N	\N	978-851-0998	www.cabhealth.org	public_program_email@testing123.net	Residential treatment home for women and children	H.A.R.T. House is a residential treatment home for women and children who are recovering from addiction to drugs and alcohol. In addition to other substance use, we also accept women who are methadone maintained.	Females age 18 and over and their children. Must already be detoxed.	Yes	Contact program for more information	Program runs year round. Program length is 6 months. Enrollment subject to availability, waiting lists may develop. For central admission, call 866-705-2807	Please contact CAB directly	English!DELIM!Spanish	Yes	Contact program at 978-851-0969		Parking available on site.	program provides some transportation. Call the site.	Yes	No	\N	Yes	Varies. Call the program. For referral or intake, please call 866-705-2807.	\N	PO Box 477	\N	Tewksbury	MA	01876	Yes	Interested volunteers should contact program for more information	No	\N	No	\N	Yes	Agency newsletter, "Insights" available by U.S. mail or as PDF file on the website, www.cabhealth.org.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
502662	100495	2008-08-20	\N	Ryan House	\N	100-110 Green Street	\N	Lynn	MA	01902	781-598-1270	\N	\N	\N	www.cabhealth.org	public_program_email@testing123.net	Residential half-way house for men and women	Ryan House is made up of two residential half-way houses for men and women recovering from drug and alcohol addiction.	See main website, www.cabhealth.org for referral and admissions information	Yes	Contact program	Services available year round. 20 beds for men; 20 beds for women. Waiting list varies.	Please contact CAB directly	English!DELIM!Spanish	Yes	Contact program directly for more information		\N	Transportation provided to various other services.	Yes	No	\N	Yes	Varies. Check with the program.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Meeting space may be available. Please contact CAB directly.	Yes	"Insights," the agency newsletter published twice a year. Available via U.S. mail or as PDF file on the agency website, www.cabhealth.org	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
502706	100495	2008-08-20	\N	Transitions	\N	P.O. Box 837	\N	Tewksbury	MA	01876	1-877-344-4440	877-344-4440	\N	978-750-3620	www.cabhealth.org	public_program_email@testing123.net	Stabilization and transition program for men recovering	The Transitions program is a stabilization and transition program for indigent men recovering from addiction to drugs and alcohol. The program provides a safe and substance-free environment and helps male clients to make a smooth transition to the next level of care.	Program open to males over the age of 18 who are already detoxed	Yes	Contact program for more information	Program runs year round. Enrollment is ongoing, there may be a waiting list at times.	Please contact CAB directly	English!DELIM!Spanish	Yes	Contact program or visit main website, www.cabhealth.org form referral or admissions information		Parking available on site	Program provides client transportation to some necessary services.	Yes	No	\N	No	Depends. Contact program.	\N	P.O. Box 837	\N	Tewksbury	MA	01876	Yes	Interested volunteers should contact program for more information	No	\N	Yes	Meeting space may be available. Please contact CAB directly.	Yes	Twice-annual newsletter, "Insights" can be received by U.S. mail or on the agency's website as a PDF file, www.cabhealth.org	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
509207	103443	2008-08-26	\N	Services To Mental Health Consumers	\N	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	800-845-6457	\N	978-689-4488	www.nilp.org	public_program_email@testing123.net	Support, educate & empower mental health consumers	This program works with people with psychiatric disabilities through individual and group empowerment, peer counseling and community organizing to promote a message of hope and recovery and to fight against stigma and human rights violations.	Have a disagnosis of mental illness from the Merrimack Valley region.	Free	\N	Open year round, Monday to Friday	Monday-Friday\r\n9:00AM to 5:00PM, some evenings by arrangement.	English!DELIM!Spanish	Yes	\N	37 Beacon St.	Free parking in our parking lot.	NILP can provide limited transportation upon request and need. Contact program first for additional information.	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Contact program directly and indicate interest areas.	Yes	Pamphlets and materials on many aspects of disability and relevant programs and services.	Yes	Limited conference room area 25+ people. Priority to agency programs and participants.	Not Applicable	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	800-845-6457	\N	\N	No	0	\N
509218	103443	2008-08-26	\N	NILP Adults with Physical Disabilities Program	APD	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	800-845-6457	\N	978-689-4488	www.nilp.org	public_program_email@testing123.net	Assist you in gaining independence and control of your life,	The APD Program works with adults who have physical disabilities.  Specialized services include personal care assistance(PCA), peer counseling, advocacy and skills training.	The PCA program requires Mass Health eligibility.	Yes	Must be Mass Health eligible for PCA services.	Open year round, Monday-Friday	Monday-Friday\r\n9:00AM to 5:00PM	English!DELIM!Spanish	No	\N	37 Beacon St.	Free parking in our parking lot.	Limited accessible transportation is available upon request and need.  Contact program first to review eligibility for transportation.	Yes	Not Applicable	\N	Yes	Sometimes there is a wait list for PCA services.	\N	\N	\N	\N	\N	\N	Yes	Inquire.	Yes	Pamphlets and materials on many aspects of disability and relevant programs and services.	Yes	Limited conference room area 25+ people. Priority to agency programs and participants.	Not Applicable	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	800-845-6457	\N	\N	Not Applicable	0	\N
509229	103443	2008-08-26	\N	NILP, A Smoother TYransition	AST	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	800-845-6457	\N	978-689-4488	www.nilp.org	public_program_email@testing123.net	Help adolescents w/disabilities achieve their full potential	Works with adolescents with disabilities and their families to facilitate the trtansition from both institutional and public schools into adult human services systems.  Specialized services include advocacy during the development of Individualized Education and Transition plans. Additionally, after school bi-weekly peer groups are offered to address the unique issues facing youth with disabilities.	Age 14 through age 22	Free	\N	Open year round.\r\nAST Peer groups, after school during school year.	Monday to Friday\r\n9:00AM to 5:00PM	English!DELIM!Spanish	Yes	Proof of age	37 Beacon St.	Free parking in our parking lot.	Limited accessible transportation available depending on location and need.	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Inquire	Yes	Pamphlets and materials on many aspects of disability and relevant programs and services.	Yes	Limited conference room area 25+ people. Priority to agency programs and participants.	Not Applicable	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	0	\N
507359	102860	2008-01-05	\N	Child Development Network	CDNKids	76 Bedford Street	Suite 12	Lexington	MA	02420	781-861-6655	\N	\N	781-861-6654	www.CDNKids.com	public_program_email@testing123.net	Comprehensive Evaluations and Developmental Management	The Child Development Network (CDN) is a inter-disciplinary team of pediatric health care professionals who provide comprehensive evaluations and ongoing clinical care for children, teens, and their parents.  \r\n\r\nWe offer Neuropsycological Evaluation, Behavioral Medicine Services, Speech and Language Therapy, and Education Therapy.	No restrictions.	Yes	\N	We offer services during the school year as well as summer programs.	9:00 AM - 7:00 PM	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Hebrew	Yes	4	\N
507524	102948	2008-09-02	\N	Daybreak Shelter	\N	19 Winter Street	\N	Lawrence	MA	01841	978-975-4547	\N	978-975-4547	978-975-2672	tpc1.org	public_program_email@testing123.net	30 Day Emergency Wet Shelter	Sobriety is not required for a Daybreak Admission.  Admission is for males and females 18 years of age and older on a 1st come, 1st served basis.  Admission begins at 5pm daily, Photo ID is required. Dinner and Breakfast are provided, guests are given access to showers and hygiene supplies.  Case management services including access to housing search and health insurance, health services, mental health services, substance abuse services.	Males and Females 18 years of age and older.  Photo ID required.	Free	\N	Admission year round:  Guests may enter at 5pm, and are expected to leave by 8 am.	Daily 5pm - Leave Premises at 8am	English!DELIM!Spanish	No	\N	41 Lawrence-Lowell	Limited parking for staff and guests	Daybreak provides transportation to SOAP participants	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Apply at The Psychological Center, 11 Union Street, Lawrence, Massachusetts 01843.  978-685-1337, ext 305	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy,Motivational Interviewing, Continigency Management,Substance Abuse, Mental Illness, Dual Diagnosis, Spirituality in Recovery	Yes	Very Large Community Meeting Space,Several Large Group Rooms	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
502563	100770	2008-09-02	\N	Families First Parenting and Professional Programs	\N	99 Bishop Richard Allen Drive	\N	Cambridge	MA	02139	617-868-7687	\N	\N	617-354-2902	www.families-first.org	public_program_email@testing123.net	Parenting workshops for both parents and professionals	Families First Parenting Programs builds positive relationships between parents and their children by educating both families and the professionals who serve them through inspiring, hands-on workshops. Our skilled workshop leaders stand beside parents and caregivers to lead them through the crucible of child rearing by transforming their experiences, doubts and fears into the knowledge and confidence they need to build positive discipline, communication, and self-esteem in families. Our training for professionals enhances their ability to better support and engage families. We help communities thrive by strengthening families first.	Services are open to all.	Yes	\N	We can work with organizations throughout the year to deliver parenting workshops and/or professional development for staff.	Office hours: Monday-Friday 9AM-5:30PM. Workshop hours vary.	English!DELIM!Spanish	Yes	Contact program for details		Varies by workshop site	\N	Yes	No	\N	No	\N	Languages: Families First has several sites. Languages spoken vary by site. Languages spoken on phone at main site: Spanish	\N	\N	\N	\N	\N	Yes	Donations of any kind, including services and space, are accepted. Volunteers assist in a variety of ways.	No	\N	No	\N	Yes	Please contact us or visit our website to register to receive our newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	Our workshops can be offered in Spanish.	Yes	0	\N
504873	100121	2008-09-02	\N	Indoor Youth Soccer	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	youth soccer	Indoor Soccer League\r\nYMCA Indoor Soccer runs every Sunday.  Call the YMCA for dates and times.  \r\nProgram participants will learn:\r\n* Dribbling\t\t* Teamwork\r\n* Passing\t\t* Rules\r\n* Shooting\t\t* FUN\r\n\r\n-Itty Bitty Soccer (ages 3-4) Parent/child class\r\n-Bitty Soccer (ages 4-6)\t\t\r\n-Youth Soccer League (ages 7-10)\t\t\t\t\r\nAll program participants will receive a jersey and an award	ages 3-10	Yes	\N	Fall, Winter, and Spring seasons	Sundays Sept 14th - November 16th	English!DELIM!Portuguese	No	\N		parking is available	\N	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	YMCA gym.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	YMCA Members $40 Program Members $75	\N	No	0	\N
505357	100121	2008-09-02	\N	Trails of Terror	\N	234 Hall Street	\N	Dunstable	MA	01827	978-454-7825	\N	\N	978-454-8982	www.greaterlowellymca.org	public_program_email@testing123.net	Trails of Terror	A night-time walk through the haunted woods of YMCA Camp Massapoag in Dunstable, MA.  Open on Friday and Saturday, 7-11 PM, in October.  Not recommended for youth under age 8.	Open to the public.  Not recommended for youth under age 8.	Yes	Admission is $10 per person.  Group rates available.  Proceeds benefit campership fund and teen programs.	Month of October	Fridays & Saturdays, 7 - 11 PM.	English	No	\N		Parking available on site.	\N	No	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers serve as tour guides or actors.  Must be age 13.	No	\N	No	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
505599	102013	2008-09-04	\N	Neuropsychological and Neurodevelopmental Evaluations	\N	594 Marrett Road, Suite 22	\N	Lexington	MA	02421	781-652-9594	\N	\N	781-652-9557	\N	public_program_email@testing123.net	Neurodevelopmental and Neuropsychological evaluation	Provide neurodevelopmental and neuropsychological evaluation and treatment of children ages 2 and above with developmental delays, attentional problems, and learning disorders due to various conditions, such as PDD, Genetic & Metabolic Disorders. The focus of evaluation for these children is to identify their cognitive and developmental abilities, to clarify diagnostic questions, and to provide recommendations for management and educational planning.	N/A	Yes	\N	open year round	M-F\r\n8AM-6PM	English	Yes	\N		\N	\N	No	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
509284	102948	2008-09-05	\N	PEOPLE! Recovery Center	\N	11 Union Street	\N	Lawrence	MA	01843	978-688-5767	\N	978-685-1337	978-683-0139	tpc1.org	public_program_email@testing123.net	peer led substance abuse/recovery supports	PEOPLE!  Recovery Center provides a range of supports, activities, and opportunities to hundreds of members of the Merrimack Valley community of people in recovery.  Building on effective models of peer-to-peer recovery support, the Center relies heavily on an array of community resources, including volunteers who are part of the recovery community and many partners that includes both grass-roots groups and substance abuse service providers.  Located in downtown Lawrence, the Center provides a warm, welcoming place for people in recovery to get emotional support, practical resources, social affiliation, and  information. It engages participants in setting their own agenda in sobriety and for their overall interactions in Center activities.  PEOPLE! builds community support and resources.  \r\nHours of Operation:\r\nMonday through Friday 9-5, Sunday 11-4\r\nFor more information please call       PEOPLE! at 978-688-5767.	For anyone (individuals and their families in need of recovery supports	Free	\N	Year Round	Monday - Friday 9-5, Sunday 11-4	English!DELIM!Spanish	No	na	33 North Andover!DELIM!39B North Andover Mall / Phillips	Onsite parking	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Requirements for a Volunteer\r\n1.\tMeet the requirements of an Active Member for at least a month.\r\n2.\tCommit to a schedule\r\n3.\tDemonstrate their willingness and ability to work with and for others \r\n4.\tAttend Volunteer Orientation\r\n5.\tBe flexible.\r\n6.\tBe reliable and dependable.\r\nRequirements for  being a Member of PEOPLE!\r\n1.\tSign in.\r\n2.\tHas problems with substances, whether self, family, friends.\r\n3.\tAgree to uphold and abide by the Ethics statement.\r\n4.\tContributes to the safety of People!\r\n5.\tExtends courtesy to others who may be new as well as established members\r\nRequirements for Active Member\r\n1.\tMeets all the requirements of  Member\r\n2.\tBe present 3 times a week and/or participates in 2/3 groups or activities.\r\nIf a person with limited time is involved and active while here, he/she may also\r\nbe an active member.\r\n3.\tHelpful when here\r\n4.\tSomeone who comes in regularly and promotes the well being of People!\r\n5.\tIf present, fill in for a volunteer when necessary.\r\n6.\tRecruits others	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy,Motivational Interviewing, Continigency Management,Substance Abuse, Mental Illness, Dual Diagnosis, Spirituality in Recovery	Yes	Very Large Community Meeting Space,Several Large Group Rooms	Yes	Produced by members and volunteers, contains PEOPLE! activities, events, calendars	Yes	Three computers in main room with internet access	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	0	\N
508723	102838	2008-09-09	\N	MA Alliance 2008 Teen Pregnancy Institute:	Expanding Opportunities for Youth and Young Families	105 Chauncy Street, 8th floor	\N	Boston	MA	02111	617-482-9122	\N	800-645-3750	617-482-9129	http://www.massteenpregnancy.org	public_program_email@testing123.net	MA Alliance Annual Conference	The Massachusetts Alliance on Teen Pregnancy is happy to announce our annual Teen Pregnancy Institute: Expanding Opportunities for Youth and Young Families. This event brings together social workers, nurses, health educators, and other youth workers to learn about developments surrounding the issue of teen pregnancy.\r\nWorkshop topics will include dating violence, birth control and contraception, healthy sexuality, teen parenting and more. Continuing Education Credits are currently pending with NASW, MARN, and CHES for these workshops. \r\nLast year our sold-out conference brought together over 325 attendees and 35 presenters for a day of learning. The cost is $125 per person. Special pricing rates are available. Please contact the Alliance if you are interested. \r\nSave the Date! Thursday, October 23, 2008. We hope to see you there!	n/a	Yes	The cost is $125 per person. Special pricing rates are available. Please contact the Alliance if you are interested.	Annual	October 23, 2008\r\n8:00 am  4:00 pm	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	For more information on the Alliance, please go to http://www.massteenpregnancy.org. Registration information will be up this summer, so check back! For any questions, please contact Lauren at educationintern@massteenpregnancy.org	\N	\N	\N	\N	\N	Yes	\N	Yes	Resource library in our office consisting of research materials, journals, and reports pertaining to teen pregnancy prevention and public policy related to pregnant and parenting teens.	No	DCU Center, Worcester, MA	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	0	\N
507645	103047	2008-09-22	\N	Lawrence Outdoor Youth Corps	\N	60 Island St 2nd Floor	\N	Lawrence	MA	01840	978-974-0770	\N	\N	978-974-0882	www.groundworklawrence.org	public_program_email@testing123.net	Full time summer landscape job training for youth	The Lawrence Outdoor Youth Corps offers paid, full-time summer positions for youth ages 16 to 18 to work on landscape improvements in the City of Lawrence. This program provides job training for youth interested in landscaping work and educational workshops which include financial literacy and career development skills. In addition, weekly field trips to MA DCR state parks expose Youth Corps members to Massachusetts' extensive state park system and the different types of open space and stewardship needs among them.	Ages 16-18	Free	\N	Summer employment program	M-F 9-4	English!DELIM!Spanish	Not Applicable	\N	33 North Andover!DELIM!39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	\N	\N	No	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
509108	102508	2008-07-18	\N	Lazarus House Shelter	\N	48 Holly Street	\N	Lawrence	MA	01841	978-794-8447	\N	\N	978-682-7004	http://www.lazarushouse.org/	public_program_email@testing123.net	Beds for families and individuals.	Lazarus House has 31 beds and 7 cribs, with separate rooms for men, women, and families. Lazarus House provides three meals a day, clothing, laundry, linens, toiletries, infant care supplies, as well as all of the things a family needs to be a family. Our shelter is the foundation to break the cycle of poverty and resume independent living in the future.\r\n\r\nLazarus House provides a hand up rather than a hand out:\r\n\r\n+ Structured, supervised environment\r\n+ Guests need to be clean and sober\r\n+ All guests work with staff advocates toward self responsibility\r\n+ Encouragement to save money - budget - develop life skills\r\n+ Responsibility - Household shelter chores \r\n\r\nMany people who live at the shelter work full time. Due to limited job training, experience and corresponding wages, an average of 4-6 months is required for families to save enough money to move to a permanent residence.	Homeless individuals.	Free	\N	Open year round.	ADD INFO	English	Yes	ADD INFO		\N	\N	No	No	\N	Yes	\N	Lazarus House Ministries is happy to accept the following donations:\r\n\r\nAppliances -- Baby Clothing & Toys -- Clothing -- Dishes -- Furniture -- Office Equipment -- Rugs	Lazarus House Ministries	P.O. Box 408	Lawrence	MA	01842	Yes	Items that are donated must be in good, working condition and may be dropped off at anyone of our thrift store locations Monday - Saturday, 10 a.m. - 3 p.m. (excluding holidays).	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
509097	102508	2008-07-17	\N	Job Training	SPARK*L.E. Cleaning Services	Good Shepherd Center	410 Hampshire St., 2nd Floor	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/	public_program_email@testing123.net	On-the-job and skills training, including case management.	The objective of the Lazarus House Job Training Program is to prepare low-income persons for the work world. Lazarus House provides classroom education, pre-employment skills classes, job training and ongoing mentoring. Case management, with job counseling and referrals to outside sources of support, is a daily activity. \r\n\r\nLazarus House SPARK*L.E. Cleaning Services is an office & industrial and private home cleaning business. There is training specific to those tasks incorporated into our training program. However, Lazarus House is emphasizing transferability of skills and advancement capacity through the pre-employment, work maturity and basic skill's components. The specific occupational training will also encourage participants to pursue interests in areas related to the cleaning business, such as record keeping and financial records. While employed at SPARK*L.E. Cleaning Services, the trainees are paid for 30 hours per week of work and 10 hours per week of continued classroom experience.\r\n\r\nThe initial training lasts for eight weeks. At the end of eight weeks of pre-employability, work maturity, basic educational (especially English as a Second Language-ESL), and occupational skills training, participants are offered an opportunity to be hired by Lazarus House in its cleaning business for a up to a 6 month period after that time with the assistance of Lazarus House staff the employee is expected to have secured another more permanent position.	Department of Transitional Assistance helps  establish eligible participants, as well as referrals through organizations who work with the appropriate populations.	Free	\N	Open year round.	ADD INFO	English!DELIM!Spanish	Yes	ADD INFO		\N	\N	No	No	\N	No	\N	\N	Lazarus House Job Training Program	P.O. Box 408	Lawrence	MA	01842	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
509119	102508	2008-07-18	\N	Thrift Stores	\N	Good Shepherd Center	412 Hampshire Street	Lawrence	MA	01841	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/	public_program_email@testing123.net	Thrift stores in Lawrence.	Our ministry helps sustain itself through our three thrift stores. These hubs of activity provide an incredible opportunity to strengthen the neighborhoods where they exist, provide for the needs of our shoppers and allow us to provide for those who have no other means of availing themselves of clothing and household items.\r\n\r\nGood Shepherd Thrift Store:\r\n978-686-0441   412 Hampshire Street\r\nSaint Martin's Thrift Store:\r\n978-687-1184   121 Essex Street\r\nSaint Clare's Thrift Store:\r\n978-557-5449   220 South Union Street	Open.	Sliding Scale	For those who cannot afford to purchase items, such as victims of arson or floods, and for those who are making the transition from homelessness, items are provided free of charge.	Open year round.	Monday -- Saturday:\r\n10AM -- 4PM	English	No	Items that are donated must be in good, working condition and may be dropped off at anyone of our thrift store locations Monday - Saturday, 10 a.m. - 3 p.m. (excluding holidays). \r\n\r\nPlease note due to liability issues, we are not allowed to accept baby furniture, including car seats.		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	Good Shepherd Thrift Store	P.O. Box 408	Lawrence	MA	01842	Yes	Lazarus House Ministries is happy to accept the following donations:\r\n\r\nAppliances --Baby Clothing & Toys -- Clothing -- Dishes -- Furniture -- Office Equipment -- Rugs	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
505324	101947	2008-09-16	\N	Managed Care Plan	NHP	253 Summert St	\N	Boston	MA	02210	617-772-5500	1-800-462-5449	\N	617-772-5513	www.nhp.org	public_program_email@testing123.net	Managed Care program	NHP is a Massachusetts-based HMO providing health care coverage to MassHealth members, employers and employees, individuals.In partnership with community health centers and other community responsive providers, NHP has created a comprehensive provider network encompassing over 9,000 specialists and over 40 top Massachusetts hospitals. In addition, NHP has developed disease management programs that provide outreach and education to members who have a chronic medical condition such as asthma or diabetes. NHP's core philosophy is that a health plan can only be as good as the care it provides its members. NHP is nationally-recognized for its work with underserved populations and this year is celebrating 20 years of service to its membership.	MassHealth Standard, MassHealth  Basic,  Commonwealth Care, Commonwealth Choice and Commercial Insurance members may be eligible for NHP	Yes	Depending on coverage type, co-payments may apply.	Open year round	Monday-Friday\r\nSwithboard 8:30am to 5:30pm\r\nMember Services 8:30am to 6:00pm	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	Every NHP member receives a free bilingual newsletter (English&Spanish)It includes health and wellness topics as well as member benefits information.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	NHP maintains a network of over 1,900 primary care physicians at more than 250 primary care sites. Our network also includes over 9,000 specialists and 41 teaching hospitals.	AT&T language line available for other languages as needed.	Not Applicable	0	\N
507634	103047	2008-09-22	\N	Green  Team	\N	60 Island St 2nd Floor	\N	Lawrence	MA	01840	978-974-0770	\N	\N	978-974-0882	www.groundworklawrence.org	public_program_email@testing123.net	Environmental education and stewardship program	The Green Team program offers part-time, paid positions for high school students from Lawrence to help lead local environmental initiatives, conduct environmental research, and participate in hands-on improvement projects throughout the City of Lawrence. The program is intended to promote the protection and restoration of critical ecological systems and natural resources in Lawrence by engaging teens in advocacy and service-learning efforts focused on the community's waterways, parks and open spaces. In addition, financial literacy and career development skills are intregrated into the educational workshops. The program has dual goals - to prepare Lawrences youth for a lifetime of environmental leadership, and to develop community capacity to improve the physical environment.	14-18	Free	\N	Year round program. Applications accepted in the fall.	15 hours a month in the school year and 15 hours a week in the summer	English!DELIM!Spanish	Not Applicable	\N	33 North Andover!DELIM!39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	available on Island St or N. Union parking lot	Provided to participants for events that require it.	Not Applicable	Not Applicable	\N	Yes	Please contact Rose Gonzalez at rgonzalez@groundworklawrence.org	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	Not Applicable	0	\N
507656	103058	2008-09-24	\N	YMCA Junior and Teen Achievers Program	\N	40 Lawrence Street	\N	Lawrence	MA	01840	978-681-1126	\N	\N	\N	www.mvymca.org	public_program_email@testing123.net	YMCA Teen mentoring program	Teen Achievers\t\t\t       Grades 7 - 9 (Junior Achievers)\r\nGrades  10-12 (Teen Achievers)\r\nThe YMCA Teen Achievers youth program links our partners in   business and industry, employees who have been recognized as Achievers, with youth in a community effort to develop the educational, career and leadership potential of young people. This partnership offers unique opportunities for young people to explore their interests, develop skills, and broaden their access to information and resources so they may realize their life goals. Guiding them along the way are adult Achievers, individuals who share their life experiences with program participants and support their overall development.\r\nA meeting may include:\r\nA mentor-guided company tour;\r\nA working session on interview skills\r\nthrough interview exchanges;\r\nDeveloping the teens resume;\r\nRefining college selections and college searches; and\r\nCareer discussions;\r\nYMCA Teen Achievers connects youth to invaluable resources and life lessons while giving them the scholastic tools to succeed in high school, college and beyond.	Please contact Teen Director for more information.	Free	The YMCA's "Reach Out For Youth" financial assistance program raises funds to assist needy and deserving youth participate in such programs at Teen Achievers.	Open primarily during school year; some events held during summer.	Daily after school.	English!DELIM!Spanish	No	\N	01 Haverhill - Methuen - Lawrence	Plenty of free off-street parking.	We provide transportation for all field trips.	Yes	Not Applicable	\N	No	\N	\N	40 Lawrence Street	\N	Lawrence	MA	01840	Yes	Always looking for mentors and adults available for homework assistance and tutoring.	Not Applicable	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
507667	103058	2008-09-24	\N	YMCA Single Resident Occupancy (SRO) Housing for Adult Men	Merrimack Valley YMCA Residence Program	40 Lawrence Street	\N	Lawrence	MA	01840	978-686-6191	\N	\N	\N	www.mvymca.org	public_program_email@testing123.net	Lawence YMCA Residency Program	The YMCA Resident Program is a Single-Room Section 8 subsidy program that offers safe, affordable housing and services to qualified single ment, ages 18-80. We work closely with the Lawrence Housing Authority. Interested individuals can apply through the Lawrence Housing Authority, located at 353 Elm Street, Lawrence, 978-683-2751, and they will refer people to our SRO program. \r\nThe program offers:\r\n- a wide variety of programs, including computer lab with Internet access, ESL classes, TV room/cable, laundry\r\n- fully secured building with overnight security guard\r\n- Use of YMCA Fitness Center facilities and swimming pool\r\n- Handicap accessible entrance with elevator\r\nOur services include:\r\n- a safe, secure place to live\r\n- help in staying clean and sober\r\n- help in finding appropriate programs and services\r\n- planned activities for residents\r\n- referrals for medical care\r\n- weekly community dinners	Call Lawrence Housing Authority for more information at 978-683-2751	Yes	\N	Open year round.	Every day, 24 hours a day.	English!DELIM!Spanish	Yes	\N	01 Haverhill - Methuen - Lawrence	Plenty of free, off-street parking.	\N	Yes	Not Applicable	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
507546	102959	2008-10-01	\N	Adult Rowing Program	\N	1 Eaton Street	\N	Lawrence	MA	01842	603-553-6349	\N	\N	\N	http://www.greaterlawrencerowing.com	public_program_email@testing123.net	Competitive Adult Rowing	Greater Lawrence Rowing offers a competitive rowing program for area adults year round.  Participants will be able to learn and improve technical rowing skills needed to compete, as well as improve their cardiovascular and aerobic strength.  The goal for the participants is to learn the skills to be able to compete in the sport of rowing by getting as much practice time as possible.  There is no experience necessary to join and all are welcome!	All area residents are eligible to join.	Yes	The current competitive program fee is $200 per season.  This makes us one of the most affordable programs in the Greater Boston area.	The programs are open year round.	The normal practice schedule is 3 days a week, however that may change depending on the season and coaches discretion.	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
507535	102959	2008-10-01	\N	Youth Rowing Program	\N	1 Eaton Street	\N	Lawrence	MA	01842	603-553-6349	\N	\N	\N	http://www.greaterlawrencerowing.com	public_program_email@testing123.net	Competitive Youth Rowing	Greater Lawrence Rowing offers a competitive rowing program for area high school students year round.  Students will be able to learn and improve technical rowing skills needed to compete, as well as improve their cardiovascular and aerobic strength.  The goal for the students is to learn the skills to be able to compete in the sport of rowing by getting as much practice time as possible.  There is no experience necessary to join and all participants are welcome!	All area residents are eligible to join.	Yes	The current competitive program fee is $300 per season.  This makes us one of the most affordable programs in the Greater Boston area.	We have programs that run year round.	The normal practice schedule is 5 days a week, however that may change depending on the season and coaches discretion.	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
503509	100341	2008-10-01	\N	Arc of Greater Lawrence	\N	One Parker Street	\N	Lawrence	MA	01843	978-975-8587	\N	\N	978-975-0498	www.classinc.org	public_program_email@testing123.net	supports families and individuals with disabilities	The Arc of Greater Lawrence is developing and offering services to support individuals with disabilities and their families from the towns of Lawrence, Andover, N. Andover, and Methuen, MA. The Arc's focus is to provide advocacy and information through a resource network as well as educational advocacy on behalf of school age children. The Arc provides information via publications and a website as well as offer workshops and personal counseling around planning for the future.	membership is open to all individuals with disability and interested family and community members.	Yes	Membership fees.	Open year round	Monday through Thursday 8 - 4	English!DELIM!Spanish	Yes	\N		available in front of building	Transportation is sometimes included for specific programs	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Contact the Director for additional information.	Yes	\N	Yes	Several conference rooms may be available for professional development activities. Not open to general public.	Yes	\N	Yes	Additional information about our services can be found at our website www.classinc.org.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	translation services are provided during workshops. Family Support group is offered in Spanish.	No	0	\N
502717	100341	2008-10-01	\N	Day Habilitation Services	\N	One Parker Street	\N	Lawrence	MA	01843	978-975-8587 x -1339	\N	\N	978-794-7720	www.classinc.org	public_program_email@testing123.net	Provides therapeutic services and self care training	CLASS Day Habilitation Services provide individuals with developmental disabilities developmental skills training (to include self care, self help, sensorimotor development, communication, socialization, independent living, affective development, behavior development, leisure skills)which is supported by the agency's interdisciplinary clinical services team. Clinical services include health, psychological, occupational therapy,physical therapy, behavioral, and communication. Day Habilitation services also include community recreation and volunteer experiences. CLASS also provides these services at local nursing homes in the Merrimack Valley and Lowell areas to address the expanding needs of elders with developmental disabilities. Services are offered at our Lawrence and North Reading, MA locations. The North Reading address is 48A Main Street.	Ages 22 and up. Referrals through Dept. Mental Retardation or UMASS Medical School Nursing Home Project. Self-referral welcome as well.	Yes	Majority funded through Medicaid. Other funding sources include Dept. of Mental Retardation, Massachusetts Rehabilitation Commission or UMASS. Alternative funding arrangements may be accepted.	Services available year round. Program capacity is 108 in Lawrence, 30 in N. Reading	Monday-Friday 9am - 3pm	English!DELIM!Spanish	Yes	Appointments are necessary. Call for more information.		accessible parking available at the front and rear of building.	Provided by agency or other DMR funded transportation. Eligibility determined based on funding & seat availability. Generally funded through DMR or Medicaid	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Inquiries should be directed to the Director of Day Habilitation Services. Internships available those majoring in the human service fields. Opportunities for artists, musicians, performers, office support, etc.	No	\N	Yes	Several conference rooms may be available for professional development activities. Not open to general public.	Yes	\N	Yes	Learn more about our services through our website at www.classinc.org	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA	978-683-6125	We also provide art classes conducted by an art therapist and other qualified professionals in the fine arts.	Sign Language	No	0	\N
502739	100341	2008-10-01	\N	Employment Services	\N	One Parker Street	\N	Lawrence	MA	01843	978-975-8587 x -1221	\N	\N	978-794-7720	www.classinc.org	public_program_email@testing123.net	Provides job training to adults with disabilities	Employment Services at CLASS, Inc. provides job training to adults with disabilities and employment opportunities in a variety of settings. Individuals work in our production facilities, at supported job sites and at individual placements in area businesses. Each individual is assigned a case manager and has available an array of clinical supports, including health, behavioral, occupational and communication. Services are offered at our Lawrence and North Reading, Massachusetts locations. The North Reading location is at 48A Main Street.	Ages 22 and up. Referrals through Dept. Mental Retardation, Massachusetts Rehabilitation Commission or UMASS Medical School Nursing Home Project. Self-referral as well	Yes	Majority funded through Dept. Mental Retardation, MRC, or UMASS. Alternative funding arrangements may be accepted.	Services are available year round. Hours 9AM-3PM. Program capacity is 131 in Lawrence, 21 in N. Reading	Monday-Friday 8AM-4PM	English!DELIM!Spanish	Yes	Appointments are necessary. Call for more information.		\N	Provided by agency. Eligibility determined based on funding & seat availability. Generally funded through DMR.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Contact Human Resources Department for information pertaining to Volunteer opportunities at CLASS	No	\N	Yes	Several conference rooms may be available for professional development activities. Not open to general public.	Yes	\N	Yes	www.classinc.org	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
503113	100990	2008-10-07	\N	Billerica Public Library	\N	15 Concord Road	\N	Billerica	MA	01821	978-671-0948	\N	\N	978-670-9493	www.billericalibrary.org	public_program_email@testing123.net	Library	The mission of the Billerica Public Library is to provide responsive library service to residents through free access to diverse materials, programs and information.	Open to all	Free	\N	Services offered year round	Monday-Thursday 9AM-9PM; Friday 9AM-5PM; Saturday 9AM-5PM (Sept.-June 15 only); Sunday 1-5PM (Oct. 20-April 29only)	English	No	\N		Free parking in rear of building	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	\N	Yes	\N	Yes	Quarterly	Yes	Free Internet access.  Free wireless access with your own laptop.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506204	101474	2008-10-07	\N	Extensive list of services and programs available.	Call for monthly Newsletter.	63 Hanover Street	\N	Newbury	MA	01951	978-462-8114	\N	\N	978-499-0675	\N	public_program_email@testing123.net	Extensive list of services and programs available.	Call Newbury C.O.A. for details.	Newbury residents given priority.	Free	Most free. Some to cover out of pocket costs.	Open year 'round.	Monday through Friday 8:00 - 3:00.	English	No	\N		Limited.	For some events. Call for details.	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Active volunteer program. Call C.O.V. for information.	Yes	Extensive user friendly cross referenced file of materials relevent to elders; staff available to assist and make copies.	Yes	Newbury Elementary School -500 square feet	Yes	Monthly; mailed free to Newbury residents over 60.	Limited	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Newbury, MA	\N	\N	\N	No	0	newburycoa@hotmail.com ?
508184	100759	2008-12-15	\N	Open Hand Food Pantry	\N	15 Kenoza Avenue	\N	Haverhill	MA	01830	978-372-2828	\N	\N	\N	www.ccab.org	public_program_email@testing123.net	Food pantry serving Greater Haverhill	Food and clothing available through Merrimack Valley Catholic Charities. In addition to food and clothing, other miscellaneous items such as furniture also are available.	Individuals and families in Greater Haverhill.	Free	\N	Open year round.	Tuesdays and Wednesday from 9-11:30 a.m	English	No	New clients need to bring in a photo ID, proof of address and social security cards for everyone in the family.		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Contact Catholic Charities for volunteer opportunities.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Haverhill, MA	\N	\N	\N	Not Applicable	0	\N
503421	101089	2008-07-16	\N	Habitat for Humanity of Greater Lowell	HFHGL	66 Tadmuck Road	Suite 5	Westford	MA	01886	978-692-0927	\N	\N	\N	www.lowellhabitat.org	public_program_email@testing123.net	Affordable Homeownership	Habitat for Humanity of Greater Lowell (HFHGL) strengthens families and communities through affordable homeownership. Through volunteer labor and monetary and in-kind donations, HFHGL is able to provide decent, safe, affordable homes for families earning between 25 and 50 percent of the area medican income level.	Several criteria apply. For more information, please contact familyselection@lowellhabitat.org.	Yes	Must repay a no-interest mortgage.	HFHGL operates year round, but homeownership applications are only given out when building projects become available.	9:00 a.m. - 5:00 p.m. If the HFHGL voicemail picks up, please leave a message and someone will return your call as soon as possible.	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Construction, Family Selection, Family Support, Fund Raising, PR, Property Acquisition	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA!DELIM!Wilmington, MA	\N	Burlington, Carlisle, Concord, Reading, Wakefield	\N	No	0	\N
502376	100429	2008-07-16	\N	Pulmonary Diagnostics	PDX	One Hospital Drive	\N	Lowell	MA	01852	978-934-8235	\N	\N	978-934-8268	www.saintsmedicalcenter.com	public_program_email@testing123.net	Pulmonary Diagnostics	Pulmonary Diagnostics provides a wide range of testing used to evaluate and diagnose lung conditions.With this information a treatment plan can be developed.	Contact department for information	Yes	\N	Open year round	Monday-Friday 8AM-4PM, excluding holidays	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Physician orders, insurance information		Parking available in on-site garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505368	100165	2008-12-01	\N	Enhanced Outpatient Program	EOP	126 Phoenix Ave, 3rd Floor	\N	Lowell	MA	01852	978-937-3087	\N	800-422-3035	978-937-8695	www.mspcc.org	public_program_email@testing123.net	Mental health services with community supports	The Enhanced Outpatient Program is a home-based and strengths based wraparound model that works to build partnerships with collateral and support persons in the childs life in addition to providing individual and family therapy services. Service planning and delivery goals come from the strengths and needs identified, and are tailored to meet the familys culture, values, and priorities. The program involves an Individualized Care Plan Team, including the family, professionals, and natural supports, that develops goals and provides support to the family as they work to reach these goals.  EOP emphasizes the importance of coordination, linkage, and service provision in home, school, and community environments in order to meet the mental health needs of the child and family.	MBHP insurance only. Children must be 5-18 years old. Family must live in Greater Lowell Area and agree to participate.	Yes	MBHP insurance only	year round	9 to 7, 5 days a week, 24 hour coverage.	English	Yes	\N	02 Belvidere	Parking available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	One conference room in Lowell may be available. Sign up through receptionist.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
507139	102728	2008-07-15	\N	Full-Service Financial Institution	Banking	83 Merrimack Street	\N	Lowell	MA	01852	978-328-5100	866-622-6628	\N	978-458-8933	www.macomfcu.org	public_program_email@testing123.net	Full-Service Financial Institution	At M/A-COM Federal Credit Union, we strive to be your primary financial resource by offering you the most convenient and cost-effective method to manage your finances.\r\n\r\nM/A-COM FCU is owned and operated by its members. Once you become a member of M/A-COM FCU, youre a member for life, even if you move, change employers or retire. A member may retain membership with full member rights and privileges, provided they maintain the required minimum share balance of $25.00, remain in good standing, and keep the credit union informed of a current mailing address. \r\n\r\nWe take pride in offering all of our members the highest quality financial products and services at the most competitive rates, while nurturing the friendly and cooperative spirit of the credit union. We are committed to the financial well-being of our members and will continue to work for the secure future of our community.\r\n\r\nServices include:\r\n- Savings Accounts\r\n- Rewards Checking Accounts\r\n- Loans\r\n- and more!	There are several ways to qualify for membership at M/A-COM FCU.	Yes	It costs $25 to open an account. That balance remains in your account.	Open year round.\r\n\r\nCan bank online at our website 24 hours a day, 7 days a week!	83 Merrimack Street\r\nLowell MA 01852\r\nMonday: 9:00 a.m.  4:00 p.m. \r\nTuesday: 9:00 a.m.  4:00 p.m.\r\nWednesday: 9:00 a.m.  4:00 p.m.\r\nThursday: 9:00 a.m.  7:00 p.m.\r\nFriday: 9:00 a.m.  5:00 p.m.\r\nSaturday: 9:00 a.m.  12:00 p.m	Portuguese!DELIM!Spanish!DELIM!Khmer	No	\N	41 Lawrence-Lowell!DELIM!03 South Lowell!DELIM!04 Shaw - Stevens!DELIM!05 Westford Street/Drum Hill!DELIM!06 Broadway / UMass Lowell!DELIM!07 Pawtucketville / UMass Lowell North!DELIM!08 Centralville!DELIM!09 Downtown Circulator	On-street; John Street Garage	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	\N	Yes	Quarterly newsletter sent to all members; e-news letter sent monthly	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	Gujarati	Yes	0	\N
507029	102695	2008-07-15	\N	Parent Information Center	\N	17 Kirk St.	\N	Lowell	MA	01852	978-937-8986	\N	\N	978-937-3468	www.lowell.k12.ma.us	public_program_email@testing123.net	Parent Information Center	What We Do\r\n\r\nWe offer counseling to families as they select schools for their children.\r\n\r\nWe test all English language learners  for language dominance and establish an appropriate placement for them.  \r\n\r\nWe work with the Special Education Department and the English Language Education Department to ensure that the unique needs of each student are being met.\r\n\r\nWe coordinate school assignments with the Transportation Department, since many of the students receive transportation.\r\n\r\nWe host a Kindergarten Orientation night for families of incoming Kindergarten students.  \r\n\r\nWe work closely with Attendance Office and schools to improve student attendance.\r\n\r\nWe display school work and  we have brochures of agencies and organizations for families.\r\n\r\nWe act as a  resource for families.  \r\n\r\nWe speak four languagesEnglish, Spanish, Khmer and Lao.	Preschool - Grade 8	Free	\N	Open year round	Monday  Friday\r\n8:00 AM  4:00 PM\r\n         \r\nThursday\r\n5:00 PM  7:00 PM\r\n(except during summer time)	English!DELIM!Spanish!DELIM!Khmer	No	What Is Needed For Registration\r\n\r\nProof of Birth\r\nImmunization Record\r\nProof of Lowell Address\r\nIEP/Transfer Slip (If applicable)		Street parking	\N	Yes	No	\N	Yes	WAIT LIST\r\n\r\nIn those cases when a child cannot, for any reason, be assigned to the school or program of his/her first choice, the parent/guardian may request to have the childs name placed on the Wait List for that school.  Students on the Wait List have priority over new entrants to the school system.  The order of names on the Wait List is determined by applying Criteria #1 - #4 above and by the date of request. Placements from the Wait List are made during the summer recess and continue up to the end of the first marking period.\r\nNote:  Private, Parochial or Charter School students must renew their registration each year if they do not wish to accept the offered seat for the registered school year.	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
501650	100429	2008-07-15	\N	Blood Donor Program	\N	One Hospital Drive	Ground Floor	Lowell	MA	01852	978-934-8584	\N	\N	978-934-8396	www.saintsmedicalcenter.com	public_program_email@testing123.net	Blood Donor Program	Donors give blood as a means of aiding others. Any healthy individual who meets the acceptable criteria may donate blood.	Over 17 years of age, more than 110 pounds, and feeling well (not on antibiotcs).  Most medications ARE acceptable.\r\nContact program for more information.	Free	\N	Open daily Monday thru Friday closed on all Major holidays.	Monday 7:30am - 2:30pm\r\nTuesday 7:30am - 7:00pm\r\nWednesday 7:30am - 3:30pm\r\nThursday 7:30am - 5:30pm\r\nFriday 7:30am - 2:30pm	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	Picture Identification required		Free Valet parking until 5pm\r\nFree Garage parking\r\nDesignated Blood Donor parking spot in the rear of the building	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl Satryb 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Blood Drives are held in surrounding towns at local businesses and schools.	\N	No	0	\N
501804	100308	2008-07-15	\N	Mental Retardation Residential Services	\N	99 Church Street	\N	Lowell	MA	01852	978-446-2009	\N	978-458-6282	978-441-9826	\N	public_program_email@testing123.net	Residential services for adults with mental retardation	We provide residential services and support for adults with mental retardation in a variety of living situations. The goal of the support services is to encourage and assist individuals in obtaining their goals, dreams and wishes. Personal growth, development and satisfaction are emphasized in our approach. Some of the skills training include the following areas: activities of daily living, community involvement, socialization, safety skills, recreational / leisure skills, preventive health care, crisis intervention. The services and support are based in the individuals home or apartment. The level of supports may vary from six hours a week to 24 hours a day. We currently support individuals in two different types of services: 24 hour based supports and individual hour based supports.	Must be referred by the Massachusetts Department of Mental Retardation. Ages 18 and older.	Yes	Individuals contribute fees for living expenses.	Services available year round.	24 hours /day, 7 days / week	English!DELIM!Spanish	Yes	Department of Mental Retardation will make referral.		Yes	Van transportation is provided for program residents. Individuals are assisted in utilizing public transportation as needed.	Yes	No	\N	No	All referrals are made through the Department of Mental Retardation.	\N	\N	\N	\N	\N	\N	Yes	Volunteers are welcomed to assist with  individual advocacy, to assist individuals with special projects, and spring /fall yard work	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
500231	100033	2008-07-15	\N	Housing Consumer Education Center	HCEC	167 Dutton Street	\N	Lowell	MA	01852	978-459-0551	800-698-0551	\N	978-970-1483	www.comteam.org	public_program_email@testing123.net	Educate & qualify moderate income people for homeownership	The purpose of the Housing Consumer Education Center is to educate and counsel tenants, landlords and homeowners to make informed housing decisions. Services offered to tenants: Emergency Housing Assistance; housing search assistance; FEMA; RAFT; Fair Housing; budget counseling; Tenants / Landlord's rights and responsibilities workshops; Rental property listing; Information and Referral; First Time Home Buyers classes; Post Purchase Workshop; Foreclosure prevention counseling, etc.	Contact program	Free	\N	Services available year round	Monday-Friday 9AM-5PM	English!DELIM!Spanish	No	Contact program		On-street and public parking in the area.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Yes	www.comteam.org/Housing Consumer Education Center	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	0	\N
502013	100594	2008-07-15	\N	Emergency Family Shelter	House of Hope, Inc.	812 Merrimack Street	\N	Lowell	MA	01854	978-458-2870	\N	978-458-2870	978-458-6679	www.hopelowell.org	public_program_email@testing123.net	Family Homeless Shelter & Homeless Services	The Emergency Family Shelter provides 24 hour shelter, 3 meals a day, case management services, supportive workshops, advocacy, a nurse on site and job and educational services. The Department of Transitional Assistance refers families in need to House of Hope, which is contracted with DTA for all 18 available units. We do not work from a waiting list, however 20-30 families call each month and we are unable to accommodate them due to a lack of space. Referral Assistance is offered to each call to help them to determine their next step.	Referrals are usually required. Homeless women and their children referred by the Department of Transitional Assistance.	Free	\N	Services available year round. Average length of stay: 6-9 months. Capacity: 18 families	24 hours a day, 7 days a week	English!DELIM!French!DELIM!Spanish	Yes	At intake have clothing, personal toiletries & proper paperwork like; eviction notices, DTA paperwork, social security card, birth certificates for all family members and other identification.	07 Pawtucketville / UMass Lowell North	Parking available in side lot.  On street parking is usually available.	House of Hope is able to provide emergency transportation.	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers assist us in a variety of areas: child care, reception, food prep etc. Contact Edna at the shelter.	No	\N	Yes	House of Hope does have a small conference room that could be available for community meetings.	Yes	Newsletter mailed 3-4 times per year.  Contact Andrea at House of Hope to be included on mailing list.	No	www.hopelowell.org	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
500275	100033	2008-07-15	\N	YouthBuild of Lowell	CTI YouthBuild	391 Pawtucket Street	\N	Lowell	MA	01854	978-446-9803	\N	\N	978-446-9469	www.comteam.org	public_program_email@testing123.net	Educ & skills training For HS drop out 17-24 yrs.	The purpose of CTI YouthBuild of Lowell program is twofold:\r\n1. To provide disadvantaged young adults (ages 17-24) with job readiness and job skills training, GED preparation, leadership training and resulting employment or education placement.\r\n2. To expand the supply of affordable housing by building and rehabilitating housing for low-income and homeless persons.	17-24 years old, low-income, dropped out of school and work, residing in the Enterprise Community, Lowell and the surrounding towns	Free	\N	Contact Program. 9 month program with groups starting in  September each year.	Monday-Friday 8AM-4PM	English!DELIM!Portuguese!DELIM!Spanish	Yes	Social Security Card, birth certificate, proof of residence, household income.	06 Broadway / UMass Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking available on site	\N	No	No	Assistance provided to enroll applicants children in CTI daycare.	Yes	Accept applications 12 months.  New cycles begin in September of each year.  Competitive acceptance based on strength of application.  Waiting list updated monthly. Application includes intake, Academic assessment, interview, volunteer service, submit required documentation.  2 week orientation beginning 3rd week of September.  Successful applicants are invited to start program with a mandatory 1 month probationary period.	\N	\N	\N	\N	\N	\N	Yes	Program needs tutors, Job shadow opportunities, esp in contracting	No	\N	Yes	Old Red Cross Building.  2 classrooms, computer lab, kitchen and bathroom facilities	Yes	semi-annual	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
500242	100033	2008-07-15	\N	Housing Programs	\N	167 Dutton Street	\N	Lowell	MA	01852	978-459-0551	800-698-0551	\N	978-453-9128	www.comteam.org	public_program_email@testing123.net	Provides information and resources for affordable housing	The Division of Housing and Homeless Services  provides low-income families and individuals with housing, shelter for families, support systems, referrals and linkages resulting in housing stabilization and financial self-sufficiency.	Contact program	Free	\N	Services available year round	Monday-Friday 8:30AM-5PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	Contact program		\N	\N	Yes	Not Applicable	\N	Yes	Some programs in the housing department have waiting lists.  Call for more information	\N	\N	\N	\N	\N	\N	Yes	General office duties and assistance with phone calls.	Yes	Housing related pamphlets available at no cost.	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
500209	100033	2008-07-15	\N	Home Modification Loan Program	\N	167 Dutton Street	\N	Lowell	MA	01852	978-459-0551 X 297	\N	\N	978-970-1483	www.comteam.org	public_program_email@testing123.net	Assist with loans to make homes handicap accessible	CTI's Home Modification Loan Program provides a loan to families with functional disabilities to increase the accessibility and utilization of their home for their disabled family member. This statewide program is funded by Mass Rehab and CEDAC. Loans range from $1,000 to $30,000 per household at 0% or 3% interest depending on eligibility. Landlords are also eligible for loans of 5% to modify an appartment for a disabled tenant.	Contact program administrator for more information. Proof of income and disability needed.	Free	Interest rate is related to income per family size.	Program services available year round	Monday-Friday 9AM-5PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Proof of income and disability needed.		Public parking is available near CTI	\N	Yes	No	\N	No	\N	Program serves Northern Middlesex county and Southern Essex County	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	0	\N
500913	100121	2008-07-15	\N	Aerobics	Group Exercise Programs	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Group exercise classes.	Group exercise classes, yoga, NIA, water Fitness all come with membership.	Open to anyone	Yes	Classes for members are free. (Passes for non-members are $105 for 21 classes).	Program runs year round	Monday-Friday 5:30AM-10PM; Saturday 7AM-7PM; Sunday 7AM-4PM	English	No	\N		Parking is available.	\N	Yes	Yes	Babysitting is available for morning clases, Monday - Friday 9:00 Am - 1:15 PM	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Space is available for racquetball/ volleyball leagues; soup kitchen; pool parties, etc.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
500924	100121	2008-07-15	\N	Aquatics/ Swim Lessons/ Swim Team	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Pool for laps and lessons as well as swim teams	Lap swimming for members. Swimming lessons for participants 6 months-adult at various sites including the YMCA, the Greater Lowell Technical School, Shawsheen Technical School, Sargisson Town beach in Groton and Camp Massapoag in Dunstable. Competitive swimming for children ages 5-18. Water fitness, aqua yoga and arthritis classes. Backyard swimming lessons. Waterfront Management Services. Pool Parties.	Each program has different requirements.	Yes	Fees vary by program.  Fees payable by cash, check or credit card	Now accepting Barracuda Swim team registrations. Fall Session registration begins 08/25/2007.	Monday-Friday 5:30AM-9:30PM; Saturday 7AM-6:30PM; Sunday 7AM-3:30PM (except Memorial thru Labor Day: Sundays 7AM-11:30AM)	English	Yes	Contact program to register, fees payable by cash, check or credit card		Parking available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Judith McGrade	No	\N	Yes	Space is available for racquetball/basketball leagues; soup kitchen; pool parties, etc.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Also serving Groton and Pepperell MA.	\N	No	0	\N
500957	100121	2008-07-15	\N	Lowell YMCA Kids Club Camp	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	www.greaterlowellymca.org	public_program_email@testing123.net	Summer camp with theme weeks	This is a summer camp which has theme weeks. One field trip a week and educational enrichment and recreational activities focused around the theme. This camp has a mix of outdoor and indoor activities as well as a daily swim time in the indoor YMCA pool.	Ages 5-13	Yes	$35 Registration fee. $160 per week, $80 half day\r\nFinancial aid may be available	Program runs June - August. Enrollment begins in April and is rolling until camp is full. Program capacity is 60.	Monday-Friday 7AM-6PM	English	Yes	Must fill out emergency forms. Parents must register for the specific dates they want their child to attend.		Parking available in lot.	For those in Lowell public summer school, there is 1/2 day transportation from school to YMCA	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are always appreciated. Contact Judith McGrade.	No	\N	Yes	Space is available for racquetball/ volleyball leagues; soup kitchen; pool parties, etc.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
500968	100121	2008-07-15	\N	Lowell YMCA's School's Out Program	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	www.greaterlowellymca.org	public_program_email@testing123.net	After School Child Care	We provide child care for those in kindergarten to age 12 before school, after school and during school vacations. We have a traditional recreational base of activities, but have an ever increasing amount of educational enrichment programs. There are three program locations: Lowell YMCA;  Reilly School: Douglas Road; Hellenic School: 41 Broadway Street. Parents have 3 transportation options, they can 1. Drop off their children 2. School bus drop off 3. Walk.	5-13 year old Lowell residents attending a Lowell public / private school where transportation to 1 of 3 sites can be arranged	Yes	$90/week for 2PM-6PM schedule, $55/week 7AM-9AM achedule. $32/ day for school vacations 7AM-6PM schedule. Financial aid may be available.	September-June. Rolling enrollment as long as space is available. YMCA site has waiting list, other sites have openings.	Monday-Friday 7AM-9PM, 2PM-6PM except school vacations: Monday-Friday 7AM-6PM	English	Yes	Call to request forms.		Each site has a parking lot	Lowell public transportation and school bus routes stop at sites	Yes	Yes	\N	Yes	YMCA site has waiting list, other sites have openings.	\N	\N	\N	\N	\N	\N	Yes	Volunteers are always appreciated. Contact Judith McGrade	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
503685	100121	2008-07-15	\N	Membership	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Financial Assistance for single or family memberships	Financial assistance is available for those that qualify.  Simply request a financial assistance application and mail completed application back to the attention of Adele Tsoukaris.	Available for anyone that qualifies.	Sliding Scale	\N	Open year round.	Monday - Friday 5:30 AM - 10:00 PM Saturdays 7:00 AM - 7:00 PM.  Sunday 7:00 AM - 4:00 PM. Sundays Memorial thru Labor Day 7:00 AM - Noon	English	Not Applicable	\N		Ample parking available.	\N	Yes	Yes	Monday - Friday Mornings free to members - 2 hour limit.	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers always welcome.  Contact Judith McGrade at ext. 11.	No	\N	Yes	Space is available for racquetball/ volleyball leagues; soup kitchen; pool parties, etc.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501001	100121	2008-09-02	\N	YMCA T-Ball	\N	Manning Field	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	T-Ball League for 4 - 7 year olds	Bitty T-Ball teaches the basics, including: fielding, positions, sportsmanship, hitting, teamwork and fun. All games will be held at Manning Field.	Ages 4-7	Yes	$45 Members. $80  Program Members. Financial aid available.	Summer 2009 TBA	Tuesdays, Wednesdays, Thursdays between 6-8	English	Yes	Register at the YMCA front desk or over the phone		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer opportunities include coaching. Contact Fred Braun, Program Director if interested.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501276	100154	2008-07-15	\N	Lowell Family Literacy Center	\N	26 Palmer Street	\N	Lowell	MA	01852	978-970-5410	\N	\N	978-970-5414	\N	public_program_email@testing123.net	Early Childhood and Adult Education Classes	The Lowell Family Literacy Center provides: early childhood classes (ages 3-5 years), adult education classes (ESOL); preschool programs; early literacy / home visiting program; Child development information and resource and referral; book recycling program; resource lending library; parent education workshops; family literacy activities (story hours, musical programs).	All programs and activities are free and open to Lowell parents with young children ages 3-5 years.  Some programs have eligibility requirements.  Please call to find out specifics.	Free	\N	Year round programming, summer program available; Open enrollment: sign up required for family literacy activities	Monday-Friday 8AM-4PM; Some nights and weekends	English!DELIM!Spanish	Yes	Some programs require completion of registration forms for both adults & children		On street metered parking available	Student bus passes are provided for families participating in our Even Start Family Literacy Program.	Yes	No	\N	No	\N	Translators who speak Khmer and Portuguese can be arranged	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Pat Murphy-Painchaud. Opportunities include library/ resource area & classroom support	Yes	Lending Library for parents of young children is available.	No	\N	Yes	An events flyer is distributed via a mailing list.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	Lowell residents only.	\N	No	0	\N
501232	100154	2008-07-15	\N	Community Development	CDBG/HOME/ESG/HOPWA Grant Programs	Division of Planning and Development	50 Arcand Drive, JFK Plaza	Lowell	MA	01852	978-446-7200	\N	\N	978-446-7014	www.lowellma.gov	public_program_email@testing123.net	Administers Federal Grant programs	Administers grants awarded on an annual basis, to programs providing affordable housing, economic development, public improvements, and supportive services for youth, elderly, special needs.  Grants are provided through HUD Federal Programs (CDBG, HOME, ESG, and HOPWA)	Non-profits/for-profit agencies whose primary purpose is to assist low/moderate income households in Lowell	Free	Not applicable	RFPs are usually issued annually in mid-winter.  Funds become available July 1.  Normal contract for award recipients is 12 months.	Monday-Friday 9AM-5PM	English	No	\N	09 Downtown Circulator	Metered street parking is available	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	Public documents available on web-site www.lowellma.gov/depts/dpd/services/comdev	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	HOPWA funds may assist agencies throughout Middlesex County	\N	No	0	\N
502497	100748	2008-07-15	\N	Project Genesis Home Buyer Training Program	\N	10 Kirk Street	\N	Lowell	MA	01852	978-459-8490	\N	\N	\N	www.mvhp.org	public_program_email@testing123.net	First time home buyer classes	Project Genesis is a comprehensive series of training seminars for 1st time home-buyers.  Down Payment assistance is available to eligible buyers who purchase a home in Lowell.	Open to all people interested in purchasing a home.	Yes	$60 per family. This fee is waived in cases of hardship - contact program for information.	New classes begin every month in both English and Spanish. Classes are held at Middlesex Community College.	Monday-Friday 9AM-5PM. Classes are held in the evenings from 6:30 p.m. until 9:00 p.m.	English!DELIM!Spanish	Yes	Please call the office to register.		Parking available on street and in the public garage.	\N	Yes	No	\N	No	\N	\N	P.O. Box 1042	\N	Lowell	MA	01853	Yes	Many bankers, attorneys, home inspectors, real estate brokers volunteer their time and talents to help teach the seminars.	No	\N	No	\N	Yes	Genesis News	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
502090	100638	2008-07-16	\N	The Brush Art Gallery and Studios	A Brush With History	256 Market Street	\N	Lowell	MA	01852	978-459-7819	\N	\N	978-453-9886	www.thebrush.org	public_program_email@testing123.net	Art Exhibitions and Cultural Programing	The mission of the Brush Art Gallery and Studios is: To nurture the working artist; To keep the visual arts alive and healthy in the heart of Lowell's Historic District; To demonstrate through exhibitions and cultural programming the diverse and rich heritage that exists in Lowell; and to make Lowell a destination for viewing some of the finest art being produced in New England and beyond.	Open to all	Free	\N	Jan 1-March 31, Wednesday through Sunday 12:00-4:00--April 1-December 31, Tuesday-Saturday 11:00A-5:00P, Sunday 12:00-4:00P	Jan 1-March 31, Wednesday through Sunday 12:00-4:00--April 1-December 31, Tuesday-Saturday 11:00A-5:00P, Sunday 12:00-4:00P	English	No	\N		Free parking is available in the Lowell National Historical Park Visitor Parking lot.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	We often have openings for volunteers to handle our front desk: answer phone, sit at desk; sometimes there is clerical work.	Not Applicable	\N	Yes	One medium size classroom for education / art classes or meetings is available on request. The Gallery can be rented out for events of up to 150 people.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502970	100924	2008-07-16	\N	Habit OPCO	Habit Management	650 SUFFOLK STREET	\N	Lowell	MA	01854	978-452-5155	\N	\N	978-970-0713	www.habitopco.com	public_program_email@testing123.net	Provides treatment for opiate dependency	Habit OPCO provides treatment for opiate dependency to individuals and communities in need. There are several clinics in the area, including Lowell, Lawrence and Fitchburg. Please refer to the website for more information.	Client must have some form of financial means, either private pay or insurance	Yes	MassHealth, Blue Cross / Blue Shield (call insurance for authorization),	Length varies. There are two program options: 6 month program and Maintenance program	Monday-Friday 6AM-1PM (counseling); Staff available until 4PM; Dosing Monday-Friday 6AM-11:15AM; Saturday-Sunday 6:15-10:15AM	English!DELIM!Spanish	Yes	Photo ID, Proof of residency, MassHealth or other insurance card		Street Parking on Suffolk Street.  Limited parking in front and side of clinic.	\N	Yes	No	\N	No	\N	\N	ATTN: Jerry Myers	650 Suffolk St.	Lowell	MA	01854	No	\N	No	\N	Not Applicable	\N	No	\N	Yes	habitopco.com	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	978-970-0724	\N	\N	No	0	\N
501023	100121	2008-07-16	\N	YMCA Preschool	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	www.greaterlowellymca.org	public_program_email@testing123.net	Pre-School Program for ages 2.9- 6years	Our program is designed to create an environment that both stimulates and strengthens each child's physical, educational and social development.  \r\nOur curriculum gives each child an opportunity to experience many different activities each day.  We incorporate Building language for literacy, specialized learning centers, weekly swim lessons, games, and character development.  We are fully licensed by the Massachusetts Department of Early Education and Care AND accredited by the National Association for the Education of Young Children. Open house 10/7!	We accept registrations for children ages 2.9 years and older. Private pay, Voucher, YMCA Scholarships, and Community Partnership for Children are accepted.	Yes	Full time: $175.00, 2 am's (9am-12) $200.00/ month, 3am's $278/month. The YMCA offers scholarships based on income. State Vouchers.	Program runs year round. Enrollment is in fall and summer. Program capacity is 50.	Monday-Friday 7AM-6PM\r\nWe offer both full day and morning only preschool slots.	English!DELIM!Spanish	Yes	Along with completed enrollment packet, children must have physical and updated immunizations		Parking available	\N	Yes	Yes	\N	Yes	Waiting list averages 3 spots	Only English is spoken on the phone	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Judy McGrade ext 11	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	Also serving Groton and Pepperell MA.	\N	No	0	\N
502277	100429	2008-07-16	\N	Interpreter Services	SMC	One Hospital Drive	\N	Lowell	MA	01852	978-934-8309	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Provides interpreters for foreign languages	Saints Medical Center provides interpreters for a number of foreign languages including: Spanish, Cambodian (Khmer), and Portuguese.	Contact program for more information.\r\nPatient Hotline: 978-934-8255, messages available in English, Spanish, Portuguese and Khmer	Free	Contact Program for More Information	Open year round	Contact program for more information	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Parking available in on-site garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Contact Mary Hession 978-458-1411	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4009 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County	\N	\N	Patient Hotline: 978-934-8255. Messages available in English, Spanish, Portuguese and Khmer	No	0	\N
502266	100429	2008-07-16	\N	Growth and Nutrition Clinic	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8458	\N	\N	978-934-8358	www.saintsmedicalcenter.com	public_program_email@testing123.net	Growth and Nutrition Clinic	The Growth and Nutrition Clinic provides outpatient medical, nutritional, developmental, psychosocial and sensory assessments for children up to 5 years old. Home visits are offered to evaluate the feeding environment, family stresses and need for additional community services. An inservice and education program for medical, nursing and community groups is available.	Program is for families with children up to age 5	Free	\N	Services are available by appointment only, contact program for more information	Monday - by appointment only	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Free parking is available in the hospital garage	\N	No	No	\N	No	\N	\N	One Hospital Drive, 4th Floor	\N	Lowell	MA	01852	No	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502420	100429	2008-07-16	\N	Volunteer Department	SMC	One Hospital Drive	\N	Lowell	MA	01852	978-934-8309	\N	\N	978-934-8526	www.saintsmedicalcenter.com	public_program_email@testing123.net	Volunteer Department	The Saints Medical Center Volunteer Department places volunteers throughout the hospital in a variety of positions, including the gift shop, patient transport, information desk, radiology, medical records, sugical day care, and the cancer center. Several flexible shifts can be arranged. Interviews are required. A general hospital orientation is mandatory for all volunteers. It is a one-time 8 hour session.	Must be 14 years or older to volunteer. Volunteers need to speak English. Bilingual applicants welcome.	Free	\N	Available year round	Monday-Friday 8:30AM-4:30PM	English	Yes	An application needs to be completed. A record of health status is required, along with shot history, chickenpox, measles, etc		Free parking is available in hospital garage.	\N	Yes	No	\N	No	\N	\N	One Hospital Dr.	\N	Lowell	MA	01852	Yes	This is a volunteer program.	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
502431	100726	2008-07-16	\N	Adult Literacy Program	\N	401 Merrimack Street	\N	Lowell	MA	01852	978-446-7276	\N	\N	978-970-4117	http://www.pollardml.org/	public_program_email@testing123.net	Reading, Writing and Speaking Tutors	The Pollard Memorial Library Adult Literacy Program provides free, confidential, individualized tutoring for adults age 16 and over with low level literacy skills who wish to learn to read, write, or speak English. Tutors are available for Basic Reading and English for Speakers of Other Languages (ESOL). Potential tutors must be 18 or older with a high school diploma, and successfully complete 18 hours of tutor training. All training is provided.	The Pollard Memorial Library Adult Literacy Program provides free, confidential, individualized tutoring for adults age 16 and over with low level literacy skills who wish to learn to read, write, or speak English. Tutors are available for Basic Reading and English for Speakers of Other Languages (ESOL). Potential tutors must be 18 or older with a high school diploma, and successfully complete 18 hours of tutor training. All training is provided.	Free	\N	Services available year round.	Monday-Friday 9AM-5PM	English	No	Contact program - initial interview and assessment is required		Library parking lot on corner of Colburn & Moody streets. Metered on street parking.	\N	Yes	No	\N	Yes	There is a waiting list for the program, length varies.	\N	\N	\N	\N	\N	\N	Yes	All tutors are volunteers. Contact program for details.	Yes	\N	Yes	Contact library for more information	Yes	Quarterly newsletter is mailed to current students and tutors in program.	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
502453	100726	2008-07-16	\N	Friends of the Pollard Memorial Library	\N	401 Merrimack Street	\N	Lowell	MA	01852	978-970-4120	\N	\N	978-970-4117	http://www.pollardml.org/	public_program_email@testing123.net	Member-led group who support the work of the library	The Friends of the Pollard Memorial Library is a member-led organization whose mission is to support the work of the Pollard Memorial Library. The Friends sponsor an annual book sale each May and provide many programs for adults and children each year. They purchase most of the Library's museum passes.	Open to anyone interested in the group's mission	Yes	Yearly membership fees: Student / Senior $5; Individual $10; Family $15; Benefactor $100; Corporate $250; Bookseller $30	Not Applicable	Not Applicable	English	No	\N		Library parking lot on corner of Colburn & Moody streets. Metered parking on streets.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Friends are all volunteers. Contact library for details.	Yes	\N	Yes	Contact library for more information	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
502442	100726	2008-07-16	Hidden on 04-14-2004 by: the MVHUB Administrator<BR>Reason: <i>Not available at this time b/c of lack of funding, will reinstate as soon as funding is available</i>	Bookmobile	\N	401 Merrimack Street	\N	Lowell	MA	01852	978-970-4120	\N	\N	978-970-4117	http://www.pollardml.org/	public_program_email@testing123.net	Public Library's Branch on Wheels	The Pollard Memorial Library Bookmobile is a " branch on wheels" bringing the PML to residents of all ages, promoting literacy and bringing the joy of reading to the citizens of Lowell. Anyone may use the bookmobile at any stop. The bookmobile stops at various locations throughout the city of Lowell on a rotating schedule. Check the library website or call the library to request a schedule of stops.	Anyone wishing to borrow materials must have a valid PML card or a valid library card from a MVLC member library	Free	\N	Program is usually available year round	Bookmobile schedule is posted on library's website	English	No	\N		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
502464	100726	2008-07-16	\N	Museum Pass Program	\N	401 Merrimack Street	\N	Lowell	MA	01852	978-970-4120	\N	\N	978-970-4117	http://www.pollardml.org/	public_program_email@testing123.net	Borrow Free Museum Passes	The Pollard Memorial Library allows adult residents of Lowell, who have a valid PML library card, to borrow passes to local and regional museums and attractions. Reservations may be made in person or by phone. Outstanding fines must be paid before using a pass. The late fee for an overdue pass is $5.00 per day. Most passes are purchased by the Friends of the PML. Museums include: Boot Cotton Mills Museum; Children's Museum - Boston; Harvard Museum of Natural History - Cambridge; John F. Kennedy Library - Boston; Museum of Fine Arts - Boston; Museum of Science - Boston; New England Aquarium - Boston; New England Quilt Museum - Lowell; Whistler House Museum of Art - Lowell; Zoo New England: Franklin Park or Stone Zoo; DeCordova Museum and Sculpture Park - Lincoln; Isabella Stewart Gardner Museum - Boston; The Discovery Museums - Acton.	Valid Pollard Memorial library card required except for the 4 Lowell museums - these are available to any MVLC card holder.	Yes	Some passes offer free museum admission; others offer admission at a discounted price.  Refer to library's museum pass brochure for more detailed information.	Program available year round. Passes may be reserved up to a month in advance of the date of use.  Passes may be picked up the week of, day before, or day of use, depending on type of pass.  See library staff member for more information.	Monday-Thursday 9AM-9PM; Friday 9AM-5PM; Saturday 9AM-5PM (Summer closed Saturday)	English	No	Must have valid library card and potential dates you would like to reserve.  Outstanding fines must be paid prior to picking up musuem pass.		Library parking lot on corner Colburn & Moody Streets. Metered on street parking.	\N	Yes	No	\N	No	\N	Pollard Memorial Library card for museums outside of Lowell, any valid Merrimack Valley Library Consortium card holder for Lowell museums	\N	\N	\N	\N	\N	No	\N	Yes	\N	Yes	Contact library for more information	No	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
502475	100726	2008-07-16	\N	Youth Programs	\N	401 Merrimack Street	\N	Lowell	MA	01852	978-970-4122	\N	\N	978-970-4117	http://www.pollardml.org/	public_program_email@testing123.net	Variety of services for youth. Infants - High School	The Pollard Memorial Library provides a variety of services for youth, from infants through high schoolers. Drop-in storytimes are conducted in the fall, spring, and winter. The library's summer reading program encourages reading through special activities. Additional events are offered during school vacations as well as other times during the year. Programs for middle and high schoolers include crafts, movies, and guest performers.	Open to all youth, from infants through high schoolers.	Free	\N	Different programs run throughout the year	Monday-Thursday 9AM-9PM; Friday 9AM-5PM; Saturday 9AM-5PM (Summer closed Saturday)	English!DELIM!Spanish	Yes	Names and ages of children enrolling in programs needed at enrollment.		Library parking lot on corner of Colburn & Moody Streets.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers often present programs or assist those who are presenting. Interested volunteers should contact Beth Brassel.	Yes	\N	Yes	Contact library for more information	No	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
500748	100066	2008-07-16	\N	Center for Family Development of Lowell	\N	45 Merrimack Street, Suite 200	\N	Lowell	MA	01852	978-459-2306	\N	\N	978-453-9394	\N	public_program_email@testing123.net	Outpatient mental health clinic	The Center for Family Development program provides a full service outpatient mental health clinic for children, adolescents, and adults. We provide diagnostic and medication evaluations, and individual, couples, family and group therapy.	Must have health insurance or the ability to self-pay.	Yes	Fees vary by type of service; must have insurance or the ability to self-pay.	Services available by appointment.	Monday-Thursday 8AM-8PM; Friday 8AM-5PM	English!DELIM!Spanish	Yes	Bring identification and insurance information to appointment	09 Downtown Circulator	Parking available in the John Street garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	Coming soon	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	2 Spanish speaking therapists and psychiatrist	No	0	\N
505577	102002	2008-07-16	\N	Aspirations	\N	The University of Massachusetts Lowell, Dept. of Psychology	1 Mahoney Hall, 870 Broadway Street	Lowell	MA	01854	978-934-2930	\N	\N	978-934-3074	http://faculty.uml.edu/ahillier/aspirations.htm	public_program_email@testing123.net	Social and vocational skills support group program.	Aspirations is a social and vocational skills support group for high functioning adolescents and young adults on the autism spectrum aged 18-30 years.  Aspirations is designed to provide the opportunity to discuss social and vocational skills in a small group setting. Group members are guided in ways to improve their social and vocational skills. Group leaders familiar with autism spectrum disorders facilitate an atmosphere of acceptance and friendship. Developing self-awareness, social insight, and independence are central themes of Aspirations. Aspirations also provides an opportunity for group members to meet others on the autism spectrum and gain support and understanding from one another.  Participants are encouraged to develop friendships with other group members. \r\nAspirations runs for nine weeks and meets once a week on Tuesdays from 6.15-7.30pm at the University of Massachusetts, Lowell south campus. We run the program twice times a year, once in Fall and once in Spring. \r\nWe also have a facilitated parent support group which runs at the same time and place as the Aspirations program. This gives parents the opportunity to meet, talk and share useful information.	Diagnosis of autism spectrum disorder. 18-30 years old.	Yes	The fee for the 9-week program is $150. No fees for parent support groups. Scholarships are available for those who cannot afford the fees.	Aspirations runs twice a year, once in Fall and once in Spring.	The 9-week program runs on Tuesdays 6.15-7.30pm.	English	Yes	Proof of diagnosis; most recent psychological report / evaluation.		Parking is available on the University of Massachusetts Lowell south campus where Aspirations takes place.	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
504114	101430	2008-07-16	\N	Lowell Special Education Parent Advisory Council	Sped PAC	PO Box 1192	\N	Lowell	MA	01853	978-454-1967	\N	\N	\N	\N	public_program_email@testing123.net	Parent Workshops	Meetings include workshops on various topics of interest to parents of children serviced by special education.  Programs are always free.  Meetings are open to all interested persons.	Meetings are free and open to all.	Free	\N	Meetings are held the first Thursday of the month October through June.  September date varies. No meetings during the Summer.	Meetings are held at the Butler School (1140 Gorham St) from 7:00 P.M. to 9:00 P.M. in the Library.	English	Not Applicable	\N		Parking is just outside the door in a well lit parking lot.	\N	Yes	No	\N	Not Applicable	\N	The PAC will try to accommodate any special needs by a visitor/member. For further info regarding handicap accessibility or other needs call 978-454-1967.	\N	\N	\N	\N	\N	Yes	There are many opportunities within the PAC for a volunteer including but not limited to community outreach, newsletter and other computer tasks and developing/maintaining a Resource Library.	Yes	The PAC has some books and videos related to various disabilities.  However, the computer has become the greatest tool in researching information for parents, educators with questions.	Yes	Meetings are held at the Butler School (1140 Gorham St). For directions call 978-454-1967.	Not Applicable	The PAC does not currently publish a newsletter, however, it does use email and the monthly agenda to send information out to its members.	Limited	We do not currently have our own website. The PAC can be contacted at Lishcrti@aol.com.	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Meetings are geared towards parents of children in the Lowell, MA schools. However, the PAC welcomes educators and other community members to attend and participate.	Meetings are conducted in English. The PAC tries to accomodate other languages as needed with notice ahead of time. Visitors are also welcome to bring their own interpreters.	Not Applicable	0	\N
502101	100649	2008-07-18	\N	Out Patient Mental Health Services	\N	10 Bridge Street	\N	Lowell	MA	01852	978-453-5736	\N	978-453-5736	978-970-5595	www.ArbourHealth.com	public_program_email@testing123.net	Outpatient mental health services	Arbour Counseling Services provides outpatient mental health services including individual, family or group therapy. Psychotropic medication consults are available for therapy patients.	Services available to anyone age 4 and older.	Yes	Fees vary. Medicaid, Medicare and most commercial health insurances accepted	Services available all year.	Monday-Friday 9AM-9PM; Saturday 9AM-3PM	English!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Appointments required. Must bring insurance card to appointment.	09 Downtown Circulator	Parking available in John St. garage	\N	Yes	No	\N	Yes	Average waiting list 2-4 weeks for children / individuals requiring services in certain languages	Languages: \r\nOn the phone: English & Spanish \r\nAt program site: English, Spanish, Khmer, Russian, Greek, French	\N	\N	\N	\N	\N	No	\N	Yes	Resources on mental health with a focus on Southeast Asians available. Appointments needed, contact agency.	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	Greek, Russian, Polish	No	0	\N
501177	100154	2008-07-21	\N	Council on Aging	Senior Center	276 Broadway St	\N	Lowell	MA	01854	978-970-4131	\N	\N	978-970-4134	\N	public_program_email@testing123.net	Senior Center & Variety of Services for Seniors	The Lowell Council on Aging is a senior center that provides services of nutrition, health, recreation, transportation, information and referral programs to the elderly of Lowell. Activities include country line dancing, computer lessons, bingo, exercise classes and more.  We have an expert in Food Stamps & Fuel Assistance and one in Medicare, Social Security and other programs.  We now have Police Intervention so that you can voice any concerns.  We have a part time woker specializing in the Citizenship process who also speaks Khmer.	Lowell residents ages 60 and older	Yes	Lunch Monday-Friday: $2 donation requested. Weekend & holiday lunch $2.00. Activity fees vary, call for information.	Services available year round	Monday-Friday 6:30AM-4PM; Saturday & Sunday 7AM-12PM.	English!DELIM!Portuguese	No	$5 for becoming a member of the Friends to receive a monthly newsletter mailed to your home.	06 Broadway / UMass Lowell!DELIM!09 Downtown Circulator	Parking available.	Lunch pick up and drop off provided.\r\nPlease call before 9:00 am.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers assist in a variety of programs and activities, including bingo, administration, outreach and telephone reassurance.	Yes	The City of Lowell Pollard Memorial Library has an annex located within the Senior Center.  The second floor is equipped with a library, computers, book & video rentals (w/ Local library card).	Yes	Available by appointment for non-profit agencies and City of Lowell departments.  Space limited.	Yes	The "Heritage" newsletter is available monthly.  Newsletters can be picked up at Senior Center or area agencies, local grocery stores, pharmacies, hospitals, churches; or for $5.00 a year, we will mail it to your address.	Yes	The Pollard Memorial Library annex is equipped with computers and Internet accessibility.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501221	100154	2008-07-22	\N	English to Speakers of Other Languages (ESOL)	ESL	408 Merrimack Street	\N	Lowell	MA	01854	978-937-8989	\N	\N	978-458-9007	\N	public_program_email@testing123.net	English classes for non-native speakers	ESOL classes are offered to foreign-speaking adults who want to learn the English language. Students stay in the classes as long as they need to - progressing from one level of ESOL to the next. Classes are offered in the morning from 9AM-10:30AM Monday-Friday and 11AM-12:30PM Monday-Friday. Classes are offered at night either Monday & Wednesday or Tuesday & Thursday from 6-9PM. Night classes are held at Lowell High School, 50 Father Morrissette Blvd. There are several satellite programs throughout Lowell.	Must be 16 or older and live in Lowell or one of the 7 surrounding communities (picture id required)	Free	\N	Program runs September -June, enrollment is ongoing (best to register end Aug.) Students stay as long as they need to.	Monday-Thursday 9AM-9PM; Friday 9AM-3PM	English	No	Registration is daily at 9AM or 6PM. Calling ahead is not necessary. Bring picture id with proof of residency. Placement tests		Parking not available at building, located near city lots	\N	No	No	\N	Yes	\N	At times, student aides or staff members are available to translate.	\N	\N	\N	\N	\N	Yes	Program recommends volunteers contact Pollard Memorial Library's Literacy Volunteers program to volunteer	No	\N	No	\N	No	\N	Yes	Lowelladulteducation.com	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	fabisi@lowell.k12.ma.us
501254	100154	2008-07-22	\N	Family Literacy Classes -Lowell Adult Education Center	\N	408 Merrimack Street	\N	Lowell	MA	01854	978-937-8989	\N	\N	978-458-9007	\N	public_program_email@testing123.net	Literacy classes for adults and there children under 6	Family Literacy Classes are offered to adults with their children who are between the ages of 2 months and 6 years at the downtown Lowell Family Literacy Center. ESOL classes are held at this site. Pre GED, GED, and other ESOL classes are offered at Lowell Adult Education at 408 Merrimack Street in downtown Lowell.	Must be 16 years of age or older and live in Lowell	Free	\N	Program runs from September until June. There is no waitlist in September, but there may be one if you enter at a later date	Classes are Monday-Thursday 9:30AM- 11:30 AM at the Family Literacy Center. Other classes are held in mornings as well as nights.	English	No	Registration is daily at 9AM or 6PM. Calling ahead is not necessary. Bring picture id with proof of residency. Placement tests		Parking not available at building, located near city lots.	\N	Yes	No	\N	Yes	There is no wait-list in September, but there may be one if you enter at a later date	At times, Student Aides or Staff members are available to translate	\N	\N	\N	\N	\N	Yes	Program recommends volunteers contact Pollard Memorial Library's Literacy Volunteers program to volunteer	No	\N	No	\N	No	\N	Yes	lowelladulteducation.com	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
501309	100154	2008-07-22	\N	Pre-GED & GED Preparation: Lowell Adult Education Center	\N	408 Merrimack Street	\N	Lowell	MA	01854	978-937-8989	\N	\N	978-458-9007	\N	public_program_email@testing123.net	Prep classes for GED & GED Testing Center	Classes are offered to adults 16 years and older who are formally withdrawn from the K-12 school system. Classes are offered Monday through Friday from 9AM-12:15PM and at night from 6-9PM Monday & Wednesday or Tuesday & Thursday. We are also an official GED testing center. We test monthly on Friday night and Saturday morning and afternoon. You must sign up and pay beforehand for testing. Students should call for more information: 978-937-8989.	Must be 16 or older and live in Lowell or one of the seven surrounding communities (must have picture id).	Free	No fees for the classes, although the state does charge a fee for the GED test.  Call for more information.	Sept.-May. Open enrollment - enter classes as soon as possible beginning in Sept. Adults study until ready to take GED test	Monday-Thursday 9AM-9PM; Friday 9AM-3PM	English	No	Registration is daily at 9AM or 6PM. Calling ahead is not necessary. Bring picture id with proof of residency. Placement tests		Parking available in nearby city lots	\N	Yes	No	\N	No	\N	At times, student aides or staff members are available to translate.	\N	\N	\N	\N	\N	Yes	Program recommends volunteers contact Pollard Memorial Library's Literacy Volunteers program to volunteer	No	\N	No	\N	No	\N	Yes	lowelladulteducation.com	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	Gunther new contact, called 04/22
504455	101122	2008-07-22	\N	Caleb Resident Service Coordination	\N	77 Chestnut Street	\N	Lowell	MA	01852	978-454-2870	\N	\N	\N	www.thecalebfoundation.org	public_program_email@testing123.net	Caleb Service Coordination	The Caleb Group is an interfaith organization with the mission of freeing the human spirit through fostering caring communities to enhance and empower the lives of those less fortunate.  Working with The Caleb Foundation and others, which own and operate affordable housing throughout New England, The Caleb Group coordinates supportive services for and with residents.\r\n\r\nCaleb works with 3500 residents around New England who are of low and very low income, making on average 60% of median income.  Caleb communities have clearly been affected by welfare reform.  While only a handful of residents still receive welfare benefits and more families are working, many families are still living on temporary and part-time jobs, housing and food subsidies, and with out the benefits of many amenities afforded higher income neighbors.  Some families continue to receive welfare benefits, soon facing more severe limitations.  Service Coordinators are working with these families on the long road to economic self-reliance.\r\n\r\nCaleb service coordinators work with elderly, family and disabled residents to increase their level of independence by helping to identify their needs and connecting them with community resources to fulfill these needs.	Living in a Caleb affiliated community in New England: Mass, Connecticut, Maine and New Hampshire.	Free	\N	The schedule of programs offered varies per housing community.  Generally programs are available during the week, and resident driven programs could be scheduled on weekends as well.	The office is open generally from 9-5 Monday through Friday.	English!DELIM!Spanish	Yes	Calling ahead to meet with Caleb staff is encouraged		Chestnut Square Apartments has parking available.	n/a	Yes	No	\N	No	\N	Our volunteers tutor individuals on basic computer skills. Volunteers also assist with aftershool programs.	\N	\N	\N	\N	\N	Yes	The Caleb Group always welcomes volunteers.  Some of the opportunities include: teaching computer classes, tutoring youth or adult residents, mentoring young people, leading exercise and nutrition classes, or any other program of interest to residents or volunteers.  All potential volunteers must pass a screening before working directly with Caleb residents.	Not Applicable	\N	Yes	Two community rooms are available as meeting spaces.	Yes	Caleb has a semi-annual newsletter that is distributed by mail.	Yes	We have 8 computers in a computer lab, along with software to help Lowell residents through introductory self-paced learning.	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	The Caleb Group works in 21 affordable housing communities around New England.	\N	No	0	\N
501078	100132	2008-07-23	\N	Homeless Housing & Services	\N	94 Rock Street	\N	Lowell	MA	01854	978-459-3387	\N	\N	978-970-1096	http://www.bridgewell.org	public_program_email@testing123.net	Provides transitional housing for homeless men	Pathfinder/Safe Haven provides transitional housing for homeless men, conducts homeless outreach and provides meals. Substance abuse education is provided.	Referrals are required for Residential services. All other individuals served through outreach services must be homeless in order to receive services.	Free	\N	Services available year round, with intake ongoing. Computer available.	Outreach & Meals: Monday-Sunday 8AM-8PM.	English!DELIM!Spanish	No	\N		Street parking is available in front of and on side of building.	Located on Broadway / Willey St. LRTA stop	Yes	No	No children under the age of 18 are allowed on the premises	No	\N	\N	\N	\N	\N	\N	\N	Yes	Call 978-459-3387 for more information on volunteer opportunities	No	\N	No	\N	No	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	Currently we have no bilingual staff	No	0	\N
501683	100429	2008-07-23	\N	Cancer Center	\N	Two Hospital Drive	\N	Lowell	MA	01852	978-934-8425	\N	978-458-1411	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Cancer Center	The Cancer Center provides: Hematology, medical, and radiation oncology, prevention and early detection, comprehensive treatment programs, national cancer research protocols, patient education, counseling and support.\r\nOther special programs include: breast and surgical cancer initiative, prostate cancer screening, skin cancer screenings and positron emission tomography (PET)	Contact Cancer Center	Sliding Scale	No information provided	Opem year round	Monday-Friday 8AM-4:30PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Free parking available.  Valet, outside or in parking garage	\N	Yes	No	\N	No	\N	\N	2 Hospital Drive	\N	Lowell	MA	01852	Yes	Interested volunteers should contact 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	contact has changed to replacement director, Gary Sawicki
505533	101958	2008-07-24	\N	Adolescent Partial Hospitalization Program	\N	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5160	\N	800-222-2237	\N	www.arbourhealth.com	public_program_email@testing123.net	Adolescent Partial Hospitalization Program	Patients are referred directly to the program as an alternative to inpatient hospitalization, or as a transition from more intensive levels of care. The Partial Hospitalization Program (PHP)is part of the continuum of care for youth that Lowell Treatment Center offers.\r\n\r\nThe program is completely voluntary, and, as such, fosters an environment of trust, caring and support with an eye to permanent, long-term change. The program views problems systematically, and involves both the patient and his or her family in the therapeutic process whenever appropriate. The PHP staff are sensitive\r\nand responsive to the individual needs of each patient, and provide a comprehensive spectrum of services.	The Partial Hospitalization Program (PHP) serves adolescents ages 13 through 18.	Yes	\N	Year round	Full day of programming, from 9:00 AM until 3:00 PM.	English	Yes	\N		Free Parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
505544	101958	2008-07-24	\N	Family Stabilization Team	\N	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5160	\N	800-222-2237	\N	www.arbourhealth.com	public_program_email@testing123.net	Family Stabilization Team	The Family Stabilization Team (FST) provides\r\nshort-term, in-home services to assist parents and other caregivers in stabilizing children and adolescents in their home settings after an acute mental illness or\r\nsubstance abuse episode. The program can also be used as a treatment alternative to hospitalization and/or as a step-down program between hospitalization and returning home. The FST is an integral component of the continuum of care that Lowell Treatment Center offers.	The program typically targets the following populations:\r\n Individuals of any age who have been hospitalized more than three times\r\n Individuals with a dual diagnosis of a major mental illness and substance abuse\r\n Pregnant women who are actively abusing substances \r\n Individuals newly diagnosed with an Axis I psychotic disorder	Yes	\N	Year round	Available 24 hours a day, 7 days a week	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County	\N	\N	\N	Not Applicable	0	\N
505346	101958	2008-07-24	\N	Inpatient Treatment for Children and Adolescents	\N	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5160	800-222-2237	800-222-2237	\N	www.arbourhealth.com	public_program_email@testing123.net	Inpatient Treatment for Children and Adolescents	Lowell Treatment Centers inpatient program serves children and adolescents ages 5 through 17. The goal of the program is to provide short-term intensive psychiatric intervention, diagnosis, stabilization and treatment through a multi-disciplinary approach that emphasizes family involvement and collaboration with the clients community network.	The program offers treatment to children and adolescents in need of acute care services.  Typically, these are individuals with serious behavioral problems such as suicidal or violent acts. Underlying these presentations are often undiagnosed and/or undertreated affective illness, learning disabilities, substance abuse or trauma.	Yes	\N	Open year round	The Lowell Treatment Center is operates 24 hours a day, 7 days a week.	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
501474	100297	2008-07-24	\N	Homeless Shelter & Intensive Supportive Services	\N	189 Middlesex Street	\N	Lowell	MA	01852	978-458-9888	\N	978-458-9888	978-452-2186	\N	public_program_email@testing123.net	Multi Service Men & Women's homeless shelter	Lowell Transitional Living Center is a community supported non-profit organization that provides safe, temporary shelter for homeless men and women of Greater Lowell. In a multi-service environment, volunteers and staff provide opportunities for individuals to end their homelessness. Services include: emergency shelter, housing referrals, job preparation, detox transportation, case management, street outreach, community social service linkages, mental health services, post-shelter transitional support, community meals, medical clinic / health care, computer classes, and GED preparation.	18 & up single adults. Referrals required for hospitals, mental health facilities, other shelters and detoxes.	Free	\N	Program services available 24 hours per day, 365 days a year. Program capacity is 90	24 hours/day, 365 days per year	English!DELIM!Spanish	Yes	Proper identification showing current local address or ID showing former local address required. Must be sober.		Parking available on the street	Transportation is provided for detox appointments whenever possible	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are always welcome to help in the kitchen, the Front Desk, some administrative, etc.	No	\N	Yes	AA meetings daily from 2-3 pm	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
504906	101738	2008-07-28	\N	Lowell House Inc. S.O.A.P. Program	S.O.A.P.	555 Merrimack Street	\N	Lowell	MA	01854	978-454-2997	\N	\N	978-937-2559	\N	public_program_email@testing123.net	Substance Abuse Treatment	Full time day program operating Monday through Friday from 9:00am to 4:00 pm.\r\nCase management,Individual counseling, Group therapy, Health care management, Mental health evaluations, housing and employment training guidance. Lunch is provided free of charge.	Those covered under Mass Health (MBHP) and those with no insurance.	Free	Mass Health and those with no insurance	Open year round closed most legal holidays.	Monday through Friday 8:00 am to 4:00 pm\r\nNight program: Monday through Thursday 5:30\r\ntill 9:00 pm	English!DELIM!Spanish	Yes	Mass health card.		yes	none	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
504741	101705	2008-07-28	\N	Living Waters Ministry of Hope	Living Waters	250/254 Middlesex Street	\N	Lowell	MA	01852	978-446-9800	\N	\N	\N	www.livingwaterslowell.org	public_program_email@testing123.net	Outreach/Drop-in Center	Living Waters Ministry of Hope is a non-denominational faith based ministry. Our mission is to offer support to all people in the community, by helping to rebuild lives, fostering healthy relationships, Teaching life skills and providing supportive and affordable housing.\r\n\r\nWe share the heart of God in building relationships.\r\n\r\nLiving Waters storefront offers an alternative place for the people of the community to come and spend time in an atmosphere of love and acceptance. We provide on a daily/weekly basis various meetings; groups and functions that help strengthen individuals in both their  spiritual and emotional growth.\r\n\r\nWe believe that we can make a meaningful contribution in eliminating homelessness among people with addiction, mental illness & disabilities.\r\n\r\nWith God's help we attempt to restore self-worth in all individuals.\r\n\r\nWe provide, weekly bible studies, Prayer and fellowship, Clothing, Referrals, functions, Trips, food packs, Community outreach,	Homeless, and the who so ever	Free	\N	Open year round.	Sundays 7pm- 9:30, Tues 9am-12pm 7pm-9:30pm, Thursday 7pm-9:30pm, Fridays 9am-12pm,7pm-9:30 Saturday 9am-12pm	English!DELIM!Spanish	Not Applicable	\N		\N	Some transportation provided.	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	P.O. Box  2157	\N	Lowell	MA	01851	Yes	We are always looking for volunteers, that would be willing to help in anyway,serving the homeless, doing volunteer community service,There is so many opportunities.	Not Applicable	\N	Yes	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	0	\N
505335	100099	2008-07-30	\N	After School Program	Boys & Girls Club of Greater Lowell	657 Middlesex Street	\N	Lowell	MA	01851	978-458-4526	\N	\N	978-453-9740	www.lbgc.org	public_program_email@testing123.net	After School Program	The Boys & Girls Club of Greater Lowell offers a variety of after school programs for youth between the ages of 7 to 18.  Program offerings are in the areas of Sports Fitness and Recreation, The Arts, Health and Life Skills, Character and Leadership Development and Education and Career Development.\r\n\r\nPrograms are offered in six week increments.  For in-depth information about each program, click on the additional program links	Ages 7 to 18	Yes	There is a $26.00 per school year membership to join the Club.	During the school year, the Club is open from 2:00 pm to 6:00 pm  On School Vacations and Holidays, the Club is opened 10:00 am to 5:00 pm	Monday through Friday 2-6pm\r\nMonday & Wednesday 2-8pm for ages 13-18	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	All new members must attend new member orientation which is held every Monday evening at 6:30 pm sharp.		There is parking in our Middlesex Street lot adjacent to the Highlands Precinct.	After School transportation is provide by the Lowell School Department Transportation Center.  However, there is limited transportation.  Visit the Parent Information Center Transportation Department on Kirk Street Downtown Lowell for more information.	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer positions are available in arts & crafts, tutoring, coaching, games room, and administration.  To volunteer, please contact Von Mom at 978-458-4526	Yes	Newly developed Learning Center complete with books for youth and teens, computers for research, homework help and reading and writing projects.	Yes	Teen Room, computer lab, matted room, multipurpose room, Fitness facility, gym,pool, Learning Center, Game room, Arts & Crafts room, Snack Room. To rent the facility, please contact Michelle Phillips at 978-458-4526	Yes	The Boys & Girls Club of Greater Lowell publishes a monthly e-news letter which contains upcoming events and Club news.  If you would like to join the newsletter, email the Club at mphillips@lbgc.org	No	Internet access is for Club members only.	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
502992	100946	2008-07-30	\N	Clarendon Family Day Care, Inc.	\N	C/O Holden Center 220 Pawtucket Street, 2nd floor	\N	Lowell	MA	01854	978-454-3026	800-875-1234	\N	978-452-2073	www.clarendonfamilydaycare.org	public_program_email@testing123.net	Day Care & professional development for providers	Clarendon's mission is to provide quality, culturally sensitive and affordable child care services to local families; promote small business and professional development opportunities for men and women in the Massachusetts family child care industry; provide professional child care services and additional support services to assist the efforts of parents working to improve their family's quality of life and to ensure a safe, nurturing environment to enhance the growth and development of each child to whom we provide services.	Community Partnership, Income eligible, DTA voucher or Private pay	Yes	Fees vary, contact program for more information	Year round, including non-traditional hours: night care, weekend care	Hours vary - contact program	English!DELIM!Spanish!DELIM!Khmer	No	Contact program		Free parking	Transportation is provided for children. If parents need to meet with providers & have no access, transportation is provided.	No	Yes	\N	Yes	Waitlist for subsidized contract	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Books, Toys, Puzzles, Many learning games and coloring activities installed on our childrens computer, Older children can access the Internet for doing research	No	\N	Yes	Quarterly	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	Clarendon has 5 offices across the state of Massachusetts	\N	No	0	\N
501628	100407	2008-07-30	\N	University Gallery at UMass Lowell	University Gallery at UMass Lowell	McGauvran Student Union	71 Wilder Street, First Floor	Lowell	MA	01854	978-934-3491	\N	\N	\N	http://www.uml.edu/dept/art/	public_program_email@testing123.net	Art Gallery featuring exhibits by professional artists	The University Gallery at UMass Lowell is a Non-Profit Art Gallery coordinated by the Art Department and sponsored by the College of Arts and Sciences. The Art Department offers a BFA degree in Art and Design and is an accredited member of the National Association of Schools of Art and Design (NASAD). Six exhibits by established and emerging professional artists are organized within each school year (September  May). The exhibits are an important educational tool for the students as part of their undergraduate program. All gallery exhibits and events are free and open to the general public.	Free; Open to all; Handicapped Accessible	Free	\N	Gallery is open during the school year - September through May	Monday - Thursday 11 - 4 and by appointment throughout the school year. Call 978.934.3491 for current gallery hours or appointment.	English	No	\N		Free Parking is available in the Riverview Lot off Broadway; limited parking in the Visitor Lot across from the McGauvran Student Union on Wilder Street.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	Yes	Sign up for the mail list for event announcements by contacting: Michele_Gagnon @uml.edu	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
506644	100088	2008-08-01	\N	Youth Counseling Services	\N	34 Hurd St	\N	Lowell	MA	01852	978-441-9949	\N	978-265-7173	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	Youth Counseling Services	Free individual counseling to teens of UTEC, crisis intervention with\r\nteens, some family sessions, referrals as needed.  Also available,\r\nupcoming open discussion group for teens.	open	Free	\N	Please contact Elena Ansara for schedule and appointments.	Monday-Thursday, 2:30 PM - 6 PM (some flexibility)	English	Yes	\N		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Teen volunteers needed to help plan open discussion group.	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a large room with conference table.	Yes	please sign up at www.utec-lowell.org	Limited	30-60 minute sessions for teens only	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
501529	100374	2008-08-01	\N	Rape Crisis Services	\N	144 Merrimack Street, Suite 304	\N	Lowell	MA	01852	978-452-7721	800-542-5212	800-542-5212	978-458-2822	www.rcsgl.org	public_program_email@testing123.net	Free Counseling & Support Groups for survivors & families	Rape Crisis Services of Greater Lowell provides a 24 hour hotline, free individual counseling and support groups for survivors and their families. Medical and legal advocacy. Prevention education and community trainings.	Men and women ages 12 and up.	Free	\N	Services available year round.	Monday-Friday 9AM-5PM; After hours availability through 24 hour hotline	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Contact program for services.		Parking is available on the street (meters)or in Municipal Garages.	Program may pay cab fare when applicable.	Yes	No	\N	No	\N	(978) 452-8723 TDD Number	\N	\N	\N	\N	\N	Yes	Volunteers attend 8 week training class to become certified rape crisis counselors. Contact Trish Glover.\r\nThere are also opportunities to participate in the Child Asault Prevention Program. Contact Meredith Trueblood.	Yes	Available by pre-arrangement only	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		978-452-8723	\N	\N	No	0	\N
504719	100539	2008-08-01	\N	Domestic Violence	\N	11 Mill Street	\N	Lowell	MA	01852	978-970-1250	\N	\N	978-970-0843	www.maps-inc.org	public_program_email@testing123.net	Domestic Violence	Domestic Violence program develops Awareness and Education in the Portuguese-speaking community in Lowell and provides assistance and support to victims/survivors of domestic violence.	program available to all Portuguese-speakers	Free	\N	open every working day	9:00- 5:00	English!DELIM!Portuguese	No	\N		parking is available in front of building	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Some information available in Portuguese	Yes	Yes, there is a conference room. Please call for more details.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Lowell area	\N	No	0	\N
501661	100440	2008-08-01	\N	VNA Home Health Care	VNA of Greater Lowell	336 Central Street	\N	Lowell	MA	01852	978-459-9343	800-349-8585	978-459-9343	978-441-0007	www.vnalowell.org	public_program_email@testing123.net	Health care in your home	The Visiting Nurse Association of Greater Lowell provides professional and personalized in-home health care, including specialty nursing, rehabilitation therapies, home health aide assistance, medical social services.  Nursing specialties include: diabetes education, wound care, infusion therapy, asthma management, palliative care and maternal-child health.\r\nInterpreter services, for people with limited English proficiency, are provided at no cost to the patient.	Services available to all residents of Greater Lowell who have a medical need for home health care.  Your physician must order the home health care.	Yes	Medicare, Medicaid, HMOs, private insurance accepted and billed directly; sliding scale available for patients without resources.	Length and frequency of services depends on the medical needs of the patient and the physician's orders.  We provide professional care and teaching for the acute need.	Access to a VNA professional available 24 hours a day 7 days a week; most home visits made between 8AM-8PM.  Office open Monday-Friday 8AM-5PM; Saturday-Sunday 8AM-3PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	Call or fax the VNA with information on: diagnosis and health care needs, name, address and phone of patient, physician's name and phone, health incusrance info		Not applicable: services are provided in the patient's home	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers assist with a variety of tasks, including friendly visiting to Hospice families, office work, professional expertise. Contact Cathie Milliken, Coordinator of Volunteers, 978-459-9343 Ext. 2725	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	also Pelham, NH	Language Interpretation (through interpreters or telephone service) available to all patients with limited English proficiency, at no cost to the patient.	No	0	\N
501639	100440	2008-08-01	\N	VNA Hospice of Greater Lowell	VNA of Greater Lowell	336 Central Street	\N	Lowell	MA	01852	978-458-3123	800-349-8585	978-459-9343	978-458-3116	www.vnalowell.org	public_program_email@testing123.net	Medical care at home for patients with advanced illness	VNA Hospice is a comprehensive care program for patients and their caregivers, at the patient's place of residence, which emphasizes quality of life, pain and symptom management during a life-limiting illness and the time of bereavement. The VNA Hospice team provides medical, emotional, social, spiritual and practical support for the patient and for all those involved in caring for the patient.	Open to people of all ages in Greater Lowell who have a life-limiting illness, when medical treatment is refocused from aggressive measures to care and comfort. Services are provided in the patient's place of residence. A Physician order is necessary to provide services.	Yes	Care is covered by Medicare, Medicaid and most private insurance and is billed directly.  Sliding scale fees available for those with limited health care resources.	Program services are available year round, 24 hours a day.	Clinical staff is available 24 hours per day, 7 days a week.  Home visits are generally made between 8AM and 8PM.	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	To refer a patient, call or fax us with patient's name, address, phone and Physician's name and phone. We also welcome calls from those who just want more info about Hospice.		Not applicable: services are provided in the patient's place of residence (home, assisted living, nursing home).	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are needed for visiting with patients/families & general office tasks. Training provided. Contact Cathy Milliken 978-459-9343, Ext. 2725	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	also Pelham, NH	Language interpretation (by interpreter or telephone service) is available to all patients with limited English proficiency, at no cost to the patient.	No	0	\N
504389	100440	2008-08-01	\N	VNA Senior Health Clinics	VNA of Greater Lowell	336 Central Street	\N	Lowell	MA	01852	978-459-9343	800-349-8585	\N	978-441-0007	www.vnalowell.org	public_program_email@testing123.net	Drop-in clinic at senior centers and senior housing sites	Brief one-on-one meeting with a VNA nurse for blood presssure check, weight check, medication review, and health teaching.	Anyone over 60 years of age in Greater Lowell. No appointments are needed and no fees are charged.	Free	Services provided at no cost.  Donations accepted.	Clinics are held at 18 sites in Greater Lowell every month.  Call us for specific info.	Call for information about days/hours for each site.	English!DELIM!Portuguese	No	No documentation required.  Bring medication bottles if you have question on your medications.		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	Portuguese interpreter provided for the clinic at the Portuguese-American senior center.	No	0	\N
503597	101166	2008-08-01	\N	The Single Life	TSL	P O Box 8222	\N	Lowell	MA	01853	978-441-2765	\N	\N	\N	thesinglelife.org	public_program_email@testing123.net	for mature singles age 50 plus . . . Contact us today!	Chapters usually schedule social activities such as house parties, weekend trips, seasonal activities, dances and cultural events.  All activities are announced on our web site www.thesinglelife.org. Click on "Activities" then click "Upcoming Events" for just some of the happenings that month.\r\nYou may join at an event, activity or at a chapter meeting --- talk to the Chapter Leader.	Must be 21 or older\r\nCurrently majority of membership is 50 plus.	Yes	Membership (Yearly) $25.00\r\nSome events may be free, pay your own way, bring a dish to share or activity may be formulated at a "break even" cost in addition; some events are financially underwritten by the chapter.\r\nCurrently, meeting fees are between $2.00 to 5.00 for members and a dollar or two more for some guests depending on chapter location. First visit FREE. Some chapters may charge 5.00 depending on the activity scheduled such as a "Guest Speaker". Failure to adhere to notice guidelines for Potlucks or special meetings may result in an additional 5.00 surcharge.	Open year round -- T S L meetings are held weekly on Monday through Thursday in several different communities in a relaxed atmosphere of a members home/Condo and/or at a rented hall. Arrival time is usually between 6:30 and 7:30 p.m. with refreshments and socialization followed by discussions on topics of interest or may include a pot luck, pizza night, or games or cards . . . etc which continues until 9 or 10 p.m. Changes in meeting format may vary and are announced under "Meeting Locations" in the WEBletter. If you would like to receive a newsletter please request it in an e-mail to info@thesinglelife.org	Office:A Prerecorded message advises callers of the different types of activities held by various chapters as well as any Socials-In-The-Works. Calls returned promply. Visitations by appointment only.	English	Yes	MA or N H driver's license or State ID\r\nPhone, Electric, Gas, or Oil Bill		Varies; upon location	Attendees choice	No	No	None.\r\n\r\nSome activities are Handicapped accessible	No	\N	\N	\N	\N	\N	\N	\N	Yes	Because T S L is a non-profit organization, most of the planning and administration is accomplished by members who enjoy T S L and want to contribute their time and talent to its continued success. You too can be involved...Contact us today!	No	\N	No	No space at office for meetings. The office processes inquiries, and produces a monthly WEB letter.	Yes	The monthly WEBletter details over two dozen social, cultural and special events that are sponsored by T S L Chapters each month. The WEB letter and the web site also keeps you up to date on all T S L activities. Some chapters print and mail their own monthly newsletter via subscription for members at an additional cost beyond the annual dues.	Yes	http://www.thesinglelife.org\r\n\r\nSite hosted by Lowell Telecommunications Corporation in collaboration with the Community Software Lab.	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	Acton, Billerica, Beverly, Cape Ann, Concord, Chelmsford, Danvers, Everett, Greater Haverhill, Ipswich, Lexington, Lowell, Lynn, Peabody, Malden, Medford, Merrimack Valley, Melrose, Reading, Rockport, Tewksbury, Wilmington, Woburn and Winchester. \r\n \r\nWe also serve Southern New Hampshire including Nashua, Pelham, Salem, Plastow and Windham areas.	\N	No	0	\N
502189	100715	2008-08-02	\N	Program Playback Coordinator	LTC Channel 8 & 95	246 Market Street	\N	Lowell	MA	01852	978-458-5400	\N	\N	978-937-0361	http://www.ltc.org	public_program_email@testing123.net	Dedicated to the free flow of Ideas	LTC is involved in a number of content and information production and distribution activities designed to serve Lowell's vibrant and diverse urban population. Currently LTC offers cablecast and web resources for individuals and groups looking to learn, create and share their message with others.\r\n\r\nRecognizing that certain constituencies need extra support, LTC provides additional services to youth and NPO oraganizations. Through special educational and production projects, LTC staff seek to expand opportunities for these target groups.	Residents, workers & students in Lowell, MA	Yes	Annual Membership fees.\r\nIndividual ($30 cable subscriber / $45 non-subscriber)\r\nFamily ($60 cable subscriber / $75 non-subscriber)\r\nNon-Profit Organizations ($125 for 6 members)\r\nBusiness ($250 for 6 employees)	Open year round.  Quarterly new member orientations offered in January, April, July, and October.	Monday - Thursday\r\n10am to 9:30pm\r\nFriday \r\n10am to 6pm\r\nSaturday\r\n10am to 4pm\r\n\r\nClosed Saturdays preceding a Monday Holiday	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Must attend a new member orientation.  Call to register.  Proof of residency and cable TV bill.		Free parking available in the Lowell National Historic Park Visitor's Center	\N	Yes	No	\N	No	\N	\N	PO Box 803	\N	Lowell	MA	01853	Yes	variety of volunteer and intern positions available at:\r\nhttp://www.ltc.org/about/jobs.shtml	No	\N	Yes	10-12 person conference room\r\n50-75 person studio space	Yes	Quarterly inFocus print and online (http://www.ltc.org/news/news1.shtml)	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
503762	101243	2008-08-04	\N	Mobile  Pantry	Merrimack Valley Food Bank, Mobile Pantry	735 Broadway Street, 2nd Floor	\N	Lowell	MA	01854	978-454-7174	\N	\N	978-454-1717	www.mvfb.org	public_program_email@testing123.net	Delivers groceries to home-bound elderly & disabled	The program is part of the Merrimack Valley Food Bank. Clients receive nutritious non-perishable foods, bakery items and produce that is appropriate to their dietary requirements, delivered by volunteers, using their personal vehicles.	Referral required (VNA, doctor's office, community agencies, Elder Services, social workers, hospital).	Free	\N	Clients receive food once per month as long as needed.	Mon - Fri 8:30-4:30	English	Yes	Upon referral from an agency, clients must meet economic and health restriction eligibiltiy.  All clients must be homebound and proof is requried from a Doctor's office.		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Opportunities include sorting donated foods, preparing bags according to client dietary needs and delivering bags to client homes.  CORI check is mandatory for all volunteers who deliver to client homes.	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
505027	100308	2008-08-05	\N	F.O.C.U.S.	Finding Options for Change Understanding & Safety	99 Church Street	\N	Lowell	MA	01852	978-458-6282	\N	\N	978-441-9826	\N	public_program_email@testing123.net	Free services for victims of violent crime and drunk driving	F.O.C.U.S. is a program of the Mental Health Association of Greater Lowell, Inc., which provides FREE mental health counseling and case management services to victims of violent crimes and drunk/drugged driving crashes. Services include: individual/group counseling, crisis intervention, safety planning, case management/advocacy and referrals.\r\n\r\nThis project is supported by the Massachusetts Office for Victim Assistance through the Victims of Crime Act of 1984 (VOCA) Grant from the Office of Victims of Crime, Office of Justice Programs, U.S. Department of Justice and a grant from the Drunk Driving Trust Fund (DDTF).	ANYONE WHO HAS BEEN AFFECTED BY A VIOLENT CRIME OR DRUNK DRIVING CRASH. \r\nExamples of eligible persons and crimes include, but are not limited to:\r\n-A witness to violence at home\r\n-An immigrant/refugee who experienced political trauma or was tortured in their home country\r\n-A person who has been physically or sexually abused \r\n-Someone who has lost a loved one to murder\r\n-Anyone who has been emotionally or physically impacted by a drunk driving crash\r\n\r\nPLEASE CONTACT US IF YOU HAVE ANY QUESTIONS ABOUT ELIGIBILITY.	Free	\N	Open year round	Monday-Friday 9AM-5PM\r\nEvening appointments available upon request.	English!DELIM!Spanish!DELIM!Khmer	Yes	It is beneficial to call and schedule a time to meet with staff.		\N	\N	Yes	Not Applicable	\N	No	Please contact F.O.C.U.S. for more information.	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	If we do not serve your area, please contact us for helpful resources.	Other languages may be arranged as needed.	No	0	\N
503817	100154	2008-08-06	\N	Solid Waste & Recycling Office	\N	Health Dept	341 Pine St	Lowell	MA	01851	978-446-7277	\N	\N	\N	http://www.lowellrecycle.org/	public_program_email@testing123.net	Solid Waste & Recycling services within Lowell	Weekly curbside collection of trash, recycling and yard waste materials for Lowell residents.  Two annual Household Hazardous Waste Days (Spring & Fall).	Lowell residential buildings with six or fewer units.	Yes	$125/yr covers all curbside services including trash, recycling and yard waste. Fees for 'bulky items', white goods, and CRTs (begins Jan. '09).	Operated year round	Monday - Friday, 8AM - 5PM	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Please call or email.	Not Applicable	\N	Not Applicable	\N	Yes	Join at: www.egovlink.com/lowell select 'subscriptions'	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	Not Applicable	0	\N
501793	100308	2008-08-07	\N	Mental Health Residential Services	Mental Health Association of Greater Lowell	99 Church Street	\N	Lowell	MA	01852	978-250-6725	\N	978-458-6282	978-250-6726	\N	public_program_email@testing123.net	Provides residential services to adults with mental illness	Residential services are provided to adults with major mental illness in a variety of living situations. All services are flexible and responsive to the needs of the individual clients. Services are provided in highly structured community residences as well as independent living situations. Staff provide skills training in: Activities of Daily Living, Community Integration, Leisure Activities, Legal Rights / Responsibilities, and Crisis intervention.\r\nClients can access support from residential staff twenty-four hours per day.	Greater Lowell residents age 18 and over referred by the Department of Mental Health.	Yes	Rent and living expenses. Contact program for more information.	Services available year round. Program capacity is 32.	24 hours/day, 7days/ week	English!DELIM!Spanish!DELIM!Khmer	Yes	Must be referred by the Department of Mental Health.		Parking available at residence.	Residents utilize public transportation. Van transport is also provided for residents.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers assist with spring and fall yard maintenance. Contact program for more details.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
505401	100055	2008-12-02	\N	Now What...Life After Loss	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6129	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support group for people who have experienced a significant	Meets the second Wednesday of the month at 6:30pm. A support group for people who have experienced a significant loss. Group discussions include taking care of yourself, managing your grief, and moving on with life. Share your concerns, hopes, and fears. To register please call 978-937-6129.	Available for people who have experienced a significant loss	Free	\N	Offered year round	Second Wednesday of the month at 6:30 pm	English	Yes	To register please call (978) 937-6129	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the Cancer Center	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509163	103410	2008-08-08	\N	Transformations	\N	144 Merrimack Street, Suite 435	\N	Lowell	MA	01852	781-333-4696	\N	\N	\N	www.transformationshealing.org	public_program_email@testing123.net	individual, couples and group psychotherapy/ consult	We offer creative, empowering therapeutic services for people of all ages. Utlizing creative arts and mind-body therapy techniques, our specialities include:\r\ntrauma, rape and sexual abuse, domestic violence, self-injury, self-esteem, stress, eating disorders, depression, anxiety, GLBT community, parents of children with special needs, addictions, life transitions, relationship issues and disabilities. \r\n\r\nTransformations also offers consultation services and educational presentations on several different topics including:\r\nrape\r\nincest and child sexual abuse\r\ndomestic violence\r\nptsd and trauma\r\nmind-body healing and the arts	open to all	Sliding Scale	\N	varies	flexible	English	Yes	call for informations	09 Downtown Circulator	downtown lowell, meters or parking garage on market or john streets	no transportation provided	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
504070	100253	2008-08-18	\N	Massachusetts Equipment Distribution Program	Mass EDP	169 Merrimack Street	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_program_email@testing123.net	Provides specialized telephone equpiment for the disabled.	Provides specialized telephone equipment for MA residents.  Applications are available by calling the LAB office.  \r\n\r\nDisabilities include blind/visually impaired, deaf/hearing impaired, motion and cognitive impairments.	must apply and be approved for equipment.	Sliding Scale	most equipment distributed free of charge	Open year round	Monday through Friday from 8AM to 4 PM	English	Yes	Appointment required to pick up equipment		three handicapped parking spaces available on Kirk St., on street metered parking, or in the parking garage on Market St.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Not Applicable	0	\N
500088	100022	2008-08-15	\N	PrimeTime	\N	97 Central St. Unit 302	\N	Lowell	MA	01852	978-454-5405	\N	\N	978-454-5043	www.ywcaoflowell.org	public_program_email@testing123.net	Activities-based counseling ages 7 - 12 (DSS)	PrimeTime is an activities-based counseling group for children ages 7-12 whose families are involved with the Department of Social Services (DSS).	Referrals required through child's Department of Social Services worker. Ages 7-12, Lowell residents	Free	\N	Program runs throughout the year. Each session meets twice a week for 6 months. Program capacity: 15 children / session.	Tuesdays & Thursdays 4:00-6:00 p.m.	English	Not Applicable	DSS makes the referral		\N	Children are picked up from and dropped off at their homes	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
501287	100176	2008-08-15	\N	Merrimack Valley Legal Services, Inc.	Merrimack Valley - North Shore Legal Services	35 John Street, Suite 302	\N	Lowell	MA	01852	978-458-1465	800-336-2262	978-458-1465	978-458-3481	www.mvlegal.org	public_program_email@testing123.net	Provides free legal advice and representation	Merrimack Valley Legal Services is a private non-profit program providing free legal advice and representation to low-income and elderly persons. Representing those with a denial of public benefits (TAFDC, EAEDC, Food Stamps, Veteran's Services, MassHealth, SSI, EA and Unemployment Compensation), disabled children and adults who are denied Social Security, domestic violence cases, divorce cases involving domestic violence, housing and landlord / tenant cases. Service area is throughout the Merrimack Valley including Northern Middlesex and Essex counties.	Open to low-income families in the Merrimack Valley	Free	\N	Program is available year round.	Monday-Friday 9AM-5PM	English!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Call to register by telephone		Parking available in city lots such as John St. and at meters. Located on downtown LRTA bus route.	\N	Yes	No	\N	No	\N	MVLS may close applications for services to different areas of law.  Applicant must call to find out whether services are available.	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Carol Coyne	Yes	Contact agency for more information about the resource library	Yes	Meeting space is available if it concerns an MVLS client.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	978-452-4740	applicants should call to see if they live within service area because it is determined by town	\N	No	0	\N
502937	100891	2008-08-15	\N	Adoptions in Russia and China, and referrals	Florence Crittenton League	119 Hall Street	\N	Lowell	MA	01854	978-452-9671	\N	\N	978-970-0070	www.fcleague.org	public_program_email@testing123.net	Adoption Programs and Home Studies	The Florence Crittenton League Adoption Agency has international adoption programs in China and Russia. We also have out of state sources for  other programs.Home studies are provided for Massachusetts residents working with out of state agencies for adoption placement. We just gort reaccredited in Russia.	Services are open to everyone	Yes	Fees vary. Contact agency for more information	Services available year round	Monday-Friday 9AM-4PM	English	Yes	Contact program for more information. Appointments are required.		Parking available on street	\N	No	No	\N	No	\N	we also make home visits if family canit climb stairs to our 2nd floor office.	\N	\N	\N	\N	\N	No	Volunteers are not used in this agency, with the exception of the Board of Directors	Yes	\N	No	\N	Yes	Quarterly	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
500055	100022	2008-08-15	\N	City Camp	YWCA City Camp	41 Rock Street	\N	Lowell	MA	01854	978-454-5405	\N	\N	978-454-5043	www.ywcaoflowell.org	public_program_email@testing123.net	Summer camp for Children 7 - 13	Camp activities include: swimming at a city-operated swimming pool (supervised by certified lifeguards), arts and crafts, nature studies, sports and games, special events and local and regional field trips. Free breakfast, lunch and snacks for all campers is provided. The camp has a multicultural staff, with a staff to camper ratio of 1:5, and is licensed by the Massachusetts Department of Public Health and the Department of Social Services.	Lowell Children Ages 7 - 13, although Non-Lowell children who have transportation or are able to access stops on YWCA's bus route are welcome.	Yes	Each two week session is $300. Registration fee is $30. Limited number of camperships available on sliding scale	July 7 - August 15, 2008 (Three 2-week sessions). Registration begins in spring, continues till program filled, usually 1st week of August.	Monday-Friday 9AM-3:30PM	English!DELIM!Spanish!DELIM!Khmer	Yes	When you register, you will have to complete the registration packet including emergency contacts, immunization records, health status, financial aid forms, etc.		Very limited short-term on-street parking.	Bus transportation provided to and from 8 locations throughout Lowell. Call for more details.	Yes	No	\N	No	\N	English is the only language spoken on the phone. See above to see the languages spoken at the camp site.	YWCA of Lowell	97 Central Street Unit #302	Lowell	MA	01852	Yes	Volunteers are used occasionally to assist with special projects. Contact Andy Chandonnet at 978-454-5405 ext. 117	No	\N	Not Applicable	\N	Yes	newsletter created by campers	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
500132	100022	2008-08-15	\N	Youth Services	Acre Youth Center	41 Rock Street	\N	Lowell	MA	01854	978-454-5405	\N	\N	978-454-5043	www.ywcaoflowell.org	public_program_email@testing123.net	Youth programs children 9-21	The Youth Services program provides after school, evening, weekend and summer educational, recreational, community service, cultural enrichment and youth development programs at our youth center. Girls and boys, ages 9 to 21; teen employment readiness training; environmental peer leadership; summer camp; computer and digital technology training; mentoring opportunities; and community service projects.	Open to all Lowell residents age 9-21. No referrals are required	Free	no fees except for summer camp which is $30 registration fee and $300 for a 2-week session. "Campership" scholarships and sliding scale are available	Year round	School year: 2:00PM-7:00PM; Summer: 8:30AM-3:30PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Call with mailing address to receive membership forms; Take or mail completed membership forms to YWCA		Limited on street parking at 41 Rock Street; parking lot at 206 Rogers Street	some transportation available for specific programs; call for details	Yes	No	\N	No	\N	English is the only language spoken on the phone, although see above to see the languages spoken on site.	YWCA of Lowell	97 Central Street Unit #302	Lowell	MA	01852	Yes	YWCA uses volunteers for facilitating program activities and program support. Contact Andy Chandonnet	No	\N	Yes	Small conference room (up to 15 people), community technology lab - contact front office	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
504708	101573	2008-08-15	\N	Community Organizing	\N	517 Moody Street	3rd floor	Lowell	MA	01854	978-452-7523 ext. 809	\N	\N	978-452-4923	http://www.coalitionforabetteracre.org	public_program_email@testing123.net	Community Organizing, Leadership Development and Membership	The Coalition for a Better Acre is a membership based community development corporation dedicated to resident empowerment and sustainable community revitalization for current and future residents of Lowell. We promote healthy, vibrant neighborhoods by developing resident leaders, affordable housing and economic opportunities, and by responding to community needs through collective action. \r\nVision for CBA \r\nWe strive for Lowells neighborhoods to be affordable, vibrant and diverse, places of choice where residents are engaged leaders, and where local stakeholders work in partnership to build family and community assets.\r\nOur Values and Beliefs \r\n\tWe believe in respect and dignity for everyone.\r\n\tWe value openness, honesty, and genuine trust between people.\r\n\tWe value listening to others and being responsive to the communitys needs.\r\n\tWe believe in the importance of leadership development and building resident engagement across a diverse spectrum of people.\r\n\tWe value collaboration, partnership, and teamwork within our organization and between our organization and others.\r\nCBA programs address the most pressing community needs and create avenues to promote true grassroots community development. \r\nOur activities include \r\n\tAffordable housing development \r\n\tCommunity improvement projects \r\n\tHomeownership promotion \r\n\tForeclosure prevention \r\n\tLeadership development \r\n\tYouth activities \r\n\tFamily asset building \r\n\tAdvocacy \r\nOur members are very important to us... \r\nAs a member you benefit from access to: \r\n\tValuable information about upcoming events, affordable housing, and other opportunities for you, your family and community \r\n\tParticipate in social and community events \r\n\tShape the direction of CBA to meet our most pressing community needs \r\n\tCommunity leadership development \r\n\tTraining to effectively communicate with elected officials to ensure that our voice is heard \r\n\tSatisfaction of being part of a group to help improve Lowells communities	Any interested person	Yes	We have a $5 suggester individual membership fee.	Year-round; activities and initiatives vary.  \r\nAnnual events include the Acre Clean-Up and Community BBQ in June, a Lake Trip in August and an Annual Membership Meeting in October.	9-5 M-F; evening and weekend activities also	English!DELIM!Spanish!DELIM!Khmer	Not Applicable	\N	07 Pawtucketville / UMass Lowell North!DELIM!09 Downtown Circulator	On-street parking is available.	\N	Yes	Yes	Child care provided at most meetings and events. Call ahead.	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	Meeting room that can accommodate up to 15 people	Yes	CBA issues a quarterly Community News Flash.	Limited	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	Our organizing effort focus in the city of Lowell and we are members of the MACDC (State-Wide association)	We invite speakers of other languages to join us and help in this effort.	Yes	0	\N
505258	101573	2008-08-15	\N	Homeownership Promotion and Preservation	Foreclosure Prevention	450 Merrimack Street	\N	Lowell	MA	01852	978-970-0600	\N	\N	\N	http://www.coalitionforabetteracre.org	public_program_email@testing123.net	Foreclosure Prevention	We at CBA would like to extend an open invitation to you and all the people you know who are interested in learning more about foreclosure prevention options.\r\n\r\nWe do not have to look very far to see that foreclosure rates are soaring; it is an issue that affects our local and world economies and has received much media attention. Foreclosure can happen to anyone, regardless of socioeconomic status, demographic or type of mortgage. Millions of people across the United States have trouble with their mortgages every year. \r\n\r\nIf you feel like you may be in danger of facing foreclosure, the time to get help is now. The sooner you call, the sooner you can regain your piece of mind.	Anyone concerned about foreclosure.	Free	\N	Year Round	M - F 9:00 - 5:00	English!DELIM!Spanish!DELIM!Khmer	Yes	Call Rasy at 978/970-0600, ext 1 to make an appointment.	06 Broadway / UMass Lowell!DELIM!07 Pawtucketville / UMass Lowell North!DELIM!09 Downtown Circulator	On-street parking is available.	\N	Yes	No	\N	Not Applicable	\N	\N	517 Moody Street	Floor 3	Lowell	MA	01854	Yes	Call Rasy for more information.	Not Applicable	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
503355	101056	2008-08-15	\N	Comprehensive Refugee Employment Services	CRES and Matching Grant and ASSIST	144 Merrimack St., Suite 202	\N	Lowell	MA	01852	978-459-9031 ext. 14	\N	\N	978-459-0154	www.iilowell.org	public_program_email@testing123.net	Employment assistance and literacy for refugees/asylees	CRES offers employment counseling and job search for refugees and individuals who have received asylum letters from the US government within the past five years.   Basic English literacy is offered to clients who need it, as is employment-oriented English (filling in applications, interview skills, etc.) Services are provided on-site, by appointment, one-to-one or in small groups.	All clients must be receiving refugee case management services as well, and a referral will be requested from the case manager. We can offer case management to individuals who are not already linked to a refugee agency case manager.	Free	\N	Year round	Monday through Thursday 9 to 5	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Letter of asylum if applicable,  name of case manageer if client already has one.		Garages at Market St. or John St.	Limited transportation may be available by prior arrangement.	Yes	No	\N	No	No wait list at this time	\N	\N	\N	\N	\N	\N	Yes	Experienced ESOL teachers/literacy volunteers; occasionally workshop presenters on employment-related issues.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	Potential clients must be Mass residents, those who live outside Greater Lowell can still qualify, and should call for more information. Routine client transportation is not provided, however.	\N	No	0	\N
506710	101056	2008-08-15	\N	Initiative for the Elimination of Domestic Violence	Domestic Violence Outreach and Advocacy Program	144 Merrimack St., Suite 202	\N	Lowell	MA	01852	978-459-9031	\N	\N	978-459-0154	www.iiboston.org/iiLowell.htm	public_program_email@testing123.net	Domestic Violence Outreach and Advocacy Program	The IEDV provides outreach and advocacy to Portuguese-speaking women who are victims of domestic violence.  The program can also serve Spanish and French speakers, and will help both men and women who have experienced domestic violence.	Open to all	Free	\N	Open Year Round	Monday-Friday 9-5	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish	No	Best to call ahead to make sure advocate is in the office	09 Downtown Circulator	On street or at Market St. or St. John St. Garage	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
503344	101056	2008-08-15	\N	Legal Services	\N	144 Merrimack St., Suite 202	\N	Lowell	MA	01852	978-459-9031	\N	\N	978-459-0154	www.iilowell.org	public_program_email@testing123.net	Immigration-related Legal Services	Legal services are available to refugees, asylum seekers and immigrants, especially to low and moderate income clients. Services include green cards, citizenship, petitions for relatives, employment authorizations, and removal of restrictions.  Fees are reasonable, and all information is confidential. We have a notary on staff, and can take required photos on site.	Immigrants, or US born individuals seeking assistance with immigration issues. No student or employment visas.	Yes	Our fees are approximately 5 to 15% of those charged by private attorneys. We serve primarily low income and moderate income clients.	Year round, Full Time	Please call for an appointment\r\nMonday thru Friday, 9:00- 5:00	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Ask when making the appointment, as this varies by services needed.		Garages at John St. or Market St. are within two blocks.	\N	Yes	Not Applicable	\N	Yes	Typically, we schedule appointments two weeks out.	For emergencies, it is sometimes possible to  see a client sooner -- please be prepared to describe the emergency to a staff person when calling for an appointment. ALL services are confidential.	\N	\N	\N	\N	\N	Yes	Staffing front desk to make appointments; individuals with legal studies or experience may be trained to assist with immigration forms.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	We offer additional languages on an occasional basis, but recommend that non-English speakers bring a translator when possible.	Not Applicable	0	\N
506721	102574	2008-08-18	\N	Merrimack River Water Quality MAPP Project	MAPP Project	600 Suffolk Street	Fifth Floor	Lowell	MA	01854	978-275-0120	\N	\N	978-275-0125	www.merrimack.org	public_program_email@testing123.net	Merrimack River Water Quality Monitoring	Merrimack River Water Quality Monitoring, Analyzing, Protecting and Promoting (MAPP) Project  Once per month, May to October\r\n \r\nThe MAPP Project needs volunteers who own and operate boats and/or volunteers who are interested in taking water quality samples from boats or by wading.  Water quality monitoring consists of traveling to 7 or 8 predetermined sites in one section of the Merrimack River and recording the information displayed on a water quality meter.  A few teams will also collect water samples for bacteria analysis in the lab.  Monitoring takes three to four hours each trip. Training is a four hour session in April on how to use the equipment.	Everyone	Free	\N	Annually, May through October	M-Th 10am-6pm	English	No	\N		On street and a visitor lot	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Water Quality Monitoring\r\nWebsite Design & Maintenance\r\nOffice Manager\r\nOffice Assistance	No	\N	No	\N	Yes	Mid-Stream News - view online at www.merrimack.org	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
509196	103432	2008-08-18	\N	New England Culture Fest	NECF	172 Middle Street	\N	Lowell	MA	01852	978-455-6025	\N	\N	\N	www.second-world.com	public_program_email@testing123.net	Non-Profit concert/art festival promoting fair trade.	The Fifth Annual New England Culture Fest returns . . . September 6th, 2008, 2PM-10PM in the Enterprise Bank Parking Lots! In an exciting fusion of film, fashion, world culture and music, celebrating our efforts for a better world the NECF will transform the Lowell Arts District into a carnival on the cobblestones in the Enterprise Bank Parking Lots (2-10 PM).  The streets will come alive with the sounds of samba from twenty piece Brazilian troupe Samba Tremeterra along with more than a dozen world class acts, speakers and disc jockeys gathering to raise awareness for a more sustainable, vibrant and unified world! Get ready for prizes and giveaways, delicious Indian cuisine from Bollywood Grill and other global treats! When night falls brace yourself for large scale video projections from The Second World Fair Trade Story Film Archives and The Fashionably Fair Runway Show featuring sassy models and ethical designs. The festival features the interactive Ethical Art & Biz Expo featuring nearly one hundred amazing artists and dynamic businesses.  Green Mountain Coffee, Organic Valley and Equal Exchange will be giving away free goodies! Visitors can enjoy salsa lessons, henna, face painting, children's activities, ethnic cuisine and an array of handmade crafts and fair trade arts from jewelry to textiles.  Look for our two hundred foot long NECF Art Fence lining Middle Street and created from Boott Cotton Mill fabric and painted by youth artists! For Sponsorship Opportunities or to become a Featured Exhibitor call 978-455-6025 or visit www.second-world.com	Anyone	Yes	For this event there is a $5 to $10 admission fee.	The culture fest is an annual event, this year it takes place on September 6th.  Second-World works year round to promote fair trade and art empowerment.	September 6th from 2pm to 10 pm for the festival.  Other days throughout the year by appointment.	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	The entire event is volunteer run.  All positions are open to volunteers and we always welcome new ideas and people into our staff.	Yes	10 years worth of multimedia footage in archive from world travels and outreach efforts on behalf of fair trade.  In development for documentaries and fair trade promotional materials.	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	This is an annual event located in the Downtown Lowell Arts District but open to attendees from anywhere.	\N	Yes	0	\N
501397	100253	2008-08-18	\N	Low Vision and Adaptive Equipment	\N	169 Merrimack Street	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_program_email@testing123.net	Adaptive equipment for visually impaired and blind	Many low vision and adaptive equipment items are available for use at the LAB office. Including: use of color and black and white CCTV (closed circuit TV), computer equipment and software such as JAWS (screen reader), ZoomText, Duxbury and a Braille Embosser. Assistance and catalogs such as magnifiers, talking watches, talking calculators, etc. Training programs for adaptive equipment, computers, the Internet and software programs and Braille are available on site.	Referrals from the Massachusetts Commission for the Blind may be requested. Open to Merrimack Valley and southern NH	Free	No fees for the blind and visually impaired.  Equipment available for purchase.	Services available year round	Monday-Friday 8AM-4PM	English	Yes	Contact program. Referral may be requested.		On street/ Handicapped parking on Kirk St.	Many clients eligible for Road Runner service.	Yes	No	\N	No	\N	Serve people in the Greater Merrimack Valley and southern New Hampshire areas	\N	\N	\N	\N	\N	Yes	Volunteers are used in all aspects of LAB operations. Contact Derek Lessieur for more information	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
501408	100253	2008-08-18	\N	Radio Reading Service	Talking Information Center	169 Merrimack Street	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_program_email@testing123.net	Information radio station that provides reading newspapers	The Lowell Association for the Blind (LAB) and the Talking Information Center is an information radio station that provides the reading of daily newspapers by volunteers. Newspapers include the Lowell Sun, Nashua Telegraph and the Lawrence Eagle Tribune. Special receivers are available to pick up this station by contacting the LAB office.	Blind and visually impaired residents of the Greater Merrimack Valley and southern New Hampshire	Yes	There may be a one time fee to purchase receiver	Program runs year round	Monday-Friday 8AM-4PM	English	Yes	Contact program for info about receiver		Handicap parking is available. Metered and garage parking also available.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers read each morning on LAB/TIC radio. Volunteers tape in afternoon for use in the morning. Contact Derek Lessieur	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
505302	100253	2008-08-18	\N	Senior Teen Employment Program (STEP)	\N	169 Merrimack Street	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_program_email@testing123.net	employment and training for blind teens.	Provides employment and training opportunities for blind and visually impaired teens ages 14-21 living in the Greater Lowell and Greater Lawrence areas.	must be legally blind	Free	\N	program meets on Saturdays and one evening per week.	times vary	English	Not Applicable	\N		on street parking at meters or parking in the Market Street Parking Garage.	transportation provided for program participants.	Yes	No	\N	Yes	if program is filled a waiting list will be established	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	Yes	monthly schedule mailed to all participants	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	0	\N
501419	100253	2008-08-18	\N	Youth and Senior Youth Programs	\N	169 Merrimack Street	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_program_email@testing123.net	Social/ educational opportunities for blind  youth	The Lowell Association for the Blind operates two youth programs for blind and visually impaired youth in the Greater Merrimack Valley. The Youth program serves children ages 5 to 14 years of age referred by the Massachusetts Commission for the Blind. The Senior Youth program serves children ages 15-20. Both programs provide social, recreational and educational opportunities for blind and visually impaired children.	Youth program: ages 5-14, referrals required through Mass. Commission for the Blind. Senior Youth: ages 15-20.	Free	\N	Program runs year round. Enrollment is ongoing.	Weekends	English	Yes	youth program participants must have referral from Mass. Commission for the Blind. Senior Youth: contact program		On street & handicapped parking available on Kirk Street.	\N	Yes	Yes	\N	No	contact LAB office for availability	Area Served: Greater Merrimack Valley	\N	\N	\N	\N	\N	No	Interested volunteers should contact LAB for application	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
505654	102057	2008-08-19	\N	Poetry in Action	Family Literacy & Expression Project	365 West Meadow Rod	\N	Lowell	MA	01854	978-821-0953	\N	\N	\N	www.cultureiscool.org/whatiscool/cool-projects/coolmobile	public_program_email@testing123.net	Family art classes to promote literacy & self-expression	Poetry in Action is a series of poetry and art workshops for families living in transitional housing. \r\n\r\nEach week, a group of volunteers work with families at the shelter on different art projects. The mothers participate in creative writing workshops while their children engage in arts and crafts activities. At the end of the session, mothers and their children come together to share what they learned and created.	Women and children living in transitional housing	Free	\N	Year Round	Mondays and Tuesdays, 6:30 p.m. - 7:30 p.m.	English!DELIM!Spanish	Not Applicable	\N		\N	Poetry in Action, a program of the Art Ship Enterprise, facilitates workshops on site.	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	We are looking for creative, energetic and caring individuals to volunteer one evening a week facilitating art activities with children ages 3-13. Every Monday and Tuesday evening, we work with amazing families at House of Hope and Milly's Place. We need volunteers to lead art classes with the children while the mothers have their creative writing class.	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA	\N	\N	\N	Not Applicable	0	\N
501089	100143	2008-08-20	\N	Career Services	\N	18 John Street	\N	Lowell	MA	01852	978-458-2503	\N	\N	978-805-4913	www.cclowell.org	public_program_email@testing123.net	Resources & Assistance with regards to employment & career	Services include workshops on the following: Career Planning and Job Search Transitions, Labor Market Research, Resume Workshop, Mock Interviewing, Interviewing Techniques, Salary Negotiations, Job Search Letters, information for the Mature Worker, and Positive Thinking for Success. There is a Library of resource materials to assist you in every stage of your job search, as well as a Resource room with computers and laser printer, software for resume writing, a copy machine, fax machine and telephones. The Resource Room includes local, statewide and nationwide job listings, a Talent Bank to post resumes and labor market information. Assessment tests are available, including Myers Briggs Type Indicator, Self-Directed Search and Test of Adult Basic Education. Additional training is available to those who are eligible. Special assistance is available to Veterans through the Veteran's Unit. An Unemployment Insurance Representative is located on site to assist individuals with any matters pertaining to their Unemployment Insurance claim.	The Career Center is a membership organization. Fill out membership form at Center - membership open to all.	Free	No fees charged for basic services. Fees may apply for certain classes. Call for information.	Program services are available year round.	Monday, Tuesday,& Wednesday 8:30 AM - 5:00 PM, Thursday 8:30 AM - 7:00 PM, Friday 9:30 AM - 5:00 PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	Call or stop by for more information on membership.		Validated parking is available in the John Street Garage.	\N	Yes	No	\N	No	\N	On the phone: English only	\N	\N	\N	\N	\N	Yes	\N	Yes	Call for more information	Yes	Meeting space accommodating up to 25 people. Call for availability.	Yes	Some come through state.	Yes	For job search and career research purposes	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
501144	100154	2008-08-20	\N	Lowell Health Department	\N	341 Pine Street	\N	Lowell	MA	01851	978-970-4010	\N	\N	978-446-7100	http://www.lowellma.gov/depts/health	public_program_email@testing123.net	Protect the health and wellness of the people of Lowell	The mission of the Health Department is to preserve and maintain the city's public health standards to promote and protect the health and wellness of the people within Lowell. Public health services provided by the department include:\r\n-Health Inspections: the Health Department is responsible for sanitary code and food establishment inspections.\r\n-Permitting\r\n-Community Health: Outreach Health Education, Tobacco Control Program, \r\n-School Health	Services are available to Lowell residents	Yes	Fees may apply for permitting and some inspections. Contact program for more information	Services available year round	Monday-Friday 8AM-5PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Contact for more information		Parking available	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Yes	City of Lowell's web site.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
506787	102585	2008-08-20	\N	Project Connect	\N	900 Chelmsford Street	\N	Lowell	MA	01851	978-291-0342	\N	\N	978-291-0344	www.littlesprouts.com	public_program_email@testing123.net	GED instruction and preparation, workforce development	Project CONNECT is for DSS involved youth ages 16-21 who are in need of a GED or are in need of Adult Basic Education.  Services include outreach and casemanagement, home visiting, advocacy, workforce development, youth development, job search and placeement.	Participants must be involved with the Department of Social Services	Free	\N	Year round.  Monday - Friday	Hours vary.  GED classes are generally 2 days per week for 4 hours with tutoring occuring one time per week.  Other days of the week are spent job search and developing, teaching life skills and home visiting.	English!DELIM!Spanish!DELIM!Khmer	No	Referal from DSS Social Worker		\N	\N	Yes	Yes	Child Care available through Voucher and private pay	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
506776	102585	2008-08-20	\N	Project STRIVE	\N	900 Chelmsford Street	\N	Lowell	MA	01851	978-291-0342	\N	\N	978-291-0344	http:.//www.littlesprouts.com	public_program_email@testing123.net	GED preparation and Early Childhood Occupational Skills	Participants ages 16-21 are eligible.  This program provides GED instruction, occupational skills training in early childhood education, leadership skills development, three (3) college credits towards associate degree and job placement in an early childhood education setting.	Participants must be eligible through the Career Center of Lowell and ValleyWorks Career Center of Lawrence.	Free	\N	Year round	Monday-Friday 9:30-2:00	English!DELIM!Spanish	Yes	Proof of residency, proof of income, birth certificate, social security card, school withdrawl form	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!05 Westford Street/Drum Hill	\N	\N	Yes	Yes	Child care provided through vouchers	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	Monthly newsletter on website www.littlesprouts.com	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	Not Applicable	0	\N
505390	100902	2008-08-20	\N	Driver Alcohol Education Classes in Spanish	DAE in Spanish	9 Central St	Suite 400. 4th Floor	Lowell	MA	01852	978-459-3366	\N	\N	978-446-0817	www.lhi.org	public_program_email@testing123.net	Driver Alcohol Education Classes for first time offenders	DAE is a first offender driver alcohol education program in accordance with Massachusetts Statute 24D and is licensed by the Massachusetts Department of Public Health.\r\n\r\nThe goal of this program is to reduce alcohol related highway fatalities. Through our interactive and dynamic curriculum, we facilitate the participants ability to take responsibility for their previous drinking and driving behaviors. Our students are then encouraged and guided to make the necessary changes in their behavior that can prevent future offenses.\r\n\r\nAs needed, additional support for this objective is offered through referrals treatment oriented substance abuse and mental health services.\r\n \r\nThe referring court will receive a written notification once an initial intake is completed. They will receive weekly status reports for the duration of the clients program involvement.	To enroll in the program you must: Be court referred, complete an initial intake in person, schedule an assessment interview, pay an initial deposit of $200\r\nSelect your class schedule	Yes	There is mandatory fee that is imposed by the Court system of $567.22. All students must pay a $200 deposit upon enrollment.	We are open year round Monday through Friday 9am to 5PM.	Classes are held:\r\nMondays\r\n11 am to 1PM\r\nTuesdays\r\n6:00 PM to 8:00 PM\r\nWednesdays\r\n6:00 PM to 8:00 pm\r\nThursdays\r\n6:00 PM to 8:00 pm	English!DELIM!Spanish	Yes	\N	02 Belvidere	On street meter parking and public garages	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Space is available for meetings, group workshops and training sessions. Call for more information.	Yes	\N	Yes	Available to the public for job search, health information, and resources	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
502948	100902	2008-08-20	\N	El Poder del Cambio	IDU Prevention and Education Program	9 Central St	Suite 400	Lowell	MA	01852	978-459-3366 ext. 222	\N	\N	978-446-0817	www.lhi.org	public_program_email@testing123.net	Health Education, Prevention & Support Services	Project ElPoder del Cambio, provides 6-12 confidential education sessions in which HIV / AIDS, STD's, Substance Abuse and other topics are discussed. We offer prevention services and counseling and testing for our clients. Referrals are available if the client needs them. The El Poder del Cambio, project also runs support group workshops on important topics that clients can relate to.	Open to Spanish speaking IDUs and their partners who are 18 years of age or older. Any person meeting this criteria who is interested in educating themselves on ways to prevent HIV and STD infections are welcome	Free	\N	Services available year round. Enrollment is rolling and ongoing. There is a minimum of 6-12 sessions (more if client desires). Vouchers available for clients.	Monday-Friday 9AM-5PM	English!DELIM!Spanish	Yes	Referrals are recommended and welcome and can come from other agencies, individuals or self-referrals. Drop-in will be taken care of if staff is available.		Metered street parking or garage on Market Street.	Transportation provided to program participants who need detoxification or have other emergency.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Space is available for meetings, group workshops and training sessions. Call for more information.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	On the phone: English, Spanish. At program site: English, Spanish	No	0	\N
503586	100902	2008-08-20	\N	Gay mens health project	\N	9 Central St - Suite 400- 4th Floor	\N	Lowell	MA	01852	978-459-3366 ext 228	978-328-7976	\N	978-446-0817	www.lhi.org	public_program_email@testing123.net	Health Promotion and Education for Gay Men	The Merrimack Men Project (MMP) is a program that targets Spanish speaking men of the Merrimack Valley. The program sponsors social and educational activities in the Lowell area and nearby cities.\r\nWe use individual and group modalities to deliver information.  \r\nWe also provide case management, condoms and other prevention materials as well as health information and referrals.	Men who live or frequent the Merrimack Valley	Free	\N	Monday to Friday 9:00 amd to 5:00 pm	Monday to Friday 9:00 am to 5:00 pm	English!DELIM!Spanish	Yes	None walking welcome		Metered parking or at the Market street garage.	\N	Yes	No	\N	No	\N	We provide food vouchers and other certificates to clients. We also provide health promotion education in other agencies in the community.	\N	\N	\N	\N	\N	Yes	Peer leaders	No	\N	Yes	Space is available for meetings, group workshops and training sessions. Call for more information.	No	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
504169	100902	2008-08-20	\N	INFORMATE	\N	9 Central St Suite 400	\N	Lowell	MA	01852	978-459-3366	\N	\N	978-446-0817	www.lhi.org	public_program_email@testing123.net	HIV Cunseling, Testing and Referrals.	Informate serves the Latino Community providing culturally competent and linguistically appropiate HIV testing, counseling services, and referrals. Our staff includes (English & Spanish) counselors who are experienced and friendly. All services are free and confidential. Services are offered as follows:	Anyone	Free	\N	LHI Lowell. 9 Central St. Suite 400, 4 th floor, Lowell.  Monday: 10:30am to 12:30 2:00pm to 3:45pm\r\nHabit Managment Lowell. 650 Suffolk St: Mondays 9:00am to 10:00am\r\nNew Challenge. 209 Summer st, Lowell: Wednesdays 10:30am to 11:30am\r\nPathfinder: 94 Rock St, Lowell. Wednesdays 12:00pm to 2:00pm \r\nHabit Managment Lawrence. 599 Canall St, Lawrence. 8:45am to 10:00am.	See the Program Schedule. The other hours by appointment.	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Space is available for meetings, group workshops and training sessions. Call for more information.	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
507843	103157	2008-08-26	\N	New England Prison Ministries	Life Boat	PO Box 1785	\N	Lowell	MA	01853	978-458-1721	\N	\N	\N	www.nepm.org	public_program_email@testing123.net	Helping discharged prisoners build new lives.	New England Prison Ministries is a faith-based program that helps discharged prisoners build new lives through relationship. The Life Boat Mentoring Program connects inmates with church members who help develop a re-entry plan prior to relase and provide encourgement and support after release. The Life Boat Re-entry Resource Center will provide case management, employment prepapration and life planning support to discharged prisoners.	Inmates returning to the Merrimack Valley or Framingham areas. Families of incarcerated individuals.	Free	\N	Year round.	Week days 9-5. Life Boat Fellowship meetings open 7.00 PM Mondays at New Life Christian Community Church, Lowell and Tremont Temple, Boston.	English	Not Applicable	\N	09 Downtown Circulator!DELIM!18 Express Shuttle	\N	\N	Not Applicable	Not Applicable	\N	Yes	Mentoring wait list based on availability of mentors.	\N	\N	\N	\N	\N	\N	Yes	Life Boat Life Coaches\r\nLife Boat Employment Coaches\r\nWeb Content Manager	Not Applicable	\N	Not Applicable	\N	Yes	Email newsletter available on request.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County	\N	Limitied support is available in Boston and other cities in the Metro Boston areas based on the availability of mentors.	\N	Not Applicable	0	\N
501540	100396	2008-08-27	\N	Parent/Child Playgroup	The Family Connection Playspace	Pawtucket Congregational Church	15 Mammoth Road	Lowell	MA	01854	781-738-2392	\N	\N	\N	www.TFCplayspace.weebly.com	public_program_email@testing123.net	Parent/Child Playgroup	The Family Connection features a large indoor area where young children can play in a safe environment, and where parents can connect with one another. Our playspace offers climbing structures, tricycles, infant toys, a train table, a toy kitchen with accessories, giant building blocks ... and much, much more! Every playdate includes time for free play, snacks, arts and crafts, and a music circle. Since it is our members  not paid  employees  who run our playspace,  we ask all members to volunteer in some  way.	Open to all	Yes	A TFC membership costs $37 per year (July 1st  June 30th), plus $1 per visit per family. (If you are joining after March 1st, the membership fee is $20.) Membership fees cover the cost of rent, snacks, craft supplies, and general upkeep of the playspace.  Non-members pay $5 per visit.	The playspace is open year round but closed when the Lowell Public Schools are closed or delayed due to inclement weather.	We are open Monday and Thursday mornings from\r\n10 a.m. to 12 noon. Families are welcome to arrive and depart any time within this time frame.	English	No	\N	07 Pawtucketville / UMass Lowell North	Church parking lot and on-street parking on Mammoth Road.	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	The Family Connection is a volunteer-run, organization.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
505940	102134	2008-08-27	\N	Legal Advocacy Project	\N	P.O. Box 2100	\N	Lowell	MA	01851	978-937-5777	1-888-291-6228	1-888-291-6228	978-937-5595	www.alternative-house.org	public_program_email@testing123.net	Legal advocacy services are free and confidential.	Our legal advocates can assist with the restraining order process in court, answer questions, assist with shelter and safe housing and provide referrals to other needed resources.  We also work in collaboration with  other agencies to provide support and information on immigration issues as they relate to domestic violence.	Women who are in an abusive relationship and are in need of assistance are eligible for services. Teens in an abusive relationship who are in need of assistance are also eligible.	Free	\N	Alternative House is open year round.	Emergency Hotline is available 24 hours a day and staff are available to provide assistance, information and referrals. Legal Advocates are also available during normal business hours.	English!DELIM!Portuguese!DELIM!Spanish	No	Appointments are not necessary for legal advocacy assistance in the court.  A woman can find a legal advocate in the clerk's office at the court or inform personnel at the clerk's office of her need if she does not see an advocate available.		Parking is available.  Talk to staff for assistance with parking information.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Anyone interested in volunteer opportunities can contact the Alternative House Outreach Office at (978) 937-5777 or the 24 hour hotline at 1-888-291-6228.	Not Applicable	\N	Not Applicable	Alternative House will secure a safe place to meet with women in need of assistance.	Yes	Alternative House provides a quarterly agency newsletter to the community. Please call the Outreach Office or Hotline if you would like your name on the list to receive the quarterly newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	We work with other community agencies to obtain translation assistance for women in need if we currently do not have staff available to speak that language.	No	0	\N
505929	102134	2008-08-27	\N	Supervised Visitation	\N	P.O. Box 2100	\N	Lowell	MA	01851	978-937-5777	1-888-291-6228	1-888-291-6228	978-937-5595	www.alternative-house.org	public_program_email@testing123.net	Children's Supervised Visitation Center	Alternative House provides supervised visitation at our center in Lowell and at our center in Haverhill.  Supervised visitation programs were established to ensure the safety and welfare of the children and adults during supervised contact and to enable an ongoing relationship between the non-custodial parent and child in a safe and structured environment.  Direct communication between custodial and non-custodial parents and visiting parents is not permitted.  Our Visitation Centers provide a warm, child friendly setting where the focus of all parties involved is on the children.  We offer fully supervised visitation, semi-supervised visitation and monitored exchanges.	Many of our clients are referred by the Court or by other social service agencies, but self referrals are also welcome.  Parents, attorneys and other advocates may contact the Visitation Coordinator to determine if the use of the Center is appropriate for a family.	Sliding Scale	There is a flat rate fee for all intakes.  Visitation services are provided on a sliding fee scale based on the income of the parent(s) paying for services.  Additional fees will be charged for collateral contacts, such as providing Summary Reports for court.  The Center provides only the facts as documented through visitation and other contacts.  We do not make recommendations about future visits, provide evaluations, or assess parenting skills.  Current fee rates can be obtained by calling the Visitation Coordinator at (978) 937-5777.	The Visitation Centers are open year round.  There is night and weekend availability at both at both centers.	Visitation is held at the Lowell Site Tuesday-Friday afternoons and evenings and all day Saturdays.  Visitation at the Haverhill site is available on Saturdays and Monday evenings only.  The center can be reached by calling our hotline at 1-888-291-6228 or outreach office at (978) 937-5777.  If no one from visitation is available to take your call, please leave a message and someone will return your call at their earliest convenience.	English!DELIM!Spanish	Yes	All parties in need of visitation services must speak with the Visitation Coordinator to obtain an appointment.  Each party will be told what to bring for an intake upon scheduling the appointment.  During the intake rules and policies are reviewed as to what is permitted during visitation.		There are strict guidelines with regard to parking to maintain safety at all times.  All parties must communicate with the Visitation Coordinator as to where their designated parking is located.	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	Alternative House sends out a quarterly agency newsletter.  Please call (978) 937-5777 if you would like your name put on the list to receive our newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	0	\N
509240	102134	2008-08-27	\N	Emergency Shelter for Domestic Violence	\N	P.O. Box 2100	\N	Lowell	MA	01851	978-937-5777	1-888-291-6228	978-454-1436	978-937-5595	www.alternative-house.org	public_program_email@testing123.net	Short-term Emergency Shelter for Domestic Violence	Alternative House provides temporary emergency shelter for women and children fleeing domestic violence.  We have accomodations for women with children and single women.  Our emergency shelter is free and confidential.  Anyone in need of emergency shelter due to domestic violence should call our 24 hour hotline at 1-888-291-6228 (tollfree) or (978) 454-1436.	Women and children who are in need of emergency shelter due to domestic violence.	Free	\N	Open year round	24 hours a day, 7 days a week.	English!DELIM!Spanish	No	\N		\N	Women in need of shelter will work with an advocate regarding transportation.	Yes	Yes	On site after school programs and summer camp programs provided to children of residents.	No	Entrance is based on availability at time of call.	\N	\N	\N	\N	\N	\N	Yes	Anyone interested in volunteering can contact the Alternative House Outreach Office at (978) 937-5777.	Not Applicable	\N	Not Applicable	\N	Yes	Alternative House provides a quarterly agency newsletter to the community.  Please call the outreach office or Hotline if you would like your name on the list to receive the newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
509251	102134	2008-08-27	\N	Safe Home Program	\N	P.O. Box 2100	\N	Lowell	MA	01851	978-937-5777	1-888-291-6228	1-888-291-6228	978-937-5595	www.alternative-house.org	public_program_email@testing123.net	Free and confidential emergency safe home program	Alternative House can provide an emergency safe place for women and children fleeing domestic violence during the middle of the night, weekends and/or holidays.  Women and children can remain in the safe home until the next business day when they can then meet with an advocate to further discuss options and plan to meet their immediate needs for safety and shelter.	Women and children fleeing domestic violence who are in need of an emergency safe place to stay until the next business day.  Referrals can be made by police, social service agencies, community agencies, individuals etc...	Free	\N	Open year round	Safe homing is provided to women and children needing a safe place to stay when it is after normal business hours.	English!DELIM!Portuguese!DELIM!Spanish	No	Women in need or referring agent can call the hotline directly at 1-888-291-6228 for immediate assistance.		\N	Transportation needs will be coordinated during phone intake.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Anyone interested in volunteering can contact the Alternative House Outreach Office at (978) 937-5777 or the 24 hour hotline at 1-888-291-6228.	Not Applicable	\N	Not Applicable	\N	Yes	Alternative House provides a quarterly agency newsletter to the community.  Please call the Outreach Office if you would like to be added to our list of recipients.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County	\N	\N	\N	No	0	\N
509262	102134	2008-08-27	\N	Transitional Housing	\N	P.O. Box 2100	\N	Lowell	MA	01851	978-937-5777	1-888-291-6228	1-888-291-6228	978-937-5595	www.alternative-house.org	public_program_email@testing123.net	Two year Transitional Housing Program	Alternative House provides multi-family transitional housing for up to two years for women and children coming from either a domestic violence or homeless shelter.  Our transitional program is communal living style where each family is provided with their own suite but also shares common areas with other families in the house.  Women are required to pay 30% of their total income for rent and are responsible to either work, volunteer, attend school or a training program for 30 hours a week.  Family Service Workers provide round the clock coverage and case management assistance.  On site computer training is also provided for women in addition to assistance in a variety of other areas such as job skills and parenting support.	Women and children who are currently in either a domestic violence or homeless shelter may apply by calling the Outreach Office at (978) 937-5777 or the 24 hour hotline at 1-888-291-6228.	Yes	Women are required to pay 30% of their total income for rent.	Open year round	Our Transitional Housing Program operates 24 hours a day, 7 days a week.	English!DELIM!Spanish	Yes	Those interested in applying need only call the outreach office at (978) 937-5777 and/or toll-free hotline at 1-888-291-6228.  Further information about the intake process will be provided upon receipt of completed application.		\N	Transportation will be coordinated upon acceptance into the program.	Yes	Yes	An after school program and summer and vacation camps are provided for resident children only.	No	Availability depends on level of occupancy within the program at time of application.	\N	\N	\N	\N	\N	\N	Yes	Those interested in volunteering can contact the Alternative House Outreach Office at (978) 937-5777.	Not Applicable	\N	Not Applicable	\N	Yes	Alternative House provides a quarterly agency newsletter to the community.  Please call the Outreach Office or Hotline if you would like your name added to the list of recipients.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
507381	102882	2008-09-01	\N	The Mentoring Program	\N	10 Favor Street	\N	Lowell	MA	01852	978-458-5777	\N	\N	978-458-7328	www.rfkchildren.org	public_program_email@testing123.net	Become a Mentor of a Teen	We are looking for mentors, male or female to help us in our goal to provide a role model for teens involved in the juvenile justice system.\r\n\r\nThe mission of The Mentoring Program, an agency-based program, is to offer system involved youth the chance to develop a trusting relationship with an adult mentor who can become their friend, role model and advocate.\r\n\r\nThrough specific, mentor-guided activities we hope to assist youth in building many skills such as confidence, self esteem, and leadership. Youth will also develop or refine skill, career and personal goals. The mutual experiences they impart, with the support of program staff will ultimately instill pro-social behavior; reduce delinquency, recidivism and other destructive and self destructive behaviors.	Adults between the ages of 25 and 60, male or female are encouraged to inquire.	Free	\N	Call for information.	Mon thru Fri - 4pm until 6pm	English!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Valid MA driver's license	41 Lawrence-Lowell!DELIM!03 South Lowell!DELIM!09 Downtown Circulator	Adequate parking available.	Call for Information.	No	No	\N	No	\N	\N	10 Favor Street	\N	Lowell	MA	01852	Yes	We are looking for adult mentors willing to spend just 8 hours a month with a teen involved in the juvenile justice system. You should be a great listener, non-judgemental and speak from experience.	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	It is a plus in building your mentoring relationship if you speak any of these languages.	Yes	0	\N
500044	100011	2008-09-02	\N	One-to-one Mentoring	\N	45 Merrimack Street, Suite 227	\N	Lowell	MA	01852	978-453-9339	\N	\N	978-452-0838	www.bbbslowell.org	public_program_email@testing123.net	Matching Adult mentors to 7 - 15 year old children	Matching mentors with children, 7-15 year old, primarily from single parent families, with caring adult volunteers who are 18 years of age or older.\r\n\r\nAlso work with select area schools to bring mentor to school environment for one hour a week.  Program limited to the established school programs.\r\n\r\nCollege Mentoring starting in Fall 2007, matching screened and trained adults to HS students to help with college application process.	Open to youth between the ages of 7-15 yrs old who live in the Greater Lowell area, or the following towns: Ashby, Ayer, Carlisle, Groton, Littleton, Pepperell, Pelham (NH), Shirley, Townsend. Referral from a Parent / Guardian, Teacher, Counselor, Youth workers, Social worker. The parent / guardian must agree and child must want to participate	Free	\N	Program runs year-round and lasts a minimum of 1 year for communtiy based or 1 school year for school program, enrollment is ongoing.	Monday-Thursday 7AM-5PM\r\nFriday          9am-5pm	English	Yes	\N		Parking is available in the John Street garage	\N	Yes	No	\N	Yes	Call for more details	Translators are available by appointment.	\N	\N	\N	\N	\N	Yes	Volunteer opportunities are available for both school-based and community based programs. Must be 18 years of age or older.  For more details, contact Big Brothers Big Sisters or visit our website, www.becomeabig.com.  Application available on line.	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Serve Ashby, Ayer, Billerica, Carlisle, Chelmsford, Dracut, Dunstable, Groton, Littleton, Lowell, Pelham-NH, Pepperell, Shirley, Tewksbury, Townsend, Tyngsboro and Westford	\N	No	0	\N
500066	100022	2008-09-02	\N	Lowell Alliance for Families and Neighborhoods	\N	97 Central Street	Unit #302	Lowell	MA	01852	978-454-5405 x 137	\N	\N	978-454-5043	www.lowellalliance.org	public_program_email@testing123.net	Community-based coalition working with families	The Lowell Alliance for Families and Neighborhoods is a community-based coalition that helps Lowell families work together to ensure family wellness, safety and stability, through their own neighborhood-based and school-based initiatives. Residents who want to become involved or have neighborhood concerns should contact the Alliance office.	Not applicable, the Alliance does not provide services.	Free	\N	Not Applicable	Not Applicable	English!DELIM!Spanish!DELIM!Khmer	Yes	Residents who want to become involved or have neighborhood concerns should contact the Alliance office.	02 Belvidere	Parking is available next to the building	Transportation may be available for small group meetings	No	No	\N	No	\N	The coalition focuses on the following neighborhoods: Lower Highlands, The Acre, Centralville, Back Central, Lower Belvidere	\N	\N	\N	\N	\N	Yes	Newcomers are always welcome. Special projects available. Contact Deanna Lima at the Alliance office.	No	\N	Yes	Small conference room (up to 15 people)	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Lowell, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
500077	100022	2008-09-02	\N	Parent Aide Program	\N	97 Central St. Unit 302	\N	Lowell	MA	01852	978-454-5405	\N	\N	978-454-5043	www.ywcaoflowell.org	public_program_email@testing123.net	Weekly Home Visits for Families involved with DSS	The Parent Aide program provides intensive work through weekly home visits to families involved with the Department of Social Services (DSS). The work includes counseling, advocating, education and support.	Participants must be referred by the Department of Social Services (DSS)	Free	\N	Services available year round. Participants receive a minimum of 3 months of service, with possibility of extensions.	Contact program for more information.	English!DELIM!Khmer	Not Applicable	DSS makes the referral		Parking available Leo Roy parking garage on Market St or on-street meter parking, however the program is a home visiting program.	It is a home visiting program, so transportation is not an issue.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
504290	100121	2008-09-02	\N	Family Fun Day	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Family Fun Day	Family Fun Day will be held at the Greater Lowell Family YMCA on TBA. Events will include:\r\n-Trolley Pizza catered lunch\r\n-Autographed memorbelia\r\n-MVP invitational basketball game\r\n-Slam dunk/lay-up competitions\r\n-Special guest appearance by Lucky, the Boston Celtics mascot\r\n\r\nAdmission tickets are $7/person (slam dunk extra.) All proceeds benefit YMCA basketball	boys and girls ages 3-14	Yes	$7 admission (slum dunk contest extra.) Proceeds benefit YMCA basketball	TBA	TBA	English	Not Applicable	\N		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	contact Fred Braun	No	\N	Yes	YMCA	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
500946	100121	2008-09-02	\N	Leaders Club	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Youth group program that helps develop leadership skills	The Leaders Club is a nationwide youth group program that helps develop leadership skills through weekly meetings, overnights, group activities, service learning, and weekend rallies with teens from other YMCAs in the region.	Open to makes and females grades 8th-12th	Free	Free with YMCA membership	Program runs September 16th- early June.	Tuesdays 6:30-8PM during the school year	English	Yes	Contact the program for more details		Parking available in lot.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Leaders Club members are volunteers as part of the program criteria	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
503795	100121	2008-09-02	\N	Martial Arts	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Martial Arts	YMCA Martial Arts Teaches:\r\n~Respect\r\n~Self-Confidence\r\n~Discipline\r\n~Self-Control\r\n~Self-Defense\r\nAND MUCH MORE!\r\n\r\nThis program is taught by Master An Studios	boys and girls ages 4-12	Yes	YMCA Members: $49\r\nProgram Members: $69	Program runs every 5 weeks during the school year. Contact the YMCA front desk for a schedule.	Lil Tigers (4-6 year olds) Saturdays 12-12:40pm\r\n\r\ncall for dates of availability\r\n\r\nJuniors (7-12 year olds) Saturdays 12:45-1:30pm	English	Yes	register at the YMCA or over the phone		parking is available	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
504884	100121	2008-09-02	\N	Middle School Dances	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	http://www.greaterlowellYMCA.org	public_program_email@testing123.net	Middle School Dances	Dances are offered every month at the YMCA for middle school students grades 5-8 from 6:30-9:00. Call the YMCA to find out when the next one is.	grades 5-8	Yes	$6 admission & food and drinks extra	once a month TBA	Friday nights 6:30-9:00	English	No	\N		parking is available	NA	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Yes	YMCA Gym	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	0	\N
501045	100121	2008-09-02	\N	Youth Basketball	\N	35 YMCA Drive	\N	Lowell	MA	01852	978-454-7825	\N	\N	978-454-8982	www.ymcabasketball.com	public_program_email@testing123.net	Basketball Program for youth ages 3 - 14	Itty Bitty Basketball: This class provides children ages 3 and 4 the opportunity for an introduction to the game of basketball. \r\n\r\nBitty Basketball: This program teaches 4-6 year old beginners the basic skills of basketball.\r\n\r\nYouth Basketball: This league is designed for youths ages 7-10 to develop fundamentals and have fun!\r\n\r\nJunior Basketball: This league is for boys and girls ages 11-14. One game and one practice a week. Tournaments are held twice a season\r\n\r\n*All YMCA Basketball programs are coed and provide shirts and awards. Games and practices are held at the YMCA and refereed by trained staff.	Ages 3-14	Yes	$45 for members. $80 for non-members. Financial aid may be available based on family income.	Program held in 5 sessions a year. Each session lasts between 5-8 weeks. Sessions begin in Fall, Winter, Spring, and Summer. Contact the YMCA for resistration dates.	Ages 3-6: Saturday mornings\r\nAges 7-14: Games Saturday afternoons and one practice a week TBA	English	Yes	Contact the program for more information		Parking available	\N	No	No	\N	No	\N	Some Spanish may be spoke on site	\N	\N	\N	\N	\N	Yes	Volunteer opportunities available as coaches and referees.	No	\N	Yes	Space is available for racquetball/ volleyball leagues; soup kitchen; pool parties, etc.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501947	100539	2008-09-02	\N	HIV / AIDS Education and Prevention	\N	11 Mill Street	\N	Lowell	MA	01852	978-970-1250	800-232-7725	\N	978-970-0843	www.maps-inc.org	public_program_email@testing123.net	AIDS/HIV Education & Prevention	Program provides education and prevention on AIDS/HIV and STDs, especially geared toward women between the ages of 18-45 of the Portuguese-speaking communities.	Must be Portuguese speaking. Contact program for details.	Free	\N	Year round. Contact program specific schedule.	Monday-Friday 9:00AM-5:00PM	English!DELIM!Portuguese	No	Contact program to enroll\r\nMaristela Tosato	03 South Lowell	Parking is available in the front of the building at 11 Mill Street	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Lots of information about AIDS /HIV and STD's all in Portuguese	Yes	Yes, there is a conference room. Please call for more details.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	0	\N
501958	100539	2008-09-02	\N	Immigrant Social Services Program	\N	11 Mill Street	\N	Lowell	MA	01852	978-970-1250	\N	\N	978-970-0843	www.maps-inc.org	public_program_email@testing123.net	Helps immigrants link w/ job, housing & government  services	The Immigrant Social Services, connecting Portuguese-speaking individuals and families to social, health and other services through information, referrals, case management, advocacy and other assistance.	Must be Portuguese speaking. Contact program for more details.	Free	Donations are encouraged	Year round. Contact program for specific schedule.	Monday-Friday 9AM-5PM	English!DELIM!Portuguese	Yes	Contact program for details\r\nMaristela Tosato	03 South Lowell	Parking is available in front of building 11 Mill Street	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Some information is available in Portuguese	Yes	Yes, there is a conference room. Please call for more details.	Yes	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	services are provided to Portuguese-speakers lving in the Lowell area	\N	No	0	\N
504334	101540	2008-09-04	\N	Saints Medical Center  Growth & Nutrition Clinic	\N	One Hospital Drive	4th Floor	Lowell	MA	01852	978-934-8458	\N	\N	978-934-8358	saintsmedicalcenter.com	public_program_email@testing123.net	We work with Children under 7 years w/feeding or growth prob	The Growth & Nutrition Clinic works with families and children who are not gaining weight as expected, who are hard to feed, who have medical issues affecting their nutritional status, who have reflux or other GI issues, who were premature and are having difficulty growing, who are anemic.  It is important to intervene early with children who are not growing well, to prevent nutritional deficiencies that are associated with developmental delays, behavior problems and attention deficits that last into school years.	Any child under age 7 who is not gaining weight or growing as expected by their pediatrician, losing weight,having trouble eating.	Free	We are a Mass Dept of Public Health grant funded clinic.  We request insurance authorization for visits, but do not charge for any uncovered visits.	Open year round	Mondays 8:30-5:30pm	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Must call for an appointment.  Request a referral from the child's Primary Care Physician, then call for an appointment or additional information.	02 Belvidere	Free parking in Saints  Medical Center Parking garage or\r\nFree Valet Parking-in front of Main Entrance	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	Staff is available for inservices, community education programs for variety of topics.	Yes	Hospital conference rooms are available	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Call for info.	\N	No	0	\N
501386	100253	2008-09-04	\N	Adult Program	\N	169 Merrimack St.	2nd floor	Lowell	MA	01852	978-454-5704	\N	\N	978-458-5563	www.lowellassociationfortheblind.org	public_program_email@testing123.net	Social, recreational & educational programs for blind adults	The Lowell Association for the Blind Adult Program provides social, recreational and educational opportunities on Tuesday and Thursday mornings.	Blind and visually impaired adults over the age of 21 living in the greater Merrimack Valley and Southern New Hampshire	Free	\N	Program available year round. Enrollment is ongoing	Tuesday and Thursday morning	English	Yes	Contact program	06 Broadway / UMass Lowell!DELIM!09 Downtown Circulator!DELIM!18 Express Shuttle	handicapped&metered parking on Kirk St.	Handicapped drop off spot on Kirk St. Most clients eligible for Road Runner service.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers provide sighted guide assistance during program and on field trips outside the agency.	Yes	Contact agency for information about the resource library - use is by appointment only	No	\N	No	monthly schedule of events sent to the blind and visually impaired in large print or braille	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
503465	100330	2008-09-04	\N	School Success for Newcomer Parents	\N	9 Central Street	Suite 203	Lowell	MA	01852	978-654-6957	\N	\N	978-454-6992	www.onelowell.net	public_program_email@testing123.net	Parent Liaison Program	The School Success for Newcomer Youth and Parents Initiative provides support services to under-performing immigrant and refugee youth and their families in the Lowell public schools.  Through the targeted outreach of a Parent Liaison, this initiative is aimed at reducing high levels of absenteeism, increasing the retention rate of at-risk youth and improving their academic success.  Outreach to families will also increase their capacity to navigate the complexities of the American school system while becoming more involved in their childs academic success.	There is a closed referral system through the Lowell Public schools for at-risk youth.	Free	\N	Open year round	Mon-Friday	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	Currently this program employs Cambodian, Brazilian and Latino Parent Liaisons.	Not Applicable	0	\N
503223	100484	2008-09-05	\N	Dental Hygiene Clinic	\N	44 Middle Street	\N	Lowell	MA	01852	978-656-3250	\N	\N	\N	https://www.middlesex.mass.edu/DHClinic/	public_program_email@testing123.net	Offers dental hygiene care	The Middlesex Community College Dental Hygiene program operates a teaching clinic at MCC's Health Science and Technology Center at 44 Middle Street in Lowell. \r\nThis patient centered educational facility offers dental hygiene care September through mid-May. Services available include the following:\r\nFull mouth x-rays\r\nBite wing x-rays\r\ncomplete prophylaxis,\r\nathletic mouth protectors\r\npit and fissure sealants\r\nTo make an appointment for dental hygiene care, please contact the front desk manager at 978-656-3250. For Dental Hygiene Program information, please contact Nancy Baccari at 978-656-3096.	Dental Clinic is open to all ages. Because the Dental Hygiene Program is designed and maintained for teaching procedures, scheduling and dental hygiene services differ from those of a private dental office.  Those accepted for treatment are patients whose dental problems have teaching value and who have the extra time required for treatment.	Yes	Fees vary depending on procedure. \r\nFees for Dental Hygiene Care:\r\nAdult $25.00\r\nChildren $20.00\r\nAdults age 60 or over $20.00\r\nFull mouth radiographs\t$25.00\r\nBite wing radiographs $15.00\r\nSealants $10.00 per surface\r\nAthletic mouth Protectors $15.00\r\nPayment by cash or check only. There is no sliding scale or third party payments	Program runs on semester schedule: Fall semester runs September through mid December, and the Spring semester runs from the last week of January to mid May	Fall semester:Monday 8-12 and 4-6 WF 8-12 and 1-5; \r\nSpring semester: Monday 8-12 and 4-6 and WF 8AM-5PM;TR 8:00AM-7:00PM	English	Yes	\N		Parking available in Roy garage and Lower Locks municipal parking lots	\N	Yes	No	\N	No	\N	\N	MCC Dental Hygiene Clinic; ATTN: Nancy Baccari	33 Kearney Square, Talbot Building	Lowell	MA	01852	No	\N	Yes	\N	No	Meeting space is available when space not occupied. Contact Molly Sheehy, 978-656-3105 for information	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
504785	101716	2008-10-07	\N	St.Patrick School	\N	311 Adams Street	\N	Lowell	MA	01854	978-458-4232	\N	\N	978-458-4233	www.saintpatricklowell.org	public_program_email@testing123.net	Catholic Elementary School : K-1 - Grade 8	St.Patrick School is a K-1 (4 yr.olds) through Grade 8 school. It is run by the Sisters of Notre Dame de Namur and has served the children of the Lowell area since 1852. The school is officially accepted as Candidate for Accreditation by the New England Associaltion of Schools and Colleges. Our school community is culturally and religiously diverse. ESOL classes are provided for children who are new to the English language. St.Patrick's provides a Before and After School Program and participates in the federal nutrition program.Financial Aid is distributed yearly based upon financial need.St.Patrick School is authorized to issue I-20 forms.	Age requirements for K-1, K-2.\r\nTranscript/Report Card from previous school.	Yes	Yearly Tuition ( divided into 10 payments)\r\nRegistration Fee (yearly-per child)\r\nBook Fee\r\nFundraising : all families are asked to assist with school fundraising rojects throughout the year.	St.Patrick School follows the Lowell Public School year calendar.	Monday-Friday\r\n8:30-2:50	English	Yes	School Application\r\nBirth Certificate\r\nMost Recent Report Card (transfer students)\r\nBaptismal Certificate ( if Catholic)		yes	Lowell School Bus Transportation is provided for students 5 and older. Students must meet distance requirements.	No	No	\N	Yes	There is a waiting list for K-1 and K-2. There are seats open in Grades 1-8.	\N	\N	\N	\N	\N	\N	Yes	Various volunteer opportunities within the school day. All volunteers must have CORI checks and participate in the Virtus training program (3 hrs)	Not Applicable	\N	Not Applicable	\N	No	We would love to have a volunteer help us with producing a newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	We are able to have translation for Spanish, Khmer,Vietnamese as needed.	No	0	\N
506061	100088	2008-09-16	\N	Lowell Alternative Diploma Program	\N	34 Hurd St	\N	Lowell	MA	01852	978-856-3927	\N	\N	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	LOWELL ALTERNATIVE DIPLOMA PROGRAM	ADP is a partnership between Lowell High School, Lowell Adult Education, and the United Teen Equality Center. The goal of the Lowell Alternative Diploma Program (LADP) is to support student success through combining high expectations for academic success with strong and structured support systems. This competency and credit based program will provide Lowell's out-of- school youth a chance to earn a Lowell High School Diploma in a small, structured, alternative setting.\r\n\r\nYou are eligible if you have been out of school for at least six months and have at least 40 High School credits including two English classes completed. If you are interested, give Dirk Knoedler a call at 978.856.3911 or bring a copy of your high school transcript to UTEC and ask for Dirk.\r\n\r\nClasses are held at UTEC's new building on 34 Hurd Street in Lowell.	You are eligible if you have at least 40 High School credits.	Free	\N	Classes are held at Lowell Adult Education.  Call Shana for more info.	Classes are held at Lowell Adult Education.  Call Shana for more info.	English	Yes	Bring a copy of your high school transcript to UTEC and ask for Shana.		On Street (metered) or at Lower Locks Garage	\N	No	No	\N	Yes	There is currently no wait to attend LADP.  If class becomes full, student will be admitted on a rolling basis.	\N	\N	\N	\N	\N	\N	Yes	Please contact Shana for more information.	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a large room with conference table.	Yes	Go to www.utec-lowell.org to sign up for our free e-newsletter.	Yes	Free Internet to teens 13-21 during drop-in hours.	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
500836	100088	2008-09-17	\N	Youth Development Programming	\N	34 Hurd Street	\N	Lowell	MA	01852	978-856-3928	\N	978-265-7173	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	Teen Programs for Youth Development	Cultural Arts Programming Schedule\r\nSound Recording - Monday & Wednesday 4:00 to 6:00\r\nHip Hop Tuesday & Wednesday 6:00 to 8:00 \r\nSalsa - Monday 6:00 to 8:00\r\n\r\nUpcoming Workshops and Demonstrations\r\nHandbell Ringing\r\nMuseum Trips\r\nPerformances\r\nAnd Much More\r\n\r\nContact Tania Ormonde at 978.856.3928 for more information.	Must be a Lowell resident, or have transportation to downtown Lowell and be between the ages of 13-20	Free	\N	Services available year round. Enrollment is ongoing. Programs run on quarterly basis.	Cultural Arts Programming Schedule\r\n\r\n\r\nSound Recording - Monday & Wednesday 4:00 to 6:00\r\nHip Hop Tuesday & Wednesday 6:00 to 8:00 \r\nSalsa - Monday 6:00 to 8:00	English!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	An ID is needed if participant is over 18. Parental permission required for participants under 18.  Membership is always free. Call for more information about programs.		Limited on-street parking is available	Transportation may be provided for activities	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Grant-writing; accounting projects; landscaping and building help; volunteer to teach a class or seminar.\r\n\r\nCultural Arts Instructors: Volunteers in this position teach teens about a\r\ntalent, hobby, or skill they have, such as photography or African dance.\r\nVolunteers can do a one time workshop or a weekly class.  Please contact\r\nEmilie Steinnagel, cultural arts coordinator, at 978.856.3913 with interest\r\nor questions.	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a conference room with a large table; we also have a large multi-purpose space.	Yes	Periodic e-mailings.  Please visit our website www.utec-lowell.org to sign up for our e-newsletter.	Limited	Internet access to teen members only during drop-in hours	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
506105	100088	2008-09-17	\N	Young Women's Project	YWP	34 Hurd St	\N	Lowell	MA	01852	978-856-3928	\N	\N	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	The Young Women's Project (YWP)	Young Women's Programming at UTEC is made up of three components, Social and\r\nCommunity Development, Personal Expression and Personal Development.\r\nSupport groups, workshops and classes encourage young women to examine\r\nsocial expectations and limitations, express their hopes and fears, and\r\nbegin to work on the self-development needed to become strong and\r\nself-sufficient leaders, both in their personal lives and within the\r\ncommunity. \r\n\r\nDuring the summer the Young Women's Project will meet every Thursday at noon\r\nat UTEC to do activities and field trips such as going to the beach, making\r\njewelry, journaling and discussions.  On Tuesdays at 4:00 there will be\r\nexercise and nutrition classes such as yoga, palates etc.   \r\n\r\nDuring the year the YWP meets on Tuesdays and Thursdays at 3:00 for\r\ndiscussions, activities and a volunteer project organized by participants\r\nthemselves.  This program is called the Teen Outreach Program (TOP).\r\n\r\nThere are also various cultural arts classes offered such as clothes design,\r\nart and cooking. Call for schedules.\r\n\r\nVolunteers and instructors needed for workshops and cultural arts classes.\r\n\r\nContact the Young Women's Coordinator at 978-856-3914.	Young women between the ages of 13-21.	Free	\N	Open year round.  Schedules change in summer months.	UTEC is open for drop-in Mon-Thurs 2:30 - 8 PM.\r\n\r\nDuring the year the YWP meets on Tuesdays and Thursdays at 3:00 for\r\ndiscussions, activities and a volunteer project organized by participants\r\nthemselves.  This program is called the Teen Outreach Program (TOP).\r\n\r\nThere are also various cultural arts classes offered such as clothes design,\r\nart and cooking. Call for schedules.	English!DELIM!Spanish!DELIM!Khmer	Yes	For YWP programming, please call ahead. Drop-in is open to all, no appointment required.		Metered on street parking or at Lower Locks Garage	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Please call.	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a large room with conference table.	Yes	Please visit www.utec-lowell.org to sign up for our free e-newsletter.	Limited	Available to teens during drop-in hours.	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
507700	103091	2008-10-01	\N	MSPCC Healthy Families	\N	126 Phoenix Ave 3rd floor	\N	Lowell	MA	01852	978-937-3087	1-800-541-3004	\N	978-937-8695	www.mspcc.org	public_program_email@testing123.net	Parenting program for youth aged 14-20	Healthy Families Massachusetts is a statewide home visiting program administered by the Massachusetts Children's Trust Fund.  Healthy Families offers first-time parents  ages 20 and under the information and individual support they need to raise healthy, happy and safe kids.  MSPCC is the lead agency providing Healthy Families in five areas: Springfield, Holyoke, Worcester, Lawrence, and Lowell.  \r\n\r\nFor families in need of support, we offer comprehensive program of services delivered where they have been shown to have the greatest impact - in the home. Services are presented as an integrated package designed specifically to meet the needs of each family.	First time parents between ages 14-20.  Prenatal up to the child's first birthday.	Free	\N	Open year round	Monday-Friday 8:45-5:00	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Not Applicable	Call 978-681-9554 to give a referral or call 978-513-2367 for program information		\N	\N	Yes	Yes	Child care provided for groups.	Yes	Waitlist changes.  We do not always carry a waitlist	Limited services provided once a referral consents to being on wailist	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Yes	Conference room 20-30 person capacity.  Must be scheduled in advance.	Yes	Brief, one to two pages.  Quarterly.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
507689	103080	2008-10-01	\N	Community Gardening	\N	220 Aiken Street	\N	Lowell	MA	01853	978-957-8863	\N	\N	\N	www.communitygardensgreenhouse.org	public_program_email@testing123.net	Community gardening & education	The focus of the Community Gardens Greenhouse is on educational gardening \r\nworkshops that encourage collaboration, creativity and innovation while fostering \r\nrelationships that transcend ethnicity, character, class and age.\r\n\r\nOur goal is to establish our organization as an innovative, community based horticultural  development tool that will be responsive to the needs of the flowering city movement.\r\n \r\nImplement, promote and preserve community gardens in ethnically and socially diverse neighborhoods.\r\n\r\nEncourage residents to participate in horticultural and beautification projects.\r\n \r\nIncrease residents awareness of their responsibilities for Lowell's natural resources.\r\n\r\nPartner with established neighborhood groups to foster networking opportunities.	Open to anyone in the Lowell/Dracut area. Greenhouse in not wheelchair accessible; community gardens are outside, no steps or curbs. Some off-site garden projects may or may not be wheelchair accessible.	Free	Some fees may be applicable when taking certain classes. There is no charge or membership fees to garden.	Greenhouse opens in February for seed starting.Currently opened on Sat. 9am-1PM. Closed in fall/winter	Depends on volunteer schedules.	English	Not Applicable	\N		Parking is available on site.	No transportation is provided.	No	No	\N	Not Applicable	\N	This is an all volunteer organization. Projects are taken on based on staff availability.	Keep Lowell Beautiful / Greenhouse	P.O. Box 8625	Lowell	MA	01853	Yes	Anyone wishing to volunteer in gardens or work on certain garden projects can send email through web site.	Not Applicable	\N	Yes	Greenhouse meetings are generally held on the grounds in season	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA!DELIM!Dracut, MA	\N	\N	\N	Not Applicable	0	\N
509086	101496	2008-10-01	\N	Networking for the Prevention of Addiction	\N	194 River Road	\N	Lowell	MA	01852	508-572-8828	\N	\N	\N	http://www.universal-child.org	public_program_email@testing123.net	Online Education for Professionals and Concerned Adults	This pilot program is designed to facilitate the development of a professional online virtual classroom for medical professionals interested in addiction research. Presently, all adults are welcome and the only requirement is an interest in the concept of addiction: substance use disorders, gambling, eating disorders, etc.	Adults interested in current research and available resources associated with addiction disorders.	Free	All introductory courses are free.	We are presently using an open enrollment schedule in collaboration with class facilitators.	TBA	English!DELIM!Spanish	Yes	TBA		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	194 River Rd.	\N	Lowell	MA	01852	Yes	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
502607	100473	2008-11-12	\N	Lowell Adult Day Treatment	\N	35 Market Street	\N	Lowell	MA	01852	978-459-0389	\N	\N	978-459-7642	\N	public_program_email@testing123.net	Adult psychiatric day treatment services	Lowell Adult Day Treatment provides psychiatric day treatment services for those covered by Medicaid insurance. Structured 6 hour treatment days. Full and part-time enrollment options are available.	Ages 18 and older, with Medicaid coverage. Prefer referral from a professional source, but full assessment available on-site.	Yes	Medicaid full coverage. Spend-down option for direct payment may be available	Program available year round. Groups run from 9AM-3PM daily	Monday-Friday 9AM-5PM	English	Yes	Therapist/referring professional should ask to have referral form faxed to them for completion. Bring insurance card.		Metered street and municipal garage parking.	PT-1 Medicaid cab transport may be authorized within few weeks of start	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
500396	100044	2008-10-14	\N	Adolescent Health	\N	17 Warren Street, 1st Floor	\N	Lowell	MA	01852	978-746-7826	\N	\N	978-654-6731	www.lchealth.org	public_program_email@testing123.net	Primary Care for students at Lowell High & Rogers Middle Sch	The School Health Program provides primary care services to students through the health centers based at the Rogers Middle School and Lowell High School. The staff is comprised of a caring Medical Assistant and  licensed Nurse Practitioners. Service include sports physicals, sick visits, immunizations, health education, asthma management, mental health and regular check-ups. Referrals to other services.	Services provided to school aged youth enrolled at the Rogers Middle School and Lowell High School.	Sliding Scale	Fees vary. Insurance billed as appropriate. Sliding fee scale available, dependent on family income. Open to all. If you do not have insurance you may meet with a Health Benefits Coordinator to apply for insurance plans.	Schedule mirrors the Lowell Public School calendar.  referrals to LCHC pediatrics during non-school hours.	Monday-Friday 8:30AM-3PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Students need a signed parental consent form, copy of insurance card.		Parking available in visitors lot at Rogers School and at the Ayotte Garage at Lowell High School.	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	LCHC Happenings - A free newsletter for friends of the Health Center.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	Direct phone numbers:\r\nRogers School 978-459-7100 (phone)\r\n978-459-7905(fax)\r\n\r\nLowell High School 978-458-6642 (phone)\r\n978-458-6643 (fax)	Interpreter services available in 18 additional languages.	No	0	\N
500352	100044	2008-10-14	\N	LCHC Teen Coalition	\N	17 Warren St.	1st Floor	Lowell	MA	01852	978-322-8521 ext. 322	\N	\N	\N	www.lchealth.org	public_program_email@testing123.net	Provides education and prevention on teen health issues	The primary mission of the LCHC Teen Coalition is to reduce risky behaviors that lead to teen pregnancy, HIV/STD, substance abuse and violence in Lowell, in the belief that promoting healthy behaviors will enhance future prospects for young people. LCHC Teen Coalition subcontracted youth programs include: UTEC, Big Brother / Big Sisters, YWCA of Lowell, Greater Lowell Boys and Girls Club.	Must be between the ages of 12-18 residing in the Lowell area.	Free	\N	Teen Coalition-affiliated youth programs run year round.\r\nwww.myspace.com/lowellteencoalition	Youth Space hours Monday-Thursday 2:00PM-7:00PM which is located on Paige St. behind CVS in Downtown Lowell.\r\nProgram Staff are available Monday-Thursday 9:00AM-7:00PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Call with any questions about programs, referrals or help for youth.	09 Downtown Circulator	\N	Big Brothers Big Sisters of Greater Lowell in collaboration with LCHC's Teen Coalition provides transportation.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Please call for information on volunteer opportunities.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
504675	101661	2008-09-15	\N	Greater Lowell Elder Mental Health Collaborative	www.eldermentalhealth.org	18 east meadow lane	\N	Lowell	MA	01854	978-458-1512	\N	\N	\N	www.eldermentalhealth.org	public_program_email@testing123.net	web site	Web site which has information for consumers and providers about elder mental health including:Mental health issues(ex.- depression,substance abuse, dementia, etc.);Descriptive statements about elder mental health providers and services; Current legislative and policy issues; Caregiving; positive aging; Meetings and conferences; Information about state-wide coalition(MassAging and Mental Health Coalition- MAMHC).	all eligible	Free	\N	open year round	7 days a week; 24 hours a day	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	1. Develop revenue development plan;\r\n2. Assist with updating of web site(particularly with page about agencies).	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
506050	100088	2008-10-01	\N	UTEC Fresh Roots Program (farm, culinary, business programs)	Fresh Roots	34 hurd st	\N	Lowell	MA	01852	978-856-3910	\N	\N	978-654-6727	www.utec-lowell.org	public_program_email@testing123.net	UTEC's Sustainable Food Farm and Culinary Program	UTEC's Fresh Roots program runs throughout the year.  the farm is operational from april until october.  business and culinary programs run all months.   \r\n\r\nWe host volunteer days for teens and adults every Monday when we harvest, and we distribute our food to \r\n1. a local farmers market\r\n2. wholesale to 3 local restaurants \r\n3. the Merrimack valley food bank \r\n4. the Lowell transitional living center and 5. the families of the teen farmers and volunteers who help us out.  \r\n\r\nPlease contact Derek Mitchell if you wish to volunteer; on harvest days, people can take home veggies to their very\r\nown homes!)  \r\n\r\nThe culinary program will be hiatus for the summer, but will begin our weekly meal prep at UTEC in August.	Teens 13-21; volunteers of any age.	Free	\N	weekend opportunities in spring and fall; otherwise, programming happens after school during school months and during the day in summer months	varies	English!DELIM!Spanish!DELIM!Khmer	Yes	\N		\N	Van transporation is available from UTEC on 34 Hurd St. to the Farm location in Dracut.	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Please call Derek Mitchell at 978.804.6989	Yes	Please call to learn more about the resource library, 978-441-9949	Yes	We have a large room with conference table. Space is also available at the Farm itself.	Yes	Please sign up for our e-newsletter at www.utec-lowell.org	Yes	Internet access at UTEC during drop-in hours to teens 13-21	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
505610	102024	2008-10-17	\N	Food Pantry	\N	370 West Sixth Street	\N	Lowell	MA	01850	978-454-7445	\N	\N	978-452-4043	CFMinistry.org	public_program_email@testing123.net	Food Pantry	Clients receive 5 days of food for each member of their family.  They must have social security cards for each family member and proof of address for family members over the age of 18.  Clothing is available at all times.  Household items and toys are frequently available.	Anyone in need of food is eligible to come every 30 days.	Free	\N	Open every week except between Christmas and New Years.	Tuesday, Wednesday, Thursday:  9:30 a.m. to 1:00 p.m.  Thursday evening:  7:00 to 8:00 p.m.	English!DELIM!Spanish	Yes	Clients must call on the day they want to come in.  They must bring social security cards for each family member and proof of residence for any member over age 18.	08 Centralville	\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Packing food boxes; stocking shelves; registering clients; sorting clothing;	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County	\N	\N	\N	No	0	\N
503388	101078	2008-10-17	\N	Family Resource Program	\N	18 Palmer Street	\N	Lowell	MA	01852	978-452-3600	\N	978-937-1877	978-275-9697	www.caseyfamilyservices.org	public_program_email@testing123.net	Support & Programs for Families with Children in LHA	The Family Resource Program offers services to residents of North Common Village and George Flanagan housing projects in partnership with the Lowell Housing Authority. Services are also available to members to the community.  Services are designed to provide support to families and children. Services include counseling, information and referral, groups for parents and children, a homework club, advocacy and empowerment.	Must be a Lowell Housing Authority resident or a Lowell resident.	Free	\N	Services available year round	Monday, Tuesday, Thursday, Friday 9AM-5PM	English!DELIM!Spanish!DELIM!Khmer	No	\N		Parking is available.	Transportation may be available for program activities as needed.	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer tutors are needed for the Homework Clubs, offered at the two sites on Mondays, Tuesdays,and Thursdays from 2:30 to 5:00 p.m. If interested, please contact Marta Romero at 978-275-9719.	No	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	North Common Village and George Flanagan housing developments and resident of Lowell.	\N	No	0	\N
505632	102035	2008-10-21	\N	T.H.E. FARM	Tewksbury Hospital Equestrian FARM	365 East Street	PO Box 754	Tewksbury	MA	01876	978-851-5540	\N	\N	\N	www.t-h-e-farm.org	public_program_email@testing123.net	Therapeutic equine program for people with disabilities	Many different riding and/or unmounted therapeutic programs are available to adults and children with disabilities. Riding for able-bodied individuals also available.	Anyone is eligible for our programs. An evaluation session is mandatory for all potential riding clients.	Yes	Visit our website at www.t-h-e-farm.org	Open all year round; potential closings during severe weather conditions	Closed on Mondays	English	No	FARM visitors welcome. Appointment necessary for first evaluation.		Parking available	\N	Yes	No	\N	Yes	There is usually a waiting list especially during the peak season, however list is constantly changing. Please inquire.	\N	\N	\N	\N	\N	\N	Yes	Barn, lesson, fundraising and event volunteers always needed. Please visit our site at www.t-h-e-farm.org	Not Applicable	\N	Yes	Small meeting room; seats 15-20	Yes	Upcoming events & news of the organization	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509185	102871	2008-10-21	\N	JA Academy	\N	33 Broad St. Floor 7	\N	Boston	MA	02109	617-368-3566 x.17	\N	\N	617-368-6898	www.jaem.org	public_program_email@testing123.net	Build your résumé, college application with business program	JA Academy is a free Junior Achievement after-school program for ambitious high school students interested in running their own company and selling products with the help of business professionals and college students.  \r\nThe program takes place at local colleges/universities from 3:30-5:30pm one day a week.  Applications are available online.  Contact acunningham@ja-easternmass.org with subject line JA Academy for more information.	High school students grade 9-12	Free	\N	Fall semester Sept.-Dec.; Spring semester Jan.-April	One day a week from 3:30-5:30pm.	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	JA Academy adult mentors serve as part of a three or four-person team to facilitate the 12-session after-school program.  As a  mentor, you offer real world understanding to what the students are learning each week.  Guest speakers and board meeting hosts also welcome!	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	Merrimack College now available.\r\n\r\nSeveral colleges/universities throughout eastern MA: Babson College, Boston College, Boston University, MIT Sloan, Northeastern University, Suffolk University, UMASS Boston, Dean College(new), Merrimack College(new)	\N	Yes	0	\N
503520	100550	2008-10-23	\N	The GRIP Project and Evolution, Justice Resource Institute	Growing Responsibility and Independence in People	35 Varnum Avenue	\N	Lowell	MA	01854	978-458-3622	\N	800-636-1586	978-459-2049	www.JRI.org/grip_project.htm	public_program_email@testing123.net	Coed ITIL and supported apartments, and teen homelessness	Multi-funded transition to independent living program and supported apartments for young people in and out of state services. Teens with out family are our specialty. Referrals come directly to listed address and may be state involved, or not. Ages 16 to 23 served.	please contact for more information	Yes	Please contact for more information	Open year round	24/7	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	\N		\N	\N	Yes	No	\N	Yes	please contact	\N	\N	\N	\N	\N	\N	Yes	Rachel Chandler 978.458.3622	No	\N	Yes	Please call 978-458-3622 for more information	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	10-22-08 left message
507458	102915	2008-10-22	\N	Annual Rebuilding Day-Last Saturday in April	Saturday,  April 25, 2009	PO Box 5161	\N	Haverhill	MA	01835	978-469-0800	\N	\N	978-556-9376	\N	public_program_email@testing123.net	Annual Rebuilding Together Day	The Annual Rebuilding Together Day takes place on the last Saturday in April.\r\n\r\nRebuilding Together Greater Haverhill, Inc. (RTGH) is a non-profit organization working in partnership with the community, volunteers and skilled labor, to stabilize and revitalize homes in our city and surrounding communities at no cost to the recipient. The RTGH initiative brings homeowners in need, together with volunteers from all walks of life, to preserve home ownership and reduce the risk of homelessness by improving the quality of the lives of the program service recipients.	Applications for services are received for consideration by December 1st for the following year. Eligibility and guidelines can be requested by contacting the RTGH office.	Free	\N	Office open year round	Weekday hours vary and by appointment	English	Yes	\N	01 Haverhill - Methuen - Lawrence!DELIM!13 Main St. / North Ave.!DELIM!14 Bradford / Ward Hill!DELIM!15 Hilldale Ave. / Westgate!DELIM!16 Washington St. / Westgate!DELIM!18 Riverside!DELIM!19 Summer St. / Westgate!DELIM!51 Haverhill - Amesbury - Newburyport!DELIM!72 Haverhill	Merrimack Street Municipal Parking Deck-Free	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	Pentucket Bank Community Room\r\n35 Merrimack Street\r\nHaverhill, MA 01830	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Groveland, MA!DELIM!Haverhill, MA!DELIM!Boxford, MA!DELIM!Merrimac, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Georgetown, MA	\N	Plaistow, NH	\N	No	0	\N
507623	102420	2008-10-29	\N	Youth Empowerment Services Inc.	YES Inc.	19 Acadia Kimball Rd.	\N	Amesbury	MA	01913	978-388-3647	\N	\N	\N	http://www.empowertheyouth.org/	public_program_email@testing123.net	HIPP, Help Increase the Peace Program	HIPP is a violence prevention/ peace building program for youth that uses fun experiential activities to allow youth to be safe and respected as they share their experiences and explore ways to work together to promote peace in their schools and communities.	All youth over the age of 12 are eligible.\r\nHIPP leaders training is for adults and youth 15 and older.	Sliding Scale	Fee vary depending on the group and the individuals ability to pay. Please inquire.	Custom designed HIPP trainings for camps, schools and other youth serving programs\r\nare offered year round..	M-F 9-5. We come to you.\r\nTrainings may be scheduled into evening hours and on Saturdays	English!DELIM!Portuguese!DELIM!Spanish	Yes	Nothing required.		\N	\N	Yes	No	\N	Not Applicable	\N	\N	19 Acadia	Kimball Rd.	Amesbury	MA	01913	Yes	Please call to inquire.	Not Applicable	\N	Yes	Amesbury Friends Peace Center\r\nspace for trainings and meetings	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Primarliy Northeast Mass and southern NH.\r\nExceptions are made for training outside this area.	Lead trainer, Barbara Hildt was a Peace Corps volunteer and speaks Portuguese. She has done HIPP with youth in Kenya.	No	0	\N
507678	103069	2008-10-29	\N	Windrush Farm	\N	30 Brookview Road	\N	Boxford	MA	01921	978-682-7855	\N	\N	\N	www.windrushfarm.org	public_program_email@testing123.net	Equine Assisted Activities	Windrush Farm offers equine assisted activities for children and adults with disabilities, including: therapeutic riding, hippotherapy, and barn management opportunities.	Anyone deemed appropriate	Yes	Discounted rate of $45/hour	Year-Round	M-F 8-6	English	Yes	\N		Plenty of parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers 14 year and above are encouraged!	Not Applicable	\N	Not Applicable	\N	Yes	Quarterly newsletter and weekly enewsletters	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
507997	102948	2008-09-05	\N	Lawrence S.O.A.P Program	\N	11 Union Street	\N	Lawrence	MA	01843	978-685-1337	\N	978-685-1337	978-681-1281	tpc1.org	public_program_email@testing123.net	Structured Outpatient Addiction Program	Our mission is to aid the underserved population of Greater Lawrence in addiction recovery through evidenced-based treatment. Participants receive a full day of services which includes group and individual therapy.	ADD	Yes	ADD	Open year round	M-F, 8:30am to 3:30pm	English	Yes	\N	01 Haverhill - Methuen - Lawrence	\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy,Motivational Interviewing, Continigency Management,Substance Abuse, Mental Illness, Dual Diagnosis, Spirituality in Recovery	Yes	Very Large Community Meeting Space,Several Large Group Rooms	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
509317	102948	2008-10-31	\N	Adolescent  SOAP Structured Outpatient Addictions Program	\N	11 Union Street	\N	Lawrence	MA	01843	978-685-1337	\N	978-685-1337	978-681-1281	tpc1.org	public_program_email@testing123.net	Structured Outpatient SA Treatment for Adolescents	The four main components of service for the Adolescent SOAP program will be clinical, psycho-educational, prevention, and recreation/leisure time instruction and activities.  TPCs Adolescent SOAP programs will be located at The Psychological Center, 11 Union Street, Lawrence, Massachusetts 01840, and will admit adolescents ages 13-18 from the Merrimack Valley region with addiction and/or co-occurring disorders.  Individual program cycles will include 20 service units with a minimum of one unit service (2.5 hours) each day of participation in the program.  TPC staff will recruit clients from the local middle and high schools, juvenile courts, DYS and other community agencies.  Self-referrals and referrals from parents and friends will also be encouraged. We anticipate eager cooperation with the schools and local judges, all of whom have expressed their frustration at having few resources for substance abusing adolescents.\r\n\r\nTPCs experience with our Pegasus program indicates that the major issues for substance abusing youth are abuse of opiates, poly drug use and anger.  According to a Mass DPH and BSAS fact sheet, almost 50% of adolescent admissions to substance abuse treatment had also received prior mental health treatment, implying a high percentage of co-occurring disorders.  This number closely reflects our own experience.\r\n\r\nThe clinical component will employ Motivational Interviewing (MI) and Cognitive Behavioral Therapy (CBT) as the major treatment approaches.  Both approaches meet the client at their own particular starting (and re-starting) point.  Both are uniquely suited for a short-term, harm reduction outpatient treatment program and with clients who will not have been detoxed.\r\n\r\nThe four principles of MI used by clinicians are to express empathy, to develop discrepancy, to roll with resistance, and to support self-efficacy.   These principles allow a client to learn harm reduction elements of his behavior as he struggles to overcome his addiction; for example, drinking less is acknowledged as a gain.  MI works with clients own agendas and directs them to examine their attachment to their substance abusing behaviors, employing a stages of change model.\r\n\r\nCBT is based on the assumption that most emotional and behavioral reactions are due to early schemas which result in cognitive distortions.  CBT is structured, focused and typically a brief therapy.  CBT is delivered in a collaborative manner between the clinician and the client and is fact-based:  by looking at thoughts as hypotheses, they can be questioned and tested.  Between Session Training is assigned and progress is measured in part by completion and the clients use of what is learned.  CBT employs two levels of change: altering ways of thinking for the better and helping a person cope with challenges and opportunities in life with a clearer focus.\r\n\r\nGroup topics will include harm reduction measures, relapse triggers, relapse prevention, self-calming techniques and drug education.  Discussion topics will include politics, interactive music appreciation, computer/internet time, reading, homework assistance, community service, career discussions and recreation.  The scheduled elements will be conducted four days a week. The fifth day will be devoted to healthy instructive recreation.  The program will meet from 3:30 p.m. to 6:00 p.m.  Because travel time will vary for clients in the TPC SOAP, depending on the location of their school or day program, they may begin gathering at 3:00.  The first 15 minutes of each day will begin with check-in, a snack, relaxation breathing and music, as the client switches from the frantic activity level of the day to a more singularly focused period of time.\r\n\r\nIn addition to the structured daily program, clients will receive assisted referrals to any additional specialized services needed, such as, medical, psychiatric and social service.  Family involvement is particularly important to a young persons treatment and recovery plan, and will be encouraged and provided either on-site or in the home.  Clients will be supported in maintaining their ties with the community, particularly school, and encouraged to foster additional community ties through community service activities.  \r\n\r\nA mental health and addictions assessment will be conducted on either the first day of the program or prior to the clients first day of SOAP if possible.  Preference will be for the client to be admitted as quickly as possible, even immediately upon first contact.  Every effort will be made to have a parent or guardian present for the assessment.  Individual therapy sessions and family group meetings will be part of the structured service units.  Family therapy, Psychiatric and psychopharmacology services will be provided in TPC Outpatient Department as a separate service when deemed clinically necessary by the treatment team.\r\n\r\nCase management services begin immediately upon admission.  A review of current medical and behavioral health providers, with necessary contact information, will become part of the clients record.  Clients will be encouraged to continue their current provider relationships, if appropriate.  School and after-school activities and other community commitments will be considered in constructing each individuals SOAP treatment plan. \r\n\r\nRecovery, Discharge and Aftercare planning will be initiated on day one of the program and will include referrals to further recommended treatment, a review of living arrangements and housing assistance, if needed.  The plans will be updated weekly.\r\n\r\nAppropriate resource materials will be available for all program participants to facilitate thought provoking discussions, both during the program hours as well as during the adolescents home, school, and leisure time.  A peer support group will be offered at TPCs main office location at least once per month.  TPC will work with local AA/NA/Alateen organizers to develop this as an ongoing community service.\r\n\r\nThe program will conduct TOPS outcome assessments at Intake, and at clients discharge at minimum.  A satisfaction survey will also be presented to the client at discharge.  The results of this survey will be incorporated into the programs Quality Management program.\r\n\r\nA peer support group will be offered at TPCs main office location at least once per month.  TPC will work with local AA/NA/Alateen organizers to develop this as an ongoing community service.\r\n\r\nTPC will staff the Adolescent SOAP based on a ratio of 1:8 full-time staff to clients.  A full time Program Director (Clinician I) will be hired to begin the program.  Additional staff (Assistant Program Director, Clinician) will be hired based on program census.  Clinicians I and II will be trained in and have significant experience and understanding of adolescent development and adolescent clinical issues and represent cultural, linguistic and ethnic groups in the target population.  TPC believes that role modeling is extremely effective in substance abuse treatment.  To enhance the staffing of the Adolescent SOAP, we will recruit a part time SOAP Mentor who has graduated from TPCs Adolescent SOAP and has demonstrated responsibility, integrity, and a desire to share his/her own experiences to facilitate learning and understanding of the consequences of substance use.  \r\n\r\nEach staff person will attend a daily team meeting to review cases and program issues, in-service trainings as scheduled, maintain required records for each client and make necessary client referrals.  Clinician I shall receive a minimum of one hour per week of individual or group consultation from another Clinician I from existing TPC outpatient staff.  A Clinician II, if employed will receive a minimum of one hour group or individual supervision from a Clinician I. The SOAP Mentor will receive supervision by Adolescent SOAP staff.	Adolescents ages 13-18 from the Merrimack Valley region with addiction and/or co-occurring disorders having MBHP, and Neighborhood Health	Yes	Fees are covered by MBHP and Neighborhood Health ;  If one is self pay, the rate is $120.00 per day	Open Year Round	Monday through Friday 3:30 - 6:00	English	Yes	Call 978 685 1337 to schedule an appointment for an intake for Adolescent SOAP	33 North Andover	Behind the building and diagonally across the street	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Call Adolescent SOAP's Program Director or Human Resourses at 978 685  1337 for information.	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy,Motivational Interviewing, Continigency Management,Substance Abuse, Harm Reduction, Mental Illness, Dual Diagnosis, Spirituality in Recovery, Healthy Relationships,	Yes	Very Large Community Meeting Space,Several Large Group Rooms	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	Services avaialable to anyone having MBHP, or Neighborhood Health, or who wishes to be self pay.	\N	Not Applicable	0	\N
502486	100737	2008-10-31	\N	Exhibitions, Workshops and Classes	\N	18 Shattuck Street	\N	Lowell	MA	01852	978-452-4207	\N	\N	978-452-5405	www.nequiltmuseum.org	public_program_email@testing123.net	Preserve&make known the art, craft &history of quilt making	The New England Quilt Museum preserves, interprets and celebrates the art of quilting past and present through offering monthly workshops, celebrity guest lectures, school programs, and guided tours.	Open to all	Free	\N	Contact for information	Monday Closed, Tuesday-Saturday 10AM-4PM, Sunday May through December: 12PM-4PM	English	No	\N		Next door parking available on weekends	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are always needed. Contact the museum at 978.452.4207.	Yes	Resource library available on quilt history and quilt making, contact museum for more information	Yes	100 person room available for meetings and entertainment purposes. Contact museum for details	Yes	Monthly electronic newsletter, and print newsletter mailed to members 6/year.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509328	102948	2008-10-31	\N	Women's View	\N	582 Haverhill Street	\N	Lawrence	MA	01840	978-687-1658	\N	978-685-1337	978-681-6075	tpc1.org	public_program_email@testing123.net	6 month residential tx for women with chemical dependency	Women's View is a six month residential treatment setting for women with chemical dependency. The program offers a structured and nurturing environment that provides evidence based treatment through contingency management, motivational interviewing, and cognitive behavioral counseling. \r\nThe program focuses on early recovery issues, daily attendance of AA/NA meetings, reentry into the community, outpatient therapy and aftercare services.\r\nWomen must be medically cleared and have documentation of a recent TB test prior to admission.\r\nWomen's View accepts referrals from hospitals, detoxes, shelters, Department of Corrections,  Department of Social Services, and other community sources.\r\nReferrals can be made Monday through Friday from 9:00a.m. to 4:00p.m by calling (978) 687-1658.	Females over the age of 18 having a chemical dependence	Yes	Client fees  are determined on a sliding scale ;  Services billed through  Dept of Public Health / Bureau of Substance Abuse Services	Open Year	6 Month Residential Program	English	Yes	Call Women's View for an Intake Appointment - results of a recent TB test and a recent Bio PSych Social will be required	41 Lawrence-Lowell	On street	n/a	Yes	No	\N	Yes	Varies from one week to two months	\N	\N	\N	\N	\N	\N	Yes	Call Women's View for information	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy,Motivational Interviewing, Continigency Management,Substance Abuse, Harm Reduction, Mental Illness, Dual Diagnosis, Spirituality in Recovery, Healthy Relationships,	Yes	Very Large Community Meeting Space,Several Large Group Rooms	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
502618	100473	2008-11-12	\N	Bridgewell Counseling Services	\N	35 Market Street	\N	Lowell	MA	01852	978-459-0389	\N	\N	978-459-7642	\N	public_program_email@testing123.net	Provides outpatient mental health services	Bridgewell Counseling Services provides outpatient mental health services, psychopharmacology and therapy. Specialize in treating people who have a serious mental illness or serious mental illness in conjunction with mental retardation or a medical illness.	18 or older and have Medicaid coverage. Referral from a professional preferred, but full assessment on-site is available	Yes	Full Medicaid coverage. Spend-down for medicaid may be negotiated.	Program available year round.	Monday-Friday 9AM-5PM	English	Yes	Therapist/referring professional should ask to have referral form faxed to them for completion.		Metered street or municipal garage parking available.	Located on downtown LRTA bus route	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
509339	103465	2008-11-12	\N	Health Insurance Enrollment	\N	9 Central Estreet, Suit 400	\N	Lowell	MA	01852	978-394-2026	866-362-6006	\N	\N	http://mvhealthenrollment.net/	public_program_email@testing123.net	Health Insurance Enrollment	Merrimack Valley Health Enrollment Network has locations throughout the Merrimack Valley with health insurance experts on site to answer your questions, help you choose the best plan for you and your family, and complete the necessary forms	Directly mostly to low income individuals and families	Free	\N	9:00 A.M to 5:00 P.M	Monday to Friday	English!DELIM!Portuguese!DELIM!Spanish	Yes	Proof of income: 2 recent pay stubs\r\nProof legal status: passport or birth certificate, green card, asylum approval	01 Haverhill - Methuen - Lawrence	\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Contact respective agency	No	\N	Yes	Conference room for 20 people	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	Merrimack Valley Health Enrollment Network has partnered with the Latin American Health Institute, the Greater Lawrence Community Action Council and the Lowell Community Health Center to offer health insurance enrollment assistance at nine convenient locations in Lawrence, Lowell, Haverhill and Methuen.See web site for locations.	\N	No	0	\N
504367	101562	2007-07-01	\N	Greyhound Pets of America New Hampshire Chapter	GPA-CNHC	P.O. Box 778	\N	Concord	NH	03302	888-507-9597	888-507-9597	\N	\N	www.gpa-cnhc.org	public_program_email@testing123.net	Greyhound Adoption and fostering	Greyhounds placed through the GPA program usually are between two and five years old. These ex-racers are intelligent, quiet, clean, gentle, good-natured, adaptable dogs who adjust readily to their new home environment. They are eager to please and respond positively to any attention. They seem grateful for their new homes and regard their new owners with never-ending affection. They do not require a great deal of space to run, as many people think, but they do need exercise. Inside the house they enjoy curling up in a corner on a soft blanket. Greyhounds spend most of their lives in the company of other dogs, their trainers, and handlers. Greyhounds have all the advantages of breeding and training when it comes to being a good pet. Their sociability generally makes them great with other pets (including cats!). Their temperament makes them wonderful around children and friends. Their intelligence and breed make them perfect house pets, clean, quiet, and a loving companion. Whether you are a family looking for a playful friend, or an individual looking for a devoted companion, a well-mannered greyhound could be just the pet for you. Because greyhounds have been raised in kennels, a careful introduction into a suitable home environment is a must for successful placements.	Everyone is eligible to adopt a greyhound, or foster. A volunteer will do a short and friendly home visit. Continued support is offered.	Yes	Adoption fees cover the medical routine care of hounds and care and feeding. GPA is a non-profit organization	year round	by appointment	English!DELIM!Spanish	Yes	n/a		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	fostering, meet'n'greets, and many other needs. please see our website, www.gpa-cnhc.org for more information	Not Applicable	\N	Not Applicable	\N	Yes	view website	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	All of New Hampshire, Middlesex and Essex Counties in Massachusetts; some of Vermont, some of Maine	\N	Yes	4	\N
507766	101804	2008-11-17	\N	Lowell Housing Authority Congregate	Bishop Markham Village Congregate	174 South Street	\N	Lowell	MA	01852	978-364-5368	\N	\N	978-656-4665	www.lhma.org	public_program_email@testing123.net	Congregate Housing For The Frail Elderly/Disabled	The Congregate Building consists of 20 units on three floors with elevator/handicap access. \r\nEach efficiency unit includes a combination livingroom/bedroom, small kitchen and bathroom (with sink & toilet). Shower rooms are private and located in the common areas. The common areas include a main dining, kitchen and sitting room as well as a terrace/patio. Coin-operated laundry facilities as well as resident mailboxes are located on the first floor. The building is secured and accessed with a resident key card or via intercom. \r\nElder Services Case Management/Home Care Services are available for eligible residents. Lunch is served daily during the week for a nominal donation.	Contact the Director at 978-364-5368 for Application Requirements	Yes	Rent is calculated on 30% of one's income.\r\nHeat, hot water and electricity are included in the rent. The resident is responsible to pay for cabel and telephone if they so choose to subscribe.	Public Housing	Public Housing staff are available Monday through Friday from 8:30AM-4:30PM	English!DELIM!Spanish	Yes	Contact the Director at 978-364-5368 for Application Information	09 Downtown Circulator	On-street parking is available as well as city lot access. Handicap parking can be requested if necessary.	The program does not provide any type of transportation but can assist residents in locating resources for such.	Yes	Not Applicable	\N	Yes	If determined eligible, applicants are placed on a Waiting List until a vacant unit becomes available.	\N	21 Salem Street	P.O. Box  60	Lowell	MA	01853	Not Applicable	\N	Not Applicable	\N	Yes	A staff Conference Room is available in the building.	Yes	Monthly newsletters from the Council On Aging, Community Teamwork and the Salvation Army are posted on the bulleting board on the first floor.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
505137	101881	2008-07-29	\N	Beacon Hospice Volunteer Program	\N	350 Main Street	\N	Haverhill	MA	01830	978-372-4211	800-981-4805	\N	\N	www.beaconhospice.com	public_program_email@testing123.net	Become a Hospice Volunteer	No matter who you are, you've got something to offer to hospice patients.  Typically, the needs are simple- companionship and friendship, emotional support, someone to listen-but the small things you can do can make an enormous difference in someone's life.\r\n\r\nVolunteering takes many forms.  You can assist patients by reading, writing letters, playing music, or just sitting quietly and holding their hand.  You might help organize appointments, provide respite for a primary caregiver, or just walk the dog.  You could also offer your computer skills in our office and assist with special projects.	A comprehensive 24-hour hospice volunteer training is offered throughout the year and provided free of charge. The next training is scheduled to begin in Sept 2008 here at Beacon Hospice, 350 Main Street, Haverhill, MA. All are welcome and there are still spaces available. Please call with any questions or to register today!	Free	\N	open year round	Volunteers will see patients based on their own availability. Whenever is most convenient for both patient and volunteer. Whether the visit is for fifteen minutes or an afternoon, your presence will make a difference in someone's life.	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	\N	Yes	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	therrera@beaconhospice.com old email
500176	100033	2008-12-01	\N	Energy Programs	Fuel Assistance, weatherization, heating system repair	45 Kirk Street	\N	Lowell	MA	01851	978-459-6161	877-451-1082	\N	978-458-8594	www.comteam.org	public_program_email@testing123.net	Assist low-income families to maintain utilities	The Energy programs assist low-income families to maintain heat, lights, and other vital utilities in their homes.	Must meet income requirements. Please call 978-459-6161 for more information.	Free	\N	Call for more information	Monday-Friday 8:30AM-4:30PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Call for more information		Parking is available at Community Teamwork	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Yes	www.comteam.org	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Energy Programs serve all of Greater Lowell and the western suburbs of Waltham, Watertown, Lexington, Belmont, and Arlington.	\N	No	0	\N
501155	100165	2008-12-01	\N	Healthy Families	\N	126 Phoenix Ave	3rd Floor	Lowell	MA	01852	978-937-3087	\N	\N	978-937-8695	www.mspcc.org	public_program_email@testing123.net	Provides home visiting services to 1st time teen parents	The Healthy Families program provides home visiting services to 1st time parents (or expecting parents) age 20 and younger for the purpose of parenting education and support, information on bonding and child development. Child development screening, discipline techniques, connections to community services, medical services and other parents are provided as well. Services also include parent educational groups, support groups, prenatal and pregnancy groups and other activities.	Male or female first time parent age 20 or younger from Greater Lowell. This is a free and voluntary program.	Free	\N	Program runs year round. Parents enroll while pregnant/during 1st year of child's life. Services last until child turns 3 years old.	Monday-Friday 8:45AM-5:00PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	\N	02 Belvidere	\N	Parking available in lot. Located on LRTA bus route. Transportation is provided by the program for group activities only.	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
501364	100242	2008-12-01	\N	Adult Postsecondary Programs	\N	250 Pawtucket Blvd.	\N	Tyngsborough	MA	01879	978-441-4874	\N	978-454-5411	978-441-5388	www.gltech.org	public_program_email@testing123.net	Adult Post-Secondary Program includes certificate programs	The Adult Post-Secondary Program includes certificate programs and single course continuing education programs. The Practical Nursing program is the most active 10 month certificate program.	Contact for more information	Yes	Practical Nurse training:$5,500 in-district, $6,800 out of district. Payment plan is available. Financial aid available	PN: Sept-June each year.\r\nContinuing Education classes: Fall and Spring	Hours vary according to specific programs.	English	Yes	PN Program and Continuing education course registrations may be completed w/program secretary		Parking is available in lot next to building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Contact GLTHS for information about resource libraries.	Yes	Theater in the round style lecture hall. Function room adjacent to school's Artisan Restaurant. Contact school for information	No	\N	Yes	PN Program Brochure and Adult Continuing Education information is avaiable on the school's website. http://www.gltech.org/AdultEducation.htm	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
501298	100154	2008-12-01	\N	Lowell Regional Wastewater Utility	Duck Island	451 First St. Boulevard (Rt. 110)	\N	Lowell	MA	01850	978-970-4248	\N	978-970-4248	978-459-3826	\N	public_program_email@testing123.net	Provides for the collection and treatment of wastewater	The Lowell Wastewater Utility provides for the collection and treatment of wastewater generated by homeowners, businesses and industries in the City of Lowell and towns of Dracut, Chelmsford, Tewksbury and Tyngsboro. Wastewater personnel are responsible for cleaning and maintaining the City's catch basins and for cleaning and rodding City owned sewer lines. The Utility accepts septic from those homeowners who are not tied into the sewer system but have septic tanks.	Not applicable	Free	Not Applicable	Not Applicable	Facility operates 24 hours/day, 7 days/week; Maintenance personnel 7AM-3PM; Operations personnel 8AM-4PM	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
507810	102761	2008-12-01	\N	Mass Rehab Commission- Homecare Assistance Program	MRC-HCAP	27 Wormwood Street, Suite 600	\N	Boston	MA	02210	617-204-3853	1800-223-2559	\N	617-727-2809	www.mass.gov/mrc	public_program_email@testing123.net	Provides homemaking services to persons with disabilities	Provides paid homemaking services to persons with a disabling condition or illness, throughout Massachusetts. Homemaking services of laundry, light cleaning, grocery shopping, meal preparation and medication pickup. We do not provide heavy cleaning, personal care, companionship, assistance with errands or transportation. \r\nServices are provided either through a contract with one of our many contracted homemaking agencies or by a homemaker of the person's own choice, who can independently contract with MRC. \r\nWe allow a maximum of 12 hours per week of services per person. Most people receive an average of 3-5 hours a week of homemaking services. Scheduling is flexible and is organised between the consumer and the vendor or homemaker providing services.	Must be aged 18-59 years\r\nMust be a Massachusetts resident\r\nRequires homemaking assistance to prevent injury, hospitalization or institutionalization\r\nMust be living alone, or with another adult who is unable to performing homemaking tasks due to disability, or has dependent children under 18 years \r\nIncome cap varies annually. For 2008, applicant's income must not exceed $2600 gross per month. We allow $900 for each additional member of the household\r\nMust not be receiving or be eligible for a comparable state-funded benefits program such as PACE, the PCA program or the Veteran's Aid and Attendance Program - these programs provide duplicate services.\r\nMust not be legally blind - Mass Commission for the Blind provides their own homemaking services	Free	All services are fully funded by the Commonwealth.	Program office hours are 8:30-5pm. Consumers receive services according to their and vendor's available schedule.	Monday-Friday - 8:30-5pm.	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Yes	No	\N	Yes	Our waitlist is currently 4-5 months upon the date of approval. PLEASE NOTE it can take up to 1-3 months to get approved for services due to waiting on medical documentation, etc.	Application involves a 4 -stage process. Initial intake is done via telephone by calling the intake line at 617-204-3774. Referral source must have demographic information, income, medical info and an idea of what services they need. \r\nSecondly, an information packet and release forms are mailed to the consumer to sign and return. Proof of income is required. \r\nUpon receipt of these, a 2 page medical form is sent to the doctors specified. At least one form must be received by an MD, that verifies the applicant's need for homemaking services to prevent injury or hospitalisation\r\nThe last stage is to refer the applicant for a nurse evaluation to meet with the applicant, observe their living environment, needs, and an estimate of the amount of hours and types of homemaking services needed. Consumers are kept informed at all stages of the application as to the status of their application.	\N	\N	\N	\N	\N	No	Although college internships may be available	Yes	Information on our program is available on the MRC website at: www.mass.gov/mrc and can be mailed to interested parties.	Yes	Conference rooms on the 5th and 6th floor may be available for booking	Yes	We provide an annual newsletter,featuring articles contributed by MRC-HCAP staff, consumers and vendors. All consumers are welcome to contribute to the newsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	617-204-3853	\N	We have several staff who speak both English and Spanish. Our materials are available in both languages. We are in the process of translating our information into other languages also. If needed staff from other departments with other language abilities may be available for consultation.	No	0	\N
501375	100242	2008-12-01	\N	Technical Studies	\N	250 Pawtucket Blvd.	\N	Tyngsborough	MA	01879	978-441-4899	\N	\N	978-441-5344	www.gltech.org	public_program_email@testing123.net	Technical High school w/ 24 technical programs grades 9 - 12	Twenty four (24) technical programs are provided to students in grades 9-12 from the district: Lowell, Dracut, Tyngsboro and Dunstable.	Lowell, Dracut, Tyngsboro and Dunstable choice students are accepted.	Free	\N	Runs on public school schedule: September-June.	Monday-Friday 8AM-3PM	English	No	\N		Parking is available on campus.	Transportation provided by towns.	Yes	No	\N	Yes	There is a wait list for incoming 9th graders.\r\nTenth graders are accepted if there are openings.\r\nOnly transfers from other technical high schools are accepted at the 11th and 12th grades.	\N	\N	\N	\N	\N	\N	No	\N	Yes	Contact GLTHS for information about resource libraries.	Yes	Theater in the round style lecture hall. Function room adjacent to school's Artisan Restaurant. Contact school for information	Yes	Parent Newsletter\r\nBusiness/Community Newsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
504323	101529	2008-12-01	\N	Community Health Education Center	CHEC	144 Merrimack St	2nd Floor	Lowell	MA	01826	978-452-0003	978-452-0003	\N	978-452-6602	\N	public_program_email@testing123.net	Outreach Educator Training Facility	CHEC specializes in providing cross training, education and professional development opportunities for local /or regional outreach educators and their supervisors.  As adult learners, over 500 Outreach Educators participate in various learning tracks to acquire or broaden their scope of knowledge, skills and cultural competence.  We offer a comprehensive certificate program, community health education, health modules, seminars and workshops.  We also facilitate monthly networking luncheons and publish a biannual newsletter in support of our efforts to provide interactive services.  Outreach Educators exchange respective information while connecting with other community resources, learn more about current public health issues and establish peer support.	People who are employed as outreach educators.	Free	No fees involved. ALL trainings are free of charge. Use of library is also free to Outreach and Health Educators.	Office opened year round. Trainings conducted from September to July.	M-F 8:30am-4:30pm	English!DELIM!Spanish	No	\N		Local Parking lots, average 1 dollar and hour. We cannot Validate.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Library with Health Topic information. Books, videos. CDC information sheets are also available.	Yes	Large meeting space. Can fit 30 people comfortably.	Yes	We do and annual newsletter with staff updates, Current health updates, and outreach educator profiles.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
503245	100484	2008-12-01	\N	Middlesex Mediation Programs	Middlesex Community College Law Center	88 Middle St.	\N	Lowell	MA	01852	978-656-3342	\N	\N	978-441-1749	http://www.middlesex.mass.edu/lawcenter/	public_program_email@testing123.net	Provides free consumer protection services through telephone	The MCC Law Center is funded by the Massachusetts Office of Attorney General and provides free consumer protection services including education and mediation of individual consumer complaints against local, national and international businesses, agencies and governments. Since 1990 over 10,000 cases have been mediated with savings to consumers of more than $1.5 million dollars. \r\nPrivate mediation services are also offered on a sliding scale basis. \r\nThe MCC Law Center is approved by the Massachusetts Superior Court and the District Court trial departments to provide mediation services in civil and criminal cases.\r\nMediation training and conflict resolution courses are offered three times per year.\r\nSince 1997 the Law Center, under contract with the US State Department, has provided conflict resolution training in Cambodia at the national university.	Services available to all in the greater Merrimack Valley area	Free	\N	Monday-Friday 9AM-4PM	Monday-Friday 9AM-4PM	English!DELIM!Portuguese!DELIM!Spanish	Yes	Individuals must call or go online for a complaint form and return it by mail or in person before mediation will be started.		Parking available in Roy Garage on Market Street, Lowell, MA	\N	Yes	No	\N	Not Applicable	\N	\N	33 Kearney Sq.	\N	Lowell	MA	01852	Yes	Qualified MCC students may enroll in the Mediation Internship Course and volunteer in the Consumer Protection Program. Other propspective volunteers will be interviewed by the Director if they have experience in mediation services.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA!DELIM!Wilmington, MA	\N	We also serve the following towns: Ashby, Ayer, Bolton, Burlington, Fitchburg, Groton, Harvard, Lancaster, Leominster, Lunenburg, North Reading, Pepperell, Reading, Shirley, Townsend, Wakefield, Wilmington and Woburn.	\N	No	0	Kevin Dunn is Director -- dunnk@middlesex.mass.edu\r\n\r\nDeborah Walsh is Coordinator
504499	100033	2008-12-01	\N	Division of Community Resources	\N	167 Dutton Street	\N	Lowell	MA	01852	978-459-0551	800-698-0551	\N	978-453-9128	www.comteam.org	public_program_email@testing123.net	Child Care Resource/Referral, Senior Volunteer Corps, etc	Child Care Resource/Referral,intake for child care subsidies, Senior Volunteer Corps, Immigrant Services, etc	varies by program type	Yes	\N	varies by program	varies by program	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Not Applicable	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	0	\N
503212	100484	2008-12-01	\N	Adult Learning Center	\N	591 Springs Rd. Bldg. 9	\N	Bedford	MA	01730	781-280-3662	\N	\N	781-275-7126	www.middlesex.mass.edu	public_program_email@testing123.net	Provides free classes in literacy, basic ed., GED	The Adult Learning Center provides free classes funded by the Massachusetts Department of Education in Literacy, Adult Basic Education and GED preparation.	Open to students 16 and older. Contact program for further information.	Free	\N	Classes run September through early June. Program capacity is 44, there may be a short waiting list, usually under a month	Program runs Tuesday, Wednesday, Thursday 9AM-12PM; Monday and Friday only the office is open	English	Yes	If under 18, please provide letter of withdrawal from high school.	13 Billerica via Edson!DELIM!14 Burlington Mall / Lahey Clinic	Parking available in student lots.	Public transportation available to nearby locations which are serviced by the MCC shuttles	Yes	No	\N	Yes	Occasionally, there is a brief waiting list.	\N	\N	\N	\N	\N	\N	No	\N	Yes	\N	Yes	Meeting space is available when space not occupied. Contact Molly Sheehy, 978-656-3105 for information	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
504411	100484	2008-12-01	\N	Links	\N	591 Springs Rd.	\N	Bedford	MA	01730	781-280-3654	\N	\N	781-275-7126	www.middlesex.mass.edu	public_program_email@testing123.net	Transition to college program	Transitioning students are supported through a grant designed for GED grads and those holding a high school diploma.  Applicants must take MCC placement test and be eligible for either Basic Writing or Algebra I.	16 years old and up, priority is given to GED completers from DOE funded program.	Free	\N	New classes start in September and January.	Tuesday and Thursday from 9:00 a.m. until \r\n3:00 p.m.	English	Yes	Call Ruth Clark, Program Coordinator\r\n781-280-3654	13 Billerica via Edson!DELIM!14 Burlington Mall / Lahey Clinic	Student parking available	College shuttle from/to Billerica Mall	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	\N	Yes	Meeting space is available when space not occupied. Contact Molly Sheehy, 978-656-3105 for information	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
500770	100077	2008-12-01	\N	Community Media Project	Thinking Out Loud	Center for Family, Work and Community	600 Suffolk Street	Lowell	MA	01854	978-458-5400	\N	\N	978-937-0361	www.thinkingoutloud.us	public_program_email@testing123.net	Public Affairs Radio in 5 languages + Cable TV (English)	The Thinking Out Loud Media Project brings multi-cultural community-based public affairs radio in five languages to the Merrimack Valley area. Our mission is to make a connection between the larger issues that affect our society and people's day-to-day lives, provide media access to the area's under served communities and overcome the limitations of commercial broadcasting by raising issues you don't hear elsewhere. We address our core subjects of the community, the workplace and the environment on UMass Lowell's FM station WUML-91.5. We address these issues by way of live interviews, taped features of local activists involved in those fields and by recorded public affairs features from progressive commentators and thinkers from around the United States and the World. Our volunteers consist of UMass Lowell staff, students and community members including more than thirteen from Lowell Southeast Asian, Latino and Brazilian communities. Our board is made up of representatives from the Lowell community,labor, environmental groups, grassroots radio activists, social service and faith based organizations.  \r\nThinking Out Loud is now airing a 1/2 hour show on Lowell cable channels 8 & 95.	Not Applicable	Free	\N	Program runs year round	Monday through Friday 10:00 AM - 12:00pm	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	\N		Somewhat difficult because of limited parking. Guests can call ahead on the day of their interview for help to get a parking space.	\N	Yes	No	\N	No	\N	English only is spoken on the phone	Lowell Telecommunications Corporation	246 Market Street - P.O. Box 803	Lowell	MA	01852	Yes	We have an average of 25 volunteer co-hosts throughout the year.  We don't have any openings at the time of this writing but we do have openings for new volunteers from time to time.  People Interested in Volunteering should contact Dan Toomey	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Thinking Out Loud radio airs in and beyond the Merrimack Valley area of northeastern Massachusetts and southern New Hampshire.  There are potentially 2.5 million listeners within our signal range.\r\nThinking Out Loud TV is presently limited to Lowell	Laotian also	No	0	\N
500792	100077	2008-12-01	\N	GEAR-UP	\N	Center for Family, Work and Community	600 Suffolk Street, First Floor	Lowell	MA	01854	978-934-4667	\N	\N	978-934-3026	www.uml.edu/centers/cfwc	public_program_email@testing123.net	Help middle and high school students prepare for college	GEAR-UP stands for Gaining Early Awareness and Readiness for Undergraduate Programs. It is a federally funded program designed to help middle and high school students prepare for college that works in partnership with the University of Massachusetts - Lowell, Middlesex Community College, the Lowell Public Schools, parents, businesses, and religious and non-profit groups to increase the likelihood of more children pursuing post-secondary education. A resource library is available with college awareness and readiness resources, information on colleges and universities throughout the state, parent training video, etc.	All 10th grade students (2008-2009) at Lowell High School.	Free	\N	Gear Up runs year round	Monday-Friday 9:00AM-5:00PM\r\nafter school, evening, weekend programs	English!DELIM!Portuguese!DELIM!Spanish	Yes	\N		\N	Transportation is provided by the program. M	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer Mentors are used at Gear Up. Call 978-934-4667 for more information	Yes	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
500803	100077	2008-12-01	\N	Program Evaluation	\N	600 Suffolk Street	First Floor	Lowell	MA	01854	978-934-4677	\N	\N	978-934-3026	www.uml.edu/centers/cfwc	public_program_email@testing123.net	Process and outcome measures for local organizations	Program evaluation provides services to local organizations, both qualitative and quantitative process and outcome measures and processes.	Call for more information 978-934-4677	Yes	Call for more information, this is a fee based service	Services are available year round	Monday-Friday 9:00AM-5:00PM	English	Yes	Call with information about the program or project needing an evaluation		Limited parking is available in the Visitor's lot	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Limited space is available, call 978-934-4677 to make arrangements	No	\N	Yes	community computer lab mainly for high school students	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
507799	100077	2008-12-01	\N	Science of Small Things	\N	600 Suffolk Street	First Floor	Lowell	MA	01854	978-934-4779	\N	\N	978-934-3026	www.uml.edu/centers/cfwc	public_program_email@testing123.net	Afterschool science based program for 7-10 graders	Youth participants discover what nanotechnology is all about through hands on activities, experiments and field trips to local companies.	7th and 8th graders must be enrolled in the Robinson Middle School Citizen Schools Program or go to the Butler Middle School. Two other middle schools and the high school will be added in 2009	Free	\N	Two 8 week afterschool sessions and a summer program	At the Robinson, Citizen Schools runs afterschool Mon-Thursday, the Science of Small Things is run on one of those days. The Butler school program is run after school on Tuesdays.	English	Yes	For Robinson students register through Citizen Schools. For Butler and other schools look for flyers to learn about registering. No requirements other than parental permission.		\N	transportaion is porvided for middle schoolers	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Limited space is available, call 978-934-4677 to make arrangements	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
500825	100077	2008-12-01	\N	River Ambassador Program	RAP	Lowell Telecommunications Corp	246 Market Street PO Box 803	Lowell	MA	01852	978-458-5400	\N	\N	978-937-0361	http://www.lowellrap.org	public_program_email@testing123.net	After school environmental training, events & leadership	The River Ambassador Program is volunteer-based youth program supported by LTC, the Center for Family, Work and Community at the University of Massachusetts Lowell and the National Parks. It is now in it's tenth year of bringing youth together with the University, Parks and community groups to address environmental health concerns in Lowell.  Over 250 high school students have participated throughout the years. The program not only provides project-based hands-on activities on environmental justice, but also give students the opportunity to develope and create their own environmental activities. In addition, the students also participate in local community events to promote and exchange ideas about the environment.	Open to high school students. Numbers are limited, but there are no eligibility requirements. Program capacity is 20 to 25.	Free	\N	Program runs year round, most activities take place during the school year.	After school hours	English!DELIM!Khmer	Yes	Call 978-458-5400 for more information from Dan Toomey		Limited parking near LTC	Some transportation is provided for special events	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
505489	100055	2008-12-02	\N	Al-Anon/ Alateen	\N	295 Varnum Ave.	\N	Lowell	MA	01854	508-366-0556	\N	508-366-0556	\N	http://www.ma-al-anon-alateen.org/	public_program_email@testing123.net	Fellowship of relatives and friends of alcoholics	Al-Anon/ Alateen is a fellowship of relatives and friends of alcoholics who believe their lives have been affected by someone else's drinking.  Meetings for beginners are Fridays from 7 to 8 p.m. in Hanchett Auditorium at Lowell General Hospital.  Regular meetings are from 8 to 9:30 p.m. in Hanchett Auditorium.  Please call the Al-Anon/Alateen hotline at 808.366.0556 or visit http://www.ma-al-anon-alateen.org/ if you have questions regarding this group or other meetings and general information	relatives and friends of alcoholics	Free	\N	Offered Year Round	Meetings for beginners are Fridays from 7 to 8 p.m. in Hanchett Auditorium at Lowell General Hospital.  Regular meetings are from 8 to 9:30 p.m. in the Groton Room.	English	No	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the front of the hospital	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
500418	100055	2008-12-02	\N	Arthritis Support Group	\N	Lowell General Hospital - Clark Auditorium	295 Varnum Ave.	Lowell	MA	01854	978-454-5358	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support & educational group for arthritis sufferers	The Arthritis Support Group provides a supportive group environment for arthritis sufferers to learn and share information to better deal with their illness. For more information, call 978-454-5358.	Open to anyone with arthritis	Free	\N	Year round except for the summer months.	Meets the second Friday of every month (except July and August) from 8AM-9:30AM in the Clark Auditorium at Lowell General Hospital.	English	No	Please call group leader Cissy at 978-454-5358 for information	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator.  Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506677	100055	2008-12-02	\N	A.W.A.K.E (Alert, Well, and Keeping Energetic)	A health awarenss group for people affected by Sleep Apnea	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	A health awareness group for people affected by sleep apnea	This health awareness group is for people with sleep apnea, their family members, and friends.  The group will be part of the American Sleep Apnea Association (ASAA) A.W.A.K.E. Network.\r\n\r\nThe purpose of these sessions is to provide support through education and sharing of ideas and information among persons who are affected by sleep disordered breathing.  Together we can help each other with ideas, problems, and solutions.\r\n\r\nWe will have a meeting every three months.  There will be a TOPIC of interest, vendors from homecare companies, refreshments and support and ideas for dealing with wearing the CPAP mask.\r\n\r\nFor more information or to register, please contact Cyndy Michaels at (978) 788-7158.	Open to all	Free	\N	Open year round	The support Group will meet every 3 months...\r\nFor more information or to register, please contact Cyndy Michaels at (978) 788-7158.	English	Yes	Please contact Cyndy Michaels at \r\n(978)788-7158 or CMichaels@lowellgeneral.org for more information or to register.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the main entrance to the hospital.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500429	100055	2008-12-02	\N	Babysitting Basics	\N	Lowell General Hospital	295 Varnum Avenue	Lowell	MA	01854	1-877-544-9355	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Baby sitting classes for boys & girls ages10 and up	This is a one-session course designed to help develop babysitting skills for boys and girls ages 10 and older. Topics include: basic safety issues, first-aid, growth and development, baby-sitter activities, bottle-feeding, bath time and bed time, care and handling of infants, and more. Saturdays, 9:00am-1:00pm.	Open to boys and girls ages 10 and older	Yes	$35 Fee. May register on-line at www.lowellgeneral.org, using credit card, or register by phone at 877-544-9355, and bring check to class made out to Lowell General Hospital.	Offered on one Saturday bi-monthly from 9AM-1PM in the Hanchett Auditorium, Lowell General Hospital	Normally offered one Saturday on a bi-monthly basis from 9AM-1PM, Call 1-877-LGH-WELL (1-877-544-9355)for more information or to register.	English	Yes	Visit www.lowellgeneral.org or call 877-LGH-WELL (877-544-9355)to register; Bringing a snack or lunch on day of class is recommended	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital	\N	Yes	No	\N	No	\N	Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
505687	100055	2008-12-02	\N	Babysitting Basics Plus CPR Anytime	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Babysitting course for boys and girls ages 10 and older	This is a one-session course designed to help develop babysitting skills for boys and girls ages 10 and older. Topics include: basic safety issues, first-aid, growth and development, baby-sitter activities, bottle-feeding, bath time and bed time, care and handling of infants, and more. Each participant will also receive a CPR Anytime Kit for them to use to practice CPR as well as to use to bring home and teach family/friends CPR. Kit includes practice mannequin, DVD and booklet. 9:00am-1:00pm.	Available for boys and girls ages 10 and older	Yes	$60.00 Fee includes a CPR Anytime Kit for them to use to practice CPR as well as to use to bring home and teach family/friends CPR. Kit includes practice mannequin, DVD and booklet.	Open year Round	Offered 9:00am-1:00pm on a Saturday on a bi-monthly basis	English	Yes	Call 1-877-LGH- WELL (1-877-544-9355) for more information or to register.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the front of the hospital at the main entrance.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters are available upon request.	Yes	0	\N
505896	100055	2008-12-02	\N	Birthplace Tour	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Free tour of the Birthplace at Lowell General Hospital	Birthplace Tours\r\n\r\nAre you expecting?  Come to Lowell General Hospital for a FREE tour of the Birthplace provided by a child birth educator who will answer all of your questions. Tours are offered about 3-4 times each month on Saturdays at 11:00am and Thursdays at 7:00pm	Open to all expecting families	Free	\N	Open year round	Tours are offered the first Saturday of each month at 11:00am and the third Thursday of each month at 7:00pm	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Free parking is available at the main entrance to the hospital	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters are available upon request	Yes	0	\N
505918	100055	2008-12-02	\N	Boot Camp for New Dads	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Boot Camp prepares men to be dads in all respects	Boot Camp For New Dads is a three-hour workshop taught by fathers. Utilizing a man-to-man training approach, veteran dads and their newborns orient rookie dads who are expecting their first baby. Boot Camp prepares men to be dads in all respects, beginning with holding and comforting a real baby. The program fee is $40 and includes Hit the Ground Crawling: The Essential Guide For New Fathers, a book by Greg Bishop, founder of Boot Camp for New Dads.\r\n\r\nBoot Camp meets the 4th Saturday of every other month 9:00am-12:00pm in the Lowell Room at Lowell General Hospital.	Open to expecting fathers	Yes	The program fee is $40 and includes Hit the Ground Crawling: The Essential Guide For New Fathers, a book by Greg Bishop, founder of Boot Camp for New Dads.	The program will be offered on a bi-monthly basis throughout the year	Boot Camp meets the 4th Saturday of every other month 9:00am-12:00pm in the Lowell Room at Lowell General Hospital.	English	Yes	For more information or to register please call 1-877-LGH-WELL (1-877-544-9355) or visit lowellgeneral.org.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Free Parking is available at the main entrance to the hospital	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
500484	100055	2008-12-02	\N	Childbirth Education Classes	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6190	1-877-544-9355	978-937-6000	\N	www.lowellgeneral.org	public_program_email@testing123.net	Wide range of Child Birth Education classes	Lowell General Hospital invites you to participate in a range of Child Birth Education classes including: Six-Week Prepared Childbirth Education; Six week Hypnobirthing Class; Childbirth Education Refresher Class; One-Day Childbirth Education Class; Breastfeeding Class; Prenatal Yoga and Big Brother/Sister Sibling Classes, tours of the Birthplace, and much more. All this to help the entire family prepare for its newest member. Days and hours vary. To view upcoming class schedule visit our website at www.lowellgeneral.org and click on "Attend a Class/Event." For more information or to register, please call 1-877-LGH-WELL (1-877-544-9355)	Open to anyone	Yes	Fees vary. For more information or to register please call 1-877-LGH-WELL (1-877-544-9355) or visit lowellgeneral.org	Runs continuously through the year. Childbirth classes reach maximum enrollment 3-4 months prior to start; register early	Dates and Hours vary, please call 1-877-LGH-WELL for more information or visit our lowellgeneral.org to see a class schedule.	English	Yes	Call 1-877-LGH-WELL (1-877-544-9355) for more information.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request.	Yes	0	\N
500506	100055	2008-12-02	\N	Celebrating our Special Gifts: A Down Syndrome Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	1-877-544-9355	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support Group for parents with children with Downs	Celebrating Our Special Gifts: A Down Syndrome LGH Support Group provides support and education for the parents of children with Down syndrome. Parents encourage and strengthen each other, sharing experiences, knowledge and understanding, to enrich the lives of their children so they may reach their fullest potential. Meets the 3rd Monday of each month, from 7 - 9 p.m. in the Clark Auditorium at Lowell General Hospital. Parents of children of all ages are invited. (Children are welcome, too.) For more information, please call Julie at 978-788-7078.	Group open to all parents of children with down syndrome. Children welcome as well.	Free	\N	Program is offered year round.	3rd Monday of every month 7PM-9PM in Clark Auditorium at Lowell General Hospital.	English	No	Call 1-877-LGH-WELL (1-877-544-9355) for more information	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking available in front of hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
504125	101441	2008-12-02	\N	Case Management	DMH	391 Varnum Avenue	\N	Lowell	MA	01854	978-322-5000	\N	978-322-5120	978-322-5077	\N	public_program_email@testing123.net	Case Management	Case Management provides a comprehensive mental health assessment, individual service planning, and coordination of services for eligible clients.  After reviewing available community services, the case manager will connect each client with the most effective services possible based on the outcome of the assessment and client preferences.\r\nCase managers not only advocate for services but are also with clients on their journey toward rehabilitation and recovery.	Qualifying mental illness; if a child, adolescent or adult is determined eligible, services will be offered based on the applicant's needs, preferences, and available resources	Yes	\N	Closed on Holiday's	Mon - Fri 9 am - 5 pm	English!DELIM!Spanish!DELIM!Khmer	Yes	\N		Yes, behind building	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	Sign language	No	0	\N
505742	100055	2008-12-02	\N	CPR, AED & First Aid Combo Class	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	BASIC CPR/AED/FIRST AID 6 hour combo class	6 hour combo class for participants to become certified in BASIC CPR/AED/FIRST AID. This class does include Child and Infant CPR as well as Adult CPR. Co-sponsored by LGH and Trinity EMS. Saturdays, 10:00am-5:00pm. $45.00.\r\n\r\n\r\nFor future dates and to register call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org.	Open to all	Yes	$45.00	Open Year Round	Please call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org for more information or to regsiter.	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the front of the hospital.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request.	Yes	0	\N
505467	100055	2008-12-02	\N	Yoga of the Heart	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6396	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Stress reduction and strengthening through yoga	Yoga of the Heart: A system of learning deep breathing through imagery and movement.  Increase immunity and decrease pain while inducing deep relaxation. This course consists of 7 weekly classes. Classes run twice a year in the spring and fall.\r\nFee $ 125.00  Some scholarship money may be available for LGH Cancer Center patients.   For questions or to register please call 1-877-LGH-WELL (1-877-544-9355)	Anyone interested, especially people dealing with heart disease, cancer, and other debilitating health issues.	Yes	For information and registration call 1-877-LGH-WELL (1-877-544-9355). Space is limited	Offered year round	For future dates and times please call (1-877-LGH-WELL (1-877-544-9355) 978). Space is limited	English	Yes	For information and registration call (978)937-6396.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the hospital	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
506886	100055	2008-12-02	\N	Yoga for Mothers	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Postnatal Yoga	This series is intended for the mother who has recently given birth (from 8 weeks to 24 months ago) and would like the benefits of yoga to help with relaxation and to get her body back in shape. The series is 7 weeks long and permission from a physician is required. These workshops are presented as a collaboration between Lowell General Hospital and Family Foundations Network. Each class is 1 hour long. Instructor is Lynn Vesey. \r\n\r\nMondays, 7:00-8:00 pm, Hanchett Auditorium\r\n\r\n\r\nFor future dates and to register call 1-877-LGH-WELL (1-877-544-9355) or visit       www.lowellgeneral.org	This series is intended for the mother who has recently given birth (from 8 weeks to 24 months ago)	Free	\N	Runs continuously through the year.	For more information or to register call 1-877-LGH-WELL or visit lowellgeneral.org	English	Yes	Please call 1-877-LGH-WELL (1-877-544-9355) or visit lowellgeneral.org for more information or to register.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the front of the hospital, near the main entrance	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
506963	100055	2008-12-02	\N	Healing Through Art- Art Therapy	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6142	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Healing Through Art- Art Therapy	Healing Through Art - Art Therapy\r\nThis program offers cancer patients and their caregivers a creative way to work through their cancer journey. Join others for painting class with Sheri, a professional artist, teacher, and cancer survivor. No prior art experience necessary. Come for one class or the entire session. For more information or to register for this program, call Meg Lemire at 978-937-6142.	For cancer patients and caregivers	Free	\N	Will be offered periodically	For more information or to register for this program, call Meg Lemire at 978-937-6142.	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Adequate Parking will be available	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506952	100055	2008-12-02	\N	Exercise to Wellness	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6142	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Exercise to Wellness	Exercise to Wellness\r\nThis program is for cancer patients who have completed their cancer treatment at Lowell General Hospital and wish to begin the healing process through exercise. We offer this through a partnership with Fitcorp Health Center. The program is tailored to your needs. For questions or to register, please call Meg Lemire at 978-937-6142.	Open to cancer patients who have completed their cancer treatment and wish to begin the healing process through exercise.	Free	\N	Open Year Round	For questions or to register please call Meg Lemire at  978-937-6142	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Adequate parking is available	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506974	100055	2008-12-02	\N	Therapeutic Cancer Massage	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Therapeutic Cancer Massage	Therapeutic Cancer Massage\r\nThis program is offered to any Lowell General Hospital Cancer Center patient in treatment. Massage is a healing touch that applies pressure on the muscles of the body. Benefits of massage include an enhanced feeling of well-being, less muscle tension, improved blood and lymph circulation, lower blood pressure, increased range of motion, less anxiety, relief from depression, and less trouble sleeping. We offer therapeutic cancer massage through our partnership with Body Destinations. For questions or to register please call Meg Lemire at 978-937-6142.	This program is offered to any Cancer Center patient in treatment.	Sliding Scale	\N	Open Year Round	For questions or to register please call Meg Lemire at  978-937-6142	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Adequate parking will be available	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500539	100055	2008-12-02	\N	Guided Imagery	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6396	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Mental process that is used to changes in attitudes...	Guided imagery is the conscious use of the power of the imagination to encourage change in perceived ideas about ones state of mind or illness. Guided imagery can be used to activate physiological, psychological and/or spiritual healing. It is used to manage pain, stress, and medical procedures and chemotherapy related discomfort. \r\n\r\nPrivate sessions are available free of charge, by appointment only, for Lowell General Hospital Cancer Center patients. Please call Carole Legro at 978-937-6396 for an appointment.\r\n\r\nAbout the Instructor\r\nCarole Legro, MA, RNC, LMT is a body-oriented psychotherapist and Reiki Master with over 20 years experience in Reiki. Carole has completed training as a mind body educator at the Center for Mindfulness at University of Massachusetts Medical Center and at the Mind Body Institute at Beth Israel Deaconess Hospital.	Private sessions are available free of charge to Lowell General Hosptial Cancer Center patients only	Sliding Scale	\N	Program runs year round	By appointment only. Call (978)937-6396 to make an appointment.	English	Yes	Call program for information 978-937-6396. Appointments are required	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the Cancer Center lot in front of the Hospital	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505456	100055	2008-12-02	\N	Meditation Instruction	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6396	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Self-directed practice for relaxing the body and mind	Originally based in spiritual traditions, meditation is simply the practice of focusing and concentrating ones attention and awareness while maintaining a passive attitude. Meditation is known for its health benefits that include decreased pain, relaxation, calming the mind and accessing deep inner resources for health and healing. \r\n\r\nInstruction provided by appointment only for Lowell General Hospital Cancer Center patients. Please call Carole Legro at 978-937-6396 for an appointment.\r\n\r\nAbout the Instructor \r\nCarole Legro, MA, RNC, LMT is a body-oriented psychotherapist and Mind Body Educator with over 25 years professional experience. Carole has completed training at the Center for Mindfulness at University of Massachusetts Medical Center and at the Mind Body Institute at Beth Israel Deaconess Hospital.	For LGH Cancer Center patients only.	Free	\N	Offered year round	To schedule and appointment call Carole Legro at (978)937-6396	English	Yes	Call Carole Legro at (978)937-6396	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
507788	100055	2008-12-02	\N	Yoga	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Yoga	Reduce stress and muscle tension, learn to improve posture and flexibility, breathe more effectively and strengthen your entire body. Taught by Kripalu-certified instructor, Nancy Dorenfeld.\r\n\r\nThe program is offered Tuesday evenings from 6:00-7:15pm.  \r\n\r\nFor dates and other information, please call 1-877-LGH-WELL (877-544-9355).	Open to anyone interested (not for expectant mothers).	Yes	Cost is $90	Open year round	Tuesdays, 6:00-7:15pm	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
507722	100055	2008-12-02	\N	Prepare for Surgery, Heal Faster	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Learn how to be prepared for surgery to benefit your health	An analysis of 191 research studies with 8600 patients, documents that people who prepared for surgery had less pain, fewer complications and recovered sooner.\r\n\r\nIn this program, you will learn Peggy Huddleston's Five Steps to Prepare for Surgery:\r\n\r\nCalm preoperative jitters, guided by the relaxation CD or tape. Feeling peaceful strengthens your immune system and creates biochemistry that enhances healing.\r\nVisualize your recovery by turning worries into healing imagery.\r\nSurround yourself in the love of family and friends to feel calmer before surgery.\r\nUse "Healing Statements," words spoken during surgery that reduce the use of pain medications by 23-50%.\r\nEstablish supportive doctor-patient relationships.\r\n\r\nIdeally you will read the book and use the CD or tape one or two weeks before your operation. This workshop includes the book "Prepare for Surgery, Heal Faster" and its companion Relaxation/Healing CD or audiotape.\r\n\r\nThe workshop is $50, but a friend or family member may accompany you for free.	Open to anyone who is interested	Yes	The workshop is $50.00, but a friend or family member may accompany you for free	Offered periodically throughout the year	Call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org for more information	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request.	Yes	0	\N
505764	100055	2008-12-02	\N	Family & Friends  CPR	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Program that teaches life-saving skills	Participants will learn to recognize the signs and symptoms of heart attack, cardiac arrest and stroke. This program is designed for lay rescuers, family members of patients at risk for sudden cardiac arrest and those who want to learn life-saving skills. Co-sponsored by LGH & Trinity EMS	Open to all	Free	$15.00	Open Year Round	Dates and hours vary. Call 1-877-LGH-WELL for more information or visit our website to see a class schedule.	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the front of the hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request.	Yes	0	\N
506028	100055	2008-12-02	\N	Freedom From Smoking	Smoking Cessation Class	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	1-877-544-9355	\N	www.lowellgeneral.org	public_program_email@testing123.net	7-week Smoking Cessation Class	Freedom from Smoking is a smoking cessation program offered through the American Lung Association. This is an eight-week program that gives attendees the tools needed to end dependency on smoking and improve health and wellness. The class is offered for adults age 19 and older. Fee is $30, which is fully refundable after graduation of the course. You must complete the course to receive the refund.	The class is offered for adults age 19 and older.	Yes	Fee is $30, which is fully refundable after graduation of the course. You must complete the course to receive the refund.	Offered on a quarterly basis throughout the year.	For more information or to register please call 1-877-LGH-WELL (1-877-544-9355) or visit lowellgeneral.org	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the front of the hospital at the main entrance	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request.	Yes	0	\N
506941	100055	2008-12-02	\N	G.O.A.L.- Go On And Live	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Social group for those affected by Cancer	This is a social group for patients and family members who have been affected by cancer in some way. They meets every 6-8 weeks for various group events and outings. For questions or to register Karen at 978-453-4363.	for patients and family members who have all been affected by cancer in some way.	Free	\N	Open Year Round	They meet about every 6-8 weeks for various group events and outings.  For questions or to register please call Karen at 978-453-4363	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the main entrance to the hospital or on the side of the Cancer Center	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500594	100055	2008-12-02	\N	Look Good, Feel Better	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6142	\N	978-937-6000	\N	www.lowellgeneral.org	public_program_email@testing123.net	Cope w/ appearance-related side effects of cancer treatment	"Look Good Feel Better" is a program of the American Cancer Society that helps women undergoing cancer treatment learn to cope with the appearance-related side effects of treatment, and regain a sense of self-confidence and control of their lives. The group meets the first Monday of the month at 7:00pm. For more information, please call Leigh at the American Cancer Society at 781-314-2632.	Women undergoing cancer treatment	Free	\N	Open Year Round	Meets the first Monday of selected months at 7:00PM.	English	Yes	For questions call Colleen at 978-937-6519.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the Cancer Center lot in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	Cancer Center - Lowell General Hospital	Attn: Meg Lemire, 295 Varnum Ave.	Lowell	MA	01854	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500583	100055	2008-12-02	\N	Living with Breast & Gynecological Cancer Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6142	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Breast and GYN Cancer Support, Education & Resource Group	Living With Breast and GYN Cancer\r\nMeets the third Tuesday of the month at 7:00pm. A support group offering education, and resources for women with Breast or Gynecological Cancers. Support persons are encouraged to attend. You may register by calling 978-937-6142.	Open to women with Breast or Gynecological Cancer and support persons	Free	\N	Meets year round. Enrollment is ongoing.	Third Tuesday of every month at 7PM	English	Yes	Call 978-937-6142 for more information	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the Cancer Center Parking lot in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator.  Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	Cancer Center - Lowell General Hospital	Attn: Meg Lemire, 295 Varnum Ave.	Lowell	MA	01854	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500737	100055	2008-12-02	\N	"US TOO" Prostate Cancer Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6142	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Education and Resources group for men with Prostate Cancer	"US TOO" Prostate Cancer Support Group\r\nMeets the second Tuesday of the month at 6:30pm. A group co-sponsored by the American Cancer Society that offers support, education, and resources for men with Prostate Cancer. Newly diagnosed patients and support persons are encouraged to attend. You may register by calling 978-937-6142.	Men with Prostate Cancer (and support persons)	Free	\N	Meets second Tuesday of every month at 6:30PM year round. Enrollment is ongoing.	6:30PM second Tuesday of every month	English	Yes	Call 978-937-6142 for information	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking available in the Cancer Center Parking Lot in front of Hospital	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator.  Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500572	100055	2008-12-02	\N	Patients and Families Living with Cancer	\N	Lowell General Hospital	295 Varnum Ave.	Lowell	MA	01854	978-937-6142	\N	978-937-6000	\N	www.lowellgeneral.org	public_program_email@testing123.net	For patients living with a life threatening disease	Patients/Families Living With Cancer\r\nMeets the second Wednesday of the month at 6:30pm. A general support group for patients and their families and friends living with cancer. You may register by calling 978-937-6142.	Open to all cancer patients and their families.	Free	\N	Enrollment is ongoing.	Meets the second Wednesday of the month at 6:30 p.m.	English	Yes	Register by calling, 978-937-6142.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the Cancer Center Parking lot in front of the Center	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	Cancer Center - Lowell General Hospital	Attn: Meg Lemire, 295 Varnum Ave.	Lowell	MA	01854	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
504763	100055	2008-12-02	\N	Caring for a Loved One	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6129	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	A Support Group for the Caregiver	Meets the second Monday of the month at 6:30pm. A support group for caregivers to help them cope with the stress involved in providing care to a loved one while attending to all other family demands. How does one "step up to the plate", attend to all family demands and not lose oneself? We'll talk about this and other related issues. To register please call 978-937-6129.	Open to all caregivers.	Free	\N	Meets year round.	Meets second Monday of the month at 6:30 p.m.	English	Yes	For more information or to register, please call 978-937-6129.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the Cancer Center	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505412	100055	2008-12-02	\N	What About Me...?	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6129	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support group for adolescents ages 11-13 affected by cancer	A support group of adolescents ages 11 to 13 who have been affected by cancer in their family. This is a time to talk about the losses and changes in the home since cancer. Learn about cancer and share with other kids. The group meets the second Monday of each month from 3:30-4:30pm. This group is for bereavement and support. To register please call Cammie Caron at 978-937-6129.	Adolescents ages 11 to 13 who have been affected by cancer in their family.	Free	\N	Offered year round	This group meets the second Monday of each month at 3:30pm	English	Yes	Please call Cammie Caron at (978) 937-6129 for more information.	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the Cancer Center	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505423	100055	2008-12-02	\N	What About Me...? Jr.	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6129	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	support group for children(7-10) affected by cancer	What About Me...?Jr.\r\nThis group meets the second Wednesday of each month from 4:00-5:00pm and is for children ages 7 to 10 who have been affected by cancer in their family. A time to talk about the losses and changes in your home since cancer. Learn about cancer, and share your hopes and fears. This group is for bereavement and support. To register please call Cammie Caron at 978-937-6129.	This group is for children between the ages of 7-10.	Free	\N	Offered Year Round	Meets the second Wednesday of each month from 4-5pm	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the Cancer Center	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505434	100055	2008-12-02	\N	Teen Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support group for teens (14-17) affected by cancer	Teen Support Group\r\nA support group for teens ages 14 to 17 who have been affected by cancer in their family. Meets the second Tuesday of each month from 3:30-4:30pm. This group is for bereavement and support. Please call Cammie Caron at 978-937-6129 to register.	Teens ages 14 to 17 who have been affected by cancer in their family.	Free	\N	Offered Year Round	Meets the second Tuesday of each month from 3:30-4:30pm.	English	Yes	Call Cammie Caron at (978)937-6129 for more information	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the Cancer Center	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509350	100055	2008-12-02	\N	Lowell Fibromyalgia Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Fibromyalgia Support Group	The Lowell Fibromyalgia Support Group will strive to provide support, information, education, referrals, dietary suggestions, have guest speakers, and provide answers to your questions about Fibromyalgia.\r\n\r\nMeets the first Thursday of each month from 6:30-8:30pm in Donovan Conference Room. Please contact Carolyn Mooney at 978-204-8253 or CMMooney@ymail.com to register.	Open to anyone affected by Fibromyalgia	Free	\N	Offered year round	Meets the first Thursday of each month from 6:30-8:30pm in Donovan Conference Room.	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the hopsital.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request.	No	0	\N
505753	100055	2008-12-02	\N	Pacemaker-Implantable Cardioverter Defibrillator (ICD) Suppo	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support group for people with Pacemakers and ICDs	The Pacemaker-Implantable Cardioverter Defibrillator (ICD) Support Group is a support group for Pacemaker and ICD recipients, their families and friends in the greater Lowell area. Sponsored by Lowell General Hospital with the support of device manufacturers, the Support Group offers a forum for education and a network for support. The group meets on the second Thursday evening every other month from 6-7:30 in the Clark Auditorium at Lowell General Hospital at no cost to participants. Meetings feature interactive lectures on topics that are of interest to those with pacemakers and ICDs. Open discussion between speakers and attendees is encouraged. Come with your questions and concerns, light refreshments provided. \r\n\r\nTo register or for more information please call 978-937-6326\r\n\r\n \r\n\r\nFacilitators: Laura Pruyn, RN and Deirdre Proudman, RN	This is a support group for Pacemaker and ICD recipients, their families and friends in the Greater Lowell area.	Free	\N	Open Year Round	The group meets on the second Thursday evening of every other month at Lowell General Hospital	English	No	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in the front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509372	100055	2008-12-02	\N	Helping Your Child Through Divorce	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Divorce support group including parent education	Helping Your Child Through Divorce is a support group for parents to get together and discuss healthy ways to help their child cope with divorce. A trained parent educator will offer a brief educational presentation at the beginning of each group and will then help facilitate the discussion. \r\n\r\nThis group will meet the 3rd Tuesday of every month starting September 16, from 6:00-7:30pm in the Wallace Board Room, Lowell General Hospital. \r\n\r\nFor more information, please call Marianne Blanchet at 978-884-6981.	Open to any parents or caregivers affected by a divorce	Free	\N	Open year round	This group will meet the 3rd Tuesday of every month from 6:00-7:30pm	English	No	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the hospital.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Interpreters can be provided upon request	No	0	\N
500671	100055	2008-12-02	\N	Heart and Soles Mall Walking Program	\N	310 Daniel Webster Highway	\N	Nashua	NH	03060	1-877-544-9355	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	One mile walk/ walking group	Heart & Soles Mall Walking program is co-sponsored by LGH's Cardiac Rehabilitation Department and the Simon Pheasant Lane Mall. Enjoy a measured, one-mile walk for your health and fitness. \r\n\r\nThe climate controlled mall provides an excellent atmosphere for walking. You must register and obtain a badge at the Pheasant Lane Mall's Guest Services desk before walking. \r\n\r\nFree blood pressure is offered on the third Wednesday of the month. Click here to visit The Pheasant Lane Mall website. \r\n\r\nFor more information call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org	Must register at walk site.\r\nFor more information call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org	Free	\N	Year Round	For more information call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org	English	Yes	For more information call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at all entrances to the Pheasant Lane Mall	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
507777	100055	2008-12-02	\N	Heart Failure Management Program	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Heart Failure Management Program	Heart Failure Management Program-This self-management program is designed to offer education to those suffering from heart failure to help them manage at home. The Heart Failure Management program offers two topics each month.  Participants are free to attend one or both sessions. All sessions are held from 5-6pm in Donovan.  Registration is required. \r\nThe first Tuesday of each month is Nutrition Management for Heart Failure.  Which Include\r\n\tNutrition Guidelines for Heart Failure\r\n\tWhy sodium and fluid balance is important.\r\n\tHow to change your diet to reduce sodium but keep food flavorful.\r\n\tPractical tips.\r\n\tHands on food label reading.\r\nThe third Tuesday of each month is Symptom Management for Heart Failure.  Which include\r\n\tWhat is heart Failure\r\n\tCauses of heart failure\r\n\tSymptoms of heart failure\r\n\tManaging your heart failure.\r\n\r\nFor more information or to register, please call 1-877-LGH-WELL (1-877-544-9355)or visit www.lowellgeneral.org	Available to anyone who is suffering from Heart Failure	Free	\N	Offered throughout the year	The first Tuesday (nutrition) and third Thursday (symptom management)of each month.	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is availbe at the main entrance to the hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TTY Number. Contact Patient Advocacy Department to request interpreter services at 978-937-6458.	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
500605	100055	2008-12-02	\N	Mindfulness Based Stress Reduction Program	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6396	\N	978-937-6000	\N	www.lowellgeneral.org	public_program_email@testing123.net	Stress reduction educational program	Mindfulness Based Stress Reduction\r\nHome » Health Information » Health & Wellness Programs » Mind/Body Medicine Programs » Mindfulness Based Stress Reduction\r\nThis training offers participants experiential training in well-researched methods for developing a range of self-regulatory skills and competencies that can be practically integrated into their everyday lives as a way of coping with stress, pain, illness and the challenges of everyday life. MBSR originated in the work of the Center for Mindfulness at the University of Massachusetts Medical Center, founded by Dr. John Kabat-Zinn in 1979.\r\n\r\nThis course includes an 8-week training and one all-day retreat. This program is offered twice yearly (Spring & Fall). A pre-interview process is required. \r\n\r\nFor more information or to register, please call 1-877-LGH-WELL (1-877-544-9355).	Open to all	Yes	For more information or to register, please call 1-877-LGH-WELL (1-877-544-9355).	Dates may vary, please call 1-877-LGH-WELL (1-877-544-9355).	For more information or to register, please call 1-877-LGH-WELL (1-877-544-9355).	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
500649	100055	2008-12-02	\N	Perinatal Loss Support Group	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6324	\N	978-937-6000	\N	www.lowellgeneral.org	public_program_email@testing123.net	Support Group for those who lost baby during pregnancy	LGH Perinatal Loss Support Group provides support and comfort when your family has suffered a loss through miscarriage, ectopic, stillbirth or neonatal death. To help you through this difficult time, we offer free counseling, as well as support from other families who have shared a similar loss. A free support group is held in the hospital chapel the third Tuesday of each month at 6:30 p.m. If you have suffered a loss and need to speak to someone right away, please call the Mother-Infant Unit at (978) 937-6324 and ask that Darlene Bennett Sampson be contacted.	Families who have suffered Perinatal loss.	Free	\N	Open year round	Third Tuesday of every month 6:30PM	English	Yes	Call Darlene Bennett Sampson at 978-937-6324 for information	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available in front of the Hospital.	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506127	100055	2008-12-02	\N	REIKI Level II Training	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	\N	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Reiki II provides instruction and awareness in energy healing	Reiki is a simple, natural process for healing by the laying on of hands. It originates in the work of Japanese physician Dr. Mikao Usui. The Usui system of natural healing has been taught through an oral tradition since its inception in the late 1800's, meaning that a practitioner learns the system in person and hands on from a teaching master.\r\n\r\nReiki Level II Training \r\nReiki II training provides ongoing instruction and awareness in energy healing. Participants will review Reiki I, learn to extend Reiki energy in distance healing and practice methods of working energetically with mental and emotional states. Participants will receive Level II attunements and meet their Reiki guide. Cost: $200. Reiki Level I is a prerequisite.\r\n\r\n\r\nFor more information call Carole Legro at 978-937-6396	Open to all who are interested	Yes	For more information call Carole Legro at 978-937-6396	Occures periodically throughout the year	Dates and times vary- For more information call Carole Legro at 978-937-6396.	English	Yes	\N	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the main entrance to the hospital	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
506875	100055	2008-12-02	\N	Prenatal Yoga Class	\N	295 Varnum Ave.	\N	Lowell	MA	01854	978-937-6000	1-877-544-9355	\N	\N	www.lowellgeneral.org	public_program_email@testing123.net	Prenatal Yoga Sessions	Our Prenatal yoga class uses stretching, modified yoga postures, breathing and relaxation techniques to help honor the body during pregnancy and the birthing process. Medical permission is required for this class, which is for women at any stage of pregnancy. Class is for expectant mothers only. Prenatal Yoga meets for six weeks. Instructed by Monica Morell, PhD.\r\n\r\nWednesdays evenings, 5:15pm-6:10pm and 6:15-7:10pm\r\n\r\n\r\n\r\nFor more information or to register call 1-877-LGH-WELL or visit lowellgeneral.org	The program is available for Women who are in their first, second or third trimester of pregnancy.	Yes	For more information or to register please call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org	Runs continuously throughout the year.	For more information or to register please call 1-877-LGH-WELL (1-877-544-9355) or visit www.lowellgeneral.org	English	Yes	Call 1-877-LGH-WELL (1-877-544-9355) or visit lowellgeneral.org for more information or to registration	41 Lawrence-Lowell!DELIM!07 Pawtucketville / UMass Lowell North	Parking is available at the front of the hospital, near the main entrance	\N	Yes	No	\N	No	\N	(800) 439-0183 TDD Number / Relay Operator. Contact Patient Advocacy Department to request Interpretive Services 978-937-6458	\N	\N	\N	\N	\N	No	\N	Yes	Please call 978-937-6000 for more information	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	0	\N
501199	100165	2008-12-03	\N	Kid's Net	MSPCC/ KID'S NET	439 South Union St.	\N	Lawrence	MA	01843	978-682-9222	800-541-3004	800-486-3730	978-681-9534	www.mspcc.org	public_program_email@testing123.net	Provides services to Foster/Pre-adoptive/Kinship Parents	The Kid's Net program provides respite, short-term / non-emergency daycare, in-service training, support group referrals, funding for camp / after-school programs, hot line (Kid's Net Connection), advocacy / membership services. All of the services are available only to Foster,Pre-Adoptive and Kinship Parents.	Must be a Foster, Kinship or Pre- Adoptive parent in Northeast Massachusetts / DCF	Free	\N	Program available year round, although activity decreases during the summer months	Monday-Friday 9:00AM-5:00PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	\N		Parking is available at both Lawrence and Lowell sites	\N	Yes	Yes	Child Care for Foster,Pre-Adoptive & Kinship Parents that have an Open case at DCF	Not Applicable	\N	\N	ATTN: Soledad Berrios	439 South Union Street	Lawrence	MA	01843	Yes	\N	No	\N	Yes	One conference room in Lawrence may be available. Sign up through receptionist	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	0	\N
500165	100033	2008-12-04	\N	Senior Corps Volunteer Programs	FGP/SCP/RSVP	169 Merrimack Street	3rd Floor	Lowell	MA	01852	978-654-5080	\N	\N	978-654-5071	www.comteam.org	public_program_email@testing123.net	Provides volunteer opportunities for seniors 55 and up.	The Senior Corps Program works with and advocates for seniors, providing them with meaningful volunteer opportunities that help them keep active in their communities. For low income volunteers, small tax-free stipends may be available to work with other seniors or with special needs children. Non-profit and health care agencies, as well as schools and state or municipal offices, can request to have volunteer opportunities publicized.	Call 978-654-5080 ext. 14 or 22 for more information.	Free	\N	Program services are available year round	Monday-Friday 9:00AM-4:30PM	English!DELIM!Spanish	Yes	Call for more information - information needed varies by program.	09 Downtown Circulator	Garages on Market St.or John St. are within 2 blocks of our office, on street parking may be available on surrounding streets for $1 per hour.	Transportation costs to and from volunteer assignments can be reimbursed to the extent possible. This would include public transport, the Road Runner, or mileage and parking for those who drive. \r\nSome Foster Grandparents (low income stipended volunteers working with special needs children)are transported by their assigned station. And when possible, placements may be arranged within walking distance of volunteers' homes.	Yes	No	\N	Yes	RSVP never has a wait list. There may be wait lists for stipended programs.	\N	\N	\N	\N	\N	\N	Yes	Please contact Joan Aseltine at 978-654-5080 ext 14 or  Jane Benfey at 978-654-5080 ext 22 for more information about volunteer opportunities	No	\N	Yes	By arrangement; call Monica Negron at 978 654-5080 ext. 20 to check on availability.	Yes	Bi-monthly newsletter includes volunteer opportunities at area non-profits, schools, or municipalities.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Placements for some programs may be available in other towns. Call for more information	\N	No	0	\N
500154	100033	2008-12-06	\N	Division of Child & Family Services	\N	167 Dutton Street	\N	Lowell	MA	01852	978-459-0551	800-698-0551	\N	978-453-9128	www.comteam.org	public_program_email@testing123.net	Provides a full range of services to families in Gr. Lowell	The Division of Child and Family Services provides a full range of services to families throughout Greater Lowell including early care and education, transportation, health and nutritional education (WIC), special needs, social services and parental involvement. This Division of CTI provides seamless and comprehensive delivery of Early Care and Education and School Age Programs to more than 1,300 children from birth to 12 years of age throughout Greater Lowell.  Additionally, CTI partners with Citizen Schools to offer cognitive and leadership activities for adolescents at the Wang and Robinson Middle Schools.	Greater Lowell children Birth - 13years of age	Sliding Scale	Sliding fee based on income	Full Year and School Year programs are available. Check program for enrollment dates. WIC services available year round, appointments and walk-ins welcome.	Monday-Friday 7AM-6PM	English!DELIM!Spanish!DELIM!Khmer	Yes	Contact individual program for details		Parking available	Transportation provided by some programs. Check with individual program.	Yes	Yes	\N	Yes	Families must be on the EEC state waitlist to access services in the Early Childhood and School Age programs.	\N	\N	\N	\N	\N	\N	Yes	Check with individual program for volunteer details.	No	\N	Yes	The James Houlares Early Learning Center hosts open training/meeting space for community agencies.  Please call 978-454-5100	Yes	Internal to enrolled families.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
505830	101925	2008-12-08	\N	Home Health Partners	Home Health Foundation	360 Merrimack Street, Building 9	\N	Lawrence	MA	01843	978-552-4525	800-933-5593	\N	\N	www.homehealthfoundation.org	public_program_email@testing123.net	Volunteer assignments support the agencies mission	Home Health Partners are men and women who support the Home Health Foundation in a variety of ways from providing administrative support to coordinating events and fundraising affairs.  Time commitments are flexible and office locations include Lawrence, Chelmsford, Peabody, Newburyport and Haverhill.	Over 18 year of age	Free	\N	Flexible	Flexible	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Drivers License, Auto Registration		Free	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	The organization has a number of health care resources including printed and electronic format.	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	Office locations are in Chelmsford, Peabody, Lawrence, Haverhill and Newburyport	\N	No	0	\N
507865	103223	2008-12-15	\N	StarWorks	\N	102 Glenn Street	\N	Lawrence	MA	01843	978-685-2151	\N	\N	978-683-5124	americantraininginc.com	public_program_email@testing123.net	Evaluations and trainings for referrals from MRC	StarWorks' referrals are usually from the Massachusetts Rehabilitation Commission.  We evaluate, train and job place disabled adults from the Merrimack Valley area.  We receive referrals from MRC's Lawrence and Lowell area offices and from Boston's Statewide Employment Services division of MRC.  We also receive referrals from MCB (Mass. Commission for the Blind) and SHIP (Statewide Head Injury Program).	Disabled adults who qualify for MRC services.  Speakers of English or Spanish.	Free	\N	Open year round, open entry.	We are open Monday through Friday, from 8:00 am - 4:30 pm. Closed holidays	English!DELIM!Spanish	Yes	\N	37 Beacon St.	Free parking and also on MVRTA's bus Route 37	Transportation available for qualified applicants.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Please contact us.  We'd be happy to speak with anyone who is interested in sharing their talents with us!	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	0	\N
502508	100759	2008-12-15	\N	Brigid's Crossing	\N	48 Lawrence Street	\N	Lowell	MA	01852	978-454-0081	\N	978-454-0081	978-454-0210	\N	public_program_email@testing123.net	Teen living program for pregnant and parenting teens	Brigid's Crossing is a teen living program - a residential and educational program for pregnant and parenting teens. The residents actively participate in the learning process and work cooperatively with staff and interns. Brigid's is a place where young mothers can experience independent living while they learn to access resources in the community and while they develop appropriate parenting skills.	Must be 14-20 year old female referred by the Department of Social Services. 3 Community beds available which may be an open referral	Sliding Scale	Residents pay 30% of their income	Services are available year round. Program capacity is 7 moms, 8 children.	24 hours/day, 7 days/week	English	Yes	All referrals come from the Department of Social Services		Parking is available.	\N	No	No	\N	No	\N	Bilingual staff may be available if necessary	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
508195	100759	2008-12-15	Hidden on 06-24-2008 by: the MVHub Administrator<BR>Reason: <i>temporary -- unapproved</i>	Healthy Families Haverhill	\N	191 Merrimack St., Suite 511	\N	Haverhill	MA	01830	978-521-6265	\N	\N	\N	http://mctfcommunity.org/	public_program_email@testing123.net	Home visiting program available for first-time parents	Healthy Families is home visiting program available for first-time parents. Healthy Families visits are conducted by trained staff and involve the entire family, including fathers, mothers, children, and extended family members.\r\n\r\nStaff members encourage participants to develop, at their own pace, the information, skills, and support networks necessary to be strong parents. Home visitors answer questions, teach parenting skills, and help new parents find health care, day care, transportation, and other services they may need.	First-time parents age 20 and under. Your child cannot be older than age 1 when you join the program. There are no income limits.	Free	\N	Open year round.	M-F, 9AM-5PM.	English!DELIM!Spanish	Yes	ID for parents.		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Healthy Families provides resource information and literature to clients.	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Groveland, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Salisbury, MA!DELIM!Boxford, MA!DELIM!Merrimac, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA	\N	To find other regional Healthy Families offices, call the Children's Trust Fund toll-free at 1-888-775-4KIDS.	\N	No	0	\N
507986	102948	2008-12-15	\N	Pegasus Youth Residence	\N	482 Lowell Street	\N	Lawrence	MA	01841	978-687-4257	\N	\N	978-687-3338	www.tpc1.org	public_program_email@testing123.net	8 bed program for substance abusing females 13-17.	Pegasus has successfully provided a culture of recovery for adolescents since 1985. It is a 15 bed program designed for substance abusing adolescent females, ages 13-17, who are in crisis and need comprehensive treatment in a therapeutic, structured, non-institutional setting.\r\n\r\nClients are at high risk due to their substance abuse. As a result, they are in need of a multidimensional, evidence based treatment approach.The treatment components have been designed to focus on the physical, psychological, social and familial aspects of the adolescents life.\r\n\r\nThe program offers an environment that reinforces a drug and alcohol free lifestyle that includes attitudinal and behavioral changes. Clients are encouraged to leam how to cooperate with each other, and leam the skills and responsibilities needed to care for themselves and develop positive relationships with adults.\r\n\r\nPegasus is funded by the Department of Public Health.	Female, ages 13-17.  Parent must agree to participate in daughter's treatment. Please call the Program @978-687-4257 and we will help you with the referral process.\r\n\r\n\r\nClient must have: \r\n+ a significant substance abuse problem.\r\n+ ability to control behavior and follow program rules.	Sliding Scale	Sliding scale fees based upon parents' income.	Residential program, averages 90 days with treatment.	24/365	English!DELIM!Spanish	Yes	Client needs results of a physical exam within the last year.	01 Haverhill - Methuen - Lawrence	\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	CORI check required.	Yes	Drug education resource and health prevention center libraries	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
509064	100759	2008-12-15	Hidden on 07-17-2008 by: the MVHub Administrator<BR>Reason: <i>ADD INFO</i>	Parent Aide Program	\N	354 Merrimack Street	\N	Lawrence	MA	01843	978-685-5930	\N	\N	\N	www.ccab.org	public_program_email@testing123.net	In-home visitation and support for parents and children.	The Parent Aide Program of Merrimack Valley Catholic Charities provides support to parents and their children through weekly in-home visitation and support.\r\n\r\nThrough caring and consistent interventions, the program is designed to reduce incidences of child abuse and neglect, to prevent the removal of children from their homes, and to assist with the reintegration of a child or children into the home.\r\n\r\nWhile families strive to effectively identify and meet their own goals and needs, the Parent Aide Program exists to achieve the following three outcomes: assist families in meeting their basic needs, develop healthy proactive parenting skills, and build self-reliance and family stability.\r\n\r\nThe Parent Aide Program is committed to serving parents from a variety of cultural and linguistic backgrounds and of all ages. Parent Aides work with teen mothers and babies, as well as with grandparents raising their grandchildren and all age groups in between.	Department of Social Service referred parents and their children.	Free	\N	Open year round.	ADD INFO	English!DELIM!Spanish	Yes	DSS referral.		\N	\N	No	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Haverhill, MA!DELIM!Lawrence, MA!DELIM!Lowell, MA	\N	\N	\N	No	0	\N
502530	100759	2008-12-15	\N	Outpatient Counseling Services	Mental Health Servicies	70 Lawrence Street	\N	Lowell	MA	01852	978-452-1421	\N	978-452-1421	978-454-9968	www.ccab.org	public_program_email@testing123.net	Counseling for Individuals, Families and Couples	The Outpatient Counseling Services program provides individual, couples and family counseling for children, adolescents, and adults to help with difficulties including depressed or anxious feelings, troubling behaviors, life changes, trauma, and relationship issues.  Most insurances are accepted, including most MassHealth and Commonwealth Care plans.   For more information, call our Intake office at 978 774-6820.	Services are open to anyone.  Call our Intake office at 978 774-6820 to check insurance coverage, to arrange for a sliding scale fee plan, or to schedule an appointment.	Yes	Most insurances accepted.  Coverage and co-pays can be checked by calling our Intake office at 978 774-6820.  A sliding scale fee can also be set-up through the Intake office.	Open year round.	Mondays through Fridays, 9 to 5 pm.	English	Yes	Bring insurance information names and addresses of primary care physicians and other providers to appointment, if applicable.	41 Lawrence-Lowell!DELIM!02 Belvidere!DELIM!03 South Lowell!DELIM!09 Downtown Circulator!DELIM!12 Tewksbury via Rte. 38	Street parking available	\N	Yes	No	\N	No	Currently, there is no waiting list.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Most convenient for those in the Greater Lowell area.	Counseling in English.	No	0	\N
509075	100759	2008-12-15	Hidden on 07-17-2008 by: the MVHub Administrator<BR>Reason: <i>ADD INFO</i>	Latino Outreach Program	\N	354 Merrimack Street	\N	Lawrence	MA	01843	978-685-5930	\N	\N	\N	www.ccab.org	public_program_email@testing123.net	Counseling and peranting education for Spanish-speakers.	The Latino Outreach program of Merrimack Valley Catholic Charities provides culturally sensitive, bilingual pregnancy counseling and parenting education through individualized case management and parenting groups to low income, primarily Spanish-speaking teen and adult parents in the Greater Lawrence area. It is the only program in Lawrence that provides crisis pregnancy counseling to pregnant teens and adults who are trying to sort out their feelings and consider their options for dealing with their pregnancies.\r\n\r\nServices offered through the program include crisis pregnancy counseling, long-term case management, outreach to family members, home visits, advocacy, parenting and education support groups, and baby supplies. Targeting teen and adult Latinas, goals of the program include the following: healthy pregnancies and births, improvement in parenting and life management skills, a change in attitudes and behaviors to avoid the risk of unwanted pregnancies and exposure to sexually transmitted diseases, and the avoidance of repeat crisis pregnancies to establish stable living situations.	Pregnant teen and adult Latino females in Lawrence.	Free	\N	Open year round.	ADD INFO	English!DELIM!Spanish	Yes	ADD INFO		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
507909	102178	2008-12-15	\N	Child Care Center	\N	581 Andover Street	\N	Lawrence	MA	01843	978-681-4910	\N	\N	978-681-4913	www.glcac.org	public_program_email@testing123.net	Accredited, State Funded and Non-Profit Child Care Center.	The Child Care Center has been offering quality care for over 38 years. Our mission is to provide a quality program that focuses on the education and care of our children, parents and staff. We strive to provide the most nurturing environment where children and their families can meet their full potential.\r\n\r\nWe are accredited by NAEYC and a Universal Pre-K Grant recipient. We are licensed and funded by the Department of Early Education and Care. Our Programs include: Toddler, Preschool, Kindergarten and a After School, School-Age program. We serve breakfast, lunch and snacks daily and schedule monthly educational field trips for each classroom.	We are an accredited, state funded program that offers care for Toddlers (starting at 15 months), Preschool children (2.9 Years), full day Kindergarten for children (5 years by Sept. 1st) and a School-Age program for children in first grade to their 11nth birthday.	Sliding Scale	Fees are determined by income on a sliding fee scale set by the state. Contracted, Voucher, Supportive and Private slots available	Open year round, Full Time Program	Monday-Friday\r\n7:00 a.m. to 5:30 p.m.	English!DELIM!Spanish	No	Proof of residency, proof of income, family size, child's updated physical and shot records.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	\N	The School-Age program offers transportation from all Lawrence Public Schools.\r\n\r\nTransportation provided in Lawrence only.	Yes	Yes	Non-Profit, Accredited by NAEYC, UPK Grant Recipient, and State Funded. Over 38 years experience in providing quality care to the Merrimack Valley.	Yes	If care is not available at the time you apply your application will be kept on the wait list until a slot is available.	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	Yes	Monthly Newsletters are sent home to all enrolled families that list upcoming events, field trips and classroom information. Menus and monthly calendars are also provided to each family.	Yes	The agency's website has descriptions of all the programs involved in GLCAC Inc.	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
507942	102178	2008-12-15	\N	Leahy Family Literacy Program	A Pathways to Family Success Program	100 Erving Street	\N	Lawrence	MA	01841	978-681-4996	\N	\N	978-681-4949	www.destinationservices.org	public_program_email@testing123.net	After school co-enrolled family literacy program	Program serves immigrant families seeking to improve their English skills. Program has 4 service components:\r\n1. ESOL for parents\r\n2. Academic support for their children\r\n3. Inter-active family literacy activities\r\n4. Support services\r\nParents & their children attend program together. While the parents receive ESOL instruction, their children receive academic support services. Every other week, parents and children come together for 30 minute inter-active literacy activity. Once a month, parents attend a one hour workshop to learn about local resources and/or a topic of interest.\r\n\r\nThis is a Pathways to Family Success Program. The mission of Pathways to Family Success is to establish a coalition of human service providers who will provide integrated literacy and support services to families with the intent of improving their lives.	Families	Free	\N	Follows school calendar year.	Monday & Wednesday   5:30-8 PM	English	Yes	Please contact the Leahy Elementary School to enroll: 978-975-5959		Parking at school	\N	No	Yes	Only school age children served, \r\n5-12 years old.	Yes	Contact Leahy School for wait list information:\r\n978-975-5959	\N	Pathways to Family Success	305 Essex St	Lawrence	MA	01840	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	\N	\N	\N	Not Applicable	0	\N
501925	100539	2008-12-15	\N	Citizenship Assistance	\N	11 Mill Street	\N	Lowell	MA	01852	978-970-1250	\N	\N	978-970-0843	www.maps-inc.org	public_program_email@testing123.net	Citizenship assistance, including citizenship classes	As part of its Education Programs, MAPS provides citizenship assistance, including citizenship classes.	Must be Portuguese speaking. Contact program for details.	Yes	$20 donation for citizenship application\r\n$30 for citizenship/ESOL classes	Year round. Contact program for specific schedule	Monday-Friday 9AM-5PM	English!DELIM!Portuguese	Yes	Contact program for information	03 South Lowell	Contact program for information	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Osvalda Rodrigues	No	\N	Yes	Yes, there is a conference room. Please call for more details.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
504257	101507	2008-12-15	\N	Greater Lowell Workforce Investment Board	\N	107 Merrimack Street	\N	Lowell	MA	01852	978-937-9816	\N	\N	978-459-2111	www.glwib.org	public_program_email@testing123.net	workforce development assistance	Workforce Development Assistance\r\nLabor Market information\r\nYouth at-risk funding opportunities\r\ntechnical assistance to local business re: grant training opportunities and tax credits	Generally limited to business and organizations located in Greater Lowell and towns listed below.	Free	\N	Year Round	Monday-Friday 9-5	English	Yes	n/a		Joseph Downes (John Street) Parking lot.\r\nvalidated tickets	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	The Greater Lowell Workforce Investment Board seeks volunteers representing business in Greater Lowell	Not Applicable	\N	Yes	The Greater Lowell WIB can assist groups with meeting space needs by utilizing the Career Center of Lowell.  Please contact Erin for information	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	capabilities of translation in Spanish, Portuguese, Khmer	Not Applicable	0	\N
506908	102640	2008-12-15	\N	Generazio & Nguyen	\N	77 Franklin Street	Suite 300	Boston	MA	02110	617-423-0700	\N	\N	617-423-0799	www.gnimmigration.com	public_program_email@testing123.net	Full service immigration law firm	Generazio & Nguyen is a Massachusetts-based law firm with locations in Andover and Boston, devoted exclusively to the practice of immigration and nationality law.  Our firm's unique experience enables us to navigate our clients effectively and efficiently throught the complex immigration process.   Providing cost effective immigration solutions represents the cornerstone of our practice.	anyone	Yes	\N	Open year round	Monday through Saturday 8am to 6pm	English	Yes	All immigration papers		Street Parking	\N	Not Applicable	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
507953	102178	2008-12-15	\N	Greater Lawrence CPC/Local Early Education and Care Council	Community Partnerships for Children Council	305 Essex St	\N	Lawrence	MA	01840	978-681-4996	\N	\N	978-681-4948	glcac.org	public_program_email@testing123.net	Early Education and Care Programs and Services	The Greater Lawence Community Partnership/Local Early Education and Care Council Is funded through the MA, Dept. Of EArkly Education and Care and covers Andover/North Andover/Methuen and Lawrence. We provide trainings and workshops for parents, Professional Development for EEC staff including courses in Higher Education leading to Associates Degrees and Bachelor's degrees.  We support centers and Family Child Care Providers through the NAEYC process or the NAFCC process. We are the local entry point for families who are income eligible for  EEC financial assistance. We place the families on the Central EEC wait list. Our partnership has over 76 organizations represented at the council table.\r\n\r\nIn addition, we serve as the local entry point for families to access the centralized wait list; provide support to agencies seeking NAFCC or NAEYC accreditation, support for staff seeking a CDA, support transition activities for children moving on to kindergarten and act as a resource for multiple services families may need.	For the centralized list families must be low-income working families or families attending training or educational programs; preschool age children; Massachusetts residency required.	Free	Families must be at or below the 50% of the State median income (SMI) upon initial assessment and remain eligible if their income remains at or below the 85% SMI and they meet the other eligibility requirements.	Scholarships are for full day full year early education and care programs or part time programs.	Full day/ full year or part time depending upon the families need.	English!DELIM!Spanish	No	Full day/ full year or part time depending upon the families need.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	The Buckley garage on Common Street is directly behind our building, street parking is available for two hour periods.	Some individual early education and care providers provide transportation.	Yes	Yes	We place eligible families on the MA centralized list and provide support to parents and EEC staff through trainings and workshops.	Yes	We are mandated to utilize The Dept. of Early Education and Care's centralized waitlist.	Our goals are:\r\n1) To provide financial assistance opportunities for preschool children from low income families to participate in quality early education and care programs that promote healthy development and school success; 2) to serve as a local entry point for families to the centralized wait list; 3) to support the achievement of NAEYC accreditation, CDA or NAFCC accreditation; to support professional development opportunities; 4) to ensure access to comprehensive services and programs such as Parent Home Child Program, MA Family Network, and Transitioning to kindergarten; 5) to outreach to non traditional families; and 6) to maximize family education and support through collaboration and partnerships with local and regional programs and agencies. We serve Andover, North Andover Lawrence and Methuen.	\N	\N	\N	\N	\N	Yes	\N	Yes	Our resource kits are located at the Methuen Nevin's library and the two Lawrence Libraries. These can be taken out by parents or early education and care staff.	Yes	A large conference room and a small conference room for a 15 person conference room.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	We support the centers and FCC providers with Mental Health consultations for children experiencing social and emotional issues. Our goal is to kekep children with MH issues at their present centers and avoid termination.	\N	No	0	\N
507095	102706	2008-12-15	\N	GED	General Education Diploma or High School Diploma Program	270 Jackson Road	\N	Devens	MA	01434	978-784-2600	\N	\N	978-784-2710	http://bostonregion.jobcorps.gov	public_program_email@testing123.net	G.E.D.	Students can obtain a GED or may be eligible for a High School Diploma while enrolled as a student at Job Corps.	16 - 24 years old, income guidelines may apply, must be enrolled in the Job Corps Program to receive services	Free	This program is federally funded and is offered at no cost to the student. Students receive a weekly stipend and a transitional allowance upon successful completion.	This is a residential program open 24 hours per day, 365 days per year. GED training is held on weekdays from 8am - 3:30 pm.	8:00 am - 3:30 pm.	English!DELIM!Spanish	Yes	Enrollment is in person, by appointment.		Free parking on center.	Students accepted to the program will receive free transportation to selected areas.	Yes	No	A stipend may be provided.	No	\N	Enrollment is open (new classes start every week.)	\N	\N	\N	\N	\N	Yes	We are always looking for guest speakers as well as job sites at which our students can perform work based learning or internships.	Yes	Not open to the public, staff and students have access to career assessment and career placement resources.	Yes	Conference room, gymnasium, classrooms and cafeteria	Yes	Center Director's office prepares a weekly newsletter for staff and students.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Shriver Job Corps is one of over 120 centers nationwide.	Shriver does not offer ESOL training, however eligible students may take the GED in Spanish. Some staff of the program are bilingual. All classes and training are conducted in English.	No	0	\N
507117	102717	2008-12-15	\N	Voluntary Adoption and Foster Care	Haverhill Area Office	3 Ferry Street	\N	Bradford	MA	01835	978-469-8800	\N	1-800-792-5200	978-469-8990	www.mass.gov/dss	public_program_email@testing123.net	Adoption and Foster Care services	The Department of Children and Families (DCF)also offers voluntary Adoption, and Foster Care services to assist families.	Contact us at 978 469 8800 for more information.	Free	\N	Open Year Round	Monday  Friday 8:45  5:00 \r\nAfter Hours: Child-At-Risk Hotline: 1-800-792-5200	English!DELIM!French!DELIM!Spanish!DELIM!Khmer	No	\N	14 Bradford / Ward Hill	Yes	No	Yes	Not Applicable	\N	No	\N	Voluntary Applications appointment recommended	\N	\N	\N	\N	\N	Not Applicable	Educational Internships	Yes	Northeast Regional Resource Center\r\nhttp://www.marrcne.org \r\n\r\nAdoption Journeys of Massachusetts\r\n1-800-972-2734\r\nhttp://www.adoptionjourneysma.org  \r\n\r\nMassachusetts Adoption Resource Exchange, Inc. (MARE)1(800)54-ADOPT\r\nhttp://www.mareinc.org \r\n\r\nMassachusetts Society for the Prevention of the Cruelty of Children (MSPCC), Kids Net Program\r\n(617) 587-1500\r\nhttp://www.mspcc.org \r\n\r\nThe Center for Family Connections\r\n(617) 547-0909\r\nhttp://www.kinnect.org \r\n \r\nThe Center for Adoption Research\r\nhttp://www.centerforadoptionresearch.org  \r\n\r\nOpen Door Society\r\nhttp://www.odsacone.org \r\n\r\nAdoptUSKids\r\nwww.adoptuskids.org \r\n\r\nBaby Safe Haven\r\nhttp://www.babysafehaven.com	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Salisbury, MA!DELIM!Boxford, MA!DELIM!Merrimac, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Bradford, MA\r\nIpswich, MA\r\nTopsfield, MA	Greek, Polish, Chinese, and Arabic	No	0	\N
507084	102706	2008-12-15	\N	Job Training	Career Development	270 Jackson Road	\N	Devens	MA	01434	978-784-2600	800-733-5627	\N	978-784-2710	http://bostonregion.jobcorps.gov	public_program_email@testing123.net	Job Training	Students can obtain hands on training in Automotive, C.N.A., Culinary, Computers, Business Office Skills, Carpentry, Cement, Painting, Security, Transportation & Communications Union, College Placement, etc. All programs are free of charge.	16 - 24 year olds, financial guidelines may apply, age can be waived for persons with disability, contact a career counselor for more information.	Free	All costs are paid by the Federal Government. Students earn a weekly cash stipend and a transitional allowance upon successful completion of the program.	Enrollment is open (new classes start each week.) Most programs are residential and include room and board. Training hours are 8:00 am - 3:30 pm	Center is open 24 hours per day, 365 days per year.	English	Yes	Applications are taken in person, by appointment. Tours of the program are given weekly on Wednesday's, call 978-784-2622 for an appointment.		Free parking on center.	Students receive transportation to the center at enrollment and may be eligible for transportation home each weekend.	Yes	No	A stipend may be offered to those who qualify to help cover child care costs.	Yes	Varies depending on amount students currently enrolled.	\N	\N	\N	\N	\N	\N	Yes	We are always looking for guest speakers and for job sites at which our students can perform work based learning and internships.	Yes	Not open to the public, staff and students have access to career assessment and career placement resources.	Yes	Conference room, gymnasium, classrooms and cafeteria	Yes	Weekly newsletter for staff and students prepared by the Center Director's Office.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Shriver is one of 120 job corps programs in the United States and P.R. Students will choose a center to attend based on their choice of trade (training) and proximity to their home.	We do not offer ESOL classes, however, our program is open to everyone, some staff may be bi-lingual.	No	0	\N
508206	102717	2008-12-15	\N	Child Welfare & Case Management	\N	3 Ferry Street	\N	Bradford	MA	01835	978-469-8800	\N	1-800-792-5200	978-469-8990	www.mass.gov/dss	public_program_email@testing123.net	Protecting children from child abuse and neglect	DCF responds to reports of abuse or neglect 24 hours a day. DCF becomes involved if there are any concerns that caretakers, parents, step-parents, guardians or other persons responsible for caring for children may be abusing or neglecting these children.\r\nWhenever possible and appropriate, DCF attempts to keep families intact. DCF reviews all the reports of child abuse and neglect received by the agency. If it is determined that abuse or neglect has occurred, or if a child appears to be at risk of being hurt, or is being neglected, DCF takes action to protect that child.	Caretakers and children.	Free	\N	Open year round	Monday  Friday 8:45  5:00 \r\nAfter Hours: Child-At-Risk Hotline: 1-800-792-5200	English!DELIM!French!DELIM!Spanish!DELIM!Khmer	No	\N	14 Bradford / Ward Hill!DELIM!03 South Lowell	Yes	No	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Northeast Regional Resource Center\r\nhttp://www.marrcne.org \r\n\r\nAdoption Journeys of Massachusetts\r\n1-800-972-2734\r\nhttp://www.adoptionjourneysma.org  \r\n\r\nMassachusetts Adoption Resource Exchange, Inc. (MARE)1(800)54-ADOPT\r\nhttp://www.mareinc.org \r\n\r\nMassachusetts Society for the Prevention of the Cruelty of Children (MSPCC), Kids Net Program\r\n(617) 587-1500\r\nhttp://www.mspcc.org \r\n\r\nThe Center for Family Connections\r\n(617) 547-0909\r\nhttp://www.kinnect.org \r\n \r\nThe Center for Adoption Research\r\nhttp://www.centerforadoptionresearch.org  \r\n\r\nOpen Door Society\r\nhttp://www.odsacone.org \r\n\r\nAdoptUSKids\r\nwww.adoptuskids.org \r\n\r\nBaby Safe Haven\r\nhttp://www.babysafehaven.com	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Salisbury, MA!DELIM!Boxford, MA!DELIM!Merrimac, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Bradford, MA\r\nIpswich, MA\r\nTopsfield, MA	Greek, Polish, Chinese, and Arabic	No	0	\N
501595	100429	2008-12-16	\N	American Heart Association CPR Courses	SMC	One Hospital Drive	\N	Lowell	MA	01852	978-934-8591	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Heartsaver and Healthcare Provider	Health Care Provider Certifications and \r\nHeathh Care Provider Re-Certifications\r\n\r\nHeartSavers Cert and Recertifications\r\n\r\nDay Care Provider Certifications	No information provided	Yes	various	On-Going classes offered	Class hours vary, contact program for specifics ccurtis@saintsmedicalcenter.com or 978.934.8591	English!DELIM!Portuguese!DELIM!Spanish	No	No information provided		\N	\N	Yes	Not Applicable	No information provided	No	If class if full we will add an additional\r\nclass	\N	\N	\N	\N	\N	\N	No	No information listed	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Yes	0	\N
501716	100429	2008-12-16	\N	Wellness Institute	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8591	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Community Education / Physician Referral	The Community Education Department at Saints Memorial offers the following:\r\n-Babysitter Training - attendees receive certificate of completion\r\n-AHA CPR Certification and Re-certification\r\n-Yoga\r\n-National Safety Council 1st Aid\r\n-Referrals to SMMC physicians - this is a free service	Everyone is eligible	Sliding Scale	Community Ed fees vary, call for more information. Fees payable by cash, check, or money order.\r\n\r\nReferrals to SMMC physicians is a free service	Program runs all year, although individual program length and dates vary.	Monday-Friday 9AM-5PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Call to register for classes. If enrolling in Babysitter Training, you will need a doll. Space is limited for each class.		Free parking available in Hospital garage.	\N	Yes	No	\N	No	\N	\N	Cathy Curtis, Director	One Hospital Drive	Lowell	MA	01852	No	Volunteers are not used in this program	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
501727	100429	2008-12-16	\N	Corporate WellCare	\N	85 Parkhurst Road	\N	Chelmsford	MA	01824	978-458-6868	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Corporate WellCare	Corporate WellCare is for injury management.  Corporate WellCare assists in getting appropriate treatment and is a liason between the patient and the Co.\r\nAlso pre-employment and drug testing services.	Contact program	Yes	Contact program Director for more information\r\non prices.	Occupational Health Center open 8:00AM to\r\n4:30PM. 24/7 Emergency Room access	Monday-Friday 8AM-6:00PM\r\nSat. 8:00 - 1:00PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Appointment is encouraged except in the case \r\nof an injury. Walk-In service is available.		Parking available at all site locations	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502321	100429	2008-12-16	\N	Occupational Health Center	SMC	85 Parkhurst Road	\N	Chelmsford	MA	01824	978-458-6868	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Occupational Health Center	The Center provides occupational health services including pre-employment physicals, vaccinations, drug screenings and worker's compensation treatment.	Call for more information	Yes	Employers are billed for all services. Some are available as self pay.	Services available all year, closed most major holidays	Monday-Friday 8AM-4:30PM by appointment\r\nSat. 8:00 - 1:00PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	Nothing is needed if it is a worker's comp. visit. A request for client services from employer is required for other services		Free parking available	\N	Yes	No	\N	No	\N	\N	85 Parkhurst Road	\N	Chelmsford	MA	01824	No	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Not Applicable	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
502398	100429	2008-12-16	\N	Summer Courses	SMMC	One Hospital Drive	\N	Lowell	MA	01852	978-934-8591	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Summer Courses	A variety of courses are available for healthcare professionals and the general public, including: CPR, First Aid, Yoga and Babysitting training. Visit http://www.saintsmedicalcenter.com/announcements.htm for a full listing with schedule and fee information, or call 978-934-8591 for more information.	Contact program for information @ 978.934.88591 or email ccurtis@saintsmedicalcenter.com	Yes	Vary, check with program	Classes run over the course of the summer. fall,winter and spring.Check website for specifics @ www.saintsmedcalcenter.com	Class hours vary. Check website or call for more information.	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Parking available in on-site garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
500880	100110	2008-12-16	\N	After School Enrichment Program	\N	220 Worthen Street	\N	Lowell	MA	01852	978-458-6529	\N	978-458-6529	978-458-4837	www.girlsinclowell.org	public_program_email@testing123.net	After school program for girls ages 6 - 18	The Girls Inc. After School Enrichment Program provides a safe, supervised setting in which our members participate in structured educational, cultural, and recreational activities, receive homework help and socialize.	Girls ages 5 thru 18 in the Greater Lowell area	Sliding Scale	$15 annual membership. Monthly fee $60 - $235 based on family size and income.	After school program during school year, full day summer program July-August. Closed on half and snow days	2PM-6:00PM days Lowell public schools are in session. 8:30AM-6:00PM school vacation weeks. 8:30-5:30 for 9-week summer program; extended day available vacation/summer for nominal fee	English	No	Proof of income if applying for reduced rate		On street parking	Program provides transportation on a contract basis	Yes	Yes	Structured educational, cultural and recreational activities	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Program Director Jen Demers or Site Coordinator Kali Diggs	No	\N	Yes	Girls Inc. will rent out meeting space to qualified non-profit agencies and groups.	Yes	agency newsletter 3 times a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
500891	100110	2008-12-16	\N	Summer Program	\N	220 Worthen Street	\N	Lowell	MA	01852	978-458-6529	\N	978-458-6529	978-458-4837	www.girlsinclowell.org	public_program_email@testing123.net	9 week summer day camp for girls 6 - 14	The Girls Inc. Summer Program is an 9 week summer day camp program providing a safe, structured setting with organized activities and field trips.	Girls ages 6 thru 14	Sliding Scale	$15 annual membership. $70 - $285 / week depending on family size and income	Program runs for 9 weeks in the summer, beginning June 25 and ending August 24. Enrollment begins April 1st.	Monday-Friday 8:30AM-5:30PM\r\nExtended day 7:30AM-8:30AM $5 per day	English	No	Proof of income if applying for reduced rate;\r\n50% deposit for each week reserved		On street parking	\N	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	For volunteer information contact either Site Coordinator Kali Diggs or Program Director Jennifer Demers	No	\N	Yes	Girls Inc. will rent out meeting space to qualified non-profit agencies and groups.	Yes	agency newsletter 3 times a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
500902	100110	2008-12-16	\N	Voucher Day Care	\N	220 Worthen Street	\N	Lowell	MA	01852	978-458-6529	\N	978-458-6529	978-458-4837	\N	public_program_email@testing123.net	After School & Full Day Child care thru Child Care Circuit	Girls Inc. provides after school and full day child care services on a contractual basis with Child Care Circuit.	Referrals required through Child Care Circuit	Yes	Voucher day care through Child Care Circuit	Services available year round	After school hours -- 2:00 PM-6:00PM\r\nVacation week hours -- 8:30 AM-6:00 PM\r\nSummer program -- 8:30 AM-5:30 PM	English	No	Contact program		Street parking.	Transportation available on contract basis	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Jennifer Demers, Program Director or Kali Diggs, Site Coordinator	No	\N	Yes	Girls Inc. will rent out meeting space to qualified non-profit agencies and groups.	No	agency newsletter 3 times a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
504191	101452	2008-12-17	\N	Free & Discounted Prescription Drug Program	\N	Not Applicable	\N	Not Applicable	MA	01007	800-762-4636	800-762-4636	\N	\N	Helpingpatients.org	public_program_email@testing123.net	Database of resources for free & discount drug programs	HelpingPatients.org helps direct patients to the patient assistance program that will best serve them. HelpingPatients.org also contains a database of all the member companies that conduct these programs and the medicines that are covered. It also describes how to request assistance. The application process and eligibility requirements vary for each company and program.\r\n\r\nMedicines available through the programs include many of the newest and most innovative brand-name treatments available, including medicines that treat: cancer, high cholesterol, diabetes, high blood pressure, stroke, depression, schizophrenia, and Alzheimer's.\r\n\r\nPhRMA and its member companies are proud to present HelpingPatients.org, an interactive Web site that provides a comprehensive one-stop link to thousands of medicines offered through hundreds of patient assistance programs sponsored by member companies, non-member companies and government and local organizations.\r\n\r\nA number of our companies have pledged that no patient in need of their medicines will go without them. Our companies have long been worldwide leaders not only in pharmaceutical innovation, but also in philanthropic initiatives - and their long-standing patient assistance programs have been especially helpful. \r\n\r\nIn 2003, programs supported by PhRMA members helped 6.2 million under insured or uninsured patients to obtain more than 17.8 million prescriptions, up from 2.4 million patients and 8 million prescriptions in 2000.	Geared toward low and moderate income people who are uninsured, or under insured, but site has an eligibility calculator so all uninsured and under insured are encouraged to  check the site out.	Free	\N	On the Web Year Round	24 hours a day, seven days a week.	English!DELIM!Spanish	Not Applicable	\N		Not Applicable	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
502354	100429	2008-12-18	\N	Pastoral Care	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8416 ex: 8416	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Pastoral Care	Pastoral care provides emotional and spiritual support for patients and their families. Prayer and pastoral presence are available to everyone of all denominations.	Open to all patients and their families	Free	\N	Open year round	Monday-Saturday at 12PM; Sunday at 10AM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	No appointment necessary - patients and their families may attend as needed		Parking available in onsite garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl at 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
502310	100429	2008-12-18	\N	Nutrition Program	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8458	\N	\N	\N	www.saintsmedicalcenter. com	public_program_email@testing123.net	Nutrition Program	Registered dieticians are available to answer questions about dietary restrictions, supplements and nutrition concerns patients may have. Outpatient nutrition services are available which offer therapy for a number of medical conditions. The outpatient Growth and Nutrition clinic provides pediatric services to children experiencing growth delays or feeding difficulties.	Contact program for more information	Sliding Scale	Contact program for more information	Contact program	By appointment - call 978-934-8458	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	\N		Parking available in on-site garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl Satryb 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
507469	102926	2008-12-23	\N	Irish Cultural Committee	ICC - St Patrick Parish	282 Suffolk Street	\N	Lowell	MA	01854	978-459-0561	\N	\N	\N	www.stpatricklowell.org	public_program_email@testing123.net	SAINT PATRICK 25th ANNUAL IRISH CULTURAL WEEK	SAINT PATRICK 25th ANNUAL IRISH CULTURAL WEEK\r\nSponsored by the Irish Cultural Committee\r\n\r\nSaturday March 8\t\r\nCHILDREN'S IRISH STORY HOUR\t\r\n10:00  A.M.  \t\r\nSt. Patrick Church Hall, Lowell, MA\r\nChildrens Storytelling & Activity.   (Free) \r\n\r\nWALKING TOUR OF THE ACRE\t\r\n2:00 P.M.\t\r\nNational Historic Park \r\nVisitor Center, Market St. \r\nGuided Tour of the Acre and all of its Historic Significance to Contributions within the City of Lowell.\r\n\r\nSunday March 9\t\t\r\nOPENING MASS\t\r\n10:00 A.M.\t\r\nSt. Patrick Church, \r\n282 Suffolk Street, Lowell\r\nPrincipal Celebrant & Homilist  Rev. Richard C. Conway,\r\nConcelebrants  Rev. James E. Taggart, OMI, Rev. Anthony Nguyen, \r\nRev. Daniel C. OConnell, Rev. Arthur Coyle, Rev. Frank J. Silva,\r\nRev. Richard P. Cornell, Rev. Paul Ouellette, OMI, and Deacon An Peter Ros. \r\nMusic led by Harpist Aine Minogue\t\r\nGaelic Readings by Eileen Sullivan\t\r\n\r\nIRISH CULTURAL WEEK PARADE\t\r\n11:00 A.M.\t\r\nFrom St. Patrick Church to City Hall \r\nRaising of the Irish Flag and Laying of the Wreath at the Irish Monument.  Bagpipes performed by Philip J. Klibansky.  Irish National Anthem sung by Eileen Sullivan\r\nUnited States National Anthem sung by Mayor Edward Bud Caulfield\r\n\r\nBRUNCH  \r\nSt. Patrick Church Hall \r\n282 Suffolk Street, Lowell\t\r\nImmediately following Mass / Parade, music by Chuck Campbell\r\n\r\nDonation:  Adults $5.00, Children $ 3.00\r\n\r\nCEILI  (Irish Folk Dancing)  \r\n2:00 - 6:00 P.M.  \r\nLowell Lodge of Elks, \r\nOld Ferry Rd., Lowell  \r\nFree Admission - Sponsored by the Ladies Ancient Order of Hibernians Division 1.\r\n\r\n        \t\r\nTuesday March 11\tCELTIC SYMBOLS   \r\n(A workshop & presentation of Celtic Art and Symbolism) \r\n7:00 P.M.   \r\nMiddlesex Community College \r\nMorse Federal Building Library \r\nNoted Celtic artist Art Ketchen will give a workshop and presentation on Celtic Art and Symbolism at Some of Arts work will be on display.  \r\nFree admission.\r\n\r\nThursday March 13\t\r\nCABARET\t(An Evening of Irish Harp Music and Irish Story Telling)\r\n7:00 P.M.\t\r\nLong Meadow Golf Club, \r\n165 Havilah Street, Lowell \r\nNoted Irish Harpist Áine Minogue will perform in a Cabaret setting at.  She will be accompanied by Kevin OHara, Author and Story Teller. \r\nTickets are $20.00 and may be purchased by contacting the Rectory: 978-459-0561, by e-mail: stpatricklowell@comcast.net, or at the door the night of the event. \r\n\r\nFriday March 14\t\t\r\nIF YOURE IRISH   \r\nLowell Memorial Auditorium  \r\nFrank McNamara presents Derek Moloney (No. 1. Selling Irish Tenor), Deirdre Shannon (Irish Soprano, featured in Celtic Woman & Lord of the Dance), Cora Smyth (dynamic Irish Fiddler, featured in Lord of the Dance), a full orchestra, and a host of local Church Choirs in an evening of Great Irish Music.  Although not sponsored by the ICW (Were Poor) we would highly recommend this performance to anyone who enjoys Irish Music.\r\n\r\nSaturday March 15\t\r\nIRISH CONCERT\r\n3:15 P.M. \t\r\nIrish Organ Music by Flo Ayotte Sheehy \r\n3:30 P.M.\t\r\nMusic Performed by Franco American Choir, Directed by Robert Gaudette, Accompanied by Flo Ayotte Sheehy \r\nSt. Patrick Church, \r\n282 Suffolk Street, Lowell \r\n\r\nMEMORIAL MASS\r\n4:00 P.M.\t\r\nPrincipal Celebrant & Homilist  Rev. James E. Taggart, OMI; Concelebrants  Rev. Anthony Nguyen, Rev. Paul Ouellette, OMI and Deacon An Peter Ros.\r\n\r\nDINNER DANCE\r\n\r\n6:00 P.M. Cocktails,   \r\n7:00 p.m. Dinner.   \r\nLenzis Catering & Function Facilities, \r\n810 Merrimack Avenue, Rt. 110) Dracut, MA   Music Provided by Tony Coit and The Journeymen.  \r\nTickets $ 30.00 (by reservation only).  \r\nDrawing of "Lucky Leprechun" will be held.  Tickets are $25.00 and may be purchased from any ICW committee member or by calling the rectory at 978-459-0561.	ALL ARE ELIGIBLE TO ATTEND.	Yes	MARCH 13TH\r\n\t\r\nCABARET\t(An Evening of Irish Harp Music and Irish Story Telling)\r\n7:00 P.M.\t\r\nLong Meadow Golf Club, \r\n165 Havilah Street, Lowell \r\nNoted Irish Harpist Áine Minogue will perform in a Cabaret setting at.  She will be accompanied by Kevin OHara, Author and Story Teller. \r\nTickets are $20.00 and may be purchased by contacting the Rectory: 978-459-0561, by e-mail: stpatricklowell@comcast.net, or at the door the night of the event. \r\n\r\nMARCH 15TH\r\n\r\nDINNER DANCE\r\n6:00 P.M. Cocktails,   \r\n7:00 p.m. Dinner.   \r\nLenzis Catering & Function Facilities, \r\n810 Merrimack Avenue, Rt. 110) Dracut, MA   Music Provided by Tony Coit and The Journeymen.  \r\nTickets $ 30.00 (by reservation only).  \r\nDrawing of "Lucky Leprechun" will be held.  Tickets are $25.00 and may be purchased from any ICW committee member or by calling the rectory at 978-459-0561.	SEE ABOVE	WEEK OF MARCH 8TH - MARCH 15TH, 2008	English!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	\N	06 Broadway / UMass Lowell	PARKING IS FREE AND AVAILABLE AT ALL FUNCTIONS.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	GAELIC, KHMER, ENGLISH AND VIETNAMESE AT ALL MASSES DURING THE WEEK	Not Applicable	0	\N
504312	101518	2008-12-23	\N	Renaissance Club	\N	176 Walker Street, 2nd Floor	\N	Lowell	MA	01854	978-454-7944	\N	\N	978-937-7867	\N	public_program_email@testing123.net	Mental Health Program	Renaissance Club, a program of Eliot Community Human Services, is a nonprofit clubhouse model program that follows the Standards for Clubhouse Model Programs as set forth by the International Center for Clubhouse Development. Renaissance Club is free and open to anyone who has a history of persistent mental illness in Greater Lowell.	18 years of age or older; residing in Middlesex County; history of mental illness.	Free	\N	Open 365 days a year. Holidays celebrated on actual day observed.	Monday-Wednesday  7:30am-5:00pm\r\nThursday          7:30am-4:00pm\r\nFriday            7:30am-6:00pm\r\nSaturday & Sunday 12:00pm-4:00pm	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Potential members must complete an application with a mental health professional, verifying DSM-Axis 1 diagnosis.	06 Broadway / UMass Lowell	Parking is available on site.	Participants who cannot take public transportation, and who reside in Lowell, may qualify for program van transport.	Yes	No	\N	No	\N	Renaissance Club offers community trips on select Saturdays to museums, theatrical performances, sporting events, tourist attractions, and nature parks.\r\nContact for information.	\N	\N	\N	\N	\N	Yes	All volunteers must pass C.O.R.I. and driving record check to be eligible.	Yes	A small resource library is available in Membership Services Unit for in house use.	No	\N	Yes	Renaissance Club produces a monthly newsletter for members entitled "The Spirit of Renaissance"	No	Internet access is available for all members of Renaissance Club.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Groton	\N	No	0	\N
506017	101518	2008-12-23	\N	Youth Group	\N	176 Walker Street, 2nd Floor	\N	Lowell	MA	01854	978-454-7944	\N	\N	978-937-7867	\N	public_program_email@testing123.net	Youth Activites Group (Ages 18-25)	A social group for youth that includes movies and other activities.	Youth 18 to 25 years.	Free	\N	Open 365 days a year. Holidays celebrated on actual day observed.	Thursday          3:00pm-7:00pm	English	Yes	Potential members must complete an application with a mental health professional, verifying DSM-Axis 1	06 Broadway / UMass Lowell	Parking is available on site.	Participants who cannot take public transportation, and who reside in Lowell, may qualify for program van transport.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	All volunteers must pass C.O.R.I. and driving record check to be eligible.	Yes	Self-help and addiction books.	Yes	\N	Not Applicable	\N	No	Internet access is available for all members of Renaissance Club.\r\n\r\n4 computer cyber cafe.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Groton	Attendees speak languages including Khmer and others.	No	0	\N
501518	100352	2008-12-23	\N	Food Pantry	\N	200 Central Street	\N	Lowell	MA	01852	978-453-6693	\N	\N	\N	\N	public_program_email@testing123.net	Food Pantry	Open Pantry of Greater Lowell provides 5 days of groceries to hungry persons in the Greater Lowell area. They must wait 30 days to return.	Require Social Security Card and Proof of Residence.	Free	\N	Services are available year round.	Tuesday-Thursday 9AM-12PM\r\nWednesday 5:30PM-7:30PM	English	No	Social Security ID is needed for all members of the family, as well as proof of residence.		Very limited on street parking metered parking.  DO NOT park in front of the building.  Those who do risk a $150 fine.	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact.	No	\N	No	\N	Yes	Twice a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	Occasionally have someone available who speaks Spanish and or Portuguese	No	0	\N
507832	103146	2008-12-23	\N	Non-Profit Alliance of Greater Lowell	NPA	c/o UMass Lowell Center for Family, Work, and Community	600 Suffolk Street, First Floor South	Lowell	MA	01854	978-934-4682	\N	\N	978-934-3026	http://www.npagl.org/	public_program_email@testing123.net	Association of nonprofit organizations	The Mission of The Non Profit Alliance is to build collaboration that strengthens the influence and impact of community-based organizations through sharing resources and advancing excellence in administration.	Any nonprofit organization that wants to complete an application and join	Yes	$100 annual dues requested, but organizations that cannot pay the full amount are requested to make a contribution as they see fit. Membership in the organization is not based upon the ability to pay the full amount of the dues.	monthly meetings, plus special events	monthly meetings are the first Wednesday of the month from 8:30am-10:00am, except in the months of July and August	English	No	\N		visitor parking	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	email David Turcotte at: info@npagl.org	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Merrimack Valley	\N	Yes	0	\N
509394	103487	2008-12-29	\N	Youth Villages, Inc.	\N	400 West Cummings Park	\N	Woburn	MA	01801	781-937-7913	\N	781-937-7924	781-937-7901	www.youthvillages.org	public_program_email@testing123.net	Intercept Program - Intensive In-home Services	Youth Villages provides the Intercept program in Alabama, Florida, Georgia, Massachusetts, Mississippi, Tennessee and Virginia.\r\n\r\nIntercept provides treatment to troubled children and families in their own homes at times convenient for the families. Counselors with small case loads -- four to five families -- focus on helping the child and family at home, in school, and in the community. The program serves children of any age (infant to 18) who have serious emotional and behavioral problems.  Intercept is based on the principles and practices of research-based mental health treatment.  Youth Villages' fourteen years of data from on-going outcome evaluation studies have demonstrated that 80% of youth are successful up to two years after discharge from the program, although random assignment clinical studies have not yet established the effectiveness of the program.\r\n\r\nIntercept serves a broad population of youth, including those involved with multiple child-serving systems and those at high risk of removal from their families.  The program specializes in diverting youth from out of home placements by helping their families safely maintain youth in their home environment.  Intercept also focuses on reunification of youth who are in a residential treatment or foster home setting; counselors are skilled at reuniting families even when the child has been out of the home for an extended period.\r\n\r\nMany of the elements of the Intercept program are common to a variety of intensive family preservation programs including:\r\n\r\nservices provided to the family, rather than just the identified youth  \r\ntime-limited services, usually lasting three to six months \r\nintensive services -- we meet with families at least three times weekly and are on call 24/7 \r\nservices are delivered in the family's home or wherever the family needs us (school, court, etc.) and are flexibly scheduled to meet the family's needs \r\nservices are designed to meet each individual family's needs; ongoing assessment and review of treatment progress assures individualized service \r\nsmall caseloads -- counselors are responsible for four to six families; supervisors are responsible for four to five counselors\r\nSince 1994, Youth Villages has helped nearly 15,000 children in their homes and communities.  Treatment for reunification cases averages six to nine months, and four to six months for prevention or diversion.  All treatment is family-centered and includes strength-based interventions.	Youth Villages Offers Help To\r\n\r\n\r\nHard-to-help, hard to place children; children who have failed at numerous other programs often find success here \r\nChildren with emotional and behavioral problems plus co-occurring medical issues \r\nChildren who have severe acting out behaviors \r\nBoys and girls, ages 7-18 \r\nChildren with Verbal IQ 50 and above \r\nChildren with problem sexual behaviors \r\nEmotionally-disturbed children with developmental delays or disabilities \r\nChildren who are medically fragile	Yes	Referrals made through Department of Child and Family Service.	Open Year Round	Please call.	English!DELIM!Spanish	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	Referrals made through Department of Child and Family Service.	400 West Cummings Park	\N	Woburn	MA	01801	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Currently expanding to Worcester area	\N	Not Applicable	0	\N
508118	103322	2008-12-30	\N	Adolescent Sibling Pregnancy Prevention	SIBS	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6600	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Assists youth setting goals and finding health information.	The SIBS Program, formally known as the Adolescent Sibling Pregnancy Prevention Program, is a voluntary program designed to prevent early and unplanned pregnancy for the siblings of pregnant and/or parenting teens in the City of Lawrence, MA.\r\n\r\nThe program assists youth in setting goals for their future by providing case management, referrals to needed resources, transportation, health information, and education on topics related to preventing teenage pregnancy.\r\n\r\nSIBS meets two to three times a month. One of the meetings is for a Wellness Group. This is when topics such as self esteem, contraception, and decision making skills are discussed. During the second or third meeting, the group goes out on such activities such as the movie theater, bowling, camping, Red Sox or Celtics games.	SIBS is open to adolescents in the city of Lawrence who are between the ages of 11-17 with a pregnant or parenting sibling.	Free	\N	SIBs meets two to three times a month.	Contact for more information.	English!DELIM!Spanish	Yes	Proof of residency in Lawrence.		Locations vary according to activity.	Transportation and meals are always provided.	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Information and referral provided through program.	Yes	Conference room seats 75 persons.	Yes	Agency newsletter incorporates SIBS	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
502959	100913	2008-12-30	Hidden on 06-12-2008 by: the MVHub Administrator<BR>Reason: <i>unknown email address -- agency location?</i>	Access to Early Education and Care Services	\N	600 Washington Street	350Main St.  Malden, MA 02148	Boston	MA	02111	617-988-6600	\N	\N	\N	www.mass.gov/eec	public_program_email@testing123.net	Information about child care and early education	The Department of Early Education and Care's web site is the place for parents and child care providers to get answers to their important questions about child care and early education.The agency aim is to make sure all children receive the highest quality child care from providers in Massachusetts.	Open to all Mass residents	Free	\N	Services available year round	contact program for more information	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
509405	100484	2009-01-05	\N	Middesex Community College TRiO Programs and GEAR UP:MASS	Upward Bound, Talent Search, and GEAR UP	Middlesex Community College	33 Kearney Square	Lowell	MA	01852	978-656-3401	800-818-3434	\N	978-656-3150	www.middlesex.mass.edu	public_program_email@testing123.net	College Awareness Programs for Lowell Public School Students	Provide academic enrichment services and early college awareness to low income and potential first generation college students.	Low income	Free	\N	Summer school programs	School Hours	English!DELIM!Spanish!DELIM!Khmer	Yes	Proof of income, and parent and guardian signature		\N	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	\N	Yes	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	Only at the following target schools: Lowell High School, Pyne Arts, Butler, and Stoklosa middle schools.	\N	Not Applicable	0	\N
509427	100484	2009-01-05	\N	The Transition Program at Middlesex Community College	Transition Program	591 Springs Rd.	\N	Bedford	MA	01852	781-280-3630	800-818-3434	\N	978-656-3150	www.middlesex.mass.edu/transition	public_program_email@testing123.net	The Transition Program - an alternative certificate program.	Designed expressly for students with significant learning disabilities, MCCs nationally recognized Transition Program is a two-year, noncredit certificate-of-completion program that teaches consumer and business skills, independent living, and personal and social development. The cornerstone of the program is an internship each semester in local business settings. \r\nThe  Transition Program helps students who would find traditional college-level academics too challenging, even with extensive support services, achieve their potential through specialized coursework and hands-on experience in work settings.  Students work two days per week in competitive employment situations in office and business support environments. During the other three days, students complete specially designed courses in the following areas: computer skills and vocational preparation, consumer education, personal and social development, job seeking and job survival skills.  Small class size ensures that students receive individualized attention and a program tailored to meet their unique needs.  Graduates receive a Certificate of Completion in Office and Business Support Skills. The Transition Program is selective, accepting 20 students each year.  Applicants and their families must attend an information sessions; held periodically throughout the year.	For information about the program and application procedures, call 781-280-3630.	Yes	MCC per credit tuition rates; based on 15 credit courseload	Full-time, Fall and Spring semester - 2 year certificate program	M-W-F - 8:30-12:30 (Bedford Campus only)\r\nT-Th 8-4 Internship placements (Middlesex and Suffolk County)	English	Yes	selective admissions - attend an Information Session for details	13 Billerica via Edson	\N	not provided	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	Meeting space is available when space not occupied. Contact Mandy Vozzo, 978-656-3106 for information	Yes	student produced newsletter each year; available from website: www.middlesex.mass.edu go to Disability Support then to Transition Program-Alumni News	Yes	www.middlesex.mass.edu go to Disability Support then to Transition Program	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
503278	100484	2009-01-05	\N	English Learner Institute (ELI)	ELI	33 Kearney Square	\N	Lowell	MA	01852	978-656-3267	\N	\N	978-656-3322	www.middlesex.cc.ma.us	public_program_email@testing123.net	ESL or Basic English as preparation to attend college.	The English Learner Institute (ELI)is a  non-credit program that focuses on developing basic English and skills essential to American culture such as technology, life skills, and social conventions.  Anyone who is a newcomer to the United States or needs to develop their basic English skills is suitable for this program.	Must be a high school graduate age 17 or older	Free	The EL Institute offers PAC (Prepare to Attend College) classes for free. Basic English part 1 and 2 along with Pronunciation classes, American Expressions and Workplace Communication Skills are offered for $200.	Program runs in fall, spring and summer semesters. Registration varies, please call for informaton. Most classes meet for 42 instructional hours.	International & Multicultural Office Hours: Monday-Friday 8AM-5PM; ELI class schedules vary. They are held Monday, Tuesday, Wednesday, Thursday & Friday in the day and evening.	English!DELIM!Spanish!DELIM!Khmer	Yes	Contact ELI staff to register.	09 Downtown Circulator	Parking available in the Lowell Public parking garages and lots, such as the Davidson Street lot and Lower Locks garage.	\N	Yes	No	\N	Yes	When classes fill, a waiting list is used. Students placed on a waiting list will be notified if a space becomes available.	\N	\N	\N	\N	\N	\N	No	\N	Yes	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
501815	100462	2009-01-06	\N	Vocational Rehabilitation Services	Mass Rehab- local area office services	325 Chelmsford St.	\N	Lowell	MA	01851	978-458-4544	800-245-6543	\N	978-937-9879	www.state.ma.us/mrc/vr/towns.htm	public_program_email@testing123.net	Provides vocational training and resources	The Vocational Rehabilitation Services program provides:\r\n* vocational evaluation and counseling\r\n*referrals for additional supportive services\r\n*financial assistance for vocational training\r\n*handicap vehicle and home modification services\r\n*a job club, resume, Internet, job placement and follow-up services. All applicants must go through an eligibility process and be accepted in order to access services.	Ages 16 and up. Individuals must meet eligibility of having a significant disabling condition(s) that limits functioning and is a handicap to employment	Sliding Scale	No fees charged for evaluation services. There may be fees for training, if so, financial need determination reviewed with counselor	No predetermined length of time, customers proceed at their own pace. There is a 3 month-long waiting list for paid services.	Monday-Friday 9AM-5PM	English!DELIM!Khmer	Yes	Any relevant medical information with dates and specific diagnosis identified.	16 Chelmsford via Chelmsford Street	Parking available on site.	Located on LR TA #16 Chelmsford St. route. While in training program, transportation stipend may be given	Yes	No	\N	Yes	Prior to implementing any  paid services under an Individual Plan for Employment (IPE), a waiting period of six(6) months is requirement from the date of application.	Services are individually tailored to the customer's need - everyone will meet with a qualified vocational rehab. counselor	\N	\N	\N	\N	\N	Yes	Volunteers opportunities are occasionally available. Interested individuals should call Carolyn Langevin at ext. 311	No	\N	Yes	Limited space for up to 15 people limited to 9-5PM hours available by appointment only. Call 978-458-4544 x303	No	\N	Yes	www.mass.gov	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	Concord, Carlisle, Boxboro, Wilmington, Burlington,	\N	No	0	\N
500286	100044	2008-06-17	\N	Behavioral Health Services -Inpatient	\N	365 East Street	Tewksbury Hospital	Tewksbury	MA	01876	978-858-0533	\N	\N	978-858-0473	www.lchealth.org	public_program_email@testing123.net	Inpatient detoxification program	Behavioral Health Services Inpatient program provides inpatient detoxification from opiates and alcohol, to inluding pregnant addicted women.  Short-term and long-term aftercare placements referrals are given as well as intensive outpatient counseling and acupuncture.	Ages 18-65. Referrals required for long term substance abuse treatment and methadone management / detoxification.	Sliding Scale	$200.00 per day for those that do not qualify for free care. Insurance: Mass Health, Network Health and Neighborhood Health. Some private insurances and sliding scale fees are available. Call to meet with a Health Benefits Coordinator to apply for insurance programs @ 978-746-7777.	Program runs 365 days/year. Individual stays may last 4-6 days. Average waiting list 3-5 days (may be waived).	24 hours / day seven days per week	English!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Contact program with information about the substance abuse problem. Bring approximately 3 days clothing and toiletries for stay.		Must be dropped off. No overnight parking allowed at hospital.	Transportation to follow-up referrals.	Yes	No	\N	Yes	Average waiting list 3-5 days (may be waived). Please call for more information: 978-858-0533.	978-858-0490 TDD Number	Joan Psomos	P.O. Box 359	Tewksbury	MA	01876	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		978-858-0490	\N	\N	No	1	\N
508085	103311	2008-06-18	Hidden on 06-18-2008 by: the MVHub Administrator<BR>Reason: <i>temporary -- unnapproved</i>	Clean Up Crusade	\N	112 A East Haverhill Street	\N	Lawrence	MA	01841	978-975-8793	\N	\N	978-975-3294	http://www.lmcconline.com/programs/index.htm	public_program_email@testing123.net	A grassroots effort to keep Lawrence clean	This is an LMCC organized project, in partnership with community groups, faith based organizations, businesses, local government and residents. We have joined neighbors and organizations in a grassroots effort to keep the city of Lawrence clean. Volunteers paint over graffiti, clean streets and vacant lots, plus plant for the Fall.	Residents and volunteers clean areas of Lawrence.	Free	\N	Focused on summer-time and / or outdoor activities.	Meetings and events held regularly. Contact LMCC for details.	English!DELIM!Spanish	Yes	Contact LMCC for times and dates of events and to see other organizations participating.		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Contact LMCC about volunteering your time or connecting your organization with Clean Up events.	Not Applicable	\N	Yes	Conference room seats 20 persons	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	Not Applicable	1	\N
505071	101837	2008-06-19	\N	One Tough Job	\N	55 Court Street	4th floor	Boston	MA	02108	617-722-8957	888-775-4543	617-722-8957	617-727-8997	www.onetoughjob.org	public_program_email@testing123.net	On line parenting tips and information	Being a parent is one tough job. Full of many rewards, beautiful moments, great sharing, laughs and love. But the reality of raising children today is that parents become overwhelmed, exhausted and stressed.\r\nWe get it. We're the Massachusetts Children's Trust Fund and we know how hectic the life of a parent is today. In communities throughout the state and in the pages of this website, we provide parents with the expert information, tips and support they need and deserve to be the best parent they can be.	No restrictions	Free	\N	Year round	24/7	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Over 300 curricula, technical assistance resources, videos, and other material available free to community-based groups to better meet the needs and interests of families.	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
501771	100451	2008-06-19	\N	Child Care Information & Referral Services	\N	190 Hampshire Street	\N	Lawrence	MA	01840	978-686-4288	877-823-2273	\N	978-975-3120	www.childcarecircuit.org	public_program_email@testing123.net	Provides child care resource and referral information	Child Care Circuit provides child care resource and referral information free of charge to the general public. Circuit also provides low-cost parent and provider trainings, manages the local state voucher program parent and provider trainings, manages the local state voucher program, wait list information and provides management for other state funded subsidies. CCC also provides corporate work life benefits for local businesses, offers child care center development consultation and maintains a resource room for parents and providers. NAEYC consulting is also available.\r\nA training calendar is published three times each year, June, September and January, and is available on-line or via mail. Parent tip sheets are available on-line as are assorted training publications.	Depends on the program. For basic resource and referral info there are no eligibility restrictions.	Free	Resource & Referral is free. Trainings have a small fee. Corporate benefit programs have a fee.	Services available year round	Monday 9AM-7PM; Tuesday-Friday 9AM-5PM	English!DELIM!Spanish!DELIM!Khmer	Not Applicable	R&R services are available via telephone and via our web site at www.childcarecircuit.org Voucher services require an appointment.		Parking available	\N	Yes	No	\N	Yes	Child Care Circuit manages the EEC Income Eligible wait list for subsidized child care. Register on-line at www.childcarecircuit.org	Contact us at info@childcarecircuit.org for any additional questions.	\N	\N	\N	\N	\N	No	\N	Yes	Contact Child Care Circuit for more information on the resource library	Yes	Call 978-722-9999 for more information.	Yes	Offers information on available trainings.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Yes	1	\N
507161	102750	2008-06-19	\N	MRC Turning 22 Program	\N	27 Wormwood Street	\N	Boston	MA	02210	617-204-3600	\N	\N	617-727-1354	www.mass.gov/mrc	public_program_email@testing123.net	T22 Program	The Turning 22 Program was established in 1986 to design and implement transition services for young people who have a severe physical disability with a mobility impairment, and who would need residential services after high school graduation or turning 22.	High school aged students who have a physical disability with a mobility impairment.  Must be 688 eligible.	Free	\N	24/7	n/a	English	Yes	\N		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	Not Applicable	1	\N
501551	100407	2008-06-19	\N	Dept. of Music Concert Series	\N	35 Wilder Street	Suite 3	Lowell	MA	01854	978-934-3850	\N	\N	\N	www.uml.edu/umlmainpage/calendar.asp	public_program_email@testing123.net	Free Faculty and Ensemble Concerts	The UMass Lowell Department of Music offers its concert series free of charge to the public.  Concerts and recitals by music faculty, guests and ensembles are offered throughout the school year.	open	Free	\N	Program available September through May	Call or visit for schedule information	English	Not Applicable	Contact program for more information		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
501133	100165	2008-06-19	\N	Family Counseling Center	\N	439 South Union Street	\N	Lawrence	MA	01843	978-682-9222	\N	800-422-3035	978-681-9508	www.mspcc.org	public_program_email@testing123.net	Provides mental health counseling	The Family Counseling Center provides mental health counseling and outreach in homes or other community settings. We provide office-based psychiatry with Board Certified Adult, Child and Adolescent psychiatrist.	Services available to individuals ages 3-65 and families in the Merrimack Valley	Yes	Third party reimbursement	Contact program for more information	Monday,Friday 9AM-5PM ; Tuesday,Wednesday, Thursday 9AM-7PM, with phone coverage after hours	English!DELIM!Spanish	Yes	\N		Ample parking in Lawrence	Services provided primarily in homes or community settings	Yes	No	Lawrence Office is handicap accessible	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	One conference room in Lawrence may be available. Sign up through receptionist	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	1	\N
504422	101606	2008-06-19	\N	Travelling Food Pantry	\N	95 East Haverhill Street	P.O. Box 447	Lawrence	MA	01841	978-685-8321	\N	\N	978-689-2006	NeighborsinNeed.net	public_program_email@testing123.net	Food pantry serving families in Greater Lawrence	Clients may visit any of our 8 pantries once a week, bur are limited to only one bag of food per household per week. Please arrive at least an hour earlier than the time listed for the pantry, because it is first come first serve.\r\n\r\nAlso, we work with our clients to be sure that they are accessing appropriate services including food stamps and WIC, make referrals when appropriate and counsel families as they strive to become self-sufficient.	We serve those who present themselves in need without qualification. Only one bag of food per household per week.	Free	\N	Open Year Round, Closed whenever school in Lawrence is cancelled due to snow. Closed on most holidays please call to be sure	Our distribution points are located at:\r\n\r\nTuesday 9:00 a.m.\r\n198 South Broadway, Lawrence                       \r\n\r\nTuesday 10:30 a.m.                       \r\n207 Haverhill Street, Lawrence\r\n\r\nWednesday 9:00 a.m. \r\n96 East Haverhill Street, Lawrence                   \r\n\r\nWednesday 10:30 a.m.\r\n1411 Penney Lane, Methuen                 \r\n\r\nThursday 9:00 a.m. \r\n136 Water Street, Lawrence                     \r\n\r\nThursday 10:00 a.m. \r\n297 South Union Street, Lawrence                     \r\n                                                            \r\nFriday 9:30 a.m.   \r\n22 Warren Street, Lawrence	English!DELIM!Spanish	No	\N		Yes	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are needed for food distribution, warehouse operations and special food programs. Call (978) 685-8321	Not Applicable	\N	Not Applicable	\N	Yes	4-6 pages with news and updates on programs	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	1	\N
508096	101606	2008-06-19	\N	Infants In Need	\N	95 East Haverhill Street	P.O. Box 447	Lawrence	MA	01841	978-685-8321	\N	\N	978-689-2006	NeighborsinNeed.net	public_program_email@testing123.net	Supplies for parents and caregivers of infants.	The Infants In Need program provides formula, diapers and other baby products to over 500 mothers and caregivers of infants and toddlers annually. All food, formula and diapers are provided free of charge to those who request them.\r\n\r\nWe work with our clients to be sure that they are accessing appropriate services including food stamps and WIC, make referrals when appropriate and counsel families as they strive to become self-sufficient.	We serve those who present themselves in need without qualification. However, individual households may not receive formula more than once a month.	Free	\N	Open year round.	M-F, 9am-13pm	English!DELIM!Spanish	No	Photo IDs of parents and caregivers.		Parking at office site.	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Yes.	Yes	\N	No	\N	Yes	Periodic.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA	\N	\N	\N	No	1	\N
506831	100616	2008-06-20	\N	Behavioral Treatment Residence	BTR	100A Haverhill Street	\N	Methuen	MA	01844	978-682-5276	\N	\N	978-688-4932	www.st.annshome.org	public_program_email@testing123.net	Residential Care for Children and Adolescents	Children and Adolescents in our BTR program and receive treatment for their emotional/behavioral/mental health issues, but attend public school.	Children ages 5-17 referred by a state agency such as DSS or DMH	Yes	\N	year round	7 days, 24 hours	English!DELIM!Spanish!DELIM!Khmer	Not Applicable	\N		Ample	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Yes	Conference rooms are occasionally made available for public or professional education use. Call for more info.	Yes	\N	Yes	Web site	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	1	\N
506820	100616	2008-06-20	\N	Community Based Acute Treatment	CBAT	100A Haverhill Street	\N	Methuen	MA	01844	978-682-5276	\N	\N	978-688-4932	www.st.annshome.org	public_program_email@testing123.net	Hospital Diversion Program	Our CBAT program is a short term in patient stabilization program for children and adolescents who are experiencing an acute psychiatric episode	Paid for an authorized by Mass Health/Medicaid and other health insurances.	Yes	Paid for by Mass Health/Medicaid and other health insurances	Year round, 24 hrs per day	7 days/24 hours	English!DELIM!Spanish!DELIM!Khmer	Not Applicable	Admission must be approved by Mass Health or Insurance Company		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Yes	Conference rooms are occasionally made available for public or professional education use. Call for more info.	Yes	\N	Yes	St. Ann's Home web site	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
503740	100616	2008-06-20	\N	Day Treatment Program	\N	100A Haverhill Street	\N	Methuen	MA	01844	978-682-5276	\N	\N	978-688-4932	www.st.annshome.org	public_program_email@testing123.net	Special Educatio/ Day Program	Chapter 766 approved day school for emotionally and behaviorally disturbed children.	Eligibility is determined by the local school district.	Yes	\N	School year	8:30-2:45	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Yes	At times, there may be a short wait list.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Conference rooms are occasionally made available for public or professional education use. Call for more info.	Yes	Twice a year	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	All towns within a one hour range from Methuen	\N	No	1	\N
502057	100616	2008-06-20	\N	Outpatient / Outreach Services	\N	100A Haverhill Street	\N	Methuen	MA	01844	978-682-5276	\N	\N	978-688-4932	www.st.annshome.org	public_program_email@testing123.net	Outpatient/Outreach program for mental health	St. Ann's Home Outpatient / Outreach Program includes home or community-based mental health and case management services for at-risk families. Services include individual and family therapy, psychiatric services, and case management. Other services on the campus of St. Ann's include residential treatment, and a special education school.	Department of Social Service referral, MassHealth or other third party insurance is required	Yes	Third party billing is available.	Program services are available year round, enrollment is ongoing. At times there may be a short wait for services.	Monday-Friday 8:30AM-5PM with evening and weekend emergency coverage	English!DELIM!Spanish!DELIM!Khmer	Yes	DSS referral required for Intensive Family Intervention Program. Mental Health clinic is accessible without referral		Yes	\N	Yes	No	\N	Yes	At times there may be a short wait for services.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Conference rooms are occasionally made available for public or professional education use. Call for more info.	Yes	Twice a year, January and June	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
502882	100880	2008-06-20	\N	Adult / Adolescent Immunizations	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0932	\N	\N	978-671-0919	www.town.billerica.ma.us	public_program_email@testing123.net	Immunizations for adolescents & adults	The Billerica Board of Health offers adolescents and adults (high risk or attending college) the following immunizations: Td boosters, MMR boosters, and the Hepatitis B series-depending upon availability.	Billerica residents	Free	\N	Program runs year round, times vary	By appointment	English	Yes	Immunization record(s)if available.\r\nCollege health forms		Parking available in front lot of Town Hall	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA	\N	\N	\N	No	1	\N
504928	101749	2008-06-20	\N	English as a Second Language Program	Center for International Education  Salem State College	8 Harrison Road	\N	Salem	MA	01970	978-542-6351	\N	\N	978-542-7104	http://www.salemstate.edu/cie/	public_program_email@testing123.net	English as a Second Language- all levels	Our ESL program prepares students and professionals. Class sizes are small and students come from all over the world. Courses include: Conversation, Reading and Writing, Academic Skills, TOEFL preparation. The Center for International Education provides academic and immigration advice (including I-20's).	students must be over 18 and have completed high school, in the US or in their home country	Yes	$150-920 per 12-week course	Three twelve-week semesters, year-round.	Monday-Friday 7:45am-9:00pm	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish	No	no documents required		Available	\N	Yes	Not Applicable	\N	No	\N	\N	Center for International Education	Salem State College 352 Lafayette Street	Salem	MA	01970	Yes	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	Arabic, Japanese, German, Tagalog, Swedish	Not Applicable	1	\N
506996	102662	2008-06-20	\N	Dept. of Mental Health, Essex North Office	\N	15 Union St., Suite #2	\N	Lawrence	MA	01840	978-738-4500	\N	\N	978-738-4559	www.state.ma.us/dmh	public_program_email@testing123.net	Support services for individuals with severe mental illness	DMH seeks to support adults with severe mental illness, and children/adolescents with severe emotional disturbance, as they engage in their journey toward rehabilitation and recovery.  Potential services are: Case Management, supports to help people live as independently as possible, and programs to help them move along in goals toward education and employment.	Individuals are determined eligible based on the severity and type of their mental illness, how long it has (or may) go on for, and how much this illness affects their daily functioning.  Applications are available on the DMH website in the Service Application Forms area.	Free	\N	Open year round.	9 am - 5 pm	English!DELIM!Portuguese!DELIM!Spanish	No	If you want to file an application, bringing your psychiatric/medical records is helpful.  Otherwise, these will be requested if a Release is signed.	01 Haverhill - Methuen - Lawrence!DELIM!32 Andover!DELIM!33 North Andover!DELIM!34 Prospect Hill!DELIM!35 Water Street!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!37 Beacon St.!DELIM!38 Hampshire St.!DELIM!39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips!DELIM!40 Methuen Square!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence!DELIM!83 Salisbury Beach / Hampton Beach	On-street parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Boxford, MA!DELIM!Merrimac, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	DMH is a statewide agency.  The Essex North Site Office serves the towns indicated.	\N	No	1	\N
505962	100187	2008-06-23	\N	Advocacy Department	\N	6 Southside Road	\N	Danvers	MA	01923	978-762-8352	\N	\N	978-762-3980	www.nsarc.org	public_program_email@testing123.net	Educational Advocacy and Transition into Adulthood Advocacy	Parents are often overwhelmed by the complexity of special education laws that entitle a child with disabilities to a free and appropriate education in the least restrictive environment. Having an advocate guide them through the process enables parents to become more active members of their childs TEAM. \r\n\r\nAdvocates work cooperatively with families and schools to develop an appropriate individualized education program that meets the individual needs of the child. By keeping up to date on federal and state laws that entitle children with disabilities to appropriate programs and services, our advocates provide consultation and record review, as well as representation at TEAM meetings and mediations. The advocates also provide families with referrals to various resources both in and outside their school systems, and confer with medical professionals, counselors, and educational consultants in an effort to develop a full understanding of the childs needs.\r\n\r\nThe Advocacy Department also offers free training workshops throughout the school year on:\r\n\r\n\r\nOverview of state and federal special education laws \r\nEntering special education, the referral process, and getting organized \r\nEligibility \r\nDeveloping an appropriate individualized education plan \r\nOther educational needs: social/emotional, vocational, assistive technology, communication, behavior, travel training, extracurricular activities \r\nMCAS: accommodations, appeals process and student success plan \r\nSummer programming: recreational, academic and therapeutic \r\nResolving IEP disputes through due process \r\nTransitioning from Early Intervention \r\nTransitioning to adult services (chapter 688)	fee for service	Yes	\N	Monday thru Friday	9:00 to 5:00	English!DELIM!Spanish	Yes	Copies of current year and previous 3 year's I.E.P.'s.		yes	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Contact North Shore Arc for information about the resource library	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	1	\N
501749	100187	2008-06-23	\N	Autism Support Center	\N	6 Southside Road	\N	Danvers	MA	01923	978-777-9135	\N	\N	978-762-3980	www.nsarc.org	public_program_email@testing123.net	Provides support services to families affected by ASD	The Autism Support Center provides support services to families of children with autism spectrum disorders, including: information and referral, support groups, quarterly newsletter, resource library, parent and professional training, conferences and some limited, short-term flexible funding.  Broker services are provided under the DMR Autism Waiver to eligible families.	Age 0-18 living in Northeastern Massachusetts. For short-term funding, or Waiver-related supports, eligibility for DMR services is required	Free	Fees are requested for some trainings or activities which are not otherwise funded.	Services available year round.	Monday-Friday 9:00AM-5PM	English!DELIM!Spanish	No	\N		\N	Most services provided by phone or off site, only the resource library is on site (materials can be sent if preferred)	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	contact Suzanne Ryan at sryan@nsarc.org for futher information on volunteer opportunities.	Yes	Extensive collection of materials on autism spectrum disorders.  Additional material on other developmental disabilities and special education.	Yes	Limited availability.  May be made available to affiliates/collaborators upon request.	Yes	The Spectrum, available on line or by mail.	Yes	Available in library	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	North Shore, Cape Ann, Metro-North, Central Middlesex, Cambridge/Somerville/Waltham/Belmont	We will attempt to procure interpreter services in other languages (including ASL) when the need arises.	Not Applicable	1	\N
505973	100187	2008-06-23	\N	Building Blocks	\N	6 Southside Road	\N	Danvers	MA	01923	978-762-8352	\N	\N	978-762-3980	www.nsarc.org	public_program_email@testing123.net	Internsive Early Intervention Program for Children/Autism	BUILDING BLOCKS Intensive Early Intervention is a program that supplements early intervention services. Through this program, the North Shore Arc has been certified by the Department of Public Health (DPH) as a Specialty Service Provider to provide intensive intervention services to children under the age of three who have a diagnosis on the autism spectrum.\r\nOur educators specialize in working intensively with young children who have a diagnosis on the autism spectrum. The educators provide home-based services starting with five hours per week and increasing to 30 hours or more per week, depending on the needs of the child and the family.	Referral by Early Intervention.  Must have diagnosis of Autism	Free	\N	open year around	Monday Thru Friday	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Contact North Shore Arc for information about the resource library	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
501760	100187	2008-06-23	\N	Personal Care Services	\N	6 Southside Road	\N	Danvers	MA	01923	978-624-2330	800-231-5409	\N	978-777-7362	www.nsarc.org	public_program_email@testing123.net	Provides in-home support for individuals with disabilities	The Personal Care Services program provides in-home support for individuals with cognitive or physical disabilities. They must need hands on assistance for personal care and be Medicaid eligible.	Must be Medicaid eligible. Referrals to the program are accepted from anyone	Free	Medicaid funded program	available year round. There is no waiting list, but it does take approx. 3 months to get approval	Monday-Friday 9AM-5PM	English!DELIM!Spanish	Yes	Call with information about disability, doctor, in & out of home supports, reason for needing service; Social Security #		\N	Services are provided in the client's home	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Contact North Shore Arc for information about the resource library located in the Autsim Support Center	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	We also serve; North Shore, Metro North, Worcester, North Central, Dorchester, Boston, Middlesex West, Central Middlesex, Charles River West, Harbor and Newton South Norfolk.	Russian	No	1	\N
505951	100187	2008-06-23	\N	Recreation	\N	6 Southside Road	\N	Danvers	MA	01923	978-762-8352	\N	978-762-4878	978-762-3980	www.nsarc.org	public_program_email@testing123.net	Recreation and Leisure Activities	The goal of our Recreation service is to give individuals the opportunity to participate in social and recreational activities for personal growth and development. This goal is met by implementing various programs and supports so that people can be successful in various recreational settings.\r\n\r\nThe recreation staff includes a Recreation Director, Recreation Specialist, and a number of relief staff whom each offer expertise in a specific recreational area. The recreation department also benefits from a number of volunteers who support the staff in various areas.\r\n\r\nThe recreation department offers an inclusive summer camp program, Life Skills Group, Bowling Leagues, Swim Programs, Art Groups, Aerobics Class, Sports Teams, Friendship Club Dances, Teen and Women Social Groups and other individualized supports.	open enrollment and inclusionary	Yes	\N	Monday thru Saturday.  Year round	Afternoons and Evenings	English	No	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Contact North Shore Arc for information about the resource library	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	1	\N
505676	102079	2008-06-19	\N	Storm Corps	\N	101 Main St	\N	Salem	NH	03089	978-416-2002	\N	\N	\N	\N	public_program_email@testing123.net	Full-Time Volunteer Program	Youth Storm is a movement that unites, equips and mobilizes the generations to become active voices through service in their communities and world through volunteerism.\r\n\r\nYouth Storm interns come from many different backgrounds and contexts, including Teen Challenge and other faith-based rehabilitative centers.\r\n\r\nStromp Corps is a full-time volunteer program. The Storm Corps interns participate in prayer, academic classwork, and 12 hours of community outreach and service work per week.	18 and older	Free	\N	Varies depending on the volunteer schedule and any special events	Varies	English	Yes	\N		\N	\N	No	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	If you speak another language, great!	No	1	\N
505808	102079	2008-06-19	\N	The Training Center	The Training Center	7 Main St	\N	Salem	NH	03089	978-416-2002	\N	\N	\N	\N	public_program_email@testing123.net	The Training Center	The Training Center has evolved into a holistic training center focused on equipping the student in the areas of character development, relational development, leadership development, inner healing and identity formation, conflict resolution, responsibility management, as well as biblical academics.\r\n\r\nThe Training Center has branched off into three major focuses to facilitate this goal. The three areas include the School of Leaders, which is the academic focus. The Discipleship program is the full-time residency program focused on the character development, inner healing, identity formation, relational development, and conflict resolution focuses. \r\n\r\nThe Internship program is the full-time working internship with the YouthStorm leadership team, focused on the leadership development, responsibility management, and vision casting aspects of the Training Center.	All	Yes	Charges are based on the individual class and there is a discount for taking more that one class	Evening classes	Evening classes available	English	Yes	\N		\N	\N	Not Applicable	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	1	\N
503003	100957	2008-07-01	\N	Challenge Unlimited, Inc.	\N	450 Lowell Street (Rt. 133)	\N	Andover	MA	01810	978-475-4056	877-632-8386	\N	978-475-4046	www.challengeunlimited.org	public_program_email@testing123.net	therapeutic services for people with disabilities	A nonprofit program that provides therapeutic services for people with physical, cognitive and emotional disabilities and communication disorders using the horse and the working farm environment.	Ages 5 and older. If a need for therapy exists prior to age 5, needs may be met by enrolling in Ironstone Therapy	Yes	Fees vary. Payment options: private pay, insurance (Ironstone Therapy only), grants, third party payments	Program runs in 8 or 9 week blocks year round, in all weather. Call office for registration deadlines - they vary year to year.	Program hours: Monday-Saturday 9AM-8PM; Office hours: Monday-Friday 8AM-5PM	English	Yes	Complete registration process	11 IRS/Raytheon via Rte. 133	Parking available.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers assist with grooming, tacking, leading, side-walking & assisting with special need clients. Training is provided.	Not Applicable	\N	Yes	A tent is available in the spring, summer and fall only. Call for more information.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Yes	1	\N
503014	100957	2008-07-01	\N	Ironstone Therapy, Inc.	\N	450 Lowell Street (Rt. 133)	\N	Andover	MA	01810	978-475-4056	877-632-8386	\N	978-475-4046	www.challengeunlimited.org	public_program_email@testing123.net	Physical, Occupational & Speech Therapies using a horse	Ironstone Therapy is a medically based program that offers physical therapy, occupational therapy and speech-language therapy in a non-traditional setting. Licensed therapists provide hippotherapy, which incorporates the movement of the horse as part of each session, as well as traditional therapy.	Prescription is required for each client. Ages 9 months and older eligible for services.	Yes	Fees vary - most insurance accepted. Private and third-party payer option.	Services available year round. Individual sessions last 30-60 minutes. Enrollment is ongoing.	Monday-Saturday, various times.	English	Yes	Complete necessary paperwork for the program prior to first appointment.	11 IRS/Raytheon via Rte. 133	Parking available in lot on property.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer opportunities are available to lead the horses and sidewalk. Call the office for specifics 978-475-4056	No	\N	Yes	A tent is available in the spring, summer and fall only. Call for more information.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	1	\N
504851	101727	2009-01-15	\N	The Peter Pan Center	\N	280 Ayer Road	P.O. Box 312	Harvard	MA	01451	978-772-1255	\N	\N	978-772-4711	www.peterpancenter.homestead.com	public_program_email@testing123.net	Parenting Resource	Individual/Family Parenting Consultations\r\nParent Education and Support\r\nSocial Skills Programs/Playgroups\r\nADD & Social Skills Coaching\r\nBehavioral Interventions\r\nAnger Management Programs for Children/Families\r\nClassroom Observations\r\nSpecial Education Consultation\r\nTutoring & Learning Specialist Services\r\nSensory Integration Evaluations\r\nTelephone Consultations\r\nSpeaking Engagements & Workshops\r\nProfessional Development Programming\r\nReferrals & Resources	All	Yes	Fees vary depending on service provided	Open year round by appointment	By appointment only	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
503872	101298	2008-07-01	\N	Specialized Foster Care	\N	Everett Mills	15 Union Street, 5th Floor	Lawrence	MA	01840	978-794-7966	866-375-5114	\N	978-794-9890	www.TheMentorNetwork.com	public_program_email@testing123.net	Specialized Foster Care services for children.	Massachusetts MENTORs Lawrence Childrens Program in Lawrence, MA offers Specialized Foster Care services in the service areas North and Northwest of Boston. \r\n\r\nThis flexible program provides youths from troubled families, institutions and other settings  including teen parents and their babies  with a warm, stable environment coupled with the interventions needed to address severe emotional and behavioral disorders often stemming from abuse and neglect. Placement in the home of a specially trained caregiver, called a Mentor, is carefully matched according to each childs needs. Mentors provide children in their care with therapeutic intervention, nurturing support and the consistent guidance in addition to routine family- and community-based activities. A Clinical Coordinator oversees implementation of an Individualized Treatment Plan with emphasis on discharge planning to assure the youths appropriate progress and transition to home, permanent placement or independent living.	MENTOR foster parents must be at least 21 years of age.	Free	\N	Open year round.	Mondays-Fridays 9am-5pm	English!DELIM!Spanish	No	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	\N	No	1	\N
508239	103322	2008-07-02	\N	Samaritans 24-Hour Befriending Hotline	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-688-6607	866-912-4673	\N	\N	www.familyserviceinc.com	public_program_email@testing123.net	Fre and confidential 24-hour crisis line.	The Samaritans provide a free and confidential 24-hour crisis line to those who are lonely, despairing or suicidal. This service is provided by trained volunteers who provide unconditional and non-judgmental TLCtalking, listening and caring. Many callers may be suffering from mental health illness, such as depression, or may be going through a difficult time because of life's stressors. Some are lonely, elderly, and don't have the support of family and friends. Other callers are concerned friends and family looking for advice on how to care for a loved-one who is at-risk for suicide.\r\n\r\nSamaritans also has a state-wide hotline at 877-870-4673.	All persons looking to reach out. In addition, the crisis line also takes calls from individuals looking for referrals to other support services in the area.	Free	\N	Year round.	24-7	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	The Samaritans of Merrimack Valley rely on volunteers who, through the goodness of their hearts, reach out to help those in crisis. People from all walks of life are trained by Samaritans to offer support and caring for those who are lonely, despairing or suicidal. Trainings are held throughout the year. For more information, please call 978-688-0030 or email Samaritans@familyserviceinc.com.	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	1	\N
508514	102948	2008-07-03	\N	Day One	\N	11 Union Street	\N	Lawrence	MA	01843	978-688-4628	\N	978-685-1337	978-681-1281	www.tpc1.org	public_program_email@testing123.net	A rehabilitation program for adults.	Day One offers a wide range of education in areas such as: symptom management, anger management, stress management, relaxation, job-preparation, nutrition, pre-vocational (English, social skills mathematics, reading), hygiene.\r\n\r\nDay One staff members are responsive to each client's individual needs and are always available for support and encouragement. Staff offer regular one-to- one meetings with clients to assess progress and barriers.\r\n\r\nOther special topics and activities include budgeting, substance abuse, expressive arts, current events, quarterly newsletter, Spanish les- sons, occasional community outings, Friday movies, and social events. \r\n\r\nPlease call for more details.	Participants must be currently living within the Merrimack Valley area and have been referred through Lawrence DMH.	Free	Participants must have been referred through Lawrence DMH.	Open year round.	The Day One schedule:\r\n9:00-9:45 Arrive, breakfast available \r\n9:30-10:00 Morning checklist \r\n10:00-10:15 Morning meeting \r\n10:15-1 :00 Group session \r\n11:00-11:15 Break \r\n11:15-12:00 Group session\r\n12:00-12:30 Lunch\r\n12:30-1:00 Chores\r\n1:00-1:30 Outside walk/indoor exercise\r\n1:30-2:30 Social activities\r\n2:30-3:00 Departure	English	Yes	\N		\N	\N	No	Not Applicable	\N	Not Applicable	\N	In addition to these services, we offer a Pre-Vocational Training program for people entering back into the workforce. Registration for the training is independent of all other Day One programming.\r\n\r\nDAY ONE REHABILITATION RULES\r\n1. RESPECT: Personal Boundaries, No Verbal or Physical Aggression\r\n2. NO SMOKING in the program area, only in designated areas.\r\n3. APPROPRIATE attire is to be worn at all times.\r\n4. ATTENDANCE is expected each day. The program must be phoned if you are\r\nnot attending for the day.\r\n5. NO STEALING or BORROWING of money or cigarettes.\r\n6. NO SHARING of food, drinks, or offering to buy others snacks or other items.\r\n7. NO ENTERING OF STAFF OFFICES unless you are being accompanied by\r\nstaff. .\r\n8. VENDING MACHINES will only be visited once a day at breaks at 11 or 1.\r\nTrips to the vending machine should not intenupt individual's treatment.\r\n9. NO CONSUMPTION of alcohol or any illegal substances, in or outside of the\r\nprogram.\r\n10. NO LEAVING the Day One Area unless you are accompanied by Day One staff.	\N	\N	\N	\N	\N	No	\N	Yes	Information focusing on Evidence Based Treatments-Cognitive Behavioral Therapy, Motivational Interviewing, Contingency Management,Substance Abuse, Mental Illness, Dual Diagnosis, Spirituality in Recovery	Yes	Very Large Community Meeting Space,Several Large Group Rooms	Yes	Quarterly newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	1	\N
508624	103344	2008-07-08	\N	Movement City Youth Network	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-685-9393	\N	\N	\N	www.myspace.com/movementcity	public_program_email@testing123.net	Engaging Lawrence youth through arts and techonlogy.	Movement City is an empowerment network for young Lawrencians between the ages of 10-18. Young people explore their potential through design, technology and performing arts. \r\n\r\nMembers can also choose to participate in a wide range of highly creative economic, academic, leadership development and collective action activities.\r\n\r\n*Summer Youth Program*\r\nMovement City hosts 3 activity camps for youth, in the fields of:\r\nVisual Arts & Design\r\nSports\r\nPerformance Arts	Lawrence teens, 10-18.	Yes	Annual fee of $40.00\r\n\r\nSummer Youth Program:\r\n$50 registration fee.\r\nProgramming fee includes 3 field trips, daily lunch and weekly pool visits	Open year round for most program activities.	Office Hours, M-F 9am-5pm\r\n\r\nSummer Youth Program:\r\n4 days a week in the afternoons. Contact for more information.	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508613	103344	2008-07-08	\N	New Skills Academy	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-722-2623	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Improve skills for entry-level adminstrative job-seekers.	This program will prepare participants to achieve their goal of getting better job\r\nopportunities. \r\n\r\nThis program consists of a six month training where you will to learn a variety of skills such as advanced ESL, advanced computer skills, Basic Accounting, and "Soft Skills" such as Etiquette, Time Management, and Dress Code.\r\n\r\nA program of the Department of Family Asset Building (FAB) at Lawrence Community Works.	Participants:\r\n + Need to have an Intermediate ESL level (Test will be given)\r\n + Need to have basic Computer-knowledge (Test will be given)\r\n + Commit to a six month intensive training \r\n + Must be able to work in the USA	Free	\N	Open year round.	Weekday evenings, 4 nights per week. Call for more information.	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	Yes	Child care will be provided.	Yes	Participants must commit to completing the entire program.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly LCW newsletter.	Yes	Provides Internet access to members.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	Reyna Burgos or Annery Butten\r\n978-722-2623 or 978-722-2622
508635	103344	2008-07-09	\N	Individual Development Accounts	IDA Program	168 Newbury Street	\N	Lawrence	MA	01841	978-722-2623	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Matched savings account savings program.	IDA is a savings program, which is an educational program that helps the community of the City of Lawrence to realize homeownership; higher education; or to start their own business. \r\n\r\nClients save $75 every month and recieve $225 monthly for 16 months, attend monthly workshops and get financial education, and work with program to keep and fix their credit.	Residents of the City of Lawrence, must work full or part-time, and must not own more than one house, or own a house that's not the individual's main residence.	Free	A program of the Department of Family Asset Building (FAB) at Lawrence Community Works.	Open year round.	Office Hours:\r\nMonday -- Friday, 9am-5pm	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly newsletter from LCW.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508646	103344	2008-07-09	Hidden on 07-09-2008 by: the MVHub Administrator<BR>Reason: <i>ADD INFO -- temp -- ADD INFO</i>	Real Estate Development	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-685-3115	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Affordable housing units.	Lawrence CommunityWorks develops affordable housing, both rental and first-time homebuyer units.\r\n\r\nLCW owns and manages 71 units in 9 separate buildings, and has almost 200 additional units planned or in process.	ADD INFO	Yes	Rental units and first-time homebuyer units.	Open year round. Development projects in various stages of completion.	Office Hours,\r\nMonday -- Friday, 9am-5pm	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Yes	Ask for referral to management company for openings.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly LCW newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508657	103344	2008-07-09	\N	Homebuyer and Post-Purchase Workshops	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-722-2627	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Homebuyer and post-purchase workshops.	These seminars focus on educating participants on the phases of the home buying process and the provision of critical services during each phase. \r\n\r\nThe participants hear from local professionals directly allowing them access to industry contacts, tools, and materials. Some of the topics discussed in the curriculum include the pros and cons of renting and owning a home, common credit problems and credit-restoring solutions, setting budgeting goals, issues relating to lead poisoning prevention, loan prequaliflcation and, most importantly, avoiding abusive schemes and predatory lenders.\r\n\r\nIn the Homebuying phase, members are guided through every step of the process. The level of counseling and assistance is tailored to the individual. It may involve facilitating their relationships with attorneys, real estate brokers, lenders, and other professionals or simply sewing as a sounding board to reaffirm their decisions. Once homeowners, members are invited to participate in postpurchase counseling workshops which include topics such as avoiding predatory solicitations and smart refinancing, home budgeting, landlord education, maintenance and repair, and foreclosure prevention.	Families considering home purchases in Lawrence, MA.	Yes	$30 fee per family	Periodic workshops.	Seminar includes 2 day-long sessions. \r\nParticipants must attend both sessions (spaced about 1 week apart).	English!DELIM!Spanish	Yes	To enroll in Homebuyer and Post-Purchase Workshops, please contact Ana Bello at 978-722-2627 abello@lcworks.org		\N	\N	Yes	No	\N	No	\N	Homeownership Education Seminars are certified by NeighborWorks, the National Council of La Raza, HUD, CHAPA, and Mass Housing and cover every step of the home-buying process.	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly newsletter from LCW.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508668	103344	2008-07-09	\N	Foreclosure Prevention	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-685-3115	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Foreclosure prevention workshops and assistance.	Intensive foreclosure prevention and mitigation services for homeowners who have fallen behind.  Services include credit counseling, loan modification negotiations, refinancing and short-sale referrals, and financial analysis and planning.\r\n\r\nTo enroll in the up-coming Foreclosure Prevention Workshops, contact Ruth Rojas at 978-722-2612 rrojas@lcworks.org	Anyone with home foreclosure issues, or who needs assistance with loan renegotiation.	Free	\N	Office open year round; contact office to enroll in periodic Foreclosure Workshops.	Monday -- Friday, 9am -5pm	English!DELIM!Spanish	Yes	Contact program to enroll in workshops.\r\n\r\nContact program to get preparation materials necessary for a successful client assessment. \r\n\r\nTo do the assessment, clients may need mortgage agreement, correspondence with lenders, and information such as W-2s to show family income.		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly newsletter from LCW.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508701	103344	2008-07-09	\N	Computer Technology Workshops	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-685-3115	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Beginners Computer, Computer I, Computer II	These classes are designed to teach the skills needed to be able to use a PC computer. They look at the hardware aspect of working with a computer, the Windows operating system, as well as programs such as Microsoft WORD, EXCEL & PowerPoint.	Lawrence Adults.	Yes	$75 per course.	Open year round. Contact to enroll.	Beginners Computer Class\r\nMonday-Wednesday 6-8:30pm\r\nAlternate time:\r\nSaturday 9-12:30pm\r\n\r\nComputer I\r\nSaturday 9-12:30pm\r\n\r\nComputer II\r\nTuesday-Thursday 6-8:30pm	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	Yes	Child care is provided during time of learning, accommodating parents needs.	Yes	First come first served.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly LCW newsletter	Yes	Class teaches computer use to students.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508679	103344	2008-07-09	\N	ESL	English as a Second Language	168 Newbury Street	\N	Lawrence	MA	01841	978-685-3115	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	ESL Beginners, ESL I, ESL II, ESL III	These classes will help students to improve their listening, speaking and pronunciation skills. Students will learn to understand more of the English that they hear in day-to-day interactions. They will also learn to speak more clearly so that others can understand them better.\r\n\r\nA program of the Department of Family Asset Building (FAB) at Lawrence Community Works.	Adults in Lawrence.	Free	Each class is $100.	Open year round.	ESL Beginners\r\nTuesday-Thurday, 6-8:30pm\r\n\r\nESL I\r\nSaturday 9-12:30pm\r\n\r\nESL II\r\nSaturday 9-12:30pm\r\nAlternate time:\r\nThurday-Friday, 6-8:30pm	English	Yes	Contact Reyna Burgos, 978.722.2623		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly LCW newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	Program helps students acquire English.	No	1	\N
508690	103344	2008-07-09	\N	GED and Other Classes	\N	168 Newbury Street	\N	Lawrence	MA	01841	978-685-3115	\N	\N	\N	www.lcworks.org	public_program_email@testing123.net	Pre-GED, GED classes.	These classes will prepare students with the needed skills to earn a Diploma that will enable them to seek higher education or provide greater employment opportunities.\r\n\r\nOther programs include:\r\n\r\nWallet Wise, a class which will concentrate on personal finances. Instruction on the principles of healthy credit, basic banking, simple budgeting and goal setting.\r\n\r\nHands-On Networ, a monthly network workshop that involves the different talents of the network members. Workshop are to include "The Taste of the Month (cooking)," "Beautiful within my Budget (interior design),"\r\n\r\nA program of the Department of Family Asset Building (FAB) at Lawrence Community Works.	Lawrence adults.	Yes	GED costs $100 per course, other classes are free to $20.	Open year round.	Pre-GED Class\r\nMonday-Wednesday 6-8:30pm\r\n\r\nGED Class\r\nSaturday 9-12:30pm	English	Yes	\N		\N	\N	Yes	Yes	Child care is provided during time of learning, accommodating parents needs.	Yes	First come first serve.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	100 persons, call for more information about capacity.	Yes	Monthly LCW newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	1	\N
508888	103377	2008-07-10	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Skills Start Program	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Training program for those elibilge for Food Stamps.	The Skill Start program provides short-term occupational skill training and job search services to individuals who have been determined eligible for Food Stamps by the Massachusetts Department of Transitional Assistance (DTA) and referred to the Division of Career Services.	To be eligible for Skill Start services, an individual must be newly enrolled in the food Stamp Program by DTA and ineligible for other public assistance.	Free	\N	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	Participants must be referred.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	Not Applicable	1	\N
508833	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Career Center Workshops	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Workshops help aquire new skills and job readiness.	The ValleyWorks Career Center offers a variety of workshops to assist people  in gaining new skills and enhancing current skills. Our workshops include professional development topics, various levels of computer software training, career advancement, job readiness, career exploration, job search topics, and English as a Second Language. \r\n\r\nPlease see our workshop calendar for more detailed information, speak with our staff about scheduling a workshop, or visit our website at: www.valleyworks.cc.	Job seekers.	Yes	Some advanced level workshops require a nominal fee.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 7:00PM\r\nEvening hours include Workshops and Career Center Seminars.	English	No	Visit website at www.valleyworks.cc to see cost and time of workshops	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Call to schedule a workshop.	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	Services may be most helpful for those seeking employment in the Greater Lawrence area.	\N	No	1	\N
508767	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp - unapproved</i>	Disability Program Navigator	DPN	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Information and resource coordination for disabled persons.	The Disability Program Navigator is available to help persons in coordinating Career Center and Community Based Organization (CBO) services. Through this position, many resources are available at the Career Center to assist in accessing our services. See other programs of Career Center to understand available services.	Disabled adults and youth.	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Open year round. Please call ahead in case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	Please speak with our Customer Service Representatives about scheduling an appointment with the Disability Navigator.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	\N	\N	Yes	No	\N	No	\N	The role of the Disability Navigator is to:\r\n\r\nIncrease employment opportunities to individuals through coordination of services, promoting program awareness and enhancing a seamless referral system with community service providers.\r\n\r\nAssist individuals with disabilities in accessing and navigating the programs that impact their ability to gain, return or retain employment.\r\n\r\nServe as a resource and trainer to the one-stop career center and other community-based organizations. \r\n\r\nProvide referrals and information for benefits planning, child care, housing, transportation, medical and employment services.	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Services will be most helpful for those seeking services in the Greater Lawrence area.	\N	No	1	\N
508778	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Employment Services Representative	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Help for job seekers with accessing information & event info	Employment Services (ES) staff provide a wide range of job search resources to job seekers as well as employers. ES staff can assist with job search, career direction, community resources, workshops, and  in accessing computer assistance and learning through Career Center workshops. ES staff can also provide information about our on-site employer recruitments and Job Fair events taking place at the career center and in our region.	Job seekers (everywhere).	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Open year round. Please call ahead in case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	No	Speak to Customer Service at the Career Center's main line.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Services will be most helpful for those seeking employment in the Greater Lawrence area.	\N	No	1	\N
508899	103377	2008-07-11	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Trade/TAA Programs	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Job resources for persons effected by foreign competition.	The Trade Adjustment Assistance Act (TAA) provides a variety of services and benefits to eligible customers. Trade services and allowances may include Employment Counseling, Vocational Testing, Job Placement, Supportive Services, Paid Retraining Programs, Job Search Allowances, Relocation Allowances, and Weekly Subsistence Benefit Payments Older Worker Assistance and Health Coverage Tax Credit.	Persons laid off from a trade certified company which has been adversely affected by foreign competition.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508844	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	National Emergency Grants Services	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Employee skills development after economic dislocation.	Federal grants provide funds to support skills development and job search preparedness and assistance to affected workers. \r\n\r\nThe grants are written for specific industries, companies or clusters of companies that are closing or implementing downsizing initiatives of significant proportions.	Workers affected by major economic dislocations such as workforce reductions and company closings.	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	1	\N
508756	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Department of Education Resources	Adult Basic Education	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Direct connection with on-site DOE Representative	The Career Center offers a direct connection with an on-site Department of Education (DOE) Representative. DOE programs enable people of all ages to improve their learning and language skills. These programs may include Adult Basic Education, English as a Second Language, or Family Literacy programs.\r\n\r\nDOE community providers serve the needs of the adult population in need of a high school GED, and provide adults with opportunities to develop the literacy and language skills needed to qualify for further education, job training, and better employment.	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Free	\N	Open year round. Please call ahead in case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English!DELIM!Spanish	Yes	Call to speak with our Customer Service Representatives about scheduling an appointment with our on-site DOE Representative.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	No	1	\N
508866	103377	2008-07-10	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Post Employment Services	PES	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Enhance individuals' job retention and self-sufficiency.	Post Employment Services (PES) program services can help resolve issues that impact an individual's ability to retain employment, as well as assist with job advancement. The services available through the PES program include: transition, support, job retention, re-employment, and job advancement services. PES counselors can work with participants to  help develop goal setting strategies that lead to keeping a job and increasing earnings capacity.	TAFDC recipients who enroll in Employment Service Programs and who are placed into employment are eligible to receive Post-Employment Services.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	Gas allowance in the form of financial payments.	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508789	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Self-Paced Learning Services	SPLS	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Computer based learning tools to enhance career skills.	A broad range of self-paced computer based learning tools are available to help persons enhance skills and achieve personal and professional career objectives.  SPLS is designed to help with literacy, English as a Second Language skills, computer, work readiness, and resume writing skills without the structure of formalized classroom training.	Job seekers (everywhere).	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Open year round. Please call ahead in case of inclement weather.	SELF-PACED LEARNING CENTERS\r\n    HOURS OF OPERATION:\r\n\r\n   Mon: 9AM - 11AM\r\n   Thu: 1:30PM - 3:30PM\r\n   Fri: 1:30PM - 3:30PM	English!DELIM!Spanish	No	Speak to Customer Service at the Career Center's main line.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Closer services may be available (at other Career Centers) to those outside the Greater Lawrence area.	Program provides ESL training.	No	1	\N
508712	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	VWCC Core Services	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Labor market information and resorces for job-seekers.	At the core level, clients may take advantage of the Career Centers many resources. Core services can be self-directed, or may also require staff assistance.\r\n\r\nThese resources include: workshops, one-on-one staff assisted job search, job matching, labor market information and employment statistics, internet job search, copy and fax machines, job lists, newspapers, periodicals and journals.	ADD INFO	Free	\N	Open year round. Please call ahead in case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 7:00PM\r\nEvening hours include Workshops and Career Center Seminars only.	English	No	\N	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.\r\n\r\nNo Resource Room services during evening hours.	No	\N	No	\N	Yes	For job-seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Services will be most helpful for those seeking employment in the Greater Lawrence area.	\N	No	1	\N
508910	103377	2008-07-11	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Youth Services	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	In-school and out-of-school programs for youth.	Specialized training to assist youth in developing job related skills. We offer in-school programs that assist youth in exploring career options while completing their high school education and successful completion of MCAS testing. \r\n\r\nWe also offer programs for out-of-school youth that assist in earning their GED and gaining specific work skills. Youth are selected for participation in each program based on their career goals and the program's purpose.	Youth aged 14-21 who meet eligibility criteria. \r\n\r\nContact Megan Shea at (978) 722-7026 or mshea@detma.org.\r\n\r\nSee: www.valleyworks.cc/youth.htm\r\n\r\nThis program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives. \r\n\r\nYouth are asked to bring proof of age, citizenship, family income, and family size to their initial meeting.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508954	103377	2008-07-11	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Job Search / Job Readiness	JSJR / EAS II	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Job search services for TAFDC recipients.	In partnership with the Department of Transitional Assistance, the ValleyWorks Career Center provides comprehensive job search services to applicants and recipients of Transitional Aid for Families with Dependent Children (TAFDC). A minimum of 20 hours per week is required for job search skill development and job readiness training.	TAFDC recipients.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	Services may be most helpful for those seeking employment in the Greater Lawrence area.	\N	No	1	\N
508800	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Senior Community Service Employment Program	SCSEP	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Assistance for mature adults in gaining employment.	The ValleyWorks Career Center refers mature workers, 55 and older, to this program which assists with achieving gainful employment and personal development through community service and training. \r\n\r\nSCSEP participants earn at least minimum wage while working up to 20 hours a week in eligible host agencies. Participants contribute to their communities while gaining valuable experience that will assist them in finding unsubsidized employment. Participants meet regularly with SCSEP staff to work on a variety of job-finding skills, review their progress and goals, and develop realistic plans to overcome any obstacles they face.\r\n\r\nFor more information, visit the National Able Network website at: www.nationalable.org.	Adults 55+.	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Open year round. Please call ahead in case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508811	103377	2008-07-10	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>hidden -- unapproved</i>	Unemployment Insurance Claims	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Provides Unemployment Insurance.	File an Unemployment Insurance (UI) claim by phone or  in person or at the ValleyWorks Career Center. Customers can: file a new claim for benefits, reopen an existing claim, be interviewed if there are issues that affect eligibility, resolve problems with claims, or attend a Career Center Seminar orientation.	Massachusetts workers having lost their jobs through no fault of their own.	Free	\N	Open year round. Please call ahead in case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4PM\r\nThu: 8AM - 4PM	English	No	Contact program for more information. Be prepared to furnish:\r\n\r\nSSN; date of birth; home address and telephone number; the names and addresses of all of the employer in the 15 months prior to filing your claim; the names, dates of birth and social security numbers for any dependent children; alien registration number for non-U.S. citizens.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508855	103377	2008-07-10	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Job Corps Office	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Job Corps representative at the Career Center.	Job Corps is the nations largest and oldest federally funded job training and education program for income-eligible young adults. Training is available in one or more than 60 career areas such as nursing, construction, automotive repair, culinary arts, and information technology.\r\n\r\nThe Career Center's Job Corps representative can help youth to find the appropriate program for each individual.	Young adults ages 16 through 24.	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	No	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Job Corps Centers operate across the nation.	\N	No	1	\N
508921	103377	2008-07-11	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Workforce Investment Act (WIA) Programs	WIA Title I Employment and Training	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Employment asssitance for WIA eligible persons.	The Low Income Adult and Dislocated Worker Program, under Tile I of WIA, are designed to provide quality employment and training services to assist eligible individuals in finding meaningful, self-sustaining employment. \r\n\r\nIndividualized career services are available for customers who meet the eligibility guidelines of WIA programs. Individualized one-on-one services include: career planning and development, occupational skills training, case management and job search assistance.	WIA eligibility may best be determined through visiting the Career Center. \r\n\r\n\r\nEligible persons may include: \r\nindividuals terminated from employment as a result of a permanent closure of, or any substantial layoff at, a plant, facility, or enterprise, or are formerly self-employed, and who are unlikely to return to a previous industry or occupation.\r\n\r\nOther individuals may be eligible for WIA Title I programs, so contact the Career Center for more information.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips!DELIM!51 Haverhill - Amesbury - Newburyport	Ample free parking available in various lots.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508943	103377	2008-07-11	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	DTA Skills	\N	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Occupational skills training for TAFDC recipients.	TAFDC recipients have access to a variety of educational occupational skills training programs that are available through the Employment Services Program (ESP). \r\n\r\nThe ESP programs  primary goal is to provide occupational skills training and support that will help attain and retain employment that  provides economic self-sufficiency for TAFDC families.	TAFDC recipients.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	1	\N
508932	103377	2008-07-11	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Individual Career Services	Career Services Advisor	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Advisor helps with additional employment services.	For those seeking access to more significant one-on-one services, some persons may be eligible to receive intensive services. \r\n\r\nIntensive services may include any one or more of the following services: \r\n\r\n+ comprehensive and objective assessment\r\n+ individual counseling and career planning  \r\n+ case management\r\n+ occupational skills training \r\n+ On-the-Job Training (OJT) \r\n+ entrepreneurial training	Participants must qualify under determined program eligibility guidelines.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	Services may be most helpful for those seeking employment in the Greater Lawrence area.	\N	No	1	\N
508503	102178	2009-01-15	\N	MassHealth, Commonweath Care, Commonwealth Choice Program	MassHealth Enrollment Outreach Program	305 Essex St	\N	Lawrence	MA	01840	978-681-4978	\N	\N	978-681-4949	www.glcac.org	public_program_email@testing123.net	Information and application assistance regarding MassHealth.	This program connects eligible Massachusetts residents with approved health insurance plans and helps them pay for health insurance coverage through the plans. \r\n\r\nConducts outreach to target populations and provides information and application assistance regarding: \r\n\r\nMassHealth\r\nCommonwealth Cares\r\nCommonwealth Choice.	Adults currently without health insurance and those needing to requalify for MassHealth.	Sliding Scale	Commonwealth Care pays the total cost of health insurance for qualified individuals who have income at or below 100% FPL (Federal Poverty Level).	Open year round.	Monday - Friday 8:30 am - 4:30 p.m.	English!DELIM!Spanish	No	\N	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence.	\N	Yes	No	\N	No	\N	\N	GLCAC, Inc. MassHealth Enrollment Outreach Project	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
508316	103322	2008-07-03	\N	Big Friends Little Friends	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6640	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	One-on-one mentorship with young people.	Big Friends Little Friends matches adult mentors in one-on-one relationships with young people ages 6  14. \r\n\r\nBig Friends Little Friends provides matches and support to mentors, parents and youth in the Merrimack Valley and Southern New Hampshire.\r\n\r\nBigs and Littles stay in touch weekly and form a close and trusting relationships that give the Littles a positive role model and emotional support.	Youth ages 6  14 in the Merrimack Valley, primarily Greater Lawrence.	Free	\N	Open year round.	Office Hours M-F, 9am-5pm.	English	Yes	\N		Not applicable.	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	\N	Yes	Conference room seats 75 persons.	Yes	Family Services, Inc. offers periodic newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	1	\N
508261	103322	2008-07-02	\N	Samaritans Outreach and Education	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-688-0030	\N	\N	\N	www.familyserviceinc.com	public_program_email@testing123.net	Support for organizations with at-risk constituents.	Samaritans staff and volunteers provide educational support for schools, businesses, youth and senior centers, or any type of organization with constituents who may be at-risk. \r\n\r\nSamaritans provide information in outreach and education sessions to help the audience identify individuals who may be at risk, to educate them to enable them to intervene with a person at-risk, and to provide resources for those at risk.	Schools, youth centers and other agencies and businesses.	Yes	Contact Samaritans for more information.	Depends on organizations' interest.	Schedule appointment.	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
508283	103322	2008-07-02	\N	Samaritans Postvention	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-688-0030	\N	\N	\N	www.familyserviceinc.com	public_program_email@testing123.net	Postvention for organizations with grief of a suicide.	The Samaritans offer help and support to individuals and groups in the wake of a suicide. The Samaritans provide support and debriefing for organizations and mental health professional by being on site and talking through the tragedy with an open mind and sympathetic ear. This process helps affected individuals cope with the tragedy of suicide and gives them the skills to grieve in a healthy way.	Schools, mental health professionals, and businesses.	Free	\N	Open depending on need.	Contact for Samaritans postvention services.	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
508250	103322	2008-07-02	\N	Samaritans Safe Place	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-688-0030	\N	\N	\N	www.familyserviceinc.com	public_program_email@testing123.net	A suicide survivor support group.	Safe Place is a suicide survivor support group for those who have lost a loved-one to suicide. Safe Place is a place for individuals to grieve, share difficulties and feelings, and find support from others who are grieving in the wake of suicide. Meetings are facilitated by a peer suicide survivor and are confidential and free.	Those who have lost a family member of friend through suicide.	Free	\N	Year round.	Safe Place meets at three locations twice each month, in North Andover, Amesbury, and Lowell. \r\n\r\nFor more information, contact the Samaritans at 978-688-0030.	English	Yes	Determine location and time of meetings by contacting Samaritans of Merrimack Valley.		Varies based upon location of meeting. Also, all to determine which location will best meet your needs as to handicapped accessibility.	\N	Not Applicable	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	Not Applicable	1	\N
508272	103322	2008-07-02	\N	Samartians Gatekeeper Training	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-688-0030	\N	\N	\N	www.familyserviceinc.com	public_program_email@testing123.net	Trainings to reduce suicide among persons 55 or older.	The Samaritans of Merrimack Valley offers training for gatekeepers to reduce the incidence of suicide among people ages 55 and older in the Merrimack Valley. After the training, participants are able to assess an older adult's suicide risk, intervene with an individual at risk, and connect older adults with ongoing professional help. \r\n\r\nContinuing Education Credits (CEUs) are available for nurses, social workers, Licensed Mental Health Counselors (LMHCs) and EMTs for 6.5 hours. The Gatekeeper training is 8 hours in length and can be provided in one 8-hour session or two four-hour segments. Trainings can be held on site at interested organizations.	Gatekeepers are people who, through their personal, professional or volunteer activities, have regular contact with older adults.	Free	Free, Department of Public Health-funded.	Contact Samaritans Training Coordinator at 978-688-0030 or mquinn@familyserviceinc.com.	Variable	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
508305	103322	2008-07-03	\N	Strengthening Couples	\N	430 North Canal Street	\N	Lawrence	MA	01840	978-327-6634	\N	\N	978-327-6601	www.familyserviceinc.com	public_program_email@testing123.net	Relationship enhancement program for couples with children.	The Strengthening Couples Program is a 14 sessions relationship enhancement program for couples with children. The educational workshops help strengthen relationships by having better communication between partners, and be able to express your true feelings, thoughts, concerns and desires.\r\n\r\nParticipants must commit to completing the program. Program includes free relationship enhancement workshops in English and Spanish, case management, referrals, transportation, childcare, and dinner.	Participants must: have children, be Lawrence, Methuen, Andover or\r\nNorth Andover residents, be low-income.	Free	\N	Contact for openings.	Thursdays 6-8:00pm	English!DELIM!Spanish	Yes	\N		Parking in lot near Family Services, Inc.	Transportation provided.	Yes	Yes	\N	Yes	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Conference room seats 75 persons.	Yes	Family Services Inc publishes periodic newsletter.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	1	Additional contact:\r\nJuan LLuberes\r\njlluberes@familyserviceinc.com
508976	102508	2008-07-12	\N	AIDS Residence	\N	Corpus Christi Residence	155 Franklin Street	Lawrence	MA	01841	978-689-4321	\N	\N	\N	http://www.lazarushouse.org/	public_program_email@testing123.net	Residence for families and individuals HIV positive.	This program provides housing and comprehensive services for low income and homeless individuals and families with HIV/AIDS.\r\n\r\nAIDS residences operate at two supported sites: an eight unit congregate program for individuals (Corpus Christi House) and a two-family building to house adults and their families (Bethany House). \r\n\r\nThe Corpus Christi AIDS Residence is a home for men and women who are living with HIV or AIDS. The Bethany House Family AIDS Residence is for families with one or more members living with HIV or AIDS.\r\n\r\nCorpus Christi Residence\r\n\r\n+ Facilities for 8 single men and/or women (currently there are vacancies)\r\n+ Individualized service plans\r\n+ Onsite visiting nurse care\r\n+ 24hr. supportive staff\r\n+ Pastoral care and education \r\n\r\nBethany House\r\n\r\n+ Facilities for 2 families living independently\r\n+ Opportunity to participate in Corpus Christi programs	Applicants must meet the following criteria:\r\n\r\n+ Homeless as defined by HUD\r\n+ Confirmed diagnosis of HIV/AIDS\r\n+ Three months clean and sober with intention of continuing clean and sober\r\n+ Needs supportive housing\r\n+ Be able to live safely with other people\r\n+ Have life expectancy of at least six months\r\n+ Demonstrate ability and desire to participate in program	Sliding Scale	Residents pay 30% of their income for rent and another 20% program fee which includes food, utilities and costs of other services. \r\n\r\nAll residents have an individualized service plan and are required to participate in some activity to help maintain long-term residential and economic stability.	Open year round.	Office Hours:\r\nMon-Fri: 8:30AM-4:30PM	English	Yes	\N		\N	Families that live at Bethany House may include help in securing jobs and transportation.	No	Yes	Families that live at Bethany House may include help in day care and scholarships for childrens programs.	Yes	Openings depend on vacancies at residences.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lawrence, MA	978-689-4524	\N	\N	No	1	\N
508998	102508	2008-07-13	\N	St. Martha's Food Pantry	\N	Good Shepherd Center	48 Holly St.	Lawrence	MA	01842	978-689-8575	\N	\N	978-682-7004	http://www.lazarushouse.org/	public_program_email@testing123.net	Food pantry.	Located in the basement of the Lazarus House Shelter, the Food Pantry supplies emergency food supplies to as many as 300 families each week. \r\n\r\nWednesday afternoons people receive groceries to last them up to four days. During the week, bagged lunches are given out to those who may have to work, but do not have enough money to buy meals.\r\n\r\nProgram also sponsors complete holiday meals to as many as 750 people.	Low-income and homeless persons.	Free	\N	Open year round. Special services in winter and at holidays.	St. Marthas Food Pantry:\r\nGroceries distribution (Wednesday afternoons)\r\nBagged lunches distributed during the week. \r\n\r\nSoup Truck:\r\nOperates on Friday and Sunday Nights during the Months of January, February and March, and additionally provides food, blankets, gloves and hats. \r\n\r\nHoliday Food Orders:\r\n3 day meal supply distributed on Thanksgiving, Christmas, and Easter.	English	No	ADD INFO		\N	\N	No	No	\N	No	\N	\N	St. Martha's Food Pantry	P.O. Box 408	Lawrence	MA	01842	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	1	\N
506732	102552	2008-06-20	\N	Cub Scouting	\N	36 AMESBURY RD	\N	Haverhill	MA	01830	978-372-0591	\N	\N	978-373-9134	http://www.yccbsa.org	public_program_email@testing123.net	Boys in grades 1-5	family based program - taches a boy to do his best and help other people at all times.\r\nfamily oriented activities.	all	Yes	depends on individual programs	year round	varies	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	all of essex county, northern middlesex county, souther NH	\N	No	1	\N
507150	102739	2008-06-21	\N	SAHELI	SAHELI BOSTON	P O BOX  52063	\N	Boston	MA	02205	1-866-472-4354	1-866-472-4354	1-866-472-4354	\N	www.saheliboston.org	public_program_email@testing123.net	Information,Support and Friendship to women in need.	Saheli, Friendship for South Asian Women is a group dedicated to helping South Asian women in Boston and surrounding areas. It provides friendship, support, guidance and resources in the areas of career and economic empowerment, physical and mental health, legal and immigration issues, support for families, and social and cultural volunteer opportunities. Our organization gets stronger with every new member and we warmly welcome you to join us.	Mostly, New England Area	Free	\N	Open year round	24/7	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	send an email to sahelihelp@gmail.com	Not Applicable	\N	Yes	Burlington Public Library	Yes	Will be on saheliboston.org very soon.	Yes	saheliboston.org	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
500319	100044	2008-06-17	\N	Behavioral Health Services -Outpatient	\N	17 Warren Street	135 Jackson Street and 585 Merrimack Street	Lowell	MA	01852	978-937-9488	\N	\N	978-970-0057	www.lchealth.org	public_program_email@testing123.net	Counseling and Substance Abuse Treatment	The Behavioral Health Services department provides counseling and appropriate referrals to clients of LCHC. Medical Social Work services are also available. Licensed Clinical Social Workers provide counseling on a variety of issues including: depression, post traumatic stress disorder, anxiety, domestic violence, substance abuse and other areas of concern.	Services available to all residents of Greater Lowell, referrals are strongly recommended	Sliding Scale	Meet with a Health Benefits Coordinator to determine your insurance eligibility and to apply for an insurance plan please call 978-746-7777. Private insurances accepted.	Services available year round, except for major holidays.	Monday, Wednesday, Thursday and Friday 9am to 5pm.  Tuesday 9am to 8pm.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Contact program. May need referral and identification for appointment.		Parking available in visitor lot next to the Health Center or at metered spaces.	\N	Yes	No	\N	No	At times there may be a wait list	(978) 452-4774 TDD Number	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	LCHC Happenings - A free newsletter for friends of the Health Center.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	Services are offered at the three sites in Lowell as described above.	No	1	\N
500308	100044	2008-06-19	\N	Adult Medicine	\N	585 Merrimack Street	\N	Lowell	MA	01854	978-937-9700	\N	978-937-9700	978-970-0359	www.lchealth.org	public_program_email@testing123.net	Primary health care services for adults	Lowell Community Health Center's Adult Medicine  provides complete primary care services including physical exams, routine laboratory services, blood pressure detection and management, asthma management, cholesterol and diabetes screenings, cancer detection prevention and treatment, nutrition and weight management and health education.	Open to adults over 18 seeking primary medical care as well as additional health and wellness services.	Yes	Everyone is welcome.  Fees vary depending on income. Private insurances, medicaid and medicare accepted.  Please call for more information.  LCHC has Health Benefits Counselors ready to assist with the insurance enrollment process for those who are uninsured.Please call to make an appointment	Services are available year round	Monday, Tuesday,and Thursdays  8:00AM-8PM; \r\nWednesday and Friday 8:00AM-5PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Appointments required for all visits unless a sick visit.  New patients interested in insurance enrollment, bring ID and 2 paycheck stubs. Needs to make an appointment with a Health Benefits Coordinator if no insurance.	07 Pawtucketville / UMass Lowell North	Parking available behind building in patient lot or on street at metered spaces.	\N	Yes	No	\N	Yes	Currently new patients making appointments at Adult Medicine may be placed on a wait list, or may be asked to be seen at another Health Center site.	(978) 452-4774 TTY Number	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	LCHC Happenings - A newsletter free to to friends of the Health Center.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	Interpreting available in over 18 languages.	No	1	\N
500374	100044	2008-06-19	\N	Pediatric Medicine	\N	597 Merrimack Street	\N	Lowell	MA	01854	978-937-9700	\N	\N	978-453-3289	www.lchealth.org	public_program_email@testing123.net	Primary care for children	Pediatric Medicine provides complete primary care services for children and infants including physicals, hearing and vision exams, school physicals, lead screening, asthma management, immunizations, health education, nutrition services and routine laboratory work.	Services generally available to all up to age 18	Sliding Scale	MassHealth and private insurance accepted. IF you do not have insurance you can speak with a Health Benefits Coordinator to apply for insurance plans. No one is refused care because of inability to pay.	Services are available year round	Wednesday, Friday 8:30AM-5PM; Monday, Tuesday & Thursday 8:30AM-8PM;	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Insurance card, identification and name of provider. To apply for free care/ sliding scale, bring ID and 2 paycheck copies. No Social Security number is needed to apply free care.		Parking available in lot behind building.	\N	Yes	No	\N	No	\N	978-452-4774 TTY Number	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	Interpreter services available in 18 additional languages through the language line.	No	1	\N
501243	100165	2008-06-19	\N	Supervised Visitation Center & Parent Support Program	\N	126 Phoenix Ave Building #2	Third Floor	Lowell	MA	01852	978-513-2362	\N	\N	978-937-8695	www.mspcc.org	public_program_email@testing123.net	Provides a safe/comfortable setting for family visitation	The Supervised Visitation and Parent Support Program provides a safe and comfortable setting for families impacted by abuse, neglect or domestic violence to visit with their children, improve parent-child relationships and to ensure permanency. The Visitation Center also provides a safe and comfortable setting for situations such as divorce or post adoption. Parents receive assistance in understanding child development, safety, nutrition, appropriate discipline techniques and community resources.	Referrals required for families served by DSS. Referrals for other services received from anyone in the community	Yes	No fees charged for families referred by DSS. For Probate Court cases there is an intake fee of $35.00 for both visiting and custodial parents.  Weeknights visit are $35.00 per hour.  Weekend visits are $50.00 per hour.  We will do visits off-site with clients we have established a relationship with.  The cost is $60.00 per hour on week-nights and $75.00 per hour on week-ends.	Program runs year round. A family's stay at the center is determined on an individual basis, generally within a year	Monday-Friday 8:45AM-5:00PM. Evening and weekends visits are provided by appointment until 8:00 PM on weeknights and during the day on weekends.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Any court orders regarding visitation, copies of restraining orders.	02 Belvidere	Large Parking lot	Transportation is available for children if social worker or foster parent are unable to provide it in DSS cases only	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	For parents who use the center we have resources for them on parenting.	Yes	There is a conference room available in Lowell and also rooms available for parent education groups	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	If a visiting parent lives in another state we do visits for them.  If both parties live in other states but we provide a central location for them to travel to we will do visitation.	\N	No	1	\N
500363	100044	2008-06-19	\N	Metta Health Center	\N	135 Jackson Street	\N	Lowell	MA	01852	978-441-1700	\N	978-441-1700	978-454-1681	www.lchealth.org	public_program_email@testing123.net	Combines Eastern traditional medicine with Western medicine	The Metta Health Center combines traditional eastern medicine with western medicine. Services include primary health care, behavioral health services, nutrition counseling, acupuncture and community health education. Metta Health Center is a full-service primary care site. Staff are proficient in Southeast Asian cultural practices and languages.	Services available to all residents of Greater Lowell. Focus on Southeast Asian population. Referrals and walk-ins accepted	Yes	Private insurance accepted. Sliding scale fees may be charged based on income. Health Benefit Coordinators can help you apply for insurance programs. No one is denied care because of inability to pay.	Services available year round with the exception of major holidays	Monday-Friday 9AM-5PM\r\nTuesday evening 5PM -8PM	English!DELIM!Khmer	Yes	Please call for an appointment. Walk-ins also welcome.		Parking is available in the Edward Early Parking Facility across the street from health center with a minimum charge. Short-Term Parking spaces, with metter, are also available on the ground level.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Please contact the Metta Health Center Director at 978-322-8755	No	\N	No	\N	Yes	LCHC Happenings - A newsletter free to friends of the Health Center.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	978-452-4774	\N	Laotian	No	1	\N
500385	100044	2008-06-20	\N	Prenatal Services	\N	17 Warren Street	3rd floor	Lowell	MA	01852	978-446-0788	\N	978-446-0788	978-453-1777	www.lchealth.org	public_program_email@testing123.net	Prenatal Services and Family Practice	Prenatal services offer certified obstetricians, certified nurse midwives and physicians to assist pregnant women throughout labor and delivery.	All women in need of prenatal services	Sliding Scale	Open to all. Accepting private insurance and MassHealth. If you do not have insurance you may meet with a Health Benefits Coordinator to apply for insurance plans.	Services available year round.	Monday-Thursday 8:30-5\r\nFriday 8-4:30	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Insurance card, if applicable	09 Downtown Circulator	Parking available in a small lot on the side of the building or in Lower Locks Municipal Garage next to Doubletree Hotel.	\N	Yes	No	\N	No	\N	978-937-9597 TTY Number	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	Yes	LCHC Happenings - A free to newsletter for friends of the Health Center.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Wilmington, MA	\N	\N	Interpreter services available in 18 additional languages.	No	1	\N
500407	100044	2008-06-20	\N	Women's Services	Women's Services of Greater Lowell	17 Warren Street	2nd floor	Lowell	MA	01852	978-446-0236	\N	\N	978-446-0248	www.lchealth.org	public_program_email@testing123.net	Womens Health Services	Women's Services provides complete gynecological exams, including pap smears, hematocrit, chlamydia and urine analysis testing, birth control information and services, pap smear follow-up, colposcopy and some cryosurgery treatment, pregnancy testing and counseling, treatment and counseling of STDs, testing, treatment of urinary tract infections and hormone replacement therapy.	Females and their partners living in Massachusetts	Sliding Scale	Open to all. Adolescent services very low or no cost. Accepting private insurance and MassHealth. If you do not have insurance you may meet with a Health Benefits Coordinator to apply for insurance plans.	Services available year round.	Monday-Thursday 8AM-5PM; Wednesday 8:00AM-7PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Contact the department for more information.	09 Downtown Circulator	Parking available in lot next to the building and in Lower Locks Garage next to the Doubletree Hotel.	\N	Yes	No	\N	No	\N	978-452-4774 TTY Number	\N	\N	\N	\N	\N	No	\N	Yes	Teen Clinic meets weekly and has educational material on healthy relationships, adolescent issues and sexuality.	No	\N	Yes	LCHC Happenings - A free newsletter for friends of the Health Center.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Bedford, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Wilmington, MA	\N	\N	Interpreter services in 18 other languages available.	No	1	\N
501342	100220	2008-06-20	\N	Capacity Grant Program	\N	100 Merrimack St, Suite 202	\N	Lowell	MA	01852	978-970-1600	\N	\N	978-970-2444	www.glcfoundation.org	public_program_email@testing123.net	Capacity building grants for local non-profits	Capacity grant guidelines and application are available on our website www.glcfoundation.org from January to March 15 (deadline). The Capacity Grant Program annually distributes approximately $100,000 in amounts averaging $2,500 -$3,000 to nonprofit agencies and coalitions of agencies in our service area for projects which increase the agencies' capacity to accomplish their missions. Examples include consultants to assist in board development, fund raising campaigns and strategic planning, office equipment, which help staff work more efficiently, training seminars for staff.	Recipient agency or coalition lead agency must be 501(c)(3) nonprofit organization	Free	\N	30-40 grants awarded annually in the spring. Agencies have 1 year to complete and submit final reports on projects	Monday-Friday 9AM-5PM	English	No	\N		\N	\N	Yes	No	\N	No	\N	Only non-profits in the Greater Lowell, Merrimack, Concord and Nashoba River Valleys are eligible.	\N	\N	\N	\N	\N	Yes	Office assistance in managing the administrative and program operations of the Community Foundation.	No	\N	No	\N	Yes	General newsletter is distributed periodically and is available on GLCF's website, www.glcfoundation.org.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Bedford, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA!DELIM!Wilmington, MA	\N	Also the towns of Ashby, Ayer, Burlington, Carlisle, Concord, Groton, North Reading, Pepperell, Reading, Shirley, Stoneham, Townsend, and Wakefield	\N	No	1	\N
508173	100220	2008-06-20	\N	Field of Interest Grant Funds	\N	100 Merrimack St, Suite 202	\N	Lowell	MA	01852	978-970-1600	\N	\N	978-970-2444	www.glcfoundation.org	public_program_email@testing123.net	Field of Interest Grant Funds	The Community Foundation is working with intersted donors to grow our field of interest funds as a permanent way to annually support nonprofit organizations active in the areas of water resource management, youth development, women and girls programs, elder services, and Lowell's economic development. \r\n\r\nWater resource  and elder service grants are annually distributed on a competitive bases with application and guidelines available from January to March 15 (deadline).	Donors everywhere are invited to build the funds. Eligible nonprofit organizations must serve one or more of the 24 towns in the Community Foundation's service area in northern Middlesex County.	Free	\N	Applications will be available on the Community Foundation's website from January to March 15 (deadline)	9-5	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	We have office volunteer and philanthropic planning, marketing, public relations, special events and other volunteer opportunities available.  Please call David Kronberg at 978-970-1600.	No	\N	Yes	A small conference room (maximum 12)	Yes	Foundation Forum newsletter is published twice annually.  Past editions are available on our website: www.glcfoundation.org	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Greater Lowell area and towns in the Merrimack and Nashoba Valleys in Northern Middlesex County	\N	Not Applicable	1	\N
500330	100044	2008-06-20	\N	CARIÑO / HIV Services	\N	585 Merrimack Street	\N	Lowell	MA	01854	978-937-9700	\N	\N	\N	www.lchealth.org	public_program_email@testing123.net	AIDS/HIV prevention,education, testing & counseling services	CARINO programming provides the following:  HIV testing, counseling,and referral services, HIV Prevention and Education,  medication adherence, acupunture, HIV infectious disease specialty care and case management for adults and children. Patient advocates available. Services are confidential.	Services open to all adults and children.	Sliding Scale	Fees vary. Private insurance accepted. You may meet with a Health Benefits Coordinator to apply for reduced cost insurance plans.	Services available year round.	Monday, Wednesday, Thursday, Friday 9AM-5PM\r\nTuesday 9am - 7:30pm.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	No	Call for information. Walk-ins accepted for HIV testing.		Parking available in lot behind building or on street at metered spaces.	\N	Yes	No	\N	No	\N	978-452-4774 TTY Number	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	LCHC Happenings -A free newsletter for friends of the Health Center.	Yes	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	1	\N
501980	100561	2008-06-20	\N	Anne Sullivan Center Early Intervention	\N	126 Phoenix Ave.	\N	Lowell	MA	01852	978-453-8331	\N	\N	978-453-9254	www.thomchild.org	public_program_email@testing123.net	Serve at risk children Birth - 3 years old	The Anne Sullivan Center serves families of children ages birth to three years who have or who are at risk for having developmental disabilities or delays. We provide:\r\n-Home visits providing service coordination, developmental and therapeutic activities, parent education and support, resources and referrals\r\n-Parent Support and Education groups\r\n-Parent/Child groups; Child groups\r\n-Developmental assessments / evaluations\r\nServices are provided by Educators, Teacher Assistants, Physical Therapists, Occupational Therapists, Nurses, Speech Therapists, Social Workers, Counselors, Interpreters, a Consulting Nutritionist and a Consulting Developmental Pediatrician.\r\nMany of the groups are open to community families not enrolled in the Early Intervention Program. A limited amount of home visiting is available to non-enrolled families through a small grant funded program.	Families with children birth-3. Child must have or be at risk of having a developmental disability or delay.	Sliding Scale	There is no fee for families of children receiving Mass health. For all other families there is an annual participation fee charged on a sliding fee scale basis.	Program available year round. Program size is approximately 650.	Monday-Friday 8AM-4:30PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Call with insurance information and name of pediatrician. Intake/Developmental Assessment takes place in the home	02 Belvidere	Parking available next to building	Transportation provided to center based activities,if the family is unable to transport.	Yes	No	\N	No	\N	Spanish, Portuguese, Khmer, Vietnamese, Hindi. Other languages available through interpreters	\N	\N	\N	\N	\N	Yes	Volunteers are involved with Child Group activities, call the Center at 978-453-8331	Yes	Call 978-453-8331 for more information about the Resource Library	Yes	Space is available during the evenings. Call 978-453-8331 for more information	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	Also, Hindi, Korean, Cantonese, Mandarin, and Sign Language	No	1	\N
503718	100561	2008-06-20	\N	Anne Sullivan Early Intervention Partnerships	Pregnancy and Newborn Support	126 Phoenix Ave.	\N	Lowell	MA	01852	978-453-8331	\N	\N	978-453-9254	www.thomchild.org	public_program_email@testing123.net	Home visiting program for pregnant and post-partum mothers	This home visiting program serves pregnant women, post-partum mothers and their infants. The services are provided by a Maternal Child Health Nurse, Bilingual Family Liaisons and a Social Worker. The goals are to improve maternal health and perinatal outcomes, and to support optimal infant growth and development through the first year of life. Home visits focus on prenatal education, breastfeeding information and support, post-partum and newborn education including family safety and injury prevention, family planning, reproductive health and infant screening.	Pregnant women and post-partum mothers and their infants.The program is focused on supporting women who are experiencing risks that could negatively impact their pregnancy or development of their newborn. Examples of these risks include but are not limited to domestic violence, depression, health problems, high levels of stress, substance abuse, high risk pregnancy, lack of prenatal care.	Free	\N	open year round	8:00 A.M. to 4:30 P.M.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Not Applicable	Insurance Information	02 Belvidere	\N	The program is a home based program and does not require the families to come to the Center.	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Call 978-453-8331 for more information about the Resource Library	Yes	Space is available during the evenings. Call 978-453-8331 for more information	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	1	\N
501914	100528	2008-06-24	\N	Threads of Learning	\N	American Textile History Museum	491 Dutton Street	Lowell	MA	01854	978-441-0400	\N	\N	978-441-1412	www.athm.org	public_program_email@testing123.net	Educational programs with interactive learning opportunities	Threads of Learning, the American Textile History Museum's standards-based school-age programs, offer a variety of K-12 grade appropriate on-site and off-site interactive learning opportunities. Themed tours for all ages are also available. Special needs students welcomed. Handicap accessible facility.\r\nOnsite Programs:\r\nFleece to Fabric - Grades 3-8\r\nJunk to Jackets - Grades 3-8\r\nPacific Print Works - Grades 4-8\r\nFiber Facts - Grades 4-8\r\nTechnical Tools - Grades 8-12\r\nColonial Days - Grades 3-5\r\nNative American Life - Grades 3-5  \r\nTraveling Programs:\r\nThe Goat in the Rug - Grades K-3\r\nThreaded Together - Grades 3-5\r\nFUN-damentals of Weaving - Grades 4-6\r\nA Stitch in Time: Coming to America - Gr.4-8\r\nMothers (and Fathers) of Invention - Gr. 4-8\r\nEstimation Mania - Grades 7-8\r\n\r\nSpecial Tours:\r\nBusiness Tour - High School-College-Adult\r\nWomen's Tour - High School-College-Adult\r\n\r\nEducators may select from the seven onsite and six off-site programs. Our hands-on intensive Colonial Days program is offered in the spring and fall. A new Native American history/life program is now available, as well. Programs for boy scouts, cub scouts, girl scouts, and programs for pre-school available during regular hours. A tour focused on business aspects of the textile industry is available for high school and college students, and adults. A tour focusing on women and fashion history is available, as well. Reservations required for all programs:reservations@athm.org. or call 978-441-0400 X250\r\n\r\nMuseum educators are available to present programs in your classroom, or combine hands-on activities with a museum tour here.\r\n\r\nSpecial theme week presentations include Colonial Days each spring and fall, when costumed educators take children on a journey to Colonial America through projects, games, and activities. A Lowell Women's Week women's fashion show and tour is available during that special week each year.	Ages: Pre-school through college	Yes	Fee is $5 per student for on-site programs payable on arrival. Fee is $100 plus mileage for off-site programs ($75 for each subsequent same-day program).  Call for more information.978-441-0400 ext. 250, or email reservations@athm.org	Programs available year round. Length: 2-2.5 hours (other arrangements possible). Maximum of 60 per visit. Lunch space.	While the Museum is open to the public Wednesday through Sunday, 10-5 PM, educational programs may be scheduled at a group's convenience. Call 978-441-0400 X250 for options, or email reservations@athm.org.	English	Yes	Contact reservations@athm.org or phone 978-441-0400 exy. 250 for information		Free parking available on-site.	Limited grant funds may be available for buses for Colonial Days.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers, please contact the Claire Matte at 978-441-0400 ext. 243 to learn more about volunteer opportunities	Yes	Resource materials on: Textile History, cloth, fabric, Lowell history, Lawrence history; includes historic images	Yes	Call 978-441-0400 X241 for information about our various meeting spaces in our newly redesigned space.	Yes	The Overshot, our member newsletter, is sent to all members and donors. The Tapestry newsletter is sent to teachers and schools.	Yes	website www.athm.org	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Serving the Greater Merrimack Valley and New England	\N	No	1	\N
503058	100484	2008-07-01	\N	Bridge Program at the David McHugh Alternative Middle School	\N	73 Woburn Street	\N	Lowell	MA	01852	978-453-1115	\N	978-245-2828	978-441-1050	\N	public_program_email@testing123.net	Alternative Middle School	The Bridge Program is an alternative middle school for Lowell Public School students, ages 12-16, grades 7 and 8, who are exhibiting behavioral problems in their middle school setting. The Bridge services 50 students annually and is a program of Middlesex Community College.\r\nAdmission is rolling. Program lasts 1-2 years depending on what grade students are in when they are referred. Program capacity is 50 students. Students who are being referred need to come with their parents, a social security number, and a health record.	Lowell students age 12-16 grades 7 and 8	Free	\N	Program follows Lowell Public School Calendar (August - June). Admission is rolling.	Monday - Friday 7:30AM-3:30PM	English	Yes	Call for information. Referrals from Lowell middle school principles or their designees & Superintendent's office accepted	03 South Lowell	Parking outside building.	Bus passes given to students in AM. Yellow bus in PM to downtown.	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Tutors/Classroom aids are welcome. Call for more information.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	1	\N
500187	100033	2008-07-01	\N	Family Foundations Network	\N	169 Merrimack St., 3rd floor	\N	Lowell	MA	01852	978-654-5080 ext. 16, 23	\N	\N	978-970-1965	www.comteam.org	public_program_email@testing123.net	Parenting resources and support	The Family Foundations Network supports parents in the demanding role of raising children. Programs and services include parent education workshops, parent support groups and play groups. A resource library is available with materials on: general parenting, behavior management, child development, infant massage, etc. There is also a wall with materials on community resources and parenting articles.	The child must be under 4 years old. Must live in Lowell, Billerica, Chelmsford, Dracut, Tewksbury, Wilmington	Free	\N	Program runs year round	Monday-Friday 8:30AM-4:30PM	English!DELIM!Spanish	Yes	Call for more information		Meter and public garage.	\N	Yes	Yes	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Chris ext. 23 or Judith ext. 16	Yes	A resource library is available with materials on: general parenting, behavior management, child development, infant massage, etc. There is also a wall with materials on community resources and parenting articles.	No	\N	Yes	Bimonthly newsletter with calendar, schedule, articles, community resources.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA	\N	Wilmington	\N	No	1	\N
502211	100715	2008-07-01	\N	Municipal Programming	LTC Channel 10	246 Market Street	\N	Lowell	MA	01852	978-458-5400	\N	\N	978-937-0361	http://www.ltc.org	public_program_email@testing123.net	How Your City Works	LTC's Municipal Channel 10 is Lowell's award-winning government access channel and features programs focusing on municipal agencies and their activities. The activities of city government have a profound impact on citizens of Lowell and one of our goals is to keep citizens informed about city hall. We also try and showcase various municipal agencies and services so that people know who to call when they have a particular need or problem.	Employees and departments of the City of Lowell and other state and federal agencies located in Lowell.	Free	\N	Open Year Round	Monday - Thursday\r\n10am to 9:30pm\r\nFriday \r\n9am to 6pm\r\nSaturday\r\n10am to 4pm	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer	Yes	Should attend a new member orientation offered Quarterly.  can also contact director of program to discuss options.	01 Christian Hill!DELIM!02 Belvidere!DELIM!03 South Lowell!DELIM!04 Shaw - Stevens!DELIM!05 Westford Street/Drum Hill!DELIM!06 Broadway / UMass Lowell!DELIM!07 Pawtucketville / UMass Lowell North!DELIM!08 Centralville!DELIM!09 Downtown Circulator!DELIM!10 Dracut / Tyngsboro!DELIM!11 IRS/Raytheon via Rte. 133!DELIM!12 Tewksbury via Rte. 38!DELIM!13 Billerica via Edson!DELIM!14 Burlington Mall / Lahey Clinic!DELIM!15 Chelmsford via Rte. 129!DELIM!16 Chelmsford via Chelmsford Street!DELIM!17 North Chelmsford via Middlesex Street!DELIM!18 Express Shuttle	Free parking in the Lowell National Historic Park Visitor Center parking lot.	\N	Yes	No	\N	No	\N	\N	246 Market St	\N	Lowell	MA	01853	Yes	Video producuction opportunities.  For more information see:\r\nhttp://www.ltc.org/about/jobs.shtml	No	\N	Yes	10-12 person conference room\r\n50-75 person studio space	Yes	Quarterly inFocus avaiable in print and online at http://www.ltc.org/news/news1.shtml	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	1	\N
507282	100803	2008-07-01	\N	Individual Development Account	IDA	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	Individual Development Account	With the funding from the US Office of Refugee Resettlement, the IDA program will, over the course of five years, distribute over $ 1,300,000.00 in matching fund to approximately 400 refugees and asylees. The program participants may use this fund to (1) make a down payment and closing cost for the purchase of a principal residence, (2) capitalize a micro-enterprise which provide full or part-time employment for the new business owners, (3) pay for post-secondary education or vocational training for the program participants or for the refugee child of the program participant, and (4) become re-certified as a professional in an occupation held by the refugee prior to coming to the United States.	a refugee/asylee whose earning income about or less than 200% of Federal Poverty Level	Free	\N	Open year round	Monday-Friday from 8:30 AM-5:00 PM	English!DELIM!Khmer	No	\N		\N	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Some space, 30 seats, is available on a limited basis, preferably business related to CMAA	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
502761	100803	2008-07-01	\N	Monorom Family Services - CMAA	\N	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	978-654-7300	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	Support services for Cambodians with disabled children	The Monorom program provides support services for Cambodian families with disabled children. Services include: transportation, translation, liaison assistance, help with paperwork, home visits, after school programs, traditional dance and summer field trips.	Cambodian children and young adults ages 0-22 living in Lowell. Referrals from schools & agencies are preferred	Free	\N	Services available year round. Enrollment is ongoing.	Monday-Friday 8:30AM-5PM	English!DELIM!Khmer	Yes	It is preferable to have a referral from a school or agency		Parking available in CMAA lot.	Transportation provided by staff of CMAA.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers provide some aid with programs, however it is not a formalized program at the moment	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA!DELIM!Dracut, MA	\N	\N	\N	No	1	\N
504466	100803	2008-07-01	\N	Cambodian Health Promotion	\N	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	Health Education for Cambodian refugees/health care provider	The Cambodian Health Promotion Program provides information through a 9-week series of classes covering health topics of particular interest to recent refugee arrivals.  We also sponsor health fairs and provider education.	Must meet federal definition of "refugee"  Please contact us for specifics.	Free	\N	Open year round	Monday-Friday from 8:30 AM-5:00 PM	English!DELIM!Khmer	No	Proof of refugee/asylee status		Free parking across the street.	Limited transportation in CMAA van but seating is limited and participants are encouraged to use alternate/public transportation.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Help with transportation, or help with health education, if volunteer has expertise.	No	\N	Yes	Some space, 30 seats, is available on a limited basis, preferably business related to CMAA	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	1	\N
505291	100033	2008-07-01	\N	CTI-Children's Collaborative CPC	Community Partnerships for Children program	169 Merrimack St., 3rd floor	\N	Lowell	MA	01852	978-654-5080 ext 24, 11	\N	\N	978-970-1965	www.comteam.org	public_program_email@testing123.net	Childcare for preschoolers	Provides childcare assistance to working families with a child ages 2 years 9 months until the child is old enough to attend kindergarten.	Age of child; family income according to state guidelines; town lived in.	Sliding Scale	\N	Year round.	8:30 am to 4:30 pm	English!DELIM!Spanish	Yes	Call first to determine if you meet the guidelines.		Meter and public garage	no	Yes	No	\N	Yes	Program uses the Massachsuetts State EEC Childcare Waitlist & Application	\N	\N	\N	\N	\N	\N	Yes	Contact Gail 978-654-5080 ext. 24.  Looking for professionals to do free or low-cost trainings for parents or childcare teachers/professionals	Yes	Resource Library is for families and teachers of preschool-age children.	No	\N	Yes	Bimonthly parent and childcare provider newsletter.  Contains articles, family events, free parent workshops, parenting & community information.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Dracut, MA!DELIM!Wilmington, MA	\N	\N	\N	No	1	\N
505511	101221	2008-07-10	\N	Backyard Adventures	Stewardship Through Leadership	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	978-454-7637	www.lowelllandtrust.org	public_program_email@testing123.net	After-school program	Our goal is to expose urban youth to the outdoor classroom where they will build upon their in-school curricula, become engaged in constructive environmental stewardship in their community, and build self-esteem through team and leadership opportunities.	Must participate in an established after-school enrichment program	Free	Fees are set by the organizations hosting our program & vary accordingly.  We do not charge separately for the program.	Varies	School days	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Individuals with some local nature/historic/cultural expertise are welcome to help.	Yes	Hydro power kit and curriculum.	Yes	Spalding House, 383 Pawtucket Street, Lowell\r\n50 person meeting/presentation room - Pentucket Hall	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	Not Applicable	1	\N
503663	101221	2008-07-10	\N	Concord River Alewife Restoration Project	\N	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	\N	www.lowelllandtrust.org	public_program_email@testing123.net	Concord River Alewife Restoration Project	Since the late 1990s, the Lowell Parks & Conservation Trust, Inc. has worked in cooperation with U.S. Fish and Wildlife Service Central New England Fishery Resources Office to implement a plan to return alewife to the 29 miles of the Sudbury, Assabet, and Concord Rivers.  The Sudbury-Assabet-Concord (SuAsCo) watershed is a 377-square mile area encompassing, partially or wholly, 36 Massachusetts towns, primarily in Middlesex County. \r\n\r\nThe alewife is one of two species of herring originally native to our rivers; the other being the blueback herring. The native breeding population became extirpated due to pollution and river dams constructed in the 1800s during the industrial revolution.	Open to all volunteers	Free	\N	Spring, Summer & Fall	7 days a week\r\n6:00 AM - 7:00 PM	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Be a part of history in the making.\r\nVolunteers are needed to monitor the upstream and downstream migration of the alewife (river herring) at the Wamesit Falls fish ladder on Centennial Island in Lowell in the spring, late summer, and early fall.  Volunteers take water & air temperatures, note wildlife and presence/absence of migrating fish.\r\n\r\nWe have a middle school level curriculum available for teachers who are interested.	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	Sudbury~Assabet~Concord River Watershed	\N	Not Applicable	1	\N
503696	101221	2008-07-10	\N	Concord River White Water Rafting	\N	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	\N	www.lowelllandtrust.org	public_program_email@testing123.net	Concord River White Water Rafting	You can experience the thrill of Class III and IV whitewater right in the heart of Lowell, Massachusetts!  On Saturdays and Sundays during the months of April and May, the Lowell Parks and Conservation Trust runs rafting trips on Lowells Hidden Jewel, the Concord River.  Rafts pass through some of the most beautiful historic sights in the city, and through some of the most exhilarating whitewater in New England.\r\n\r\nExperienced guides from Zoar Outdoor, Inc. steer paddlers through three sets of rapids.  The whitewater youll experience is the same Class III and IV rapids youll find in Maine, only a lot closer to home.  Trips leave at 9 AM and 1 PM, and include two runs of the river. The second run ends by passing through the 18th century lock chambers, the Lower Locks in the Pawtucket Canal behind the DoubleTree Hotel.	All inclusive	Yes	$79/person.\r\n\r\nSpecial group rates available.	April & May\r\nRafting starts Saturday, April 7, 2007 and continues through May 27, 2007.	Reservations accepted online at www.zoaroutdoor.com or at 800-532-7483	English	Yes	Reservations are required		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Assistance is needed with operation of the lock chambers.  Registration volunteers and van drivers are also needed.	Not Applicable	\N	Yes	Registered rafters meet at the DoubleTree hotel in Lowell.	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Yes	1	\N
503707	101221	2008-07-10	\N	Concord River Greenway Park	\N	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	\N	www.lowelllandtrust.org	public_program_email@testing123.net	Concord River Greenway Park	The Concord River Greenway Park will be a multi-purpose trail constructed primarily on an abandoned railroad right-of-way paralleling the river.  \r\n\r\nThe Greenway Park will provide public access to the Concord River for walking, biking, fishing, bird watching, white water rafting, kayaking, and other rejuvenating activities. \r\n\r\nThe Greenway will be part of the Bay Circuit Alliance Trail (200 miles of trails in MA) and connect to the soon to be established Bruce Freeman Trail in Chelmsford.\r\n\r\nCRG Construction began July 2008.	All inclusive	Free	\N	Under development - if you would like to join us for our next tour, please contact us.	Under development - if you would like to be involved please contact us.	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Community input is necessary to make this trail.  Please contact us to be involved.	Not Applicable	\N	Not Applicable	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
503674	101221	2008-07-10	\N	Spalding House Restoration	\N	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	\N	www.lowelllandtrust.org	public_program_email@testing123.net	Spalding House Restoration	Donated to the Trust in 1996 by the Molly Varnum Chapter of the Daughters of the American Revolution, the Trust is working on a four-phase plan to turn the house into an Urban Environmental Education Center and Museum. Lowell Parks & Conservation Trust received a Massachusetts Historical Commission 2000 Preservation Award for demonstrating an outstanding commitment to preservation in the Commonwealth of Massachusetts for Phase I  Exterior Restoration of the Spalding House.\r\n\r\nRestoration on the house's 42 windows began spring 2008.  The historic window restoration will permit us to continue our work on other areas of the house, both interior and exterior.\r\n\r\nThe Trust holds meetings, exhibits, and open house tours throughout the year.	Tours are open to everyone.	Yes	A suggested donation of $5	Open house & special events are scheduled throughout the year.\r\n\r\nDoors Open Lowell - May 19, 2007 3-6PM\r\n\r\nSaturday, May 6, 2006 1-4PM Join us for Doors Open Lowell.  We will have spinners from the American Textile History Museum with us that day too.  www.doorsopenlowell.org	Varies.  By appointment.	English	Yes	\N		Parking is available.	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are needed to help with the Home's restoration.	Not Applicable	\N	Yes	Spalding House, 383 Pawtucket Street, Lowell\r\n50 person meeting/presentation room - Pentucket Hall	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	1	\N
508745	101221	2008-07-10	\N	Volunteer Opportunities	\N	PO Box 7162	\N	Lowell	MA	01852	978-934-0030	888-375-1115	\N	978-454-7637	www.lowelllandtrust.org	public_program_email@testing123.net	Volunteer Opportunities	Volunteer Opportunities include: Lock Tending (historic canal locks), river clean-up projects, mailings, restoration of our historic building, land stewardship, landscaping, alewife restoration, and various other opportunities as programs form. \r\n\r\nThe Lowell Parks & Conservation Trust is always looking for volunteers to assist us with our ongoing programs.  We are a staff of only two.  Therefor, we greatly value our volunteers, as we would not be able to offer all of the programs that we have available without them.	Open to all.	Free	\N	Ongoing, year-round.	None specified.	English!DELIM!Spanish	No	\N	09 Downtown Circulator	Street parking, John Street Garage, Lower Locks Garage.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer Opportunities include: Lock Tending (historic canal locks), river clean-up projects, mailings, restoration of our historic building, land stewardship, landscaping, alewife restoration, and various other opportunities as programs form. \r\n\r\nThe Lowell Parks & Conservation Trust is always looking for volunteers to assist us with our ongoing programs.  We are a staff of only two.  Therefor, we greatly value our volunteers, as we would not be able to offer all of the programs that we have available without them.	Not Applicable	\N	Yes	Spalding House, 383 Pawtucket Street, Lowell\r\n50 person meeting/presentation room - Pentucket Hall	Yes	Please call for a newsletter, 978-934-0030.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	Yes	1	\N
503366	101067	2008-07-11	\N	Child Care	\N	14 Kirk St	\N	Lowell	MA	01852	978-937-5899	\N	978-937-5899	978-937-5148	www.acrefamily.org	public_program_email@testing123.net	Home Based Child Care	Acre offers child care in the homes of a network of family child care providers. Most of the families cared for receive state-subsidized child care services at reduced fees or no fee. The staff and child care providers speak Spanish, Khmer, Vietnamese, Portuguese and English. Transportation is provided for the children to and from child care, and the program participates in the USDA food program.	Must live in Greater Lowell. Call for more information.	Sliding Scale	Income-eligible families receive reduced-rate child care. Call for more information.	Services available year round. Enrollment is ongoing.	Monday-Friday 6AM-6PM, Saturday-Sunday and overnight hours as needed	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Call office for appointment information		Metered parking available on Merrimack and Kirk Streets.	Transportation provided for children to and from child care	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	1	\N
503377	101067	2008-07-11	\N	Training for Family Child Care Providers	\N	14 Kirk St	\N	Lowell	MA	01852	978-937-5899	\N	978-937-5899	978-937-5148	www.acrefamily.org	public_program_email@testing123.net	Training & Support for women starting home based child care	Acre offers training and ongoing support for women wanting to start child care businesses in their own homes. Training is given in Spanish, Khmer and English throughout the year.	Contact program for more information.	Sliding Scale	\N	Program runs year round. Contact office for training schedules.	Monday-Friday 9AM-5PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer	Yes	Contact program for more information.		Metered parking available downtown off Merrimack Street	\N	Yes	Yes	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	1	\N
502145	100682	2008-07-14	\N	Group, Family & Individual Threrapy	\N	77 East Merrimack Street	\N	Lowell	MA	01852	978-453-6800	800-244-4691	\N	978-453-6767	www.southbaymentalhealth.com	public_program_email@testing123.net	Provides individual, couples, family and group therapy	South Bay Mental Health Center provides individual, family and group therapy. South bay works with all ages and all mental health issues, including substance abuse. South Bay has offices in Lowell, Lawrence, Brockton, Weymouth, Fall River, Plymouth and Cape Cod.	Call 1-800-244-4691 and ask for intake to make a referral.	Yes	Most insurances accepted. No sliding scale is available. Reduced rate may be available, call for more information.	Services are available year round.	Monday-Friday 9AM-5PM. Appointments are scheduled by clnicians and availability may vary.	English!DELIM!Spanish!DELIM!Khmer	Yes	Have insurance information ready when calling intake.	02 Belvidere	Parking is available in the lot behind the building, marked for 77 E. Merrimack Street	\N	Yes	No	\N	Yes	Waiting lists do develop and can average 2-4 weeks. Depends on clients' insurance. Please call for current waitlist.	South Bay can provide services in client's home, school, etc.	\N	\N	\N	\N	\N	No	\N	Yes	Contact South Bay for more information about the resource library.	Yes	Space is available for professional use/ counseling sessions	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Newburyport, MA	\N	\N	Varies -- depends on site and staff. Call for more information.	No	1	\N
502585	100781	2008-04-28	\N	LifeLinks Community Center	Community Center Day Program	55 Middlesex Street	\N	North Chelmsford	MA	01863	978-251-0610 x 523	\N	\N	978-251-0570	www.lifelinks.ws	public_program_email@testing123.net	Therapeutic, educational &recreational classes, & activities	The LifeLinks, Inc. Community Center provides therapeutic, educational and recreational classes, activities and sessions for people with developmental disabilities.	Contact program	Yes	Fees paid by DMR or Division of Medical Assistance (Medicaid). Individuals unable pay may be eligible for fee reductions	Program runs throughout the year.	Monday-Friday 8:15AM-2:15PM	English	Yes	Contact program		Parking and transportation varies by event, contact program	\N	Yes	No	\N	No	\N	\N	Omolayo Yvonne Omoruyi	145 Lexington Ave.	Lowell	MA	01854	Yes	Interested volunteers should contact Gail Bartlett, Community Center Director, at 978-251-0610, ext. 523	No	\N	No	\N	Yes	Annual catalog of services; quarterly Director letter	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	sign language	No	1	\N
507755	100781	2008-04-28	\N	LifeLinks Community Connectors	\N	145 Lexington Avenue	\N	Lowell	MA	01854	978-454-4781	\N	\N	978-454-0318	www.lifelinks.ws	public_program_email@testing123.net	Serving people with developmental disabilities	Our mission is to providee a bed and breakfast environment for persons with developmental disabilities. Typical activities would include sports, arts-n-crafts, guided nature hikes, bird watching, dinner theatre, and more. Possible activites are limited only by our guest's imagination!	State	Yes	\N	Open year round	24 hours a day	English	Yes	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	145 Lexington Ave	\N	Lowell	MA	01854	Yes	\N	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	1	\N
503773	100781	2008-04-28	\N	LifeLinks Evening Education	Evening Ed	145 Lexington Avenue	\N	Lowell	MA	01854	978-459-6179	\N	\N	978-453-7077	www.lifelinks.ws	public_program_email@testing123.net	Evening Education classes	Weekly classes held at Middlesex Community College for adults with DD.  Sessions are planned by the particpants, and taught by qualified Instructors feauturing guest speakers from the community.  Topics include: safety, sexuality, money management, human rights, voter responsibility, community knowledege and more.	Adults (over 18) with Developmental Disabilities	Yes	$15.00 annual registration fee; $5.00 per month - waivers available	Open year round	Wednesday - 5p-7p or 5:30p-7:30p	English	No	\N		Double Tree Hotel parking garage	must provide own	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	Yes	Informational brochure	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA	\N	\N	\N	No	1	\N
505280	101914	2008-05-14	\N	DBSA Merrimack Valley	Depression and Bipolar Support Alliance	The First Church, 1195 Varnum Ave., Lowell, MA	\N	Lowell	MA	01852	978-458-0541	\N	\N	\N	www.dbsalliance.org	public_program_email@testing123.net	One hour to one and a half of peer led support group	Groups are open to anyone who has or knows someone who has difficulties with Depression and or Bipolar Disorder at no charge.Stard group formality: Sign in, greet, topic, discussions and resource table.Groups are 1-1/2 hour and meet weekly	Merrimack Valley and surrounding towns	Free	\N	Year round every week	Thursdays at 6:00 at The First Church, 1195 Varnum Ave., Lowell, MA	English	No	self seeking support	07 Pawtucketville / UMass Lowell North!DELIM!09 Downtown Circulator	Parking available at The First Church	Can provide rides with advance notice	Yes	No	\N	No	\N	\N	246 Market Street	Unit 327	Lowell	MA	01852	Yes	Help market the program, set up of groups, material organizer,group facilitate groups, become a Chapter of your area.	Not Applicable	through the main web site www.dbsa.org	Yes	Comfortable conference room holds up to 30 people.  Peaceful environment with ample parking.	Yes	Quarterly through www.dbsalliance.org\r\nor at groups.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	Merimack Valley	\N	No	1	\N
501694	100429	2008-05-27	\N	Cardiac Rehabilitation	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8238	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Cardiac Rehabilitation	The Cardiac Rehabilitation program is an outpatient program for people who are recovering from a cardiac event or who have chest pain. This program assists patients with reestablishing a productive, independent lifestyle. The program aims to: increase the patient's exercise capacity through conditioning, provide the patient with emotional support from the cardiac rehabilitation team and fellow program participants,to reduce the patient's known cardiac risk factors through education and psychosocial support.	Contact program for more information	Yes	No information provided	No information provided	Monday-Friday 7AM-5PM	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Contact the program for more information		Parking available in on-site garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers should contact Cheryl Satryb 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	1	Pager-9782452446\r\n12-18-08 left message and paged
502233	100429	2008-05-27	\N	Customer Service Department	SMC	One Hospital Drive	\N	Lowell	MA	01852	978-934-8255	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Customer Service Department	The Customer Service Department is available to help plan for the needs of patients and their families and answer any questions they may have about SMMC.	Anyone may call with questions	Free	\N	Open year round	No information provided	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	\N		Parking available in on-site garage.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Interested volunteers: contact Cheryl 978-934-8309	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	1	12-18-08 left message with Mary and sent email
503839	100429	2009-01-15	\N	Sleep Disorders Center	\N	One Hospital Drive	\N	Lowell	MA	01852	978-934-8373	\N	\N	\N	www.saintsmedicalcenter.com	public_program_email@testing123.net	Sleep Center	A sleep study is an overnight recording that includes measurements used to identify different sleep stages and classify various sleep problems.	Contact department for information.	Yes	\N	Sleep Study Night Time Hours: \r\nSunday through Friday 7:00pm to 7:00am	Sleep Office Day Time Hours: \r\nMonday through Friday 8:00am to 4:00pm	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Sleep Questionnaire, Physician order unless already sent by the physician office.		On-site parking garage	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	Health related resource library. Call 978-458-1411 for more information	Yes	Outside groups are required to pay a room rental charge. Call extension 4017 to reserve a conference room	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
503542	100506	2009-01-15	\N	Community Support Program	\N	11 Lawrence Street, Suite 322	\N	Lawrence	MA	01840	978-687-1617	\N	978-687-1617	978-687-1597	www.familycontinuity.org	public_program_email@testing123.net	Case mgmt. services to those at risk within the community	The Community Support Program (CSP) is a short term voluntary program which provides intensive case management to individuals who are considered to be "at risk" within the community.  This program offers a single point of contact for coordination and follow up on a consumer's individual care.  The Community Support providers are mobile, flexible and support consumers in the community.  The program works with clients of all ages, including children and adolescents.	Referral Required	Free	MassHealth managed through MBHP or Beacon Health Strategies (Neighborhood Health Plan)	Services available year round	Monday-Friday 9AM-5PM; Saturday, Sunday and evenings:  on call for emergencies only	English!DELIM!Spanish!DELIM!Khmer	Yes	Referral required		Parking available in Common Street garage and on street	FCP provides in home services; transportation is not required	Yes	No	\N	No	\N	Treatment provided: Symptom management, Therapeutic Alliance, Daily Living Skills, Service Coordination, Resource Management and Crisis Management.	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	A conference room that seats about 20 is available for meetings. Contact agency for more information.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
503289	100506	2009-01-15	\N	Family Based Services	\N	11 Lawrence St, Suite 322	\N	Lawrence	MA	01840	978-687-1617	\N	\N	978-687-1597	www.familycontinuity.org	public_program_email@testing123.net	Intensive home-based services and parent aide program.	Family Based Services Program based in the Lawrence office, provides intensive home-based services to families referred by Lowell DSS, Lawrence DSS and Haverhill DSS. The services are involved for three to six months and are designed to be flexible in order to meet the challenging needs of the family. The services are strength based and include various treatment modalities, such as individual, couple and family therapy as well as parent training, advocacy and crisis intervention.  Family Continuity's Parent Aide Program helps parents learn to meet the needs of their children.Parent Aides are partners to parents and help them meet the demands of raising children, job, and home.	Referral from Dept. of Social Services required	Free	\N	Services available year round	Monday-Friday 9AM-5PM; Saturday, Sunday and evenings: on call for emergencies only	English!DELIM!Spanish!DELIM!Khmer	Yes	Referral required		Parking available in Common Sreet garage and on street	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	A conference room that seats about 20 is available for meetings. Contact agency for more information.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	Russian, Armenian	No	0	\N
503300	100506	2009-01-15	\N	Flex Program	\N	11 Lawrence Street, Suite 322	\N	Lawrence	MA	01840	978-687-1617	\N	978-687-1617	978-687-1597	www.familycontinuity.org	public_program_email@testing123.net	Provides services to families of children w/ mental illness	The Flex program provides a number of services to families that have children with chronic mental illnesses and require intensive therapeutic involvement. They provide office or home based individual, couples or family therapy as well as child and parent advocacy, therapeutic horseback riding, and mentoring for the child. Other services, such as art therapy, or home tutoring are provided depending on the family's needs.\r\nCommunity Support Services (CSS) provides free, high quality consultations and lectures on mental health issues by professionals in the field. Topics such as: depression, anger, anxiety, trauma, parenting and discipline can be addressed as well as topics for individual or agency needs.	Flex program requires a referral from Dept. of Mental Health. CSS program is open to all	Free	There are no individual costs as services are contracted through the Dept. of Mental Health	Programs run year round. Waiting lists may develop at times.	Monday-Friday 9AM-7PM	English!DELIM!Spanish	Yes	Contact program for more information. For Flex program, have information on child's diagnosis and DMH referral information		Parking available in Common St. garage and on street	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	A conference room that seats about 20 is available for meetings. Contact agency for more information.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	N. Andover	Spanish, Armenian, Russian	No	0	\N
504059	101408	2009-01-15	\N	Network Health is a MassHealth managed care plan	\N	101 Station Landing	Fourth Floor	Medford	MA	02155	888-257-1985	888-257-1985	\N	\N	www.network-health.org	public_program_email@testing123.net	Network Health is a health plan for MassHealth members	As a MassHealth member enrolled in the Network Health plan, you get all the benefits of MassHealth, plus additional benefits and services from Network Health. As a Network Health member, you can get FREE bike helmets \r\nFREE home child-proofing kits, FREE voice mail, discounts on children's car seats, and more.	This plan offers health care benefits to individuals meeting MassHealth eligibility requirements which may include:  \r\n\r\nchildren under age 19 \r\npregnant women \r\nindividuals who work for a qualified employer who participates in the Insurance Partnership \r\nindividuals under the age of 65 and HIV positive \r\ndisabled people according to the standards set by federal law \r\nan adult relative living with children with whom they are the primary carertaker	Free	\N	Open year round	Monday - Friday, 8:30 a.m. - 5 p.m.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	\N		Parking on-site	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Yes	Network Health members receive a free newsletter, MemberCare, four times a year. It includes information on health topics and Network Health's benefits.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	888-391-5535	\N	Translation services are available in 140 languages	Not Applicable	0	\N
503333	100506	2009-01-15	\N	Individual Parents Training Program	\N	11 Lawrence Street	\N	Lawrence	MA	01840	978-687-1617	\N	978-687-1617	978-687-1597	www.familycontinuity.org	public_program_email@testing123.net	Helping parents grow and learn new parenting skills	The Individual Parents Training program is aimed at adult parents and grandparents who are struggling with anger, poor communication, or parenting skills. The training is done one on one and focuses on helping parents grow personally and learn new parenting skills.	Program is open to all parents and grandparents. To obtain certificate of completion, must attend at least 7 group sessions	Sliding Scale	Private insurance (MassHealth, BCBS, Tufts or NHP) or self-pay. Sliding scale available	12 weeks starting any time. Must enroll by phone first	evening hours	English	Yes	Must call for registration. Bring insurance card to appointment if applicable		Parking available in garage on Common Street, or free on street parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Contact agency for more information.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	COUNTY	Essex County	\N	\N	Russian, Armenian	No	0	\N
507073	100506	2009-01-15	\N	Lowell Creative Family Collaboration Program	\N	391 Varnum Street	\N	Lowell	MA	01854	978-620-2528	\N	978-687-1617	978-687-1597	www.familycontinuity.org	public_program_email@testing123.net	Wrap Around Program	Family Continuitys Wraparound Program uses a strengths-based, client- and family-centered approach to meet the needs of multi-agency-involved families in the community. Services are developed through a team process facilitated by a Wraparound Director and driven by the family.\r\nThe Wraparound Program provides a process for developing effective service interventions based on a familys strengths and needs. Wraparound Program brokers services through state agencies and community providers and have access to flexible funding when resources cannot be found elsewhere.\r\n \r\n Specific Services Offered\r\nInter-Agency Team: Meets with referred families to identify and implement needs-based services.\r\nInnovation: Programs frequently pilot cutting-edge treatment interventions.\r\nParent Support Specialists: Parents of special needs children partner with Wraparound Directors to provide support to other families.	Multi-agency-involved families are referred by community social service providers inclusive of special education.	Free	\N	Open year round	9:00 a.m. to 5:00 p.m. Monday through Friday	English!DELIM!Spanish	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	0	\N
508877	103377	2009-01-15	Hidden on 07-11-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Supported Work Program	SWP	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-725-4761	www.valleyworks.cc	public_program_email@testing123.net	Employment services for TAFDC recipients.	The Supported Work Program (SWP) provides intensive job search, support services and subsidized and unsubsidized job placement to recipients of Transitional Aid to Families with Dependent children (TAFDC). Participants attend daily pre-worksite training between thirty and forty hours a week at the ValleyWorks Career Center. \r\n\r\nAdditional services include: short term skills training, job readiness and job search skills training, buss passes, assistance in arranging child care, and interview clothing if necessary. Extended assistance is available to participants after obtaining employment through Post Employment Services.	Recipients of Transitional Aid to Families with Dependent children.	Free	This program is a Job Seeker Service of the VWCC, and participation is based on program eligibility.	Open year round. Please call ahead in the case of inclement weather.	Mon, Tues, Fri: 8AM - 4:30PM\r\nWed: 9:30AM - 4:30PM\r\nThu: 8AM - 4:30PM	English!DELIM!Spanish	Yes	For more information, please speak with our Customer Service Representatives.	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	After enrollment, gas allowances in the form of financial payments.	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
507106	102178	2009-01-15	\N	Michael B. Christensen Family Support Center	Family Support Center	305 Essex St	\N	Lawrence	MA	01840	978-681-4985	\N	\N	978-681-4949	www.glcac.org	public_program_email@testing123.net	Child Abuse Prevention and Family Education	The Family Support Center, recently renamed the Michael B. Christensen Family Support Center,  was established in 1996, in cooperation with GLCAC, Inc. and  the Exchange Clubs of the Merrimack Valley as part of of a National Exchange Club campaign to prevent child abuse and neglect.  The mission of the Family Support Center is to increase the ability of families to nurture their children, to foster a sense of family, self-sufficiency and empowerment.\r\n\r\nThe Family Support Center has 5 programs working towards the education of parents and children. In the parent aide programs, case managers work with the parents educating them on parenting skills, and other necessary topics depending on the family needs. Our two Parent Child Home Programs send home visitors to work with the parents and their child fostering bonding and literacy through reading and play.  Our Parent Education Program is a twelve week curriculum focused class for parenting skills.	Parent Aide Program 1- DSS referred Greater Lawrence, Lowell and Haverhill.\r\nParent Aide Program 2- The entire Merrimack Valley and lower Rockingham County.\r\nParent Education Program - Anywhere\r\nParent Child Home Program 1 - Lawrence Only, low income- child at risk for low - literacy (teen parent, drug addictions, etc)\r\nParent Child Home Program 2 - Lawrence and Methuen, homeless or doubling - up.  (Same as Parent Child Home Program 1)	Free	All programs are free to clients, no third party billing - grant funded.	Both Parent Aide Programs and Parent Education Program are year round - Monday - Friday (Saturday if necessary). Parent Child Home Program runs school calendar year.	Monday - Friday, 8:30 - 5:30	English!DELIM!Spanish	Yes	INSURANCE ID- SS cards, birthdays of all family members.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	4 garages within walking distance	We do provide bus passes to Parent Education Program participants however, the other programs are home based.	Yes	No	\N	Yes	PARENT CHILD HOME PROGRAM AND PARENT EDUCATION PROGRAM ONLY	\N	305 Essex St	\N	Lawrence	MA	01840	Yes	\N	No	\N	Yes	Meeting space for trainings - 12 maximum	No	\N	Limited	www.glcac.org	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	Lower Rockingham County - Salem NH, Plaistow NH, Seabrook NH, Hampton NH, Pelham NH	\N	No	0	\N
502673	100803	2009-01-15	\N	Adult Education	\N	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	ESL, Young parent program--GED preparation and life skills	The Adult Education program provides instruction in the following areas: adult basic education, including ESOL classes as a satellite of Lowell Adult Education, GED classes for Young Parents.	Open to all adults who live in the Greater Lowell area. YPP for youth between 14 and 21 who are pregnant or parenting.	Free	\N	Services available year round (with vacations); Enrollment is ongoing as slots become available.	Hours vary, contact program for more information	English!DELIM!Khmer	Yes	Contact program for more information on availability - individual programs may have waiting lists		Limited parking in lot across the street from CMAA building.  Do not park in Broadway plaza parking--that parking is for small businesses there.  Plenty of on street parking is available.	\N	Yes	No	\N	Yes	A waiting list may develop	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Some space, 30 seats, is available on a limited basis, preferably business related to CMAA	No	\N	Limited	small computer lab for clients	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
501188	100154	2009-01-15	\N	Department of Veterans Services	\N	375 Merrimack Street	Lowell City Hall, Room #7	Lowell	MA	01852	978-970-4070	\N	\N	978-446-7270	Lowellma.gov	public_program_email@testing123.net	Financial assistance for qualified veterans	The Department of Veterans Services provides financial assistance to qualified veterans, limited medical benefits, and support for widows of veterans and their children.Provides assistance in completing V.A. documents. By appointment, the agent will Visit the Veteran/widow of the Veteran at home.	Must be a veteran or a widow of a veteran and live in Lowell.	Free	\N	Services available year round\r\nBy appointment please	Monday-Friday 8AM-4:30PM	English!DELIM!Spanish	Yes	Have copy of military discharge, DD214 when you call. Bring DD214, rent receipt or mortgage to appointment.Call your Veterans Representative for a complete list of required documentation.		Metered street parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Yes	Veterans services has a small " Veterans corner " located in the Lowell Senior Center News letter	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	This is a Massachusetts benefit, and is available in every city and town. Call this office for a complete listing	\N	No	0	\N
503168	100803	2009-01-15	\N	Young Parent Program	\N	120 Cross Street	\N	Lowell	MA	01854	978-454-6200	\N	\N	978-454-6229	www.cmaalowell.org	public_program_email@testing123.net	alternative education program for young parents	The Young Parent Program is a Department of Transitional Assistance and foundation-funded alternative education program for students between the ages of 14 and 21. Students prepare for their GED (some working at the pre-GED level). The program's main goal is self-sufficiency through basic education and life skills.	Must be between the ages of 14-21.	Free	\N	Services available year round. Enrollment ongoing as slots become available. Waiting list does develop, length varies	Tuesday through Friday 9AM-2PM	English!DELIM!Khmer	Yes	Birth Certificate and another form of identification		Parking is available with on street parking and in a small lot across the street from the CMAA.  Do not park in Broadway plaza parking lot--that is for their small businesses. We do, however, provide transportation for YPP clients.	As funding allows, transportation provided by program	Yes	No	\N	Not Applicable	Wait list sometimes develops, length varies	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	Limited	small computer lab available for students	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	On the phone: English. At program site: English, Khmer	No	0	\N
505038	101826	2009-01-15	\N	Billerica Council on Aging	Billerica Senior Center	25 Concord Road	\N	Billerica	MA	01821	978-671-0916	\N	\N	978-671-1347	www.town.billerica.ma.us	public_program_email@testing123.net	The Senior  Center is a community focal for adults 55+	The Senior  Center is a community focal point where adults 60+ come together for activities, programs and services that reflect their experience and skills; respond to their diverse needs and interests; enhance their dignity; support their independence; and encourage their involvement in and with the Center and the community.\r\n \r\nThe Center also serves as a resource for the entire community for information on aging, support for family caregivers and for development of innovative approaches to addressing aging issues.	Programs and services are geared for older adults age 60 and older. However, we do not discrimminate and those yin their late 50's may participate in sertain programs.	Yes	Some classes have fees associated with them if instructors are paid.	Open year round	8:30-4:00, Monday-through Friday	English	No	\N	13 Billerica via Edson	Free	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are the backbone of the Senior Center.  Their willingness to help and their dedication to Billericas senior population are essential in accomplishing the COAs goal of making a positive difference in the lives of senior citizens.  If you are interested in volunteering, or have a skill to share, we welcome your call.  A variety of opportunities are available.	Yes	The Out Reach department maintains resources on a variety of elder care options.  From services available in the privacy and dignity of ones home to institutional care and other housing alternatives.	Yes	Meeting space is limited to Billerica Sr. Center and community non-profit organizations	Yes	We produce a monthly newsletter, The Outlook	Limited	We have a computer classroom with 8 work stations. When classes are not in session, internet access is available	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
508822	103377	2009-01-15	Hidden on 07-10-2008 by: the MVHub Administrator<BR>Reason: <i>temp -- unapproved</i>	Veterans Services	LVER	439 South Union Street	Building 2, Suite 106	Lawrence	MA	01843	978-722-7000	\N	\N	978-722-7090	www.valleyworks.cc	public_program_email@testing123.net	Veterans representatives provide job search assistance.	Local Veterans Employment Representatives (LVER) are located at the Career Center and are available to assist with job search and career development. The LVER also plays an active role in making referrals to community services and supports.	Veterans.	Free	This program is a Job Seeker Service of the VWCC, and provides universal access to all potential customers	Open year round. Please call ahead in the case of inclement weather.	Mon - Fri: 8AM - 4:30PM**\r\n** 2nd and 4th Wed of each month: 10:00AM - 4:30PM	English	No	Requires membership to ValleyWorks Career Center. Please bring DD-214 to determine eligibility for Veteran Services	39A Colonial Heights / N. Andover Mall!DELIM!39B North Andover Mall / Phillips	Ample free parking available in various lots.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	Members are welcome to use our Resource Room to access a variety of resources including: computer and internet access, newspapers, magazines and other periodicals, fax and copy machines, job listings and databases, computerized labor market information/statistics, and information regarding on-site employer recruitments and job fairs.	No	\N	No	\N	Yes	For job seekers in Resource Room.	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
501562	100407	2009-01-15	\N	Discovery Series	Family Discovery Series	35 Wilder Street, Suite 1	\N	Lowell	MA	01854	978-934-4444	\N	\N	978-934-2017	www.uml.edu/centerforarts	public_program_email@testing123.net	Family oriented performing arts	The Discovery Series presented by the UMass Lowell Center for the Arts offers nationally and internationally known performing artists in a series of Sunday afternoon shows geared for families. Performances may include theatre, music, dance, puppets, magic, acrobatics and comedy.	Open to all.  Ticket sold in advance or at door, if available.	Yes	Tickets range from $10 - $12 per show depending on how many shows are purchased.  Patrons can "make their own series."  Groups of 10 or more are $10 per ticket.	Six performances on Sunday afternoons from October through early May.  One special event in April 2009.	Contact Box Office for dates and times of events; to request a brochure or to purchase tickets.	English	Not Applicable	Not applicable		Free parking in Staff/Faculty lot less than one block from theatre.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer ushers are used to seat patrons.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
503069	100484	2009-01-15	\N	Lowell Middlesex Academy Charter School	\N	67 Middle Street	\N	Lowell	MA	01852	978-656-3165	\N	\N	978-459-0456	http://www.LMACS.org	public_program_email@testing123.net	School catering to dropouts who seek high school diploma	LMACS caters to students who have dropped out of high school who seek a high school diploma. The mission of the school is to enable students to achieve academic, career and social success by providing a supportive community that identifies, encourages and develops students interests and abilities.	Students age 16-21 who have dropped out of school or been referred by school officials. Lowell students receive preference.	Free	\N	New students enter in September, November or March.	Contact the program for hours	English	Yes	High school transcript, health forms		Parking available in Market St. garage.	\N	Yes	No	\N	Yes	Wait List does develop	\N	\N	\N	\N	\N	\N	No	\N	Yes	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
501991	100572	2009-01-15	\N	Community Teamwork SuitAbility Program	SuitAbility	536 Pawtucket Street	\N	Lowell	MA	01854	978-934-8898	\N	\N	978-934-8899	http://www.suitabilityma.org	public_program_email@testing123.net	Free interview/work clothing & work related support	SuitAbility provides free interview/work clothing and work related support services such as "Work Your Image" workshops.	Referrals are required. Economic eligibility is determined by the referring agency.	Free	\N	Services are available year round by appointment. Referrals are required. SuitAbility needs 2 weeks lead time for scheduling.	Monday-Thursday 9AM-5PM	English	Yes	Generally referring agency contacts SuitAbility. Statement of understanding & referral paperwork required at 1st appointment	06 Broadway / UMass Lowell	Parking available on street.	\N	Yes	No	\N	No	\N	\N	Kathleen Hayes	536 Pawtucket St.	Lowell	MA	01854	Yes	Contact Kathleen Hayes for information about volunteer opportunities	No	\N	No	\N	Yes	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA	\N	\N	\N	No	0	\N
500264	100033	2009-01-15	\N	WIC	Women, Infants and Children	45-47 Kirk Street	\N	Lowell	MA	01852	978-454-6397	\N	\N	978-454-9651	www.comteam.org	public_program_email@testing123.net	Provide families with food vouchers and nutrition education	The WIC program promotes the health and well being of women, infants, and children. We provide our families with nutrition education, healthy foods, breastfeeding support and referrals to health and social services in the Greater Lowell area. Lowell WIC is committed to meeting the needs of diverse cultures and collaborates with the larger community to link clients with comprehensive services. NOTICE: Pregnant moms, can count unborn baby as a member of the family. WIC is for working families too! ALL foster children under 5 qualify. US citizenship is not required!.\r\n\r\nServices at three satellite offices are:\r\n\r\nPrenatal Clinic (Thursdays) at Lowell \r\nCommunity Health Center (LCHC)for a pregnant mom's initial visit, 17 Warren St., Lowell, MA 978-446-0788. \r\n\r\nPediatric Clinic (Thursdays)at LCHC, 555 Merrimack St., Lowell, MA 978-746-7724\r\n\r\nTewksbury (2 Fridays/month), 126 Phoenix Ave. 2nd Floor, Lowell, MA 978-454-5100 ext. 199	Pregnant, nursing, or recently delivered women, and children under age 5	Free	\N	Services available year round.	Kirk Street office is open: Monday, Tuesday & Thursday 9:00AM-5:00PM; Wednesday 10:30AM-7:30PM; Friday 8:30AM-4:30PM; Two Saturdays a Month 8:30AM-3:30PM	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Yes	Proof of residency, proof of id, proof of income, proof of birth for baby/children		Parking is located next to the building. Entrance is on the side next to the parking.	\N	Yes	No	\N	No	\N	We offer five Breastfeeding Peer Counselors, 24/7 Warmline support 978 654-5203, Electric pump free rental program.	\N	\N	\N	\N	\N	Yes	Call Program Director for details	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	STATE	MA	\N	\N	Korean and Japanese	No	0	\N
508019	101936	2009-01-15	\N	Boys Exercise Groups at Alternative Clinic	\N	73D Winthrop Ave.	\N	Lawrence	MA	01844	978-685-4280	\N	\N	\N	www.glfhc.org	public_program_email@testing123.net	Boys Exercise Groups Ages 8-16	Empowerement group offered to boys who are obese with risk of heart disease. These groups are activity and exercise based.	Must be referred by a physician.	Free	\N	Open year round.	Monday 4:00 to 5:30pm\r\nThursday 4:00 to 5:15pm	English!DELIM!Spanish	Yes	Insurance information, referral from doctor.	07 Pawtucketville / UMass Lowell North	Parking available.	We have transportation services available for those participants whose parents can not bring them to the program.	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	Large conference room.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Littleton, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
508008	101936	2009-01-15	\N	GLFHC at Lawrence High School	Teen Clinic	70-71 North Parish Road	\N	Lawrence	MA	01843	978-722-8391	\N	\N	978-722-8511	www.glfhc.org	public_program_email@testing123.net	School Based Health Center	GLFHC at Lawrence High School is a School Based Health Center providing primary care to students while they are in school.	Student at Lawrence High School.	Free	Accepts insurance.	Open in school year	School Hours	English!DELIM!Spanish	No	\N	01 Haverhill - Methuen - Lawrence	Around building.	\N	Yes	Yes	Student day care.	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Lawrence, MA	\N	\N	\N	Yes	0	\N
508030	101936	2009-01-15	\N	Family Exercise Groups at Alternative Clinic	\N	73D Winthrop Ave.	\N	Lawrence	MA	01844	978-689-6761	\N	\N	978-685-4280	www.glfhc.org	public_program_email@testing123.net	Family Exercise Groups	Activity and exercise based groups for families with obesity.	Patient must be referred by their physician.	Free	\N	Open year round.	Monday and Wednesday 6:30-7:30pm\r\nTuesday and Thursday 5:15- 6:30pm\r\nTuesday and Thursday 6:30-7:30pm	English!DELIM!Spanish	Yes	Insurance information, referral from doctor if not a patient of the Greater Lawrence Family Health Center.		Parking available.	\N	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Large conference room/gym.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
505016	101815	2009-01-15	\N	Adults with Physical Disabilities Program	\N	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	\N	\N	978-689-4488	www.nilp.org	public_program_email@testing123.net	Services to adults with physical disabilities	We provide advocacy, skills training, peer counseling and Information and Referral. We are also a Personal Care Assistance provider for people needing physical assistance with activities of daily living.	No fees	Free	\N	Open year round	Mon - Fri 9:00 am -5:00 pm	English!DELIM!Spanish	No	\N		Parking is available	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Yes	\N	Yes	\N	Yes	\N	Limited	\N	secret	secret	secret	secret	chester@testing123.net	Yes	COUNTY	Essex County	\N	\N	\N	No	0	\N
509020	101815	2009-01-15	\N	Lawrence Organizing Voices for Empowerment	L.O.V.E.	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	\N	\N	978-689-4488	www.nilp.org	public_program_email@testing123.net	Advocacy group for mental health consumers.	L.O.V.E. Stands for Lawrence Organizing Voices for Empowerment. This program is a self-advocacy group run by and for adults with mental health conditions for the purpose of improving mental health services for themselves in the Merrimack Valley.	Inpatient, outpatient or ex- patient interested in improving the mental health system in the Greater Lawrence Area.	Free	\N	Meetings of this group are help once per month.	Contact program to schedule attendance at group.	English	Yes	\N		Lot parking at NILP.	Accessible transportation is provided.	Yes	No	\N	No	\N	\N	L.0.V.E c/o NILP	20 Ballard Road	Lawrence	MA	01843	Yes	As members of L.O.V.E., we learn to speak up for ourselves as a group, become better informed consumers of mental health services, and advocate for ourselves. We need you to join L.O.V.E. because there is strength in unity.	Yes	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
509031	101815	2009-01-15	\N	The Greater Lawrence Peer Counseling Project	\N	20 Ballard Road	\N	Lawrence	MA	01843	978-687-4288	\N	\N	978-689-4488	www.nilp.org	public_program_email@testing123.net	Peer conseling for mental health consumers.	The Greater Lawrence Peer Counseling Project, funded by the Essex North Site Area of the Massachusetts Department of Mental Health, supports the development of onetoone, caring relationships between mental health consumers. By teaching listening and problemsolving skills to those with psychiatric disabilities, the Project empowers them to provide sensitive, practical support to their peers. \r\n\r\nAll peer counselors receive 11 weeks of interactive training in listening and problemsolving techniques. Those who graduate are paired with DMH-eligible consumers. Everything shared by the two individuals is kept in confidence.	To receive peer counseling services, however, a consumer must be eligible to receive Department of Mental Health Services from its Essex North Site Office, which covers the Greater Lawrence and Greater Haverhill areas.\r\n\r\nAny mental health consumer in the Greater Lawrence or Greater Haverhill Areas, including Newburyport, may become a peer counselor. Peer counselors receive extensive, ongoing supervision. They are expected to work four hours weekly and receive a $50 monthly stipend.	Free	\N	Open year round. Peer counselors receive supervision about once per month.	Counselors and consumers weekly at locations that are mutually convenient.	English!DELIM!Spanish	Yes	\N		In NILP Lot.	Limited transportation to supervision for th peer counselors is provided by staff 1-2 times per month. No transportation to DMH-eligible consumers.	Yes	No	\N	No	\N	\N	Peer Counseling Project c/o NILP	20 Ballard Road	Lawrence	MA	01843	Yes	Peer Counselors Receive Training about:\r\nStigma\r\nDisability awareness\r\nActive listening skills\r\nBurnout prevention\r\nSupervision\r\nCoping with suicide & other crises\r\nRelationship dynamics\r\nConfidentiality policies\r\nFriendship\r\nResources and referrals	Yes	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!Dunstable, MA!DELIM!Newburyport, MA	\N	\N	Peer counseling training and services are offered in both English and Spanish. Those interested in Spanish-language services should contact Oliva Tejada at extension 13.	No	0	\N
502167	100704	2009-01-15	\N	RARA Activities Evening Program	\N	295 High Street	\N	Lowell	MA	01852	978-453-5598	\N	\N	978-446-0270	raraagency.org	public_program_email@testing123.net	Provides activities for mentally challenged individuals	The evening program at RARA provides activities including bingo, bowling, dances, dining out programs, field trips(Canobie Lake Park,Mt. Washington Dinner Cruise, Boarding House Park,concerts,etc). We also offer sport programs (Swimming,Track&Field,Bocce,Cross-country Skiing)so that we can participate in the Special Olympics (Summer, Winter and Senior Games).\r\nProgram guides and calendars are created quartely for mailing. We try to diversify our programs to include new ventures as well as standard requested activities.	Ages 18 and older. Participants must attend an intake interview and orientation to determine what services are best suited for the individual	Yes	Fees vary and are billed quarterly. Limited scholarships available. Annual membership fee charged to all registered participants.	Services available year round. Calendar and registration periods quarterly.	Hours vary according to scheduled events, call for more information. Office hours are Tuesday-Thursday, 9:00am to 3:00pm.	English	Yes	At intake interview, participants need to fill out forms regarding current general and medical information.	02 Belvidere	Parking available in building parking lot.	Members are generally required to meet at RARA. Transportation is provided by RARA to activity	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Most programs are volunteer-based. Volunteers are generally family members, friends, or relatives of clients. New volunteers are cordially invited to join RARA.	No	\N	Yes	Contact RARA for meeting space availability and arrangements	Yes	Issued quarterly; activities, announcements, and information.	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	Nashua and Hudson, NH	\N	No	0	\N
504807	100704	2009-01-15	\N	RARA Day Program	\N	295 High Street	\N	Lowell	MA	01852	978-453-5598	\N	\N	978-446-0270	raraagency.org	public_program_email@testing123.net	Day Program	Day Program\r\nDaily activities scheduled on monthly calendar; bowling, bingo, crafts, field trips, cooking, swimming, etc.	Open to developmentally challenged individual aged 18 and older living in the Greater Merrimack Valley region.	Yes	$15.00 per day	Open year round	Tuesday-Thursday\r\n9:30AM to 1:30PM	English	Yes	Must fill out General Information application and attend intake interview.	02 Belvidere	Yes	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Complete General information application and attend intake interview.	No	\N	No	\N	Yes	Quarterly	No	\N	secret	secret	secret	secret	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
505984	100187	2009-01-15	\N	Spotlight/OnQ Program	\N	6 Southside Road	\N	Danvers	MA	01923	978-624-2310	\N	\N	978-762-3980	www.spotlightprogram.com	public_program_email@testing123.net	Drama and theater- based social pragmatics program	The Spotlight Program is a drama and theater-based social pragmatics program for adolescents that capitalizes on students' strengths and interests in order to create a fun and accessible atmosphere in which to learn and apply social pragmatics skills.	Spotlight students range in age from 9-22 years old. Though the program was originally designed for adolescents with Aspergers Syndrome, High Functioning Autism, and Non-verbal Learning Disorder, great success has been attained with students with a much wider range of diagnoses, including ADHD, Bi-polar Disorder, Depression, Social Anxiety, Pervasive Anxiety Disorder, Williams Syndrome, PDD, PDD-NOS, Obsessive-Compulsive Disorder, Mental Retardation, and Autism. However, no diagnosis is necessary to participate in Spotlight, only an interest in developing greater social-emotional and non-verbal interaction skills.	Yes	\N	open year around for after school, saturday, vacation and summer programming	varied	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Yes	Contact North Shore Arc for information about the resource library	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
502134	100671	2009-01-15	\N	Child Abuse & Neglect Prevention & Intervention	\N	33 East Merrimack Street	\N	Lowell	MA	01852	978-275-6800	800-792-5200	800-792-5200	978-452-5896	www.state.ma.us/dcf/	public_program_email@testing123.net	Dedicated to safety and wellbeing of children	The Department of Children and Families (formerly Department of Social Services) is dedicated to the permanent safety and well being of children who have been abused or neglected in a family setting. We work first and foremost to keep families together. To achieve this goal, DSS provides family-based services to assist parents in better caring for their children. For those children who are unable to remain at home because of severe abuse or neglect or who are at risk for abuse and neglect, DCF provides temporary out of home care through foster care, group care, or residential programs. These short-term measures are meant to provide interim guidance and support to families until the children can safely be returned home. In cases where this is not possible, DCF will seek to provide a child with an alternate permanent situation, such as adoption, guardianship, or, as with older teenagers, independent living.  MSPCC is the lead agency for the Lowell DCF Family Networks system.	Not Applicable	Free	\N	Abuse reporting 24 hours/ day	Monday-Friday 8:45AM-5PM; After hours/ weekend hotline available.	English!DELIM!Portuguese!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	\N	09 Downtown Circulator	city parking lot is behind building	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Support/Volunteer opportunities available. Contact program for more information.	Yes	Resource library available, contact DCF for details.	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Not Applicable	0	\N
508602	102178	2009-01-15	\N	Community Service Center	\N	21 Lawrence Street	\N	Lawrence	MA	01841	978-681-4905	\N	\N	978-681-4979	www.glcac.org	public_program_email@testing123.net	Referrals & direct intervetion.	Formerly called the Spanish Community Program, the GLCAC, Inc. Community Service Center provides information and referral services, notary public, translation, immigration assistance, community organization, advocacy, ase well as other direct services to Hispanic and Non-Hispanic residents.\r\n\r\n + Referrals to other Agencies and/or programs\r\n + Help With Filling out Application and Forms\t\r\n + Tax Return Preparation Simple Form (Referral Only)\r\n + Help With IRS Issues\r\n + Credit Repair and Identify Theft (Referral Only)\r\n\r\nServices also include the Roberto Clemente Little League, citizenship classes, the Funeral Assistance Program, and La VOZ del PUEBLO, an informational radio talk show on WCEC-1490 AM radio.	No income guidelines for any of the programs except the Immigration program.	Free	Mainly free, but small fee for the application to the Immigration program and small fees for participation in the Roberto Clemente Little League.	Open year round.	Monday -- Friday, 8:30 -- 4:30, closed for lunch.	English!DELIM!Spanish	No	\N	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	The Buckley Transportation Center / Parking Garage is less than a block away.  Limited on-street parking also available.	\N	Yes	No	\N	No	\N	\N	GLCAC, Inc.	305 Essex St	Lawrence	MA	01840	Yes	Contact for more information.	Yes	Resources and information & referral capacity.	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	Primary Service area is Greater Lawrence, however, information and referral services are open to anyone contacting the Center.	\N	No	0	\N
508591	102178	2009-01-15	\N	Consumer Protection Program	\N	305 Essex St	\N	Lawrence	MA	01840	978-681-4900	\N	\N	978-681-4949	www.glcac.org	public_program_email@testing123.net	Consumer protection program, help with Small Claims Court.	Provides assistance to consumers in resolving complaints related to products and services. Our Small Claims Court Assistance program component provides consumers with technical assistance when filing a compliant in Small Claims Court. \r\n\r\nThese include: landlord/tenant disputes; automobiles new or used; builders and contractors; home repairs & improvement; credit problems; telemarketing; any service or product related compliant.\r\n\r\nWe also provide outreach to area schools, organizations and the general public, ans we provide free consumer related material such as pamphlets, flyers, copies of State and Federal Rules and Regulations pertaining to Consumer Rights, etc.	Residents by town.	Free	\N	Open year round.	Office hours for program:\r\nMonday -- Friday, 8:30am-4:30pm	English!DELIM!Spanish	No	Any documentation regarding the consumer complaint to be filed.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	The Buckley garage on Common Street is directly behind our building, street parking is available for two hour periods.	\N	Yes	No	\N	No	\N	\N	GLCAC, Inc.	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	Peabody, Middleton, Danvers	\N	No	0	\N
508404	102178	2009-01-15	\N	Head Start Andover Street Campus	Head Start	246-252 Andover Street	\N	Lawrence	MA	01843	978-681-4927	\N	\N	\N	www.glcac.org	public_program_email@testing123.net	Child development services for children and families.	Comprehensive full-day and part-day development services for pre-school children and their families. Head Start classrooms highlight diverse activities: passive and active, individual and group-related, child and teacher initiated.\r\n\r\nHead Start daily serves children a nutritionally balanced breakfast, snacks, and lunch. Through community collaborations, families are provided with referrals to other nutrition resources as needed.\r\n\r\nOur Family Advocates and Home Based Visitors maintain on-going contact with the families through home visits, telephone contacts, written notes, and parental visits to the centers.	Pre-school children must turn 3 or 4 years old by the Sept. 1st prior to entering the program. 10% of enrollment slots are reserved for children with diagnosed disabilities/special needs.	Free	Free services. Contact Head Start program to determine eligibility. Depends on household income and size of family unit. \r\n\r\nParticipating families must be at or below the Federal Family Income Guidelines as published annually in the Federal Register. These are based on the official poverty line derived from national census data. (10% of participants may have incomes in excess of these guidelines and are selected in accordance with criteria established by the local Program Policy Council.)	PRE-SCHOOL\r\nCenter-based -- School Year\r\nChildren attend classes 4.5 hours per day, 4 to 5 days a week, 150 days per year with 2 Teacher Home Visits, 2 parent conferences and additional services provided by Family Advocate.\r\n\r\nHome-based Combination -- School Year\r\nChildren attend classes 4 hours per day, 1 day a week with 3 Home Visits monthly.\r\n\r\nCenter-based -- Full Year\r\nChildren attend classes up to 11 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teacher Home Visits, 2 Parent Conferences and additional services provided by Family Advocate.	Office Hours, M-F 6:30 am-5pm	English!DELIM!Spanish	Yes	Parent or legal guardian must complete full head start application before registration (available at all GLCAC Head Start Centers). \r\n\r\nBring child's birth certificate, child's immunization records, heath insurance card (MassHealth or private), proof of family income from all sources for the past 12 months, including:\r\n\r\n + letter from employers listing year-to date earnings for past 12 months\r\n + 1040 with current pay stub\r\n\r\n + copy of current letter from DTA\r\n + eligibility letter for SSI\r\n\r\n + copy of court support order (alimony/child support)\r\n + letter of unemployment benefits\r\n + veterans benefit		\N	School bus transportation is available to Lawrence and Methuen children who require this service. Limited transportation is available to Andover and North Andover children. Wheel-chair accessible transportation is included.	Yes	Yes	\N	Yes	\N	Since 1965, Greater Lawrence Head Start has been providing a comprehensive, holistic program of education, social services, health, mental health, nutrition, disability services, and parent involvement opportunities to low-income young children and their families.\r\n\r\nHead Start receives federal funding from Health and Human Services (HHS), Administration for Children and Families (ACF) as well Massachusetts supplemental funding by the Department of Education (DOE).	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
507975	102178	2009-01-15	\N	Head Start at the Common	Head Start	305 Essex St	\N	Lawrence	MA	01840	978-681-4920	\N	\N	978-681-4945	www.glcac.org	public_program_email@testing123.net	Child development services for children and families.	Comprehensive full-day and part-day development services for pre-school children and their families as well as for pregnant women, infants, and toddlers through the Early Head Start Option. Head Start classrooms highlight diverse activities: passive and active, individual and group-related, child and teacher initiated.\r\n\r\nHead Start daily serves children a nutritionally balanced breakfast, snacks, and lunch. Through community collaborations, families are provided with referrals to other nutrition resources as needed.\r\n\r\nOur Family Advocates and Home Based Visitors maintain on-going contact with the families through home visits, telephone contacts, written notes, and parental visits to the centers.	Pre-school children must turn 3 or 4 years old by the Sept. 1st prior to entering the program. Infants and Toddlers must be between the ages of birth through age 3. Pregnant women may be in any trimester.\r\n\r\n10% of enrollment slots in both Head Start and Early Head Start are reserved for children with diagnosed disabilities/special needs.	Free	Free services. Contact Head Start program to determine eligibility. Depends on household income and size of family unit. \r\n\r\nParticipating families must be at or below the Federal Family Income Guidelines as published annually in the Federal Register. These are based on the official poverty line derived from national census data. (10% of participants may have incomes in excess of these guidelines and are selected in accordance with criteria established by the local Program Policy Council.)	PRE-SCHOOL\r\nCenter-based -- School Year\r\nChildren attend classes 4.5 hours per day, 4 to 5 days a week, 150 days per year with 2 Teacher Home Visits, 2 parent conferences and additional services provided by Family Advocate.\r\n\r\nHome-based Combination -- School Year\r\nChildren attend classes 4 hours per day, 1 day a week with 3 Home Visits monthly.\r\n\r\nCenter-based -- Full Year\r\nChildren attend classes up to 11 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teacher Home Visits, 2 Parent Conferences and additional services provided by Family Advocate.\r\n\r\nEARLY HEAD START\r\nCenter-based -- Full Year  \r\nToddlers age 15 months through age 3 attend classes up to 10 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teachers Home Visits, 2 Parent Conference, and additional services provided by Family Advocate.\r\n\r\nHome-based -- Full Year  \r\nPregnant women, infants, and toddlers attend group socialization experiences twice a month throughout the year with weekly Home Visits.	Office Hours, M-F 6:30am-5pm	English!DELIM!Spanish	Yes	Parent or legal guardian must complete full head start application before registration (available at all GLCAC Head Start Centers). \r\n\r\nBring child's birth certificate, child's immunization records, heath insurance card (MassHealth or private), proof of family income from all sources for the past 12 months, including:\r\n\r\n + letter from employers listing year-to date earnings for past 12 months\r\n + 1040 with current pay stub\r\n\r\n + copy of current letter from DTA\r\n + eligibility letter for SSI\r\n\r\n + copy of court support order (alimony/child support)\r\n + letter of unemployment benefits\r\n + veterans benefit	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence	School bus transportation is available to Lawrence and Methuen children who require this service. Limited transportation is available to Andover and North Andover children. Wheel-chair accessible transportation is included.	Yes	Yes	\N	Yes	\N	Since 1965, Greater Lawrence Head Start has been providing a comprehensive, holistic program of education, social services, health, mental health, nutrition, disability services, and parent involvement opportunities to low-income young children and their families.\r\n\r\nHead Start receives federal funding from Health and Human Services (HHS), Administration for Children and Families (ACF) as well Massachusetts supplemental funding by the Department of Education (DOE).	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!North Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
508382	102178	2009-01-15	\N	Head Start Methuen Center @ The Central School	Head Start	250 Lawrence Street	\N	Methuen	MA	01844	978-681-4982	\N	\N	\N	www.glcac.org	public_program_email@testing123.net	Child development services for children and families.	Comprehensive part-day development services for pre-school children and their families  Head Start classrooms highlight diverse activities: passive and active, individual and group-related, child and teacher initiated.  Methuen Center offers an A.M. or a P.M. session. \r\n\r\nHead Start daily serves children a nutritionally balanced breakfast, snacks, and lunch. Through community collaborations, families are provided with referrals to other nutrition resources as needed.\r\n\r\nOur Family Advocates and Home Based Visitors maintain on-going contact with the families through home visits, telephone contacts, written notes, and parental visits to the centers.	Pre-school children must turn 3 or 4 years old by the Sept. 1st prior to entering the program. Infants and Toddlers must be between the ages of birth through age 3. Pregnant women may be in any trimester.\r\n\r\n10% of enrollment slots in both Head Start and Early Head Start are reserved for children with diagnosed disabilities/special needs.	Free	Free services. Contact Head Start program to determine eligibility. Depends on household income and size of family unit.	PRE-SCHOOL\r\nCenter-based -- School Year\r\nChildren attend classes 4.5 hours per day, 4 to 5 days a week, 150 days per year with 2 Teacher Home Visits, 2 parent conferences and additional services provided by Family Advocate.  Methuen Center offers an A.M. session or a P.M. session.	Office Hours, M-F 7:45am-4:30pm	English!DELIM!Spanish	Yes	Parent or legal guardian must complete full head start application before registration (available at all GLCAC Head Start Centers). \r\n\r\nBring child's birth certificate, child's immunization records, heath insurance card (MassHealth or private), proof of family income from all sources for the past 12 months, including:\r\n\r\n + letter from employers listing year-to date earnings for past 12 months\r\n + 1040 with current pay stub\r\n\r\n + copy of current letter from DTA\r\n + eligibility letter for SSI\r\n\r\n + copy of court support order (alimony/child support)\r\n + letter of unemployment benefits\r\n + veterans benefit		\N	School bus transportation is available to Lawrence and Methuen children who require this service. Limited transportation is available to Andover and North Andover children. Wheel-chair accessible transportation is included.	Yes	Yes	\N	Yes	\N	Since 1965, Greater Lawrence Head Start has been providing a comprehensive, holistic program of education, social services, health, mental health, nutrition, disability services, and parent involvement opportunities to low-income young children and their families.\r\n\r\nHead Start receives federal funding from Health and Human Services (HHS), Administration for Children and Families (ACF) as well Massachusetts supplemental funding by the Department of Education (DOE).	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Methuen, MA	\N	\N	\N	No	0	\N
508393	102178	2009-01-15	\N	Head Start Park Street Campus	Head Start	62 Park Street	\N	Lawrence	MA	01841	978-681-4982	\N	\N	\N	www.glcac.org	public_program_email@testing123.net	Child development services for children and families.	Comprehensive full-day and part-day development services for pre-school children and their families as well as for pregnant women, infants, and toddlers through the Early Head Start Option. Head Start classrooms highlight diverse activities: passive and active, individual and group-related, child and teacher initiated.\r\n\r\nHead Start daily serves children a nutritionally balanced breakfast, snacks, and lunch. Through community collaborations, families are provided with referrals to other nutrition resources as needed.\r\n\r\nOur Family Advocates and Home Based Visitors maintain on-going contact with the families through home visits, telephone contacts, written notes, and parental visits to the centers.	Pre-school children must turn 3 or 4 years old by the Sept. 1st prior to entering the program. Infants and Toddlers must be between the ages of birth through age 3. Pregnant women may be in any trimester.\r\n\r\n10% of enrollment slots in both Head Start and Early Head Start are reserved for children with diagnosed disabilities/special needs.	Free	Free services. Contact Head Start program to determine eligibility. Depends on household income and size of family unit. \r\n\r\nParticipating families must be at or below the Federal Family Income Guidelines as published annually in the Federal Register. These are based on the official poverty line derived from national census data. (10% of participants may have incomes in excess of these guidelines and are selected in accordance with criteria established by the local Program Policy Council.)	PRE-SCHOOL\r\nCenter-based -- School Year\r\nChildren attend classes 4.5 hours per day, 4 to 5 days a week, 150 days per year with 2 Teacher Home Visits, 2 parent conferences and additional services provided by Family Advocate.\r\n\r\nHome-based Combination -- School Year\r\nChildren attend classes 4 hours per day, 1 day a week with 3 Home Visits monthly.\r\n\r\nCenter-based -- Full Year\r\nChildren attend classes up to 11 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teacher Home Visits, 2 Parent Conferences and additional services provided by Family Advocate.\r\n\r\nEARLY HEAD START\r\nCenter-based -- Full Year  \r\nToddlers age 15 months through age 3 attend classes up to 10 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teachers Home Visits, 2 Parent Conference, and additional services provided by Family Advocate.\r\n\r\nHome-based -- Full Year  \r\nPregnant women, infants, and toddlers attend group socialization experiences twice a month throughout the year with weekly Home Visits.	Office Hours, M-F 6:30am-5pm	English!DELIM!Spanish	Yes	Parent or legal guardian must complete full head start application before registration (available at all GLCAC Head Start Centers). \r\n\r\nBring child's birth certificate, child's immunization records, heath insurance card (MassHealth or private), proof of family income from all sources for the past 12 months, including:\r\n\r\n + letter from employers listing year-to date earnings for past 12 months\r\n + 1040 with current pay stub\r\n\r\n + copy of current letter from DTA\r\n + eligibility letter for SSI\r\n\r\n + copy of court support order (alimony/child support)\r\n + letter of unemployment benefits\r\n + veterans benefit		\N	School bus transportation is available to Lawrence and Methuen children who require this service. Limited transportation is available to Andover and North Andover children. Wheel-chair accessible transportation is included.	Yes	Yes	\N	Yes	\N	Since 1965, Greater Lawrence Head Start has been providing a comprehensive, holistic program of education, social services, health, mental health, nutrition, disability services, and parent involvement opportunities to low-income young children and their families.\r\n\r\nHead Start receives federal funding from Health and Human Services (HHS), Administration for Children and Families (ACF) as well Massachusetts supplemental funding by the Department of Education (DOE).	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
508965	102178	2009-01-15	\N	Head Start Water St. Center @ the Lawrence Boys & Girls Club	Head Start	136 Water Street	\N	Lawrence	MA	01840	978-681-4924	\N	\N	\N	www.glcac.org	public_program_email@testing123.net	Child development services for children and families.	Comprehensive full-day and part-day development services for pre-school children and their families as well as for pregnant women, infants, and toddlers through the Early Head Start Option. Head Start classrooms highlight diverse activities: passive and active, individual and group-related, child and teacher initiated.\r\n\r\nHead Start daily serves children a nutritionally balanced breakfast, snacks, and lunch. Through community collaborations, families are provided with referrals to other nutrition resources as needed.\r\n\r\nOur Family Advocates and Home Based Visitors maintain on-going contact with the families through home visits, telephone contacts, written notes, and parental visits to the centers.	Pre-school children must turn 3 or 4 years old by the Sept. 1st prior to entering the program. Infants and Toddlers must be between the ages of birth through age 3. Pregnant women may be in any trimester.\r\n\r\n10% of enrollment slots in both Head Start and Early Head Start are reserved for children with diagnosed disabilities/special needs.	Free	Free services. Contact Head Start program to determine eligibility. Depends on household income and size of family unit. \r\n\r\nParticipating families must be at or below the Federal Family Income Guidelines as published annually in the Federal Register. These are based on the official poverty line derived from national census data. (10% of participants may have incomes in excess of these guidelines and are selected in accordance with criteria established by the local Program Policy Council.)	PRE-SCHOOL\r\nCenter-based -- School Year\r\nChildren attend classes 4.5 hours per day, 4 to 5 days a week, 150 days per year with 2 Teacher Home Visits, 2 parent conferences and additional services provided by Family Advocate.\r\n\r\nHome-based Combination -- School Year\r\nChildren attend classes 4 hours per day, 1 day a week with 3 Home Visits monthly.\r\n\r\nCenter-based -- Full Year\r\nChildren attend classes up to 11 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teacher Home Visits, 2 Parent Conferences and additional services provided by Family Advocate.\r\n\r\nEARLY HEAD START\r\nCenter-based -- Full Year  \r\nToddlers age 15 months through age 3 attend classes up to 10 hours a day, 5 days a week, 51 weeks a year (program closed one week in August) with 2 Teachers Home Visits, 2 Parent Conference, and additional services provided by Family Advocate.\r\n\r\nHome-based -- Full Year  \r\nPregnant women, infants, and toddlers attend group socialization experiences twice a month throughout the year with weekly Home Visits.	Office Hours, M-F 6:30am-5pm	English!DELIM!Spanish	Yes	Parent or legal guardian must complete full head start application before registration (available at all GLCAC Head Start Centers). \r\n\r\nBring child's birth certificate, child's immunization records, heath insurance card (MassHealth or private), proof of family income from all sources for the past 12 months, including:\r\n\r\n + letter from employers listing year-to date earnings for past 12 months\r\n + 1040 with current pay stub\r\n + copy of current letter from DTA\r\n + eligibility letter for SSI\r\n + copy of court support order (alimony/child support)\r\n + letter of unemployment benefits\r\n + veterans benefit		\N	\N	Yes	Yes	School bus transportation is available to Lawrence and Methuen children who require this service. Limited transportation is available to Andover and North Andover children. Wheel-chair accessible transportation is included.	Yes	\N	Since 1965, Greater Lawrence Head Start has been providing a comprehensive, holistic program of education, social services, health, mental health, nutrition, disability services, and parent involvement opportunities to low-income young children and their families.\r\n\r\nHead Start receives federal funding from Health and Human Services (HHS), Administration for Children and Families (ACF) as well Massachusetts supplemental funding by the Department of Education (DOE).	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
508536	102178	2009-01-15	\N	Lead Worker Training	Lawrence Lead Abatement Program	29 South Canal Street, Suite 205	\N	Lawrence	MA	01843	978-722-6580	\N	\N	\N	www.glcac.org	public_program_email@testing123.net	Lead worker training and licenses.	Lead Worker Training opportunities are available through the Lawrence Lead Abatement Programs alliance with Environmental Management Training, Inc. (EMT), who will be providing lead worker training to 140 low-income Lawrence residents. \r\n\r\nIndividuals who successfully complete the training sessions will receive Lead Worker Certification and medical examinations required to work in the Lead Abatement industry. In addition, the Lawrence Lead Program will pay for lead worker licenses through the Massachusetts Division of Occupational Safety. Graduates will also receive protective personal equipment, including respirators, gloves and coveralls. ETA's trainees will also receive a pre-employment physical examinations from another program partner, Lawrence General Hospital. All successful trainees will then receive a Massachusetts Lead Worker license, also paid for by the program.	Low-income Lawrence residents.	Free	\N	Varies.  Call for additional information.	Varies.  Call for additional information.	English!DELIM!Spanish	Yes	For about this, program please contact Environmental Management Training, Inc.		\N	\N	No	No	\N	No	Class sizes are limited.  Call for information.	\N	Environmental Management Training, Inc.	29 South Canal Street, Suite 205	Lawrence	MA	01843	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lawrence, MA	\N	\N	\N	No	0	\N
508415	102178	2009-01-15	\N	Low-Income Home Energy Assistance Program	LIHEAP	305 Essex St	\N	Lawrence	MA	01840	978-681-4950	\N	\N	978-681-4954	www.glcac.org	public_program_email@testing123.net	Assists low-income persons with heating their homes.	Program provides eligible households with help in paying a portion of their winter heating bills. The program pays benefits of fixed amounts based on household income. An additional benefit is available to eligible households having a high energy burden.	Household eligibility is based on annualized income and the number of members in the household.  Households with incomes up to 200% of the Federal Poverty Level are eligible for the Fuel Assistance Program. Special provisions are made for those households whose heat is included in their rent and those living in subsidized housing.	Free	Payment is made towards the heating bills to the primary heat source vendor (oil, propane, coal, wood, gas, or electric). Homeowners and renters are both eligible.	Applications are accepted from November 1st to April 30.	Monday - Friday 8:30 am - 4:30 p.m.	English!DELIM!Spanish	No	Participants will need to provide proof of income from all sources for the last four weeks. Income includes employment, TAFDC, Social Security, SSI, child support, alimony, dividends & interest on bank accounts, rental income, unemployment benefits, pensions, cash prizes and regular support from family and friends. In addition, participants must provide personal identification and address verification.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence	\N	Yes	No	\N	Not Applicable	\N	\N	GLCAC, Inc. LIHEAP Program	305 Essex St	Lawrence	MA	01840	No	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	Also Reading and North Reading	\N	No	0	\N
508492	102178	2009-01-15	\N	Merrimack Valley Lead Poising Prevention Program	MVLPPP	305 Essex St	\N	Lawrence	MA	01840	978-681-4940	\N	\N	978-681-4949	www.glcac.org	public_program_email@testing123.net	Lead poisining screening for children, and deleading help.	MVLPPP provides lead poising screening and follow-up for children under age 6, and provides technical assistance for deleading throughout the Merrimack Valley. \r\n\r\nWhen screening, a small amount of blood is taken from the childs finger and tested for lead. If the test is elevated, another test will be done. (Lead poisoning is a condition caused by swallowing or inhaling lead.)\r\n\r\nMVLPPP also provides clients with a centralized source of information, referrals and education on all aspects of the Massachusetts Lead Law.	Children up to the age of six 6.	Free	\N	Open year round.	Lead Screening \r\nMonday -- Friday, 9:00 A.M. to 3:00 P.M.	English	No	\N	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence.	\N	Yes	Not Applicable	\N	No	\N	The Merrimack Valley Lead Poisoning Prevention Program (MVLPPP) was established in 1976, as a provider to the Massachusetts Department of Public Health Lead Poisoning Prevention Program (CLPPP). Originally, the service area covered Greater Lawrence communities. In 1979, the program expanded to Greater Haverhill and Greater Lowell serving a total of twenty-three (23) communities with emphasis to the core neighborhoods of Lawrence, Lowell, Haverhill and Methuen.	GLCAC, Inc. Merrimack Valley Lead Poisoning Prevention	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	The primary service area for home visiting includes 23 Merrimack Valley communities.  Environmental services are provided to those 23 communities plus an additional 22 cities and towns.   Call for information on those communities.	\N	No	0	\N
508580	102178	2009-01-15	\N	Project SENIOR	\N	305 Essex St	\N	Lawrence	MA	01840	978-681-4900 x406	\N	\N	978-681-4949	www.glcac.org	public_program_email@testing123.net	Home rehabilitation for seniors.	Project SENIOR (Serving Elder's Needs In Our Region)provides assistance to senior citizens and/or disabled residents with needed home repairs related to safety or accessibility concerns.	Clients must be over 60 years old, low-income or disabled.	Free	\N	Office is open year round.	Office Hours: M-F, 8:30am-4:30pm	English	Yes	ADD INFO	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	Buckley Transportation Center / Parking Garage is behind 305 Essex Street.	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	GLCAC, Inc. Project SENIOR	305 Essex St	Lawrence	MA	01840	Yes	Contact program for more information about volunteering with Project Senior.	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	Primary service area is Lawrence with some funding available to serve Andover, Methuen and North Andover.	\N	No	0	\N
508437	102178	2009-01-15	\N	Weatherization Assistance Program	WAP	305 Essex St	\N	Lawrence	MA	01840	978-681-4956	\N	\N	978-681-4954	www.glcac.org	public_program_email@testing123.net	Home energy efficiency services.	This program was designed to assist low-income households with their heating bills by providing home energy efficiency services. After a household is determined eligible, A certified Massachusetts Weatherization Energy Auditor completes an inspection of the home for energy, health & safety concerns. Work will be done by local licensed and insured contractors at no cost to the occupants. \r\n\r\nTypical work includes, air sealing, attic and/or sidewalk insulation, weather-stripping, and or minor repairs associated with the weatherization work. All work receives a thorough Quality Control Inspection by the agencies Energy Auditor. Households with incomes up to 200% of the Federal Poverty Level are eligible. Households with exceptionally high energy bills also may receive priority service.	Program eligibility is based on household eligibility for the (LIHEAP) Program. \r\nTenants (with landlord approval) as well as homeowners are eligible.\r\n\r\nIn addition, those households with a member receiving AFDC or SSI are categorically eligible.	Free	\N	Open year round.	Monday - Friday 8:30 am - 4:30 p.m.	English!DELIM!Spanish	Yes	Participants may need to provide proof of income from all sources for the last four weeks. Income includes employment, TAFDC, Social Security, SSI, child support, alimony, dividends & interest on bank accounts, rental income, unemployment benefits, pensions, cash prizes and regular support from family and friends. In addition, participants must provide personal identification and address verification.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence	\N	Yes	No	\N	Yes	Due to limited funding, priority points are awarded to households with members who are elderly, handicapped, and have children under 6.	\N	GLCAC, Inc. Weatherization Program	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Lowell, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA	\N	and Pepperell	\N	No	0	\N
508426	102178	2009-01-15	\N	Heating Emergency Assistance Retrofit Task Weatherization	HEARTWAP	305 Essex St	\N	Lawrence	MA	01840	978-681-4955	\N	\N	978-681-4949	www.glcac.org	public_program_email@testing123.net	Heating system repair and replacement services,	The HEARTWAP Program was designed to provide heating system repair and replacement services to low income households. During the winter months, the program serves primarily as an emergency intervention service to provide assistance to low income property owners having problems with the operation of their primary heating system. Heating companies, under contract, provide services necessary to keep the heating system operating efficiently and safely.	Households with emergency, no-heat situations, or households simply burdened with excessively high heating bills due to inefficient or malfunctioning heating systems.\r\n\r\nParticipants must be LIHEAP (fuel assistance) eligible and the homeowner.	Free	\N	Emergency program for maintaining winter heat.	Monday - Friday 8:30 am - 4:30 p.m.	English!DELIM!Spanish	Not Applicable	When applying for Fuel Assistance, participants will need to provide proof of income from all sources for the last four weeks. Income includes employment, TAFDC, Social Security, SSI, child support, alimony, dividends & interest on bank accounts, rental income, unemployment benefits, pensions, cash prizes and regular support from family and friends. In addition, participants must provide personal identification and address verification.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence	\N	Yes	No	\N	Yes	\N	\N	GLCAC, Inc. HEARTWAP Program	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Billerica, MA!DELIM!Lowell, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA	\N	Pepperell	\N	No	0	\N
508448	102178	2009-01-15	\N	Mass Electric Appliance Management Program	MECOAMP	305 Essex St	\N	Lawrence	MA	01840	978-681-4955	\N	\N	978-681-4954	www.glcac.org	public_program_email@testing123.net	Free audit of high household electric usage.	The MECOAMP Program is interested in helping clients save on their electric bills. If qualified, clients will receive a free assessment of their electric appliances by a highly trained auditor who can determine the cause of high electric usage. \r\n\r\nA plan will then be provided on how much you might be able to save. Clients will receive energy efficient light bulbs, water conservation materials, and possibly be eligible for a new refrigerator or freezer if the existing appliance is using too much energy.	You have to live in your place or residence for one year, have a R-2 rating on your bill, and use more than 10kWh per day to be eligible. Homeowners as well as renters can qualify.	Free	\N	Open year round.	Monday - Friday 8:30 am - 4:30 p.m.	English	Yes	\N	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	City owned parking garage on Common Street behind GLCAC, Inc. 305 Essex Street, Lawrence.	\N	Yes	No	\N	No	\N	\N	GLCAC, Inc. Weatherization Program	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	North Andover, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Methuen, MA	\N	\N	\N	No	0	\N
508481	102178	2009-01-15	\N	Women Infants and Children Program	WIC	305 Essex St	\N	Lawrence	MA	01840	978-681-4960	\N	\N	978-681-4917	www.glcac.org	public_program_email@testing123.net	nutrition services for women with children under age 5.	The Women, Infants and Children Program provides free nutrition services to help keep pregnant women, infants and children under five healthy and strong.\r\n\r\nThese services include: \r\n\r\n + Counseling on how to use food to improve health.\r\n + WIC checks to buy foods.\r\n + Information on eating well during pregnancy, breastfeeding and postpartum recovery.\r\n + Breastfeeding support.\r\n + Infant feeding and care issues.\r\n + Child feeding and care issues.\r\n + Immunization screening and referrals.\r\n + Discussion on food shopping and recipes.\r\n\r\nIn addition, program staff will assist with referrals to doctors, health centers and clinics, dentists and programs like Food Stamps, TAFDC, MassHealth, Healthy Start, Early Intervention, Children's Medical Security Plan, Child Support Services, daycare and fuel assistance.	Applicants must:\r\n\r\n+ Be pregnant, breastfeeding or postpartum, or have a child under five\r\n + Live in Massachusetts\r\n + Have a total household income of less than 185% of the federal poverty guidelines\r\n + Have a nutritional need\r\n\r\nFathers, guardians and foster parents may apply for WIC on behalf of their children.	Free	\N	Open year round.	M W F: 830am-430pm, closed 12-1pm for lunch\r\nT Th: 10am-6pm, closed 1-2 for lunch\r\nOpen one Saturday per month.	English!DELIM!Spanish	Yes	Call office to schedule appointment. Persons may be income eligible for WIC if they currently receive TAFDC, Food Stamps or MassHealth.	01 Haverhill - Methuen - Lawrence!DELIM!36 Lawrence Street / Holy Family Hospital!DELIM!41 Lawrence-Lowell!DELIM!73 Lawrence	Buckley Transportation Center / Parking Garage is located behind 305 Essex Street.	\N	Yes	No	\N	No	\N	\N	GLCAC, Inc. WIC Program	305 Essex St	Lawrence	MA	01840	No	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
503399	101078	2009-01-15	\N	Foster Care/PermanencyProgram	\N	18 Palmer Street	\N	Lowell	MA	01852	978-937-1877	800-883-8836	978-937-1877	978-937-2262	www.caseyfamilyservices.org	public_program_email@testing123.net	Foster Care Training & Support/permanency services	The Foster Care program helps children who cannot live with their birth parents. The focus of our program is to ensure that youth in foster care achieve permanent, legal family membership, whether through reunification with their birth families or through adoption or legal guardianship.  Casey Family Services recruits committed, skilled foster parents and supports them with training and a range of ongoing services to insure a stable, supportive family. Casey Family Services wraps additional individually developed supports and services to address the unique needs of each child. Referrals are received exclusively from the Department of Social Services.	Children are referred exclusively from the Department of Social Services.	Free	Services are provided through contracts with the Department of Children and Families.  Services are free to children, youth, foster families and birth families.	Contact program	Contact program	English!DELIM!Spanish	Yes	Referrals required		on street and garage	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Contact Casey for more information on becoming a foster parent	No	\N	Yes	we have two conference rooms. Contact Sonia DeJesus for more information.	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	STATE	MA	\N	\N	\N	No	0	\N
502893	100880	2009-01-15	\N	Flu and Pneumonia Clinic	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0932	\N	\N	978-671-0919	www.town.billerica.ma.us	public_program_email@testing123.net	Flu and pneumonia clinics to Billerica residents	The Billerica Board of Health provides an annual flu and pneumonia clinic to Billerica residents only.  Clinics are prioritized/scheduled according to age/high risk individuals; depending upon availability-to all others.  Age 18 and under need a doctors order and/or can schedule an appointment with the town pediatrician through the Board of Health.	Billerica residents	Free	\N	Flu vaccine available Late October through January.  Pneumonia vaccine available throughout the year.	Clinics run at different times and days, contact Board of Health or check Lowell Sun,\r\nBillerica Minuteman, and cable channel	English	No	Call for clinic times and sites. Bring current insurance card - HMO plan or Medicare		Parking available in front lot of Town Hall	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer nurses and elders to help register community members. Call the nurse at \r\n978-671-0932.	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA	\N	\N	\N	No	0	\N
502915	100880	2009-01-15	\N	Lead Testing	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0932	\N	\N	978-671-0919	www.town.billerica.ma.us	public_program_email@testing123.net	Free Lead Testing for children 9 months - 7 years	The Billerica Board of Health provides free lead testing to children age 9 months to 4 years by appointment only.	Billerica residents age 9 months to 4 years of age or older if at high-risk.	Free	\N	Program runs year round	By appointment only. Call for information	English	Yes	Appointments are required		Parking available in front lot of Town Hall	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA	\N	\N	\N	No	0	\N
502904	100880	2009-01-15	\N	Free Blood Sugar / Glucose Screening	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0932	\N	\N	978-671-0919	www.town.billerica.ma.us	public_program_email@testing123.net	Free blood sugar screening	The Billerica Board of Health provides blood sugar screenings.  Appointments are needed.	Billerica residents of any age (if under 18 they must have parental consent)	Free	\N	Program runs year round	Hours vary, call for times.	English	No	Appointments are necessary.		Parking available in front lot of Town Hall	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA	\N	\N	\N	No	0	\N
502926	100880	2009-01-15	\N	Total cholesterol and HDL	\N	365 Boston Road	\N	Billerica	MA	01821	978-671-0932	\N	\N	978-671-0919	www.town.billerica.ma.us	public_program_email@testing123.net	Provides total cholesterol and HDL testing	The Billerica Board of Health provides total cholesterol and HDL testing. Cholesterol is analyzed using the Cholesterol LDX. The program is open to residents only. An appointment is necessary. Please call 978-671-0932 to schedule an appointment. The Board requests residents take one test per calendar year to allow more residents the opportunity to be tested. Every April town health-fair.	Billerica residents of any age. For residents under 18, parental consent required	Yes	Call for information	Offered 1-2x/year.	Call for information.\r\nAppointments are required	English	Yes	Contact program for more information		Parking available in lot in front of Town Hall	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Not Applicable	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA	\N	\N	\N	No	0	\N
500253	100033	2009-01-15	\N	Lowell Small Business Assistance Center	\N	88 Middle Street  Room 202	\N	Lowell	MA	01852	978-322-8400	\N	\N	978-441-6824	www.lowellsmallbusiness.org	public_program_email@testing123.net	Information & Resources for new & existing small businesses	The Small Business Assistance Center provides entrepreneurs from ethnically and economically diverse groups the education, tools and resources needed to launch, sustain or expand small businesses.	All ages and income levels welcome, contact program for more information	Free	There is no charge for Lowell residents to use the Center.  People from outside of Lowell pay a $50 one year membership fee for the use of the Center. The Right Start entrepreneurship training program is $90 for non residents and $60 for Lowellians.	Program available year round, enrollment is ongoing	Monday-Tuesday 9AM-5PM; Wednesday 9:00AM-9:00PM Thursday-Friday 9AM-5PM	English!DELIM!Portuguese!DELIM!Spanish	Yes	Contact program	09 Downtown Circulator	\N	\N	Yes	No	\N	No	\N	Located 2 blocks from downtown bus stop	\N	\N	\N	\N	\N	No	\N	Yes	Books on business, and how to manuals. Computers with Internet access	No	\N	Yes	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Groveland, MA!DELIM!North Andover, MA!DELIM!Andover, MA!DELIM!Haverhill, MA!DELIM!Rowley, MA!DELIM!Bedford, MA!DELIM!Lawrence, MA!DELIM!Salisbury, MA!DELIM!Billerica, MA!DELIM!Littleton, MA!DELIM!Tewksbury, MA!DELIM!Boxford, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Merrimac, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Methuen, MA!DELIM!West Newbury, MA!DELIM!Dunstable, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Georgetown, MA!DELIM!Newburyport, MA	\N	\N	\N	No	0	\N
507040	102112	2009-01-15	\N	Cambodian Folktale/Theater	\N	9 Central Street, Suite 203	\N	Lowell	MA	01852	978-459-0333	\N	\N	\N	www.reaksmey.org	public_program_email@testing123.net	Preserving Cambodian Folktale	Throughout 2007, a remarkable group of Cambodian teenagers and elders worked together in Lowell, Massachusetts to collect this and other Khmer folktales, with the goal of preserving and publishing these important cultural texts. \r\n\r\nThe Cambodian Folktale project began because individuals in the Cambodian community recognized the need to document and circulate these stories, which carry a multitude of life lessonsabout how we should live together in this world, what can go wrong, and how to right the wrongs. Our hope is that families will read this story together and preserve it for future generations.	Open to all youth ages 7 - 21	Free	\N	year round	Tuesday & Thursday 3 pm - 5 pm	English!DELIM!Khmer	No	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Yes	0	\N
507062	102112	2009-01-15	\N	Mentoring and Advocacy	\N	9 Central Street, Suite 203	\N	Lowell	MA	01852	978-459-0200	\N	\N	\N	www.reaksmey.org	public_program_email@testing123.net	Encourage. Enrich. Educate.	MAP help positive impact the lives of at-risk youth by facilitating their healthy transition from adolescence to adulthood.	Youth ages 12 - 18.	Free	\N	Year round	Depend on mentor and mentee schedule	English!DELIM!Khmer	Yes	Referral and registration form		\N	\N	Yes	No	\N	Yes	It's depend. Some quarter, youth need to be on a waiting list due the volume of application.	\N	\N	\N	\N	\N	\N	Yes	Be a mentor	No	\N	Yes	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	Yes	0	\N
504664	101177	2009-01-15	\N	Destination World	Multicultural Events on Weekends	375 Merrimack Street	\N	Lowell	MA	01852	978-674-1482	\N	\N	978-446-7089	www.cultureiscool.org	public_program_email@testing123.net	Film, Music, Art, Family Fun and more in Downtown Lowell!	Destination World celebrates a different culture through film, art, music, dance and family fun! We seek to bring new artists, filmmakers, residents, families and youth to Downtown Lowell to participate in activities that promote a vibrant cultural life.	We are open to the public.	Free	This program is free and open to the public!	Year Around	12 pm - 9pm, Saturdays	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	Not Applicable	\N		Free Parking in the National Park Visitor Center Parking Lot	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	The COOL office can comfortably accommodate a meeting for 8-10 people.	Yes	Monthly E-Newsletter	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	Greater Merrimack Valley; Greater Boston; New England	\N	Not Applicable	0	\N
501496	100319	2009-01-16	\N	NAMI Greater Lowell	\N	P.O. Box 692	\N	Tyngsborough	MA	01879	978-677-0618	\N	\N	\N	www.nami.org/sites/greaterlowell	public_program_email@testing123.net	Support group for mental illness	NAMI of Greater Lowell is a support group of concerned individuals responding to the tragic neglect of our mentally ill citizens. They provide sharing and caring, compassionate support and understanding for all our members and anyone else seeking this support.	Anyone who is interested may contact the program	Free	Not applicable	Meetings are held the 3rd Wednesday of each month at 7PM at Solomon Mental Health Center 391 Varnum Ave. Lowell, MA	3rd Wednesday of the month 7PM at Solomon Mental Health Center	English	No	Not applicable		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA	\N	\N	\N	No	0	\N
503894	101331	2009-01-16	\N	Lawrence HERC	\N	78-82 Amesbury St.	\N	Lawrence	MA	01841	978-738-7634	\N	\N	\N	www.lawrence-herc.org	public_program_email@testing123.net	SAT Prep Classes, scholarship searches, college advising,	Lawrence Higher Education Center (HERC)provides  college access information to students and their families. Our program offerings include: SAT Prep classes, Passport/Bridge - a college prep program, and a drop in center open daily that includes a computer lab with Internet access, a library of college and scholarship information and college advising.	Lawrence HERC is dedicated to serving the under privileged and underrepresented youth of Lawrence and the Merrimack Valley.  The current class costs $30 for the semester.	Free	All services except for the Passport program and SAT prep are free.  Costs incurred by students are very low.	Open year round!	Monday thru Friday\r\nBridge: Wednesday 4-6\r\nPassport: Tuesday 4-6	English!DELIM!Spanish	Not Applicable	\N		There is a large parking lot in the back of the Northern Essex Community College building, in which we are located. We are also directly across from the Buckley Transportation Depot and its parking garage.	\N	Yes	No	\N	Yes	\N	\N	P.O. Box 1758	\N	Lawrence	MA	01842	Yes	If you are interested in volunteering please contact Meghan Aceto at 978-738-7634. We are currently looking for Math SAT Prep volunteers during our weekly Bridge meetings.	Yes	A small resource library of college access and high school access materials.	Yes	We have a student resource room, access to a larger computer lab and classroom space as necessary.	No	\N	Yes	\N	secret	secret	secret	secret	chester@testing123.net	Yes	TOWN	Haverhill, MA!DELIM!Lawrence, MA!DELIM!Lowell, MA!DELIM!Methuen, MA	\N	\N	\N	Not Applicable	0	\N
501100	100154	2009-01-16	\N	Assessors Office	Board of Assessors	City Hall	375 Merrimack Street	Lowell	MA	01852	978-970-4200	\N	\N	978-970-4205	City of Lowell.com	public_program_email@testing123.net	Assess property and inform public about assessments for taxe	The city of Lowell Assessors strive to insure assessments are accurate and fair to the market (real estate) and uniform across the city. We also make every attempt to get information out to the elderly, veterans and others about statutory exemptions. The assessors office processes all statutory exemptions for elderly, widow(ers), veterans (at least 10% disability), blind, disabled taxpayers. Abatements are processed for all excise taxes when taxed vehicles are sold, traded, etc.. We provide certified abutters letters for all parcels located within the city inspect properties for upgrades through the building permitting process. Provide certified homeownership letters.	Not applicable	Yes	The fee for an abutters list is $25.00.  Informational reports vary according to the class or information requested. Record cards free to the property owner, $1.00 a copy for all others.	Open Year Round	Monday-Friday 8AM-5PM	English	No	If this is the first time filing for an elderly exemption, a birth certificate is required.  First time Widow/Widowers exemption, a death certificate.  Veteran's should bring in proof of disability from the Veterans Administration.		Metered street parking outside City Hall.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	The Assessors Office accepts volunteers to type, file and work on special clerical projects	No	\N	No	\N	No	\N	Yes	www.lowellma.gov	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
509438	103498	2009-01-14	\N	Attorneys	\N	200 Littleton Road	\N	Westford	MA	01886	978-692-3107	\N	\N	978-692-5454	www.lawbas.com	public_program_email@testing123.net	Law firm offering immigration law services	We practice in the areas of both family based and employment based immigration law.  Our experience is in representing both individuals and companies in their immigration needs.	must need immigration services	Yes	We offer both pre-set fees and hourly rates.	Open year round.	Monday through Friday from 8:30 to 5:00.	Portuguese	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Yes	We have a conference room that will accomodate 12-15 people for a meeting	Yes	Monthly newsletter on immigration issues via e-mail.	No	\N	secret	secret	secret	secret	chester@testing123.net	Yes	EVERYWHERE		\N	\N	Attorney Santos is fluent in Portuguese and English.	Not Applicable	0	\N
504774	101056	2008-11-13	\N	Elder Outreach to Southeast Asian communities	\N	144 Merrimack St., Suite 202	\N	Lowell	MA	01852	978-459-9031	\N	\N	978-459-0154	www.iilowell.org	public_program_email@testing123.net	Assistance to Southeast Asian elders (60 and up)	Assists elderly who speak Khmer, Thai and Vietnamese to obtain citizenship, obtain legal assistance on immigration issues and to access benefits such as Social Security, unemployment,state ID cards.  Interpreting is provided; citizenship exam tutoring is available for clients who have held green cards for 15 years or longer.	BY REFERRAL ONLY -- Southeast Asians individuals aged 60+, with limited English, needing translation to access benefits	Free	Contributions are accepted to enhance the program services.	Very limited hours; so referrals must come from other providers of services. Services may be provided outside of office.	varies	Khmer!DELIM!Vietnamese	Yes	identification showing age		Services may be provided outside of office; for on-site services parking is available at john St. or Market St. garages	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Opportunities for fluent speakers of Southeast Asian languages to provide tranlation.	No	\N	No	\N	No	\N	No	\N	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Billerica, MA!DELIM!Tewksbury, MA!DELIM!Lowell, MA!DELIM!Tyngsborough, MA!DELIM!Chelmsford, MA!DELIM!Westford, MA!DELIM!Dracut, MA!DELIM!Dunstable, MA	\N	\N	\N	No	0	\N
500341	100044	2008-11-13	\N	Southeast Asian Resources for Culture and Health Program	Cambodian Community Health 2010 (CCH2010)	586 Merrimack Street	\N	Lowell	MA	01854	978-746-7829	\N	\N	978-937-3918	www.cch2010.info and www.lchealth.org	public_program_email@testing123.net	Focus on cardiovascular health and diabetes & education	The goal of SEARCH is to eliminate health disparities among the Cambodian population in Greater Lowell.	Services available to Cambodian residents of Greater Lowell, Massachusetts	Free	\N	Services available year round	Monday-Friday 9AM-5PM	English!DELIM!Khmer	Yes	Please contact the Program Director at 978-746-7829 for more information.		In patient lots behind 585-597 Merrimack Street.	\N	Yes	No	\N	No	\N	978-452-4774 TDD	\N	\N	\N	\N	\N	Yes	Please contact the Program Director at 978-746-7829 for more information.	No	\N	No	\N	Yes	Topics include heart disease, HBP, diabetes printed in Khmer.	No	www.cch2010.info	secret	secret	secret	secret	chester@testing123.net	No	TOWN	Lowell, MA	\N	\N	\N	No	1	\N
502871	100869	2008-11-13	\N	Lowell  Asian Project  Against Domestic Violence	ATASK	319 Wilder St.	\N	Lowell	MA	01852	978-452-4522	\N	617-338-2355	978-937-2048	www.atask.org	public_program_email@testing123.net	Collaborative provides efficient services to Cambodians	LAPADV is an unfunded collaborative among several local private, non-profit and state agencies. The mission is to provide adequate resources and efficient services to the Cambodian community in Lowell. Services include legal resources, domestic violence victim advocacy and shelter and rape crisis intervention. Agencies include DSS, Lowell DAs office, Lowell police and many more.	Services available to Cambodian residents of Lowell	Free	\N	Services available year round	Contact Program for Hours	English!DELIM!Khmer	No	Contact program		\N	\N	Yes	No	\N	No	\N	\N	319 Wilder St.	\N	Lowell	MA	01852	Yes	Interested students / volunteers should contact the program for more information.	No	\N	No	\N	No	\N	No	\N	Heather	Tower	978-452-4522	\N	chester@testing123.net	Yes	TOWN	Lowell, MA	\N	\N	\N	No	0	\N
509526	103982	2009-06-05	\N	Save The Whales	\N	319 Wilder Street	\N	Lowell	MA	01852	978-452-4522	666-666-6666	888-888-8888	555-555-5555	www.savethewhales.org	public_program_email@testing123.net	Save Whales	Keep whales safe and healthy	All	Free	\N	Winter only available for seasoned employees.	m-f 8a-7p	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish!DELIM!Khmer!DELIM!Vietnamese	No	Positive ID, Proof of address and eligibilty.		We have a large lot out front of the building.	Must be able to get to the main office. We however provide transportation to any event or outreach work we schedule.	Yes	No	\N	Yes	We only have a certain amount of slots available for participants. Our wait list usually lasts about 6 months to a year.	\N	319 Wilder Street	\N	Lowell	MA	01852	Yes	Please contact Corey Felt at cfelt@savethewhales.org to fill out an application.	No	\N	Yes	Meeting space is able to comfotably sit 25 people comfortably.	Yes	Our volunteer workers are proud of the bi-annual newsletter they create describing all the great work we do!	No	\N	Jennifer	Cloutier	978-452-4522	978-452-6522	chester@testing123.net	Yes	EVERYWHERE		444-444-4444	\N	Please feel free to ask! we have many on staff people can speak multiple language.	Yes	0	\N
\.


--
-- Data for Name: program_category; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY program_category (category_id, program_id) FROM stdin;
802145	501078
801441	501078
802090	501078
803080	501078
801364	501078
800308	501078
803014	501078
802277	501078
801155	501078
802508	502266
802629	502266
802574	502266
802882	502266
802431	502266
804433	502266
802607	502893
802519	502893
802651	502893
802728	502882
802607	502882
802519	502882
802651	502882
802728	502904
802574	502904
802607	502904
802882	502904
802651	502904
802728	502926
802574	502926
802882	502926
802651	502926
801287	503201
804422	503201
802431	503201
804070	503201
801397	503201
800176	501298
800110	501298
802728	503850
802574	503850
802882	503850
802651	503850
802728	502376
802574	502376
802882	502376
802651	502376
803663	501331
803575	501331
800220	501331
800297	501331
800198	501331
800242	501331
801837	501331
800231	501331
802805	503223
803839	503399
804961	503399
804576	503399
804301	503399
804389	503399
803157	503399
804323	503399
803685	503399
803707	503399
800429	503399
803245	503399
803674	503399
800495	503399
801529	503399
801573	503399
800528	503399
800693	503399
801342	503399
803762	503399
803773	503399
803784	503399
804664	503399
800352	503399
801903	503399
804180	503399
802288	502981
801034	502981
802035	500264
802827	500264
802123	500264
802431	500264
804433	500264
803751	500264
802299	500264
800033	502090
800077	502090
800055	502090
804202	502629
804345	502629
804301	502629
804378	502629
804389	502629
804246	502629
804213	502629
804323	502629
804191	502629
804279	502629
804334	502629
804268	502629
804367	502629
804180	502629
804235	502629
800143	501210
800187	501210
800176	501210
801364	501111
801298	501111
801584	501111
801364	501221
801584	501221
801243	501221
801441	501254
801364	501254
801298	501254
801584	501254
801243	501254
801441	501309
801364	501309
801298	501309
801584	501309
801606	501309
802189	501474
802090	501474
802277	501474
801859	501474
802266	501474
800462	501474
801364	501474
801177	501474
801826	501474
801441	501474
804895	501474
802255	501474
802046	501474
800660	501474
800352	501474
801199	501474
804367	501474
803498	501474
800418	501474
802145	501474
801936	500165
800198	500165
803916	500165
804818	500165
802299	500165
801485	500165
802442	504169
802530	504169
802684	504169
802321	504169
802640	504169
802596	504169
800814	501749
800825	501749
800913	501749
800946	501749
801540	501749
800495	501749
800847	501749
802508	501749
804587	503388
804961	503388
804389	503388
803157	503388
804323	503388
803707	503388
800429	503388
803245	503388
803179	503388
804631	503388
800143	503388
801562	503388
803674	503388
800286	503388
801243	503388
801529	503388
800440	503388
800495	503388
804466	503388
801595	503388
800528	503388
800693	503388
801342	503388
803773	503388
804411	503388
804620	503388
803784	503388
803575	503388
801903	503388
802508	503388
801331	503069
801320	503069
801606	503069
803960	503762
800946	503762
802299	503762
802244	503762
800924	501804
800913	501804
803498	501804
801078	501804
800187	502211
800088	502211
800297	502211
800198	502211
800242	502211
800176	502211
800099	502211
800803	504719
803135	504719
803157	504719
800594	504719
800759	504719
800737	504719
803245	504719
800616	504719
803179	504719
800627	504719
803212	504719
801243	504719
800517	504719
800748	504719
800792	504719
800726	504719
800704	504719
803190	504719
804785	505676
800198	505676
801496	505808
801276	505808
800429	505808
800418	505808
802563	504477
802607	504477
802453	504477
803443	504477
802574	504477
804389	504477
800759	504477
804323	504477
803245	504477
802442	504477
804631	504477
802838	504477
802651	504477
802431	504477
802783	504477
802728	504477
802882	504477
800726	504477
802321	504477
804884	504477
802849	504477
804301	503586
802453	503586
802706	503586
804389	503586
803542	503586
803157	503586
800077	503586
800429	503586
801705	503586
802442	503586
802409	503586
803641	503586
804972	503586
800495	503586
800462	503586
801364	503586
800451	503586
802431	503586
804411	503586
804334	503586
802321	503586
802640	503586
804180	503586
802464	503586
804202	503586
804576	503586
803443	503586
804323	503586
803245	503586
804268	503586
800330	503586
801562	503586
802838	503586
802651	503586
804697	503586
804235	503586
802596	503586
804444	503586
802684	503586
802530	503586
804565	503586
802783	503586
802728	503586
804378	503586
804246	503586
804213	503586
804279	503586
802849	503586
804202	502948
804301	502948
804389	502948
804323	502948
804268	502948
804235	502948
800495	502948
800462	502948
802684	502948
802827	502948
802431	502948
804378	502948
804213	502948
804246	502948
800385	502948
804279	502948
804334	502948
802321	502948
802640	502948
804180	502948
802464	502948
803278	505390
803355	505390
804587	501914
804576	501914
800000	501914
804686	501914
800077	501914
800066	501914
804851	501914
804697	501914
800154	501914
804466	501914
804565	501914
804796	501914
804499	501914
800055	501914
804620	501914
804675	501914
804664	501914
804532	501914
804818	501914
804576	505951
800033	505951
800066	505951
801496	505951
801056	505951
804697	505951
800935	505951
801078	505951
800825	505951
804565	505951
804796	505951
800913	505951
804499	505951
801155	505951
804664	505951
801067	505951
800946	501760
801089	501760
800825	501760
800990	501760
801012	501760
801078	501760
800176	501100
804345	505049
804257	505049
804389	505049
804246	505049
804323	505049
804191	505049
804356	505049
804334	505049
804279	505049
804367	505049
804235	505049
800022	506006
800033	506006
800066	506006
803784	505577
804576	505577
800825	505577
804697	505577
800495	505577
804202	502640
804290	502640
804345	502640
804301	502640
804389	502640
804323	502640
802277	502640
804268	502640
804235	502640
804246	502640
804191	502640
804334	502640
804279	502640
802046	502640
804180	502640
804367	502640
802145	502640
804290	502651
804345	502651
804301	502651
804257	502651
804389	502651
804246	502651
804323	502651
804191	502651
804279	502651
804334	502651
804356	502651
804180	502651
804235	502651
804202	502662
804345	502662
804257	502662
804389	502662
804246	502662
804323	502662
804191	502662
804279	502662
804356	502662
804268	502662
804334	502662
804235	502662
804279	502684
804334	502684
804345	502684
804389	502684
804246	502684
804323	502684
804235	502684
804191	502684
804202	502706
804345	502706
804257	502706
804389	502706
804246	502706
804323	502706
804191	502706
804279	502706
804334	502706
803014	502706
804356	502706
804268	502706
800880	502706
804235	502706
800198	504367
803806	504367
802145	504367
801782	503355
803135	503355
803146	503355
801705	503355
803245	503355
801298	503355
801738	503355
801243	503355
803245	504774
803960	504774
803971	504774
803982	504774
803157	504774
803245	503344
803179	503344
803190	503344
803201	503344
803256	503344
803168	503344
803234	503344
803146	503344
802233	506160
802387	506160
802453	506160
800088	506160
802442	506160
802123	506160
802321	506160
802134	506160
803069	506160
804202	504741
802233	504741
800484	504741
800198	504741
804268	504741
802277	504741
801793	504741
803113	504741
802123	504741
802134	504741
800495	504741
802992	504741
803652	504741
800088	504741
800374	504741
802244	504741
804279	504741
804334	504741
801034	504741
800022	504741
802255	504741
802024	504741
801199	504741
804180	504741
801749	504741
800176	504741
803421	505280
800341	505280
803553	505280
803531	505280
803443	505280
800374	505280
803542	505280
801155	505280
803465	505280
803487	505280
803454	505280
804664	500044
804609	500044
804565	500044
804499	500044
804521	500044
800748	506710
803157	506710
800594	506710
800759	506710
800737	506710
800616	506710
803377	506710
803179	506710
801562	506710
803201	506710
803366	506710
803960	501342
803619	501342
803597	501342
800198	501342
801320	506776
804433	506776
801254	506776
802354	506776
802409	506776
804521	506776
801639	506776
801463	506776
801782	506776
801408	506776
802365	506776
804620	506776
801606	506776
804576	506776
802860	506776
801562	506776
804697	506776
801584	506776
802299	506776
804565	506776
803861	506776
801595	506776
801276	506776
801342	506776
801991	506776
801265	506776
801386	506776
804664	506776
804950	506776
801749	506776
804708	506776
802508	506776
804961	506776
804598	506776
801221	506776
804653	506776
804422	506776
801760	506776
803685	506776
801705	506776
803674	506776
801298	506776
801859	506776
804642	506776
801364	506776
801628	506776
801980	506776
803729	506776
802431	506776
804477	506776
803784	506776
804411	506776
801309	506776
801430	506776
801793	506776
801353	506776
803696	506776
804631	506776
801496	506776
804543	506776
801210	506776
804444	506776
804466	506776
801826	506776
800913	506776
801397	506776
801441	506776
801287	506776
804774	506776
801716	506787
802453	506787
801727	506787
801320	506787
801551	506787
804433	506787
801254	506787
802442	506787
802354	506787
804521	506787
801925	506787
801639	506787
801782	506787
801474	506787
804675	506787
801606	506787
804576	506787
800198	506787
804554	506787
802860	506787
801562	506787
804697	506787
801584	506787
800935	506787
802299	506787
804565	506787
803861	506787
802882	506787
801595	506787
800693	506787
801342	506787
801276	506787
801386	506787
804664	506787
804708	506787
804961	506787
804598	506787
801221	506787
804653	506787
804609	506787
804422	506787
803707	506787
801705	506787
801298	506787
804642	506787
801364	506787
801628	506787
803729	506787
801771	506787
804411	506787
802321	506787
804587	506787
801309	506787
801430	506787
802387	506787
801969	506787
804686	506787
801694	506787
801353	506787
803696	506787
804631	506787
801540	506787
804807	506787
801496	506787
804543	506787
801738	506787
801210	506787
804466	506787
801826	506787
800913	506787
802728	506787
801903	506787
804488	506787
804741	506787
801287	506787
804774	506787
801617	506798
804730	506798
804081	506798
801320	506798
801551	506798
804433	506798
802354	506798
801463	506798
801474	506798
804092	506798
804620	506798
804675	506798
801518	506798
801606	506798
803839	506798
804576	506798
801452	506798
804554	506798
804026	506798
802860	506798
804697	506798
801584	506798
801529	506798
804565	506798
803861	506798
801595	506798
800693	506798
801342	506798
801276	506798
801386	506798
804664	506798
804950	506798
804708	506798
801221	506798
804653	506798
804609	506798
801760	506798
803685	506798
803707	506798
803674	506798
801298	506798
804642	506798
804719	506798
801364	506798
801628	506798
803652	506798
803729	506798
803784	506798
804411	506798
801309	506798
801430	506798
801507	506798
801353	506798
803696	506798
804631	506798
801540	506798
801496	506798
801188	506798
804543	506798
801210	506798
804444	506798
804466	506798
800913	506798
801397	506798
803740	506798
801441	506798
804488	506798
804741	506798
801287	506798
804136	506798
804774	506798
800407	505137
802497	505137
802618	505137
800198	505137
800495	505137
800429	505137
801166	505632
803608	506996
802233	506996
801716	506996
802453	506996
803542	506996
802057	506996
800429	506996
801254	506996
802442	506996
802277	506996
803201	506996
804521	506996
801639	506996
801045	506996
804499	506996
801782	506996
804620	506996
803971	506996
802079	506996
801199	506996
800891	506996
801518	506996
803025	506996
804576	506996
804323	506996
801375	506996
803003	506996
802123	506996
801562	506996
800682	506996
804697	506996
801584	506996
801804	506996
804565	506996
804378	506996
803861	506996
800374	506996
804213	506996
800693	506996
803773	506996
804664	506996
804950	506996
801749	506996
802508	506996
803421	506996
804609	506996
804389	506996
803685	506996
803465	506996
801705	506996
802134	506996
803674	506996
803520	506996
803509	506996
801859	506996
803553	506996
803729	506996
802431	506996
803784	506996
802321	506996
803322	506996
802464	506996
804587	506996
802387	506996
803443	506996
803531	506996
803245	506996
801793	506996
801694	506996
801353	506996
803179	506996
804631	506996
801540	506996
804807	506996
801496	506996
801738	506996
803069	506996
804444	506996
801573	506996
801155	506996
803762	506996
801441	506996
803487	506996
802255	506996
803883	506996
804774	506996
801364	501276
804433	501276
803773	501276
801353	501276
803784	501276
804631	501276
804675	501276
803674	501276
801584	501276
801298	501276
801243	501276
804774	501276
802508	501276
803421	504125
801100	504125
801089	504125
802453	504125
803531	504125
803443	504125
801001	504125
803542	504125
802871	504125
803465	504125
802442	504125
800924	504125
800946	504125
800836	504125
803509	504125
803564	504125
803454	504125
800847	504125
801133	504125
803553	504125
804378	504125
800374	504125
804213	504125
801155	504125
803487	504125
803476	504125
800990	504125
803498	504125
801012	504125
801210	507029
801386	507029
801353	507029
803784	507029
801562	507029
802123	507029
804631	507029
804675	507029
803674	507029
801188	507029
801364	503278
800000	503278
801243	503278
801177	507150
800748	507150
800363	507150
800594	507150
800759	507150
800649	507150
800737	507150
800616	507150
800627	507150
800605	507150
800660	507150
803212	507150
801584	507150
800781	507150
801243	507150
800099	507150
803421	504906
804301	504906
802453	504906
804389	504906
803542	504906
803465	504906
800429	504906
802442	504906
803520	504906
804972	504906
800495	504906
802288	504906
803553	504906
801782	504906
802431	504906
804334	504906
802321	504906
801199	504906
804180	504906
802464	504906
804202	504906
803025	504906
803443	504906
803531	504906
804323	504906
803014	504906
804268	504906
802838	504906
803355	504906
802299	504906
804235	504906
802596	504906
804378	504906
804246	504906
804213	504906
803487	504906
804279	504906
803476	504906
802255	504906
802849	504906
800693	501155
800528	501155
803784	501155
803696	501155
803674	501155
804972	501155
800440	501155
800418	501155
800495	501155
803498	501155
802508	501155
800869	505962
804444	505962
800825	505962
800913	505962
803542	505962
801254	505962
803432	505962
800968	505962
801067	505962
800935	505962
801078	505962
800913	505973
804741	505973
801958	500253
801309	500253
801837	500253
803245	503476
803179	503476
803201	503476
801562	503476
800143	503476
803157	503476
803234	503476
803146	503476
803839	503465
801210	503465
801628	503465
803861	503465
801386	503465
803784	503465
803212	503465
803674	503465
801606	503465
802508	503465
803190	507249
803201	507249
803157	507249
803146	507249
804719	501012
804081	501012
804499	501012
804092	501012
804686	501012
804026	501012
804664	501012
804532	501012
804059	501012
800869	505016
804576	505016
801716	505016
801089	505016
801001	505016
801760	505016
800429	505016
802442	505016
801694	505016
801881	505016
801496	505016
800495	505016
800847	505016
803971	505016
802970	505016
803982	505016
800891	505016
801012	505016
802233	500330
802607	500330
803443	500330
802574	500330
800198	500330
804389	500330
804323	500330
800429	500330
803245	500330
802123	500330
802651	500330
802134	500330
803069	500330
802772	500330
800462	500330
802684	500330
802431	500330
802321	500330
802640	500330
803322	500330
802453	500385
802574	500385
803157	500385
802871	500385
804433	500385
802761	500385
802442	500385
802695	500385
802552	500385
802717	500385
802827	500385
802398	500385
802431	500385
802343	500385
804895	500385
802321	500385
805016	500385
802563	500385
802607	500385
802794	500385
803443	500385
803245	500385
802838	500385
803696	500385
802651	500385
802299	500385
802783	500385
802728	500385
802882	500385
803773	500385
804884	500385
802519	500385
802849	500385
804774	500385
802563	500407
802607	500407
802794	500407
802453	500407
803443	500407
804389	500407
803157	500407
802871	500407
800759	500407
804323	500407
800429	500407
802761	500407
802442	500407
803696	500407
802651	500407
802354	500407
802552	500407
802717	500407
802827	500407
802684	500407
802728	500407
802431	500407
803773	500407
800726	500407
804895	500407
802321	500407
804884	500407
802640	500407
802849	500407
802464	500407
803729	505071
800693	505071
803762	505071
803740	505071
803773	505071
803784	505071
803696	505071
803674	505071
802508	505071
800693	504917
803740	504917
803773	504917
803784	504917
800605	504917
800660	504917
800715	504917
803674	504917
800550	504917
800495	504917
804719	500968
804587	500968
804620	500968
804664	500968
804499	500968
801705	501023
804642	501023
801672	501023
804675	501023
804653	501023
804741	501023
804059	501023
804279	502970
804290	502970
802607	502915
802574	502915
802882	502915
802937	502915
800869	506094
801716	506094
801705	506094
801254	506094
801694	506094
801562	506094
800946	506094
801859	506094
800847	506094
801045	506094
800825	506094
800913	506094
801782	506094
801199	506094
801067	506094
801012	506094
802167	501188
803630	501188
802068	501188
802134	501188
800088	501188
800176	501188
802442	504620
802783	504620
802596	504620
800869	504114
800198	504114
803685	504114
801254	504114
800946	504114
801540	504114
801188	504114
800847	504114
801133	504114
801628	504114
800088	504114
800913	504114
803762	504114
803773	504114
801386	504114
803784	504114
801067	504114
800869	507359
801023	507359
803531	507359
801562	507359
800968	507359
803674	507359
800935	507359
803564	507359
801485	507359
801529	507359
801078	507359
801463	507359
800825	507359
800913	507359
801595	507359
803806	507359
801342	507359
801265	507359
803784	507359
801067	507359
802409	507392
802365	507392
803564	507392
800198	502420
803245	502277
803322	502277
803157	502277
804048	500913
802409	500913
804081	500924
804158	500924
804048	500924
804092	500924
802365	500924
804026	500924
802409	500924
804070	500924
804136	500924
804818	500924
804059	500924
804532	500957
804686	500957
800066	503685
802409	501034
802365	501034
804081	503509
800429	503509
801254	503509
803520	503509
801078	503509
800495	503509
803960	503509
801133	503509
804048	503509
800088	503509
804092	503509
803971	503509
803322	503509
801067	503509
800891	503509
800869	503509
800198	503509
801001	503509
801353	503509
803663	503509
800946	503509
801562	503509
803113	503509
804026	503509
804631	503509
801540	503509
800847	503509
804444	503509
800913	503509
803762	503509
803773	503509
801012	503509
800869	502717
801001	502717
801254	502717
800946	502717
804851	502717
801056	502717
800979	502717
801078	502717
801133	502717
803960	502717
804048	502717
800913	502717
801155	502717
800902	502717
801199	502717
801067	502717
801012	502717
803036	507458
803102	507458
802035	504334
800825	504334
802574	504334
800913	504334
802882	504334
804433	504334
802244	504334
803696	504334
802123	504334
802860	504334
800935	504334
802299	504334
801617	502222
801177	502222
801364	502222
801309	502222
800033	502222
801452	502222
800297	502222
800077	502222
800066	502222
800308	502222
800242	502222
804994	502222
804576	502189
801309	502189
801430	502189
800033	502189
800000	502189
800077	502189
803663	502189
800308	502189
800066	502189
804851	502189
804697	502189
800242	502189
801826	502189
800088	502189
804796	502189
800297	502189
800231	502189
804664	502189
803575	502189
800176	502189
803960	501826
801177	501826
801133	501826
801111	501826
800946	501826
801067	501826
801639	501826
801078	501826
801100	502739
801133	502739
801826	502739
801045	502739
801782	502739
800913	502739
800902	502739
801155	502739
800946	502739
800836	502739
801067	502739
801639	502739
801078	502739
805038	506688
802442	506688
802453	506688
801100	506534
801133	506534
801826	506534
801045	506534
800913	506534
801760	506534
800946	506534
800836	506534
801804	506534
801067	506534
801639	506534
801859	506534
801078	506534
800924	506523
801067	506523
801012	506523
801078	506523
803960	504675
803993	504675
803553	504675
803531	504675
801155	504675
803487	504675
803476	504675
803971	504675
803982	504675
803498	504675
803784	502563
801309	502563
801430	502563
803674	502563
801276	502563
803839	507381
804961	507381
804576	507381
804609	507381
800198	507381
800000	507381
801760	507381
801353	507381
801793	507381
802354	507381
802409	507381
804521	507381
804114	507381
801738	507381
804466	507381
804444	507381
804565	507381
801573	507381
803861	507381
804092	507381
801441	507381
802244	507513
804587	507656
804466	507656
804576	507656
801463	507656
804609	507656
804565	507656
801595	507656
801342	507656
804664	507656
804697	507656
804521	507656
803014	507667
802277	507667
802981	507667
802970	507667
803080	507667
800957	507755
800033	507755
800198	507755
800000	507755
802057	507755
802310	507755
802277	507755
800066	507755
801496	507755
801584	507755
801892	507755
800517	507755
800495	507755
801133	507755
800165	507755
800748	507755
801826	507755
800253	507755
800374	507755
802816	507755
802244	507755
802343	507755
800792	507755
802079	507755
800176	507755
801133	502585
800913	502585
801078	502585
800143	503773
804466	500792
801617	500792
804576	500792
801463	500792
801628	500792
804609	500792
804565	500792
803575	500792
803256	506908
803168	506908
803234	506908
803146	506908
803245	506908
803179	506908
803377	506908
803278	506908
803201	506908
803190	506908
803366	506908
804576	504257
801727	504257
801683	504257
801815	504257
801760	504257
801705	504257
801353	504257
801793	504257
804631	504257
801496	504257
804697	504257
801738	504257
801859	504257
801837	504257
804466	504257
801826	504257
801573	504257
801782	504257
801848	504257
801199	504257
800946	507810
801089	507810
802662	507810
800847	507810
801012	507810
802222	500176
803102	500176
802057	500176
804587	504455
804917	504455
801969	504455
802057	504455
801705	504455
801793	504455
804631	504455
803113	504455
801496	504455
801584	504455
802002	504455
801529	504455
804466	504455
804444	504455
801177	504455
801595	504455
801342	504455
804620	504455
802255	504455
801958	504455
802970	504455
803575	504455
801441	503212
801364	503212
801430	503212
801298	503212
801584	503212
801430	504411
804950	504411
800033	500770
800297	500770
800242	500770
800077	500770
802497	501683
802607	501683
802387	501683
802453	501683
802728	501683
802574	501683
802882	501683
802442	501683
802629	501683
802585	501683
802651	501683
804906	501683
802651	502288
802728	502288
802574	502288
802882	502288
803553	502233
802387	502233
802453	502233
803443	502233
804389	502233
800759	502233
804323	502233
800429	502233
800726	502233
802442	502233
802321	502233
803520	502233
800715	502233
802486	501694
802651	501694
802728	501694
802574	501694
802882	501694
802486	502387
802607	502387
802651	502387
802728	502387
802574	502387
802882	502387
803575	506226
804719	507909
804620	507909
804653	507909
804499	507909
804741	507909
801364	507931
804576	507931
801595	507931
801243	507931
804576	507942
801463	507942
801595	507942
801243	507942
803773	507953
801353	507953
803971	507953
804653	507953
804741	507953
801232	507953
804763	507953
804202	507997
804389	507997
804246	507997
804323	507997
804268	507997
804334	507997
804279	507997
804235	507997
804576	508041
804499	508041
804092	508041
802365	508041
802343	508041
804026	508041
804664	508041
805016	508041
802299	508041
804136	508041
804202	500286
804345	500286
802607	500286
803531	500286
803443	500286
803465	500286
804268	500286
804972	500286
804235	500286
800462	500286
800374	500286
804246	500286
800528	500286
804191	500286
803487	500286
804279	500286
804334	500286
804367	500286
800418	500286
803498	500286
804202	500319
802607	500319
804301	500319
803531	500319
803443	500319
804389	500319
804323	500319
803465	500319
800429	500319
804268	500319
804972	500319
804235	500319
800462	500319
800374	500319
804246	500319
800528	500319
803487	500319
804279	500319
804334	500319
804367	500319
804180	500319
800418	500319
803498	500319
800033	504664
800044	504664
800000	504664
800077	504664
800055	504664
800066	504664
800011	504664
800022	504664
803487	508063
802651	508063
802728	508063
802574	508063
802871	508063
801155	508063
803498	508063
802123	507975
800913	507975
802882	507975
801232	507975
802299	507975
802508	507975
803718	507106
803784	507106
803674	507106
800550	507106
803685	507106
800693	507106
802508	507106
804576	508085
804664	508085
800198	508085
800286	508085
802035	508096
802123	508096
804433	508096
804994	503916
802563	500341
803553	500341
803443	500341
802431	500341
800374	500341
802816	500341
803487	500341
803179	500341
802310	500341
802442	500341
802838	500341
804884	500341
802409	500341
805016	500341
802299	500341
803498	500341
802453	500308
802574	500308
802871	500308
802442	500308
802695	500308
802354	500308
802409	500308
804521	500308
804972	500308
802552	500308
802827	500308
804367	500308
803839	500308
804576	500308
804510	500308
802607	500308
800594	500308
804323	500308
802838	500308
804697	500308
802299	500308
804235	500308
804565	500308
800374	500308
804246	500308
802882	500308
804279	500308
804664	500308
800418	500308
804609	500308
804301	500308
800759	500308
804389	500308
803465	500308
802761	500308
802772	500308
802717	500308
800462	500308
803553	500308
802431	500308
800528	500308
802343	500308
804191	500308
804411	500308
804334	500308
802321	500308
804895	500308
802640	500308
804180	500308
805016	500308
802464	500308
802563	500308
804290	500308
804202	500308
802497	500308
802794	500308
803443	500308
803531	500308
804268	500308
804631	500308
802651	500308
802596	500308
803564	500308
804466	500308
804444	500308
802530	500308
802684	500308
802728	500308
802816	500308
803487	500308
802585	500308
804884	500308
804488	500308
802849	500308
803498	500308
804719	501771
804653	501771
801221	501771
804763	501771
804752	501771
803696	501771
804631	501771
801540	501771
803674	501771
804741	501771
804774	501771
801364	501573
801430	501573
801276	501573
802563	500374
802607	500374
802453	500374
802574	500374
802871	500374
804433	500374
802442	500374
802695	500374
802299	500374
804972	500374
802596	500374
800462	500374
802728	500374
802431	500374
802882	500374
800528	500374
802343	500374
804895	500374
802321	500374
804884	500374
802849	500374
802519	500374
805016	500374
800418	500374
804576	507161
801089	507161
801573	507161
800913	507161
804411	507161
800946	507161
801540	507161
804697	507161
801012	507161
800022	501551
800033	501551
800066	501551
800044	501551
800077	501551
803553	505368
803542	505368
800528	505368
800429	505368
803773	505368
800330	505368
800440	505368
800418	505368
803498	505368
800330	501133
803553	501133
800528	501133
803498	501133
800418	501133
800440	501133
800748	501243
800561	501243
800693	501243
800737	501243
803718	501243
800539	501243
803674	501243
800550	501243
803663	501584
801430	501584
801936	501584
804818	501584
803256	506897
803344	506897
803234	506897
803289	506897
803377	506897
803190	506897
803278	506897
803333	506897
803300	506897
803388	506897
800803	500363
802453	500363
802574	500363
803157	500363
802871	500363
800759	500363
800737	500363
800429	500363
802442	500363
802695	500363
800770	500363
800517	500363
800462	500363
803553	500363
800748	500363
802431	500363
800528	500363
802343	500363
800792	500363
800726	500363
804895	500363
802321	500363
800660	500363
805016	500363
802563	500363
802607	500363
802794	500363
800594	500363
803245	500363
803179	500363
800627	500363
802838	500363
802651	500363
802299	500363
802728	500363
800374	500363
801155	500363
802882	500363
802816	500363
804884	500363
802519	500363
803498	500363
800418	500363
802024	508107
802200	508107
802233	504422
800088	504422
802156	504422
803751	504422
802244	504422
802123	504422
802134	504422
803278	508129
803289	508129
804444	508140
804576	508140
803278	508140
803366	508140
803861	508140
803421	505500
803795	505500
800803	505500
804301	505500
803267	505500
802871	505500
800737	505500
803278	505500
800605	505500
803905	505500
800770	505500
801639	505500
804972	505500
801045	505500
800748	505500
801782	505500
800451	505500
800319	505500
804334	505500
800660	505500
804180	505500
803839	505500
801100	505500
804576	505500
804510	505500
803630	505500
800396	505500
800616	505500
800572	505500
800330	505500
800627	505500
800836	505500
803355	505500
804697	505500
801804	505500
804543	505500
804235	505500
803564	505500
801210	505500
803861	505500
804246	505500
801155	505500
803487	505500
804279	505500
800704	505500
800550	505500
803872	505500
800418	505500
803498	505500
802475	508162
802310	508162
801441	502673
801177	502673
801364	502673
803696	502673
801298	502673
801243	502673
801441	503168
804576	503168
803696	503168
801573	503168
804697	503168
804576	506743
804598	506743
804609	506743
804565	506743
804301	506743
804686	506743
804477	506743
801760	506743
800693	506743
804411	506743
804664	506743
804488	506743
804532	506743
804521	506743
804180	506743
804576	506732
804598	506732
804510	506732
804609	506732
804301	506732
804686	506732
801760	506732
804697	506732
804521	506732
804642	506732
804565	506732
803861	506732
804499	506732
800693	506732
804477	506732
804411	506732
804488	506732
804664	506732
804708	506732
803531	506831
800913	506831
800374	506831
801155	506831
800528	506831
803465	506831
803487	506831
803476	506831
800924	506831
800352	506831
800935	506831
800550	506831
803498	506831
800440	506831
800418	506831
803531	506820
802871	506820
801155	506820
803465	506820
803487	506820
803476	506820
800352	506820
803509	506820
800550	506820
800440	506820
800418	506820
800913	503740
803531	502057
803443	502057
803465	502057
800946	502057
803509	502057
804972	502057
803564	502057
801078	502057
800440	502057
800374	502057
800913	502057
801155	502057
800528	502057
803487	502057
803476	502057
800352	502057
800550	502057
800418	502057
803498	502057
800605	503751
800913	503751
800550	503751
801364	504928
801309	504928
801430	504928
803256	504928
803234	504928
801276	504928
803245	504928
801353	504928
803179	504928
803201	504928
801562	504928
804950	504928
803212	504928
801243	504928
804433	501980
803685	501980
802442	501980
803696	501980
800946	501980
802860	501980
803674	501980
803520	501980
801056	501980
800440	501980
801078	501980
800847	501980
800913	501980
802882	501980
800528	501980
803773	501980
803784	501980
802508	501980
802453	503718
803443	503718
803685	503718
800429	503718
802442	503718
803696	503718
802860	503718
802354	503718
803674	503718
803564	503718
803553	503718
803729	503718
802431	503718
800693	503718
803751	503718
803762	503718
803773	503718
803784	503718
802508	503718
804774	503718
800869	507271
801089	507271
800957	507271
801001	507271
800924	507271
800946	507271
803509	507271
800682	507271
800847	507271
801078	507271
801133	507271
801045	507271
801573	507271
800825	507271
801408	507271
800374	507271
801155	507271
801397	507271
801199	507271
800880	507271
800990	507271
803498	507271
801012	507271
804510	504851
801309	504851
803685	504851
801375	504851
800429	504851
801254	504851
801353	504851
801562	504851
801540	504851
803674	504851
801496	504851
800935	504851
801485	504851
801529	504851
804642	504851
801573	504851
800825	504851
804499	504851
800913	504851
801595	504851
803762	504851
801342	504851
803773	504851
801265	504851
803784	504851
801199	504851
801067	504851
801518	504851
804026	503003
800946	503003
801166	503003
804081	503003
804048	503003
804092	503003
804136	503003
804642	500187
803729	500187
802398	500187
803685	500187
802365	500187
803773	500187
801353	500187
803784	500187
803696	500187
803674	500187
804774	500187
800946	503014
801166	503014
800825	503014
800935	503014
801078	503014
804961	507282
801826	507282
803168	507282
803146	507282
801837	507282
802926	507282
800946	502761
800902	502761
802508	502761
803487	504466
802442	504466
802387	504466
802453	504466
803443	504466
802431	504466
803146	504466
803498	504466
802563	507293
804895	507293
802871	507293
804917	507304
802794	507304
802431	507304
802871	507304
801034	507304
803190	507304
802695	507304
805016	507304
801221	503872
800693	503872
803707	503872
804653	505291
803553	501496
800374	501496
801386	503058
801320	503058
803487	508239
800341	508239
803443	508239
803454	508239
803553	508250
800374	508250
803553	508261
801155	508261
803553	508272
801309	508272
801419	508272
801155	508272
803476	508283
801155	508283
803498	508283
801837	508294
803498	508294
800473	508305
800528	508305
804664	508316
804609	508316
804499	508316
804521	508316
803696	508327
803674	508327
803762	508327
804279	508338
802827	508338
802321	508338
804301	508338
804389	508338
804587	508349
804664	508349
804565	508349
801309	508360
803674	508360
803784	508371
803498	508371
804741	508382
801232	508382
804741	508393
801232	508393
804675	508404
804741	508404
801232	508404
802222	508415
802222	508426
802222	508437
804917	508437
802222	508448
802882	508492
802937	508492
804895	508503
802794	508503
802453	508503
802871	508503
804345	508514
804191	508514
803498	508514
800066	508547
804664	508547
804851	508547
804796	508547
800066	508558
804851	508558
804796	508558
800077	508569
800055	508569
804917	508580
803102	508580
803311	508591
803377	508591
803278	508591
803322	508591
803245	508602
803190	508602
803322	508602
803212	508602
803135	508602
803157	508602
803234	508602
801771	508613
801177	508613
801826	508613
801782	508613
804587	508624
804466	508624
804576	508624
804598	508624
804565	508624
804686	508624
804807	508624
804664	508624
804961	508635
801958	508635
801980	508635
801925	508635
802970	508646
802926	508646
803025	508657
802970	508657
802926	508657
803025	508668
802959	508668
803047	508668
803113	508668
801364	508679
801243	508679
801441	508690
801177	508701
801452	508701
804642	505082
804675	505082
804741	505082
804433	505082
804763	505082
804774	505082
804587	505511
804466	505511
803619	505511
801573	505511
800198	505511
800253	505511
800000	505511
800077	505511
804411	505511
804620	505511
804037	505511
800066	505511
801496	505511
803586	505511
804587	503663
803619	503663
800000	503663
803586	503663
803619	503696
804048	503696
804092	503696
804818	503696
803586	503696
804587	503707
804037	503707
803619	503707
803586	503707
803619	503652
803619	503674
800000	503674
800077	503674
803586	503674
800055	503674
804587	508745
804466	508745
803619	508745
804565	508745
800000	508745
800077	508745
804620	508745
800066	508745
800286	508745
801353	508756
801364	508756
801540	508756
801243	508756
801045	508712
801848	508712
801914	508712
801815	508712
801760	508712
801859	508712
801133	508767
801034	508767
801045	508767
800847	508767
801705	508778
801782	508778
801793	508789
801177	508789
801364	508789
801826	508789
801243	508789
801815	508800
801683	508811
801738	508833
801859	508833
801826	508844
801859	508844
801364	508855
804697	508855
801760	508855
801782	508899
801859	508899
804576	508910
804697	508910
801760	508910
804719	503366
803696	503366
804752	503366
804653	503366
801221	503366
801408	503366
804741	503366
804763	503366
804774	503366
804719	503377
801826	503377
804653	503377
801221	503377
801408	503377
804763	503377
804752	503377
803696	503377
804741	503377
804774	503377
802035	508481
801826	508921
801804	508921
801859	508921
801826	508932
801782	508932
801804	508932
801639	508932
801859	508932
801826	508943
802178	508943
801639	508943
801859	508943
801705	508954
801782	508954
802178	508954
801782	508866
802178	508866
801859	508866
801826	508888
801782	508888
802178	508888
801859	508888
802948	508976
802805	508987
802244	508998
802277	509009
802244	509009
804741	508965
801232	508965
803531	502145
800473	502145
804246	502145
801155	502145
803685	502145
800528	502145
803487	502145
804279	502145
804334	502145
800330	502145
800352	502145
804235	502145
803498	502145
800440	502145
800418	502145
803542	509020
803487	509031
803443	509031
803498	509031
800418	509031
803663	507139
804961	507139
801980	507139
801881	507139
801914	507139
801947	507139
800176	507139
802233	507183
802453	507183
801683	507183
802156	507183
804389	507183
800759	507183
800671	507183
800429	507183
802442	507183
802134	507183
803520	507183
800715	507183
800099	507183
800495	507183
800088	507183
804499	507183
803751	507183
800726	507183
803971	507183
802321	507183
802024	507183
803322	507183
804576	507183
802387	507183
800198	507183
800363	507183
803443	507183
804323	507183
803245	507183
801353	507183
801793	507183
804631	507183
802123	507183
802860	507183
801540	507183
800286	507183
801738	507183
803069	507183
804444	507183
804565	507183
802244	507183
802200	507183
803773	507183
804664	507183
803883	507183
802178	507183
804774	507183
804444	507194
804576	507194
804609	507194
804565	507194
802475	507194
802860	507194
804631	507194
804521	507194
801309	503828
800132	503828
800088	503828
802431	503828
800121	503828
800110	503828
802475	503828
802310	503828
802442	503828
802519	503828
800176	503828
802750	501650
802123	504202
802244	504202
802123	504213
804961	503630
803597	503630
801309	503630
803575	503630
803366	503630
803025	500231
802233	500231
802959	500231
801969	500231
803003	500231
802112	500231
802123	500231
802134	500231
803058	500231
802904	500231
802926	500231
803069	500231
802167	500231
802222	500231
803047	500231
803036	500231
802255	500231
802970	500231
802233	502013
802277	502013
802255	502013
802123	502013
802134	502013
802046	502013
802002	502013
803069	502013
802145	502013
801441	500275
804411	500275
801826	500275
804697	500275
801760	500275
803927	501837
804917	501837
802453	501837
802871	501837
801815	501837
800429	501837
802277	501837
802981	501837
803916	501837
803949	501837
803960	501837
800088	501837
801936	501837
802431	501837
800902	501837
803234	501837
803971	501837
803366	501837
802662	501837
800660	501837
803938	501837
803322	501837
800869	501837
801144	501837
802563	501837
803630	501837
801969	501837
802794	501837
803443	501837
800198	501837
800407	501837
803014	501837
802123	501837
802651	501837
800682	501837
802618	501837
802299	501837
803069	501837
802266	501837
802167	501837
802222	501837
803993	501837
802992	501837
803036	501837
800649	501837
804004	501837
803377	501837
801034	501837
802629	501837
803982	501837
802970	501837
803498	501837
803960	501848
800748	501848
800649	501848
800737	501848
804004	501848
802277	501848
803971	501848
802255	501848
800660	501848
803322	501848
800682	501848
803949	501848
803388	501848
802145	501848
803960	501859
803993	501859
803971	501859
803322	501859
800429	501859
802167	501870
803960	501870
803971	501870
803949	501870
802167	501881
803960	501881
803025	501881
803993	501881
804917	501881
802453	501881
803036	501881
800429	501881
802442	501881
803971	501881
802970	501881
803949	501881
800495	501881
803927	501892
800814	501892
801144	501892
804917	501892
802453	501892
802376	501892
801815	501892
803146	501892
803311	501892
802981	501892
802123	501892
802134	501892
800682	501892
802266	501892
802167	501892
803960	501892
802222	501892
801111	501892
803971	501892
801034	501892
802629	501892
802662	501892
803938	501892
803322	501892
803927	502849
801144	502849
803619	502849
804917	502849
800132	502849
802431	502849
800121	502849
802442	502849
802695	502849
802519	502849
803916	502849
804818	502849
803586	502849
802937	502849
803025	500242
802959	500242
803080	500242
802948	500242
803003	500242
802277	500242
803014	500242
803113	500242
802981	500242
803058	500242
802904	500242
802926	500242
803069	500242
802222	500242
803047	500242
803036	500242
802970	500242
800891	500242
803102	500242
802937	500242
802959	500209
800891	500209
803102	500209
801012	500209
802277	500220
803080	500220
801584	500220
802002	500220
802145	500220
803531	509042
800374	509042
801155	509042
800528	509042
803465	509042
803487	509042
803476	509042
803674	509042
803498	509042
800440	509042
800418	509042
803564	509042
803597	501232
800176	501232
803025	502497
802959	502497
801430	502497
801980	502497
801969	502497
801562	502497
801958	502497
801925	502497
802904	502497
802926	502497
801309	504521
801309	504532
801562	504532
803663	503927
801353	503927
801364	503927
803971	503927
800088	503927
803047	503421
802970	503421
802926	503421
803421	506039
804576	506039
804389	506039
802871	506039
804323	506039
803685	506039
800429	506039
800330	506039
800066	506039
803674	506039
804697	506039
804972	506039
804235	506039
800440	506039
802772	506039
802684	506039
804565	506039
804378	506039
804246	506039
804213	506039
800528	506039
804279	506039
804334	506039
803718	506039
803784	506039
802321	506039
804895	506039
804664	506039
804224	506039
802464	506039
800418	506039
801364	502431
800220	502431
801298	502431
801243	502431
803652	502112
800154	502112
803663	502453
800033	502453
800220	502453
800055	502453
800220	502442
800264	502442
800033	502464
800000	502464
800077	502464
800055	502464
804576	502475
800066	502475
804664	502475
800220	502475
804499	502475
804697	502475
804433	502475
804785	502123
800198	502123
800594	502123
800121	502123
800077	502123
800572	502123
800616	502123
800605	502123
800209	502123
800539	502123
800682	502123
800154	502123
801243	502123
803652	502123
800253	502123
800110	502123
800704	502123
802046	502123
803421	500748
800396	500748
803531	500748
803465	500748
800737	500748
800616	500748
800330	500748
800627	500748
800605	500748
803674	500748
800770	500748
804235	500748
803564	500748
801078	500748
800440	500748
803553	500748
800473	500748
800451	500748
804246	500748
801155	500748
800528	500748
800319	500748
800649	500748
803487	500748
803476	500748
800704	500748
800352	500748
800550	500748
803498	500748
800418	500748
800407	503025
803531	507348
803443	507348
801001	507348
801375	507348
800429	507348
801793	507348
801353	507348
800330	507348
801540	507348
801496	507348
800935	507348
800847	507348
800440	507348
801133	507348
803553	507348
801463	507348
801628	507348
801573	507348
800825	507348
800913	507348
801595	507348
800528	507348
801342	507348
803487	507348
801265	507348
801386	507348
804950	507348
801199	507348
801067	507348
801518	507348
802651	509053
802728	509053
802574	509053
802882	509053
801826	509097
801859	509097
802090	509108
802046	509108
802002	509108
802145	509108
802024	509119
803421	502101
800396	502101
803531	502101
800616	502101
800330	502101
800627	502101
800605	502101
804972	502101
800440	502101
800374	502101
800473	502101
801155	502101
800528	502101
800649	502101
803487	502101
803476	502101
800352	502101
800550	502101
803498	502101
800418	502101
803960	501177
803630	501177
802794	501177
802453	501177
802871	501177
802123	501177
803971	501177
802695	501177
803938	501177
802409	501177
803916	501177
804818	501177
801177	503256
801364	503256
801826	503256
801628	503256
801309	503256
801045	503256
801727	503256
801683	503256
801782	503256
801760	503256
801705	503256
801848	503256
801639	503256
801738	503256
801859	503256
801837	503256
801606	503256
804158	504807
804048	504807
800913	504807
804114	504807
801067	504807
801078	504807
803421	505533
803531	505533
803465	505533
800737	505533
800330	505533
800605	505533
804235	505533
803564	505533
800440	505533
804246	505533
801155	505533
800319	505533
803487	505533
803476	505533
800660	505533
800352	505533
800418	505533
803498	505533
803421	505522
803531	505522
804246	505522
801155	505522
800319	505522
803465	505522
803487	505522
803476	505522
800352	505522
800660	505522
803674	505522
804235	505522
803498	505522
800418	505522
803564	505522
803421	505544
803531	505544
804389	505544
803542	505544
804323	505544
800671	505544
803465	505544
800737	505544
800330	505544
800605	505544
803674	505544
803564	505544
800440	505544
801155	505544
800528	505544
800319	505544
800693	505544
803487	505544
803476	505544
800352	505544
800660	505544
802508	505544
803498	505544
800418	505544
803421	505346
803531	505346
804389	505346
803542	505346
804323	505346
800671	505346
803465	505346
800330	505346
800605	505346
804235	505346
803564	505346
800440	505346
803861	505346
804246	505346
801155	505346
800319	505346
800693	505346
803487	505346
803476	505346
800352	505346
800660	505346
803498	505346
800418	505346
804587	505335
804576	505335
804081	505335
800198	505335
800077	505335
800066	505335
801496	505335
804697	505335
801298	505335
801529	505335
801210	505335
804466	505335
804565	505335
804499	505335
801595	505335
804092	505335
801342	505335
804620	505335
804664	505335
804070	505335
804587	507436
804466	507436
804620	507436
802409	507436
804961	503124
801958	503124
801980	503124
801969	503124
801947	503124
801925	503124
801892	503124
804961	503135
801958	503135
801980	503135
801969	503135
801947	503135
801925	503135
803025	503146
804961	503146
801958	503146
801980	503146
801969	503146
801925	503146
802926	503146
804961	503157
801958	503157
801980	503157
801969	503157
801925	503157
802453	505269
801573	505269
800759	505269
803751	505269
803685	505269
803806	505269
800429	505269
803773	505269
803014	505269
800726	505269
800352	505269
802354	505269
803674	505269
801199	505269
802552	505269
804730	502992
804653	502992
801221	502992
803157	502992
804433	502992
803245	502992
803179	502992
803696	502992
801562	502992
802123	502992
802299	502992
804719	502992
800088	502992
803652	502992
800297	502992
804763	502992
801397	502992
803762	502992
804752	502992
804741	502992
801287	502992
802508	502992
804774	502992
800033	501628
800077	501628
800055	501628
802563	503102
801144	503102
800033	503102
804048	503102
801903	503102
802695	503102
802409	503102
803916	503102
802563	504488
802607	504488
802453	504488
803245	504488
802442	504488
802838	504488
804895	504488
804884	504488
800462	506644
803443	506644
800374	506644
800429	506644
800330	506644
804972	506644
800418	506644
803498	506644
802233	501529
800803	501529
800363	501529
804389	501529
804323	501529
800737	501529
800572	501529
800616	501529
802123	501529
802134	501529
800770	501529
800517	501529
800748	501529
800451	501529
800693	501529
800792	501529
800726	501529
800704	501529
800352	501529
800638	501529
800814	504400
804345	504400
801089	504400
802574	504400
803542	504400
800924	504400
803432	504400
803509	504400
803520	504400
801859	504400
801078	504400
803553	504400
801045	504400
800825	504400
800902	504400
804191	504400
802662	504400
801199	504400
801067	504400
800990	504400
801100	504400
800957	504400
803443	504400
801001	504400
800946	504400
800836	504400
802651	504400
800935	504400
800847	504400
800913	504400
800374	504400
802728	504400
801155	504400
802882	504400
801397	504400
803476	504400
802629	504400
801287	504400
803498	504400
801012	504400
804840	504378
804785	504378
800022	504378
800033	504378
800066	504378
804807	504378
800044	504378
802497	501661
802486	501661
802794	501661
802574	501661
802442	501661
800946	501661
802695	501661
802651	501661
803674	501661
802618	501661
802596	501661
800462	501661
802783	501661
802728	501661
802882	501661
802585	501661
802629	501661
805016	501661
802508	501661
800462	501639
802497	501639
800396	501639
802728	501639
802574	501639
802882	501639
800407	501639
802629	501639
802651	501639
802618	501639
805016	501639
803960	504389
802607	504389
802695	504389
800033	503597
801936	503597
800077	503597
804983	503597
803663	503597
803971	503597
800011	503597
800022	503597
804818	503597
800495	503597
800407	506072
800407	500451
802959	508525
802893	508525
800121	508525
800176	508525
803102	508525
802937	508525
801826	508536
801309	508536
802937	508536
801276	508536
802244	509130
802431	509141
800198	509141
802244	509141
800946	506545
800825	506545
801001	506545
800913	506545
801067	506545
801078	506545
800803	505027
800759	505027
800671	505027
800583	505027
800737	505027
800429	505027
800605	505027
803905	505027
800715	505027
800770	505027
804972	505027
800517	505027
800495	505027
800748	505027
800451	505027
800528	505027
800792	505027
800726	505027
800660	505027
800781	505027
800396	505027
800594	505027
800616	505027
800572	505027
800407	505027
800627	505027
800440	505027
800374	505027
800649	505027
804004	505027
800704	505027
800352	505027
800550	505027
800418	505027
800110	503817
800165	503817
800121	503817
803586	503817
800924	501793
803509	501793
801155	501793
803498	501793
800418	501793
803421	501485
800396	501485
803531	501485
803465	501485
800429	501485
800330	501485
804235	501485
800440	501485
803564	501485
800473	501485
800451	501485
804246	501485
801155	501485
800528	501485
803487	501485
803476	501485
800352	501485
800418	501485
803498	501485
800803	509163
804301	509163
804389	509163
800759	509163
800671	509163
800737	509163
800429	509163
800605	509163
800715	509163
800770	509163
804972	509163
800517	509163
803960	509163
800462	509163
800451	509163
800528	509163
800792	509163
804334	509163
803784	509163
800726	509163
800660	509163
804180	509163
800594	509163
804323	509163
800616	509163
800572	509163
800330	509163
803696	509163
802860	509163
800627	509163
800682	509163
804235	509163
800440	509163
800473	509163
800374	509163
800913	509163
804246	509163
800693	509163
800649	509163
804279	509163
804004	509163
800704	509163
800352	509163
800550	509163
800418	509163
800869	509174
804587	509174
801023	509174
801320	509174
801507	509174
800429	509174
801254	509174
803663	509174
802860	509174
801562	509174
800968	509174
801188	509174
801485	509174
800440	509174
804466	509174
804444	509174
801133	509174
801463	509174
801628	509174
800913	509174
801474	509174
800528	509174
801265	509174
804620	509174
801386	509174
801067	509174
804642	502772
803784	502772
800088	502772
800286	502772
800077	502772
800869	501287
803630	501287
802871	501287
800594	501287
800759	501287
802057	501287
801254	501287
800616	501287
803278	501287
803113	501287
801562	501287
803333	501287
803388	501287
803960	501287
800748	501287
803289	501287
800649	501287
804004	501287
803377	501287
801034	501287
802255	501287
803982	501287
803366	501287
803410	501287
803322	501287
802079	501287
802178	501287
803806	502937
800429	502937
804587	507480
804466	507480
801386	507480
804576	507480
801628	507480
804576	500055
804598	500055
804609	500055
804565	500055
804499	500055
804686	500055
801760	500055
804664	500055
804532	500055
801529	500055
803608	500132
801617	500132
804598	500132
804609	500132
804081	500132
801760	500132
802409	500132
804499	500132
802431	500132
804620	500132
804532	500132
803839	500132
804587	500132
804576	500132
801452	500132
800198	500132
804686	500132
801793	500132
804026	500132
804631	500132
801496	500132
804697	500132
802299	500132
801529	500132
803586	500132
804466	500132
803619	500132
801573	500132
804565	500132
800253	500132
801595	500132
801342	500132
804664	500132
804664	500088
804499	500088
800440	500088
803025	504708
802233	504708
804961	504708
802959	504708
800198	504708
802893	504708
803003	504708
803663	504708
802981	504708
803113	504708
802134	504708
800286	504708
801925	504708
802926	504708
803069	504708
803047	504708
801980	504708
803036	504708
801958	504708
803575	504708
802970	504708
803102	504708
803025	505258
804961	505258
802959	505258
801969	505258
800363	505258
800198	505258
801947	505258
800429	505258
803311	505258
803113	505258
801925	505258
801892	505258
802926	505258
803069	505258
800099	505258
803047	505258
800088	505258
801980	505258
803399	505258
801914	505258
803036	505258
802915	505258
801958	505258
803575	505258
802970	505258
803322	505258
801199	505258
803663	506721
803619	506721
800198	506721
803586	506721
800033	509196
800044	509196
800000	509196
800077	509196
800066	509196
800011	509196
800022	509196
803575	509196
800242	509196
800946	501397
801122	501397
801023	501397
801111	501397
800847	501397
800979	501397
800946	504070
801111	504070
801056	504070
800979	504070
800946	501408
801023	501408
801111	501408
800297	501408
800847	501408
800979	501408
800231	501408
801133	505302
804576	505302
800946	505302
801045	505302
804565	505302
804697	505302
800847	505302
800979	505302
804576	501419
800946	501419
804664	501419
804565	501419
804499	501419
804697	501419
800847	501419
800979	501419
803619	507491
803575	507491
800198	507491
803586	507491
800066	505665
804675	505665
801298	505665
801364	505654
800066	505654
801298	505654
800407	503036
801430	503960
802794	503960
803443	503960
800198	503960
803157	503960
802871	503960
803146	503960
803245	503960
803179	503960
801353	503960
803201	503960
800143	503960
801562	503960
803212	503960
801584	503960
801298	503960
801243	503960
803960	503960
801364	503960
803234	503960
803190	503960
801177	503949
803256	503949
803157	503949
803234	503949
803146	503949
803245	503949
803190	503949
800143	503949
803201	503949
803212	503949
801298	503949
800286	503949
801243	503949
804642	504818
804675	504818
802508	504818
804774	504818
804642	503993
804675	503993
804774	503993
801826	501089
803630	501089
801727	501089
801683	501089
801782	501089
801705	501089
801793	501089
801859	501089
801639	501089
803619	501144
800132	501144
800165	501144
802431	501144
800110	501144
802310	501144
802442	501144
802475	501144
802519	501144
800176	501144
803586	501144
802937	501144
802596	501144
800495	507843
800429	507843
800869	509207
802233	509207
801089	509207
802794	509207
802453	509207
803443	509207
800198	509207
802871	509207
800429	509207
801793	509207
801694	509207
802981	509207
800946	509207
800066	509207
801540	509207
803069	509207
800847	509207
804466	509207
803960	509207
801133	509207
801045	509207
800374	509207
801155	509207
800902	509207
803971	509207
801034	509207
802662	509207
800352	509207
800891	509207
800418	509207
801012	509207
801089	509218
800902	509218
800429	509218
802981	509218
800891	509218
801012	509218
800847	509218
803069	509218
800869	509229
801089	509229
802453	509229
803443	509229
801001	509229
803542	509229
800044	509229
800429	509229
802442	509229
801353	509229
802981	509229
800946	509229
800066	509229
801540	509229
803069	509229
800847	509229
804466	509229
800902	509229
801034	509229
801012	509229
804642	501540
803784	501540
804433	501540
804136	501540
803685	501540
803256	505940
800759	505940
800583	505940
800737	505940
800429	505940
802277	505940
803278	505940
803201	505940
802134	505940
800539	505940
804972	505940
800495	505940
800748	505940
803289	505940
803322	505940
800781	505940
802145	505940
802189	505940
803443	505940
800363	505940
803080	505940
800594	505940
800616	505940
803245	505940
801353	505940
803014	505940
803179	505940
800627	505940
802002	505940
803718	505940
803377	505940
802255	505940
803883	505940
802046	505940
802970	505940
800880	505940
803300	505940
802189	505929
803256	505929
803080	505929
800594	505929
800759	505929
800583	505929
800737	505929
800616	505929
803179	505929
803014	505929
802277	505929
803278	505929
803201	505929
800627	505929
802134	505929
800539	505929
802002	505929
800748	505929
803718	505929
803377	505929
802255	505929
803883	505929
800660	505929
803322	505929
800781	505929
802145	505929
802189	509240
800748	509240
803080	509240
800759	509240
800737	509240
802277	509240
802255	509240
803366	509240
802046	509240
802134	509240
800539	509240
800781	509240
802002	509240
802145	509240
802189	509251
803256	509251
800363	509251
800594	509251
800759	509251
800737	509251
800429	509251
800616	509251
803179	509251
802277	509251
803201	509251
802134	509251
800539	509251
802002	509251
800495	509251
800748	509251
802915	509251
803718	509251
802255	509251
800781	509251
802145	509251
802189	509262
803256	509262
803080	509262
800363	509262
800594	509262
800759	509262
800583	509262
800737	509262
800429	509262
800616	509262
803179	509262
803014	509262
802277	509262
803201	509262
802134	509262
800539	509262
802002	509262
800495	509262
803124	509262
800748	509262
803718	509262
803377	509262
802255	509262
802046	509262
800781	509262
802145	509262
802233	507524
802453	507524
803542	507524
800429	507524
802442	507524
802277	507524
803641	507524
804972	507524
800495	507524
800319	507524
800726	507524
802145	507524
803080	507524
804323	507524
802310	507524
800330	507524
802123	507524
804235	507524
804378	507524
800473	507524
800374	507524
804246	507524
804213	507524
804279	507524
800704	507524
804224	507524
800550	507524
800418	507524
803421	507524
804345	507524
800803	507524
804389	507524
800759	507524
800737	507524
803465	507524
802090	507524
800605	507524
802134	507524
800770	507524
800715	507524
803520	507524
800517	507524
801078	507524
800748	507524
803553	507524
800451	507524
800528	507524
804191	507524
804334	507524
802321	507524
800660	507524
802464	507524
804202	507524
800396	507524
803443	507524
803531	507524
800572	507524
800616	507524
804268	507524
803014	507524
800627	507524
800440	507524
803069	507524
803564	507524
804444	507524
801155	507524
801397	507524
800649	507524
803487	507524
802475	507524
803476	507524
802255	507524
800352	507524
802046	507524
803498	507524
803839	500033
804587	500033
804466	500033
804576	500033
800198	500033
804092	500033
804477	500033
800330	500033
804026	500033
804664	500033
802354	500033
804488	500033
804576	506754
801463	506754
804609	506754
802233	500066
802453	500066
803157	500066
802871	500066
803773	500066
802123	500066
803883	500066
803575	500066
802079	500066
800286	500066
803553	500077
803729	500077
803652	500077
803443	500077
803542	500077
800693	500077
800528	500077
803762	500077
803784	500077
803696	500077
803674	500077
802508	500077
803564	500077
804444	504290
804576	504290
804664	504290
804565	504290
804499	504290
804015	504873
804081	504873
804499	504873
804433	504873
804136	504873
804411	500946
804576	500946
804664	500946
805027	503795
804576	503795
804026	503795
804664	503795
804565	503795
804081	503795
804499	503795
804697	503795
804444	504884
804576	504884
800033	504884
804565	504884
804499	504884
800011	504884
800022	504884
804664	504884
804576	505357
800033	505357
804664	505357
800066	501001
804081	501001
804499	501001
804103	501001
804026	501045
804664	501045
804081	501045
804499	501045
804114	501045
804026	501056
804664	501056
804081	501056
804499	501056
802321	501947
802640	501947
802431	501947
802464	501947
803245	501958
803025	501958
801782	501958
803135	501958
803234	501958
803025	509273
800946	501386
801122	501386
800979	501386
801133	505599
800968	505599
800825	505599
800935	505599
801067	505599
801078	505599
804444	508723
803696	508723
801309	508723
801430	508723
803674	508723
802233	500121
802453	500121
804389	500121
800759	500121
803685	500121
800429	500121
802354	500121
802134	500121
803674	500121
800715	500121
800495	500121
802827	500121
800748	500121
803729	500121
802431	500121
803751	500121
803784	500121
800726	500121
802321	500121
802079	500121
801969	500121
800594	500121
804323	500121
801353	500121
801793	500121
802310	500121
803696	500121
800143	500121
801562	500121
802123	500121
801738	500121
802299	500121
802596	500121
804565	500121
801573	500121
800693	500121
803762	500121
803773	500121
802255	500121
802849	500121
802508	500121
804774	500121
801210	506061
804444	506061
801364	506061
804576	506061
801463	506061
801331	506061
801628	506061
801573	506061
801320	506061
800198	506061
800330	506061
803696	506061
801496	506061
804697	506061
801606	506061
803421	505324
802706	505324
802574	505324
803542	505324
802871	505324
803465	505324
802761	505324
802442	505324
802695	505324
802354	505324
803520	505324
802552	505324
802717	505324
802772	505324
803553	505324
802827	505324
802321	505324
802662	505324
805005	505324
802640	505324
802805	505324
802563	505324
802387	505324
802486	505324
803531	505324
803443	505324
803696	505324
802860	505324
802651	505324
803564	505324
802684	505324
802728	505324
802882	505324
803487	505324
803476	505324
805038	505324
802585	505324
802629	505324
804884	505324
802849	505324
802739	505324
803839	500836
801617	500836
804576	500836
804301	500836
802277	500836
800330	500836
803696	500836
800066	500836
801562	500836
802354	500836
804807	500836
802409	500836
801496	500836
804114	500836
804466	500836
801463	500836
801826	500836
804565	500836
804796	500836
804092	500836
801595	500836
801441	500836
804411	500836
804664	500836
804180	500836
801606	500836
803443	506105
800198	506105
800759	506105
802442	506105
800330	506105
804631	506105
803696	506105
800066	506105
802354	506105
804807	506105
802409	506105
800715	506105
804114	506105
804543	506105
802299	506105
802596	506105
802717	506105
804466	506105
804444	506105
802827	506105
800748	506105
804565	506105
801573	506105
804796	506105
802431	506105
802365	506105
800726	506105
802321	506105
802464	506105
804961	507634
804576	507634
801309	507634
804609	507634
800198	507634
801760	507634
801793	507634
801562	507634
802409	507634
801298	507634
802299	507634
800286	507634
803586	507634
804466	507634
803619	507634
800165	507634
801628	507634
801826	507634
804565	507634
801573	507634
800253	507634
802431	507634
804411	507634
803575	507634
800176	507634
804961	507645
804576	507645
803619	507645
801826	507645
801628	507645
800165	507645
801573	507645
800198	507645
800253	507645
801760	507645
804697	507645
800176	507645
803586	507645
804576	501166
804565	501166
804081	501166
804158	501166
804092	501166
801760	501166
804037	501166
804026	501166
804664	501166
800176	501166
804059	501166
803729	509295
804763	509295
803751	509295
803685	509295
800693	509295
803773	509295
803784	509295
804752	509295
803696	509295
802508	509295
804609	507700
802453	507700
803443	507700
804433	507700
803685	507700
803245	507700
801353	507700
803696	507700
802860	507700
801540	507700
802354	507700
803674	507700
801496	507700
802299	507700
801738	507700
803069	507700
800495	507700
804565	507700
803729	507700
801573	507700
802398	507700
802431	507700
800693	507700
803773	507700
803784	507700
804774	507700
804444	506050
803619	506050
801826	506050
804565	506050
801573	506050
800198	506050
800253	506050
801760	506050
802123	506050
801496	506050
802299	506050
801617	506655
801309	506655
804565	506655
801573	506655
801452	506655
801760	506655
801276	506655
800308	506655
804994	506655
804587	507557
804466	507557
804598	507557
801463	507557
800033	507557
804565	507557
804796	507557
804686	507557
801265	507557
804620	507557
800066	507557
804851	507557
804532	507557
801485	507557
804048	507546
802409	507546
804026	507535
802409	507535
804092	507535
800198	507689
800253	507689
800286	507689
803421	509086
804301	509086
802706	509086
804389	509086
803542	509086
803465	509086
803674	509086
802409	509086
801364	509086
803553	509086
802827	509086
802431	509086
802365	509086
802343	509086
804334	509086
803784	509086
805005	509086
801199	509086
804180	509086
804202	509086
801430	509086
801309	509086
803531	509086
804323	509086
805027	509086
802310	509086
804268	509086
801562	509086
802860	509086
802299	509086
802596	509086
803564	509086
804378	509086
804213	509086
802816	509086
803487	509086
804279	509086
804224	509086
800220	503113
800176	503113
800814	503575
804587	503575
804576	503575
804609	503575
802574	503575
801001	503575
800066	503575
800946	503575
803674	503575
801496	503575
804697	503575
804521	503575
800935	503575
801078	503575
804466	503575
801133	503575
804499	503575
800913	503575
804763	503575
803773	503575
804620	503575
804752	503575
804664	503575
801067	503575
802508	503575
804587	504785
801177	504785
801507	504785
801342	504785
801265	504785
804620	504785
801386	504785
803190	504785
801298	504785
801188	504785
801485	504785
801529	504785
801243	504785
804620	507370
803553	507370
803674	507370
802563	500396
802607	500396
802453	500396
803443	500396
802574	500396
804389	500396
803157	500396
802871	500396
802442	500396
804631	500396
802354	500396
802299	500396
802596	500396
802684	500396
804565	500396
802431	500396
802728	500396
804895	500396
802321	500396
804884	500396
802849	500396
802519	500396
803498	500396
802464	500396
800418	500396
804609	500352
804301	500352
800077	500352
802442	500352
800066	500352
802354	500352
804521	500352
802717	500352
801463	500352
802431	500352
804796	500352
804477	500352
804411	500352
800011	500352
802321	500352
803575	500352
804180	500352
802464	500352
803839	500352
804576	500352
800198	500352
800000	500352
801353	500352
802310	500352
804631	500352
804807	500352
801540	500352
801496	500352
804697	500352
802596	500352
804466	500352
804444	500352
801573	500352
802783	500352
801595	500352
801342	500352
801265	500352
802475	500352
804664	500352
804488	500352
802123	505610
802024	505610
802156	505610
802244	505610
804466	509185
804961	509185
804576	509185
801628	509185
804609	509185
804565	509185
801573	509185
800198	509185
804411	509185
804785	503520
801969	503520
803080	503520
800198	503520
801760	503520
802277	503520
800330	503520
804631	503520
804807	503520
801496	503520
804543	503520
801925	503520
801639	503520
802552	503520
804444	503520
802530	503520
801573	503520
803861	503520
801782	503520
801441	503520
801771	503520
804411	503520
802101	503520
800022	503520
802255	503520
802970	503520
802145	503520
801606	503520
801364	509306
804576	509306
801628	509306
804697	509306
801584	509306
801606	509306
803839	507623
804587	507623
804576	507623
804609	507623
801760	507623
804697	507623
804521	507623
804444	507623
804466	507623
801826	507623
804565	507623
803861	507623
804411	507623
804664	507623
803872	507623
801606	507623
800825	507678
800913	507678
800924	507678
800968	507678
801166	507678
800946	507678
801067	507678
800935	507678
800979	507678
804587	502486
804576	502486
800033	502486
800000	502486
800077	502486
800066	502486
804851	502486
804697	502486
800275	502486
804796	502486
804499	502486
800055	502486
804620	502486
800220	502486
804664	502486
804675	502486
804818	502486
803421	502618
803476	502618
800946	502618
801155	502618
803498	502618
801078	502618
803421	502607
803476	502607
801155	502607
803498	502607
802563	509339
802442	509339
802794	509339
802453	509339
802431	509339
802871	509339
802376	509339
804917	507766
802992	507766
800891	507766
801705	503267
801441	503267
801793	503267
804576	503267
804697	503267
801606	503267
801177	501364
801364	501364
801826	501364
801309	501364
801430	501364
801419	501364
801276	501364
801364	501375
801331	501375
801628	501375
801430	501375
801573	501375
801474	501375
800913	501375
801595	501375
801067	501375
801419	501375
802233	504323
802453	504323
801221	504323
802706	504323
802574	504323
803542	504323
802761	504323
801254	504323
802442	504323
802277	504323
803432	504323
802695	504323
802354	504323
802134	504323
802420	504323
802332	504323
803641	504323
803454	504323
802772	504323
801364	504323
803553	504323
802827	504323
802431	504323
802321	504323
802640	504323
801199	504323
805016	504323
802464	504323
801309	504323
802387	504323
802948	504323
800198	504323
802673	504323
801353	504323
801694	504323
803113	504323
802838	504323
802860	504323
801562	504323
802123	504323
802651	504323
801540	504323
801584	504323
802299	504323
803586	504323
803564	504323
802596	504323
802530	504323
801826	504323
802068	504323
802783	504323
802882	504323
802816	504323
801276	504323
801397	504323
802585	504323
802255	504323
804884	504323
800220	504323
801287	504323
802541	504323
803311	503245
803795	503245
804510	503245
803399	503245
803322	503245
804961	504499
804653	504499
801221	504499
800198	504499
804763	504499
803773	504499
803696	504499
804752	504499
803982	504499
801903	504499
802508	504499
804774	504499
803597	500803
804587	507799
804466	507799
804576	507799
804664	507799
804565	507799
804466	500825
804576	500825
803619	500825
800165	500825
802728	500825
802783	500825
800198	500825
800077	500825
802596	500825
803586	500825
804268	505489
800495	505489
802431	500418
800198	500418
800385	500418
800385	506677
800495	506677
804422	500429
802431	500429
801496	500429
804422	505687
801287	505687
801397	505687
803674	505896
803674	505918
802398	500484
803674	500484
802431	500484
800913	500506
803685	500506
800385	500506
800495	500506
801287	505742
801397	505742
802827	505467
802585	505467
802706	505467
802431	505467
802409	505467
802365	505467
802827	506886
802365	506886
802585	506963
800033	506963
802585	506952
802409	506952
802585	506974
803564	500539
802497	505456
802585	505456
803564	505456
802651	500693
802816	500693
803564	500693
802409	507788
802365	507788
802431	507722
801397	505764
802310	506028
802585	506941
804906	506941
800385	506941
800495	506941
802827	500594
802585	500594
800385	500594
802827	500583
802585	500583
802387	500583
800385	500583
802585	500737
802387	500737
802706	500737
804906	500737
800385	500737
802585	500572
804906	500572
800385	500572
804906	504763
800528	504763
800385	504763
800407	505401
802585	505401
804906	505401
800385	505401
802585	505412
804906	505412
800495	505412
802585	505423
800396	505423
804906	505423
800385	505423
800495	505423
800440	505423
802585	505434
804906	505434
800495	505434
800385	509350
802431	505753
800385	505753
800495	505753
804334	509361
804268	509361
803674	509372
803289	509372
802695	500671
802431	500671
802409	500671
802695	507777
802431	507777
803564	500605
803784	505478
802827	505478
803674	505478
800495	505478
802431	500627
802299	500627
802343	500627
800407	500649
802816	506127
803564	506127
802827	506875
802398	506875
802365	506875
802497	506666
804906	506666
800385	506666
804719	501199
804587	501199
804466	501199
804444	501199
804730	501199
804653	501199
800561	501199
800693	501199
803707	501199
803740	501199
803773	501199
804554	501199
804620	501199
804741	501199
804587	500154
804653	500154
801232	500154
804433	500154
801562	500154
801540	500154
803674	500154
802299	500154
801364	500154
804719	500154
800913	500154
801408	500154
802431	500154
804499	500154
803784	500154
804620	500154
804675	500154
804664	500154
804741	500154
802497	505830
802387	505830
802794	505830
802453	505830
802706	505830
802574	505830
802871	505830
802442	505830
802651	505830
802618	505830
802772	505830
803960	505830
803993	505830
802827	505830
802728	505830
802882	505830
802321	505830
802629	505830
802662	505830
804906	505830
803498	505830
803784	502508
803696	502508
803729	502508
801573	502508
803674	502508
803773	508195
803696	508195
803729	508195
803674	508195
802508	508195
804774	508195
802772	502519
802530	502519
802321	502519
801441	505797
803696	505797
804565	505797
801606	505797
804576	507986
804345	507986
802486	507986
804664	507986
802728	507986
804367	507986
804191	507986
803784	509064
803674	509064
800693	509064
802508	509064
800330	502530
800660	502530
800352	502530
803806	502530
800528	502530
800440	502530
802024	508184
802244	508184
802200	508184
802024	502541
802156	502541
802244	502541
802200	502541
803784	509075
803696	509075
803674	509075
802387	501925
802453	501925
802706	501925
800198	501925
800429	501925
800616	501925
803663	501925
802442	501925
800143	501925
803069	501925
801243	501925
800462	501925
802827	501925
800748	501925
803234	501925
800528	501925
800693	501925
803190	501925
801034	501925
802321	501925
800880	501925
801749	501925
802464	501925
801617	507095
801716	507095
802453	507095
801727	507095
802871	507095
804103	507095
800429	507095
802442	507095
802695	507095
802354	507095
802409	507095
804114	507095
801925	507095
801639	507095
801837	507095
802552	507095
804994	507095
802827	507095
801463	507095
804048	507095
800088	507095
801782	507095
802365	507095
801958	507095
803575	507095
802805	507095
801199	507095
801606	507095
803025	507095
804576	507095
804510	507095
802607	507095
800198	507095
804323	507095
802310	507095
800143	507095
804697	507095
801584	507095
801419	507095
802299	507095
804235	507095
802167	507095
804565	507095
800275	507095
801914	507095
804246	507095
801276	507095
801991	507095
800187	507095
804950	507095
804070	507095
802750	507095
802541	507095
800418	507095
804961	507095
804389	507095
800759	507095
800077	507095
801760	507095
801705	507095
801298	507095
801859	507095
801364	507095
800462	507095
801628	507095
801980	507095
802431	507095
802343	507095
804411	507095
802024	507095
803322	507095
802464	507095
801331	507095
801309	507095
801430	507095
801969	507095
800000	507095
805027	507095
801353	507095
801694	507095
801793	507095
804631	507095
801496	507095
801738	507095
803069	507095
800847	507095
801177	507095
802684	507095
801826	507095
801573	507095
801397	507095
804015	507095
801441	507095
802475	507095
801903	507095
801287	507095
800550	507117
803806	507117
803707	507117
800715	508206
800561	508206
800550	508206
800671	508206
800693	508206
800429	508206
800099	508206
802233	507084
801617	507084
801320	507084
804103	507084
800429	507084
802695	507084
802409	507084
804114	507084
801925	507084
801639	507084
802552	507084
801463	507084
801782	507084
802365	507084
801958	507084
803575	507084
802805	507084
801199	507084
801606	507084
804576	507084
804323	507084
802310	507084
802123	507084
800143	507084
804697	507084
801584	507084
801419	507084
802299	507084
802167	507084
801914	507084
801276	507084
801991	507084
804950	507084
804070	507084
802750	507084
802541	507084
800418	507084
804961	507084
804389	507084
800077	507084
801760	507084
801705	507084
802134	507084
801298	507084
802717	507084
801364	507084
800462	507084
802431	507084
802024	507084
802321	507084
802640	507084
803322	507084
802464	507084
801331	507084
801309	507084
801430	507084
801969	507084
800000	507084
801793	507084
801353	507084
801738	507084
803069	507084
801177	507084
801826	507084
802684	507084
801573	507084
801397	507084
803036	507084
801441	507084
802475	507084
800220	507084
801903	507084
801287	507084
801122	507865
803465	507865
801254	507865
803520	507865
801056	507865
801639	507865
801859	507865
801243	507865
800979	507865
801078	507865
802288	507865
801364	507865
801133	507865
803553	507865
801045	507865
800825	507865
801782	507865
800902	507865
801771	507865
801067	507865
801100	507865
801023	507865
803443	507865
801793	507865
800968	507865
800836	507865
801804	507865
800935	507865
801584	507865
800847	507865
801177	507865
801111	507865
801826	507865
800913	507865
801155	507865
801276	507865
801441	507865
801034	507865
804422	501595
802431	501595
801397	501595
801221	504279
804422	504279
802387	501716
804422	501716
803443	501716
800594	501716
800759	501716
800583	501716
802442	501716
803520	501716
802409	501716
800715	501716
803564	501716
800517	501716
800088	501716
802398	501716
802431	501716
802365	501716
801397	501716
802343	501716
802321	501716
802849	501716
801287	501716
805016	501716
802541	501716
802629	501727
802651	501727
802728	501727
802596	501727
802651	502321
802728	502321
802519	502321
802431	502321
802497	502398
802387	502398
804422	502398
803531	502398
803443	502398
804389	502398
804323	502398
803465	502398
802442	502398
800374	502398
802431	502398
801155	502398
802365	502398
800385	502398
801397	502398
803487	502398
804906	502398
801287	502398
802541	502398
802431	505104
802409	505104
804587	500880
804576	500880
804598	500880
804653	500880
804686	500880
800077	500880
800066	500880
804807	500880
804521	500880
801298	500880
801529	500880
804466	500880
804796	500880
804499	500880
801342	500880
804620	500880
804664	500880
804532	500880
804664	500891
804653	500891
804499	500891
804532	500891
804686	500891
804719	500902
804587	500902
804576	500902
804598	500902
804653	500902
804499	500902
804686	500902
801342	500902
804620	500902
804664	500902
804532	500902
804521	500902
801529	500902
805038	504191
804895	504191
802607	504191
804884	504191
802211	501892
802211	501287
802211	504422
802211	504202
802211	504213
802211	508888
802651	503839
802728	503839
802607	502354
802651	502354
800484	502354
802728	502354
802574	502354
802882	502354
802607	502310
802651	502310
802728	502310
802574	502310
802882	502310
802299	502310
800869	501815
801100	501815
801309	501815
800429	501815
801705	501815
801793	501815
801694	501815
800968	501815
800946	501815
800836	501815
801056	501815
801804	501815
800935	501815
801639	501815
801738	501815
800847	501815
800462	501815
801133	501815
801177	501815
801826	501815
801045	501815
800825	501815
801782	501815
801771	501815
801441	501815
801067	501815
803102	501815
800022	509383
800033	509383
800011	509383
800044	509383
800077	509383
803597	507469
800022	507469
800033	507469
800066	507469
800000	507469
800077	507469
800055	507469
803421	504312
800869	504312
803608	504312
801100	504312
803553	504312
801045	504312
803542	504312
801155	504312
803762	504312
803377	504312
801793	504312
801562	504312
803113	504312
801804	504312
803498	504312
800374	506017
801155	506017
802244	501518
803663	507832
800330	509394
804631	509394
800352	509394
803861	509394
803443	509394
800528	509394
800440	509394
804576	508118
803696	508118
804664	508118
802354	508118
801386	508074
804664	508074
804609	508074
804499	508074
801188	508074
804719	502959
804752	502959
804653	502959
804741	502959
804763	502959
804774	502959
804587	509405
804466	509405
801463	509405
801595	509405
801342	509405
801265	509405
804411	509405
802860	509405
801463	509416
801177	509427
801133	509427
801826	509427
801045	509427
801001	509427
801276	509427
801705	509427
801199	509427
801067	509427
803190	509438
803366	509438
803256	509438
803234	509438
800869	503542
801969	503542
801001	503542
800198	503542
803542	503542
802057	503542
800429	503542
803432	503542
803113	503542
802838	503542
801925	503542
803564	503542
802255	503542
800352	503542
801199	503542
800418	503542
804961	503289
801969	503289
803531	503289
803542	503289
800594	503289
800671	503289
800737	503289
800616	503289
800330	503289
803432	503289
800627	503289
800605	503289
803674	503289
803564	503289
800440	503289
800748	503289
800528	503289
800693	503289
803784	503289
800704	503289
800352	503289
800660	503289
802508	503289
803553	503300
804609	503300
802783	503300
804213	503300
801166	503300
804521	503300
803498	503300
802508	503300
802563	507315
800462	507315
802607	507315
802453	507315
802871	507315
802376	507315
802442	507315
804895	507315
804884	507315
802563	504059
800462	504059
802827	504059
802607	504059
802453	504059
802706	504059
802728	504059
802574	504059
802871	504059
802882	504059
802651	504059
804884	504059
803784	503333
803674	503333
801254	507073
801793	508877
802024	508877
801826	508877
801859	508877
800022	505060
800033	505060
800011	505060
800066	505060
800044	505060
800077	505060
801144	505038
803960	505038
803993	505038
801936	505038
800275	505038
800407	505038
804004	505038
803971	505038
803982	505038
803916	505038
804818	505038
803630	508822
801848	508822
800022	501562
800033	501562
800011	501562
800066	501562
804499	501562
800044	501562
800077	501562
801793	501991
802024	501991
801199	501991
801859	501991
800462	508019
804576	508019
804499	508019
802728	508019
802882	508019
804664	508019
804884	508019
802409	508019
802299	508019
800495	508019
802607	506699
804301	506699
802453	506699
802706	506699
802574	506699
804389	506699
804323	506699
800429	506699
802277	506699
802442	506699
802838	506699
802651	506699
802354	506699
804235	506699
802596	506699
802772	506699
800462	506699
802827	506699
802684	506699
802530	506699
804378	506699
802728	506699
802431	506699
802783	506699
802882	506699
804279	506699
802321	506699
804884	506699
802640	506699
802849	506699
802464	506699
802717	508008
802563	508008
802827	508008
802684	508008
802728	508008
802761	508008
802442	508008
805038	508008
802354	508008
802640	508008
802519	508008
802409	508030
802365	508030
802299	508030
805016	508030
802343	508030
800946	502167
800033	502167
804048	502167
800913	502167
801067	502167
801078	502167
800913	505984
800561	502134
803685	502134
800671	502134
800693	502134
803707	502134
803773	502134
800660	502134
800715	502134
803674	502134
800550	502134
804543	502134
803839	507040
804576	507040
804598	507040
800033	507040
800044	507040
800000	507040
804686	507040
801760	507040
800077	507040
800330	507040
804026	507040
800066	507040
804807	507040
801496	507040
804697	507040
804114	507040
801210	507040
804444	507040
804565	507040
804499	507040
803861	507040
801595	507040
804092	507040
800528	507040
801342	507040
804015	507040
804664	507040
804532	507040
800418	507040
803795	507051
804598	507051
804609	507051
804081	507051
804169	507051
804103	507051
801760	507051
800077	507051
800066	507051
803674	507051
802409	507051
804521	507051
804114	507051
804499	507051
804796	507051
804092	507051
804477	507051
804532	507051
803322	507051
803839	507051
804576	507051
804510	507051
800000	507051
804686	507051
804323	507051
801562	507051
804026	507051
801496	507051
804697	507051
801210	507051
804444	507051
804565	507051
800253	507051
803861	507051
801595	507051
801342	507051
804015	507051
804664	507051
804488	507051
802508	507051
804708	507051
803839	507062
804576	507062
804510	507062
804609	507062
801760	507062
804026	507062
804807	507062
804697	507062
804521	507062
804543	507062
801210	507062
804444	507062
801463	507062
804565	507062
803861	507062
804092	507062
804477	507062
801265	507062
804411	507062
804664	507062
804488	507062
804708	507062
804466	503894
804961	503894
801617	503894
801463	503894
804609	503894
804565	503894
801573	503894
801452	503894
801595	503894
801991	503894
804411	503894
801353	503894
801793	503894
800220	503894
800814	502871
802574	502871
803542	502871
803157	502871
802871	502871
801551	502871
800583	502871
804037	502871
803432	502871
803201	502871
802981	502871
802695	502871
802409	502871
802904	502871
800495	502871
803124	502871
801463	502871
802013	502871
804092	502871
803806	502871
803234	502871
803289	502871
800385	502871
800726	502871
803575	502871
803366	502871
801199	502871
802145	502871
803025	502871
802035	502871
804510	502871
802607	502871
800957	502871
800594	502871
803003	502871
800330	502871
802838	502871
802860	502871
800143	502871
803828	502871
804851	502871
800682	502871
800935	502871
802222	502871
802992	502871
802882	502871
800693	502871
801265	502871
801034	502871
800187	502871
800550	502871
802178	502871
801012	502871
803421	502871
804785	502871
802959	502871
800803	502871
801089	502871
804653	502871
800759	502871
803146	502871
800671	502871
801760	502871
803707	502871
800737	502871
803905	502871
800715	502871
803949	502871
800099	502871
801243	502871
801485	502871
802772	502871
804840	502871
803597	502871
800748	502871
800902	502871
804477	502871
800660	502871
803322	502871
800781	502871
801144	502871
802387	502871
803531	502871
802948	502871
800616	502871
803663	502871
803179	502871
803696	502871
802651	502871
800440	502871
804466	502871
800341	502871
802684	502871
801573	502871
803223	502871
804763	502871
804015	502871
803487	502871
801441	502871
803476	502871
802255	502871
804884	502871
802519	502871
800033	509515
800066	509515
804851	509515
804796	509515
800000	509515
800077	509515
803619	509526
804565	509526
804158	509526
801287	509526
\.


--
-- Data for Name: search_log; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY search_log (search_id, session_id, ip_address, "timestamp", search_phrase, referer) FROM stdin;
\.


--
-- Data for Name: synonym_stem; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY synonym_stem (synonym_id, word_stem, submitted_word) FROM stdin;
900000	add	ADD
900000	adhd	ADHD
900000	hadd	HADD
900011	alcohol	alcohol
900011	drink	drinking
900011	drunk	drunk
900022	assist	assist
900022	help	help
900033	cessat	cessation
900033	quit	quit
900044	child	child
900044	children	children
900044	kid	kid
900044	youth	youth
900055	class	class
900055	lesson	lesson
900066	comput	computer
900066	technologi	technology
900066	internet	internet
900077	counsel	counseling
900077	therapi	therapy
900088	detoxif	detoxification
900088	detox	detox
900099	disabl	disabled
900099	handicap	handicapped
900110	drug	drug
900110	substanc	substance
900121	elder	elder
900121	elderli	elderly
900121	senior	senior
900132	fit	fitness
900132	exerc	exercise
900143	health	health
900143	medic	medical
900143	healthcar	healthcare
900154	infant	infant
900154	babi	baby
900165	job	job
900165	career	career
900165	vocat	vocation
900165	employ	employment
900176	literaci	literacy
900176	read	reading
900176	write	writing
900176	languag	language
900187	marit	marital
900187	marriag	marriage
900198	methadon	methadone
900198	heroin	heroine
900209	monei	money
900209	financ	finance
900220	nurs	nurse
900220	rna	rna
900231	nutrition	nutrition
900231	nutritionist	nutritionist
900242	pregnant	pregnant
900242	pregnanc	pregnancy
900253	regist	register
900253	registr	registration
900264	teen	teen
900264	adolesc	adolescent
900275	televis	television
900275	tv	tv
900286	tenant	tenant
900286	rent	rent
900286	renter	renter
900286	rental	rental
900297	tobacco	tobacco
900297	smoke	smoking
900308	util	utility
900308	electr	electricity
900308	energi	energy
900308	fuel	fuel
900308	heat	heat
900308	ga	gas
900308	oil	oil
900319	vote	vote
900319	voter	voter
900319	elect	elect
\.


--
-- Data for Name: town; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY town (town_state, county, region, display) FROM stdin;
Bedford, MA	Middlesex County	\N	t
Billerica, MA	Middlesex County	Greater Lowell	t
Chelmsford, MA	Middlesex County	Greater Lowell	t
Dracut, MA	Middlesex County	Greater Lowell	t
Littleton, MA	Middlesex County	\N	t
Lowell, MA	Middlesex County	Greater Lowell	t
Tewksbury, MA	Middlesex County	Greater Lowell	t
Tyngsborough, MA	Middlesex County	Greater Lowell	t
Westford, MA	Middlesex County	Greater Lowell	t
Amesbury, MA	Essex County	\N	t
Andover, MA	Essex County	Greater Lawrence	t
Haverhill, MA	Essex County	Greater Lawrence	t
Lawrence, MA	Essex County	Greater Lawrence	t
Merrimac, MA	Essex County	\N	t
Methuen, MA	Essex County	Greater Lawrence	t
Newburyport, MA	Essex County	\N	t
Salisbury, MA	Essex County	\N	t
Acton, MA	Middlesex County	\N	f
Arlington, MA	Middlesex County	\N	f
Ashby, MA	Middlesex County	\N	f
Ashland, MA	Middlesex County	\N	f
Ayer, MA	Middlesex County	\N	f
Belmont, MA	Middlesex County	\N	f
Boxborough, MA	Middlesex County	\N	f
Burlington, MA	Middlesex County	\N	f
Cambridge, MA	Middlesex County	\N	f
Carlisle, MA	Middlesex County	\N	f
Concord, MA	Middlesex County	\N	f
Everett, MA	Middlesex County	\N	f
Framingham, MA	Middlesex County	\N	f
Groton, MA	Middlesex County	\N	f
Holliston, MA	Middlesex County	\N	f
Hopkinton, MA	Middlesex County	\N	f
Hudson, MA	Middlesex County	\N	f
Lexington, MA	Middlesex County	\N	f
Lincoln, MA	Middlesex County	\N	f
Malden, MA	Middlesex County	\N	f
Marlborough, MA	Middlesex County	\N	f
Maynard, MA	Middlesex County	\N	f
Medford, MA	Middlesex County	\N	f
Melrose, MA	Middlesex County	\N	f
Natick, MA	Middlesex County	\N	f
Newton, MA	Middlesex County	\N	f
North Reading, MA	Middlesex County	\N	f
Pepperell, MA	Middlesex County	\N	f
Reading, MA	Middlesex County	\N	f
Sherborn, MA	Middlesex County	\N	f
Shirley, MA	Middlesex County	\N	f
Somerville, MA	Middlesex County	\N	f
Stoneham, MA	Middlesex County	\N	f
Stow, MA	Middlesex County	\N	f
Sudbury, MA	Middlesex County	\N	f
Townsend, MA	Middlesex County	\N	f
Wakefield, MA	Middlesex County	\N	f
Waltham, MA	Middlesex County	\N	f
Watertown, MA	Middlesex County	\N	f
Wayland, MA	Middlesex County	\N	f
Weston, MA	Middlesex County	\N	f
Winchester, MA	Middlesex County	\N	f
Woburn, MA	Middlesex County	\N	f
Beverly, MA	Essex County	\N	f
Danvers, MA	Essex County	\N	f
Essex County, MA	Essex County	\N	f
Gloucester, MA	Essex County	\N	f
Hamilton, MA	Essex County	\N	f
Ipswich, MA	Essex County	\N	f
Lynn, MA	Essex County	\N	f
Lynnfield, MA	Essex County	\N	f
Manchester-by-the-Sea, MA	Essex County	\N	f
Marblehead, MA	Essex County	\N	f
Middleton, MA	Essex County	\N	f
Nahant, MA	Essex County	\N	f
Peabody, MA	Essex County	\N	f
Rockport, MA	Essex County	\N	f
Salem, MA	Essex County	\N	f
Saugus, MA	Essex County	\N	f
Swampscott, MA	Essex County	\N	f
Topsfield, MA	Essex County	\N	f
Wenham, MA	Essex County	\N	f
Dunstable, MA	Middlesex County	Greater Lowell	t
North Andover, MA	Essex County	Greater Lawrence	t
Boxford, MA	Essex County	\N	t
Georgetown, MA	Essex County	\N	t
Groveland, MA	Essex County	\N	t
Newbury, MA	Essex County	\N	t
Rowley, MA	Essex County	\N	t
West Newbury, MA	Essex County	\N	t
Wilmington, MA	Middlesex County	\N	t
\.


--
-- Data for Name: user_type; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY user_type (type_id, type_name) FROM stdin;
1	Data Entry
\.


--
-- Data for Name: users; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY users (user_id, first_name, last_name, username, "password", user_type, created_on) FROM stdin;
300000	Test	User	tester	disabled4now	1	2007-07-08 14:42:24.571937
300011	Chelsea	Garcia	nyzfin3st	disabled4now	1	2007-07-09 12:04:26.629588
300022	Jon	McCluskey	jm3675a	disabled4now	1	2007-07-09 12:05:16.500683
300033	Luz	Mangual	luz123	disabled4now	1	2007-07-11 17:04:00.298847
300044	Eric	Bryant	ebryant	funwitholdfolks	1	2008-05-30 12:13:35.860725
300055	Dan	MacNeil	omacneil	bedbug33	1	2008-06-08 11:29:26.523136
\.


--
-- Data for Name: web_requests; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY web_requests (id, host, username, date, rtype, file, proto, code, bytes, refer, agent) FROM stdin;
\.


--
-- Data for Name: zip_distances; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY zip_distances (start_zip, end_zip, distance) FROM stdin;
\.


--
-- Name: agency_agency_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY agency
    ADD CONSTRAINT agency_agency_name_key UNIQUE (agency_name);


--
-- Name: agency_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY agency
    ADD CONSTRAINT agency_pkey PRIMARY KEY (agency_id);


--
-- Name: alias_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY alias
    ADD CONSTRAINT alias_pkey PRIMARY KEY (category_id, alias_name);


--
-- Name: category_category_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY category
    ADD CONSTRAINT category_category_name_key UNIQUE (category_name);


--
-- Name: category_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY category
    ADD CONSTRAINT category_pkey PRIMARY KEY (category_id);


--
-- Name: data_entry_agency_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY data_entry_agency
    ADD CONSTRAINT data_entry_agency_pkey PRIMARY KEY (id);


--
-- Name: data_entry_program_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY data_entry_program
    ADD CONSTRAINT data_entry_program_pkey PRIMARY KEY (id);


--
-- Name: data_entry_status_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY data_entry_status
    ADD CONSTRAINT data_entry_status_pkey PRIMARY KEY (status_id);


--
-- Name: data_entry_status_status_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY data_entry_status
    ADD CONSTRAINT data_entry_status_status_name_key UNIQUE (status_name);


--
-- Name: date_agency_created_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY date_agency_created
    ADD CONSTRAINT date_agency_created_pkey PRIMARY KEY (agency_id);


--
-- Name: date_program_created_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY date_program_created
    ADD CONSTRAINT date_program_created_pkey PRIMARY KEY (program_id);


--
-- Name: heading_category_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY heading_category
    ADD CONSTRAINT heading_category_pkey PRIMARY KEY (category_id, heading_name);


--
-- Name: heading_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY heading
    ADD CONSTRAINT heading_pkey PRIMARY KEY (heading_name);


--
-- Name: log_category_search_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY log_category_search
    ADD CONSTRAINT log_category_search_pkey PRIMARY KEY (search_id);


--
-- Name: program_category_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY program_category
    ADD CONSTRAINT program_category_pkey PRIMARY KEY (category_id, program_id);


--
-- Name: program_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY program
    ADD CONSTRAINT program_pkey PRIMARY KEY (program_id);


--
-- Name: search_log_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY search_log
    ADD CONSTRAINT search_log_pkey PRIMARY KEY (search_id);


--
-- Name: synonym_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY synonym_stem
    ADD CONSTRAINT synonym_pkey PRIMARY KEY (synonym_id, word_stem);


--
-- Name: town_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY town
    ADD CONSTRAINT town_pkey PRIMARY KEY (town_state);


--
-- Name: user_type_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY user_type
    ADD CONSTRAINT user_type_pkey PRIMARY KEY (type_id);


--
-- Name: user_type_type_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY user_type
    ADD CONSTRAINT user_type_type_name_key UNIQUE (type_name);


--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (user_id);


--
-- Name: users_username_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_username_key UNIQUE (username);


--
-- Name: log_category_search_index_by_session_id; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE INDEX log_category_search_index_by_session_id ON log_category_search USING btree (session_id, phrase_stems);


--
-- Name: session_id_index_on_agency_search_log; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE INDEX session_id_index_on_agency_search_log ON agency_search_log USING btree (session_id, search_phrase);


--
-- Name: session_id_index_on_category_search_log; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE INDEX session_id_index_on_category_search_log ON category_search_log USING btree (session_id, phrase_stems);


--
-- Name: start_zip_index_on_zip_distances; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE UNIQUE INDEX start_zip_index_on_zip_distances ON zip_distances USING btree (start_zip, end_zip);


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY users
    ADD CONSTRAINT "$1" FOREIGN KEY (user_type) REFERENCES user_type(type_id) ON DELETE SET NULL;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY data_entry_agency
    ADD CONSTRAINT "$1" FOREIGN KEY (updated_by) REFERENCES users(user_id);


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY data_entry_program
    ADD CONSTRAINT "$1" FOREIGN KEY (updated_by) REFERENCES users(user_id);


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY program
    ADD CONSTRAINT "$1" FOREIGN KEY (agency_id) REFERENCES agency(agency_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY alias
    ADD CONSTRAINT "$1" FOREIGN KEY (category_id) REFERENCES category(category_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY heading_category
    ADD CONSTRAINT "$1" FOREIGN KEY (category_id) REFERENCES category(category_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY program_category
    ADD CONSTRAINT "$1" FOREIGN KEY (category_id) REFERENCES category(category_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY date_agency_created
    ADD CONSTRAINT "$1" FOREIGN KEY (agency_id) REFERENCES agency(agency_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY date_program_created
    ADD CONSTRAINT "$1" FOREIGN KEY (program_id) REFERENCES program(program_id) ON DELETE CASCADE;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY data_entry_agency
    ADD CONSTRAINT "$2" FOREIGN KEY (update_status) REFERENCES data_entry_status(status_id) ON DELETE SET DEFAULT;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY data_entry_program
    ADD CONSTRAINT "$2" FOREIGN KEY (update_status) REFERENCES data_entry_status(status_id) ON DELETE SET DEFAULT;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY program_category
    ADD CONSTRAINT "$2" FOREIGN KEY (program_id) REFERENCES program(program_id) ON DELETE CASCADE;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY heading_category
    ADD CONSTRAINT "$2" FOREIGN KEY (heading_name) REFERENCES heading(heading_name) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

--
-- PostgreSQL database dump complete
--

