--
-- PostgreSQL database dump
--

SET client_encoding = 'SQL_ASCII';
SET check_function_bodies = false;
SET client_min_messages = warning;

SET search_path = public, pg_catalog;

--
-- Name: get_agency_name(integer); Type: FUNCTION; Schema: public; Owner: omacneil
--

CREATE FUNCTION get_agency_name(integer) RETURNS text
    AS $_$select agency_name from agency where agency_id = $1;$_$
    LANGUAGE sql STABLE STRICT;


--
-- Name: get_password(integer); Type: FUNCTION; Schema: public; Owner: omacneil
--

CREATE FUNCTION get_password(integer) RETURNS text
    AS $_$select password from agency where agency_id = $1;$_$
    LANGUAGE sql STABLE STRICT;


--
-- Name: our_concat(text, text); Type: FUNCTION; Schema: public; Owner: omacneil
--

CREATE FUNCTION our_concat(text, text) RETURNS text
    AS $_$select $1 || ', ' || $2;$_$
    LANGUAGE sql STABLE STRICT;


SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: agency; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE agency (
    agency_id integer DEFAULT nextval(('agency_id_sequence'::text)::regclass) NOT NULL,
    last_updated date NOT NULL,
    hidden boolean DEFAULT false NOT NULL,
    "password" text NOT NULL,
    agency_name text NOT NULL,
    agency_alias text,
    address1 text NOT NULL,
    address2 text,
    city text NOT NULL,
    state text NOT NULL,
    zip text NOT NULL,
    main_phone text NOT NULL,
    toll_free_phone text,
    hotline text,
    fax text,
    website text,
    email text,
    mission text,
    hours text NOT NULL,
    other_info text,
    mail_address1 text,
    mail_address2 text,
    mail_city text,
    mail_state text,
    mail_zip text,
    chief text NOT NULL,
    title text NOT NULL,
    parent_organization text,
    resource_library text NOT NULL,
    resource_library_description text,
    meeting_space text NOT NULL,
    meeting_space_description text,
    staff_internet_access text NOT NULL,
    contact_first_name text NOT NULL,
    contact_last_name text NOT NULL,
    contact_phone text NOT NULL,
    contact_fax text,
    contact_email text NOT NULL,
    contact_email_available text NOT NULL,
    tty_tdd text,
    reminders_sent integer DEFAULT 0 NOT NULL,
    admin_notes text,
    CONSTRAINT agency_address1 CHECK ((address1 <> ''::text)),
    CONSTRAINT agency_agency_name CHECK ((agency_name <> ''::text)),
    CONSTRAINT agency_chief CHECK ((chief <> ''::text)),
    CONSTRAINT agency_city CHECK ((city <> ''::text)),
    CONSTRAINT agency_contact_email CHECK ((contact_email <> ''::text)),
    CONSTRAINT agency_contact_email_available CHECK ((contact_email_available <> ''::text)),
    CONSTRAINT agency_contact_first_name CHECK ((contact_first_name <> ''::text)),
    CONSTRAINT agency_contact_last_name CHECK ((contact_last_name <> ''::text)),
    CONSTRAINT agency_contact_phone CHECK ((contact_phone <> ''::text)),
    CONSTRAINT agency_hours CHECK ((hours <> ''::text)),
    CONSTRAINT agency_main_phone CHECK ((main_phone <> ''::text)),
    CONSTRAINT agency_meeting_space CHECK ((meeting_space <> ''::text)),
    CONSTRAINT agency_password CHECK (("password" <> ''::text)),
    CONSTRAINT agency_resource_library CHECK ((resource_library <> ''::text)),
    CONSTRAINT agency_staff_internet_access CHECK ((staff_internet_access <> ''::text)),
    CONSTRAINT agency_state CHECK ((state <> ''::text)),
    CONSTRAINT agency_title CHECK ((title <> ''::text)),
    CONSTRAINT agency_zip CHECK ((zip <> ''::text))
);


--
-- Name: agency_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE agency_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: agency_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('agency_id_sequence', 103927, true);


--
-- Name: search_log_search_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE search_log_search_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: search_log_search_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('search_log_search_id_seq', 36908, true);


--
-- Name: search_log; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE search_log (
    search_id integer DEFAULT nextval('search_log_search_id_seq'::regclass) NOT NULL,
    session_id text,
    ip_address text,
    "timestamp" timestamp without time zone DEFAULT now() NOT NULL,
    search_phrase text,
    referer text
);


--
-- Name: agency_search_log; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE agency_search_log (
    hit_count integer NOT NULL
)
INHERITS (search_log);


--
-- Name: alias; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE alias (
    category_id integer NOT NULL,
    alias_name text NOT NULL,
    stems_name text NOT NULL,
    CONSTRAINT alias_alias_name CHECK ((alias_name <> ''::text)),
    CONSTRAINT alias_stems_name CHECK ((stems_name <> ''::text))
);


--
-- Name: category; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE category (
    category_id integer DEFAULT nextval(('category_id_sequence'::text)::regclass) NOT NULL,
    category_name text NOT NULL,
    stems_name text NOT NULL,
    CONSTRAINT category_category_name CHECK ((category_name <> ''::text)),
    CONSTRAINT category_stems_name CHECK ((stems_name <> ''::text))
);


--
-- Name: program; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE program (
    program_id integer DEFAULT nextval(('program_id_sequence'::text)::regclass) NOT NULL,
    agency_id integer NOT NULL,
    last_updated date NOT NULL,
    hidden_reason text,
    program_name text NOT NULL,
    program_alias text,
    address1 text NOT NULL,
    address2 text,
    city text NOT NULL,
    state text NOT NULL,
    zip text NOT NULL,
    main_phone text NOT NULL,
    toll_free_phone text,
    hotline text,
    fax text,
    website text,
    email text,
    short_program_description text NOT NULL,
    long_program_description text NOT NULL,
    eligibility text NOT NULL,
    fees_charged text NOT NULL,
    fees_explained text,
    program_schedule text NOT NULL,
    hours text NOT NULL,
    languages text NOT NULL,
    appointment text NOT NULL,
    bring text,
    bus_lines text,
    parking text,
    transportation text,
    handicapped_accessible text NOT NULL,
    child_care_provided text NOT NULL,
    child_care_description text,
    wait_list text NOT NULL,
    wait_list_description text,
    other_information text,
    mail_address1 text,
    mail_address2 text,
    mail_city text,
    mail_state text,
    mail_zip text,
    volunteer_opportunities text NOT NULL,
    volunteer_information text,
    resource_library text NOT NULL,
    resource_library_description text,
    meeting_space text NOT NULL,
    meeting_space_description text,
    news_letter text NOT NULL,
    news_letter_description text,
    public_net_access text NOT NULL,
    public_net_access_description text,
    contact_first_name text NOT NULL,
    contact_last_name text NOT NULL,
    contact_phone text NOT NULL,
    contact_fax text,
    contact_email text NOT NULL,
    contact_email_available text NOT NULL,
    area_type text NOT NULL,
    area_data text,
    tty_tdd text,
    areas_served_description text,
    languages_description text,
    online_registration text NOT NULL,
    reminders_sent integer DEFAULT 0 NOT NULL,
    admin_notes text,
    CONSTRAINT area_check CHECK ((((area_data IS NOT NULL) AND (area_data <> ''::text)) OR (area_type = 'EVERYWHERE'::text))),
    CONSTRAINT program_address1 CHECK ((address1 <> ''::text)),
    CONSTRAINT program_appointment CHECK ((appointment <> ''::text)),
    CONSTRAINT program_area_type CHECK ((area_type <> ''::text)),
    CONSTRAINT program_child_care_provided CHECK ((child_care_provided <> ''::text)),
    CONSTRAINT program_city CHECK ((city <> ''::text)),
    CONSTRAINT program_contact_email CHECK ((contact_email <> ''::text)),
    CONSTRAINT program_contact_email_available CHECK ((contact_email_available <> ''::text)),
    CONSTRAINT program_contact_first_name CHECK ((public_net_access_description <> ''::text)),
    CONSTRAINT program_contact_last_name CHECK ((contact_last_name <> ''::text)),
    CONSTRAINT program_contact_phone CHECK ((contact_phone <> ''::text)),
    CONSTRAINT program_eligibility CHECK ((eligibility <> ''::text)),
    CONSTRAINT program_fees_charged CHECK ((fees_charged <> ''::text)),
    CONSTRAINT program_handicapped_accessible CHECK ((handicapped_accessible <> ''::text)),
    CONSTRAINT program_hours CHECK ((hours <> ''::text)),
    CONSTRAINT program_languages CHECK ((languages <> ''::text)),
    CONSTRAINT program_long_program_description CHECK ((long_program_description <> ''::text)),
    CONSTRAINT program_main_phone CHECK ((main_phone <> ''::text)),
    CONSTRAINT program_meeting_space CHECK ((meeting_space <> ''::text)),
    CONSTRAINT program_news_letter CHECK ((news_letter <> ''::text)),
    CONSTRAINT program_online_registration CHECK ((online_registration <> ''::text)),
    CONSTRAINT program_program_name CHECK ((program_name <> 'program_name'::text)),
    CONSTRAINT program_program_schedule CHECK ((program_schedule <> ''::text)),
    CONSTRAINT program_public_net_access CHECK ((public_net_access <> ''::text)),
    CONSTRAINT program_resource_library CHECK ((resource_library <> ''::text)),
    CONSTRAINT program_short_program_description CHECK ((short_program_description <> ''::text)),
    CONSTRAINT program_state CHECK ((state <> ''::text)),
    CONSTRAINT program_volunteer_opportunities CHECK ((volunteer_opportunities <> ''::text)),
    CONSTRAINT program_wait_list CHECK ((wait_list <> ''::text)),
    CONSTRAINT program_zip CHECK ((zip <> ''::text))
);


--
-- Name: program_category; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE program_category (
    category_id integer NOT NULL,
    program_id integer NOT NULL
);


--
-- Name: cats_with_visible_programs_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW cats_with_visible_programs_view AS
    SELECT pc.category_id, c.category_name, count(pc.program_id) AS program_count FROM program_category pc, program p, category c WHERE (((pc.program_id = p.program_id) AND (pc.category_id = c.category_id)) AND (p.hidden_reason IS NULL)) GROUP BY pc.category_id, c.category_name;


--
-- Name: all_names_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW all_names_view AS
    SELECT all_names.category_id, all_names.name, all_names.stems_name, all_names.category_name FROM (SELECT category.category_id, lower(category.category_name) AS name, category.stems_name, category.category_name FROM category UNION SELECT a.category_id, lower(a.alias_name) AS name, a.stems_name, c.category_name FROM alias a, category c WHERE (a.category_id = c.category_id)) all_names WHERE (all_names.category_id IN (SELECT cats_with_visible_programs_view.category_id FROM cats_with_visible_programs_view));


--
-- Name: category_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE category_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: category_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('category_id_sequence', 805038, true);


--
-- Name: category_search_log; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE category_search_log (
    phrase_stems text NOT NULL,
    top_score numeric(4,2) NOT NULL,
    CONSTRAINT category_search_log_phrase_stems CHECK ((phrase_stems <> ''::text))
)
INHERITS (search_log);


--
-- Name: contacts; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW contacts AS
    SELECT agency.contact_first_name, agency.contact_last_name, agency.address1, agency.address2, agency.city, agency.state, agency.zip, agency.mail_address1, agency.mail_address2, agency.mail_city, agency.mail_state, agency.mail_zip, agency.contact_email, agency.contact_phone, agency.contact_fax, agency.agency_id FROM agency UNION SELECT program.contact_first_name, program.contact_last_name, program.address1, program.address2, program.city, program.state, program.zip, program.mail_address1, program.mail_address2, program.mail_city, program.mail_state, program.mail_zip, program.contact_email, program.contact_phone, program.contact_fax, program.agency_id FROM program;


--
-- Name: contact_email_list_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW contact_email_list_view AS
    SELECT DISTINCT ((((contacts.agency_id)::text || ' <'::text) || lower(contacts.contact_email)) || '>'::text) AS verbose_email FROM contacts WHERE ((contacts.contact_email !~* '(unknown|noemail)'::text) AND (contacts.contact_last_name !~* 'contact'::text)) ORDER BY ((((contacts.agency_id)::text || ' <'::text) || lower(contacts.contact_email)) || '>'::text);


--
-- Name: date_agency_created; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE date_agency_created (
    agency_id integer NOT NULL,
    date_created date NOT NULL
);


--
-- Name: date_program_created; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE date_program_created (
    program_id integer NOT NULL,
    date_created date NOT NULL
);


--
-- Name: entity; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW entity AS
    SELECT agency.agency_id AS id, agency.contact_email AS email FROM agency UNION SELECT program.program_id AS id, program.contact_email AS email FROM program;


--
-- Name: heading; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE heading (
    heading_name text NOT NULL
);


--
-- Name: heading_category; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE heading_category (
    category_id integer NOT NULL,
    heading_name text NOT NULL
);


--
-- Name: log_category_search_search_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE log_category_search_search_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: log_category_search_search_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('log_category_search_search_id_seq', 1762, true);


--
-- Name: log_category_search; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE log_category_search (
    search_id integer DEFAULT nextval('log_category_search_search_id_seq'::regclass) NOT NULL,
    session_id text,
    ip_address text,
    "timestamp" timestamp without time zone DEFAULT now() NOT NULL,
    phrase_stems text NOT NULL,
    top_score numeric(4,2) NOT NULL,
    search_phrase text,
    CONSTRAINT log_category_search_phrase_stems CHECK ((phrase_stems <> ''::text))
);


--
-- Name: mailing_list; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW mailing_list AS
    SELECT ((c.contact_first_name || ' '::text) || c.contact_last_name) AS name, a.agency_name, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_address1 ELSE c.address1 END AS address1, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_address2 ELSE c.address2 END AS address2, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_city ELSE c.city END AS city, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_state ELSE c.state END AS state, CASE WHEN (c.mail_address1 IS NOT NULL) THEN c.mail_zip ELSE c.zip END AS zip FROM contacts c, agency a WHERE (c.agency_id = a.agency_id) ORDER BY ((c.contact_first_name || ' '::text) || c.contact_last_name);


--
-- Name: meta_data_version; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE meta_data_version (
    version integer NOT NULL,
    change_date date DEFAULT now() NOT NULL,
    note text NOT NULL
);


--
-- Name: TABLE meta_data_version; Type: COMMENT; Schema: public; Owner: omacneil
--

COMMENT ON TABLE meta_data_version IS 'track version of meta data like towns , categories, synonyms';


--
-- Name: meta_schema_version; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE meta_schema_version (
    version integer NOT NULL,
    change_date date DEFAULT now() NOT NULL,
    note text NOT NULL
);


--
-- Name: TABLE meta_schema_version; Type: COMMENT; Schema: public; Owner: omacneil
--

COMMENT ON TABLE meta_schema_version IS 'track version of tables/indexes setup';


--
-- Name: opt_in_contact_email_view; Type: VIEW; Schema: public; Owner: omacneil
--

CREATE VIEW opt_in_contact_email_view AS
    SELECT condensed_contacts_favoring_no_answers.contact_email FROM (SELECT DISTINCT ON (all_contacts.contact_email) all_contacts.contact_email, all_contacts.contact_email_available FROM (SELECT agency.contact_email, agency.contact_email_available FROM agency UNION SELECT program.contact_email, program.contact_email_available FROM program) all_contacts WHERE (all_contacts.contact_email !~* '(unknown|noemail)'::text) ORDER BY all_contacts.contact_email, all_contacts.contact_email_available) condensed_contacts_favoring_no_answers WHERE (condensed_contacts_favoring_no_answers.contact_email_available = 'Yes'::text);


--
-- Name: program_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE program_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: program_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('program_id_sequence', 510417, true);


--
-- Name: synonym_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE synonym_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: synonym_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('synonym_id_sequence', 900319, true);


--
-- Name: synonym_stem; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE synonym_stem (
    synonym_id integer NOT NULL,
    word_stem text NOT NULL,
    submitted_word text NOT NULL,
    CONSTRAINT synonym_stem_submitted_word CHECK ((submitted_word <> ''::text)),
    CONSTRAINT synonym_stem_word_stem CHECK ((word_stem <> ''::text))
);


--
-- Name: town; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE town (
    town_state text NOT NULL,
    county text NOT NULL,
    region text,
    display boolean DEFAULT false,
    CONSTRAINT town_county CHECK ((county <> ''::text)),
    CONSTRAINT town_region CHECK ((region <> ''::text)),
    CONSTRAINT town_town_state CHECK ((town_state <> ''::text))
);


--
-- Name: user_id_sequence; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE user_id_sequence
    INCREMENT BY 11
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: user_id_sequence; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('user_id_sequence', 300055, true);


--
-- Name: user_type_type_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE user_type_type_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: user_type_type_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('user_type_type_id_seq', 1, false);


--
-- Name: user_type; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE user_type (
    type_id integer DEFAULT nextval('user_type_type_id_seq'::regclass) NOT NULL,
    type_name text NOT NULL,
    CONSTRAINT user_type_type_name CHECK ((type_name <> ''::text))
);


--
-- Name: web_requests_id_seq; Type: SEQUENCE; Schema: public; Owner: omacneil
--

CREATE SEQUENCE web_requests_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: web_requests_id_seq; Type: SEQUENCE SET; Schema: public; Owner: omacneil
--

SELECT pg_catalog.setval('web_requests_id_seq', 749579, true);


--
-- Name: web_requests; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE web_requests (
    id integer DEFAULT nextval('web_requests_id_seq'::regclass) NOT NULL,
    host text NOT NULL,
    username text,
    date timestamp without time zone NOT NULL,
    rtype text NOT NULL,
    file text NOT NULL,
    proto text NOT NULL,
    code integer NOT NULL,
    bytes integer NOT NULL,
    refer text NOT NULL,
    agent text NOT NULL
);


--
-- Name: zip_distances; Type: TABLE; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE TABLE zip_distances (
    start_zip character varying(5) NOT NULL,
    end_zip character varying(5) NOT NULL,
    distance numeric(6,2) NOT NULL,
    CONSTRAINT zip_distances_end_zip CHECK (((end_zip)::text <> ''::text)),
    CONSTRAINT zip_distances_start_zip CHECK (((start_zip)::text <> ''::text))
);


--
-- Data for Name: agency; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY agency (agency_id, last_updated, hidden, "password", agency_name, agency_alias, address1, address2, city, state, zip, main_phone, toll_free_phone, hotline, fax, website, email, mission, hours, other_info, mail_address1, mail_address2, mail_city, mail_state, mail_zip, chief, title, parent_organization, resource_library, resource_library_description, meeting_space, meeting_space_description, staff_internet_access, contact_first_name, contact_last_name, contact_phone, contact_fax, contact_email, contact_email_available, tty_tdd, reminders_sent, admin_notes) FROM stdin;
103608	2009-03-19	f	@lunch4all	Cambridge Family and Children's Service	CFCS	60 Gore Street	\N	cambridge	MA	02141	617-876-4210	\N	857-998-2056	617-661-9749	www.helpfamilies.org	public_agency_email@testing123.net	Our mission is to guide and empower young adults into a healthy lifestyle with an emphasis on personal growth and community connections, using positive role models and neighborhood based programs.	24 hours	\N	\N	\N	\N	\N	\N	Bernadette Peeples	Director of Residential Services	\N	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103520	2009-02-11	f	@lunch4all	North Shore Community Action Programs, Inc.	NSCAP	98 Main St.	\N	Peabody	MA	01960	978-531-0767	\N	\N	978-531-1012	www.nscap.org	public_agency_email@testing123.net	North Shore Community Action Programs, Inc. is the designated anti-poverty agency for the communities of Salem, Peabody, Beverly and Danvers. In addition, we serve 25 cities and towns on the North Shore. Established in 1965 under the Economic Opportunity Act, NSCAPs mission is to help low-income people empower themselves as they move towards self-sufficiency and to motivate the larger community to be more responsive to the needs of low-income people.	Monday through Friday 9:00 to 5:00	\N	\N	\N	\N	\N	\N	Elizabeth Hogan	Executive Director	\N	Not Applicable	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103531	2009-02-26	f	@lunch4all	Operation Bootstrap	\N	156 Broad Street	\N	Lynn, Ma.	MA	01905	781-599-8543	\N	\N	\N	www.operationbootstrap-lynn.org	public_agency_email@testing123.net	Our mission is to address the educational needs of adults in the Greater Lynn area by providing high quality services.	Monday thru Thursday 8:00 am - 8:00 pm\nFriday 8:00 am - 12:00 pm	\N	\N	\N	\N	\N	\N	Don Philips Edwards	Executive Director	\N	Yes	Our resource library consists of materials to support the learning process for ESL, ABE, Citizenship and Literacy students.	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103553	2009-03-05	f	@lunch4all	Job Corps - Region One	Job Corps	181 Union Street	\N	Lynn	MA	01901	781-593-0585 ex 3016	\N	781-913-1056	617-727-3712	bostonregion.jobcorps.gov	public_agency_email@testing123.net	As a national, primarily residential training program, Job Corps' mission is to attract eligible young adults, teach them the skills they need to become employable and independent, and place them in meaningful jobs or further education.	24 hours a day, 365 days a year.  Career training takes place Monday through Friday from 8:00 a.m. to 4:00 p.m.	\N	\N	\N	\N	\N	\N	Carol A. Couch	Admissions Counselor	Job Corps, Department of Labor, Del-Jen	Yes	Staff and students only; not open to the public.	Yes	Conference Rooms	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103575	2009-03-10	f	@lunch4all	Beverly Affordable Housing Coalition	BAHC	234 Cabot Street, Suite 8	\N	Beverly	MA	01915	978-921-4705	\N	\N	978-922-2019	www.beverlyaffordablehousing.org	public_agency_email@testing123.net	The mission of the Beverly Affordable Housing Coalition is to increase the availability of quality affordable housing for low- to moderate-income families and individuals in Beverly.	Monday to Friday 9:00 to 5:00	\N	\N	\N	\N	\N	\N	Mickey Northcutt	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103542	2009-05-13	f	@lunch4all	Children's Law Center of Massachusetts	\N	298 Union Street	\N	Lynn	MA	01901	781-581-1977	\N	\N	\N	www.clcm.org	public_agency_email@testing123.net	The mission of the Children's Law Center of Massachusetts is to promote and secure equal justice and to maximize opportunity for low income children and youth by providing quality advocacy and legal services.	Monday to Friday\n9am to 5pm	\N	\N	\N	\N	\N	\N	Jay McManus	Executive Director	\N	No	\N	No	\N	No	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103586	2009-03-16	f	@lunch4all	Action, Inc.	\N	180 Main Street	\N	Gloucester	MA	01930	978-282-1000	\N	\N	\N	www.actioninc.org	public_agency_email@testing123.net	Action's mission is to improve the quality of life for low-income members of our community by minimizing the effects of poverty, promoting economic security and advocating for social change.  Founded in 1965 under the Office of Economic Opportunity, today Action continues to:\n- Establish access to opportunities through advocacy, a wide range of services and education and training opportunities to assist people in their process of self-help;\r\n- Mobilize resources, public and private, in order to reduce poverty; and\r\n\r\n- Develop and administer programs with the maximum feasible participation of the people served.\r\n\r\nAction's approach is comprehensive, addressing all aspects of a person's well being. Services are coordinated with colleague agencies to avoid duplication and enhance benefits and delivered with respect to promote self-esteem, empowerment and confidence.	Monday - Friday\n9a - 5p	\N	\N	\N	\N	\N	\N	William Rochford	Executive Director	\N	No	\N	Yes	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103916	2009-06-09	f	@lunch4all	Catholic Charities North-Lynn	\N	55 Lynn Shore Drive	\N	Lynn	MA	01902	781-593-2312	\N	\N	781-581-3270	www.ccab.org	public_agency_email@testing123.net	Catholic Charities is building a just and compassionate society rooted in the dignity of all people.	8:30 am - 5:00 pm, Monday through Friday	\N	\N	\N	\N	\N	\N	Virginia A. Doocy	Director	Catholic Charities Archdiocese of Boston	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103597	2009-03-17	f	@lunch4all	New American Center	Russian Community Association of MA	298 Union St	Lower Level	Lynn	MA	01902	781-593-0100	\N	\N	\N	\N	public_agency_email@testing123.net	The New American Center provides resettlement and post-resettlement services to refugees residing on the North Shore.  As a collaboration of seven community-based agencies, ESL, employment, case management, and housing services are offered for teenagers and adults.\nRefugee School Impact (RSI) is an after-school based program, and also includes advocacy in the Lynn Schools and community, cultural activities, and provides interpreters for parents and teachers.  Involved students can request college visits, help with employment, and other services on an individual basis.  	Monday and Wednesday, Middle School Homework 3 to 5 pm\nTuesday and Thursday, High School Homework 3 to 5 pm\r\nSaturday, Sports Club, 12 to 2 pm	\N	\N	\N	\N	\N	\N	Amanda Verbeke	Program Coordinator	\N	No	\N	Yes	Three full classrooms, one computer one, and a break room are available for the program's use.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103619	2009-03-20	f	@lunch4all	EOC	Educational Opportunity Center	"T" Building	Corner of Market and Broad Streets	Lynn	MA	02420	781-477-2114	\N	\N	781-477-2112	www.lynneoc.org	public_agency_email@testing123.net	Its mission is to help low-income, first generation potential college or technical school students gain access to higher education.\nStudents receive assistance with admission, financial aid and transfer procedures, academic and career advising, and general information about the educational process.  	Monday through Friday 8-4\nClosed between 12 and 12:30\r\nClosed on State and Federal holidays	All services are free of charge\nEOC is one of the Federally funded "TRIO" Programs.  It is hosted by North Shore Community College and adminstered by the Colleges of the Worcester Consortium.	EOC @ NSCC	PO Box 3340 - One Ferncroft Road	Danvers	MA	01923	Patricia Wolfe	Director	North Shore Community College and Colleges Worcester Consort	Yes	Books on Scholarships, Financial and Careers	Yes	Use of Classroom space in the MBTA Building	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103630	2009-03-20	f	@lunch4all	EDUCATIONAL TALENT SEARCH PROGRAM	LYNN ETS	"T' Building	Corner of Market and Broad Streets	Lynn	MA	01905	781-477-2114	\N	\N	781-477-2112	\N	public_agency_email@testing123.net	Assist disadvantaged Lynn Youth to successfully complete high school and continue to college or trade school.  Also work with recent drop outs to help them access resources to attain GED and continue on  to college or trade school.	Monday - Fridays 8 to 4\nClosed 12-12:30	Three ETS Advisor provide services in each of the 3 main Lynn Public High Schools: Classical, English, and Voch Tech. Outreach to Middle Schools is provided	ETS @ North Shore Community College	PO Box 3340 - One Ferncroft Road	Danvers	MA	01923	Patricia Wolfe	Director	North Shore Community College	Yes	Books on Scholarships, Careers, Colleges, Financial Aid	Yes	Use of Classroom Space in the MBTA Building	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103663	2009-04-03	f	@lunch4all	Citizens for Adequate Housing, Inc.	Inn Between, Inn Transition, and Inn Homes	40 Washington Street	\N	Peabody	MA	01960	978-531-9775	\N	\N	978-977-9008	www.cahns.org	public_agency_email@testing123.net	Every family deserves a place to call home; to have shelter from the weather, to heal when they are sick, to share meals, continuity and a place to grow and flourish.  For 25 years Citizens for Adequate Housing, Inc. has worked toward ending homelessness, providing a temporary home for homeless families and permanent affordable housing to low income families.  Our mission calls us to support families with dignity and respect, as they live in either our emergency shelter, the Inn Between, or in our sober living transitional housing shelter, the Inn Transition; or in one of our affordable housing apartments offered through Inn Homes.\nOur immediate focus; family stabilization, the restoration of hope and dignity of each family, fostering self sufficiency through advocacy, education, life skills training, permanent housing and the hope for a lifetime without homelessness.  \r\n	Monday - Friday 9:00 a.m. - 4:00 p.m.	\N	\N	\N	\N	\N	\N	Nancy Crowder	Executive Director	Citizens for Adequate Housing, Inc.	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103641	2009-04-10	f	@lunch4all	Beverly Bootstraps Community Services, Inc.	Beverly Bootstraps	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_agency_email@testing123.net	To end hunger and its root causes.	General Office Hours\nMonday - Friday 9am-4pm	Thrift Shop\nLocation:\r\n198 Rantoul Street (diagonally across from the Post Office and behind Depot Cleaners).\r\nPhone: 978-921-4710\r\nHours:\r\n    Monday-Friday: 9:30am to 5:00pm\r\n    Saturday:  9:30am to 1:30pm\r\n\r\nEvery donation made to the Beverly Bootstraps Thrift Shop works to further the mission of Beverly Bootstraps Community Services. The proceeds from sales provide revenue to support Beverly Bootstraps programming.\r\n\r\nThe store also works to fulfill the Bootstraps' mission by giving a helping hand to the population that we serve. In an effort to help those without as many resources as others, we offer a discount of 50% on items priced over $2.00 to any customer who carries a Mass Health Card, a Neighborhood Health Network card, a food stamp card, or a Medicaid Card.\r\n\r\nOn a case-by-case basis through Beverly Bootstraps Community Outreach Programs, our outreach workers can also supply free vouchers for clothing, furniture and other items to victims of catastrophes. 	\N	\N	\N	\N	\N	Sue Gabriel	Executive Director	\N	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103674	2009-04-17	f	@lunch4all	One Family Scholars @ Wellspring House, Inc.	One Family, Inc.	28 Emerson Avenue	\N	Gloucester	MA	01930	978-283-1972	\N	\N	978-283-3206	www.onefamilyinc.org	public_agency_email@testing123.net	One Family is a not-for-profit organization devoted to ending family homelessness in Massachusetts...Higher education is the best permanent pathway out of poverty. The One Family Scholars (OFS) program provides scholarships for low-income and formerly homeless mothers to attend college. Selection is based on academic promise, leadership potential, demonstrated work ethic, and financial need...Scholars are graduating the program and moving into family sustaining careers...the average Scholar Graduate salary is $34,220 an almost 300% growth from pre-program earnings. Finally, a key indicator of self-sufficiency, is a reduced reliance on state and federal benefits...The benefits of the One Family Scholars program are immediate and also multi-generational: Scholar children are more motivated to succeed in school and are in fact improving their own school performance.	Monday through Friday, 10 a.m. to 6 p.m.	\N	\N	\N	\N	\N	\N	Gail Mountain	Site Coordinator	One Family, Inc.	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103696	2009-04-22	f	@lunch4all	Healing Abuse Working for Change	HAWC	27 Congress Street	\N	Salem	MA	01970	978-744-8552	800-547-1649	800-547-1649	978-745-6886	www.hawcdv.org	public_agency_email@testing123.net	The mission of HAWC is to create social change by taking action against personal and societal patterns of violence and oppression. HAWC provides services and support to victims of domestic abuse residing in the 23 cities and towns on the North Shore in order that they may make informed, independent decisions about their futures and access alternatives to living in a violent situation.	Monday-Friday 9 am to 5 pm. with 24 hour toll free hotline	\N	\N	\N	\N	\N	\N	Candace Waldron	Executive Director	\N	No	\N	Yes	Salem, Lynn, Gloucester, Beverly and Ipswich office locations	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103707	2009-04-27	f	@lunch4all	North Shore Health Project	\N	67 Middle Street	\N	Gloucester	MA	01930	978-283-0101	888-283-0101	\N	978-283-0101	www.healthproject.org	public_agency_email@testing123.net	The North Shore Health Project promotes wellness in all those living with HIV/AIDS or Hepatitis C by providing free holistic health care, support services, information and outreach in a safe, caring and confidential environment.\nThe Health Project offers complete wraparound services, beginning with free counseling and testing referrals, case management, holistic therapies (acupuncture, massage, reiki, energy healing, etc.), nutritional counseling, drop-in services with access to office equipment (fax, copier, computer), emergency funds, congregate meals, supported referrals for legal, housng, and other concerns, and new or gently used clothing. All services are free to clients.	Hours are Monday through Friday, 9:00 a.m. to 5:00 p.m. Support groups meet evenings and weekends.	Clients can work with one of two case managers and the Client Services Director, as well as two peer leaders. Clients are welcome to drop in and spend an hour or the entire day. No appointment is necessary.	\N	\N	\N	\N	\N	Susan Oleksiw	Executive Director	\N	Yes	Some materials are available on HIV/AIDS and Hepatitis C, brochures from other agencies, and reprints of basic health information. The Health Project can obtain packets of information tailored to specific concerns from the resource library at AIDS Action Committee, Boston.	Yes	The dining room holds about 30 people.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-283-0101	0	
103718	2009-04-27	f	@lunch4all	The Fund to Prevent Homelessness	The Fund	PO box 17	\N	Manchester	MA	01944	978-927-1561	\N	\N	\N	\N	public_agency_email@testing123.net	Help working families remain in their homes.  We serve families living in Beverly, Essex, Gloucester, Manchester, Rockport and Wenham.  We provide one time assistance with rent or mortgage payments.  Intake is done by Beverly Bootstraps.	M-F, 9am to 5pm	\N	\N	\N	\N	\N	\N	Emily Brown	President of the Board	\N	Not Applicable	\N	Yes	For intake, Beverly Bootstraps offices.  For Board meetings, members homes.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103729	2009-04-29	f	@lunch4all	SeniorCare, Inc.	SeniorCare	5 Blackburn Center	\N	Gloucester	MA	01930	978-281-1750	866-927-1050	978-281-3093	978-218-1753	www.seniorcareinc.org	public_agency_email@testing123.net	SeniorCare provides and coordinates services to elders, enabling them to live independently at home and remain part of their community.	Monday - Friday, 9:00 am to 5:00 pm	\N	\N	\N	\N	\N	\N	Kay Bierwiler	Executive Director	\N	No	\N	Yes	Handicapped accessible, first floor, parking, seats 60, wireless connection.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	978-282-1836	0	
103740	2009-04-29	f	@lunch4all	Salem YMCA	YMCA of the North Shore	One Sewall Street	\N	Salem	MA	01970	978-740-9622	\N	\N	978-740-9168	www.northshoreymca.org	public_agency_email@testing123.net	The YMCA of the North Shore is committed to the values of caring, honesty, respect and\nresponsibility. Our YMCA provides all children, adults, and families, regardless of income, with opportunities to develop a healthy spirit, mind and body.	Monday through Friday 5:00 AM -10:00 PM\nSaturday 6:00 AM - 7:00 PM\r\nSunday 7:00 AM - 6:00 PM	\N	\N	\N	\N	\N	\N	Debbie Amaral	Executive Director	YMCA of the North Shore	Yes	Developmental Assets	Yes	Rooms available for fee.	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103751	2009-04-29	f	@lunch4all	North Shore Community Health, Inc.	Gloucester, Peabody and Salem Family Health Centers	47 Congress St.	\N	Salem	MA	01970	978-744-8388	\N	\N	\N	www.nschi.org	public_agency_email@testing123.net	The mission of North Shore Community Health, Inc., which includes the Gloucester, Peabody and Salem Family Health Center, is to provide high quality health, behavioral health, and dental care to all individuals regardless of age, ethnicity, religion, gender, sexual orientation, country of origin, place of residence, or economic status.	Mon-Fri 8:30am to 5:00 pm.  Call for evening hours which vary by site.	\N	\N	\N	\N	\N	\N	Gloria Riley	Chief Operations Officer	North Shore Community Health, Inc.	Not Applicable	\N	No	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-744-1847	0	
103762	2009-04-29	f	@lunch4all	Gloucester Housing Authority	\N	259 Washington Street	\N	Gloucester	MA	01930	978-281-4770	\N	978-281-4770	978-281-6945	www.ghama.com	public_agency_email@testing123.net	To promote adequate and affordable housing, economic opportunity and a suitable living environment free from discrimination	Monday - Thursday, 8:00 a.m.-4:15 p.m. Friday, 8:00 a.m.-12:00 p.m.	\N	\N	\N	\N	\N	\N	William F. Dugan	Executive Director	\N	No	\N	Yes	Conference Room for about 20 people	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103652	2009-04-30	f	@lunch4all	Training Resources of America, Inc.	\N	10 Federal Street	\N	Salem	MA	01970	978-744-2417	\N	\N	978-745-5757	www.tra-inc.org	public_agency_email@testing123.net	Training Resources of America, Inc. is dedicated to providing educationally and economically disadvantaged youth and adults with the opportunity to acquire the basic education, work skills, life skills and support they need to become economically self-sufficient, lead fulfilling lives, and ensure a better future for themselves and their families.	Monday - Thursday 8:30 - 8:00\nFriday 8:30 - 5:00	\N	\N	\N	\N	\N	\N	Linnea Rego	Regional Manager	\N	Not Applicable	\N	Not Applicable	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	800-439-2370	0	
103784	2009-05-15	f	@lunch4all	Massachusetts Rehabilitation Commission	Mass. Rehab.	35 Congress St.	Building 2	Salem	MA	01970	978-745-8085	\N	\N	978-745-9063	www.mass.gov/mrc	public_agency_email@testing123.net	MRC promotes equality, empowerment and productive independence of individuals with disabilities. These goals are achieved through enhancing and encouraging personal choice and risk-taking for independence and employment.\nThe Massachusetts Rehabilitation Commission provides comprehensive services to people with disabilities that maximizes their quality of life and economic self-sufficiency in the community.\r\nMultiple programs in the MRC work together to help individuals with disabilities.  These programs are:\r\nThe Vocational Rehabilitation Services Program\r\nThe Community Services Program and;\r\nThe Disability Determination Services Program.\r\n	Monday - Friday 9:00 AM to 5:00 PM	MRC is made up of three main multiple service programs.\nThe Vocational Rehabilitation (VR) Program:\r\nThe Vocational Rehabilitation Program assists people who have a disabling condition who would like to find or return to work. VR also works closely with employers in the community to help create job openings and to help increase employer awareness regarding the benefits of diversity in the workplace.\r\nOffice of Community Services (CS):\r\nThe Office of Community Services (CS) offers:\r\nBrain Injury and Specialized Community Services  for individuals with an acquired/traumatic head injury\r\nProtective Services for persons with physical disabilities who may be abused by their caregiver\r\nSupported Living Services for persons with disabilities needing assistance to live independently at home\r\nIndependent Living Center Services providing information, referral, and peer counseling for persons with disabilities to live independently in the community\r\nIndependent Living Programs for individuals turning 22\r\nAssistive Technology Program to enable individuals with severe disabilities equal access to employment and community life through the use of assistive technology and training.\r\nHome Care assistance for persons with disabilities ages 18-59.\r\nConsumer Involvement to improve the Commission's services by working cooperatively with members of the disability community on projects of mutual interest.\r\nDisability Determination Services (DDS): \r\nDisability Determination Services (DDS) is funded by the Social Security Administration (SSA) and determines the initial and continued eligibility for federal SSI and SSDI benefits. Special outreach efforts are made to homeless shelters and individuals with HIV.\r\nThere is a lot more to our agency than outlined above! Please continue your visit to find out about the many other programs the MRC offers. Almost every family in the Commonwealth has at least one person who might benefit from MRC services...especially if that individual is an employer looking to hire qualified workers.\r\n\r\nThe MRC can be contacted in a variety of ways:\r\n\r\nThe Vocational Rehabilitation Program has 25 Area Offices administered by three regional offices.\r\nSupported Employment Services is also part of the VR program and is located at the MRC Administrative Office at 27 Wormwood Street in South Boston.\r\nThe Community Services Program has numerous services including contracted services with eleven Independent Living centers across the Commonwealth.\r\nDisability Determination Services has two locations: Worcester and Boston.\r\nThe Commissioner of the MRC is Charles Carr. Find out more about the Commissioner and his staff in Senior Management Profiles.\r\n\r\n\r\n\r\n	\N	\N	\N	\N	\N	Robert Coughlin	Area Manager	Office of Health and Human Services (EOHHS)	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-745-8085	0	
103795	2009-05-15	f	@lunch4all	City of Peabody	\N	24 Lowell Street	\N	Peabody	MA	01960	978-538-5900	\N	\N	\N	www.peabody-ma.gov	public_agency_email@testing123.net	To serve the citizens of the City of Peabody.	Mon - Wed 8:30 - 4:00, Thur 8:30 - 7:00, Fri 8:30 - 12:30	\N	\N	\N	\N	\N	\N	Michael J. Bonfanti	Mayor	\N	Not Applicable	\N	Yes	Small conference room on the lower level, Wiggin Auditorium on 2nd floor	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103806	2009-05-16	f	@lunch4all	St. John's Church Thrift Shop	\N	19 Chestnut St	\N	Peabody	MA	01960	978-531-0002	\N	\N	\N	St. John the Baptist Church Peabody	public_agency_email@testing123.net	Provide clothing, shoes, household items at a very low price.  Items are free to those in need.	Thursdays 9-12:00\nFridays and Saturdays 9-1:00	\N	\N	\N	\N	\N	\N	Jane Harris	Manager	St. John the Baptist Church	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103773	2009-05-18	f	@lunch4all	Pioneer House	(we are also a program of Health & Education Services, Inc.)	2 Washington Street	\N	Peabody	MA	01960	978-977-7755	\N	\N	978-538-7213	hes-inc.org (follow the links to Pioneer House)	public_agency_email@testing123.net	To provide the opportunity for individuals with mental health challenges to be in an environment which enables them to build the confidence, self-esteem, and skills necessary for vocationally productive and socially satisfying lives.	Monday through Friday from 9am to 5pm. Also every other Saturday from Noon to 5pm, and holidays from Noon to 4pm.	\N	\N	\N	\N	\N	\N	Larry Marshall	Director	Health & Education Services, Inc., 131 Rantoul St, Beverly	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103817	2009-05-20	f	@lunch4all	Health & Education Services, Inc.	HES	131 Rantoul Street	\N	Beverly	MA	01915	978-921-1293	\N	\N	978-921-1294	www.hes-inc.org	public_agency_email@testing123.net	Health & Education Services, Inc. promotes the well - being that builds healthy individuals, families, and communities through the integration of quality health, behavioral health, education, and human service programs.\nWe are a unique, community-based network of prevention, intervention, and treatment services. We provide a comprehensive continuum of care for all, especially those whose needs are otherwise unmet.\r\n	24 hours a day, 7 days a week.	\N	\N	\N	\N	\N	\N	Paul C. O'Shea	President & CEO	Northeast Health Systems	Not Applicable	\N	Yes	Several small confrence rooms	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103828	2009-05-26	f	@lunch4all	Transitions	Lynn Shelter Association	149 Washington St Peabody Ma	91 Liberty St Lynn Ma	Peabody	MA	01969	978-538-5392	\N	\N	978-538-5490	lsahome.org	public_agency_email@testing123.net	LSA/TRansitions mission is to provide shelter and support services to homeless individuals in a manner that preserves their dignity and empowers them to work towards self sufficiency	24hrs ..\nmon/fri adming\r\nlimited weekend coverage. 	\N	149 Washington St	\N	Peabody	MA	01960	Donna Cahill	Program Director	LSA/HOME	Not Applicable	\N	Yes	LIMITED to 15 persons	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103839	2009-05-27	f	@lunch4all	Making A Difference In Peabody Foundation, Inc	\N	24 Main Street	Suite 2	Peabody	MA	01960	978-317-4388	\N	\N	978-854-0060	\N	public_agency_email@testing123.net	To assist Peabody residents with emergency needs such as oil, rent, electricity, food, clothing,and emergency disaster relief.	9-5 M-F	\N	\N	\N	\N	\N	\N	Scott Frasca	President	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103861	2009-06-01	f	@lunch4all	Neighborhood Legal Services, Inc.	NLS	37 Friend Street	Third Floor	Lynn	MA	01902	781-599-7730	800-747-5056	\N	781-595-2022	www.neighborhoodlaw.org	public_agency_email@testing123.net	NLS provides free civil legal services to low income and elderly clients in Essex and Northern Middlesex Counties.  NLSs organizational goals are to increase economic opportunity, safety, and family stability, and to promote fairness and equity for the areas low-income households.	Monday through Friday, 9 a.m. to 5 p.m.	\N	\N	\N	\N	\N	\N	Sheila C. Casey	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	781-599-7730	0	
103872	2009-06-02	f	@lunch4all	Citizens for Adequate Housing Inc.	Inn Between-& Inn Transition	40 Washington St.	25 Holten St. & 42 Washington St.	Peabody	MA	01970	978-531-9775	\N	978-531-9951	978-977-9008	www.cahns.org	public_agency_email@testing123.net	To provide shelter, housing and skills to work your way out of poverty.	24-7	\N	\N	\N	\N	\N	\N	Nancy Crowder	Executive Director	CAH	Not Applicable	\N	Yes	small conference room	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	978-531-9775	0	
103883	2009-06-08	f	@lunch4all	Health Quarters	\N	19 Broadway	\N	Beverly	MA	01915	978-927-9824	800-892-0234	\N	978-922-5904	www.healthq.org	public_agency_email@testing123.net	Our mission is to protect reproductive health, build healthy communities where every pregnancy is intended, and improve the health of all who seek our services.\nWe provide quality, sensitive, and affordable clinical care, education and advocacy.  \r\nWe believe everyone should have access to confidential reproductive health care and the opportunity to make informed choices.  \r\nIn all aspects of our work we encourage relationships that are respectful and that value sexuality as an integral part of life.\r\n	M,W-F  8:30 - 4:30, TH 11-7	\N	\N	\N	\N	\N	\N	Lianne Cook	Executive Director	\N	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	800-892-0234	0	
103894	2009-06-08	f	@lunch4all	North Shore Career Center - Salem / Lynn / Gloucester	\N	70 Washington Street/ 181 Union Street / 11-15 Parker Street	\N	Salem/ Lynn / Gloucester	MA	01970	978-825-7200	\N	\N	617-727-5989	www.nscareers.org	public_agency_email@testing123.net	Our mission is to maintain a strong local workforce development system, for nineteen targeted cities & towns, by working in concert with area businesses, community organizations, and the local Workforce Investment Board.\nOur goal is to help our customers remain competitive in todays workforce, by providing access to career & academic information, training opportunities, and current innovative technology.\r\n	Monday-Wednesday:  8:30 a.m. - 5:00 p.m.\nThursday:   8:30 a.m. - 7:00 p.m.\r\nFriday:  9:30 a.m. - 5:00 p.m.\r\n	\N	\N	\N	\N	\N	\N	Mark Whitmore	Executive Director	North Shore Workforce Investment Board	Not Applicable	\N	Yes	1-Large Conference Room\n1-Small Conference Room	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103905	2009-06-08	f	@lunch4all	The Salem Mission, Inc.	\N	56 Margin Street	\N	Salem	MA	01970	978-744-0500	\N	\N	978-740-3832	salemmission.org	public_agency_email@testing123.net	The Salem Mission provides housing and supportive services to adult men and women from the North Shore. We are dedicated to ending homelessness on the North Shore.	24/7	\N	\N	\N	\N	\N	\N	Mark A. Cote, M.Ed	Executive Director	\N	Not Applicable	\N	Yes	Confernece room that will seat 8-10 people	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	No	\N	0	
103685	2009-06-09	f	@lunch4all	Catholic Charities North - Gloucester	\N	74 Pleasant Street	\N	Gloucester	MA	01930	781-593-2312	\N	\N	617-832-7486	\N	public_agency_email@testing123.net	Catholic Charities is building a just and compassikonate society rooted in the dignity of all people.	Monday - 12 noon - 4 p.m.\nTuesday and Wednesday - 10 a.m. - 4 p.m.\r\nThursday 9 a.m.- 1 p.m.\r\nClosed Fridays.  Hours may vary on some days.	\N	60 Prospect Street	\N	Gloucester	MA	01930	Sarah J. Gindel	Basic Needs Coordinator	Catholic Charities of Boston	No	\N	No	\N	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103927	2009-06-10	f	@lunch4all	Cape Ann Art Haven	Art Haven	180B Main Street	\N	Gloucester	MA	01930	978-283-3888	\N	\N	\N	www.arthaven.org	public_agency_email@testing123.net	Cape Ann Art Haven seeks to address the need for artistic development in children of the Cape Ann community. We aspire to create a studio where kids will be encouraged to discover their creative ability through a variety of art\nmediums, in hopes to build character and self confidence. \r\nThe studio will serve as a headquarters for the arts, by promoting local artists, as well as studios, and galleries all over Cape Ann. The Studio will be a gathering place for artists of all ages and mediums as well as the community at large. \r\n	We currently do afterschool classes from 3:30-7:00 every day and are open all day on Fridays for a high school class and Family Studio time.	\N	\N	\N	\N	\N	\N	Dawn Gadow	Executive Director	\N	Not Applicable	\N	Not Applicable	\N	Limited	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
103850	2009-06-10	f	@lunch4all	Peabody Institute Library	\N	82 Main Street	\N	Peabody	MA	01960	978-531-0100	\N	\N	978-532-1797	www.peabodylibrary.org	public_agency_email@testing123.net	The mission of the Peabody Institute Library is to meet the educational, informational and recreational needs of the community.	Monday - Thursday\n9 a.m.-9 p.m.\r\n(Children's Room 9 a.m.-8 p.m.)\r\nFriday\r\n9 a.m.-1 p.m.\r\nSaturday\r\n9 a.m.-5 p.m.\r\nSunday\r\n1-5 p.m.\r\nClosed Sundays Memorial Day through Labor Day 	\N	\N	\N	\N	\N	\N	Martha Holden	Library Director	City of Peabody	Yes	The Peabody Library has large collection of books, DVDs and CDs.	Yes	The Peabody Library has several meeting rooms available for use by nonprofit or governmental agencies.  There is no fee for meeting room use. For complete procedures and to book a room call the library at 978-531-0100 ext. 11	Yes	Chester	Tester	111-111-1111	000-000-0000	chester@testing123.net	Yes	\N	0	
\.


--
-- Data for Name: agency_search_log; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY agency_search_log (search_id, session_id, ip_address, "timestamp", search_phrase, referer, hit_count) FROM stdin;
36542	66.80.237.78.18111109	66.80.237.78	2009-02-11 18:44:47.258492	nscap	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	1
36544	66.80.237.78.10121109	66.80.237.78	2009-02-12 10:08:06.583349	nscap	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36546	74.10.197.220.9141109	74.10.197.220	2009-02-14 09:29:12.757537	North shore community action program	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	1
36558	66.92.77.160.6261109	66.92.77.160	2009-02-26 06:02:07.529782	nscap	http://northshoreport.org/	1
36561	75.144.137.57.8261109	75.144.137.57	2009-02-26 08:50:01.602167	nscap	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=509526&search_phrase=shelter	1
36567	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:04:30.480871	north	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36570	155.212.138.182.11261109	155.212.138.182	2009-02-26 11:29:38.263896	my turn	http://northshoreport.org/	0
36576	64.206.127.11.11271109	64.206.127.11	2009-02-27 11:49:22.29741	children	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36577	66.92.77.160.19271109	66.92.77.160	2009-02-27 19:33:13.606274	law	http://northshoreport.org/	1
36581	205.188.116.10.15281109	205.188.116.10	2009-02-28 15:17:13.577537	My turn	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36582	205.188.116.10.15281109	205.188.116.10	2009-02-28 15:17:20.482974	Girls Inc	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36586	209.235.0.249.1132109	209.235.0.249	2009-03-03 11:18:58.649736	children	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36588	205.237.102.178.1632109	205.237.102.178	2009-03-03 16:22:03.87067	Catholic Charities North	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36592	155.212.138.182.1252109	155.212.138.182	2009-03-05 12:07:15.491485	my turn	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801452;search_phrase=youth	0
36593	98.217.245.77.1852109	98.217.245.77	2009-03-05 18:32:41.942266	Job Corps	http://northshoreport.org/	1
36595	72.85.251.119.1162109	72.85.251.119	2009-03-06 11:20:48.101949	Compass	http://northshoreport.org/	0
36602	72.93.187.82.11102109	72.93.187.82	2009-03-10 11:03:46.633068	Beverly Affordable	http://northshoreport.org/	0
36606	75.144.136.113.13132109	75.144.136.113	2009-03-13 13:19:49.619395	WIB	http://www.northshoreport.org/	0
36616	205.237.102.178.17192109	205.237.102.178	2009-03-19 17:53:17.847742	cope	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801782;search_phrase=counseling%20services	0
36618	24.61.37.4.22192109	24.61.37.4	2009-03-19 22:28:41.671276	Salem CyberSpace	http://www.northshoreport.org/	0
36621	24.61.37.4.22192109	24.61.37.4	2009-03-19 22:29:40.342467	NSCAP	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	1
36622	70.88.192.1.9202109	70.88.192.1	2009-03-20 09:32:38.643205	EOC	http://www.northshoreport.org/	0
36623	70.88.192.1.9202109	70.88.192.1	2009-03-20 09:32:53.150961	ETS	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36624	70.88.192.1.9202109	70.88.192.1	2009-03-20 09:33:05.425249	Educational Opportunity Center	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36625	155.212.138.182.10202109	155.212.138.182	2009-03-20 10:55:11.305831	my turn	http://www.northshoreport.org/	0
36631	68.166.124.194.8252109	68.166.124.194	2009-03-25 08:58:12.5896	LWIB	http://www.northshoreport.org/	0
36632	205.237.102.178.18252109	205.237.102.178	2009-03-25 18:01:50.443365	catholic charities	http://www.northshoreport.org/	0
36633	205.237.102.178.18252109	205.237.102.178	2009-03-25 18:02:09.105129	nscap	http://www.northshoreport.org/	1
36635	75.144.136.113.9262109	75.144.136.113	2009-03-26 09:33:05.264998	North Shore WIB	http://www.northshoreport.org/	0
36636	75.144.136.113.9262109	75.144.136.113	2009-03-26 09:33:13.636263	North Shore Career Center	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36639	205.237.102.178.13262109	205.237.102.178	2009-03-26 13:32:44.835837	catholic charities	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36640	205.237.102.178.13262109	205.237.102.178	2009-03-26 13:33:03.475263	beverly bootstraps	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36641	205.237.102.178.13262109	205.237.102.178	2009-03-26 13:36:41.080246	health and ed services	http://northshoreport.org/	0
36642	205.237.102.178.13262109	205.237.102.178	2009-03-26 13:38:21.916654	north shore community action program	http://northshoreport.org/	1
36644	72.85.251.119.10312109	72.85.251.119	2009-03-31 10:05:01.725799	COMPASS	http://northshoreport.org/	0
36646	72.85.251.119.10312109	72.85.251.119	2009-03-31 10:05:25.581417	Action, Inc	http://northshoreport.org/cgi-bin/mvhub/guide.pl	3
36652	72.72.115.114.823109	72.72.115.114	2009-04-02 08:13:54.971948	NSCAP	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	1
36654	76.19.69.90.1493109	76.19.69.90	2009-04-09 14:37:58.399855	citizens for adequate housing	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results&category_id=802002	1
36656	72.70.121.90.17153109	72.70.121.90	2009-04-15 17:39:05.216428	on the threshold	http://northshoreport.org/	0
36657	72.70.121.90.17153109	72.70.121.90	2009-04-15 17:39:13.425559	on the threshold fund	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36662	129.63.96.191.15173109	129.63.96.191	2009-04-17 15:35:09.438563	boot	http://northshoreport.org/cgi-bin/mvhub/guide.pl	2
36665	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:28:28.320159	beverly bootstraps	http://northshoreport.org/	1
36672	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:32:22.811178	aids	http://northshoreport.org/	0
36677	72.93.196.33.16223109	72.93.196.33	2009-04-22 16:21:47.065257	Beverly Bootstraps	http://northshoreport.org/	1
36678	72.93.196.33.16223109	72.93.196.33	2009-04-22 16:25:05.318843	beverly	http://www.northshoreport.org/	2
36686	76.118.140.125.13253109	76.118.140.125	2009-04-25 13:48:14.206945	Beverly	http://northshoreport.org/	2
36688	66.238.215.240.8283109	66.238.215.240	2009-04-28 08:21:01.018555	Gloucester housing authority	http://www.northshoreport.org/	0
36691	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:23:31.966577	nscap	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36698	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:39:41.307175	housing	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803025;search_phrase=housing	3
36699	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:39:44.403388	wellspring	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36702	70.22.150.207.17303109	70.22.150.207	2009-04-30 17:46:39.202242	training resources of america	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36704	68.163.130.197.914109	68.163.130.197	2009-05-01 09:53:03.829649	north shore health project	http://www.northshoreport.org/	1
36714	151.203.74.189.1144109	151.203.74.189	2009-05-04 11:38:48.519225	North Shore Health Project	http://northshoreport.org/	1
36717	151.203.74.189.1344109	151.203.74.189	2009-05-04 13:19:03.487918	nscap	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36718	151.203.74.189.1344109	151.203.74.189	2009-05-04 13:28:50.746135	bootstraps	http://northshoreport.org/	1
36719	64.61.49.154.1564109	64.61.49.154	2009-05-06 15:56:30.056642	senior	http://northshoreport.org/	1
36720	64.61.49.154.1664109	64.61.49.154	2009-05-06 16:08:53.779441	SeniorCare	http://northshoreport.org/	1
36723	66.80.237.78.1474109	66.80.237.78	2009-05-07 14:49:38.242001	St. John	http://www.northshoreport.org/	0
36724	66.80.237.78.1474109	66.80.237.78	2009-05-07 14:49:44.91085	NSCAP	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	1
36740	66.80.237.78.14124109	66.80.237.78	2009-05-12 14:27:58.804323	NSCAO	http://www.northshoreport.org	0
36741	66.80.237.78.14124109	66.80.237.78	2009-05-12 14:28:05.917372	NSCAP	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	1
36742	24.91.123.153.15134109	24.91.123.153	2009-05-13 15:07:30.608525	immigration	http://northshoreport.org/	0
36743	24.91.123.153.15134109	24.91.123.153	2009-05-13 15:07:38.318173	inmigration	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36749	204.58.32.254.9154109	204.58.32.254	2009-05-15 09:22:39.759764	Essex District Attorney's Office	http://www.northshoreport.org/	0
36751	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:29:17.063289	st. vincent de paul socity	http://www.northshoreport.org/	0
36752	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:29:36.542658	st. john the baptist church	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36753	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:29:47.911812	nascap	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36754	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:30:41.593404	woman's concern	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36755	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:31:00.01282	catholic charities	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	1
36756	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:32:15.613211	salvation army	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=103685&search_phrase=catholic%20charities	0
36757	76.19.69.255.13194109	76.19.69.255	2009-05-19 13:06:03.96151	Head Start	http://www.northshoreport.org/	0
36761	216.36.102.173.14204109	216.36.102.173	2009-05-20 14:58:27.925557	HES	http://www.northshoreport.org/	1
36764	129.63.96.180.20224109	129.63.96.180	2009-05-22 20:17:35.095173	utec	http://northshoreport.org/	0
36765	129.63.96.180.20224109	129.63.96.180	2009-05-22 20:17:36.018402	nscap	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36766	129.63.96.180.20224109	129.63.96.180	2009-05-22 20:17:36.926593	dsfkjd	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36769	129.63.96.180.21224109	129.63.96.180	2009-05-22 21:22:48.120825	utec	http://nsp.production.testing123.net/	0
36770	129.63.96.180.21224109	129.63.96.180	2009-05-22 21:22:49.02561	nscap	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	1
36771	129.63.96.180.21224109	129.63.96.180	2009-05-22 21:22:49.952494	dsfkjd	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	0
36774	129.63.96.180.13284109	129.63.96.180	2009-05-28 13:07:32.205066	utec	http://nsp.production.testing123.net/	0
36775	129.63.96.180.13284109	129.63.96.180	2009-05-28 13:07:33.151574	nscap	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	1
36776	129.63.96.180.13284109	129.63.96.180	2009-05-28 13:07:34.075777	dsfkjd	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	0
36779	74.10.197.220.17284109	74.10.197.220	2009-05-28 17:03:18.223292	Peabody Institute Library	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	1
36780	74.10.197.220.17284109	74.10.197.220	2009-05-28 17:03:24.120379	Peabody Library	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	1
36782	129.63.96.180.13294109	129.63.96.180	2009-05-29 13:57:05.647851	utec	http://nsp.production.testing123.net/	0
36783	129.63.96.180.13294109	129.63.96.180	2009-05-29 13:57:06.615653	nscap	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	1
36784	129.63.96.180.13294109	129.63.96.180	2009-05-29 13:57:07.520256	dsfkjd	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	0
36792	170.63.90.76.1335109	170.63.90.76	2009-06-03 13:51:35.244935	YMCA	http://www.northshoreport.org/	1
36793	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:48:03.825668	boys and girls club of greater lynn	http://www.northshoreport.org/	0
36794	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:48:25.313682	boys and girls club	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36795	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:54:32.970695	bridgewell	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36798	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:56:47.806048	catholic charities north	http://www.northshoreport.org/	1
36799	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:57:18.487672	cerebral palsy of eastern ma	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36800	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:57:50.4655	girls Inc. Odyssey Program	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36801	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:58:05.026322	girls Inc. school age child care	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36802	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:58:13.570728	gregg house	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36803	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:58:29.017429	gregg house- infant toddler center	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36804	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:58:38.945243	joi	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36805	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:58:47.741368	joi childcare centers	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36806	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:59:05.14963	lynn economic opportunity	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36807	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:59:19.384005	lynn housing authority	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36808	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:59:34.024184	north shore family daycare	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36809	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:59:44.112015	RAW	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36810	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:59:56.80104	salvation army	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36811	72.85.251.119.1045109	72.85.251.119	2009-06-04 10:50:08.887723	art haven	http://www.northshoreport.org/	0
36823	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:25:38.99028	NSCAP	http://northshoreport.org/	1
36827	173.48.48.8.1155109	173.48.48.8	2009-06-05 11:01:28.34346	Lynn	http://www.northshoreport.org/	2
36829	216.41.1.226.1555109	216.41.1.226	2009-06-05 15:14:21.183684	health quarters	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36830	205.237.102.178.1385109	205.237.102.178	2009-06-08 13:19:36.728281	catholic	http://northshoreport.org/	1
36831	129.63.96.191.1385109	129.63.96.191	2009-06-08 13:20:54.937249	catholic	http://northshoreport.org/	1
36832	205.237.102.178.1485109	205.237.102.178	2009-06-08 14:33:30.008213	catholic	http://www.northshoreport.org/	1
36835	205.237.102.178.1585109	205.237.102.178	2009-06-08 15:00:15.317361	catholic	http://www.northshoreport.org/	1
36836	65.217.255.2.1585109	65.217.255.2	2009-06-08 15:38:30.519839	n	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results&category_id=801452	8
36838	75.144.136.113.1685109	75.144.136.113	2009-06-08 16:08:31.391362	North Shore WIB	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=Education%2FTraining	0
36839	75.144.136.113.1685109	75.144.136.113	2009-06-08 16:08:43.027772	North Shore Workforce Investment Board	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36842	75.147.0.53.1685109	75.147.0.53	2009-06-08 16:18:23.459189	in between health insurance coverage need medication right away	http://northshoreport.org/	0
36847	75.144.136.113.1785109	75.144.136.113	2009-06-08 17:04:03.731982	North Shore Workforce Investment Board	http://northshoreport.org/	0
36848	75.144.136.113.1785109	75.144.136.113	2009-06-08 17:04:09.953328	WIB	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36849	75.144.136.113.1785109	75.144.136.113	2009-06-08 17:04:19.706645	North Shore Career Center	http://northshoreport.org/cgi-bin/mvhub/guide.pl	1
36857	205.237.102.178.895109	205.237.102.178	2009-06-09 08:50:57.423976	catholic	http://www.northshoreport.org/	1
36858	68.162.222.204.995109	68.162.222.204	2009-06-09 09:01:15.237286	Independent living center	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	0
36859	68.162.222.204.995109	68.162.222.204	2009-06-09 09:01:30.686267	Independent living center of the North shore and cape ann	http://northshoreport.org/cgi-bin/mvhub/guide.pl	0
36861	205.237.102.178.995109	205.237.102.178	2009-06-09 09:42:37.826633	catholic	http://www.northshoreport.org/	1
36863	205.237.102.178.1095109	205.237.102.178	2009-06-09 10:23:02.545218	catholic	http://www.northshoreport.org/	1
36865	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:20:09.934558	gay	http://northshoreport.org/	0
36871	205.237.102.178.1395109	205.237.102.178	2009-06-09 13:41:17.832351	carholic	http://www.northshoreport.org/	0
36872	205.237.102.178.1395109	205.237.102.178	2009-06-09 13:41:30.156427	catholic	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	2
36876	75.144.185.129.1495109	75.144.185.129	2009-06-09 14:48:32.593378	north shore elder services	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results&category_id=803960	0
36877	70.22.224.196.1895109	70.22.224.196	2009-06-09 18:19:51.897969	Triangle	http://northshoreport.org/	0
36880	70.22.150.207.12105109	70.22.150.207	2009-06-10 12:04:01.076281	training resources of america	http://www.northshoreport.org/	1
36881	173.9.9.97.14105109	173.9.9.97	2009-06-10 14:42:17.958662	HAWC	http://www.northshoreport.org/	1
36885	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:53:21.15046	esol	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=103872	0
36887	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:54:10.379479	church	http://www.northshoreport.org/	1
36888	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:56:55.247558	on the thresh	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	0
36889	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:57:01.212573	on the threshold	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36890	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:57:11.528607	on the threshhold	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36891	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:57:40.761996	threshhold	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36892	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:57:46.174929	threshold	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	0
36894	74.10.197.220.15105109	74.10.197.220	2009-06-10 15:05:15.270904	Peabody	http://www.northshoreport.org/	4
36896	74.10.197.220.15105109	74.10.197.220	2009-06-10 15:23:26.045044	library	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=510329&search_phrase=library	1
36897	74.10.197.220.15105109	74.10.197.220	2009-06-10 15:24:36.714086	peabody	http://www.northshoreport.org/	4
36899	129.63.96.180.15148109	129.63.96.180	2009-09-14 15:05:16.271929	utec	http://nsp.mansi.testing123.net/	0
36900	129.63.96.180.15148109	129.63.96.180	2009-09-14 15:05:17.367719	nscap	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	1
36901	129.63.96.180.15148109	129.63.96.180	2009-09-14 15:05:18.466903	dsfkjd	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	0
36904	129.63.96.180.9158109	129.63.96.180	2009-09-15 09:20:16.978499	utec	http://nsp.mansi.testing123.net/	0
36905	129.63.96.180.9158109	129.63.96.180	2009-09-15 09:20:18.167528	nscap	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	1
36906	129.63.96.180.9158109	129.63.96.180	2009-09-15 09:20:19.263991	dsfkjd	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	0
\.


--
-- Data for Name: alias; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY alias (category_id, alias_name, stems_name) FROM stdin;
800000	History	histori
800121	Code Violations	code violat
800132	Trash	trash
800132	Sanitation Inspection	sanit inspect
800198	Volunteering	volunt
800286	Neighborhood Organizations	organ neighborhood
800297	Community Access	access commun
800297	Public Access	access public
800308	Computer, Open Access	open access comput
800396	Counseling Bereavement, Counseling Grief	counsel bereav grief
800407	Support Group, Death	death group support
800473	Couples Counseling	counsel coupl
800539	Hotlines, Battered Woman	hotlin batter woman
800539	Crisis Lines, Domestic Violence	crisi domest line violenc
800539	Crisis Lines Battered Woman	crisi batter woman line
800561	Hotlines, Battered	hotlin batter
800561	Crisis Lines, Child Abuse	crisi child abus line
800561	Crisis Lines, Battered Children	children crisi batter line
800594	Advocacy, Battered Women	advocaci batter women
800605	Counseling, Battered Children	counsel children batter
800616	Battered Women	batter women
800616	Domestic Violence	domest violenc
800627	Counseling, Battered Women	counsel batter women
800638	Hotlines, Sexual Assault	hotlin sexual assault
800638	Crisis Lines Rape	crisi rape line
800638	Crisis Lines Sexual Assault	crisi sexual assault line
800649	Abuse, Elderly	elderli abus
800660	Domestic Violence	domest violenc
800671	Advocacy, Battered Children	children advocaci batter
800693	Battered Children Prevention	prevent children batter
800715	Information and Referral, Battered Children	children inform batter and referr
800726	Information and Referral, Sexual Assault	inform sexual and assault referr
800759	Information and Referral, Battered Woman	inform batter and referr woman
800770	Counseling Sexual Assault	counsel sexual assault
800792	Victims' Advocacy Rape	advocaci rape victim
800792	Victims' Advocacy Sexual Assault	advocaci sexual assault victim
800803	Sexual Assault	sexual assault
800517	Support Group Rape	group rape support
800869	Advocacy Mental Illness	ill advocaci mental
800880	Halfway House, Disabled	disabl halfwai hous
800935	ADD	add
800979	Blind	blind
801023	Screen Reader/Magnification Software	softwar reader screen magnif
801034	SSI	ssi
801034	SSDI	ssdi
801056	Deaf	deaf
801067	Special Education	special educ
801078	Developmental Disability	development disabl
801078	Mental Disability	mental disabl
801111	Adaptive Aids/Equipment	equip aid adapt
801188	Education Grade School	grade educ school
801210	Skipping School	school skip
801210	Playing Hooky	hooki plai
801243	English As a Second Language	languag english a second
801243	English for Speakers of Other Foreign Languages	english languag speaker foreign other
801254	Advocacy, Special Education	special educ advocaci
801265	Tutoring Junior High School	school junior high tutor
801342	Homework Assistance Junior High School	homework school junior assist high
801386	Education, Junior High School	educ school junior high
800913	Special Education	special educ
801485	Tutoring Grade School	grade school tutor
801529	Homework Assistance Grade School	grade homework school assist
801540	Information and Referral, Special Education	special inform educ and referr
801551	Hotlines, Special Education	special educ hotlin
801584	ABE	abe
801672	FMLA	fmla
801694	Employment Discrimination Assistance	assist discrimin employ
801749	Unemployment Disputes	disput unemploy
801749	Unemployment Rights	right unemploy
801749	Legal Assistance, Unemployment	legal unemploy assist
801771	Job Search Support Groups	search group job support
801826	Job Prep and Readiness Training	train readi and prep job
801837	Business Training	train busi
801892	Advocacy, Debt	debt advocaci
801903	Tax Assistance	assist tax
801903	VITA	vita
801914	Information and Referral, Paying for College	inform colleg pai and referr
801925	Debt Management	debt manag
801925	Cleaning Up Credit	up credit clean
801947	Hotlines, Debt	debt hotlin
801958	Financial Advisor	advisor financi
801969	Budgeting	budget
801980	Information and Referral, Debt	inform debt and referr
801991	Paying for College	colleg pai
802002	Homeless, Families	homeless famili
802035	Food Program	food program
802035	Woman Infants and Children	infant children and woman
802046	Homeless, Women	homeless women
802079	Advocacy, TANF	advocaci tanf
802079	Advocacy, Welfare	advocaci welfar
802090	Homeless, Men	men homeless
802123	Information and Referral Food Stamps	inform food and referr stamp
802134	Information and Referral Homeless	inform homeless and referr
802156	Furniture	furnitur
802178	TANF	tanf
802178	Welfare	welfar
802189	Hotlines Homeless	homeless hotlin
802222	Phone Assistance	assist phone
802233	Information and Referral, TANF	inform and referr tanf
802233	Information and Referral AFDC	inform and referr afdc
802255	Advocacy Shelter	advocaci shelter
802288	Disability Insurance	insur disabl
802299	Eating Healthy	eat healthi
802343	Weight Control	weight control
802343	Weight loss	weight loss
802343	Over Weight	weight over
802343	Diet	diet
802343	Lose Weight	weight lose
802376	Hotlines, Free Medical Care	free hotlin care medic
802453	Information and Referral, Free Medical Care	free inform care and medic referr
802508	Nursing	nurs
802519	Shots	shot
802574	Health Care, Pediatric	care health pediatr
802640	Sexually Transmitted Diseases	diseas transmit sexual
802662	Nurses Assistant	assist nurs
802717	Birth Control	birth control
802750	Giving Blood	blood give
802805	Dentist	dentist
802849	Find Doctor	doctor find
802915	Hotlines, Homeless	homeless hotlin
802981	Rental Assistance	rental assist
803069	Information and Referral, Homeless	inform homeless and referr
803080	Homeless Shelter	homeless shelter
803102	Get the Lead Out	lead out get
803113	Advocacy, Homeless	homeless advocaci
803124	Hotlines, Refugees	refuge hotlin
803179	Advocacy, Refugees	refuge advocaci
803201	Refugee Rights	refuge right
803245	Information and Referral Refugees	refuge inform and referr
803256	Legal Assistance, Refugees	refuge legal assist
803300	Hotlines, Law	hotlin law
803300	Hotlines, Lawyer	hotlin lawyer
803311	Consumer Protection, Consumer Advocacy	advocaci protect consum
803311	Consumer Complaints	complaint consum
803344	Lawyer Criminal	crimin lawyer
803366	Legal Assistance, Lawyer	legal lawyer assist
803388	Lawyer, Civil	civil lawyer
803399	Hotlines, Consumer Protection	hotlin protect consum
803399	Hotlines, Consumer Advocacy	advocaci hotlin consum
803399	Hotlines Consumer Complaints	hotlin complaint consum
803487	Bipolar	bipolar
803487	Depressed	depress
801034	SSI, SSDI	ssi ssdi
804026	Sports/Recreation, Junior High School	recreat school junior sport high
804081	Sports/Recreation, Grade School	recreat grade school sport
804103	Little League	littl leagu
804169	Pop Warner	warner pop
804191	Residential Program, Drug Addiction	drug addict program residenti
804202	Support Group, Drug Addiction	group drug addict support
804235	Counseling, Drug Addiction	counsel drug addict
804279	Recovery, Drug Addiction	drug addict recoveri
804279	Drug Rehabilitation	drug rehabilit
804301	Prevention Program, Drug Addiction	prevent drug addict program
804312	Hotlines, Drug Addiction	hotlin drug addict
804356	Halfway House, Drug Addiction	halfwai drug addict hous
804378	Advocacy, Drug Addiction	advocaci drug addict
804389	Information and Referral, Drug Addiction	inform drug and addict referr
804444	Advocacy, Teen	advocaci teen
804444	Advocacy Infant/Toddler	toddler infant advocaci
800693	Domestic Violence Prevention	prevent domest violenc
804477	Prevention Programs Junior High School	prevent school junior high program
804521	Companion Programs Youth	youth program companion
804554	Hotlines, Teen	hotlin teen
804554	Hotlines Infant/Toddler	toddler infant hotlin
804587	Homework Assistance Junior High School	homework school junior assist high
804609	Companion Programs Teen	teen program companion
804620	After School Programs, Grade School	grade school after program
804631	Information and Referral, Teen	inform teen and referr
804631	Information and Referral Infant/Toddler	toddler inform infant and referr
804708	Prevention Programs Grade School	prevent grade school program
804741	Day Care, 0 - 05	care dai
804719	Day Care, 05 - 12	care dai
804532	Youth Camp	camp youth
804686	Teen Camp	camp teen
804686	Youth Camp	camp youth
804598	Teen Camp	camp teen
804598	Youth Camp	camp youth
804466	After School Programs Teen	teen school after program
804466	After School Programs Youth	youth school after program
804587	After School Programs Teen	teen school after program
804587	After School Programs Youth	youth school after program
804620	After School Programs Youth	youth school after program
804796	Painting Classes, Youth/Teen	youth class paint teen
804796	Drawing Classes, Youth/Teen	youth class teen draw
804796	Video Classes, Youth/Teen	youth class teen video
804796	Graphic Design Classes, Youth/Teen	youth class teen graphic design
804807	Acting Classes, Youth/Teen	youth class teen act
804807	Theater Classes, Youth/Teen	youth class teen theatre
804026	Youth Sports	youth sport
804026	Teen Sports	teen sport
804081	Youth Sports	youth sport
804092	Youth Sports	youth sport
804092	Teen Sports	teen sport
804851	Painting Drawing Video Graphic Design Classes, Adult	paint graphic draw design video adult class
804807	Dance Classes, Youth/Teen	youth class teen danc
804862	Acting Theater Dance Classes, Adult	class danc act adult theater
804884	Discount Health Care	care health discout
804884	Sliding Scale Health Care	scale care health slide
802519	Vaccinations	vaccin
804917	Affordable Housing Elderly	elderli hous afford
802222	Bills	bill
804939	Meals	meal
804939	Food	food
804950	Going back to school	back school go
804961	Financial Education	educ financi
804961	Banking	bank
804961	Credit	credit
804532	Summer	summer
804598	Summer	summer
804686	Summer	summer
805005	Bulimia	bulimia
805005	Anorexia	anorexia
802244	Soup Kitchen	kitchen soup
805027	Karate	karat
805027	Tae Kwon Do	do kwon tae
805027	Taekwon Do	do taekwon
805027	Taekwondo	taekwondo
801221	Childcare Training	childcar train
801408	Home Based Childcare Certification	childcar certif home base
804774	Information and Referral, Childcare	childcar referr inform and
804774	Information and Referral, Day Care	referr inform dai and care
804653	Childcare Vouchers	childcar voucher
804763	Home Based Childcare 0 - 05	childcar   home base 0 05
804763	Home Based Day Care 0 - 05	  home base dai care 0 05
804752	Home Based Childcare 05 - 12	childcar  home base  05 12
804752	Home Based Day Care 05 - 12	 home base dai  care 05 12
803707	DSS	dss
800550	Battered Children	children batter
800550	DSS	dss
805038	Discount Drugs	drug discount
802211	SNAP	snap
\.


--
-- Data for Name: category; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY category (category_id, category_name, stems_name) FROM stdin;
800000	Cultural Heritage	cultur heritag
800022	Music	music
800033	Arts/Entertainment	entertain art
800044	Theater/Performance Art	art perform theater
800055	Museum	museum
800066	Arts, Youth	youth art
800077	Culture	cultur
800088	Information and Referral, City Services	servic inform citi and referr
800099	Hotlines, City Services	servic hotlin citi
800110	Waste Management	wast manag
800121	Code Enforcement	enforc code
800132	Sanitation	sanit
800143	Voter Registration	registr voter
800154	Public Transportation	public transport
800165	Recycling	recycl
800176	City Services	servic citi
800187	Elections / Campaign	campaign elect
800198	Community Service	servic commun
800209	Crime Watch	watch crime
800220	Library	librari
800231	Newspapers	newspap
800242	Media	media
800253	Community Gardens	garden commun
800264	Bookmobile	bookmobil
800275	Book Club	club book
800286	Neighborhood Groups	group neighborhood
800297	Television/Radio	radio televis
800308	Computer, Public Access	access public comput
800319	Counseling, Suicide	counsel suicid
800330	Counseling, Teen	counsel teen
800341	Suicide Hotline	hotlin suicid
800352	Counseling, Crisis	counsel crisi
800363	Hotlines, Counseling	counsel hotlin
800374	Support Group, Mental Health	group health mental support
800385	Support Group, Medical	group medic support
800396	Counseling, Death	counsel death
800407	Support Group, Bereavement	group bereav support
800418	Counseling, Mental Health	counsel health mental
800429	Information and Referral, Counseling	counsel inform and referr
800440	Counseling, Child	counsel child
800451	Counseling, Sexual Assault	counsel sexual assault
800462	Counseling, Health	counsel health
800473	Marital Counseling	counsel marit
800484	Pastoral Care	care pastor
800495	Support Group, Other	group other support
800517	Support Group, Sexual Assault	group sexual assault support
800528	Counseling, Family	counsel famili
800539	Hotlines, Domestic Violence	hotlin domest violenc
800561	Hotlines, Child Abuse	hotlin child abus
800572	Incest	incest
800583	Safe House	safe hous
800594	Advocacy, Domestic Violence	advocaci domest violenc
800605	Counseling, Child Abuse	counsel child abus
800616	Abuse, Domestic	domest abus
800627	Counseling, Domestic Violence	counsel domest violenc
800638	Hotlines, Rape	hotlin rape
800649	Abuse, Elder	elder abus
800660	Crisis Intervention, Abuse	intervent crisi abus
800671	Advocacy, Child Abuse	advocaci child abus
800682	Adult Protective Services	protect servic adult
800693	Child Abuse Prevention	prevent child abus
800704	Abuse, Sexual	sexual abus
800715	Information and Referral, Child Abuse	inform and child referr abus
800726	Information and Referral, Rape	inform rape and referr
800737	Family Violence	famili violenc
800748	Domestic Violence Prevention	prevent domest violenc
800759	Information and Referral, Domestic Violence	inform and referr domest violenc
800770	Counseling, Rape	counsel rape
800781	Battered Women Shelter	shelter batter women
800792	Advocacy, Rape	advocaci rape
800803	Rape	rape
800814	Caregivers, Disabled	disabl caregiv
800825	Autism	autism
800836	Vocational Assessment, Disabled	disabl vocat assess
800847	Information and Referral, Disabled	disabl inform and referr
800858	Hotlines Disabled	disabl hotlin
800869	Advocacy, Disabled	disabl advocaci
800880	Transitional Living Program, Disabled	disabl transition program live
800891	Housing, Disabled	disabl hous
800902	Transportation, Disabled	disabl transport
800913	Special Needs	special ne
800924	Residential Programs, Disabled	disabl program residenti
800935	Attention Deficit Disorder	deficit disord attent
800946	Services, Disabled	disabl servic
800957	Adult Day Care, Disabled	disabl care dai adult
800968	Dyslexia	dyslexia
800979	Visually Impaired	impair visual
800990	Independent Living Facility, Disabled	disabl independ facil live
801001	Life Skills Training, Disabled	disabl train life skill
801012	Independent Living, Disabled	disabl independ live
801023	Reading Aids	read aid
801034	Social Security	secur social
801045	Employment, Disabled	disabl employ
801056	Hearing Impaired	impair hear
801067	Learning Disabilities	learn disabl
801078	Mental Retardation	retard mental
801089	Independent Living Program, Disabled	disabl independ program live
801100	Vocational Rehabilitation, Disabled	disabl vocat rehabilit
801111	Assistive Devices	devic assist
801122	Braille	braill
801133	Education/Training, Disabled	disabl train educ
801144	Meals on Wheels	meal wheel
801155	Mental Illness	ill mental
801166	Therapeutic Riding	ride therapeut
801177	Computer Training, Adult	train comput adult
801188	Education, Elementary	educ elementari
801199	Life Skills Training, Adult	train life adult skill
801210	Truancy	truanci
801232	Head Start	head start
801243	ESL/ESOL	esl esol
801254	Advocacy, Special Needs	special advocaci ne
801265	Tutoring, Middle School	school middl tutor
801276	Certificate Programs	certif program
801287	First Aid Classes	aid class first
801298	Literacy	literaci
801309	Professional Training	train profession
801320	School, Alternative/Charter	charter school altern
801331	High School Diploma	diploma school high
801342	Homework Assistance, Middle School	homework school assist middl
801353	Information and Referral, Education	inform educ and referr
801364	Adult Education	educ adult
801375	Homework Assistance, Adult	homework assist adult
801386	Education, Middle School	educ school middl
801397	CPR Classes	class cpr
801419	Medical Training	train medic
801430	Continuing Education	educ continu
801441	GED	ged
801452	Computer Training, Youth	youth train comput
801463	Tutoring, High School	school high tutor
801474	School, Public	school public
801485	Tutoring, Elementary School	school elementari tutor
801496	Life Skills Training, Youth	youth train life skill
801507	School Private/Parochial	school parochi privat
801518	Homework Assistance, College	colleg homework assist
801529	Homework Assistance, Elementary School	homework school assist elementari
801540	Information and Referral, Special Needs	special inform ne and referr
801551	Hotlines, Special Needs	special hotlin ne
801562	Advocacy, Education	educ advocaci
801573	Life Skills Training, Teens	train life teen skill
801584	Adult Basic Education	educ basic adult
801595	Homework Assistance, High School	homework school assist high
801606	Dropout Program	dropout program
801617	Computer Training, Teen	train teen comput
801628	Education, High School	educ school high
801639	Vocational Assessment	vocat assess
801650	Hotlines, Unemployment	hotlin unemploy
801661	Hotlines, Employment Discrimination	hotlin discrimin employ
801672	Family Medical Leave Act	leav famili act medic
801683	Unemployment	unemploy
801694	Advocacy, Employment Discrimination	advocaci discrimin employ
801705	Job Search	search job
801716	Employee Rights	right employe
801727	Job Bank	bank job
801738	Information and Referral, Employment	inform and referr employ
801749	Advocacy, Unemployment	advocaci unemploy
801760	Employment, Youth/Teens	youth teen employ
801771	Job Club	club job
801782	Employment, Counseling	counsel employ
801793	Resume Assistance	resum assist
801804	Vocational Rehabilitation	vocat rehabilit
801815	Employment, Senior	senior employ
801826	Job Training	train job
801837	Business Assistance	assist busi
801848	Employment, Veterans	veteran employ
801859	Employment, Services	servic employ
801870	Business Loans	loan busi
801881	Information and Referral, Retirement Planning	retir inform and referr plan
801892	Advocacy, Credit	advocaci credit
801903	Tax Preparation Assistance	assist prepar tax
801914	Information and Referral, Financial Aid	aid inform and financi referr
801925	Credit Counseling	counsel credit
801936	Retirement	retir
801947	Hotlines, Credit	hotlin credit
801958	Financial Counseling	counsel financi
801969	Budget Counseling	counsel budget
801980	Information and Referral, Credit	inform credit and referr
801991	Financial Aid College	aid colleg financi
802002	Shelter, Families	famili shelter
802013	Hotlines, Utilities	hotlin util
802024	Clothing	cloth
802035	WIC	wic
802046	Shelter, Women	shelter women
802057	Advocacy, Utilities	advocaci util
802068	Shelter, Veterans	veteran shelter
802079	Advocacy, TAFDC	advocaci tafdc
802090	Shelter, Men	men shelter
802101	Shelter, Runaways	runawai shelter
802112	FEMA	fema
802123	Information and Referral, Food	inform food and referr
802134	Information and Referral, Shelter	inform shelter and referr
802145	Shelter, Homeless	homeless shelter
802156	Household Items	item household
802167	Financial Assistance	assist financi
802178	TAFDC	tafdc
802189	Hotlines, Shelter	hotlin shelter
802200	Furniture	furnitur
802222	Utility Assistance	assist util
802233	Information and Referral, Welfare	inform and welfar referr
802255	Advocacy, Homeless	homeless advocaci
802266	Shelter, Elderly	elderli shelter
802277	Homeless Services	servic homeless
802288	SSDI	ssdi
802299	Nutrition	nutrition
802310	Tobacco Education/Cessation, Adult	educ cessat adult tobacco
802321	Information and Referral, AIDS/HIV	inform and referr aids hiv
802332	Hotlines, AIDS/HIV	hotlin aids hiv
802343	Weight Management	weight manag
802354	Pregnancy Prevention	prevent pregnanc
802365	Yoga	yoga
802376	Hotlines, Health Insurance	hotlin health insur
802387	Information and Referral, Cancer	inform and referr cancer
802398	Childbirth Classes	class childbirth
802409	Fitness	fit
802420	Hotlines, Cancer	hotlin cancer
802431	Health Education	educ health
802442	Information and Referral, Health	inform health and referr
802453	Information and Referral, Health Insurance	inform health insur and referr
802464	AIDS/HIV, Prevention & Education	prevent educ aids hiv
802475	Tobacco Education/Cessation, Youth/Teen	youth educ teen cessat tobacco
802486	Rehabilitation, Medical	medic rehabilit
802497	Pain Management	manag pain
802508	Home Visiting	home visit
802519	Immunizations	immun
802530	Advocacy, AIDS/HIV	advocaci aids hiv
802541	First Aid	aid first
802552	Pregnancy Testing	test pregnanc
802563	Health Insurance	health insur
802574	Health Care, Children	children care health
802585	Cancer	cancer
802596	Prevention Programs, Health	prevent health program
802607	Free Health Care	free care health
802618	Hospice Care	care hospic
802629	Home Health Care	home care health
802640	STD	std
802651	Health Care, Adults	adult care health
802662	Homemaker Services	servic homemak
802673	Advocacy, Cancer	advocaci cancer
802684	AIDS/HIV, Testing & Screening	screen test aids hiv
802695	Blood Pressure	blood pressur
802706	Men's Health	health men
802717	Contraceptive Services	contracept servic
802728	Health Care, Teens	care health teen
802739	Abortion	abort
802750	Donating Blood	blood donat
802761	Family Planning	famili plan
802772	AIDS/HIV, Treatment & Support	treatment support aids hiv
802783	Advocacy, Health	advocaci health
802794	Medicare	medicar
802805	Dental Care	care dental
802816	Alternative Health	health altern
802827	Women's Health	women health
802838	Advocacy, Health Insurance	advocaci health insur
802849	Physician Referral	physician referr
802860	Early Intervention	intervent earli
802871	MassHealth/Medicaid	masshealth medicaid
802882	Health Care, Infants/Toddlers	infant care health toddler
802893	Code Violations	code violat
802904	Landlord Trainings	train landlord
802915	Hotlines, Housing	hotlin hous
802926	Home Buying	home bui
802937	Lead Paint	lead paint
802948	AIDS/HIV, Housing	hous aids hiv
802959	Loans, Housing	loan hous
802970	Housing, Affordable	hous afford
802981	Section 8 Housing	 hous section 8
802992	Public Housing	hous public
803003	Tenant Rights	right tenant
803014	Housing, Transitional	hous transition
803025	Housing, Counseling	counsel hous
803036	Housing, Financial Assistance	assist hous financi
803047	Fair Housing	hous fair
803058	Landlord Rights	right landlord
803069	Information and Referral, Housing	inform hous and referr
803080	Housing, Shelter	hous shelter
803091	Mediation, Housing	hous mediat
803102	Home Improvement/Modification	home improv modif
803113	Advocacy, Housing	advocaci hous
803124	Hotlines, Immigrants	hotlin immigr
803135	Immigrant Employment Services	servic immigr employ
803146	Refugees	refuge
803157	Translating/Interpreting	interpret translat
803168	Asylum	asylum
803179	Advocacy, Immigrants	advocaci immigr
803190	Citizenship Preparation	citizenship prepar
803201	Immigrant Rights	right immigr
803212	Immigrant Education Assistance	educ assist immigr
803223	Discrimination	discrimin
803234	Immigration/Naturalization	natur immigr
803245	Information and Referral Immigrants	inform immigr and referr
803256	Legal Assistance, Immigrants	legal immigr assist
803267	Crime Prevention	prevent crime
803278	Court	court
803289	Divorce	divorc
803300	Hotlines, Legal	legal hotlin
803311	Consumer Rights	right consum
803322	Information and Referral	inform and referr
803333	Legal Assistance, Child Custody	legal assist child custodi
803344	Legal Representation Criminal	crimin legal represent
803355	Offenders	offend
803366	Legal Services	servic legal
803377	Legal Advocacy	advocaci legal
803388	Legal Representation, Civil	civil legal represent
803399	Hotlines, Consumer Rights	hotlin right consum
803410	Benefits Law	law benefit
803421	Dual Diagnosis	dual diagnosi
803432	Advocacy, Mental Retardation	retard advocaci mental
803443	Information and Referral, Mental Health	inform health mental and referr
803454	Hotlines, Mental Health	hotlin health mental
803465	Panic Disorder	panic disord
803476	Psychiatric Services	servic psychiatr
803487	Depression/Manic Depression	manic depress
803498	Mental Health, Services	servic health mental
803509	Residential Programs, Psychiatric	psychiatr program residenti
803520	Information and Referral, Mental Retardation	retard inform mental and referr
803531	Anxiety	anxieti
803542	Advocacy, Mental Health	advocaci health mental
803553	Mental Health, Information and Education	inform educ health and mental
803564	Stress Management	stress manag
803575	Coalition/Partnership	partnership coalition
803586	Environmental Education	educ environment
803597	Planning/Funding Organizations	fund organ plan
803608	Clubhouse	clubhous
803619	Environment	environ
803630	Veterans	veteran
803641	Gay/Lesbian/Bisexual	bisexu lesbian gai
803652	Transportation, Other	other transport
803663	Membership Organizations	organ membership
803674	Parenting Education	educ parent
803685	Parenting Groups	parent group
803696	Teen Parents	teen parent
803718	Supervised Visitation	visit superv
803729	Single Parents	singl parent
803740	Hotlines, Parenting	hotlin parent
803751	Baby Items	item babi
803762	Advocacy, Parenting	advocaci parent
803773	Information and Referral, Parenting	inform and parent referr
803784	Parenting Support	parent support
803795	Mediation/Conflict Resolution, Family	conflict resolut famili mediat
803806	Adoption	adopt
803817	Hotlines, Crime	crime hotlin
803828	Police	polic
803839	Gang Prevention/Intervention	intervent prevent gang
803850	Law Enforcement	enforc law
803861	Juvenile Delinquency Prevention	prevent juvenil delinqu
803872	Public Safety	safeti public
803883	Information and Referral, Public Safety	inform safeti public and referr
803894	Hotlines, Public Safety	safeti hotlin public
803905	Crime Victim	crime victim
803916	Senior Center	senior center
803927	Adult Day Care, Senior	senior care dai adult
803938	Transportation, Senior	senior transport
803949	Hotlines, Senior	senior hotlin
803960	Elder Services	elder servic
803971	Information and Referral, Senior	senior inform and referr
803982	Advocacy, Senior	senior advocaci
803993	Caregivers	caregiv
804004	Elder Abuse	elder abus
804015	Soccer	soccer
804026	Sports/Recreation, Middle School	recreat school sport middl
804037	Parks	park
804048	Sports/Recreation, Adult	recreat sport adult
804059	Swimming Lessons	lesson swim
804070	Water Safety	safeti water
804081	Sports/Recreation, Elementary School	recreat school sport elementari
804092	Sports/Recreation, High School	recreat school sport high
804103	Baseball	basebal
804114	Basketball	basketbal
804136	Sports/Recreation, Preschool	preschool recreat sport
804147	Gymnastics	gymnast
804158	Swimming	swim
804169	Football	footbal
804180	Prevention Program, Alcohol Addiction	prevent alcohol addict program
804191	Residential Program, Drug Abuse	drug abus program residenti
804202	Support Group, Drug Abuse	group drug abus support
804213	Advocacy, Alcohol Addiction	alcohol advocaci addict
804224	Gambling Addiction	addict gambl
804235	Counseling, Drug Abuse	counsel drug abus
804246	Counseling, Alcohol Addiction	counsel alcohol addict
804257	Halfway House, Alcohol Addiction	halfwai alcohol addict hous
804268	Support Group, Alcohol Addiction	alcohol group addict support
804279	Recovery, Drug Abuse	drug abus recoveri
804290	Methadone Treatment	methadon treatment
804301	Prevention Program, Drug Abuse	prevent drug abus program
804312	Hotlines, Drug Abuse	hotlin drug abus
804323	Information and Referral, Alcohol Addiction	inform alcohol and addict referr
804334	Recovery, Alcohol Addiction	alcohol addict recoveri
804345	Residential Program, Alcohol Addiction	alcohol addict program residenti
804356	Halfway House, Drug Abuse	halfwai drug abus hous
804367	Detoxification	detoxif
804378	Advocacy, Drug Abuse	advocaci drug abus
804389	Information and Referral, Drug Abuse	inform drug and referr abus
804400	Hotlines, Alcohol Addiction	alcohol hotlin addict
804411	Peer Leadership	leadership peer
804422	Babysitter Training	train babysitt
804433	Infants/Toddlers	infant toddler
804444	Advocacy, Youth	youth advocaci
804466	After School Programs, High School	school after high program
804477	Drug Prevention Programs Middle School	prevent school drug program middl
804488	Drug Prevention Programs High School	prevent school drug high program
804499	Youth Programs, 05 - 10	 youth  program 05 10
804510	Mediation/Conflict Resolution, Youth	youth conflict resolut mediat
804521	Mentoring, Youth	youth mentor
804532	Camp, 05 - 10	  camp 05 10
804543	Runaways	runawai
804554	Hotlines, Youth	youth hotlin
804565	Teen Programs	teen program
804576	Youth Programs, 14 - 19	youth  program  14 19
804587	After School Programs, Middle School	school after program middl
804598	Camp, 14 - 19	 camp  14 19
804609	Mentoring, Teens	teen mentor
804620	After School Programs, Elementary School	school after elementari program
804631	Information and Referral, Youth	youth inform and referr
804642	Children's Playgroups	children playgroup
804664	Youth Programs, 11 - 15	youth  program  11 15
804675	Preschool Programs	preschool program
804686	Camp, 11 - 15	 camp  11 15
804697	Youth Programs, 17 - 22	youth   program 17 22
804708	Drug Prevention Programs Elementary School	prevent school drug elementari program
804719	Child Care, 05 - 12	 care child  05 12
804730	Kinship Care	care kinship
804741	Child Care, 0 - 05	  care child 0 05
804785	Music Lessons, Youth/Teen	youth lesson teen music
804796	Visual Arts Classes, Youth/Teen	youth class teen art visual
804807	Performance Art Classes, Youth/Teen	youth class teen art perform
804818	Senior Recreation/Social	recreat senior social
804829	Hockey	hockei
804840	Music Lessons, Adult	lesson music adult
804851	Visual Art Classes, Adult	adult class art visual
804862	Performance Art Classes, Adult	class art perform adult
804873	Prenatal	prenat
804884	Affordable Health Care	care health afford
804895	Uninsured Services	servic uninsur
804906	Support Group, Cancer	group cancer support
804917	Housing, Elderly	elderli hous
804928	Food	food
804939	Soup Kitchen	soup kitchen
804950	College Preparation, Adult	colleg prepar adult
804961	Financial Literacy	literaci financi
804972	Counseling, Group	counsel group
804983	Social/Singles Groups	singl social group
804994	Computer/Technology Providers	technologi provid comput
805005	Eating Disorder	eat disord
805016	Diabetes	diabet
802244	Food Pantry	pantri food
805027	Martial Arts	art martial
801221	Child Care Training	child train care
801408	Home Based Child Care Certification	certif home base child care
804774	Information and Referral, Child Care	referr inform child and care
804653	Child Care Vouchers	voucher child care
804763	Home Based Child Care 0 - 05	  home base child care 0 05
804752	Home Based Child Care 05 - 12	 home base child  care 05 12
803707	Foster Care	foster care
800550	Abuse, Child	abus child
805038	Prescription Drugs	drug prescript
800011	Dance	danc
802211	Food Stamps	stamp food
\.


--
-- Data for Name: category_search_log; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY category_search_log (search_id, session_id, ip_address, "timestamp", search_phrase, referer, phrase_stems, top_score) FROM stdin;
36539	66.92.77.160.5111109	66.92.77.160	2009-02-11 05:54:18.729039	test	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=Arts%2FCulture%2FEntertainment	test	0.00
36540	173.114.4.71.7111109	173.114.4.71	2009-02-11 07:59:36.053505	rent	http://northshoreport.org/	rent	0.00
36541	173.114.4.71.7111109	173.114.4.71	2009-02-11 07:59:47.315014	housing	http://northshoreport.org/cgi-bin/mvhub/guide.pl	hous	0.00
36543	66.80.237.78.10121109	66.80.237.78	2009-02-12 10:00:34.45959	shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=509515	shelter	1.00
36545	74.10.197.220.9141109	74.10.197.220	2009-02-14 09:23:46.233276	job training	http://northshoreport.org/	job train	1.00
36547	74.10.197.220.9141109	74.10.197.220	2009-02-14 09:36:37.469211	shelter	http://northshoreport.org/	shelter	1.00
36548	74.10.197.220.20171109	74.10.197.220	2009-02-17 20:40:46.111652	nscap	http://northshoreport.org/	nscap	0.00
36549	74.10.197.220.20171109	74.10.197.220	2009-02-17 20:41:04.072078	north shore community action program	http://northshoreport.org/cgi-bin/mvhub/guide.pl	action commun north shore	0.00
36550	12.151.131.131.15191109	12.151.131.131	2009-02-19 15:39:54.602304	test	http://northshoreport.org/	test	0.00
36551	66.92.77.160.20191109	66.92.77.160	2009-02-19 20:22:35.157509	test	http://northshoreport.org/	test	0.00
36552	66.92.77.160.20191109	66.92.77.160	2009-02-19 20:22:39.770904	shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl	shelter	1.00
36553	76.118.140.125.11201109	76.118.140.125	2009-02-20 11:06:21.931781	shelter	http://northshoreport.org/	shelter	1.00
36554	66.92.77.160.20211109	66.92.77.160	2009-02-21 20:40:36.675187	shelter	http://northshoreport.org/	shelter	1.00
36555	129.63.96.191.15241109	129.63.96.191	2009-02-24 15:58:16.504203	shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl	shelter	1.00
36556	129.63.96.191.16241109	129.63.96.191	2009-02-24 16:35:09.482926	shelter	http://northshoreport.org/	shelter	1.00
36557	66.92.77.160.22251109	66.92.77.160	2009-02-25 22:54:31.39761	shelter	http://northshoreport.org/	shelter	1.00
36559	75.144.137.57.8261109	75.144.137.57	2009-02-26 08:47:23.524549	test	http://northshoreport.org/	test	0.00
36560	75.144.137.57.8261109	75.144.137.57	2009-02-26 08:47:31.314394	shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl	shelter	1.00
36562	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:02:34.148773	computer	http://northshoreport.org/	comput	1.00
36563	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:02:54.367852	food	http://northshoreport.org/	food	1.00
36564	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:03:12.524706	job	http://northshoreport.org/cgi-bin/mvhub/guide.pl	job	1.00
36565	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:04:08.934804	love	http://northshoreport.org/cgi-bin/mvhub/guide.pl	love	0.00
36566	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:04:18.412128	peace	http://northshoreport.org/cgi-bin/mvhub/guide.pl	peac	0.00
36568	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:07:24.977557	shelter	http://northshoreport.org/	shelter	1.00
36569	75.144.137.57.9261109	75.144.137.57	2009-02-26 09:30:13.224785	north	http://northshoreport.org/cgi-bin/mvhub/guide.pl	north	0.00
36571	208.54.90.21.13261109	208.54.90.21	2009-02-26 13:07:00.078196	test	http://northshoreport.org/	test	0.00
36572	209.235.0.250.9271109	209.235.0.250	2009-02-27 09:20:22.949539	child 	http://northshoreport.org	child	1.00
36573	209.235.0.250.9271109	209.235.0.250	2009-02-27 09:21:26.314343	lynn	http://northshoreport.org/	lynn	0.00
36574	209.235.0.250.9271109	209.235.0.250	2009-02-27 09:21:31.553396	legal	http://northshoreport.org/cgi-bin/mvhub/guide.pl	legal	1.00
36575	64.206.127.11.11271109	64.206.127.11	2009-02-27 11:49:18.326637	children	http://northshoreport.org/	children	1.00
36578	205.188.116.10.15281109	205.188.116.10	2009-02-28 15:15:36.592602	youth	http://northshoreport.org/	youth	1.00
36579	205.188.116.10.15281109	205.188.116.10	2009-02-28 15:16:26.24067	tutoring	http://northshoreport.org/	tutor	0.00
36580	205.188.116.10.15281109	205.188.116.10	2009-02-28 15:16:34.877609	GED	http://northshoreport.org/cgi-bin/mvhub/guide.pl	ged	0.00
36583	96.39.15.106.20281109	96.39.15.106	2009-02-28 20:51:36.047453	GED	http://northshoreport.org/	ged	0.00
36584	96.39.15.106.20281109	96.39.15.106	2009-02-28 20:51:49.440508	Food	http://northshoreport.org/cgi-bin/mvhub/guide.pl	food	1.00
36585	209.235.0.249.1132109	209.235.0.249	2009-03-03 11:18:40.970955	children	http://northshoreport.org/	children	1.00
36587	205.237.102.178.1632109	205.237.102.178	2009-03-03 16:21:49.739075	Catholic Charities North	http://northshoreport.org/	cathol chariti north	0.00
36589	129.63.96.191.1052109	129.63.96.191	2009-03-05 10:08:22.895326	shelter	http://northshoreport.org/	shelter	1.00
36590	155.212.138.182.1252109	155.212.138.182	2009-03-05 12:06:57.046341	my turn	http://northshoreport.org/	turn	0.00
36591	155.212.138.182.1252109	155.212.138.182	2009-03-05 12:07:03.815805	youth	http://northshoreport.org/cgi-bin/mvhub/guide.pl	youth	1.00
36594	72.85.251.119.1162109	72.85.251.119	2009-03-06 11:17:15.327392	GED	http://northshoreport.org/	ged	0.00
36596	72.85.251.119.1162109	72.85.251.119	2009-03-06 11:20:55.200431	Compass	http://northshoreport.org/cgi-bin/mvhub/guide.pl	compass	0.00
36597	98.217.216.40.1992109	98.217.216.40	2009-03-09 19:27:38.341363	housing	http://northshoreport.org/	hous	1.00
36598	64.12.112.170.4102109	64.12.112.170	2009-03-10 04:59:21.763467	ged prep	http://northshoreport.org/	ged prep	0.50
36599	64.12.112.170.4102109	64.12.112.170	2009-03-10 04:59:54.444398	ged prep for youth	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801826;search_phrase=ged%20prep	ged prep youth	0.33
36600	64.12.112.170.5102109	64.12.112.170	2009-03-10 05:01:53.736259	ged prep for youth	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801826;search_phrase=ged%20prep	ged prep youth	0.33
36601	72.93.187.82.11102109	72.93.187.82	2009-03-10 11:02:59.823902	affordable housing	http://northshoreport.org/	afford hous	0.50
36603	72.93.187.82.11102109	72.93.187.82	2009-03-10 11:25:46.51199	housing	http://northshoreport.org/	hous	1.00
36604	72.85.251.119.17122109	72.85.251.119	2009-03-12 17:26:54.307026	Home care	http://www.northshoreport.org/	care home	0.00
36605	75.144.136.113.13132109	75.144.136.113	2009-03-13 13:19:21.246123	Job training	http://www.northshoreport.org/	job train	1.00
36607	66.80.237.78.9172109	66.80.237.78	2009-03-17 09:07:10.035079	housing	http://www.northshoreport.org/	hous	1.00
36608	76.24.183.224.11192109	76.24.183.224	2009-03-19 11:17:29.205371	housing	http://northshoreport.org/	hous	1.00
36609	70.88.192.1.17192109	70.88.192.1	2009-03-19 17:13:21.745486	shelter	http://www.northshoreport.org/	shelter	1.00
36610	70.88.192.1.17192109	70.88.192.1	2009-03-19 17:14:02.576689	food pantry	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	food pantri	0.50
36611	205.237.102.178.17192109	205.237.102.178	2009-03-19 17:47:06.625801	GED	http://www.northshoreport.org/	ged	0.00
36612	205.237.102.178.17192109	205.237.102.178	2009-03-19 17:47:29.552191	Youth Education	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	educ youth	0.50
36613	205.237.102.178.17192109	205.237.102.178	2009-03-19 17:48:35.044951	youth programs	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	program youth	1.00
36614	205.237.102.178.17192109	205.237.102.178	2009-03-19 17:51:35.773665	family strengthening	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	famili strengthen	0.50
36615	205.237.102.178.17192109	205.237.102.178	2009-03-19 17:52:15.71726	counseling services	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	counsel servic	0.50
36617	76.118.140.125.18192109	76.118.140.125	2009-03-19 18:53:35.141928	shelter	http://northshoreport.org/	shelter	1.00
36619	24.61.37.4.22192109	24.61.37.4	2009-03-19 22:28:56.262524	Salem CyberSpace	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	cyberspac salem	0.00
36620	24.61.37.4.22192109	24.61.37.4	2009-03-19 22:29:03.924185	After-school	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	after school	1.00
36626	155.212.138.182.10202109	155.212.138.182	2009-03-20 10:55:33.877841	youth	http://www.northshoreport.org/	youth	1.00
36627	205.188.116.73.20232109	205.188.116.73	2009-03-23 20:07:05.119131	job training	http://www.northshoreport.org/	job train	1.00
36628	205.188.116.73.20232109	205.188.116.73	2009-03-23 20:07:43.600643	domestic violence	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	domest violenc	0.00
36629	72.245.144.2.10242109	72.245.144.2	2009-03-24 10:17:16.827324	housing 	http://www.northshoreport.org/	hous	1.00
36630	68.166.124.194.8252109	68.166.124.194	2009-03-25 08:58:04.230787	youth	http://www.northshoreport.org/	youth	1.00
36634	205.237.102.178.9262109	205.237.102.178	2009-03-26 09:10:41.533855	food	http://www.northshoreport.org/	food	1.00
36637	205.237.102.178.13262109	205.237.102.178	2009-03-26 13:31:07.355461	family counseling	http://northshoreport.org/	counsel famili	0.50
36638	205.237.102.178.13262109	205.237.102.178	2009-03-26 13:32:07.080506	counseling	http://northshoreport.org/	counsel	1.00
36643	74.10.197.220.11302109	74.10.197.220	2009-03-30 11:45:12.291908	fuel	http://www.northshoreport.org/	fuel	1.00
36645	72.85.251.119.10312109	72.85.251.119	2009-03-31 10:05:10.5636	COMPASS	http://northshoreport.org/cgi-bin/mvhub/guide.pl	compass	0.00
36647	72.85.251.119.10312109	72.85.251.119	2009-03-31 10:08:58.460201	GED Preparation	http://northshoreport.org/	ged prepar	0.50
36648	72.85.251.119.10312109	72.85.251.119	2009-03-31 10:09:59.473082	GED Preparation COMPASS Youth Program	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803190;search_phrase=GED%20Preparation	compass ged prepar program youth	0.40
36649	72.70.106.66.16312109	72.70.106.66	2009-03-31 16:30:20.606526	family therapy	http://northshoreport.org/	famili therapi	0.50
36650	72.85.251.119.1013109	72.85.251.119	2009-04-01 10:12:03.411268	GED	http://northshoreport.org/	ged	1.00
36651	72.72.115.114.823109	72.72.115.114	2009-04-02 08:13:33.603987	housing	http://www.northshoreport.org/	hous	1.00
36653	72.70.116.44.553109	72.70.116.44	2009-04-05 05:25:22.760937	Food Stamps	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=Food%2FShelter%20%26%20Financial%20Assistance	food stamp	1.00
36655	204.58.32.254.16153109	204.58.32.254	2009-04-15 16:25:19.038278	shelter	http://northshoreport.org/	shelter	1.00
36658	72.70.121.90.17153109	72.70.121.90	2009-04-15 17:39:20.515029	on the threshold fund	http://northshoreport.org/cgi-bin/mvhub/guide.pl	fund threshold	0.00
36659	72.70.121.90.17153109	72.70.121.90	2009-04-15 17:39:24.16686	on the threshold 	http://northshoreport.org/cgi-bin/mvhub/guide.pl	threshold	0.00
36660	129.63.96.191.15173109	129.63.96.191	2009-04-17 15:32:05.681342	shelter	http://northshoreport.org/index.shtml	shelter	1.00
36661	129.63.96.191.15173109	129.63.96.191	2009-04-17 15:35:02.96703	test	http://northshoreport.org/	test	0.00
36663	24.147.160.105.1193109	24.147.160.105	2009-04-19 01:06:54.131602	job training	http://northshoreport.org/	job train	1.00
36664	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:27:55.760938	housing	http://northshoreport.org/	hous	1.00
36666	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:29:06.277384	budgeting	http://northshoreport.org/	budget	0.00
36667	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:29:11.177135	financial	http://northshoreport.org/cgi-bin/mvhub/guide.pl	financi	1.00
36668	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:29:27.046477	food	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=509790&search_phrase=financial	food	1.00
36669	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:29:30.02848	housing advpcacu	http://northshoreport.org/	advpcacu hous	0.50
36670	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:29:42.494527	foodhousing	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803113;search_phrase=housing	foodhous	0.00
36671	72.93.175.100.9213109	72.93.175.100	2009-04-21 09:31:02.333512	shelter	http://northshoreport.org/	shelter	1.00
36673	72.93.175.100.16213109	72.93.175.100	2009-04-21 16:31:06.997338	food	http://northshoreport.org/	food	1.00
36674	72.93.196.33.13223109	72.93.196.33	2009-04-22 13:37:18.677466	beverly	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=City%2FTown%20Services	beverli	0.00
36675	72.93.196.33.16223109	72.93.196.33	2009-04-22 16:21:04.178888	housing	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	hous	1.00
36676	72.93.196.33.16223109	72.93.196.33	2009-04-22 16:21:21.650434	financial assistance	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803113;search_phrase=housing	assist financi	1.00
36679	76.19.194.175.21223109	76.19.194.175	2009-04-22 21:22:03.568307	food	http://northshoreport.org/	food	1.00
36680	12.76.177.45.10233109	12.76.177.45	2009-04-23 10:32:51.125639	job	http://northshoreport.org/	job	1.00
36681	12.76.177.45.10233109	12.76.177.45	2009-04-23 10:41:52.749346	http://northshoreport.org/img/copyright.gif	http://northshoreport.org/	copyright gif http img northshoreport org	0.00
36682	64.206.127.11.10233109	64.206.127.11	2009-04-23 10:59:34.509823	children	http://northshoreport.org/	children	1.00
36683	24.91.44.177.19233109	24.91.44.177	2009-04-23 19:47:30.794596	drug treatment	http://northshoreport.org/	drug treatment	0.50
36684	24.91.44.177.19233109	24.91.44.177	2009-04-23 19:47:59.855268	drug abuse	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=804389;search_phrase=drug%20treatment	abus drug	1.00
36685	204.58.32.254.14243109	204.58.32.254	2009-04-24 14:08:16.127027	domestic violence	http://northshoreport.org/	domest violenc	1.00
36687	72.93.189.193.16273109	72.93.189.193	2009-04-27 16:06:29.478937	ged	http://northshoreport.org/	ged	1.00
36689	66.238.215.240.8283109	66.238.215.240	2009-04-28 08:21:13.694848	housing	http://www.northshoreport.org/	hous	1.00
36690	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:22:53.686307	ncscap	http://northshoreport.org/	ncscap	0.00
36692	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:24:29.102009	lawyer 	http://northshoreport.org/	lawyer	1.00
36693	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:24:55.147453	legal services	http://northshoreport.org/	legal servic	1.00
36694	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:25:42.927218	HIV	http://northshoreport.org/	hiv	0.00
36695	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:25:46.726993	AIDS	http://northshoreport.org/cgi-bin/mvhub/guide.pl	aids	0.00
36696	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:25:55.600591	food	http://northshoreport.org/cgi-bin/mvhub/guide.pl	food	1.00
36697	68.163.130.197.12303109	68.163.130.197	2009-04-30 12:28:45.604742	housing	http://northshoreport.org/	hous	1.00
36700	130.219.8.244.17303109	130.219.8.244	2009-04-30 17:38:03.247127	service	http://northshoreport.org/	servic	1.00
36701	70.22.150.207.17303109	70.22.150.207	2009-04-30 17:46:01.312029	young parent program	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=Youth	parent young	0.50
36703	203.189.42.77.114109	203.189.42.77	2009-05-01 01:22:58.114324	esl	http://northshoreport.org/	esl	1.00
36705	72.93.174.223.1514109	72.93.174.223	2009-05-01 15:31:45.63062	grants for education	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_headings	educ grant	0.50
36706	24.62.230.30.1934109	24.62.230.30	2009-05-03 19:44:56.294264	eating disorders	http://northshoreport.org/	disord eat	0.50
36707	66.92.77.160.2134109	66.92.77.160	2009-05-03 21:08:03.07004	test	http://northshoreport.org/	test	0.00
36708	66.92.77.160.2134109	66.92.77.160	2009-05-03 21:08:08.602127	food	http://northshoreport.org/cgi-bin/mvhub/guide.pl	food	1.00
36709	151.203.74.189.1144109	151.203.74.189	2009-05-04 11:37:51.459986	hepatitis c	http://northshoreport.org/	c hepat	0.00
36710	151.203.74.189.1144109	151.203.74.189	2009-05-04 11:38:03.76126	holistic	http://northshoreport.org/	holist	0.00
36711	151.203.74.189.1144109	151.203.74.189	2009-05-04 11:38:12.560437	HIV case management	http://northshoreport.org/cgi-bin/mvhub/guide.pl	case hiv manag	0.00
36712	151.203.74.189.1144109	151.203.74.189	2009-05-04 11:38:33.778403	hepatitis C case management	http://northshoreport.org/	c case hepat manag	0.00
36713	151.203.74.189.1144109	151.203.74.189	2009-05-04 11:38:37.422982	case management	http://northshoreport.org/cgi-bin/mvhub/guide.pl	case manag	0.00
36715	151.203.74.189.1344109	151.203.74.189	2009-05-04 13:16:35.190362	nscap	http://northshoreport.org/	nscap	0.00
36716	151.203.74.189.1344109	151.203.74.189	2009-05-04 13:16:39.104733	legal	http://northshoreport.org/cgi-bin/mvhub/guide.pl	legal	1.00
36721	64.61.49.154.1664109	64.61.49.154	2009-05-06 16:14:15.994439	money management	http://northshoreport.org/	manag monei	0.00
36722	64.61.49.154.1664109	64.61.49.154	2009-05-06 16:16:18.127047	Money Management Program	http://northshoreport.org/	manag monei program	0.33
36725	66.80.237.78.1474109	66.80.237.78	2009-05-07 14:54:07.815978	utility	http://www.northshoreport.org/	util	1.00
36726	75.67.131.44.1774109	75.67.131.44	2009-05-07 17:03:22.658332	Food pantry	http://www.northshoreport.org	food pantri	1.00
36727	75.67.131.44.1774109	75.67.131.44	2009-05-07 17:03:47.675918	Shelter	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802244;search_phrase=Food%20pantry	shelter	1.00
36728	72.61.36.241.9104109	72.61.36.241	2009-05-10 09:56:07.460379	homelessness	http://northshoreport.org/	homeless	1.00
36729	173.9.105.5.9114109	173.9.105.5	2009-05-11 09:31:07.168514	child care	http://northshoreport.org/	care child	1.00
36730	173.9.105.5.16114109	173.9.105.5	2009-05-11 16:26:20.383663	job training	http://www.northshoreport.org/	job train	1.00
36731	173.9.105.5.16114109	173.9.105.5	2009-05-11 16:27:15.746921	child care	http://www.northshoreport.org/	care child	1.00
36732	173.9.105.5.16114109	173.9.105.5	2009-05-11 16:27:56.103692	pre school child care	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	care child pre school	0.50
36733	173.9.105.5.16114109	173.9.105.5	2009-05-11 16:28:39.414842	housing	http://www.northshoreport.org/	hous	1.00
36734	24.61.100.137.17114109	24.61.100.137	2009-05-11 17:56:01.115308	career opportunities	http://northshoreport.org/	career opportun	0.50
36735	64.12.116.73.13124109	64.12.116.73	2009-05-12 13:09:58.249242	GED classes	http://www.northshoreport.org/	class ged	0.50
36736	64.12.116.73.13124109	64.12.116.73	2009-05-12 13:10:30.395676	ESOL classes	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801441;search_phrase=GED%20classes	class esol	0.50
36737	64.12.116.73.13124109	64.12.116.73	2009-05-12 13:11:10.660036	Math classes	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801243;search_phrase=ESOL%20classes	class math	0.00
36738	64.12.116.73.13124109	64.12.116.73	2009-05-12 13:11:21.582528	college prep classes	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	class colleg prep	0.33
36739	66.80.237.78.14124109	66.80.237.78	2009-05-12 14:25:48.547049	education	http://www.northshoreport.org	educ	1.00
36744	24.91.151.214.13144109	24.91.151.214	2009-05-14 13:58:17.609636	Housing	http://www.northshoreport.org/	hous	1.00
36745	24.91.151.214.14144109	24.91.151.214	2009-05-14 14:00:02.734168	Housing	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803014;search_phrase=Housing	hous	1.00
36746	24.91.151.214.14144109	24.91.151.214	2009-05-14 14:02:53.886751	Salvation ARmy Peabody	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=803080;search_phrase=Housing	armi peabodi salvat	0.00
36747	24.91.151.214.14144109	24.91.151.214	2009-05-14 14:03:00.8534	Salvation Army	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	armi salvat	0.00
36748	24.91.151.214.14144109	24.91.151.214	2009-05-14 14:03:23.335132	red cross	http://www.northshoreport.org/	cross red	0.00
36750	76.19.74.173.14164109	76.19.74.173	2009-05-16 14:25:43.687544	food pantry	http://www.northshoreport.org/	food pantri	1.00
36758	72.85.251.119.8204109	72.85.251.119	2009-05-20 08:08:36.079524	ged	http://northshoreport.org/	ged	1.00
36759	24.91.123.153.9204109	24.91.123.153	2009-05-20 09:12:47.62875	homeless	http://northshoreport.org/	homeless	1.00
36760	24.91.123.153.9204109	24.91.123.153	2009-05-20 09:13:12.007692	citizens for adequate housing	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802002;search_phrase=homeless	adequ citizen hous	0.33
36762	76.118.140.125.19224109	76.118.140.125	2009-05-22 19:32:14.536023	test	http://northshoreport.org/	test	0.00
36763	76.118.140.125.19224109	76.118.140.125	2009-05-22 19:53:08.012882	shelter	http://northshoreport.org/	shelter	1.00
36767	129.63.96.180.20224109	129.63.96.180	2009-05-22 20:17:38.467206	shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl	shelter	1.00
36768	129.63.96.180.20224109	129.63.96.180	2009-05-22 20:17:39.629962	trees	http://northshoreport.org/cgi-bin/mvhub/guide.pl	tree	0.00
36772	129.63.96.180.21224109	129.63.96.180	2009-05-22 21:22:51.484982	shelter	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	shelter	1.00
36773	129.63.96.180.21224109	129.63.96.180	2009-05-22 21:22:52.398367	trees	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	tree	0.00
36777	129.63.96.180.13284109	129.63.96.180	2009-05-28 13:07:35.612664	shelter	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	shelter	1.00
36778	129.63.96.180.13284109	129.63.96.180	2009-05-28 13:07:36.663749	trees	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	tree	0.00
36781	74.10.197.220.17284109	74.10.197.220	2009-05-28 17:04:13.174298	food	http://www.northshoreport.org/	food	1.00
36785	129.63.96.180.13294109	129.63.96.180	2009-05-29 13:57:09.019707	shelter	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	shelter	1.00
36786	129.63.96.180.13294109	129.63.96.180	2009-05-29 13:57:09.93849	trees	http://nsp.production.testing123.net/cgi-bin/mvhub/guide.pl	tree	0.00
36787	129.63.96.191.1125109	129.63.96.191	2009-06-02 11:54:48.89179	test	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program&program_id=510296	test	0.00
36788	129.63.96.191.1125109	129.63.96.191	2009-06-02 11:54:55.45859	shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl	shelter	1.00
36789	98.216.210.232.1225109	98.216.210.232	2009-06-02 12:42:58.302025	summer job	http://northshoreport.org/	job summer	0.50
36790	72.85.251.119.1625109	72.85.251.119	2009-06-02 16:04:22.159506	ged	http://northshoreport.org/	ged	1.00
36791	72.85.251.119.1625109	72.85.251.119	2009-06-02 16:05:20.547886	student assistance	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801441;search_phrase=ged	assist student	0.50
36796	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:54:50.938135	bridgewell	http://www.northshoreport.org/	bridgewel	0.00
36797	134.241.194.113.1435109	134.241.194.113	2009-06-03 14:55:16.512351	after school program	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	after school	1.00
36812	72.85.251.119.1045109	72.85.251.119	2009-06-04 10:50:19.280333	art	http://www.northshoreport.org/	art	1.00
36813	129.63.96.191.1145109	129.63.96.191	2009-06-04 11:58:13.606179	test	http://northshoreport.org/	test	0.00
36814	129.63.96.191.1245109	129.63.96.191	2009-06-04 12:23:29.330858	Adult	http://northshoreport.org/cgi-bin/mvhub/guide.pl	adult	1.00
36815	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:13:55.514103	homelessness	http://northshoreport.org/	homeless	1.00
36816	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:15:07.830867	eviction	http://northshoreport.org/	evict	0.00
36817	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:15:22.652721	notice to quit	http://northshoreport.org/cgi-bin/mvhub/guide.pl	notic quit	0.50
36818	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:15:35.593649	housing search	http://northshoreport.org/cgi-bin/mvhub/guide.pl	hous search	0.50
36819	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:17:51.327761	adult education	http://northshoreport.org/cgi-bin/mvhub/guide.pl	adult educ	1.00
36820	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:22:12.202378	housing court	http://northshoreport.org/cgi-bin/mvhub/guide.pl	court hous	0.50
36821	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:23:26.417176	hiv/aids	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802970;search_phrase=housing%20court	aid hiv	0.50
36822	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:24:24.924718	hiv	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801914;search_phrase=hiv%2Faids	hiv	1.00
36824	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:27:22.023393	emergency shelter	http://northshoreport.org/cgi-bin/mvhub/guide.pl	emerg shelter	0.50
36825	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:28:07.04269	library	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802002;search_phrase=emergency%20shelter	librari	1.00
36826	66.80.237.78.1445109	66.80.237.78	2009-06-04 14:29:14.381612	dress for success	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=800220;search_phrase=library	dress success	0.00
36828	216.41.1.226.1555109	216.41.1.226	2009-06-05 15:05:37.900916	health quarters	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=browse_one_heading&heading_name=Health%2FHealth%20Insurance	health quarter	0.50
36833	205.237.102.178.1485109	205.237.102.178	2009-06-08 14:58:03.559737	financial literacy	http://www.northshoreport.org/	financi literaci	1.00
36834	205.237.102.178.1485109	205.237.102.178	2009-06-08 14:58:42.724996	financial literacy classes	http://www.northshoreport.org/	class financi literaci	0.67
36837	75.147.0.53.1685109	75.147.0.53	2009-06-08 16:08:12.707036	rental assistance	http://northshoreport.org/	assist rental	1.00
36840	24.147.109.86.1685109	24.147.109.86	2009-06-08 16:14:21.578076	job training	http://northshoreport.org/	job train	1.00
36841	24.147.109.86.1685109	24.147.109.86	2009-06-08 16:14:59.249127	food	http://northshoreport.org/	food	1.00
36843	75.147.0.53.1685109	75.147.0.53	2009-06-08 16:19:22.03165	need medication in between being re-enrolled in health coverage	http://northshoreport.org/	be between coverag enrol health medic ne re	0.13
36844	75.147.0.53.1685109	75.147.0.53	2009-06-08 16:22:09.137198	rental assistance for folk that cannot afford to pay total rent	http://northshoreport.org/	afford assist cannot folk pai rent rental total	0.25
36845	65.217.255.2.1685109	65.217.255.2	2009-06-08 16:43:50.583402	shelter	http://northshoreport.org/	shelter	1.00
36846	75.144.136.113.1785109	75.144.136.113	2009-06-08 17:03:43.180069	Job Training	http://northshoreport.org/	job train	1.00
36850	76.19.74.173.2285109	76.19.74.173	2009-06-08 22:03:08.381284	food pantry	http://www.northshoreport.org/	food pantri	1.00
36851	76.19.74.173.2285109	76.19.74.173	2009-06-08 22:10:17.490429	st vincent de paul society	http://www.northshoreport.org/	paul societi st vincent	0.00
36852	76.19.74.173.2285109	76.19.74.173	2009-06-08 22:10:34.808248	st john the baptist church	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	baptist church john st	0.00
36853	206.40.172.98.895109	206.40.172.98	2009-06-09 08:47:14.53513	family counseling	http://www.northshoreport.org/	counsel famili	0.50
36854	206.40.172.98.895109	206.40.172.98	2009-06-09 08:48:08.217329	public transportation	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	public transport	0.50
36855	206.40.172.98.895109	206.40.172.98	2009-06-09 08:49:02.786921	transportation	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	transport	1.00
36856	206.40.172.98.895109	206.40.172.98	2009-06-09 08:49:13.102649	funding	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	fund	1.00
36860	68.162.222.204.995109	68.162.222.204	2009-06-09 09:01:35.594266	Independent living center of the North shore and cape ann	http://northshoreport.org/cgi-bin/mvhub/guide.pl	and ann cape center independ live north shore	0.13
36862	129.63.96.191.1095109	129.63.96.191	2009-06-09 10:18:43.169862	test	http://northshoreport.org/	test	0.00
36864	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:19:26.863795	job'	http://northshoreport.org/	job	1.00
36866	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:20:15.176951	gay	http://northshoreport.org/cgi-bin/mvhub/guide.pl	gai	1.00
36867	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:21:51.791116	hate crime	http://northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=801782;search_phrase=job	crime hate	0.00
36868	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:21:57.458791	crime	http://northshoreport.org/cgi-bin/mvhub/guide.pl	crime	0.00
36869	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:22:05.126666	counselingf	http://northshoreport.org/cgi-bin/mvhub/guide.pl	counselingf	0.00
36870	66.80.237.78.1395109	66.80.237.78	2009-06-09 13:22:07.547005	counseling	http://northshoreport.org/cgi-bin/mvhub/guide.pl	counsel	1.00
36873	205.237.102.178.1395109	205.237.102.178	2009-06-09 13:54:06.969676	ESOL	http://www.northshoreport.org/	esol	1.00
36874	205.237.102.178.1495109	205.237.102.178	2009-06-09 14:35:42.874115	home visits	http://www.northshoreport.org/	home visit	1.00
36875	205.237.102.178.1495109	205.237.102.178	2009-06-09 14:36:31.970319	fathers' support	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_program_results;category_id=802508;search_phrase=home%20visits	father support	0.50
36878	70.22.224.196.1895109	70.22.224.196	2009-06-09 18:19:59.943411	Triangle	http://northshoreport.org/cgi-bin/mvhub/guide.pl	triangl	0.00
36879	70.22.224.196.1895109	70.22.224.196	2009-06-09 18:20:09.010592	Employment	http://northshoreport.org/cgi-bin/mvhub/guide.pl	employ	1.00
36882	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:45:56.923662	food	http://www.northshoreport.org/	food	1.00
36883	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:51:30.625638	career	http://www.northshoreport.org/	career	1.00
36884	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:51:38.918393	esl	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	esl	1.00
36886	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:53:27.057853	esol	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	esol	1.00
36893	74.10.197.220.14105109	74.10.197.220	2009-06-10 14:58:01.015857	threshold	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl	threshold	0.00
36895	74.10.197.220.15105109	74.10.197.220	2009-06-10 15:19:16.545944	library	http://www.northshoreport.org/	librari	1.00
36898	74.10.197.220.15105109	74.10.197.220	2009-06-10 15:24:53.47488	internet	http://www.northshoreport.org/cgi-bin/mvhub/guide.pl?rm=show_agency&agency_id=103850	internet	2.00
36902	129.63.96.180.15148109	129.63.96.180	2009-09-14 15:05:20.307723	shelter	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	shelter	1.00
36903	129.63.96.180.15148109	129.63.96.180	2009-09-14 15:05:21.422048	trees	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	tree	0.00
36907	129.63.96.180.9158109	129.63.96.180	2009-09-15 09:20:21.107221	shelter	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	shelter	1.00
36908	129.63.96.180.9158109	129.63.96.180	2009-09-15 09:20:22.230166	trees	http://nsp.mansi.testing123.net/cgi-bin/mvhub/guide.pl	tree	0.00
\.


--
-- Data for Name: date_agency_created; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY date_agency_created (agency_id, date_created) FROM stdin;
103520	2009-02-11
103531	2009-02-26
103542	2009-02-27
103553	2009-03-05
103575	2009-03-10
103586	2009-03-11
103597	2009-03-17
103608	2009-03-19
103619	2009-03-20
103630	2009-03-20
103641	2009-03-24
103652	2009-03-26
103663	2009-04-03
103674	2009-04-17
103685	2009-04-21
103696	2009-04-22
103707	2009-04-27
103718	2009-04-27
103729	2009-04-29
103740	2009-04-29
103751	2009-04-29
103762	2009-04-29
103773	2009-05-14
103784	2009-05-15
103795	2009-05-15
103806	2009-05-16
103817	2009-05-20
103828	2009-05-26
103839	2009-05-27
103850	2009-05-28
103861	2009-06-01
103872	2009-06-02
103883	2009-06-08
103894	2009-06-08
103905	2009-06-08
103916	2009-06-09
103927	2009-06-10
\.


--
-- Data for Name: date_program_created; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY date_program_created (program_id, date_created) FROM stdin;
509460	2009-02-11
509471	2009-02-11
509482	2009-02-11
509493	2009-02-11
509504	2009-02-11
509515	2009-02-11
509526	2009-02-11
509537	2009-02-11
509548	2009-02-26
509559	2009-02-27
509570	2009-03-05
509581	2009-03-05
509603	2009-03-10
509614	2009-03-11
509625	2009-03-12
509636	2009-03-16
509647	2009-03-16
509658	2009-03-17
509669	2009-03-19
509680	2009-03-20
509691	2009-03-20
509702	2009-03-24
509713	2009-03-26
509724	2009-04-01
509735	2009-04-03
509746	2009-04-10
509757	2009-04-14
509768	2009-04-14
509779	2009-04-14
509790	2009-04-14
509801	2009-04-14
509812	2009-04-14
509823	2009-04-17
509834	2009-04-17
509845	2009-04-21
509856	2009-04-21
509867	2009-04-21
509878	2009-04-21
509889	2009-04-21
509900	2009-04-21
509911	2009-04-21
509933	2009-04-22
509944	2009-04-22
509955	2009-04-22
509966	2009-04-22
509988	2009-04-22
510010	2009-04-22
510032	2009-04-22
510043	2009-04-22
510054	2009-04-27
510065	2009-04-27
510076	2009-04-29
510087	2009-04-29
510098	2009-04-29
510109	2009-04-29
510120	2009-04-29
510131	2009-04-29
510142	2009-04-29
510153	2009-04-29
510164	2009-04-30
510175	2009-04-30
510186	2009-05-06
510197	2009-05-12
510208	2009-05-13
510219	2009-05-15
510230	2009-05-15
510241	2009-05-15
510252	2009-05-16
510263	2009-05-18
510274	2009-05-26
510285	2009-05-27
510296	2009-05-28
510307	2009-05-28
510318	2009-05-28
510329	2009-05-28
510340	2009-06-01
510351	2009-06-02
510362	2009-06-08
510373	2009-06-09
510384	2009-06-09
510395	2009-06-09
510406	2009-06-09
510417	2009-06-09
\.


--
-- Data for Name: heading; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY heading (heading_name) FROM stdin;
Arts/Culture/Entertainment
City/Town Services
Community/Civic
Counseling
Crisis/Abuse/Rape
Disabled/Handicapped
Education/Training
Employment
Financial Education/Credit
Food/Shelter & Financial Assistance
Health Education/Fitness/Well-being
Health/Health Insurance
Housing
Immigrant Services
Legal/Consumer Protection
Mental Health
Other
Parenting/Family
Public Safety
Senior/Elderly
Sports/Recreation
Substance Abuse/Addiction
Youth
\.


--
-- Data for Name: heading_category; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY heading_category (category_id, heading_name) FROM stdin;
800000	Arts/Culture/Entertainment
800022	Arts/Culture/Entertainment
800033	Arts/Culture/Entertainment
800044	Arts/Culture/Entertainment
800055	Arts/Culture/Entertainment
800066	Arts/Culture/Entertainment
800077	Arts/Culture/Entertainment
800088	City/Town Services
800099	City/Town Services
800110	City/Town Services
800121	City/Town Services
800132	City/Town Services
800143	City/Town Services
800154	City/Town Services
800165	City/Town Services
800176	City/Town Services
800187	City/Town Services
800198	Community/Civic
800209	Community/Civic
800220	Community/Civic
800231	Community/Civic
800242	Community/Civic
800253	Community/Civic
800264	Community/Civic
800275	Community/Civic
800143	Community/Civic
800286	Community/Civic
800297	Community/Civic
800308	Community/Civic
800319	Counseling
800330	Counseling
800341	Counseling
800352	Counseling
800363	Counseling
800374	Counseling
800385	Counseling
800396	Counseling
800407	Counseling
800418	Counseling
800429	Counseling
800440	Counseling
800451	Counseling
800462	Counseling
800473	Counseling
800484	Counseling
800495	Counseling
800517	Counseling
800528	Counseling
800539	Crisis/Abuse/Rape
800561	Crisis/Abuse/Rape
800572	Crisis/Abuse/Rape
800583	Crisis/Abuse/Rape
800594	Crisis/Abuse/Rape
800605	Crisis/Abuse/Rape
800616	Crisis/Abuse/Rape
800627	Crisis/Abuse/Rape
800638	Crisis/Abuse/Rape
800649	Crisis/Abuse/Rape
800660	Crisis/Abuse/Rape
800671	Crisis/Abuse/Rape
800682	Crisis/Abuse/Rape
800693	Crisis/Abuse/Rape
800704	Crisis/Abuse/Rape
800715	Crisis/Abuse/Rape
800726	Crisis/Abuse/Rape
800737	Crisis/Abuse/Rape
800748	Crisis/Abuse/Rape
800759	Crisis/Abuse/Rape
800770	Crisis/Abuse/Rape
800781	Crisis/Abuse/Rape
800792	Crisis/Abuse/Rape
800803	Crisis/Abuse/Rape
800517	Crisis/Abuse/Rape
800814	Disabled/Handicapped
800825	Disabled/Handicapped
800836	Disabled/Handicapped
800847	Disabled/Handicapped
800858	Disabled/Handicapped
800869	Disabled/Handicapped
800880	Disabled/Handicapped
800891	Disabled/Handicapped
800902	Disabled/Handicapped
800913	Disabled/Handicapped
800924	Disabled/Handicapped
800935	Disabled/Handicapped
800946	Disabled/Handicapped
800957	Disabled/Handicapped
800968	Disabled/Handicapped
800979	Disabled/Handicapped
800990	Disabled/Handicapped
801001	Disabled/Handicapped
801012	Disabled/Handicapped
801023	Disabled/Handicapped
801034	Disabled/Handicapped
801045	Disabled/Handicapped
801056	Disabled/Handicapped
800682	Disabled/Handicapped
801067	Disabled/Handicapped
801078	Disabled/Handicapped
801089	Disabled/Handicapped
801100	Disabled/Handicapped
801111	Disabled/Handicapped
801122	Disabled/Handicapped
801133	Disabled/Handicapped
801144	Disabled/Handicapped
801155	Disabled/Handicapped
801166	Disabled/Handicapped
801177	Education/Training
801188	Education/Training
801199	Education/Training
801210	Education/Training
801232	Education/Training
801243	Education/Training
801254	Education/Training
801265	Education/Training
801276	Education/Training
801287	Education/Training
800220	Education/Training
801298	Education/Training
801309	Education/Training
801320	Education/Training
801331	Education/Training
801342	Education/Training
801353	Education/Training
801364	Education/Training
801375	Education/Training
801386	Education/Training
800913	Education/Training
800935	Education/Training
801397	Education/Training
800968	Education/Training
801419	Education/Training
801430	Education/Training
801441	Education/Training
801452	Education/Training
801463	Education/Training
801474	Education/Training
801485	Education/Training
801496	Education/Training
801507	Education/Training
801518	Education/Training
801067	Education/Training
801529	Education/Training
801540	Education/Training
801551	Education/Training
801562	Education/Training
801573	Education/Training
801584	Education/Training
801595	Education/Training
801606	Education/Training
801617	Education/Training
801628	Education/Training
801177	Employment
801639	Employment
801650	Employment
801661	Employment
801672	Employment
801683	Employment
801694	Employment
801705	Employment
801716	Employment
801727	Employment
801045	Employment
801738	Employment
801749	Employment
801760	Employment
801771	Employment
801309	Employment
801782	Employment
801793	Employment
801804	Employment
801815	Employment
801826	Employment
801837	Employment
801848	Employment
801859	Employment
801870	Employment
801881	Financial Education/Credit
801892	Financial Education/Credit
801903	Financial Education/Credit
801914	Financial Education/Credit
801925	Financial Education/Credit
801936	Financial Education/Credit
801947	Financial Education/Credit
801958	Financial Education/Credit
801969	Financial Education/Credit
801980	Financial Education/Credit
801991	Financial Education/Credit
802002	Food/Shelter & Financial Assistance
802013	Food/Shelter & Financial Assistance
802024	Food/Shelter & Financial Assistance
802035	Food/Shelter & Financial Assistance
802046	Food/Shelter & Financial Assistance
801683	Food/Shelter & Financial Assistance
802057	Food/Shelter & Financial Assistance
802068	Food/Shelter & Financial Assistance
802079	Food/Shelter & Financial Assistance
802090	Food/Shelter & Financial Assistance
802101	Food/Shelter & Financial Assistance
802112	Food/Shelter & Financial Assistance
802123	Food/Shelter & Financial Assistance
802134	Food/Shelter & Financial Assistance
801034	Food/Shelter & Financial Assistance
802145	Food/Shelter & Financial Assistance
802156	Food/Shelter & Financial Assistance
802167	Food/Shelter & Financial Assistance
802178	Food/Shelter & Financial Assistance
802189	Food/Shelter & Financial Assistance
802200	Food/Shelter & Financial Assistance
802222	Food/Shelter & Financial Assistance
802233	Food/Shelter & Financial Assistance
802255	Food/Shelter & Financial Assistance
802266	Food/Shelter & Financial Assistance
802277	Food/Shelter & Financial Assistance
802288	Food/Shelter & Financial Assistance
802299	Health Education/Fitness/Well-being
802310	Health Education/Fitness/Well-being
802321	Health Education/Fitness/Well-being
802332	Health Education/Fitness/Well-being
802343	Health Education/Fitness/Well-being
801397	Health Education/Fitness/Well-being
802354	Health Education/Fitness/Well-being
802365	Health Education/Fitness/Well-being
802376	Health Education/Fitness/Well-being
802387	Health Education/Fitness/Well-being
801287	Health Education/Fitness/Well-being
802398	Health Education/Fitness/Well-being
802409	Health Education/Fitness/Well-being
802420	Health Education/Fitness/Well-being
802431	Health Education/Fitness/Well-being
802442	Health Education/Fitness/Well-being
802453	Health Education/Fitness/Well-being
802464	Health Education/Fitness/Well-being
802475	Health Education/Fitness/Well-being
802486	Health/Health Insurance
802497	Health/Health Insurance
802519	Health/Health Insurance
802332	Health/Health Insurance
802530	Health/Health Insurance
802541	Health/Health Insurance
802552	Health/Health Insurance
802563	Health/Health Insurance
802574	Health/Health Insurance
802585	Health/Health Insurance
802596	Health/Health Insurance
800385	Health/Health Insurance
802607	Health/Health Insurance
802618	Health/Health Insurance
802354	Health/Health Insurance
802629	Health/Health Insurance
802640	Health/Health Insurance
802376	Health/Health Insurance
802651	Health/Health Insurance
802662	Health/Health Insurance
802673	Health/Health Insurance
802684	Health/Health Insurance
802695	Health/Health Insurance
802706	Health/Health Insurance
802717	Health/Health Insurance
802728	Health/Health Insurance
802739	Health/Health Insurance
802420	Health/Health Insurance
802750	Health/Health Insurance
802453	Health/Health Insurance
802761	Health/Health Insurance
801166	Health/Health Insurance
802772	Health/Health Insurance
802783	Health/Health Insurance
802794	Health/Health Insurance
802805	Health/Health Insurance
802816	Health/Health Insurance
802321	Health/Health Insurance
802827	Health/Health Insurance
802838	Health/Health Insurance
802387	Health/Health Insurance
802849	Health/Health Insurance
802860	Health/Health Insurance
802871	Health/Health Insurance
802442	Health/Health Insurance
802882	Health/Health Insurance
802893	Housing
802904	Housing
800891	Housing
802915	Housing
802926	Housing
802937	Housing
802948	Housing
802959	Housing
802970	Housing
802981	Housing
802992	Housing
803003	Housing
803014	Housing
803025	Housing
802222	Housing
803036	Housing
803047	Housing
803058	Housing
803069	Housing
802277	Housing
803080	Housing
803091	Housing
803102	Housing
803113	Housing
803124	Immigrant Services
803135	Immigrant Services
803146	Immigrant Services
803157	Immigrant Services
803168	Immigrant Services
803179	Immigrant Services
803190	Immigrant Services
803201	Immigrant Services
801243	Immigrant Services
803212	Immigrant Services
803223	Immigrant Services
803234	Immigrant Services
803245	Immigrant Services
803256	Immigrant Services
803267	Legal/Consumer Protection
803278	Legal/Consumer Protection
803289	Legal/Consumer Protection
803300	Legal/Consumer Protection
801749	Legal/Consumer Protection
803311	Legal/Consumer Protection
803322	Legal/Consumer Protection
803333	Legal/Consumer Protection
803344	Legal/Consumer Protection
803355	Legal/Consumer Protection
803366	Legal/Consumer Protection
803377	Legal/Consumer Protection
803388	Legal/Consumer Protection
803399	Legal/Consumer Protection
803410	Legal/Consumer Protection
803421	Mental Health
800341	Mental Health
800374	Mental Health
803432	Mental Health
803443	Mental Health
803454	Mental Health
803465	Mental Health
803476	Mental Health
803487	Mental Health
803498	Mental Health
801078	Mental Health
803509	Mental Health
803520	Mental Health
801155	Mental Health
803531	Mental Health
803542	Mental Health
803553	Mental Health
803564	Mental Health
803575	Other
803586	Other
803597	Other
803608	Other
803619	Other
803630	Other
803641	Other
803652	Other
803663	Other
803674	Parenting/Family
803685	Parenting/Family
803696	Parenting/Family
801232	Parenting/Family
803718	Parenting/Family
803729	Parenting/Family
803740	Parenting/Family
800693	Parenting/Family
803751	Parenting/Family
803289	Parenting/Family
803762	Parenting/Family
803773	Parenting/Family
803784	Parenting/Family
802860	Parenting/Family
803795	Parenting/Family
800473	Parenting/Family
803806	Parenting/Family
800209	Public Safety
803817	Public Safety
803828	Public Safety
802937	Public Safety
803839	Public Safety
800121	Public Safety
803850	Public Safety
803861	Public Safety
803872	Public Safety
803883	Public Safety
803894	Public Safety
803905	Public Safety
800682	Senior/Elderly
803916	Senior/Elderly
803927	Senior/Elderly
803938	Senior/Elderly
803949	Senior/Elderly
803960	Senior/Elderly
801936	Senior/Elderly
801144	Senior/Elderly
803971	Senior/Elderly
801034	Senior/Elderly
803982	Senior/Elderly
803993	Senior/Elderly
804004	Senior/Elderly
804015	Sports/Recreation
804026	Sports/Recreation
804037	Sports/Recreation
804048	Sports/Recreation
804059	Sports/Recreation
804070	Sports/Recreation
804081	Sports/Recreation
804092	Sports/Recreation
804103	Sports/Recreation
802365	Sports/Recreation
804114	Sports/Recreation
804136	Sports/Recreation
802409	Sports/Recreation
804147	Sports/Recreation
804158	Sports/Recreation
804169	Sports/Recreation
804180	Substance Abuse/Addiction
804191	Substance Abuse/Addiction
804202	Substance Abuse/Addiction
804213	Substance Abuse/Addiction
804224	Substance Abuse/Addiction
804235	Substance Abuse/Addiction
804246	Substance Abuse/Addiction
804257	Substance Abuse/Addiction
804268	Substance Abuse/Addiction
804279	Substance Abuse/Addiction
804290	Substance Abuse/Addiction
804301	Substance Abuse/Addiction
804312	Substance Abuse/Addiction
804323	Substance Abuse/Addiction
804334	Substance Abuse/Addiction
804345	Substance Abuse/Addiction
804356	Substance Abuse/Addiction
804367	Substance Abuse/Addiction
804378	Substance Abuse/Addiction
804389	Substance Abuse/Addiction
804400	Substance Abuse/Addiction
804411	Youth
803696	Youth
801210	Youth
804422	Youth
801232	Youth
803861	Youth
804433	Youth
802354	Youth
801265	Youth
804444	Youth
804466	Youth
800693	Youth
804477	Youth
804488	Youth
803839	Youth
804499	Youth
804510	Youth
804521	Youth
804532	Youth
804543	Youth
801342	Youth
804554	Youth
804565	Youth
802475	Youth
804576	Youth
804587	Youth
804598	Youth
804609	Youth
804620	Youth
804631	Youth
804642	Youth
801463	Youth
804664	Youth
801485	Youth
804675	Youth
804686	Youth
804697	Youth
804708	Youth
801529	Youth
804719	Youth
802860	Youth
804730	Youth
804741	Youth
801595	Youth
801606	Youth
801760	Youth
804785	Youth
804796	Youth
804807	Youth
804785	Arts/Culture/Entertainment
804796	Arts/Culture/Entertainment
804807	Arts/Culture/Entertainment
804818	Sports/Recreation
804829	Sports/Recreation
804840	Arts/Culture/Entertainment
804851	Arts/Culture/Entertainment
804862	Arts/Culture/Entertainment
804884	Health/Health Insurance
804895	Health/Health Insurance
804906	Health/Health Insurance
804818	Senior/Elderly
804917	Senior/Elderly
804917	Housing
804950	Education/Training
804961	Financial Education/Credit
804972	Counseling
802508	Parenting/Family
800770	Counseling
804983	Other
804994	Community/Civic
805005	Health/Health Insurance
805016	Health Education/Fitness/Well-being
802244	Food/Shelter & Financial Assistance
805027	Sports/Recreation
801221	Education/Training
801221	Employment
801408	Education/Training
801408	Employment
804774	Parenting/Family
804774	Youth
804653	Parenting/Family
804653	Youth
804763	Parenting/Family
804763	Youth
804752	Parenting/Family
804752	Youth
803707	Parenting/Family
800550	Crisis/Abuse/Rape
805038	Health/Health Insurance
800011	Arts/Culture/Entertainment
802211	Food/Shelter & Financial Assistance
\.


--
-- Data for Name: log_category_search; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY log_category_search (search_id, session_id, ip_address, "timestamp", phrase_stems, top_score, search_phrase) FROM stdin;
\.


--
-- Data for Name: meta_data_version; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY meta_data_version (version, change_date, note) FROM stdin;
0	2009-02-25	created this table
1	2009-02-25	display TRUE north shore towns in table: town / add magnolia to town table
\.


--
-- Data for Name: meta_schema_version; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY meta_schema_version (version, change_date, note) FROM stdin;
1	2009-02-25	created tables: meta_schema_version meta_data_version
\.


--
-- Data for Name: program; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY program (program_id, agency_id, last_updated, hidden_reason, program_name, program_alias, address1, address2, city, state, zip, main_phone, toll_free_phone, hotline, fax, website, email, short_program_description, long_program_description, eligibility, fees_charged, fees_explained, program_schedule, hours, languages, appointment, bring, bus_lines, parking, transportation, handicapped_accessible, child_care_provided, child_care_description, wait_list, wait_list_description, other_information, mail_address1, mail_address2, mail_city, mail_state, mail_zip, volunteer_opportunities, volunteer_information, resource_library, resource_library_description, meeting_space, meeting_space_description, news_letter, news_letter_description, public_net_access, public_net_access_description, contact_first_name, contact_last_name, contact_phone, contact_fax, contact_email, contact_email_available, area_type, area_data, tty_tdd, areas_served_description, languages_description, online_registration, reminders_sent, admin_notes) FROM stdin;
509614	103586	2009-03-12	\N	Action, Inc. Employment and Training	\N	180 Main Street	\N	Gloucester	MA	01930	978-282-1000	\N	\N	978-283-0523	www.actioninc.org	public_program_email@testing123.net	Employment and Training	Career counseling, resume writing,job interviewing skills, job placement, computer classes, life skills and medical assistant training.	Residents of Cape Ann including Gloucester, Rockport, Manchester by the Sea, Essex, and Ipswich. Focused on helping low-income individuals and families become self-sufficient.	Yes	Small fee charged for computer classes only	Open year round	Business Hours Monday through Friday 9:00 to 5:00 p.m.	English!DELIM!Spanish	Yes	\N		On street metered parking	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	Not Applicable	\N	Yes	Conference Room with Seating for 20+	No	\N	Limited	Offered to job seekers	Maggie	Meffen	978-282-1000 x103	978-283-0528	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Essex County, MA!DELIM!Gloucester, MA	\N	\N	\N	No	0	\N
509702	103586	2009-03-24	\N	QUEST	\N	180 Main Street	\N	Gloucester	MA	01930	978-282-1000 X104	\N	\N	978-283-0523	www.actioninc.org	public_program_email@testing123.net	Subsidized Housing w/ Supportive Services and Referrals	You must be on lists for permanent housing including JRI and CTI, if you are not we will direct you.  Once you get subsidized housing with Action, we hope you get permanent housing within three years, if you don't and you are in compliance with the program you may keep the subsidy.	HIV positive and income eligible	Yes	30% or less of your income.  This percentage would be your monthly rental co-pay	Year round	Monday thru Friday 10:00 to 4:00	English	Yes	I.D. -  Social Security card  -  Health Insurance  -  Proof of Income  -  Diagnosis letter on Doctors Letterhead		\N	\N	Yes	No	\N	Yes	After in-take you will be put on waiting list for housing	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Yes	\N	No	\N	No	\N	Maggie	Meffen	978-282-1000 x103	978-283-0528	chester@testing123.net	Yes	COUNTY	Essex County	\N	\N	May be someone available for spanish speaking clients	No	0	\N
509944	103696	2009-04-22	\N	Shelter, Housing and Economic Stability	\N	27 Congress Street	\N	Salem	MA	01970	978-744-8552	800-547-1649	800-547-1649	978-745-6886	www.hawcdv.org	public_program_email@testing123.net	Shelter and transitional housing for victims of abuse	Emergency shelter, scattered site transitional housing and economic stability support for families leaving abusive relationships	Must be victim of domestic abuse for shelter; must be client of HAWC for transitional housing assistance	Sliding Scale	Emergency shelter is free; tenant pays partial rent for transitional housing based on income	Open year round	Shelter is staffed 24 hours a day; 7 days a week; Housing support services by appointment	English!DELIM!Portuguese!DELIM!Spanish	No	No appointment for emergency shelter but appointment required for housing assistance		Available	\N	Yes	No	\N	Yes	Shelter is frequently full	Call Safelink to find available emergency shelter in MA for domestic abuse\n1-800-785-2020	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	Salem, Lynn, Gloucester, Beverly and Ipswich office locations	Yes	Twice annually	No	\N	Candace	Waldron	978-744-2299 ext 12	978-745-6886	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
509559	103542	2009-05-13	\N	Children's Law Center	\N	298 Union Street	\N	Lynn	MA	01901	781-581-1977	\N	\N	\N	www.clcm.org	public_program_email@testing123.net	Legal representation for low-income children.	The Children's Law Center represents young people in a variety of legal and administrative proceedings including education, child welfare and juvenile justice.\nInformation, referrals and representation available for education matters, suspensions, expulsions, special education, and other school-related issues, child welfare, delinquencies, abuse & neglect, and CHINS proceedings. \r\n	Eligible clients are low-income children under the age of 18, or under 22 for special education matters.	Free	\N	Year-round.	Monday - Friday\n9am - 5pm	English!DELIM!Spanish	No	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	Not Applicable	\N	No	\N	Gardner	Casey	781-244-1428	\N	chester@testing123.net	No	COUNTY	Middlesex County!DELIM!Essex County	\N	Essex County.\nLimited Middlesex County. \r\nRevere, Chelsea and Saugus in addition.	\N	No	0	\N
510395	103916	2009-06-09	\N	Young Parents	\N	55 Lynn Shore Drive	\N	Lynn	MA	01902	781-593-2312	\N	\N	781-581-3270	www.ccab.org	public_program_email@testing123.net	Support groups, counseling  and parenting education	We offer home visits, counseling, support groups, case management and parenting education for young parents	23 years of age and younger	Free	\N	Open year round	Monday - Friday, 8:30 am - 5:00 pm	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!North Reading, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Also Arlington, Chelsea, Everett, Malden, Medford, Melrose, Reading, Revere, Stoneham, Wakefield, Winchester and Woburn	\N	No	0	\N
509636	103586	2009-03-16	\N	Action, Inc Volunteer Income Tax Site	\N	180 Main Street	\N	Gloucester	MA	01930	978-282-1000	\N	\N	\N	www.actioninc.org	public_program_email@testing123.net	Tax Preparation Services	Starting in January, volunteers at Action begin preparing tax returns for low-income households. The service is free. We file electronically. Volunteers are trained and certified by the IRS.	Low-income Cape Ann Residents.	Free	\N	January through April.	Wednesdays 5-9pm	English!DELIM!Spanish	Yes	Identification, Social Security Card, and any pertinent tax documents.		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	\N	No	\N	Limited	\N	Maggie	Meffen	978-282-1000 x103	978-283-0528	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Gloucester, MA	\N	Essex	\N	No	0	\N
509966	103641	2009-04-22	\N	Summer Park Lunch Program	Park Lunch	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Delivery of lunches to area parks for children	Our Summer Park Lunch Program is an 8 week outreach to local children who are at risk of going without a noon meal. Bootstraps supplies seven strategically located parks with peanut butter and jelly sandwiches, fruit, snack, juice boxes and granola bars	Must be attending a recreational department program at one of the parks listed below.	Free	\N	Operates for 8 weeks during the summer	Monday through Friday	English	No	\N		Not applicable	not applicable	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are needed to put the lunches together and distribute them to the area parks.	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Kathy	Kirlis	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA	\N	We work with the Beverly Recreation Department to offer our lunch program at the following parks:\n* Ahearn Park\r\n* Hannah's School\r\n    * Bessie Baker\r\n    * Bartlett Gardens\r\n    * Balch Street\r\n    * Holcroft Park\r\n    * Apple Village\r\n	\N	No	0	\N
510109	103751	2009-04-29	\N	Salem Family Health Center	\N	47 Congress St.	\N	Salem	MA	01970	978-744-8388	\N	\N	\N	www.nschi.org	public_program_email@testing123.net	Health and dental care	Health and dental care provided for family members at all stages of life.  Translation services, financial counseling, and health education provided.	No one will be denied services based upon ability to pay.	Yes	We accept most major private insurances, medicaid, medicare and the health safety net.	Open year round	Monday, Thursday and Friday 8:30 to 5:00.  Evening hours Tuesday and Wednesday until 7:00 pm.	English!DELIM!Portuguese!DELIM!Spanish	Yes	Insurance information		On-site parking available	Accessible by public transportation	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Lisa	Schott	978-282-8899 ext. 214	978-282-5599	chester@testing123.net	Yes	COUNTY	Essex County	\N	\N	\N	No	0	\N
509548	103531	2009-02-26	\N	Northshore Pathways Project	\N	156 Broad Street	\N	Lynn, Ma.	MA	01905	781-599-8543	\N	\N	\N	www.operationbootstrap-lynn.org	public_program_email@testing123.net	This is a 30 week pre-vocational training program.	We currently are offering pre-vocational training for people interested in becoming nurse assistants.\nThis Program provides low skill adults with the basic skills they need to enter and advance in post secondary education and career training. \r\n\r\nNorth Shore Pathways Project is a pilot program headed by the North Shore Workforce Investment Board and Operation Bootstrap with other community partners. 	Legal US Resident\nAdvanced English Language Skills 	Free	\N	Runs in 30 week secrements	Monday thru Friday\n5:00 - 8:00 pm	English	Yes	Proof of legal US residency\nSocial Security Card\r\nDriver's License\r\nCheck or Money Order for $25 to send out to request a CORI (Criminal Background Check)\r\nA letter of intent explaining why you would be a good student for our program		\N	\N	Not Applicable	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	Our resource library consists of materials to support the learning process for ESL, ABE, Citizenship and Literacy students.	Not Applicable	\N	Not Applicable	\N	No	\N	Krystal	de Leon	781-913-5604	\N	chester@testing123.net	Yes	COUNTY	Essex County	\N	\N	\N	No	0	\N
510329	103850	2009-06-10	\N	Public Internet Access	\N	82 Main Street	\N	Peabody	MA	01960	978-531-0100	\N	\N	978-532-1797	www.peabodylibrary.org	public_program_email@testing123.net	Free internet access and wireless available.	The Library has public computers for adults, children and young adults.  Free wireless Internet access is available on the first floor of the Library.	Open to all.	Free	\N	Open year round	Monday - Thursday\n9 a.m.-9 p.m.\r\n(Children's Room 9 a.m.-8 p.m.)\r\nFriday\r\n9 a.m.-1 p.m.\r\nSaturday\r\n9 a.m.-5 p.m.\r\nSunday\r\n1-5 p.m.\r\nClosed Sundays Memorial Day through Labor Day 	English	No	To access the Internet computers, please bring your library card.  To obtain a library card bring proof of residency (driver's lisence, utility bill, checkbook, etc.)		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	Yes	The Peabody Library has large collection of books, DVDs and CDs.	Yes	The Peabody Library has several meeting rooms available for use by nonprofit or governmental agencies.  There is no fee for meeting room use. For complete procedures and to book a room call the library at 978-531-0100 ext. 11	Not Applicable	\N	Yes	All users must agree to abide by the Library's Acceptable Use Policy.	Melissa	Rauseo	978-531-0100 ext. 14	978-532-1797	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	Visitors and residents of other cities/ towns may use the Library's Internet access.	\N	No	0	\N
509570	103553	2009-03-05	\N	Job Corps - Region One	Job Corps	181 Union Street	\N	Lynn	MA	01901	781-593-0585 ex 3016	\N	781-913-1056	617-727-3712	bostonregion.jobcorps.gov	public_program_email@testing123.net	Career Training and Education	Job Corps is the nation's oldest and largest federally funded job training and education program for economically disadvantaged youth ages 16 - 24.  Job Corps is a year-round open-entry career training program that is self-paced and individualized in order to allow student to work at their own speed and plan for graduation.	Low-income individuals between the ages of 16 and 24 who are free of any court involvement.  The income and age levels may be waived for those with disabilities.	Free	Job Corps is funded by Congress through the Department of Labor.	Open-entry, open-exit, year round.	The Career Technical Training hours are Monday through Friday from 8:00 a.m. to 4:00 p.m.	English	Yes	Social Security Card, Birth Certificate, Photo I.D., Health Insurance Card, School Transcript/diploma/GED, immunization record, proof of low income and IEP if needed.		yes	Job Corps provides transportation at enrollment, and during the two annual breaks (2-1/2 weeks in July and 2-1/2 weeks in December).	Yes	No	Child care is not offered; however a stipend is offered those students who have children.	Yes	The waiting list varies by the Job Corps Centers and the Career Technical Training requested.	There are nine Job Corps Centers in this Region; Exeter Job Corps, Grafton Job Corps,Hartford Job Corps, Loring Job Corps, New Haven Job Corps, Northlands Job Corps, Penobscot Job Corps, Shriver Job Corps and Westover Job Corps.	\N	\N	\N	\N	\N	Yes	\N	Yes	Staff and students only; not open to the public.	Yes	Conference Rooms	Yes	Published Nationally	Yes	bostonregion.jobcorps.gov	Carol	Couch	781-593-0585 ex 3016	617-727-3712	chester@testing123.net	Yes	EVERYWHERE		\N	New England	\N	No	0	\N
509581	103553	2009-03-05	\N	Job Corps - Boston Region	Job Corps	181 Union Street	\N	Lynn	MA	01901	781-593-0585 ex 3016	\N	781-913-1056	617-727-3712	bostonregion.jobcorps.gov	public_program_email@testing123.net	Free Career Technical Training and Education	Job Corps is the nation's oldest and largest federally funded career training and education program.  Training is available year-round, open entry and self-paced.	Low income individuals between the ages of 16 and 24 - age and income levels are waived for individuals with disabilities.	Free	Job Corps is funded by Congress through the Department of Labor	Year-round, open entry/open exit; with 2-1/2 weeks off in July and December.	Career Technical Training and Education are offered Monday through Friday from 8:00 a.m. to 4:00 p.m.	English	Yes	Social Security Card, Photo ID, Birth Certificate, Health Insurance Card, High School Diploma/GED or transcript, Immunization Record, proof of low income and IEP if necessary.		yes	Job Corps provides transportation on enrollment and during breaks in December and July.	Yes	No	Job Corps offers parents a small stipend to assist with child care	Yes	The waiting list varies depending on the Job Corps Center and the Career Technical Training choice.	Training in: Health Occupations, Automotive, Business, Culinary Arts, Carpentry, Electrical and Network Wiring, Computers, Masonry, Painting, Plastering, Plumbing, Welding, Urban Forestry, CDL	\N	\N	\N	\N	\N	Yes	Job Corps students volunteer in local communities and at job sites for job-based learning and internships.	Yes	Staff and students only; not open to the public.	Yes	Conference Rooms	Yes	National Publication	Yes	bostonregion.jobcorps.gov	Carol	Couch	781-593-0585 ex 3016	617-727-3712	chester@testing123.net	Yes	EVERYWHERE		\N	Throughout New England	\N	No	0	\N
509603	103575	2009-03-10	\N	Beverly Affordable Housing Coalition	\N	234 Cabot Street, Suite 8	\N	Beverly	MA	01915	978-921-4705	\N	\N	978-922-2019	www.beverlyaffordablehousing.org	public_program_email@testing123.net	Affordable Rental Housing	The Coalition manages a limited number of affordable rental properties in Beverly.  To inquire about vacancies, please call our office at 978-921-4705.\nA larger number of other properties are managed by the YMCA of the North Shore and co-owned with the Coalition.  For more information about vacancies, call Steve Carter at the YMCA of the North Shore at 978-922-0990.	Eligibility is income dependent.  Please call for more information.	Yes	Below market rate rents.	Open year round	Monday to Friday 9:00 - 5:00	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer opportunities and internships are available.  Contact David Bresnahan at 978-921-4705 for more information.	Not Applicable	\N	Not Applicable	\N	Yes	To join our mailing list, please email David Bresnahan at bahc2@verizon.net.	No	\N	Mickey	Northcutt	978-921-4705	\N	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
509625	103586	2009-03-12	\N	ACTION, INC. FUEL ASSISTANCE PROGRAM	LIHEAP	47 WASHINGTON  STREET	\N	Gloucester	MA	01930	978-281-3900	1-800-696-9276	978-283-2131	978-281-1154	www.actioninc.org	public_program_email@testing123.net	HEATING ASSISTANCE PROGRAM	ASSISTANCE WITH HEATING BILLS DISCOUNTS ON UTILITY BILLS, WEATHERIZATION SERVICE, ENERGY AUDITS ON HOME OR APARTMENT, BURNER REPAIR OR REPLACEMENT ON HEATING SYSTEMS REPLACEMENT OF OLD OR ENERGY BURDEN REFRIDGERATORS AVAILABLE FREE OF CHARGE	all	Free	\N	program opens November 1 To April 30 for assistance\nopen year round	9am to 4:30pm	English!DELIM!Portuguese!DELIM!Spanish	No	proof of household income\nInformation on all household members 18years or older, proof of address 		street parking	\N	No	No	\N	No	first come first served also by appointment	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	office at 47 Washington St	Yes	Gloucester Times, Church bulletents	No	\N	Marie	Sanviti	978-282-1003 ext 224	978-281-1154	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	\N	\N	\N	No	0	\N
509647	103586	2009-03-16	\N	Action, Inc Advocacy Program	\N	180 Main Street	\N	Gloucester	MA	01930	978-282-1000	\N	\N	\N	www.actioninc.org	public_program_email@testing123.net	Advocacy and Housing	The Advocacy Program offers assistance with a wide range of issues to low-income individuals in the Cape Ann area. Housing Assistance: landlord/tenant conflict, tenants rights support,  referral to legal assistance, completion of subsidized housing applications and application process. Assistance obtaining public benefits such as Masshealth, Foodstamps, EAEDC, TAFDC, and Social Security. Advocacy also administers emergency rental assistance funds (when available).	Cape Ann residents.	Free	\N	Open year round	Monday - Friday 9am-5pm	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	Yes	\N	No	\N	Limited	\N	Maggie	Meffen	978-282-1000 x103	978-283-0528	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Gloucester, MA	\N	\N	\N	No	0	\N
509658	103597	2009-03-17	\N	Refugee School Impact	After-School Program	298 Union St	Lower Level	Lynn	MA	01902	781-593-0100	\N	\N	\N	\N	public_program_email@testing123.net	After-School Program to help students with homework.	College volunteers are available four days a week to help students on an individual or small-group basis with homework, studying, or special projects.\nOn Saturdays, students gather in the Lynn Commons to play soccer or at the Lynn YMCA for sports (during the winter months).  \r\n\r\nIn the summer, a small program, similar to an afternoon day camp, is run to help students have fun, continue English development, and make friends.  	Refugees and Political Aslyees.  Proof of one of these is required (proper immigration documents).  For the after-school program, students must be in grades 6-12 or ages 11 to 18.   Immigrants can be accepted to the after-school and summer programs, but are not eligible for other services.	Free	\N	Open year round	School year:\n-Monday and Wednesday, Middle School Homework Help, 3 to 5 pm\r\n    -Tuesday and Thursday, High School Homework Help, 3 to 5 pm\r\n     -Saturday, Sports Club, 12 to 2 pm\r\n\r\nSummer Hours, TBD	English!DELIM!French	No	I-94 (refugee immigration document)		metered	no transportation provided	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are always welcome to help students with homework.  Opportunities to volunteer for special events are available several times a year.  All volunteers are encouraged to be involved on a regular basis in order to promote positive, mentoring relationships with children.	No	\N	Yes	Three full classrooms, one computer one, and a break room are available for the program's use.	Yes	Our first newsletter is in the works!	Limited	\N	Amanda	Verbeke	781-593-0100 x. 16	781-599-3329	chester@testing123.net	No	TOWN	Lynn, MA!DELIM!Swampscott, MA	\N	All refugees and political aslyees on the North Shore are welcome, but most of our clients reside in Lynn or Swampscott.	Russian, Bosnian, Krio, Somali, Mai-Mai, Swahili, Hindi, Burmese, Karen, Nepali, Arabic	No	0	\N
509669	103608	2009-03-19	\N	Cambridge Family and Children's Service	\N	60 Gore Street	\N	Cambridge	MA	02141	617-876-4210	\N	857-998-2056	617-661-9749	www.helpfamilies.org	public_program_email@testing123.net	Residential Services for youth involved with DCF or DYS	This residential program serves youth who are involved with DCF, ages 16-22.  We have two pre-independent living programs that are staffed on a 24 hour basis, one in Malden that is strictly a female house and one in Dorchester that is strictly for males.  These programs focus on stabalizing youth by providing them a case manager who will help them with employment, education and any other general life skills.  At the independent living level, youth are placed in scattered apartment sites in the greater Boston area.  Youth are given a case manager that they meet with once a week.  Youth are placed with a room mate; these apartments are not staffed.	Youth must be between the ages of 16-22 and be in the custody of DCF or DYS.	Yes	Based on a contract set up with DCF/DYS	open year round	Programs are open 24 hours.	English!DELIM!Spanish	Yes	Referrals are made through DCF/DYS social worker/case managers/family networks members.  When interviewing a referral packet should accompany the youth.		Yes	We provide a monthly combo pass to our youth in all programs.	No	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	\N	Yes	Sent out on a quarterly basis.	No	\N	Bernadette	Peeples	617-876-4210	617-661-9749	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
509680	103619	2009-03-20	\N	EOC	Educational Opportunity Center	"T" Building	Corner of Market and Broad Streets	Lynn	MA	02420	781-477-2114	\N	\N	781-477-2112	www.lynneoc.org	public_program_email@testing123.net	Educational Access Program	Participants receive one-on-one assistance with admission, financial aid and transfer procedures, academic and career advising, and general information about the educational process. Assistance with Financial Aid and College Admissions Applications provided.	Target population are individuals with low-income and whose parent's did not graduate from a 4-year college.  Must be a U.S. Citizen or Permanent Resident.  We serve ages 17 - 80+. This includes high school seniors, drop outs, career changers, displaced workers, displaced homemakers, and non-traditional students.	Free	\N	Open year round.  Closed Fridays in July. Closed last week of December.	Monday - Friday 8 - 4 (closed 12-12:30)	English!DELIM!Spanish	Yes	This will be explained when individual makes an appointment.  It will include income information, alien card (if non-citizen) and social security #		1) On Market Street, 2) in the MBTA Parking Garage (fee $4), 3) At North Shore Community College parking lot.	We are located at the MBTA Train and bus station.  See web site for directions.	Yes	No	\N	No	\N	Peak times are during March, April and May, August, and September.  During these times, appointments given may be for 2 weeks later.	EOC @ NSCC	PO Box 3340 - One Ferncroft Road	Danvers	MA	01923	No	\N	Yes	Books on Scholarships, Financial and Careers	Yes	Use of Classroom space in the MBTA Building for EOC meetings with other agencies.	No	\N	No	\N	Patricia	Wolfe	781-477-2115	781-477-2112	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Ipswich, MA!DELIM!North Reading, MA!DELIM!Andover, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Rowley, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Lynnfield, MA!DELIM!Salisbury, MA!DELIM!Magnolia, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Dracut, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Merrimac, MA!DELIM!Essex County, MA!DELIM!Methuen, MA!DELIM!Georgetown, MA!DELIM!West Newbury, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Newbury, MA!DELIM!Hamilton, MA!DELIM!Newburyport, MA!DELIM!Haverhill, MA	\N	We provide referrals to other EOC's located throughout Massachusetts	\N	No	0	\N
509691	103630	2009-03-20	\N	Educational Talent Search	Lynn ETS	"T' Building	Corner of Market and Broad Streets	Lynn	MA	01905	781-477-2114	\N	\N	781-477-2112	\N	public_program_email@testing123.net	Higher Educational Access for Lynn Youth	Promote successful completion of high school (or GED for drop outs) and access to higher education.  Advisors provide one-on-one services at the 3 main Lynn Public High Schools and College Awareness Activities to Lynn Middle Schools.  Services include educational, admissions, and financial aid advising as well as assistance with Financial Aid and College Admissions Applications.  Other services include Visits to Colleges, SAT Prep Classes, Math Tutoring, College Success Seminar (free 3 credit College Class), and Financial Aid Information Sessions for Parents.	Lynn Youth Ages 11 to 21, grade 6-12 or recent high school drop out.\nMust be U.S. Citizen or Permanent Resident	Free	\N	Advisors are in the high schools throughout the school year.  Summer services available at the main office in the "T" Building	Monday-Friday 8-4\nClosed State and Federal Holidays\r\nClosed Fridays in July\r\nClosed last week in December	English!DELIM!Khmer	Yes	Family Income information, alien card(if not a U.S. Citizen, Social Security Number.		\N	\N	Yes	No	\N	No	\N	\N	ETS @ North Shore Community College	PO Box 3340 - One Ferncroft Road	Danvers	MA	01923	Yes	Looking for Tutors - English, History & Science	Yes	Books on Scholarships, Careers, Colleges, Financial Aid	Yes	Use of Classroom Space in the MBTA Building	Yes	Twice a year.  Information on upcoming program events, feature local agency, college and financial aid information.	No	\N	Patricia	Wolfe	781-477-2115	781-477-2112	chester@testing123.net	Yes	TOWN	Lynn, MA	\N	\N	\N	No	0	\N
509724	103586	2009-04-01	\N	COMPASS Youth Program of Action Inc.	COMPASS	180 Main Street	\N	Gloucester	MA	01930	978-282-1000	\N	\N	\N	www.actioninc.org	public_program_email@testing123.net	GED/tutoring for at-risk students	Alternative Day School for High School Students.  GED preparation for area youth ages 16-21.  Night High School Dipolma Program for Gloucester High School Students at Senior standing who are not currently enrolled in school.	16-21 year olds facing social or ecomonic barriers.	Free	\N	Open year round.  Day School follows the academic school calendar.	Day Program 8:30-2:00\nEvening Programs 3-6\r\nSummer GED program mornings 9-12	English	Yes	\N		City meter parking is available.	none	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	tutors needed to help students prepare for GED and individual classes	No	\N	Not Applicable	\N	No	\N	No	\N	Maggie	Meffen	978-282-1000 x103	978-283-0528	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	\N	please call (978)281-9682 for more information.	\N	No	0	\N
509735	103586	2009-04-03	\N	HomeCare	\N	180 Main Street	\N	Gloucester	MA	01930	978-283-2700	\N	\N	\N	www.actioninc.org	public_program_email@testing123.net	HomeCare services for the elderly and disabled.	Action HomeCare provides Home Health Aide, Personal Care, light housekeeping, shopping, laundry and respite services.	Servicing all people in need of home care services.	Yes	We contract for Personal Care, Home Health Aide and Homemaking through Area Agencies on Aging, Mass Rehabilititation, Mass Commission for the Blind; and provide private pay billing.	Office hours are Monday - Friday, 9-5.  Emergency on-call on weekends and holidays.	Monday Friday 9-5.	English!DELIM!Portuguese!DELIM!Spanish	No	\N		on street metered parking	accessable by public transportation.	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	GerryAnne	Brown	978-283-2700	978-283-0528	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Essex County, MA!DELIM!Gloucester, MA	\N	\N	\N	No	0	\N
509757	103641	2009-04-14	\N	English as a Second Language	ESL	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	ESL classes to assist those wanting to learn English	ESL (English as a Second Language) Program began in 2006 with one class. In 2007, the program expanded and offers classes two evenings and two mornings a week.\nThis program offers basic and advanced classes, class size is limited to 15 students in each class. Students are placed into either Level I or Level II depending on the their language capabilities. Students receive instruction in American English listening, speaking, reading, and writing skills.	Priority is given to Beverly & Manchester residents. All students must take a placement test prior to starting classes.	Yes	Course registration and attendance is free. There is a $25 fee for supplies. Scholarships are available upon request.	open year round	Tuesday and Thursday 10:00am-11:00am\nMonday and Wednesday 6:00pm-8:00pm 	English	Yes	To register and take the test, please be able to provide contact information.		Parking is available in the parking lot to the left of the building, and on the street.	\N	No	No	\N	Yes	If the classes are full your contact information will be taken down and put into our wait list. When the next class session begins you will be given priority.	\N	\N	\N	\N	\N	\N	Yes	There are opportunities to volunteer during the class time to help teach the students	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Andrea	Jones	978-927-1561	\N	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
509768	103641	2009-04-27	\N	General Educational Development	GED	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Test preparation for the GED test	The General Educational Development (GED) program at Beverly Bootstraps Community Services provides high school level instruction in language arts, reading, social studies, science, and mathematics	Priority is given to Beverly and Manchester Residents	Free	\N	Open year round	Tuesday and Thursday evenings 6:00pm to 9:00pm	English	Not Applicable	\N		There is parking available in the parking lot to the left of the building and on the street.	\N	No	Yes	There are two babysitting staff who are able to watch your children while you are in class	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	There are opportunities for tutors.	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Andrea	Jones	978-927-1561	\N	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
509801	103641	2009-04-14	\N	Cybercafe	Cybercafe	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	After school homework club for middle school kids	The Cyber Caf is an after-school computer and homework assistance program available to middle-school youth. Participants typically do not have access to computers at home and are living in poverty.\nThe Cyber Caf has 8 computers and is held in The Franklin room at 371 Cabot Street, Beverly. Adjacent rooms on the second floor of the building are designated as quiet space for kids who are being tutored or working on homework. A part-time Caf Manager supervises the Cyber Caf. Tutors consist of Staff, Volunteers, and Interns.\r\nThe goals of the Cyber Caf is to give middle school students a safe place to gain access to computers, complete homework with assistance of tutors, build leadership skills through mentoring younger youth, college searches, and enrichment activities. 	The youth must reside in Beverly and attend middle school.	Free	\N	Open during the school year	Every day after school from 3:30pm to 5:30pm	English	No	\N		There is parking available in the parking lot to the left of the building and on the street.	\N	No	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	There are volunteer opportunities to tutor the children.	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Mac	Carr	978-927-1561	\N	chester@testing123.net	No	TOWN	Beverly, MA	\N	\N	\N	No	0	\N
509812	103641	2009-04-14	\N	Homework Club	Homework Club	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	After school homework club for elementary school children	The Homework Club offers tutoring to elementary-aged children. This after-school program provides mentoring and educational support to children at risk for academic failure living in the communities we serve. Students can be referred to the program by their parents or by the school they attend.	Must reside in Beverly and attend Elementary School	Free	\N	Open during the school year	Monday through Thursday from 2:30pm-4:00pm	English	Yes	Register by submitting a permission slip signed by the participants parent/guardian. This permission slip would allow Beverly Bootstraps Community Outreach Program Staff to speak with the Beverly Public Schools and allow participants to attend planned Field Trips.		\N	\N	No	Yes	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	There are opportunities to tutor the children who attend homework club	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Andrea	Jones	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA	\N	\N	\N	No	0	\N
509834	103674	2009-04-17	\N	One Family Scholars	\N	28 Emerson Avenue	\N	Gloucester	MA	01930	978-283-1972	\N	\N	978-283-3206	www.onefamilyinc.org	public_program_email@testing123.net	Education & Leadership as a pathway out of poverty	One Family Scholars provides essential tools  higher education, funding, leadership training and mentoring  to income-limited women, single heads of households, so they can obtain careers that pay a living wage and build a new path to economic independence for themselves and their children. The support offered by the program is comprehensive and the expectations of the program are high. OFS offers leadership, career, skills development, flexible funding to assist with tuition, books, childcare, etc., and a Site Coordinator to provide coaching relevant to the program. Applicants must, for example, be income eligible, complete 12-18 credits annually, maintain a GPA of at least 3.0, maintain contact/be responsive to their Site Coordinators, and attend required leadership development retreats, seminars, and activities.	Female, single head of household, dependents under the age of 18, income at or below 200% of the poverty level, 3.0 or above GPA, Massachusetts resident. Criteria is subject to change, please check our website for the most current information.	Free	This is an application process for a scholarship award, there are no fees charged.	Applications for a one-year award are available the first of each year.	Monday through Friday 10 a.m. to 6 p.m.	English	Not Applicable	\N		\N	\N	Yes	Not Applicable	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer opportunities are coordinated through Wellspring House: 978-281-3558.	Not Applicable	\N	Not Applicable	\N	Yes	Wellspring House, Inc./seasonal\nOne Family, Inc./seasonal\r\nBoth are available online.	No	\N	Gail	Mountain	978-283-1972	978-283-3206	chester@testing123.net	Yes	TOWN	Amesbury, MA!DELIM!Ipswich, MA!DELIM!Andover, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Rowley, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Magnolia, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Georgetown, MA!DELIM!Middleton, MA!DELIM!West Newbury, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Newbury, MA!DELIM!Hamilton, MA!DELIM!Newburyport, MA!DELIM!North Andover, MA	\N	Currently there are 5 One Family Sites in Massachusetts, other geographical area candidates should check our website for locations near them.	\N	Not Applicable	0	\N
509790	103641	2009-04-21	\N	Financial Literacy	\N	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Basic classes about budgeting, debt and other topics	In these difficult economic times we understand the importance of understanding money management, finding ways to decrease expenses, finding ways to increase income and building a savings plan is crucial for financial survival.\nThese classes will help the student make a budget, manage expenses, and learn valuable financial knowledge to help them better manage their finances. 	Must be a resident of Beverly, MA or the surrounding area	Free	\N	Open year round	The class schedule changes every month, please check our website for the latest schedule, or contact Greg Ezell at gezell@beverlybootstraps.org	English	Yes	\N		There is parking available in the parking lot to the left of the building and on the street.	\N	No	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Greg	Ezell	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Peabody, MA!DELIM!Beverly, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Gloucester, MA	\N	\N	\N	No	0	\N
509779	103641	2009-04-21	\N	Linking Information For Employment Skills	LIFE Skills	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Learn basic job search and computer skills	Beverly Bootstraps Community Services (BBCS) and Salem CyberSpace are teaming up to deliver an innovative program to provide low-income Beverly area residents with life skills to attain self-sufficiency and sustainability.\nTopics to be covered: Computer Fundamentals, Email, MS Word, MS Excel, Career Planning and Interviewing, and Financial Literacy. 	Must be a resident of Beverly, MA	Yes	A deposit of $10 is required at the start of the class, and will be returned upon successful completion of the course.	Runs on a six-week basis	Meets Tuesdays, Wednesdays, and Thursdays from 9:00am to 12:00pm	English	No	\N		Parking is available in the parking lot to the left of the building and on the street	\N	No	No	\N	Yes	When the class is full your information will be put on the wait list and you will be given first priority when the next session begins.	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Greg	Ezell	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA	\N	\N	\N	No	0	\N
509900	103641	2009-04-21	\N	Back To School Bonanza	Back to school	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Back to school preparation	The Back-to-School Program has been supplying back-packs filled with school supplies to children living in low-income for the past 9 years. In 2008, more than 425 children came to to Bootstraps for school supplies worth more than $38,000 of donated new backpacks filled with school supplies.\nWe collect the following items in May, June & July to distribute in late August.\r\n* Backpacks Folders Pack of Erasers\r\n    * Pencil Cases Packs of Pens Crayons\r\n    * Pencil Sharpeners Rulers Safety Scissors\r\n    * Packs of Pencils Binders Loose-leaf Paper\r\n    * Spiral Notebooks Colored Pencils Highlighters\r\n    * Assignment Pads Composition Books Protractors\r\n    * 3 x 5 Index Cards\r\n	Must be a resident of Beverly or Manchester	Free	\N	Open in late August, registration begins in July	By appointment only	English	Yes	Proof of income\nproof of residency\r\nIdentification for all adults and children in the family		There is parking available in the parking lot to the left of the building and on the street.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	There are opportunities to volunteer to put backpacks together and to distribute them to families	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Julie	Murphy	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA!DELIM!Manchester-by-the-Sea, MA	\N	\N	\N	No	0	\N
509889	103641	2009-04-21	\N	Adopt-a-Family	\N	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Holiday support for families with children	The Adopt-A-Family Program offers the ability for low-income families to get gifts for their children during the holidays. Matched with generous donors, over 170 families were served last year, totaling over 325 children. This program is often the first volunteer opportunity for many of our volunteers.	Must be a resident of Beverly, or Manchester	Free	\N	During the holidays. Please check the calendar or call outreach staff to find out when to register.	Please contact Outreach Staff	English	Yes	Proof of Income\nProof of Residency \r\nIdentification for the adults and children (Social Security number, Birth Certificates, or MassHealth Card) 		There is parking in the parking lot to the left of the building, and on the street.	\N	Yes	No	\N	No	First come first served	\N	\N	\N	\N	\N	\N	Yes	Please contact the volunteer coordinator for more information. Opportunities include putting together gift bags and distributing them to families	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Julie	Murphy	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA!DELIM!Manchester-by-the-Sea, MA	\N	\N	\N	No	0	\N
509911	103641	2009-04-21	\N	Beverly Bootstraps Community Gardens	Community Garden	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Community Garden for Beverly Residents	The Community Gardens, located on Cole Street, began in 2001 and continues to provide a place of peace and enjoyment for gardeners, the opportunity for Beverly citizens to grow their own food, a beautiful area of open space, and food for hungry families.\nOver the last few years the gardens have flourished.  Thousands of pounds of produce have gone to feed both the families of the gardeners as well as the needy families who come to our food pantry.  Many resources, including funds, donated supplies, and vast amounts of volunteer hours have been invested in the beautification of the Community Gardens. \r\n\r\nThe garden is made up of 16 beds and 20 plots. The average plot size is approximately 17 x 25.  	Must be a resident of Beverly or Manchester	Yes	\N	The Community Garden is in operation from mid April through the summer	The garden is available to the gardeners from dawn to dusk.	English	No	\N		\N	\N	No	No	\N	Yes	There are a limited number of plots available. Once they are full you will be put on a wait list for the next summer, or in case someone drops out of the program.	\N	\N	\N	\N	\N	\N	Yes	All the gardeners at the community garden are there on a volunteer basis.	No	\N	No	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Julie	Murphy	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA!DELIM!Manchester-by-the-Sea, MA	\N	\N	\N	No	0	\N
509823	103641	2009-04-21	\N	Serve Food Co-op	Serve	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	allows members to buy groceries in bulk for reduced prices	Part of a New England-wide empowerment initiative, our Serve food co-op allows individuals and families to double their food dollars through performance of community service.\nServe New England is a program designed to promote volunteer service by offering deep discounts on the highest quality grocery, meat, and vegetable items to community-minded individuals willing to volunteer 2 hours of their time each month to a worthy cause.  Serve New England have over 350 chapter sites located throughout the six New England states. \r\nSave up to 40% or more on high-quality, fresh foods by buying your meats, fruits, veggies and other staple items through the Serve New England Food Program.	The participant must be able to pick up the food once a month at Beverly Bootstraps	Yes	How Serve's Food Cooperative Works:\n* A participant registers at a local Serve Chapter.  Beverly Bootstraps is the local Serve Chapter.  Please come in to register any time that we are open for business.\r\n* Serve New England purchases food directly from manufacturers, wholesalers, and New England farms through a unique bulk buying system for all those who register.\r\n* Participants complete 2 hours of community service for each food package purchased.\r\n* Serve New England coordinates the distribution of food to the Chapters, such as Beverly Bootstraps, who in turn distribute packages locally.\r\n* Participants return to Beverly Bootstraps to pick up their order and begin the cycle of service to their community and savings for their families.\r\n2 hours of volunteering\r\n    + $25\r\n    = $62.50 worth of food!\r\n	Open year round	Contact Tom Gifford via email at giffordtj@hotmail.com	English	Yes	\N		There is parking available in a parking lot to the left of the building and on the street.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Tom	Gifford	978-927-1561	\N	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509933	103696	2009-04-22	\N	Counseling and Advocacy	\N	27 Congress Street	140 Union Street, Lynn; 5 Pleasant Street, Gloucester	Salem	MA	01970	978-744-8552	800-547-1649	800-547-1649	978-745-6886	www.hawcdv.org	public_program_email@testing123.net	24 hour confidential hotline and short term counseling	Information, support and referral by phone\nShort term individual counseling in English, Spanish and Portuguese\r\nSupport groups in Salem, Lynn and Gloucester\r\n--Lesbian, bisexual and transgender group\r\n--Spanish speaking support group\r\n--Survivors of domestic abuse struggling with substance abuse\r\n--Probate court support group	Free to all victims of domestic abuse	Free	\N	Open year round	Monday--Friday, 9 am to 5 pm. Some after hours counseling by appointment and evening support groups	English!DELIM!Portuguese!DELIM!Spanish	Yes	\N		Available	\N	Yes	Yes	On site childcare available with advance request	No	\N	\N	\N	\N	\N	\N	\N	Yes	Hotline volunteers are trained in spring and fall; must attend 40 hour training and be observed on hotline	No	\N	Yes	Salem, Lynn, Gloucester, Beverly and Ipswich office locations	Yes	Twice a year	No	For HAWC clients only	Candace	Waldron	978-744-2299 ext 12	978-745-6886	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Rowley, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Magnolia, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Georgetown, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Phone support provided to all callers regardless of area	\N	No	0	\N
510010	103641	2009-04-22	\N	Youth Literacy Program	\N	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Youth Literacy Program	The Youth Literacy Program offers books to school-age children to encourage children to keep reading through the summer. This program promotes the importance of reading and assists low-income youth to build their own home library. The child keeps book he/she has chosen. In 2008, 135 children participated in the program. Books were distributed through the Bootstraps 'bookmobile' and in the library located at our office. More than 650 free books were distributed.	Must be in either elementary or middle school.	Free	\N	Operates during the summer	Books are delivered once a week to area parks along with park lunches.	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers are needed to sort through donated books and distribute them to the children	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Andrea	Jones	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA!DELIM!Manchester-by-the-Sea, MA	\N	\N	\N	No	0	\N
509746	103641	2009-04-22	\N	The Tom Gifford Food Pantry	Food Pantry	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Food Pantry open to all Beverly and Manchester residents	The Tom Gifford Food Pantry is open every day to Beverly and Manchester residents. Each client receives one bag of staples, including milk, bread, meat, pasta, rice or Ramen, cereal, fruit and vegetables, beans, peanut butter, and pasta sauce	Clients must bring in a piece of mail dated within the last 30 days with the client's address on it as proof of residence.\nClients are eligible to use the pantry once every two weeks.\r\n	Free	The food pantry is free to residents of Beverly and Manchester.	Open year round	Every day from 11:00am-12:30pm\nTuesday and Wednesday evenings 5:00pm-6:30pm	English	No	When registering please bring proof of address.		There is parking available in the parking lot next to the building and on the street.	We do not provide transportation however some clients may be eligible to sign up for our weekend food delivery program. Please contact Kathy Kirlis at kkirlis@beverlybootstraps.org or by phone at 978.927.1561 for more information	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers can help assemble bags, conduct intakes, as well as conduct a food drive.\nPlease contact Tom Gifford at tgifford@beverlybootstraps.org for more volunteer information/	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	Yes	\N	No	\N	Kathy	Kirlis	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA!DELIM!Manchester-by-the-Sea, MA	\N	\N	\N	No	0	\N
510032	103641	2009-04-22	\N	Housing Assistance	\N	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Housing assistance for Beverly and Manchester residents	There are three main ways that Beverly Bootstraps is able to assist with housing.\n1)The Clear Point Horizons Transitional Housing\r\nIn response to a dire need for transitional housing, Beverly Bootstraps Community Services purchased a condominium with assistance from Clear Point Consult, a women owned business, located in Manchester Massachusetts. Clear Point Horizons is designed to provide a stable living environment to a Beverly family who is homeless or at risk of being homeless. This program provides support services to promote self-sufficiency. To date, we have helped 11 families.\r\n\r\n2)The Manchester Fund to Prevent Homelessness:\r\nThe Fund to Prevent Homelessness (http://www.ftph.org/) has been helping families since 1989. This organization is a tax-exempt non-profit charity focused on providing confidential assistance to working families who are facing foreclosure or eviction due to unusual circumstances that have depleted their financial resources. The fund supports families that do not qualify for other funding and would otherwise fall through the cracks.\r\n\r\nOutreach staff maintains close relationships with local housing authorities; through this relationship we are able to assist and advocate for client who are applying for Federal or State housing.	Eligibility for Clear Point Horizons Program:\nIn order to qualify for housing at Clear Point Horizons the following qualifications must be met:\r\n1. Must be a family; defined as a minimum of one adult with legal custody and possession of a minimum of one biological or adopted child.\r\n   2. All members of the family must be Beverly residents.\r\n   3. The family must be homeless or in immanent risk of being homeless.\r\n   4. Fall within or below the very low Federal income guidelines.\r\n   5. Provide written documentation reflecting income resources.\r\n   6. Qualify as potential volunteer.\r\n   7. Show potential for eventual self sufficiency coupled with a willingness to abide by the program agreement.\r\n   8. Have a positive attitude toward the program agreement.\r\n   9. Provide all required paperwork needed to complete the application process in timely fashion.\r\n  10. Willingness to participate in weekly case management meetings. \r\n\r\nEligibility for Manchester Fund to Prevent Homelessness:\r\nMeeting with an Outreach Coordinator to confirm other eligibility requirements and provide financial consulting.\r\n\r\n         1. Must be a family, as defined as a minimum of one child living with an adult.\r\n         2. Working and able to meet future payments once overdue rent or mortgage is paid.\r\n         3. Facing eviction or foreclosure.\r\n         4. Not qualified for welfare assistance.\r\n	Free	\N	open year round	by appointment only	English	Yes	Please bring proof of income,\nresidency, \r\nidentification, \r\nand any other relevant documents		\N	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	No	\N	No	\N	Greg	Ezell	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Rockport, MA!DELIM!Beverly, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA	\N	Communities Served  Beverly, Essex, Gloucester, Manchester-by-the-Sea, Rockport and Wenham.	\N	No	0	\N
510043	103641	2009-04-22	\N	Beverly Bootstraps Thrift Shop	Thrift Store	198 Rantoul St.	\N	Beverly	MA	01915	978-921-4710	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Thrifts store for clients and the public	Every donation made to the Beverly Bootstraps Thrift Shop works to further the mission of Beverly Bootstraps Community Services. The proceeds from sales provide revenue to support Beverly Bootstraps programming.\nOn a case-by-case basis through Beverly Bootstraps Community Outreach Programs, our outreach workers can also supply free vouchers for clothing, furniture and other items to victims of catastrophes. 	Open to the Public\nwe offer a discount of 50% on items priced over $2.00 to any customer who carries a Mass Health Card, a Neighborhood Health Network card, a food stamp card, or a Medicaid Card.	Free	\N	open year round	Monday-Friday: 9:30am to 5:00pm\nSaturday:  9:30am to 1:30pm	English	No	\N		There is limited parking in the parking lot in front of the Shop, and parking is available on the street	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	There are several opportunities available	Not Applicable	\N	No	\N	Yes	\N	No	\N	Thrift Store	Manager	978-921-4710	\N	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
509955	103641	2009-04-23	\N	Advocay and Referral	Outreach	371 Cabot Street	\N	Beverly	MA	01915	978-927-1561	\N	\N	978-927-1553	www.beverlybootstraps.org	public_program_email@testing123.net	Advocacy and Referral	*   Assistance applying for DTA and Food Stamp benefits online.\n* Assistance accessing local programs such as fuel assistance. \r\n* Help with an eviction, a notice to quit or court summons\r\n* Rent or Mortgage arrearages\r\n* Assistance completing public housing applications\r\n* Financial Literacy Classes \r\n    * Individual budget review\r\n\r\nOutreach staff maintains close relationships with local housing authorities; through this relationship we are able to assist and advocate for client who are applying for Federal or State housing.  \r\n\r\nIn cases of emergency situations Outreach staff are available to assist clients with furniture donations through one of the following venues:\r\n\r\n    * Bootstraps Thrift Shop\r\n    * Donations made to Bootstraps\r\n    * Furniture Bank\r\n    * Mission Of Deeds\r\n	Priority is given to residents of Beverly and Manchester	Free	\N	Open Year Round	The Bootstraps Outreach Staff is available Monday through Friday from 8:30 am to 4:30 pm	English!DELIM!Spanish	Yes	Please bring identification, proof of residency, and any relevant documentation to your Outreach appointment		There is parking available in the parking lot to the left of the building and on Cabot St.	\N	Yes	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	Yes	We have multiple rooms available for meetings, we can accommodate up to 30 people in one room. Please call for availability.	Yes	Every season, Bootstraps releases a client Newsletter updating the clients on our new and continuing programs	No	\N	Greg Ezell	Julie Murphy	978-927-1561	\N	chester@testing123.net	Yes	TOWN	Beverly, MA!DELIM!Manchester-by-the-Sea, MA	\N	\N	During certain hours or by appointment, we can arrange to have a Spanish Language interpreter on site.	No	0	\N
510054	103707	2009-04-27	\N	Case Management	\N	67 Middle Street	\N	Gloucester	MA	01930	978-283-0101	888-283-0101	\N	978-283-0101	www.healthproject.org	public_program_email@testing123.net	Case Management	Clients work with an individual case manager to resolve issues around health, housing, legal, and other needs. Case managers ensure that clients have access to all entitlement programs, health insurance, etc. Case managers also make supported referrals and accompany clients to medical appointments if necessary, housing appeals, etc.	Clients must have verification of HIV/AIDS or Hepatitis C. The case manager can help the individual obtain verification from an appropriate medical office or complete testing for HIV or HCV>	Free	\N	Open year round.	Monday through Friday, 9:00 a.m. to 5:00 p.m.	English!DELIM!Portuguese	No	No requirements for first visit.		\N	The Health Project is located on the CATA bus line, and five minutes from the T station.	Yes	Not Applicable	Clients are welcome to bring children with them.	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	We always welcome help with the front desk, congregate meals, and general office work.	Yes	Some materials are available on HIV/AIDS and Hepatitis C, brochures from other agencies, and reprints of basic health information. The Health Project can obtain packets of information tailored to specific concerns from the resource library at AIDS Action Committee, Boston.	Yes	The dining room holds about 30 people.	Yes	The Health Project sends a monthly newsletter to clients and a quarterly newletter to the general public.	No	\N	Susan	Oleksiw	978-283-0101	978-283-3311	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	Not Applicable	0	\N
510065	103718	2009-04-27	\N	The Fund to Prevent Homelessness	The Fund	PO box 17	\N	Manchester	MA	01944	978-927-1561	\N	\N	\N	www.FTPH.org	public_program_email@testing123.net	rent and mortgage assistance for working families	The Fund gives one time assistance in the form of rent or mortgage payment to working families who have had a one time event which has placed them at risk of losing their home.  The family must be sustainable going forward for us to catch them up on over due payments.  Intake in done through Beverly Bootstraps.	There must be one child in the home and one working adult.  the family must not qualify for welfare assistnce.	Free	\N	open year round	the hours of Beverly Bootstraps, M-F, 8:30am - 4pm	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Yes	For intake, Beverly Bootstraps offices.  For Board meetings, members homes.	No	\N	Yes	www.ftph.org	Laurel	Deery	978-526-4852	\N	chester@testing123.net	No	TOWN	Rockport, MA!DELIM!Beverly, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA	\N	and the town of Essex MA	\N	No	0	\N
510087	103729	2009-04-29	\N	Nutrition Program	Meals on Wheels	5 Blackburn Center	\N	Gloucester	MA	01930	978-281-1750	866-927-1050	978-281-3093	978-218-1753	www.seniorcareinc.org	public_program_email@testing123.net	Nutrition Program	Nutrition program provides home-delivered meals, senior luncheon meals, nutrition consultations, emergency meals, monthly grocery distribution and more.	Elders aged 60 and over.  No income eligibility required.	Free	Donations are requested.	open year round	Monday - Friday, 9:00am to 5:00pm	English	Yes	Meals on wheels elders will be notified prior to initial visit by care manager.		yes	Some elder dining sites have transportation to the meals ask when reserving a meal.	Yes	Not Applicable	\N	No	\N	Senior meals need phone call to reserve a meal and Meals on Wheels intake by the phone followed by in-home assessment.	\N	\N	\N	\N	\N	Yes	Meals on Wheels drivers needed in 9 communities. Senior dining assistants welcomed at meal sites.	No	\N	Yes	Handicapped accessible, first floor, parking, seats 60, wireless connection.	No	\N	No	\N	Linnea	Hagberg	978-281-1750	978-281-1753	chester@testing123.net	No	TOWN	Rockport, MA!DELIM!Beverly, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	978-282-1836	\N	interpretation services available	No	0	\N
510098	103740	2009-04-29	\N	Salem YMCA	YMCA of the North Shore	One Sewall Street	\N	Salem	MA	01970	978-740-9622	\N	\N	978-740-9168	www.northshoreymca.org	public_program_email@testing123.net	Membership , Programs, Child Care and Camp	Originally designed as affordable housing, the Salem YMCA was first founded in 1858 making Salem the oldest YMCA on the North Shore. For over 150 years the Salem YMCA has been ever changing, ensuring that as needs change the Y continues to meet its mission in providing programs and services that make an impact.\nOur programs for children don't stop there.  In fact, we offer a wide variety of programming options for children. These options range from fun and instructional swim lessons to winter-long youth basketball leagues to an art club for youth and teens. At the Salem YMCA we continuously strive to provide new and innovative programs at every turn. \r\nAnd we have plenty of programs and amenities to keep our 3,500 adult members happy and involved at the Salem YMCA. Our aerobics studio, locker rooms and pool guarantee a pleasurable experience for every visit. With two pools we have enough water to meet the needs of all ages and interests from swim lessons to lap swimming to water aerobics to our swim team. \r\n\r\nNo matter what your interest or ability we have something exciting in store for you at the Salem YMCA!	Open to all	Yes	Sliding Scale available for those who qualify.	Year round	Monday through Friday 5:00am-10:00pm\nSaturday 6:00am-7:00pm\r\nSunday 7:00am-6:00pm	English!DELIM!Spanish	No	\N		Metered	Train and bus nearby	Yes	Yes	Babysitting available, please see website for schedule	No	\N	\N	One Sewall Street	\N	Salem	MA	01970	Yes	Refer to our website at www.northshoreymca.org	Yes	Developmental Assets	Yes	Rooms available for fee.	No	\N	Limited	\N	Linda	Berger	978-744-0351	978-740-9168	chester@testing123.net	Yes	TOWN	Salem, MA	\N	North Shore	\N	Yes	0	\N
510120	103751	2009-04-29	\N	Gloucester Family Health Center	\N	302 Washington St.	\N	Gloucester	MA	01930	978-282-8899	\N	\N	978-282-5599	www.nschi.org	public_program_email@testing123.net	Medical, dental and behavioral health care	The Gloucester Family Health Center provides care for your family at all stages of life.  As a community health center, we offer translations services, financial counseling and health education.	No one will be denied services based upon the ability to pay.	Yes	We accept most private insurances, and medicaid, medicare and the health safety net.	Open year round.	Monday, Wednesday, Thursday and Friday 8:30am to 5:00 pm.  Tuesday evenings until 7pm.	English!DELIM!Portuguese!DELIM!Spanish	Yes	Proof of insurance		On site parking.	Accessible by public transportation.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Lisa	Schott	978-282-8899 ext. 214	978-282-5599	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Rowley, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Georgetown, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	\N	\N	Italian	No	0	\N
510131	103751	2009-04-29	\N	Peabody Family Health Center	\N	89 Foster St.	\N	Peabody	MA	01960	978-744-8388	\N	\N	\N	www.nschi.org	public_program_email@testing123.net	Health care	The Peabody Family Health Center provides medical care for you and your family at all stages of life.  As a community health center, we offer translation services, financial counseling, and health education.	No one will be denied services based upon the ability to pay.	Yes	We accept most major private insurance plans as well as mass health, medicare, and the health safety net.	Open year round.	Tuesday through Friday 8:30 to 5:00 pm. Monday evenings until 7:00pm.	English!DELIM!Portuguese!DELIM!Spanish	Yes	Insurance information		On-site parking available.	Accessible by public transporation.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Lisa	Schott	978-282-8899 ext. 214	978-282-5599	chester@testing123.net	Yes	COUNTY	Essex County	978-532-4914	\N	\N	No	0	\N
510142	103762	2009-04-29	\N	Public housing and rental assistance programs	\N	259 Washington Street	\N	Gloucester	MA	01930	978-281-4770	\N	978-281-4770	978-281-6945	www.ghama.com	public_program_email@testing123.net	Subsidized housing	Rent is based on household income.	At or below 80% Area Median Income	Free	\N	Open year round	Monday - Thursday, 8:00 a.m. - 4:15 p.m.\nFriday, 8:00 a.m. - 12:00 p.m.	English	No	\N		On site	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Conference Room for about 20 people	No	\N	No	\N	Maryanne	Koeller	978-281-4770	978-281-6945	chester@testing123.net	No	EVERYWHERE		\N	Public housing is in Gloucester. Mobile rental assistance vouchers are portable to other communities. project-based rental vouchers are in buildings in Gloucester	\N	Yes	0	\N
509713	103652	2009-04-30	\N	Young Parent Program	YPP	10 Federal Street	\N	Salem	MA	01970	978-744-2417	\N	\N	978-745-5757	www.tra-inc.org	public_program_email@testing123.net	GED prep for pregnant and parenting teens	GED program for pregnant and parenting teens who receive TAFDC from the Department of Transitional Assistance.  In addition to GED prep classes offered in parenting skills, life skills, vocational skills, financial literacy and job readiness training.  Placement assitance into college, job training or employment after completion of GED.	Preganant and parenting youth between the ages of 14 and 21 who are receiving TAFDC from the Department of Transitional Assistanc	Free	\N	Year round program	Monday - Friday 9:00 - 2:30	English!DELIM!Spanish	No	Referral from DTA caseworker		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Limited	Internet offered to all enrolled students	Ali	Egan	978-744-2417	978-745-5757	chester@testing123.net	Yes	COUNTY	Essex County	\N	\N	\N	No	0	\N
510186	103729	2009-05-06	\N	Money Management Program	\N	5 Blackburn Center	\N	Gloucester	MA	01930	978-281-1750	866-927-1050	978-281-3093	978-218-1753	www.seniorcareinc.org	public_program_email@testing123.net	Money Management helps elders manage bill with volunteers	Free to eligible elders, this program provides trained volunteers to help elders balance checkbooks, prepare checks for signature, sort and prioritize mail, fill out forms and paperwork, develop a budget, and negotiate with creditors, if necessary.	Age sixty plus, in SeniorCares 9 cities and towns, low-income.	Free	\N	open year around	9:00 am to 5:00 pm	English	Yes	\N		Available and handicapped parking designated.	NA	Yes	Not Applicable	\N	No	\N	Consumers are served in their homes and timing of visits may vary according to their needs.	\N	\N	\N	\N	\N	Yes	Money Management volunteers often have a background in financial issues.  Their valuable service helps elders to remain in their homes.	No	\N	Yes	Handicapped accessible, first floor, parking, seats 60, wireless connection.	No	\N	No	\N	Debby	Beveridge	978-281-1750	978-281-1753	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	978-282-1936	\N	\N	No	0	\N
509988	103520	2009-05-11	\N	Fuel Assistance	\N	98 Main St.	\N	Peabody	MA	01960	978-531-0767	\N	\N	978-531-1012	www.nscap.org	public_program_email@testing123.net	Provides heat assistance during the winter heating season	NSCAPs Fuel Assistance program helps qualified applicants pay for a portion of their home heating bill. The program runs from November 1 to April 30.\nHomeowners and renters, including households whose cost of heat is included in the rent, may apply. Eligibility is based on household size and the gross annual income of every household member, 18 years of age or older. Household income cannot exceed 60% of the state median income.\r\n	Recipients must be income-eligible.	Free	\N	Program is open year round, but applications are only taken from November through April.	Monday through Friday from 9:00 to 5:00	English!DELIM!Spanish	Yes	List of required documentation will be provided when appointment is made.		NSCAP has a parking lot in the rear of the building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Mary	Trelegan	978-531-0767 x133	978-531-1012	chester@testing123.net	Yes	TOWN	Peabody, MA!DELIM!Beverly, MA!DELIM!Salem, MA!DELIM!Danvers, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Middleton, MA!DELIM!West Newbury, MA	\N	\N	\N	No	0	\N
509515	103520	2009-05-11	\N	Adult Basic Education	\N	75 Central St.	\N	Peabody	MA	01960	978-532-8620 x204	\N	\N	978-532-8618	www.nscap.org	public_program_email@testing123.net	Adult Basic Education/English for Speakers of Other Language	The Adult Basic Education program offers instruction in English for Speakers of Other Languages (ESOL). There are three levels: Level 1 (Beginner), Level 2 (Intermediate), and Level 3 (Advanced). Students receive instruction in American English listening, speaking, reading, and writing skills. Local, regional, and national history and civics, as well as traditions, holidays, and customs, are an integral part of our program. In our classes, teachers and students also enjoy sharing their cultures and histories, taking field trips to a variety of locations, and inviting speakers from many community organizations.\nThe ABE Program also has a Workplace Education program has a 5 year grant from the Department of Education. NSCAP partners with Brooksby Village to provide ESOL and ABE classes to employees. Students also learn workplace skills relevant to their jobs.\r\n	Peabody, Beverly, Danvers, Salem area	Free	\N	Academic year Monday through Thursday 9:00 to 12:00	Monday through Friday 9:00 to 5:00	English	No	Call for information.		NSCAP has a parking lot in the rear of the building.	\N	Yes	No	\N	Yes	\N	\N	\N	\N	\N	\N	\N	Yes	Tutors are needed.	Not Applicable	\N	No	\N	No	\N	No	\N	Barbara	Reeder	978-532-8620 x204	978-532-8618	chester@testing123.net	Yes	TOWN	Peabody, MA!DELIM!Beverly, MA!DELIM!Salem, MA!DELIM!Danvers, MA	\N	\N	\N	No	0	\N
509460	103520	2009-05-11	\N	Advocacy	\N	98 Main St.	\N	Peabody	MA	01960	978-531-0767	\N	\N	978-531-1012	www.nscap.org	public_program_email@testing123.net	Advocacy	The Advocacy Program offers assistance with a wide range of issues to low-income individuals in their main service areas of Salem, Peabody, Danvers, and Beverly. The Advocates work one-on-one with individuals and/or families to resolve crises, make referrals, and help clients attain and/or maintain self-sufficiency. The Advocates assist with Immigration, Social Security Benefits, housing, public benefits, Unemployment benefits, MassHealth and Medicare, and credit and consumer issues. Advocacy also administers charitable funds, which with the Advocate's discretion may be used to resolve utility or rent crisis. Advocacy also administers the Representative Payee Program in conjunction with the Social Security Administration. This program assists individuals who are not capable of managing their own funds by maintaining a stable financial situation for them. However, it should be noted that at this time no new Representative Payee clients can be taken on due to limited staff.\nThe components staff is bi-lingual and bi-cultural in Spanish and Portuguese.\r\n	low-income residents from the Peabody, Beverly, Danvers and Salem area.	Free	\N	Open year round.	Monday through Friday 9:00 to 5:00	English!DELIM!Portuguese!DELIM!French!DELIM!Spanish	Yes	\N		NSCAP has a parking lot in the rear of the building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Deric	LePard	978-531-0767 x125	978-531-1012	chester@testing123.net	Yes	TOWN	Peabody, MA!DELIM!Beverly, MA!DELIM!Salem, MA!DELIM!Danvers, MA	\N	\N	\N	No	0	\N
509526	103520	2009-05-12	\N	Housing Pathways	\N	75 Central St.	\N	Peabody	MA	01960	978-531-5220	\N	\N	978-531-0757	www.nscap.org	public_program_email@testing123.net	Scattered site emergency shelter for families	Project Pathways provides homeless families with temporary shelter in apartments, which they occupy until they find permanent housing. NSCAP leases apartments in the Salem area, and pays the rent and utilities. Families pay for phone service. This program is designed for large families and those with, for example, health issues that preclude them from residing in congregate shelters\n.During a familys stay NSCAP provides: \r\n Food, clothing and (when moving out) furniture donation referrals;\r\n housing search assistance (including transportation) for families to find affordable, permanent housing;\r\n case management services to foster a healthy family life;\r\n\r\n referrals to other agencies for education and job training to help parents increase their incomes and move toward self-sufficiency;\r\n\r\n assistance with budgeting in order to save; and\r\n\r\n mediation, paralegal assistance, and referrals for one year to maintain permanent housing.\r\n\r\n\r\n	Referral from Dept. of Transistional Assistance	Free	\N	Monday through Friday 9:00 to 5:00	Monday through Friday 9:00 to 5:00	English!DELIM!Portuguese!DELIM!Spanish	Yes	\N		NSCAP has a parking lot at the rear of the building.	NSCAP's van provides transportation for guests to attend housing search meetings, meetings at housing authorities, and school enrollment for guests' children.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Garry	Reidester	978-531-5220 x318	978-531-0757	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
509482	103520	2009-05-12	\N	Project Pathways	\N	75 Central St.	\N	Peabody	MA	01960	978-532-8620	\N	\N	978-532-8618	www.nscap.org	public_program_email@testing123.net	Transition to College	Project Pathwyas is a program that will prepare North Shore residents, 18 years or older, to enter college courses and get good jobs in the health and human services field. The rigorous curriculum for Project Pathways will include classes in math, reading, writing, college success skills, computer instruction, health science and career awareness and exploration.  Staff of the project will assist students with developing the skills, supports and academic requirements to enter post secondary education.	Must have GED and be out of high school for at least 5 years.	Free	\N	Class schedule varies during evenings and Saturdays. Business hours are Monday through Friday 9:00 to 5:00	Monday through Friday 9:00 to 5:00.	English!DELIM!Spanish	Yes	List of required documentation to be provided when intake is scheduled.		NSCAP has a parking lot at the rear of the building.	\N	Yes	Yes	In-house child care may be provided while classes are in session.	No	\N	\N	\N	\N	\N	\N	\N	Yes	Tutors are needed.	No	\N	No	\N	No	\N	No	\N	Susan	Hershey	978-532-8620x110	978-532-8618	chester@testing123.net	Yes	COUNTY	Essex County	\N	\N	\N	No	0	\N
510175	103520	2009-05-12	\N	River House Emergency Shelter	\N	56 River Street	\N	Beverly	MA	01915	978-921-1304	\N	\N	978-921-1304	www.nscap.org	public_program_email@testing123.net	Homeless shelter/transitional housing for singles	River House, Inc. is a year-round emergency homeless shelter for single men located in Beverly, Massachusetts. The shelter can accommodate 34 men and the safe haven provides five single rooms for formely homeless men or women moving into more permanent housing.	Dry shelter for homeless individuals from the greater Beverly area.	Free	\N	Open all year.	Open from 7:00 PM to 7:00 AM.  Day services are available between 9:00 and 5:00 or by appointment.	English!DELIM!Spanish	No	\N		\N	\N	Yes	No	\N	Yes	Ocasionally we are unable to accommodate all who apply for shelter.	\N	\N	\N	\N	\N	\N	Yes	Volunteers are encouraged to provide community outreach, donation outreach, fellowship, meals, and computer tutoring.	Not Applicable	\N	No	\N	Yes	\N	No	\N	Kate	Benashski	978-921-1304	978-921-1304	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
510076	103729	2009-05-12	\N	Homecare Services	\N	5 Blackburn Center	\N	Gloucester	MA	01930	978-281-1750	866-927-1050	978-281-3093	978-218-1753	www.seniorcareinc.org	public_program_email@testing123.net	Home Care offers in-home services to elders	SeniorCare is the single entry point for homecare services.  Home Care coordinates and offers services to elders in their homes.  Through care management services are arranged for personal care, homemaking, meal preparation, chore services, personal emergency response systems, and other services.	Income level starting at 24,837; 60+ elders.	Sliding Scale	Based on a sliding scale.  Some programs have no charge.	Open year round.	Monday - Friday, 9am to 5pm.	English	Yes	Our knowledgeable staff in our Information and Referral department will let you know what to expect and what information you will need to have available.		Available	Transportation not available for this service.	Yes	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer program through RSVP program.  Multiple opportunities available in several communities.	No	\N	Yes	Handicapped accessible, first floor, parking, seats 60, wireless connection.	No	\N	No	\N	Cheryl	Krisko	978-281-1750	978-281-1753	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	978-282-1836	\N	Interpretor services available.	No	0	\N
510197	103729	2009-05-12	\N	Information, Referral and Advocacy	\N	5 Blackburn Center	\N	Gloucester	MA	01930	978-281-1750	866-927-1050	978-281-3093	978-218-1753	www.seniorcareinc.org	public_program_email@testing123.net	Information,Referral and Advocacy	Our information and referral specialists will provide a connection to the services or information that you, or your loved ones need.  We'll help you find services, explore care or housing choices, access benefits and find solutions for a range of aging related issues.	Available to any person seeking information related to elders and elder issues.	Free	\N	Open year round	Monday-Friday, 9:00 am to 5:00 pm	English	Not Applicable	\N		NA	NA	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Jane	Militello	978-281-1750	978-281-1753	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	978-282-1836	Connecting information is available for all areas of the United States.	\N	Not Applicable	0	\N
509504	103520	2009-05-13	\N	Tax Preparation Services	Earned Income Tax Credit  EITC	75 Central St.	\N	Peabody	MA	01960	978-531-0767 x222	\N	\N	978-531-0757	www.nscap.org	public_program_email@testing123.net	Tax preparation for low income workers	Tax preparation services for low income households whose gross income may qualify them for the Earned Income Tax Credit.	Typically below $42,000 gross annual income	Free	\N	Monday 3 to 6:30, Tuesday 9:00 to 12:00, Wednesday 3:00 to 6:30, Saturday 9:00 to 12:30 -- January 26, 2009 to April 13, 2009	Monday through Friday 9:00 to 5:00	English!DELIM!Spanish	Yes	\N		NSCAP has a parking lot at the rear of the building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteer Tax Preparers needed.	Not Applicable	\N	No	\N	No	\N	No	\N	Nimmi	Wadhwa	978-531-0767 x222	978-531-0757	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
509493	103520	2009-05-13	\N	Transition to Work	TtW	75 Central St.	\N	Peabody	MA	01960	978-532-8620x209	\N	\N	978-532-8618	www.nscap.org	public_program_email@testing123.net	Transition to work assistance for homeless head of household	TTW is a collaborative effort initiated between North Shore Community Action Program, Citizens for Adequate Housing, Help for Abused Women and Children (HAWC), Taking Care of Business (TCB), Turning Point and Wellspring House, shelters serving homeless families in transition. TtWs mission is to provide job related services to families referred to us from the shelters within the collaborative. The TtW Career Specialists endeavor to remove barriers to work by linking participants up with education, training, and employment opportunities in and around the North Shore area. Some of the services the Career Specialists provide include, but are not limited to, Career Counseling, GED tutoring funds, Job Preparedness and Life Skills Workshops, Job Training and Placement, and a Matching Fund program for Individual Development Savings Accounts.	Must be a shelter resident at one of the collaborative shelters.	Free	\N	Open year round.	Monday through Friday 9:00 to 5:00	English!DELIM!Spanish	Yes	\N		NSCAP has a parking lot at the rear of the building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers need to maintain files, input data, and manage the clothes closet.	Not Applicable	\N	No	\N	No	\N	No	\N	Deborah	Grace	978-532-8620 x209	978-532-8618	chester@testing123.net	Yes	STATE	MA	\N	\N	\N	No	0	\N
510153	103520	2009-05-13	\N	Energy Conservation	Weatherization	98 Main St.	\N	Peabody	MA	01960	978-531-0767	\N	\N	978-531-1012	www.nscap.org	public_program_email@testing123.net	Helps clients save on heating, cooling, and electric bills.	Heating system repairs - An emergency program to help homeowners who receive Fuel Assistance repair or replace heating systems that meet state criteria.  This program is for homeowners only.\nWeatherization Assistance Program - Designed to help you save money on your heating and cooling bills, and to make your home more comfortable.  All work is done by private contractors and may include attic and wall insulation and weather stripping.  You may be a homeowner or renter to be eligible.\r\n\r\nAppliance Management Program - Your refrigerator and/or freezer may be replaced at no cost if it is not energy efficient.  The energy manager will install energy saving light bulbs and inform you how to save on your electric bill.  You must be a National Grid customer to qualify and may be a tenant or homeowner to be eligible.	Contact program coordinator for eligibility criteria for each program.	Free	\N	Open year round	Monday through Friday 9:00 to 5:00	English	Yes	\N		NSCAP has a parking lot in the rear of the building.	\N	Yes	No	\N	Yes	See program coordinator for current information.	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Chuck	Gallant	978-531-0767 x118	978-531-1012	chester@testing123.net	Yes	TOWN	Peabody, MA!DELIM!Beverly, MA!DELIM!Salem, MA!DELIM!Danvers, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Middleton, MA	\N	\N	\N	No	0	\N
510164	103520	2009-05-13	\N	HOPWA	Housing Opportunities For People With AIDS	75 Central St.	\N	Peabody	MA	01960	978-531-5220	\N	\N	978-531-0757	www.nscap.org	public_program_email@testing123.net	Housing assistance for persons with HIV/AIDS	NSCAPs HIV/AIDS housing services are designed to utilize the resources available to those who are low-income and suffering from HIV and AIDS. The housing needs of those who are HIV positive are often very different than those who are not positive.	Participants must be living with HIV/AIDS and income must be below 80% of the median income for the area.	Free	\N	Open year round.	Monday through Friday 9:00 to 5:00	English	Yes	List of required documentation will be provided when appointment is made.		NSCAP has a parking lot in the rear of the building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	No	\N	No	\N	No	\N	No	\N	Heidi	Williams	978-531-5220 ext. 311	978-531-0757	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Lynnfield, MA!DELIM!Magnolia, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA	\N	\N	\N	No	0	\N
509537	103520	2009-05-13	\N	Salem Cyberspace	\N	98 Lafayette St	\N	Salem	MA	01960	978-740-6667	\N	\N	978-740-6668	www.nscap.org	public_program_email@testing123.net	Community technology center	Salem CyberSpace expands the educational and career opportunities for low-income youth and adults, thereby providing the foundation for achievement and self sufficiency. Utilizing state-of-the-art technology and innovative curricula, we successfully meet our goals for English Literacy, Computer Literacy, workplace readiness, and job internships.	In order to take classes at the Adult Computer Center, you will need to be at an ESL Level 3 and be at an 8th grade level.  First priority goes to students referred from the Career Centers and Mass Rehab/insurance companies.  Any empty seats are available for self-pay students.\nIn order to qualify for the LIFE Skills computer class offered at Beverly Bootstraps, you must be low-income and you will need to be at an ESL Level 3 and be at an 8th grade level.\r\nTo participate in the youth programs during the school year, you must be a Salem student on free/reduced lunch.\r\nTo participate in youth programs during the summer, you must meet the eligibility requirements of FirstJobs and apply through the North Shore Youth Career Services in Lynn, MA.\r\n	Sliding Scale	All youth programs are free.  Summer youth programs are internships and are paid positions.\nAdult programs are all fee-based.  Call for information.  Partial scholarships up to 50% are available for low-income students. Training grants are available through the Career Centers and Mass Rehab for students who meet the eligibility. \r\n	The office is open at 9am for classes, enrollment, and to answer questions. For public computer access, please see hours posted below.\nPublic Access Hours of Operation\r\nAs of May 12, 2009:\r\nMondays and Wednesdays:\r\n6:00pm - 8:00pm (time limit is one hour*)\r\nFridays and Saturdays:\r\n1:00pm - 5:00pm\r\n	Monday through Friday 9:00 to 5:00	English!DELIM!Spanish	No	A list of required documentation will be provided at the time of registration.		On-street parking.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Tutors are needed for youth, especially in math and science.	Yes	A resource library is provided for youth enrolled in programs.	Yes	A meeting area is available for community organizations or groups, space permitting.	Yes	A newsletter is printed twice a year.	Limited	Internet access is available during public access hours.	Linda	Saris	978-740-6667	978-740-6668	chester@testing123.net	No	EVERYWHERE		\N	\N	\N	No	0	\N
510208	103520	2009-05-13	\N	The Housing Law Project	\N	98 Main St.	\N	Peabody	MA	01960	978-531-0767	\N	\N	978-531-1012	www.nscap.org	public_program_email@testing123.net	Assistance to low-income tenants who are at risk of eviction	NSCAP's Housing Attorney works with low-income clients who have received notices to quit or the threat of such notices, or who have experienced housing-related difficulties.  The attorney also works with clients whose low income has resulted in foreclosure, bancruptcy, or related financial difficulties.  The Tenant/Landlord Specialist provides advocacy for clients, information on tenants' and landlords' rights and responsibilities and workshops on housing-related issues,  Mediation is also provided.	x	Free	\N	Open year round	9:00 to 5:00 Monday through Friday	English	Yes	\N		NSCAP has a parking lot at the rear of the building.	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Pro bono attorneys are needed for the Lawyer of the Day program at Housing Court.	Not Applicable	\N	No	\N	No	\N	No	\N	Lori	Fonseca	978-531-0767 x117	978-531-1012	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Salem, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Topsfield, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	\N	\N	\N	No	0	\N
509471	103520	2009-05-14	\N	Home Care	Elder Home Care; Homecare	98 Main St.	\N	Peabody	MA	01960	978-531-0767	\N	\N	978-531-1012	www.nscap.org	public_program_email@testing123.net	Home care services for low income elders	We all need a helping hand from time to time. When an elderly parent or a beloved partner requires assistance, our trained staff can provide you with the service that will best fit your needs.\nWhether it's a few hours of shopping a week or daily personal care for someone who is ill we can help.\r\nAn additional benefit for NSCAP clients is the knowledge that their dollars are returned to the community to help others.\r\n	Clients are referred to our program by the Elder Service agency in their communities. Families and/or Clients themselves who live in the North Shore should contact North Shore Elders; Beverly and Cape Ann should contact SeniorCare, Inc and clients who live in Lynn, Revere, Swampscott, Saugus and Nahant should contact Greater Lynn Senior Services for eligibility requirements. Clients who are not eligible for services from these agencies can contact our Program directly and their pay rates will be determined by a sliding scale according to their personal income.	Sliding Scale	Sliding scale is based on annual income.	Open year round	Monday through Friday 9:00 to 5:00.  Services are available according to the needs of clients.  We can assist during nights and weekends, but not overnight.	English!DELIM!Portuguese!DELIM!Spanish	Yes	Annual income and list of services needed.		NSCAP has a parking lot at the rear of the building.	Transportation is available for clients who request companion services.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	No	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Liz	Chavez	978-531-0767 x129	978-531-1012	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Peabody, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Lynnfield, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Beverly, Danvers, Essex, Hamilton, Ipswich, Salem, Marblehead, Middleton, Peabody, Topsfield, Wenham, Manchester, Lynn, Lynnfield, Swampscott, Saugus, Revere, Nahant	Russian	No	0	\N
510219	103795	2009-05-15	\N	Dept. of Public Services	\N	50 Farm Ave.	\N	Peabody	MA	01960	978-536-0600	\N	978-536-0600	\N	www.peabody-ma.gov	public_program_email@testing123.net	All Public Works Questions/Concerns	Sidewalk & street repair, water/sewer issues, snowplow complaints	City resident	Free	\N	Open year round	Mon - Fri 8:00 - 4:00	English	Not Applicable	\N		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Alice	Noble	978-536-7119	\N	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	City of Peabody only	\N	Not Applicable	0	\N
510230	103795	2009-05-15	\N	Dept. of Parks, Recreation & Forestry	\N	50 Farm Ave.	\N	Peabody	MA	01960	978-536-0600	\N	\N	\N	www.peabody-ma.gov	public_program_email@testing123.net	Parks, Recreation & Forestry	Questions regarding city owned trees, questions/concerns about city parks, questions about various recreation programs & trips	City Resident	Yes	Fees charged for recreation programs/trips	open year round	Mon - Fri, 8:00 - 4:00	English	No	\N		For Recreation only	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Richard	Walker	978-536-7131	\N	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	\N	\N	Yes	0	\N
510252	103806	2009-05-16	\N	St. John's Thrift Shop	\N	19 Chestnut St	\N	Peabody	MA	01960	978-531-0002	\N	\N	\N	St. John the Baptist Church Peabody	public_program_email@testing123.net	Clothing, shoes, household items for the family.	This is a Thrift Shop with items at very reasonable prices.  Items are free to those persons in need.	North Shore area	Sliding Scale	Items are for sale.  Proceeds go to St. John the Baptist Church.  Items free to those in need.	Closed Holy Week, 2 weeks in July; 2 weeks at Christmas.  Only open 2 days in winter.	Thursdays 9-12   Fridays and Sat. 9-1\nOnly Friday and Sat. 9-1 in winter.	English	Not Applicable	\N		Large parking lot.	None	No	No	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Volunteers to help during hours of operation.  Yard work and general cleaning .	No	\N	No	\N	No	\N	No	\N	Jane	Harris	978-531-8659	\N	chester@testing123.net	Yes	COUNTY	Middlesex County!DELIM!Essex County	\N	\N	Limited Spanish.	Not Applicable	0	\N
510241	103773	2009-05-18	\N	Pioneer House	\N	2 Washington Street	\N	Peabody	MA	01960	978-977-7755	\N	\N	978-538-7213	hes-inc.org (follow the links to Pioneer House)	public_program_email@testing123.net	Pioneer House serves adults with a history of mental illness	Our Clubhouse is a community of encouragement, providing its members an opportunity to live and work successfully within society. We focus upon health and wellness with those we serve. To that end, we provide job placement, social development, and integration into the community as primary services.	All adults 18+ with a primary diagnosis of mental illness are eligible if they live in our service area. (see cities below). Both men and women with no income restrictions.	Free	DMH funded. All program costs covered.	Open year round including all holidays	M through F 9 to 5\nTuesday and Wednesday until 8pm\r\nSaturdays noon to 5\r\nholidays noon to 4	English	Yes	Need to schedule tour and intake. Referral required		Parking nearby	Only to job sites where no public transportation available, and member doesn't have own car.	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	Limited. Need to check with us if interested in volunteering.	No	\N	No	\N	Yes	Internal only	No	\N	Larry	Marshall	978-977-7755	978-538-7213	chester@testing123.net	Yes	TOWN	Ipswich, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Rowley, MA!DELIM!Salem, MA!DELIM!Magnolia, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Hamilton, MA	\N	\N	\N	No	0	\N
510263	103795	2009-05-18	\N	Health Department	\N	24 Lowell Street	\N	Peabody	MA	01960	978-538-5926	\N	\N	\N	www.peabody-ma.gov	public_program_email@testing123.net	Health related issues	Public Health nurse administers vaccines, flu shots, in charge of school nurses, compaints about cleanliness of residences & businesses.  Inspectors inpsect businesses, residences, apartments, pools.	City resident	Yes	Depends on services	Open all year	Mon through Wed, 8:30 - 4:00; Thursday 8:30 - 7:00; Friday 8:30 - 12:30	English	Yes	Depends on services needed.		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Yes	Small conference room on the lower level, Wiggin Auditorium on 2nd floor	No	\N	No	\N	Deborah	Osgood	978-538-5926	978-538-5990	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	\N	\N	No	0	\N
510274	103828	2009-05-26	\N	TRansitions	lsa/home	149 Washington St Peabody Ma	91 Liberty St Lynn Ma	Peabody	MA	01969	978-538-5392	\N	\N	978-538-5490	lsahome.org	public_program_email@testing123.net	transitional housing program	The transitions program provides a supportive environment for homelss individuals to gain the skills necessary to become self sufficient. It is our goal to provide the support necessary for out participants to move from homelessness to permanent housing	HOMELESS..........	Yes	When applicable // 30%	open year round	24hrs.	English!DELIM!Spanish	Yes	referral/assessment/		yes on street	on route	No	No	\N	Yes	can varie .. from 3 months to 6	\N	149 Washington St	\N	Peabody	MA	01960	Yes	\N	Not Applicable	\N	Yes	LIMITED to 15 persons	Yes	\N	No	\N	Donna	Cahill	978-538-5392	978-538-5490	chester@testing123.net	Yes	EVERYWHERE		\N	\N	\N	No	0	\N
510285	103839	2009-05-27	\N	Making a Difference in Peabody Foundation, Inc.	\N	24 Main Street	Suite 2	Peabody	MA	01960	978-317-4388	\N	\N	978-854-0060	\N	public_program_email@testing123.net	Oil, rent, food, gas, clothing, emergency disaster, medical,	Oil, rent, food, gas, clothing, emergency disaster, medical, school supplies, camp	Peabody reseident, emergency situation, any age, low income, unemployed, medical illness, domestic vilolence, financially distressed, can be employed, can receive state or federal benefits, must be able to verify through documentation.	Free	\N	year round aid distributed regularly	M-F 9-5PM	English	Yes	Please email or fax letter fo request including your name, address, telephone number, and reason for need of support.  Please include backup documentation such as income, copy of rent eviction notice, shut off notice, past due gas bill, or whatever may apply.		\N	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	administration support, liaison support to various agencies in Peabody.	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	Scott	Frasca	978-317-4388	978-854-0060	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	\N	\N	No	0	\N
510307	103850	2009-06-10	\N	Museum Pass Program	\N	82 Main Street	\N	Peabody	MA	01960	978-531-0100	\N	\N	978-532-1797	www.peabodylibrary.org	public_program_email@testing123.net	Free or reduced admissionsn to area museums	The Library offers 13 daily passes to local residents at the main library, and allows one pass per day per family. To reserve a pass, call 978-531-0100.\nThe Peabody Institute Library thanks the Rotary Club of Peabody, the Friends of the Peabody Institute Libraries, and the Peabody Cultural Council for their generous sponsorship.\r\nChildren's Museum\r\nMuseum Location: Boston (Pass admits up to 4 people at $2 per person.)\r\nHarvard Museum of Natural History\r\nMuseum Location: Cambridge (Pass admits 2 adults and 2 children at $2 per person.)\r\n\r\nThe House of the Seven Gables\r\nMuseum Location: Salem (Free; admits 2 adults, 4 children maximum, for standard tours only.)\r\n\r\nIsabella Stewart Gardner Museum\r\nMuseum Location: Boston (Pass admits 4 people at $5 per person, Tuesdays through Fridays; admits 2 people at $5 per person on Saturdays, Sundays and holidays.)\r\n\r\nJohn F. Kennedy Library and Museum\r\nMuseum Location: Boston ($3 per person with pass, admits 2 people. Children 12 and under admitted free.)\r\n\r\nMassachusetts Parks Passes\r\nLocation: Massachusetts (Pass provides free parking for 1 vehicle at over 50 day-use park facilities.)\r\n\r\nMuseum of Fine Arts\r\nMuseum Location: Boston (Pass admits 2 people at $5 per person.)\r\n\r\nMuseum of Science\r\nMuseum Location: Boston (Pass admits 4 people at $5 per person.)\r\n\r\nNew England Aquarium\r\nMuseum Location: Boston (Available September-June; $8 per person with pass, admits 4 people)\r\n\r\nPeabody-Essex Museum\r\nMuseum Location: Salem (Pass admits 2 people at $5 per person.)\r\n\r\nRoger Williams Park Zoo\r\nMuseum Location: Providence, RI\r\n(Pass offers $3 discount on regular ticket price for any combination of up to four (4) Adult, Child or Senior tickets.)\r\n\r\nSports Museum of New England Boston\r\nMuseum Location: TD Banknorth Garden, Levels 5 and 6\r\nFree; admits up to 6 people.\r\n*Hours are subject to Events Schedule at TD Banknorth Garden. For updates call 617.624.1234 or click here.*\r\n\r\nZoo New England (Franklin Park and Stone Zoos)\r\nMuseum Location: Boston and Stoneham (Pass admits 2 adults, 4 children. Free Admission.)\r\n\r\nGeneral Pass Information:\r\n\r\n1.Peabody residents may reserve museum passes in advance. Passes must be picked up by 10:00 a.m., and returned by 9:00 a.m. the next day (except coupon passes, which do not need to be returned). For each pass taken, the patron must have a library account in good standing.\r\n\r\n2.After 10:00 a.m., if a pass has not been picked up, it becomes available to Peabody residents on a first-come, first-served basis. After 10:30 a.m., non-residents may borrow the passes.\r\n\r\n3.Multi-use passes must be returned to the Main Library. For patrons' convenience, these passes may be returned in the Pass Return Box located outside of the Children's Room entrance at the Main Library. Patrons who return passes late will be charged an overdue fine of $5 per day.\r\n\r\nThis program is supported in part by a grant from the Peabody Cultural Council, a local agency which is supported by the Massachusetts Cultural Council, a state agency. 	Peabody residents may reserve museum passes in advance. Passes must be picked up by 10:00 a.m.  After 10:00 a.m., if a pass has not been picked up, it becomes available to Peabody residents on a first-come, first-served basis. After 10:30 a.m., non-residents may borrow the passes.	Free	There is no charge to borrow a pass.  The fees charged by the museums with library passes are outlined above.	Varies by museum.	Varies by museum.  Passes may be picked up during library hours.	English	Yes	Library card		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	The Peabody Library has large collection of books, DVDs and CDs.	Yes	The Peabody Library has several meeting rooms available for use by nonprofit or governmental agencies.  There is no fee for meeting room use. For complete procedures and to book a room call the library at 978-531-0100 ext. 11	Not Applicable	\N	Yes	\N	Melissa	Rauseo	978-531-0100 ext. 14	978-532-1797	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	\N	\N	No	0	\N
510340	103861	2009-06-01	\N	Free Legal Services	Legal Aid	37 Friend Street	Third Floor	Lynn	MA	01902	781-599-7730	800-747-5056	\N	781-595-2022	www.neighborhoodlaw.org	public_program_email@testing123.net	Free civil legal services	We provide free civil legal services in the areas of Elder Law (60 and over) Unemployment, Family Law, Landlord/tenant, Public Benefits (Food Stamps, EAEDC, EA shelter, MassHealth, SSI/SSDI), Consumer/Bankruptcy, Immigration and Community Development	Must be low income (125% of poverty) or elderly	Free	Must meet income guidelines	Open year round	Monday through Friday, 9 a.m. to 5 p.m.	English!DELIM!Spanish!DELIM!Khmer	No	Any notices/letters/court documents relating to the case.		\N	\N	Yes	No	\N	No	But we do have limited resources and only handle a certain number of new cases each week.	\N	\N	\N	\N	\N	\N	Yes	Administrative and secretarial volunteers needed in the fall/winter	No	\N	No	\N	No	\N	No	\N	Corine	Reardon	781-244-1420	781-595-2022	chester@testing123.net	No	TOWN	Amesbury, MA!DELIM!Ipswich, MA!DELIM!North Reading, MA!DELIM!Andover, MA!DELIM!Lawrence, MA!DELIM!Peabody, MA!DELIM!Littleton, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lowell, MA!DELIM!Rowley, MA!DELIM!Billerica, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Salisbury, MA!DELIM!Chelmsford, MA!DELIM!Magnolia, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Tewksbury, MA!DELIM!Dracut, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Dunstable, MA!DELIM!Tyngsborough, MA!DELIM!Essex County, MA!DELIM!Methuen, MA!DELIM!Wenham, MA!DELIM!Georgetown, MA!DELIM!Middleton, MA!DELIM!West Newbury, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Westford, MA!DELIM!Groveland, MA!DELIM!Newbury, MA!DELIM!Wilmington, MA!DELIM!Hamilton, MA!DELIM!Newburyport, MA!DELIM!Haverhill, MA!DELIM!North Andover, MA	\N	All of Essex County and Northern Middlesex County	We utilize Language Line telephone translation services to serve clients speaking any other languages.	Yes	0	\N
510351	103872	2009-06-02	\N	Inn Transition	IT	42 Washington St.	42 Washington St.	Peabody	MA	01960	978-531-9951	\N	978-531-9951	978-977-9008	www.cahns.org	public_program_email@testing123.net	Homeless shelter for families	Providing shelter, referrals, education, food, sober living for homeless families	Must be homeless and have at least one child in your custody, or six months pregnant and emergency assistance eligible.	Free	There are no fees.  Referred by the Department of Transition Assistance and the department of Public Health.	Open year round.	24/7	English!DELIM!Spanish	Yes	Referral is through the Department of Transitional Assistance-Institute for Health and Recovery		On site parking available.	Some transportation, Bus stop within walking distance	Yes	No	\N	Yes	For non-emergency assistance families	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	Yes	Living community room	Yes	Two times per year	No	Internet is offered to guests only.	Betsy	Leeman	978-531-9775	978-977-9008	chester@testing123.net	Yes	STATE	MA	978-531-9951	\N	\N	No	0	\N
510362	103905	2009-06-08	\N	The Salem Mission,Inc.	\N	56 Margin Street	\N	Salem	MA	01970	978-744-0500	\N	\N	978-740-3832	salemmission.org	public_program_email@testing123.net	Homeless Service Provider	Providing shelter, community meals program, mental health services, medical clinic, street outreach, housing on campus, case management	homeless adult men and women over the age of 18	Free	\N	Open year round	24/7	English	Not Applicable	ID		City parking across the street	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Yes	Contact David Halter at 978-744-0500 ext. 208	Not Applicable	\N	Yes	Confernece room that will seat 8-10 people	Yes	Quarterly	No	\N	Mark	Cote	978-744-0500	978-740-3832	chester@testing123.net	No	STATE	MA	\N	\N	\N	Not Applicable	0	\N
510373	103916	2009-06-09	\N	Basic Needs	\N	55 Lynn Shore Drive	\N	Lynn	MA	01902	781-593-2312	\N	\N	781-581-3270	www.ccab.org	public_program_email@testing123.net	Assistance for food, housing and utilities	As resources allow, we provide financial assistance for food, rent, fuel or utilities.  Comprehensive information and referral, financial education and budgeting assistance is available.	Available to all	Free	\N	Open year round	Monday - Friday, 8:30 am - 5:00 pm	English!DELIM!Spanish	No	\N		\N	\N	Yes	Not Applicable	\N	Not Applicable	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	Not Applicable	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!North Reading, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Also Arlington, Chelsea,Everett, Malden, Medford, Melrose, Reading, Revere, Stoneham Wakefield, Winchester and Woburn	\N	Not Applicable	0	\N
510384	103916	2009-06-09	\N	Education Center and Job Training	\N	55 Lynn Shore Drive	\N	Lynn	MA	01902	781-593-2312	\N	\N	781-581-3270	www.ccab.org	public_program_email@testing123.net	Pre-GED and GED classes and office skills training	The Ed Center offers flexible pre-GED and GED classes, life skills training, career counseling and job readiness workshops for youth.  Office skills training for unemployed and underemployed adults is also available.	Youth 16 - 21	Free	\N	Open year round	Monday - Friday, 8:30 am - 5:00 pm	English!DELIM!Spanish	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!North Reading, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Also Arlington, Chelsea, Everett, Malden, Medford, Melrose, Reading, Revere, Stoneham, Wakefield, Winchester and Woburn	\N	No	0	\N
509878	103685	2009-06-09	\N	Financial Literacy Classes	\N	74 Pleasant Street	\N	Gloucester	MA	01930	978-283-3055	\N	\N	617-832-7486	\N	public_program_email@testing123.net	Financial Literacy Program	Available to give financial literacy classes to groups who would benefit.	All groups eligible	Free	\N	Open year round	As determinted.	English	Not Applicable	\N		Given at groups location	\N	Not Applicable	Not Applicable	\N	Not Applicable	\N	\N	60 Prospect Street	\N	Gloucester	MA	01930	Not Applicable	\N	No	\N	Yes	Financial Literacy classes given at location of group	Not Applicable	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	Yes	TOWN	Rockport, MA!DELIM!Magnolia, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Gloucester, MA	\N	Essex, too	\N	Not Applicable	0	\N
509867	103685	2009-06-09	\N	Food cards	\N	74 Pleasant Street	\N	Gloucester	MA	01930	978-283-3055	\N	\N	617-832-7486	\N	public_program_email@testing123.net	Food cards for Shaws Supermarket	Food cards - $10 for 1 or 2 in a family; $20. for 3 or more.	All eligible every three months.	Free	\N	open year round	Monday noon - 4\nTuesday and Wednesdays - 10 - 4\r\nThursday 9 - 1 \r\nClosed Fridays	English	Yes	ID for self and family members		On street	\N	No	Not Applicable	\N	Not Applicable	\N	\N	60 Prospect Street	\N	Gloucester	MA	01930	Not Applicable	\N	No	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	Yes	TOWN	Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Gloucester, MA	\N	Essex, too	\N	No	0	\N
509856	103685	2009-06-09	\N	Rental/Mortgage Assistance	\N	74 Pleasant Street	\N	Gloucester	MA	01930	978-283-3055	\N	\N	617-832-7486	\N	public_program_email@testing123.net	Help keep people in housing or move to new housing	Pay part of back rent for those whose rent is not subsidized. Help pay first/last/security for those who have to move. Fisherman funds available.	Individuals (18 and over) and families in Gloucester, Essex, Rockport, Manchester who are able are sustainable.	Free	\N	open year round	Mondays noon to 4 p.m.\nTuesdays and Wednesdays 10 - 4\r\nThursday 9 - 1 p.m.\r\nClosed Fridays	English	Yes	ID for all those in household. Proof of income.		On street	\N	No	Not Applicable	\N	Not Applicable	\N	\N	60 Prospect Street	\N	Gloucester	MA	01930	Not Applicable	\N	No	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	Yes	TOWN	Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Gloucester, MA	\N	Essex also	\N	Yes	0	\N
509845	103685	2009-06-09	\N	Utility Assistance	\N	74 Pleasant Street	\N	Gloucester	MA	01930	978-283-3055	\N	\N	617-832-7486	\N	public_program_email@testing123.net	Help with gas, electric and oil bills	We partner with other agencies to help pay gas, electric and oil bills for a maximun of every two years and three times in a lifetime. Fisherman funds available.	Individuals (18 and over) and families in Gloucester,Rockport, Essex and Manchester	Free	\N	open year round	Monday - 12 noon to 4 p.m.\nTuesdays and Wednesdays - 10 a.m. - 4 p.m.\r\nThursdays - 9 a.m. - 1 p.m.\r\nClosed Fridays. Call for appointment.	English	Yes	Identification for self and all in family; utility bill in question		on street	\N	No	Not Applicable	\N	Not Applicable	\N	\N	60 Prospect Street	\N	Gloucester	MA	01930	Not Applicable	\N	No	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	Yes	TOWN	Rockport, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Essex County, MA!DELIM!Gloucester, MA	\N	\N	\N	No	0	\N
510406	103916	2009-06-09	\N	Fathers' Support	\N	55 Lynn Shore Drive	\N	Lynn	MA	01902	781-593-2312	\N	\N	781-581-3270	www.ccab.org	public_program_email@testing123.net	Support groups and assistance for fathers	Outreach, support groups and assistance to help men develop the tools to become a positive influence in their children's lives	Open to all	Free	\N	Groups meet weekly in eight to ten-week cycles	Once a week in the evening	English	Yes	\N		\N	\N	Yes	No	\N	No	\N	\N	\N	\N	\N	\N	\N	Not Applicable	\N	No	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!North Reading, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Also Arlington, Chelsea, Everett, Malden, Medford, Melrose, Reading, Revere, Stoneham, Wakefield, Winchester and Woburn	\N	No	0	\N
510417	103916	2009-06-09	\N	Parent Aide	\N	55 Lynn Shore Drive	\N	Lynn	MA	01902	781-593-2312	\N	\N	781-581-3270	www.ccab.org	public_program_email@testing123.net	Home-visiting program to help improve parental self esteem	This program provides weekly home visits, parenting classes, budget assistance and support to parents and their children.	Open to all	Free	\N	Open year round	As scheduled	English	Yes	\N		\N	\N	Not Applicable	Not Applicable	\N	No	\N	\N	\N	\N	\N	\N	\N	Yes	\N	No	\N	No	\N	No	\N	No	\N	Sarah	Bartley	781-593-2312	781-581-3270	chester@testing123.net	No	TOWN	Ipswich, MA!DELIM!North Reading, MA!DELIM!Peabody, MA!DELIM!Rockport, MA!DELIM!Beverly, MA!DELIM!Lynn, MA!DELIM!Salem, MA!DELIM!Boxford, MA!DELIM!Lynnfield, MA!DELIM!Swampscott, MA!DELIM!Danvers, MA!DELIM!Manchester-by-the-Sea, MA!DELIM!Marblehead, MA!DELIM!Topsfield, MA!DELIM!Essex County, MA!DELIM!Wenham, MA!DELIM!Middleton, MA!DELIM!Gloucester, MA!DELIM!Nahant, MA!DELIM!Hamilton, MA	\N	Also Arlington, Chelsea, Everett, Malden, Medford, Melrose, Reading, Revere, Stoneham, Wakefield, Winchester and Woburn	\N	No	0	\N
510296	103850	2009-06-10	\N	Public Programs	\N	82 Main Street	\N	Peabody	MA	01960	978-531-0100	\N	\N	978-532-1797	www.peabodylibrary.org	public_program_email@testing123.net	Free recreational and informational events for all ages.	The Library offers hundreds of events for adults, children and teenagers every year.  For a list of current and upcoming programs, check the library website: www.peabodylibrary.org	Library programs are open to all regardless of residency, gender, or income.  Children and teen programs  may be limited by age.	Free	\N	Year round.	Varies.	English	Yes	Most library events require advance sign up due to limited space.  No proof of residency or other identification will be required.		The Library has one free lot next to the Knights of Columbus Building.  A metered lot is accessible by Wallis Street and metered on street parking is available.	\N	Yes	No	\N	Yes	In the event that a library program fills to capacity, a waiting list will be maintained.	\N	\N	\N	\N	\N	\N	Not Applicable	\N	Not Applicable	The Peabody Library has large collection of books, DVDs and CDs.	Not Applicable	The Peabody Library has several meeting rooms available for use by nonprofit or governmental agencies.  There is no fee for meeting room use. For complete procedures and to book a room call the library at 978-531-0100 ext. 11	Yes	Adult programs are advertised in a monthly news letter.  To subscribe go to http://www.nextreads.com/join.aspx?SID=73aa946f-cb52-4292-b3ec-eac510cf8965	Yes	The Library provides free Internet access and free wireless access during library hours.  All users must agree to abide by the terms of our Acceptable Use Policy.	Melissa	Rauseo	978-531-0100 ext. 14	978-532-1797	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	Library programs are open to residents of other cities and towns as well.	\N	Yes	0	\N
510318	103850	2009-06-10	\N	Summer Food Program	"Library Lunches"	82 Main Street	\N	Peabody	MA	01960	978-531-0100	\N	\N	978-532-1797	www.peabodylibrary.org	public_program_email@testing123.net	Free summer lunches and activities for children under 18.	The program provides free lunches and fun activities for children under the age of 18.  The goal of this program is to ensure a stable nutrition source for children when school is out for the summer.  Lunch and activities will be available each day of the program.\nThe Summer Food Program is a program administered by the United States Department of Agricultural, the Massachusetts Department of Elementary and Secondary Education and Project Bread.	Open to all children under the of 18.  No identification or sign ups required.  Children under the age of 8 years old must be accompanied by a responsible party over the age of 11 years old.	Free	\N	Open in the summer.  June 29- August 6, 2009	Monday- Thursday 11:30am-1pm	English	No	\N		\N	\N	Yes	No	The summer food program provides lunch and activities to children under 18.  It is not a source of child care.  Children under the age of 8 years old must be accompanied by a responsible party over the age of 11 years old.	Not Applicable	http://www.northshoreport.org/cgi-bin/mvhub/program.pl?agency_id=103850	\N	\N	\N	\N	\N	\N	Yes	Adult and teen volunteers are needed to assist with the distribution of lunches and activities.  Please contact Melissa Rauseo at 978-531-0100 ext. 14 if you are interested in volunteering.  All volunteers must pass a background check.	Yes	The Peabody Library has large collection of books, DVDs and CDs.	Not Applicable	The Peabody Library has several meeting rooms available for use by nonprofit or governmental agencies.  There is no fee for meeting room use. For complete procedures and to book a room call the library at 978-531-0100 ext. 11	No	\N	Yes	\N	Melissa	Rauseo	978-531-0100 ext. 14	978-532-1797	chester@testing123.net	Yes	TOWN	Peabody, MA	\N	\N	\N	No	0	\N
\.


--
-- Data for Name: program_category; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY program_category (category_id, program_id) FROM stdin;
802233	509460
802035	509460
801683	509460
802112	509460
802057	509460
803245	509460
802211	509460
803113	509460
802123	509460
802167	509460
802288	509460
802222	509460
803234	509460
801034	509460
803971	509460
802178	509460
803993	509471
801364	509482
804950	509482
804961	509493
801793	509493
801826	509493
801309	509493
801782	509493
801749	509493
801903	509504
801584	509515
801243	509515
802002	509526
801617	509537
801177	509537
801452	509537
801353	509548
801826	509548
801419	509548
801584	509548
802970	509603
804587	509658
803256	509658
803135	509658
803157	509658
803146	509658
801760	509658
802057	509658
803245	509658
801793	509658
803201	509658
803212	509658
801243	509658
804466	509658
804444	509658
801177	509658
801463	509658
801782	509658
801595	509658
803168	509658
801342	509658
804015	509658
801265	509658
803190	509658
802079	509658
801463	509680
801628	509680
801430	509680
801980	509680
801914	509680
801353	509680
803212	509680
801606	509680
801463	509691
801331	509691
801628	509691
801430	509691
801980	509691
801573	509691
801914	509691
801441	509691
801386	509691
801353	509691
803212	509691
801496	509691
801606	509691
801441	509724
804576	509724
801474	509724
804697	509724
802629	509735
802662	509735
802651	509735
801243	509757
804587	509801
804565	509801
801452	509801
801342	509801
801265	509801
801386	509801
804664	509801
801496	509801
804620	509812
804499	509812
801496	509812
801188	509812
801485	509812
801529	509812
801430	509834
802222	509845
803036	509845
802222	509856
802123	509856
802222	509867
803036	509867
802222	509878
803036	509878
804961	509790
801364	509790
801958	509790
801969	509790
801705	509779
801793	509779
801177	509779
801958	509779
801969	509779
801199	509779
804587	509900
804444	509900
803729	509900
804499	509900
801342	509900
804620	509900
801386	509900
803784	509900
803696	509900
804664	509900
801188	509900
801529	509900
802167	509889
803784	509889
804664	509889
803729	509889
804499	509889
803619	509911
804048	509911
800253	509911
804818	509911
802244	509911
800286	509911
803586	509911
800198	509823
802299	509823
802244	509823
800748	509933
800363	509933
800759	509933
800737	509933
800429	509933
803883	509933
800660	509933
800539	509933
803894	509933
800715	509933
800099	509933
800418	509933
800495	509933
803014	509944
800781	509944
804642	510010
800275	510010
804499	510010
801386	510010
804037	510010
800220	510010
804664	510010
801298	510010
801188	510010
802244	509746
802233	509955
804917	509955
802453	509955
801683	509955
804389	509955
802871	509955
802057	509955
800429	509955
802442	509955
802090	509955
802277	509955
802981	509955
802134	509955
801782	509955
803971	509955
802024	509955
801958	509955
800891	509955
802145	509955
800869	509955
803025	509955
802035	509955
801969	509955
803080	509955
804323	509955
803014	509955
801353	509955
804631	509955
803113	509955
802123	509955
801540	509955
801892	509955
802002	509955
803069	509955
800847	509955
802167	509955
802222	509955
803047	509955
802992	509955
801914	509955
803036	509955
801034	509955
802255	509955
803982	509955
802970	509955
801903	509955
800352	509955
802046	509955
801749	509955
802178	509955
804774	509955
804081	509966
804499	509966
802244	509966
804026	509966
804664	509966
802409	509966
802299	509966
804136	509966
804917	510032
803047	510032
802992	510032
803080	510032
803036	510032
802277	510032
803014	510032
803113	510032
802981	510032
802970	510032
800891	510032
803069	510032
802024	510043
802200	510043
802167	510065
801364	509768
801331	509768
801628	509768
801276	509768
801441	509768
804950	509768
801584	509768
801606	509768
801144	510087
803960	510087
803971	510087
803025	510142
804917	510142
802981	510142
802992	510142
802970	510142
801617	509713
804565	509713
801573	509713
801760	509713
801441	509713
803696	509713
802354	509713
803674	509713
801144	510076
803960	510076
803025	510076
804917	510076
804004	510076
802629	510076
803971	510076
802662	510076
803938	510076
803971	510197
804444	509559
803377	509559
804631	509559
803366	509559
803322	509559
800913	509559
801067	509559
800110	510219
800165	510219
800088	510219
800099	510219
800099	510230
800165	510252
800099	510252
803608	510241
803553	510241
803531	510241
803443	510241
803542	510241
801155	510241
803465	510241
803487	510241
803498	510241
800088	510263
800099	510263
804202	510274
803025	510274
802233	510274
804389	510274
804323	510274
801705	510274
803014	510274
802442	510274
802277	510274
802134	510274
803520	510274
804235	510274
801980	510274
804246	510274
804213	510274
802255	510274
802321	510274
800352	510274
802970	510274
802079	510274
802849	510274
800880	510274
803498	510274
802167	510285
802222	510285
802277	510285
802024	510285
802013	510285
802156	510285
802200	510285
804433	510296
800077	510296
800066	510296
800308	510296
801298	510296
804994	510296
801364	510296
804499	510296
800297	510296
800231	510296
800022	510296
800011	510296
804675	510296
804587	510296
800033	510296
801430	510296
804576	510296
800198	510296
800000	510296
801353	510296
800143	510296
804631	510296
801496	510296
801529	510296
801177	510296
804466	510296
800275	510296
804565	510296
800253	510296
801595	510296
801342	510296
800220	510296
801903	510296
804664	510296
800033	510307
800000	510307
800077	510307
800055	510307
804576	510318
804565	510318
804081	510318
804499	510318
800253	510318
804092	510318
802244	510318
800066	510318
804026	510318
804664	510318
800220	510318
800176	510318
800308	510329
800220	510329
800176	510329
804994	510329
800869	510340
803542	510340
802057	510340
803311	510340
803179	510340
803113	510340
802838	510340
801892	510340
803388	510340
803124	510340
803597	510340
804378	510340
804213	510340
803762	510340
803289	510340
804004	510340
803377	510340
802255	510340
801903	510340
803410	510340
803366	510340
802079	510340
801749	510340
803300	510340
802233	510351
804389	510351
803542	510351
802871	510351
801232	510351
803685	510351
802211	510351
801705	510351
802981	510351
803674	510351
803641	510351
801485	510351
801243	510351
802288	510351
803553	510351
801463	510351
801782	510351
801474	510351
803322	510351
802079	510351
801199	510351
802464	510351
803025	510351
802035	510351
803080	510351
801375	510351
801353	510351
802310	510351
803113	510351
802860	510351
802123	510351
801540	510351
801738	510351
802002	510351
801529	510351
802167	510351
801177	510351
802222	510351
802992	510351
802530	510351
801595	510351
802244	510351
803762	510351
801342	510351
803773	510351
801265	510351
801441	510351
804279	510351
801034	510351
802255	510351
802970	510351
802849	510351
802178	510351
804774	510351
802233	510362
803080	510362
802200	510362
804268	510362
803476	510362
802277	510362
802090	510362
803014	510362
803113	510362
802255	510362
802024	510362
802046	510362
803498	510362
802145	510362
802167	510373
801441	510384
801782	510384
801496	510384
801276	510384
803674	510395
802508	510395
800495	510395
803784	510406
803674	510406
803685	510406
803773	510417
803784	510417
801969	510417
802508	510417
\.


--
-- Data for Name: search_log; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY search_log (search_id, session_id, ip_address, "timestamp", search_phrase, referer) FROM stdin;
\.


--
-- Data for Name: synonym_stem; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY synonym_stem (synonym_id, word_stem, submitted_word) FROM stdin;
900000	add	ADD
900000	adhd	ADHD
900000	hadd	HADD
900011	alcohol	alcohol
900011	drink	drinking
900011	drunk	drunk
900022	assist	assist
900022	help	help
900033	cessat	cessation
900033	quit	quit
900044	child	child
900044	children	children
900044	kid	kid
900044	youth	youth
900055	class	class
900055	lesson	lesson
900066	comput	computer
900066	technologi	technology
900066	internet	internet
900077	counsel	counseling
900077	therapi	therapy
900088	detoxif	detoxification
900088	detox	detox
900099	disabl	disabled
900099	handicap	handicapped
900110	drug	drug
900110	substanc	substance
900121	elder	elder
900121	elderli	elderly
900121	senior	senior
900132	fit	fitness
900132	exerc	exercise
900143	health	health
900143	medic	medical
900143	healthcar	healthcare
900154	infant	infant
900154	babi	baby
900165	job	job
900165	career	career
900165	vocat	vocation
900165	employ	employment
900176	literaci	literacy
900176	read	reading
900176	write	writing
900176	languag	language
900187	marit	marital
900187	marriag	marriage
900198	methadon	methadone
900198	heroin	heroine
900209	monei	money
900209	financ	finance
900220	nurs	nurse
900220	rna	rna
900231	nutrition	nutrition
900231	nutritionist	nutritionist
900242	pregnant	pregnant
900242	pregnanc	pregnancy
900253	regist	register
900253	registr	registration
900264	teen	teen
900264	adolesc	adolescent
900275	televis	television
900275	tv	tv
900286	tenant	tenant
900286	rent	rent
900286	renter	renter
900286	rental	rental
900297	tobacco	tobacco
900297	smoke	smoking
900308	util	utility
900308	electr	electricity
900308	energi	energy
900308	fuel	fuel
900308	heat	heat
900308	ga	gas
900308	oil	oil
900319	vote	vote
900319	voter	voter
900319	elect	elect
\.


--
-- Data for Name: town; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY town (town_state, county, region, display) FROM stdin;
Bedford, MA	Middlesex County	\N	t
Billerica, MA	Middlesex County	Greater Lowell	t
Chelmsford, MA	Middlesex County	Greater Lowell	t
Dracut, MA	Middlesex County	Greater Lowell	t
Littleton, MA	Middlesex County	\N	t
Lowell, MA	Middlesex County	Greater Lowell	t
Tewksbury, MA	Middlesex County	Greater Lowell	t
Tyngsborough, MA	Middlesex County	Greater Lowell	t
Westford, MA	Middlesex County	Greater Lowell	t
Amesbury, MA	Essex County	\N	t
Andover, MA	Essex County	Greater Lawrence	t
Haverhill, MA	Essex County	Greater Lawrence	t
Lawrence, MA	Essex County	Greater Lawrence	t
Merrimac, MA	Essex County	\N	t
Methuen, MA	Essex County	Greater Lawrence	t
Acton, MA	Middlesex County	\N	f
Arlington, MA	Middlesex County	\N	f
Ashby, MA	Middlesex County	\N	f
Ashland, MA	Middlesex County	\N	f
Ayer, MA	Middlesex County	\N	f
Belmont, MA	Middlesex County	\N	f
Boxborough, MA	Middlesex County	\N	f
Burlington, MA	Middlesex County	\N	f
Cambridge, MA	Middlesex County	\N	f
Carlisle, MA	Middlesex County	\N	f
Concord, MA	Middlesex County	\N	f
Everett, MA	Middlesex County	\N	f
Framingham, MA	Middlesex County	\N	f
Groton, MA	Middlesex County	\N	f
Holliston, MA	Middlesex County	\N	f
Hopkinton, MA	Middlesex County	\N	f
Hudson, MA	Middlesex County	\N	f
Lexington, MA	Middlesex County	\N	f
Lincoln, MA	Middlesex County	\N	f
Malden, MA	Middlesex County	\N	f
Marlborough, MA	Middlesex County	\N	f
Maynard, MA	Middlesex County	\N	f
Medford, MA	Middlesex County	\N	f
Melrose, MA	Middlesex County	\N	f
Natick, MA	Middlesex County	\N	f
Newton, MA	Middlesex County	\N	f
Pepperell, MA	Middlesex County	\N	f
Reading, MA	Middlesex County	\N	f
Sherborn, MA	Middlesex County	\N	f
Shirley, MA	Middlesex County	\N	f
Somerville, MA	Middlesex County	\N	f
Stoneham, MA	Middlesex County	\N	f
Stow, MA	Middlesex County	\N	f
Sudbury, MA	Middlesex County	\N	f
Townsend, MA	Middlesex County	\N	f
Wakefield, MA	Middlesex County	\N	f
Waltham, MA	Middlesex County	\N	f
Watertown, MA	Middlesex County	\N	f
Wayland, MA	Middlesex County	\N	f
Weston, MA	Middlesex County	\N	f
Winchester, MA	Middlesex County	\N	f
Woburn, MA	Middlesex County	\N	f
Saugus, MA	Essex County	\N	f
Dunstable, MA	Middlesex County	Greater Lowell	t
North Andover, MA	Essex County	Greater Lawrence	t
Boxford, MA	Essex County	\N	t
Georgetown, MA	Essex County	\N	t
Groveland, MA	Essex County	\N	t
West Newbury, MA	Essex County	\N	t
Wilmington, MA	Middlesex County	\N	t
Beverly, MA	Essex County	\N	t
Danvers, MA	Essex County	\N	t
Essex County, MA	Essex County	\N	t
Gloucester, MA	Essex County	\N	t
Hamilton, MA	Essex County	\N	t
Ipswich, MA	Essex County	\N	t
Lynn, MA	Essex County	\N	t
Lynnfield, MA	Essex County	\N	t
Manchester-by-the-Sea, MA	Essex County	\N	t
Marblehead, MA	Essex County	\N	t
Middleton, MA	Essex County	\N	t
Nahant, MA	Essex County	\N	t
Newbury, MA	Essex County	\N	t
Newburyport, MA	Essex County	\N	t
North Reading, MA	Middlesex County	\N	t
Peabody, MA	Essex County	\N	t
Rockport, MA	Essex County	\N	t
Rowley, MA	Essex County	\N	t
Salem, MA	Essex County	\N	t
Salisbury, MA	Essex County	\N	t
Swampscott, MA	Essex County	\N	t
Topsfield, MA	Essex County	\N	t
Wenham, MA	Essex County	\N	t
Magnolia, MA	Essex County	\N	t
\.


--
-- Data for Name: user_type; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY user_type (type_id, type_name) FROM stdin;
1	Data Entry
\.


--
-- Data for Name: web_requests; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY web_requests (id, host, username, date, rtype, file, proto, code, bytes, refer, agent) FROM stdin;
\.


--
-- Data for Name: zip_distances; Type: TABLE DATA; Schema: public; Owner: omacneil
--

COPY zip_distances (start_zip, end_zip, distance) FROM stdin;
\.


--
-- Name: agency_agency_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY agency
    ADD CONSTRAINT agency_agency_name_key UNIQUE (agency_name);


--
-- Name: agency_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY agency
    ADD CONSTRAINT agency_pkey PRIMARY KEY (agency_id);


--
-- Name: alias_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY alias
    ADD CONSTRAINT alias_pkey PRIMARY KEY (category_id, alias_name);


--
-- Name: category_category_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY category
    ADD CONSTRAINT category_category_name_key UNIQUE (category_name);


--
-- Name: category_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY category
    ADD CONSTRAINT category_pkey PRIMARY KEY (category_id);


--
-- Name: date_agency_created_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY date_agency_created
    ADD CONSTRAINT date_agency_created_pkey PRIMARY KEY (agency_id);


--
-- Name: date_program_created_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY date_program_created
    ADD CONSTRAINT date_program_created_pkey PRIMARY KEY (program_id);


--
-- Name: heading_category_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY heading_category
    ADD CONSTRAINT heading_category_pkey PRIMARY KEY (category_id, heading_name);


--
-- Name: heading_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY heading
    ADD CONSTRAINT heading_pkey PRIMARY KEY (heading_name);


--
-- Name: log_category_search_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY log_category_search
    ADD CONSTRAINT log_category_search_pkey PRIMARY KEY (search_id);


--
-- Name: meta_data_version_version_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY meta_data_version
    ADD CONSTRAINT meta_data_version_version_key UNIQUE (version);


--
-- Name: meta_schema_version_version_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY meta_schema_version
    ADD CONSTRAINT meta_schema_version_version_key UNIQUE (version);


--
-- Name: program_category_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY program_category
    ADD CONSTRAINT program_category_pkey PRIMARY KEY (category_id, program_id);


--
-- Name: program_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY program
    ADD CONSTRAINT program_pkey PRIMARY KEY (program_id);


--
-- Name: search_log_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY search_log
    ADD CONSTRAINT search_log_pkey PRIMARY KEY (search_id);


--
-- Name: synonym_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY synonym_stem
    ADD CONSTRAINT synonym_pkey PRIMARY KEY (synonym_id, word_stem);


--
-- Name: town_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY town
    ADD CONSTRAINT town_pkey PRIMARY KEY (town_state);


--
-- Name: user_type_pkey; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY user_type
    ADD CONSTRAINT user_type_pkey PRIMARY KEY (type_id);


--
-- Name: user_type_type_name_key; Type: CONSTRAINT; Schema: public; Owner: omacneil; Tablespace: 
--

ALTER TABLE ONLY user_type
    ADD CONSTRAINT user_type_type_name_key UNIQUE (type_name);


--
-- Name: log_category_search_index_by_session_id; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE INDEX log_category_search_index_by_session_id ON log_category_search USING btree (session_id, phrase_stems);


--
-- Name: session_id_index_on_agency_search_log; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE INDEX session_id_index_on_agency_search_log ON agency_search_log USING btree (session_id, search_phrase);


--
-- Name: session_id_index_on_category_search_log; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE INDEX session_id_index_on_category_search_log ON category_search_log USING btree (session_id, phrase_stems);


--
-- Name: start_zip_index_on_zip_distances; Type: INDEX; Schema: public; Owner: omacneil; Tablespace: 
--

CREATE UNIQUE INDEX start_zip_index_on_zip_distances ON zip_distances USING btree (start_zip, end_zip);


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY program
    ADD CONSTRAINT "$1" FOREIGN KEY (agency_id) REFERENCES agency(agency_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY alias
    ADD CONSTRAINT "$1" FOREIGN KEY (category_id) REFERENCES category(category_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY heading_category
    ADD CONSTRAINT "$1" FOREIGN KEY (category_id) REFERENCES category(category_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY program_category
    ADD CONSTRAINT "$1" FOREIGN KEY (category_id) REFERENCES category(category_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY date_agency_created
    ADD CONSTRAINT "$1" FOREIGN KEY (agency_id) REFERENCES agency(agency_id) ON DELETE CASCADE;


--
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY date_program_created
    ADD CONSTRAINT "$1" FOREIGN KEY (program_id) REFERENCES program(program_id) ON DELETE CASCADE;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY program_category
    ADD CONSTRAINT "$2" FOREIGN KEY (program_id) REFERENCES program(program_id) ON DELETE CASCADE;


--
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: omacneil
--

ALTER TABLE ONLY heading_category
    ADD CONSTRAINT "$2" FOREIGN KEY (heading_name) REFERENCES heading(heading_name) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

--
-- PostgreSQL database dump complete
--

